var iServiceNg = angular.module('iService.ng', ['ngAnimate', 'ui.bootstrap', 'ui.select', 'angularMoment', 'iService.base', 'ngRoute', 'ui.bootstrap.datetimepicker']);

iServiceNg.config(['$locationProvider', function ($locationProvider) {
  $locationProvider.hashPrefix('');
}]).factory('iService.chatUtilities', [function () {
  var addTargetRegEx = /(<a)/ig;

  return {
    setLinksTargetToBlank: function (markup) {
      return markup.replace(addTargetRegEx, '$1 target="_blank"');
    }
  };
}]).factory('iService.activeChatsIdsStorage', ['$window', function ($window) {
  var key = 'activeChatsIdsStorage-contact' + iservice.loggedIn.contactID;

  return {
    getData: function () {
      var stateObj = JSON.parse($window.localStorage.getItem(key));

      return stateObj ? stateObj : [];
    },
    setData: function (data) {
      $window.localStorage.setItem(key, JSON.stringify(data));
    },
    clearData: function () {
      $window.localStorage.removeItem(key);
    }
  };
}]).factory('iService.chatFiltersStorage', ['$window', function ($window) {
  var key = 'chatFiltersStorage-contact' + iservice.loggedIn.contactID;

  return {
    getData: function () {
      var stateObj = JSON.parse($window.localStorage.getItem(key));

      return stateObj ? stateObj : { tag: 'chat', search: '' };
    },
    setData: function (data) {
      $window.localStorage.setItem(key, JSON.stringify(data));
    }
  };
}]);

iServiceNg.controller('ControllerLoginBar', ['$scope', '$http', '$timeout', '$window', '$uibModal', '$rootScope', 'iService.rootPath', 'iService.tabUrls', 'iService.service',
  function ($scope, $http, $timeout, $window, $uibModal, $rootScope, rootPath, tabUrls, service) {
    $scope.show = 'login';
    $scope.loading = false;

    openid.Ready($scope, $http, $scope.idPrefix);

    Object.defineProperty($scope, 'contactName', {
      enumerable: true,
      get: function () {
        return iservice.loggedIn.isLoggedIn ? iservice.loggedIn.contactName || iservice.loggedIn.logins[0].name : '';
      }
    });

    Object.defineProperty($scope, 'userEmail', {
      enumerable: true,
      get: function () {
        var emailLogin = iservice.loggedIn.logins.find(function (lgn) {
          return lgn.type === 'Email' || lgn.type === 'Bounced' || lgn.type === '';
        });

        return emailLogin ? emailLogin.name : '';
      }
    });

    Object.defineProperty($scope, 'customerDetailsUrl', {
      enumerable: true,
      get: function () {
        return tabUrls['cust-info-details'].replace('{id}', iservice.loggedIn.contactID);
      }
    });

    Object.defineProperty($scope, 'customerSubscriptionsUrl', {
      enumerable: true,
      get: function () {
        return tabUrls['cust-info-subscriptions'].replace('{id}', iservice.loggedIn.contactID);
      }
    });

    Object.defineProperty($scope, 'agentStockResponsesUrl', {
      enumerable: true,
      get: function () {
        return tabUrls['cust-info-mystockresponses'];
      }
    });

    $scope.logout = function () {
      openid.Logout();

      $scope.loading = true;

      $scope.LogoutRunning = service('webapp-api-login?api=logout')
        .then(function (httpResponse) {
          var data = httpResponse.data;

          if (data.errors && data.errors.length) {
            $scope.localErrors = data.errors;

            return;
          }

          iservice.ProcessLogin(data.loggedIn);
        }).finally(function () {
          $scope.loading = false;
        });

    };


    $scope.newpassword = '';
    $scope.confirmpassword = '';
    $scope.changePassword = function () {
      if ($scope.newpassword !== $scope.confirmpassword) {
        $scope.errormessage = ['Confirm password did not match.'];
        return;
      }
      var properties = {
        properties: [{ propertyID: 'password', values: [{ value: $scope.newpassword }] }],
        setLogins: false,
        logins: null
      };
      $scope.Saving = iservice.FindAnswerMyDetailsSet($http, properties, function (data) {
        if (data.errors && data.errors.length) {
          $scope.errormessage = data.errors;
        }
        else
        {
          $scope.successmessage = 'Password Changed successfully.';
        }
      });
    };
    $scope.login = function () {
      $scope.localErrors = null;
      $scope.localSuccess = null;
      if ($scope.loading)
        return;

      $scope.loading = true;
      if ($scope.loginName && $scope.password) {
        service('webapp-api-login?api=login',
          {
            emailAddress: $scope.loginName,
            password: $scope.password
          })
          .then(function (httpResponse) {
            processLogin(httpResponse.data);
          }).finally(function () {
            $scope.loading = false;
          });
      } else {
        $scope.localErrors = ['It looks like you are missing some values. Please enter both a Login Name and Password and click the Login button.'];
        $scope.loading = false;
      }
    };

    $scope.LoginAgent = function () {
      if (Running($scope.LoginRunning)) return;
      $scope.LoginRunning = iservice.LoginAgent($http, function (data) {
        processLogin(data);
      });
    }

    function processLogin(data) {
      if (data.errors && data.errors.length) {
        $scope.localErrors = data.errors;

        return;
      }

      if (iservice.loggedIn.contactID === data.loggedIn.contactID)
        return;

      $window.location.reload();
    }

    $scope.forgotPassword = function () {
      $scope.show = 'reset';
      $scope.showError = false;
      $scope.showSuccess = false;
      $scope.reset.email = '';
    };
    $scope.registerAccount = function () {
      $scope.show = 'register';
    };
    $scope.resetRegister = function () {
      $scope.show = 'login';
    };
    $scope.register = function () {
      $scope.submittedregister = true;
      if (!$scope.registration.$invalid) {
        var inputs = {
          inputrequiredcontactproperty1firstname: $scope.firstName,
          inputrequiredcontactproperty3lastname: $scope.lastName,
          inputemailrequired: $scope.email,
          inputpasswordrequired: $scope.password,
          inputpasswordrequired1: $scope.confirmPassword
        };
        
        $scope.LoginRunning = iservice.LoginRegister($http, inputs, function (data) {
          if (data.errors && data.errors.length) {
            $scope.errorregister = data.errors;
          } else {
            $scope.successregister = "Your account has been created.";
            $timeout(function () { $scope.successregister = ''; }, 4000);
          }
          iservice.ProcessLogin(data.loggedIn);
          $window.location.reload();
          $scope.register = false;
        });
      } else { return false; }
    }

    $scope.forgetCancel = function () {
      $scope.show = 'login';
    };
    // $scope.reset = '';
    $scope.showSuccess = false;
    $scope.showError = false;
    $scope.forgetSend = function () {

      $scope.localErrors = null;
      $scope.localSuccess = null;
      if ($scope.reset !== undefined && $scope.reset.email !== undefined) {
        var protocol = $window.location.protocol,
          host = $window.location.host,
          pathname = $window.location.pathname,
          dirname = pathname.substring(0, pathname.lastIndexOf('/') + 1),
          resetUrl = (protocol + '//' + host + dirname + rootPath + 'f/reset-password?confirm=$value -passwordreset(guid)$').replace('/../f/', '/f/');

        $scope.loading = true;

        service('webapp-api-login?api=resetSend',
          {
            emailAddress: $scope.reset.email,
            resetUrl: resetUrl
          })
          .then(function (httpResponse) {
            var data = httpResponse.data;

            if (data.errors && data.errors.length) {
              $scope.showError = data.errors;
              return;
            }
            $scope.showSuccess = true;
            $scope.showError = false;
          }).finally(function () {
            $scope.loading = false;
          });
      }
    };

    $scope.forgetRefresh = function () {
      $window.location.reload();
    };


    function AgentMessageReceive(msg) {
      var message = msg.getData();

      if (message.inactivityUpdate != null) {
        AgentInactivityMessage(message.inactivityUpdate);
      }

      else if (message.logoutUpdate != null) {
        AgentLogoutMessage(message.logoutUpdate);
      }
    }

    function AgentLogoutMessage(update) {
      if ($http.pendingRequests.length == 1) return;
      $scope.logoutUpdate = update;

      if ($scope.logoutUpdate) {
        $scope.modalLogoutMessage = function () {
          $rootScope.modalInstance = $uibModal.open({
            animation: true,
            ariaLabelledBy: 'modal-title',
            ariaDescribedBy: 'modal-body',
            backdrop: false,
            template: '<div class="modal-header"><h4 class="modal-title"><i class="icon fa fa-warning text-yellow"></i> You have been logged out</h4></div> <div class="modal-body text-left"><p class="text">You were automatically logged out of iService. This could be because you were inactive too long, or you were logged out from another browser window.<br></p></div> <div class="modal-body text-right"><a class="btn btn-primary" href="javascript:void(0);" ng-click="close()">Login</a></div>',
            controller: 'ControllerUibModal'
          });
          $rootScope.modalInstance.result.then(function () {

          });
        };
        $scope.modalLogoutMessage();
        $scope.RefreshSession();
      }
    }
    var isModalOpen = false;
    function AgentInactivityMessage(update) {
      var seconds = update.secondsRemaining;
      if (seconds < 0) seconds = 0;
      $scope.inactivityUpdateTime = new Date();
      $scope.inactivityCountdown = seconds;
      if (seconds > 60) $scope.logoutWarning = null;

      UpdateInactivityCountdown(seconds);
      function UpdateInactivityCountdown(totalSeconds) {
        if (totalSeconds < 0) totalSeconds = 0;
        $scope.inactivityStatusMessage = 'Auto sign out in ' + HourMinSec(totalSeconds);
        if (totalSeconds <= 30) {
          $scope.timerUpdate = $timeout(TimerUpdate, 1000);
          $scope.logoutWarning = $scope.inactivityStatusMessage;
          if ($scope.logoutWarning) {
            $scope.modalInactivityMessage = function () {

              $rootScope.modalInstanceInactivity = $uibModal.open({
                animation: true,
                ariaLabelledBy: 'modal-title',
                ariaDescribedBy: 'modal-body',
                backdrop: false,
                template: '<div class="modal-header"><h4 class="modal-title"><i class="icon fa fa-warning text-yellow"></i>Your session is about the expire.</h4> </div><div class="modal-body text-left"> <p class="text">You will be logged out in {{ totalSecs }} sec</p> </div> <div class="modal-footer"><a class="btn btn-primary" type="button" href="javascript:void(0);" ng-click="ContinueSession()">Continue Session</a><button class="btn btn-default" type="button" ng-click="LogoutSession()">Logout</button></div>',
                controller: 'ControllerUibModal'
              });
              $rootScope.modalInstanceInactivity.result.then(function () {

              });
            };
            if (isModalOpen == false) {
              $scope.modalInactivityMessage();
              isModalOpen = true;
            }
          }
        }
        else if (totalSeconds <= 180) {
          $scope.timerUpdate = $timeout(TimerUpdate, 5000);
        }
      }
      function TimerUpdate() {
        if (!$scope.timerUpdate) return;
        $scope.timerUpdate = null;
        if ($scope.logoutUpdate) return;
        var now = new Date();
        var elapsedSeconds = Math.floor((now.getTime() - $scope.inactivityUpdateTime.getTime()) / 1000);
        UpdateInactivityCountdown($scope.inactivityCountdown - elapsedSeconds);
      }
    }
    $scope.RefreshSession = function () {
      iservice.LoggedIn($http, function (data) {
        iservice.ProcessLogin(data);
        if (data.isLoggedIn) {
          $scope.logoutWarning = null;
          $scope.logoutUpdate = null;
          $scope.inactivityStatusMessage = null;
          if ($scope.timerUpdate) {
            $timeout.cancel($scope.timerUpdate);
            $scope.timerUpdate = null;
          }
        }
        else {
          $scope.logoutUpdate = {};
        }
      });
    }

    function WebsyncSubscribe() {
      WebsyncCleanup();
      if (iservice.loggedIn.contactID) {
        $scope.UnsubscribeAgent = websync.subscribe('/agent/' + iservice.loggedIn.tenantID + '/' + iservice.loggedIn.contactID, AgentMessageReceive, null);
      }
    }
    function WebsyncCleanup() {
      if ($scope.UnsubscribeAgent) {
        $scope.UnsubscribeAgent();
        $scope.UnsubscribeAgent = null;
      }
    }
    WebsyncSubscribe();
    $scope.$watch('iservice.loggedIn.contactID', function (newValue, oldValue) {
      if (newValue === oldValue) return;
      WebsyncSubscribe();
    });
    $scope.$on("$destroy", function () {
      WebsyncCleanup();
    });
  }]);

function formatDate(date) {
  date = new Date(date);
  var dd = date.getDate();
  var mm = date.getMonth() + 1;
  var yyyy = date.getFullYear();
  if (dd < 10) { dd = '0' + dd }
  if (mm < 10) { mm = '0' + mm }
  date = mm + '/' + dd + '/' + yyyy;
  return date
}
function Last7Days() {
  var result = [];
  for (var i = 0; i < 7; i++) {
    var d = new Date();
    d.setDate(d.getDate() - i);
    result.push(formatDate(d))
  }

  return result;
}
function formatDateWithoutYear(date) {
  date = new Date(date);
  var dd = date.getDate();
  var mm = date.getMonth() + 1;
  var yyyy = date.getFullYear();
  if (dd < 10) { dd = '0' + dd }
  if (mm < 10) { mm = '0' + mm }
  date = mm + '/' + dd;
  return date
}
function Last7DayswithoutYear() {
  var result = [];
  for (var i = 0; i < 7; i++) {
    var d = new Date();
    d.setDate(d.getDate() - i);
    result.push(formatDateWithoutYear(d))
  }

  return result;
}

iServiceNg.controller('ControllerDefault', ['$scope', 'iService.service', '$timeout', 'iService.interactionStatusClasses',
  'iService.interactionTypeClasses',
  'iService.interactionTypeLabels', 'iService.websites', 'iService.api.serial', function ($scope, service, $timeout, interactionStatusClasses, interactionTypeClasses, interactionTypeLabels, websites, API) {
  iservice.pageName = 'default';
  $scope.title = "My Dashboard";
  $scope.loader = true;
  $scope.interactionStatusClasses = interactionStatusClasses;
  $scope.interactionTypeClasses = interactionTypeClasses;
  $scope.interactionTypeLabels = interactionTypeLabels;

  var now = new Date(), aWeekAgo = new Date();

  aWeekAgo.setDate(aWeekAgo.getDate() - 7);
  

  websites.then(function () {
    var list = websites.list();
    var count = list.length;
    LoadWebsite(list[0], 0);
    function LoadWebsite(website, i) {
      API('webapp-api-admin?api=websitedetails', null, 'WebsiteDetails', { websiteID: website.id }).then(function (result) {
        website.details = result.details;
        i++;
        if (i < count) {
          LoadWebsite(list[i], i);
        }
        else {
          LoadWebsiteDone();
        }
      }).catch(function (errors) {
        if (doneFunc) doneFunc(errors);
      });
    }
    function LoadWebsiteDone() {
      list.forEach(function (website) {
        var interactionform = iservice.WebsiteGetPlaceholder(website.details, 'dashboard-body1');
        var answerform = iservice.WebsiteGetPlaceholder(website.details, 'dashboard-body2');
        var supportform = iservice.WebsiteGetPlaceholder(website.details, 'dashboard-body3');
        $scope.defaultBody1 = interactionform;
        $scope.defaultBody2 = answerform;
        $scope.defaultBody3 = supportform;
      });
    }
  });

  $scope.defaultPastDay = 1;
  if (localStorage.getItem('defaultPastdayValue')) {
    $scope.defaultPastDay = localStorage.getItem('defaultPastdayValue');
    }
    var rowUpdate = 0;
  var aDayago = new Date();
    aDayago.setDate(aDayago.getDate() - $scope.defaultPastDay);
    angular.element(document).ready(function () {
      /* ----- My Interactions----- */
      service('webapp-api-mq-search?api=search', {
        fields: {
          groups: [
            {
              fields: [{
                where: 'anywhere',
                fieldID: 'resolve',
                searchString: iservice.loggedIn.contactID
              }]
            },
            {
              fields: [{
                where: 'entire',
                fieldID: 'not-findanswers',
                searchString: ""
              }]
            }
          ],
          timeStartUtc: moment.utc(aDayago).format(),
          timeEndUtc: moment.utc(now).format()
        },
        pageNum: 0,
        perPage: 1000
      }).then(function ({ data }) {
        $scope.myInteractions = data.interactions;
        var columns = [
          { name: 'Interaction Type', var: 'type' },
          { name: 'Subject', var: 'subject', split: true },
          { name: 'Contact Email', var: 'fromAddress', split: true },
          { name: 'Date', var: 'date', split: true, printFunc: PrintUtcDate, ascend: false, sortRaw: true, exportType: 'timestamp', exportFunc: (date) => { return date; } }
        ];
        $scope.columnsLength = columns.length;
        var myInteractions = iservice.TableViewer($scope, $timeout,
          { itemsFunc: function () { return $scope.myInteractions; }, watchFuncs: [function () { return rowUpdate; }], columns: columns, startSort: 'date', exportName: 'My Interactions', pageSize: 5, pastDayValue: $scope.defaultPastDay, showPastField: true });
        $scope.myInteractionsView = myInteractions;
        $scope.loader = false;
      });
      $scope.$on('delete-interaction', (event, interactionID) => {
        const results = $scope.myInteractions;

        if (results && results.length) {
          for (var t = 0; t < results.length; t++) {
            var result = results[t];
            if (result.id == interactionID) {
              $scope.deleteSuccess = "Interaction deleted successfully.";
              $scope.myInteractions.splice(t, 1)
            }
            rowUpdate++;
          }
        }
      });
      /* ----- My Interactions----- */
      /*Watch if Past day value updated then update records accordingly*/
      $scope.$watch("myInteractionsView", function (newValue, oldValue) {
        if (newValue && oldValue && newValue.pastDayValue !== oldValue.pastDayValue) {
          if (newValue.pastDayValue > 0) {
            var updatedDay = new Date();
            localStorage.setItem('defaultPastdayValue', parseInt(newValue.pastDayValue));
            updatedDay.setDate(updatedDay.getDate() - parseInt(newValue.pastDayValue));
            $scope.loader = true;
            service('webapp-api-mq-search?api=search', {
              fields: {
                groups: [
                  {
                    fields: [{
                      where: 'anywhere',
                      fieldID: 'resolve',
                      searchString: iservice.loggedIn.contactID
                    }]
                  },
                  {
                    fields: [{
                      where: 'entire',
                      fieldID: 'not-findanswers',
                      searchString: ""
                    }]
                  }
                ],
                timeStartUtc: moment.utc(updatedDay).format(),
                timeEndUtc: moment.utc(now).format()
              },
              pageNum: 0,
              perPage: 1000
            }).then(function ({ data }) {
              $scope.myInteractions = data.interactions;
              rowUpdate++;
              $scope.loader = false;
            });
          } else {
            return false;
          }
        }
      }, true);

      /* ----- My Answers ----- */
      var numberWithCommas = function (x) {
        return x.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
      };


      $scope.grapghLoader = true;

      function checkPrivateNote(msg) {
        return msg.type == 'Private Note';
      }
      function checkAgentResponse(msg) {
        return msg.type == 'Agent Response';
      }
      function checkAgentEmail(msg) {
        return msg.type == 'Agent Email';
      }
      function checkResolveNote(msg) {
        return msg.type == 'Resolve Note';
      }
      function checkPublicNote(msg) {
        return msg.type == 'Note';
      }
      var dates = Last7Days().reverse();
      service('webapp-api-mq-search?api=search', {
        fields: {
          groups: [
            {
              fields: [{
                where: 'anywhere',
                fieldID: 'resolve',
                searchString: iservice.loggedIn.contactID
              }]
            },
            {
              fields: [{
                where: 'entire',
                fieldID: 'not-findanswers',
                searchString: ""
              }]
            },
            {
              fields: [
                {
                  where: 'anywhere',
                  fieldID: 'type',
                  searchString: "Private Note"
                },
                {
                  where: 'anywhere',
                  fieldID: 'type',
                  searchString: "Note"
                },
                {
                  where: 'anywhere',
                  fieldID: 'type',
                  searchString: "Agent Response"
                },
                {
                  where: 'anywhere',
                  fieldID: 'type',
                  searchString: "Agent Email"
                },
                {
                  where: 'anywhere',
                  fieldID: 'type',
                  searchString: "Resolve Note"
                }
              ]
            }
          ],
          timeStartUtc: moment.utc(aWeekAgo).format(),
          timeEndUtc: moment.utc(now).format()
        },
        pageNum: 0,
        perPage: 1000
      }).then(function ({ data }) {
        var interactions = data.interactions;

        $scope.LastWeekdates = Last7DayswithoutYear().reverse();
        var privateNotes = interactions.filter(checkPrivateNote);
        var publicNotes = interactions.filter(checkPublicNote);
        var agentEmails = interactions.filter(checkAgentEmail);
        var resolved = interactions.filter(checkResolveNote);
        var agentresponses = interactions.filter(checkAgentResponse);

        var pNotes = [0, 0, 0, 0, 0, 0, 0];
        var publicNote = [0, 0, 0, 0, 0, 0, 0];
        var agentEmail = [0, 0, 0, 0, 0, 0, 0];
        var resolve = [0, 0, 0, 0, 0, 0, 0];
        var agentresponse = [0, 0, 0, 0, 0, 0, 0];
        if (privateNotes.length) {

          dates.forEach(function (date, key) {
            privateNotes.forEach(function (row) {
              pNotes[key] = formatDate(row.date) == date ? pNotes[key] + 1 : pNotes[key];
            });
          });
        }

        if (publicNotes.length) {
          publicNotes.forEach(function (row) {
            dates.forEach(function (date, key) {
              publicNote[key] = formatDate(row.date) == date ? publicNote[key] + 1 : publicNote[key];
            });
          });
        }

        if (agentEmails.length) {
          agentEmails.forEach(function (row) {
            dates.forEach(function (date, key) {
              agentEmail[key] = formatDate(row.date) == date ? agentEmail[key] + 1 : agentEmail[key];
            });
          });
        }
        if (resolved.length) {
          resolved.forEach(function (row) {
            dates.forEach(function (date, key) {
              resolve[key] = formatDate(row.date) == date ? resolve[key] + 1 : resolve[key];
            });
          });
        }
        if (agentresponses.length) {
          agentresponses.forEach(function (row) {
            dates.forEach(function (date, key) {
              agentresponse[key] = formatDate(row.date) == date ? agentresponse[key] + 1 : agentresponse[key];
            });
          });
        }
        var defaultcount = [0, 0, 0, 0, 0, 0, 0];

        $scope.privateNotes = pNotes.length ? pNotes : defaultcount;
        $scope.publicNote = publicNote.length ? publicNote : defaultcount;
        $scope.agentEmails = agentEmail.length ? agentEmail : defaultcount;
        $scope.resolved = resolve.length ? resolve : defaultcount;
        $scope.agentresponses = agentresponse.length ? agentresponse : defaultcount;

        Chart.plugins.register({
          afterDraw: function (chart) {
            if (chart.data.datasets.length === 0) {
              // No data is present
              var ctx = chart.chart.ctx;
              var width = chart.chart.width;
              var height = chart.chart.height
              chart.clear();

              ctx.save();
              ctx.textAlign = 'center';
              ctx.textBaseline = 'middle';
              ctx.font = "16px normal 'Helvetica Nueue'";
              ctx.fillText('There is no data to display for this period.', width / 2, height / 2);
              ctx.restore();
            }
          }
        });
        var dataSets = [
          {
            label: 'Private Note',
            data: $scope.privateNotes,
            backgroundColor: "#2672ec",
            hoverBackgroundColor: "#2672ec",
            hoverBorderWidth: 0
          },
          {
            label: 'Public Note',
            data: $scope.publicNote,
            backgroundColor: "#4bacc6",
            hoverBackgroundColor: "#4bacc6",
            hoverBorderWidth: 0
          },
          {
            label: 'Agent Email',
            data: $scope.agentEmails,
            backgroundColor: "#f79646",
            hoverBackgroundColor: "#f79646",
            hoverBorderWidth: 0
          },
          {
            label: 'Resolved Note',
            data: $scope.resolved,
            backgroundColor: "#9bbb59",
            hoverBackgroundColor: "#9bbb59",
            hoverBorderWidth: 0
          },
          {
            label: 'Agent Response',
            data: $scope.agentresponses,
            backgroundColor: "#c8a6f2",
            hoverBackgroundColor: "#c8a6f2",
            hoverBorderWidth: 0
          },
        ];
        var zeroPrivateNotes = $scope.privateNotes.every(item => item === 0);
        var zeroprivateNotes = $scope.publicNote.every(item => item === 0);
        var zeroagentEmails = $scope.agentEmails.every(item => item === 0);
        var zeroresolved = $scope.resolved.every(item => item === 0);
        var zeroagentresponses = $scope.agentresponses.every(item => item === 0);

        if (zeroPrivateNotes && zeroprivateNotes && zeroagentEmails && zeroresolved && zeroagentresponses){
          dataSets = [];
        }

        var bar_ctx = document.getElementById('bar-chart');
        if (bar_ctx !== null) {
          var bar_chart = new Chart(bar_ctx, {
            type: 'bar',
            data: {
              labels: dates,
              datasets: dataSets
            },
            options: {
              animation: {
                duration: 10,
              },
              tooltips: {
                mode: 'label',
                callbacks: {
                  label: function (tooltipItem, data) {
                    return data.datasets[tooltipItem.datasetIndex].label + ": " + numberWithCommas(tooltipItem.yLabel);
                  }
                }
              },
              responsive: true,
              scales: {
                xAxes: [{
                  stacked: true,
                  gridLines: { display: false },
                }],
                yAxes: [{
                  stacked: true,
                  ticks: {
                    beginAtZero: true,
                    callback: function (value) { if (Number.isInteger(value)) { return value; } },
                    stepSize: 1
                  }
                }],
              }, // scales
              legend: { display: false },
              plugins: {
                datalabels: {
                  display: false,
                },
              }
            } // options
          }
          );

          document.getElementById('js-legend').innerHTML = bar_chart.generateLegend();
        }

        $scope.grapghLoader = false;
      });
    });
    
  /* ----- My Answers ----- */

}]);

iServiceNg.controller('ControllerUibModal', ['$scope', '$http', '$timeout', '$window', 'iService.service', '$uibModal', '$rootScope', function ($scope, $http, $timeout, $window, service, $uibModal, $rootScope) {
  $scope.test = "This is a testing";
  $scope.LogoutSession = function () {
    openid.Logout();

    $scope.loading = true;

    $scope.LogoutRunning = service('webapp-api-login?api=logout').then(function (httpResponse) {
      var data = httpResponse.data;

      if (data.errors && data.errors.length) {
        $scope.localErrors = data.errors;

        return;
      }

      iservice.ProcessLogin(data.loggedIn);
    }).finally(function () {
      $scope.loading = false;
      $rootScope.modalInstanceInactivity.close();
    });
  };

  $scope.ContinueSession = function () {
    iservice.LoggedIn($http, function (data) {
      iservice.ProcessLogin(data);
      if (data.isLoggedIn) {
        $scope.logoutWarning = null;
        $scope.logoutUpdate = null;
        $scope.inactivityStatusMessage = null;
        if ($scope.timerUpdate) {
          $timeout.cancel($scope.timerUpdate);
          $scope.timerUpdate = null;
        }
        $rootScope.modalInstanceInactivity.close();
      }
      else {
        $scope.logoutUpdate = {};
      }
    });
  }

  $scope.close = function () {
    $window.location.reload();
    $rootScope.modalInstance.close();
  }
  var totalSecs = 30;
  setInterval(function () {
    if (totalSecs > 0) {
      totalSecs = totalSecs - 1;
      $scope.totalSecs = totalSecs;
      totalSecs = totalSecs;
    }
    if (totalSecs == 0) {
      $rootScope.modalInstanceInactivity.close();
    }
  }, 1000);

}]);

iServiceNg.controller('ControllerCustomerSearchForm', ['$scope', '$route', '$timeout', '$window', 'iService.rootPath', 'iService.tabUrls',
  function ($scope, $route, $timeout, $window, rootPath, tabUrls) {
    $scope.search = function () {
      if (!$scope.param)
        return;
      
      $window.location.href = rootPath + tabUrls['cust-info-search'] + '?w1=anywhere&f1=login&t1=' + $scope.param;
      
      $timeout(function () {
        $route.reload();
      });
    };
    
  }]);
iServiceNg.controller('ControllerAgentChat', ['$scope', '$http', '$timeout','$interval',  '$attrs', 'iService.rootPath', 'iService.tabUrls', 'iService.chatUtilities', 'iService.activeChatsIdsStorage', 'iService.chatFiltersStorage',
  function ($scope, $http, $timeout, $interval, $attrs, rootPath, tabUrls, chatUtilities, activeChatsIdsStorage, chatFiltersStorage) {
    $scope.chatAnswer = function () {
      $scope.chatAnswerRunning = iservice.ChatAnswer($http, $scope.idPrefix);
    };

    $scope.numberOfChats = function () {
      return iservice.loggedIn.numChats;
    };

    $scope.showChat = function () {
      return iservice.HaveRight('Chat.Answer');
    };

    var minimal = $attrs.minimal;

    if (minimal && JSON.parse(minimal))
      return;

    $scope.customerSummaryUrl = function (chat) {
      return tabUrls['cust-info-summary'].replace('{id}', chat.customerID);
    };

    $scope.chatStatusClass = function () {
      return iservice.loggedIn.isAvailable ? 'text-success' : 'text-danger';
    };

    $scope.chatStatusString = function () {
      return iservice.loggedIn.isAvailable ? 'Available' : 'Unavailable';
    };

    function deleteChatObject(chat) {
      if (chat.removePromise)
        $timeout.cancel(chat.removePromise);

      iservice.loggedIn.chats = iservice.loggedIn.chats.filter(function (ch) {
        return ch.id !== chat.id;
      });
    }

    function removeChat(id) {
      var chat = iservice.loggedIn.chats.find(function (ch) {
        return ch.id === id;
      });

      if (!chat)
        return;

      deleteChatObject(chat);

      if (chat.unsubscribeChat)
        chat.unsubscribeChat();

      if (iservice.loggedIn.chats.length > 0 && !iservice.loggedIn.chats.some(function (ch) {
        return ch.isActive;
      })) {
        $scope.selectChat(iservice.loggedIn.chats[0]);
      }
      else
        activeChatsIdsStorage.clearData();
    }

    function chatInboxMessage(chat) {
      var existing = iservice.loggedIn.chats.find(function (ch) {
        return ch.id === chat.id;
      });

      if (!existing) {
        if (!chat.isDeleted) {
          chat.isActive = true;

          iservice.loggedIn.chats.push(chat);

          $scope.selectChat(chat, true);
        }

        return;
      }

      if (chat.isDeleted) {
        if (!chat.isActive) {
          existing.isDeleted = true;
          $scope.chatClosetime = 10;
          existing.removePromise = $timeout(function () {
            existing.removePromise = null;

            removeChat(existing.id);
          }, 10000);
          $interval(function () {
            $scope.chatClosetime = $scope.chatClosetime - 1;
            $scope.chatClosetimer = HourMinSec($scope.chatClosetime)
          }, 1000);
          
        }
        else
          deleteChatObject(existing);
      }
      else
        angular.extend(existing, chat);
    }

    function agentStatusMessage(message) {
      iservice.loggedIn.isAvailable = message.isAvailable;

      if (iservice.loggedIn.numChats === 0 && message.numChats > 0) {
        $timeout(function () {
          if (iservice.loggedIn.isAvailable && iservice.loggedIn.isLoggedIn && iservice.HaveRight('Chat.Answer')) {
            $.playSound('sounds/2Rings');

            Push.create('iService Chat Request', {
              body: 'An iService chat is waiting to be answered',
              icon: 'images/iserviceicon.png',
              timeout: 30000,
              vibrate: [100, 100, 100],
              onClick: function () {
                window.focus();

                $scope.chatAnswer();

                this.close();
              }
            });
          }
        }, 500);
      }

      iservice.loggedIn.numChats = message.numChats;
    }

    function agentMessageReceive(msg) {
      var message = msg.getData();
      if (message.agentStatus)
        agentStatusMessage(message.agentStatus);
      else if (message.chatInboxUpdate)
        chatInboxMessage(message.chatInboxUpdate);
    }

    $scope.websyncChat = websync.FeedbackObject($scope);

    $scope.toggleStatus = function () {
      if (Notification.permission != 'granted') {
        Notification.requestPermission();
      }
      $scope.toggleStatusRunning = iservice.ChatToggleAvail($http);
    };

    $scope.toggleStatusTitle = function () {
      return iservice.loggedIn.isAvailable ? 'Go Unavailable' : 'Go Available';
    };

    $scope.chatAnswerTitle = function () {
      return 'Answer Chat';
    };

    $scope.haveChats = function () {
      return iservice.loggedIn.chats.length > 0;
    };

    $scope.getChats = function () {
      return iservice.loggedIn.chats;
    };
    /**
    * Convert URLs in a string to anchor buttons
    * @param {!string} string
    * @returns {!string}
    */
    $scope.URLify = function (string) {
      var urls = string.match(/(((ftp|https?):\/\/)[\-\w@:%_\+.~#?,&\/\/=]+)/g);
      if (urls) {
        urls.forEach(function (url) {
          string = string.replace(url, '<a target="_blank" href="' + url + '">' + url + "</a>");
        });
      }
      return string.replace("(", "<br/>(");
    }

    $scope.selectChat = function (chat, force) {
      if (chat.isDeleted)
        return;

      chat.isActive = force || !chat.isActive;

      var activeIds = activeChatsIdsStorage.getData(),
        activeIdIndex = activeIds.indexOf(chat.id),
        arrayIsDirty = false;

      if (chat.isActive) {
        if (activeIdIndex < 0) {
          activeIds.push(chat.id);

          arrayIsDirty = true;
        }

        chat.chatData = {
          chatLines: []
        };

        $http.post(rootPath + 'f/webapp-api-agentchat', { 'api': 'details', 'chatID': chat.id, 'customerID': chat.customerID }).then(function (angularHttpResponse) {
          chat.custDetails = angularHttpResponse.data.custDetails;
          chat.chatData = angularHttpResponse.data.chatData;

          $scope.$parent.$broadcast('chat' + chat.id + 'Scroll');
        });

        if (!chat.stockResponseGroups) {
          $http.post(rootPath + 'f/dotnetapp-json-stockresponses', { 'segmentID': chat.segmentID }).then(function (angularHttpResponse) {
            chat.stockResponseGroups = angularHttpResponse.data;
          });
        }

        chat.unsubscribeChat = websync.subscribe('/agentchat/' + iservice.loggedIn.tenantID + '/' + chat.id, function (e) {
          chat.chatData.chatLines.push(e.getData());

          $scope.$parent.$broadcast('chat' + chat.id + 'Scroll');
        }, undefined, $scope);
      }
      else {
        if (activeIdIndex >= 0) {
          activeIds.splice(activeIdIndex, 1);

          arrayIsDirty = true;
        }
        if (chat.unsubscribeChat)
          chat.unsubscribeChat();
      }

      if (arrayIsDirty)
        activeChatsIdsStorage.setData(activeIds);
    };

    $scope.previewMessageClass = function (chat) {
      return chat.previewIsAgent ? 'text-success' : 'text-danger';
    };

    $scope.endChat = function (chat) {
      if (chat.isDeleted)
        return;

      chat.chatEndRunning = iservice.ChatEnd($http, chat.id, function (data) {
        removeChat(chat.id);
      }, $scope.idPrefix);
    };

    $scope.Unqueue = function (chat) {
      if (chat.isDeleted)
        return;

      chat.unqueueRunning = iservice.ChatUnqueue($http, chat.id, function (data) {
        chat.isForwardMenuOpen = false;

        removeChat(chat.id);
      }, $scope.idPrefix);
    };

    $scope.assignGetAgents = function (chat) {
      if (chat.isDeleted)
        return;
      $scope.reassign = true;
      chat.assignGetAgentsRunning = iservice.ChatAgentList($http, chat.id, function (data) {
        chat.reassignAgents = data;
      });
    };

    $scope.assignTo = function (chat, agent) {
      if (chat.isDeleted || !agent.isAvailable)
        return;
      chat.assignToRunning = iservice.ChatAssignTo($http, chat.id, agent.contactID, function (data) {
        chat.isForwardMenuOpen = false;
        chat.reassignAgents = null;
      }, $scope.idPrefix);
    };
    $scope.cancelReassign = function()
    {
      $scope.reassign = false;
    }
    $scope.changeTopic = function (chat, topic) {
      if (chat.isDeleted)
        return;
      chat.checkOpen = false;
      chat.changeTopicRunning = iservice.ChatChangeTopic($http, chat.id, topic.id, null, $scope.idPrefix);
     
    };

    $scope.SendTitle = function () {
      return 'Send Message';
    };

    $scope.Send = function (chat) {
      $scope.localErrors = null;
      if (chat.message == undefined || chat.message == '') {
        $timeout(function () {
          $scope.localErrors = ["Please enter a message"];
        }, 200);
        return;
      }
      if (chat.isDeleted || Running(chat.sendRunning))
        return;
      chat.sendRunning = iservice.ChatSend($http, chat.id, chatUtilities.setLinksTargetToBlank(chat.message), function (data) {
        chat.message = '';
      }, $scope.idPrefix);
    };

    $scope.pasteStock = function (chat, response) {
      $scope.$broadcast('chatBox' + chat.id + 'Paste', response.bodyHtml);
    };

    $scope.filter = chatFiltersStorage.getData();
    $scope.filterChange = function () {
      chatFiltersStorage.setData($scope.filter);
    };

    function WebsyncSubscribeChat() {
      WebsyncCleanup();
      if (iservice.HaveRight('Chat.Answer')) {
        $scope.websyncChat.Unsubscribe = websync.subscribe('/agent/' + iservice.loggedIn.tenantID + '/' + iservice.loggedIn.contactID, agentMessageReceive, $scope.websyncChat);
        var activeChatsIds = activeChatsIdsStorage.getData();
        if (!activeChatsIds.length && iservice.loggedIn.chats.length > 0)
          $scope.selectChat(iservice.loggedIn.chats[0]);
        else {
          iservice.loggedIn.chats.forEach(function (ch) {
            if (activeChatsIds.indexOf(ch.id) >= 0)
              $scope.selectChat(ch, true);
          });
        }
      }
    }

    function WebsyncCleanup() {
      if ($scope.websyncChat.Unsubscribe) {
        $scope.websyncChat.Unsubscribe();
        $scope.websyncChat.Unsubscribe = null;
      }
      iservice.loggedIn.chats.forEach(function (ch) {
        if (ch.unsubscribeChat) {
          ch.unsubscribeChat();
          ch.unsubscribeChat = null;
        }
      });
    }

    WebsyncSubscribeChat();
    $scope.$watch('iservice.loggedIn.contactID', function (newValue, oldValue) {
      if (newValue === oldValue)
        return;
      WebsyncSubscribeChat();
    });
    $scope.$on('$destroy', function () {
      WebsyncCleanup();
    });
  }]);
  iServiceNg.filter('propsFilter', function () {
  return function (items, props) {
    var out = [];

    if (angular.isArray(items)) {
      var keys = Object.keys(props);

      items.forEach(function (item) {
        var itemMatches = false;

        for (var i = 0; i < keys.length; i++) {
          var prop = keys[i];
          var text = props[prop].toLowerCase();
          if (item[prop].toString().toLowerCase().indexOf(text) !== -1) {
            itemMatches = true;
            break;
          }
        }

        if (itemMatches) {
          out.push(item);
        }
      });
    } else {
      // Let the output be the input untouched
      out = items;
    }

    return out;
  };
});

iServiceNg.controller('ControllerHistory', ['$scope', '$http', function ($scope, $http) {
  var HistoryChildrenFunc = $scope.HistoryChildrenFunc; if (!HistoryChildrenFunc) HistoryChildrenFunc = iservice.HistoryChildren;
  var HistoryDetailsFunc = $scope.HistoryDetailsFunc; if (!HistoryDetailsFunc) HistoryDetailsFunc = iservice.HistoryDetails;
  $scope.HistoryThreadUrl = function (thread) {
    return thread.children ? 'history-thread-open2.html' : 'history-thread-closed2.html';
  };
  $scope.ThreadExpandClick = function (thread) {

    thread.expanded = !thread.expanded;
    if (thread.children) {
      thread.children = undefined;
      return;
    }

    if (thread.expanded) {
      $scope.ThreadExpandRunning = HistoryChildrenFunc($http, thread.id, function (data) {
        var root = { id: thread.id, subject: thread.subject, type: thread.type, date: thread.date };
        iservice.SanitizeHistoryDetails(data.details);
        root.details = data.details;
        thread.children = [root];
        data.children.sort(SortSimple('date', -1));
        for (var i = 0; i < data.children.length; i++) {
          var child = data.children[i];
          if (FilterChild) child = FilterChild(child);
          if (child) thread.children.push(child);
        }
        iservice.SanitizeHistoryRows(thread.children);
      });
    }
  };
  var FilterChild = $scope.FilterChild;
  $scope.HistoryChildUrl = function (child) {
    if (!child.details) return 'history-child-closed2.html';
    return 'history-child-open2.html';
  };
  $scope.ChildExpandClick = function (child) {
    if (child.details) {
      child.details = undefined;
      return;
    }
    $scope.ChildExpandRunning = HistoryDetailsFunc($http, child.id, function (data) {
      iservice.SanitizeHistoryDetails(data);

      if (data.history)
        iservice.SanitizeHistoryRows(data.history);

      child.details = data;
    });
  };
  $scope.MessageDetailsUrl = function (row) {
    return iservice.MessageDetailsUrl(row);
  };
  $scope.InteractionType = function (type) {
    // If the type is 'alert' prefix it with alt- to escape naming collision with Bootstrap who has
    // a class with the same name
    return type === 'Alert' ? 'alt-alert' : type.toLowerCase().replace(/ /g, '-');
  };
  $scope.$on('updated-row', function (event, interaction) {
    var threads = $scope.rows;
    if (threads && threads.length) for (var t = 0; t < threads.length; t++) {
      var thread = threads[t];
      if (thread.id === interaction.row.id) {
        angular.extend(thread, interaction.row);
      }
      if (thread.children && thread.children.length) for (var c = 0; c < thread.children.length; c++) {
        var child = thread.children[c];
        if (child.id === interaction.row.id) {
          angular.extend(child, interaction.row);
          return;
        }
      }
      if (thread.children && thread.id === interaction.row.parentID) {
        thread.children.push(interaction.row);
      }
    }
  });
}]);
iServiceNg.controller('ControllerAnswerActions', ['$scope', '$http', function ($scope, $http) {
  var answer = $scope.ansewr || {};
  function Clear() {
    $scope.menu = '';
    $scope.reassign = '';
    $scope.forward = '';
    $scope.changeSegment = '';
    $scope.sendSecure = '';
  }
  Clear();
  $scope.$on('selected-message', function (event, message) {
    Clear();
  });
  $scope.$on('close-menu', function (event, message) {
    Clear();
  });
  $scope.SendButtonClass = function (details) {
    if (!details) return 'button-send-withdropdown';
    return details.status === 'Pending' ? 'button-approve' : 'button-send-withdropdown';
  };
  $scope.Open = function (name) {
    if (name === $scope.menu) $scope.menu = null;
    else $scope.menu = name;
  };
  $scope.OpenTopics = function () {
    $scope.Open('topic');
    if ($scope.menu === 'topic') {
      $scope.TopicsLoading = iservice.AdminList($http, 'ListTopic', ['topics'], function (data) {
        for (var i = 0; i < data.topics.length; i++) {
          var topic = data.topics[i];
          if (!topic.parentID) { topic.padding = ''; }
          topic.paddingIndent = topic.padding.replace(/\. /g, '&nbsp; ');
        }
      });
    }
  };
  $scope.ReassignStart = function (details, answer, topic) {
    var func = answer.isMassUpdate ? iservice.ManageMessageAgents : iservice.MyQueueAgents;
    var param = answer.isMassUpdate ? topic.id : details.id;
    $scope.AgentsLoading = func($http, param, function (data) {
      $scope.reassignAgents = data.agents;
      $scope.reassignAgents.splice(0, 0, { id: '', email: '-- Select An Agent --' });
      for (var i = 0; i < data.agents.length; i++) {
        var agent = data.agents[i];
        if (agent.isloggedin) agent.option = agent.email;
        else if (agent.doemptyinbox) agent.option = agent.email + ' (unavailable)';
        else agent.option = agent.email + ' (logged out)';
      }
      $scope.reassign = 'choose';
    });
  };
  $scope.ForwardExternalStart = function (details) {
    $scope.forwardExternalNotifications = [];
    $scope.ForwardExternalLoading = iservice.AdminList($http, 'ExternalNotify', ['externalnotify'], function (data) {
      ForArray(data.externalNotify, function (notify) {
        if (notify.segmentID === details.segmentID) $scope.forwardExternalNotifications.push(notify);
      });
      if ($scope.forwardExternalNotifications.length < 1) $scope.forward = 'empty';
      else $scope.forward = 'choose';
    });
  };
  $scope.ForwardExternalStartClass = function () {
    return $scope.forward === '' ? 'nglink' : '';
  };
  $scope.ChangeSegmentStart = function (answer) {
    $scope.changeSegment = 'choose';
    $scope.answer.forwardSegmentID = '';
    $scope.forwardSegments = [];
    $scope.forwardAllMailboxes = [];
    $scope.ForwardSegmentLoading = iservice.MyQueueForwardMailboxes($http, function (data) {
      var lastDefault = null;
      ForArray(data.mailboxes, function (mailbox) {
        if (!lastDefault || mailbox.segmentID !== lastDefault.segmentID) {
          lastDefault = { id: '', segmentName: mailbox.segmentName, segmentID: mailbox.segmentID, name: ' -- Default Mailbox --' };
          $scope.forwardAllMailboxes.push(lastDefault);
        }
        $scope.forwardAllMailboxes.push(mailbox);
      });
    });
  };
  $scope.ChangeSegmentPickMailbox = function (answer, mailbox) {
    answer.forwardSegmentID = mailbox.segmentID;
    answer.forwardMailboxID = mailbox.id;
  };
  $scope.SendSecureLinkClass = function () {
    return $scope.sendSecure === 'choose' && answer.notificationID ? 'nglink' : '';
  };
  var answerDetails = null;
  $scope.FilterSecureNotifications = function (autoResponse, index, array) {
    for (var l = 0; l < autoResponse.lists.length; l++) {
      if (answerDetails && answerDetails.segmentID === autoResponse.segmentID && autoResponse.lists[l] === 'securemessagenotification') return true;
    }
    return false;
  };
  $scope.SendSecureStart = function (ansDetails) {
    $scope.sendSecure = 'choose';
    answerDetails = ansDetails;
    $scope.SendSecureLoading = iservice.AdminList($http, 'ListSecure', ['autoresponses'], function (data) {
      for (var i = 0; i < data.autoResponses.length; i++) {
        var autoResponse = data.autoResponses[i];
        for (var l = 0; l < autoResponse.lists.length; l++) {
          if (answerDetails && answerDetails.segmentID === autoResponse.segmentID && autoResponse.lists[l] === 'securemessagenotification') {
            answer.notificationID = autoResponse.id;
            return;
          }
        }
      }
      $scope.sendSecure = 'none';
    });
  };
}]);

iServiceNg.controller('ControllerPropertyLogins', ['$scope', function ($scope) {
  $scope.AddLoginClick = function () {
    $scope.logins.push({ name: '', type: 'Email' });
  };
  $scope.RemoveLoginClick = function (index) {
    $scope.logins.splice(index, 1);
    if ($scope.logins.length < 1) {
      $scope.logins.splice(0, 0, { name: '', type: 'Email' });
    }
  };
  $scope.IsOpenIDLogin = function (login) {
    return login.type && login.type !== 'Email' && login.type !== 'Bounced';
  };
}]);
iServiceNg.controller('ControllerHistoryInfo', ['$scope', '$http', function ($scope, $http) {
  $scope.PropertyGroups = function (details) {
    if (!details) return [];
    return details.interactionProperties;
  };
  $scope.HavePropertyGroups = function (details) {
    if (!details) return false;
    return details.interactionProperties && details.interactionProperties.length > 0;
  };
  $scope.CaseGroups = function (details) {
    if (!details) return [];
    return details.caseProperties;
  };
  $scope.HaveCaseGroups = function (details) {
    if (!details) return false;
    return iservice.CountProperties(details.caseProperties) > 0;
  };
  $scope.CanStartEdit = function (details) {
    return false;
  };
  $scope.ShowEdit = function (details) {
    return false;
  };
  $scope.EditProperties = function (details) {
    details.editingProperties = true;
  };
  $scope.SaveProperties = function (details) {
    if (Running($scope.HistoryPropertySetRunning)) return;
    var values = iservice.CollectProperties(details.interactionProperties);
    var caseValues = iservice.CollectProperties(details.caseProperties);
    $scope.HistoryPropertySetRunning = iservice.HistoryPropertySet($http, details.id, values, caseValues, function (data) {
      if (!data.errors || data.errors.length === 0) {
        iservice.SanitizePropertyGroups(data.interactionProperties); details.interactionProperties = data.interactionProperties;
        iservice.SanitizePropertyGroups(data.caseProperties); details.caseProperties = data.caseProperties;
        details.editingProperties = false;
      }
    }, $scope.idPrefix);
  };
}]);

function FactorySegmentItems(factoryName, url, actionName, resultMemberName, loadFunc) {
  iServiceNg.factory(factoryName, ['iService.api.serial', function (API) {
    var n = factoryName;
    var list = [];
    var picker = [];
    var map = {};
    var segments = { list: [], map: {} };
    var segmentpickers = {};
    var loading = false;
    var promise;
    var updater = 0;
    function Load() {
      promise = API(url, null, actionName, {}).init(function () {
        loading = true;
      }).then(function (result) {
        list = result[resultMemberName];
        picker = [{ id: '', name: '-- None --', option: '-- None --' }];
        list.forEach(function (item) {
          Add(item);
        });
        if (loadFunc) loadFunc(list, map, service);
        updater++;
      }).catch(angular.noop);
    }
    function Add(item, segmentID) {
      picker.push(item);
      if (item.id) map[item.id] = item;
      var segmentID = segmentID || item.segmentID;
      if (segmentID) {
        var segmentItems = segments.map[segmentID];
        if (!segmentItems) {
          segmentItems = [];
          segments.map[segmentID] = segmentItems;
          segments.list.push(segmentItems);
          segmentpickers[segmentID] = [{ id: '', name: '-- None --', option: '-- None --' }];
        }
        segmentItems.push(item);
        segmentpickers[segmentID].push(item);
      }
    }
    var service = {
      list: function () {
        if (!loading) Load();
        return list;
      },
      picker: function () {
        if (!loading) Load();
        return picker;
      },
      segment: function (segmentID) {
        if (!loading) Load();
        if (segments.map[segmentID]) return segments.map[segmentID];
        return [];
      },
      segmentpicker: function (segmentID) {
        if (!loading) Load();
        if (segmentpickers[segmentID]) return segmentpickers[segmentID];
        return [];
      },
      add: function (item, segmentID) {
        list.push(item);
        Add(item, segmentID);
        if (loadFunc) loadFunc(list, map, service);
        updater++;
      },
      remove: function (item) {
        UpdateTable(list, item, 'id');
        UpdateTable(picker, item, 'id');
        delete map[item.id];
        segments.list.forEach(function (segmentItems) {
          UpdateTable(segmentItems, item, 'id');
        });
        updater++;
      },
      'getByName': function (itemName) {
        if (!loading) Load();
        for (var i = 0; i < list.length; i++) {
          if (list[i].name == itemName) return list[i];
        }
        return null;
      },
      'get': function (itemID) {
        if (!loading) Load();
        return map[itemID];
      },
      'then': function (func) {
        if (!loading) Load();
        return promise.then(func);
      },
      'catch': function (func) {
        if (!loading) Load();
        return promise.catch(func);
      },
      'finally': function (func) {
        if (!loading) Load();
        return promise.finally(func);
      },
      promise: function () {
        if (!loading) Load();
        return promise;
      },
      Watch: function () {
        return updater;
      }
    };
    return service;
  }]);
}
FactorySegmentItems('iService.mailboxesadmin', 'webapp-api-admin?api=mailboxesadminlist', 'ListMailboxes', 'mailboxes');
FactorySegmentItems('iService.mailboxes', 'webapp-api-admin?api=mailboxesall', 'ListMailboxes', 'mailboxes');
FactorySegmentItems('iService.autoResponses', 'webapp-api-admin?api=autoresponsesall', 'ListResponses', 'autoResponses');
FactorySegmentItems('iService.topics', 'webapp-api-admin?api=topicsall', 'ListTopics', 'topics', function (list, map, service) {
  list.forEach(function (topic) {
    iservice.SanitizePropertyGroups(topic.caseProperties);
    iservice.SanitizePropertyGroups(topic.answerProperties);
    iservice.SanitizePropertyGroups(topic.questionProperties);
  });
  service.CaseProperties = function (topicID) {
    var topic = map[topicID];
    if (topic) return angular.copy(topic.caseProperties);
    return [];
  }
  service.AnswerProperties = function (topicID) {
    var topic = map[topicID];
    if (topic) return angular.copy(topic.answerProperties);
    return [];
  }
  service.QuestionProperties = function (topicID) {
    var topic = map[topicID];
    if (topic) return angular.copy(topic.questionProperties);
    return [];
  }
});
FactorySegmentItems('iService.topicsDeleted', 'webapp-api-admin?api=topicsdeletedall', 'ListTopicsDeleted', 'topics');
FactorySegmentItems('iService.filters', 'webapp-api-admin?api=filtersall', 'ListFilters', 'filters');
FactorySegmentItems('iService.intProperties', 'webapp-api-admin?api=intpropertiesall', 'ListIntProp', 'intProperties');
FactorySegmentItems('iService.contactProperties', 'webapp-api-admin?api=contactpropertiesall', 'ListContactProp', 'contactproperties');
FactorySegmentItems('iService.articles', 'webapp-api-findanswer?api=articles', 'ListArticles', 'interactions');
FactorySegmentItems('iService.findanswerTopics', 'webapp-api-findanswer?api=topics', 'ListFindAnswerTopics', 'topics');

function FactoryTenantItems(factoryName, url, actionName, resultMemberName, loadFunc) {
  //loadFunc = func(list, map, service)
  iServiceNg.factory(factoryName, ['iService.api.serial', function (API) {
    var list = [];
    var picker = [];
    var map = {};
    var loading = false;
    var promise;
    var updater = 0;
    var data = {};
    var service = {
      list: function () {
        if (!loading) Load();
        return list;
      },
      picker: function () {
        if (!loading) Load();
        return picker;
      },
      add: function (item) {
        list.push(item);
        picker.push({ id: item.id, name: item.name });
        map[item.id] = item;
        if (loadFunc) loadFunc(list, map, service);
        updater++;
      },
      remove: function (item) {
        UpdateTable(list, item, 'id');
        UpdateTable(picker, item, 'id');
        delete map[item.id];
        updater++;
      },
      'get': function (itemID) {
        if (!loading) Load();
        return map[itemID];
      },
      'then': function (func) {
        if (!loading) Load();
        return promise.then(func);
      },
      'catch': function (func) {
        if (!loading) Load();
        return promise.catch(func);
      },
      'finally': function (func) {
        if (!loading) Load();
        return promise.finally(func);
      },
      promise: function () {
        if (!loading) Load();
        return promise;
      },
      data: function () {
        if (!loading) Load();
        return data;
      },
      Watch: function () {
        return updater;
      }
    };
    function Load() {
      promise = API(url, null, actionName, {}).init(function () {
        loading = true;
      }).then(function (result) {
        data = result;
        list = result[resultMemberName];
        picker = [{ id: '', name: '-- None --', option: '-- None --' }].concat(list);
        list.forEach(function (item) {
          if (item.id) map[item.id] = item;
        });
        if (loadFunc) loadFunc(list, map, service);
        updater++;
      }).catch(angular.noop);
    }
    return service;
  }]);
}
FactoryTenantItems('iService.segments', 'webapp-api-admin?api=segmentsall', 'ListSegments', 'segments', function (list, map, service) {
  var global = null;
  service.Reparent = function () {
    list.forEach(function (segment) {
      segment.children = [];
    });
    global = list.find(function (segment) {
      return !segment.parentID;
    }) || { id: '1', name: 'hidden', children: [], hidden: true };
    list.forEach(function (segment) {
      var parent = map[segment.parentID];
      if (parent) {
        parent.children.push(segment);
      }
      else if (segment != global) {
        global.children.push(segment);
      }
    });
    list.splice(0, list.length);
    global.depth = 0;
    SetDepth(global);
    function SetDepth(segment) {
      list.push(segment);
      segment.children.sort(function (l, r) {
        return (l.name.toLowerCase() < r.name.toLowerCase()) ? -1 : 1;
      });
      segment.children.forEach(function (child) {
        child.depth = segment.depth + 1;
        SetDepth(child);
      });
    }
  }
  service.Reparent();
  service.roots = function () {
    if (global) return global.children;
    return [];
  }
});
FactoryTenantItems('iService.forms', 'webapp-api-admin?api=formall', 'ListForms', 'forms');
FactoryTenantItems('iService.formsbuiltin', 'webapp-api-admin?api=formbuiltinall', 'ListFormsBuiltin', 'forms');
FactoryTenantItems('iService.websites', 'webapp-api-admin?api=websitesall', 'ListWebsites', 'websites');
FactoryTenantItems('iService.accessRights', 'webapp-api-admin?api=rightsall', 'ListRights', 'rights');
FactoryTenantItems('iService.usertypes', 'webapp-api-admin?api=usertypesall', 'ListUserTypes', 'usertypes');
FactoryTenantItems('iService.agents', 'webapp-api-admin?api=agentsall', 'ListAgents', 'agents');
FactoryTenantItems('iService.skills', 'webapp-api-admin?api=skillsall', 'ListSkills', 'skills');
FactoryTenantItems('iService.departments', 'webapp-api-admin?api=departmentsall', 'ListDepartments', 'departments');
FactoryTenantItems('iService.lists', 'webapp-api-admin?api=listsall', 'ListLists', 'lists');
FactoryTenantItems('iService.masslists', 'webapp-api-admin?api=masslistall', 'ListMasslists', 'masslists');
FactoryTenantItems('iService.massmessages', 'webapp-api-admin?api=massmessageall', 'ListMassmessages', 'massmessages');
FactoryTenantItems('iService.massmailings', 'webapp-api-admin?api=massmailingall', 'ListMasslists', 'massmailings');
FactoryTenantItems('iService.masscriterias', 'webapp-api-admin?api=masscriteriaall', 'ListMasscriterias', 'masscriterias');
FactoryTenantItems('iService.masscampaigns', 'webapp-api-admin?api=masscampaignall', 'ListMasscampaigns', 'masscampaigns');
FactoryTenantItems('iService.servicelevels', 'webapp-api-admin?api=servicelevelall', 'ListServicelevels', 'servicelevels');
FactoryTenantItems('iService.propertyTypes', 'webapp-api-admin?api=propertytypes', 'ListPropertyTypes', 'propertyTypes');
iServiceNg.factory('iService.stockresponses', ['iService.api.serial', function (API) {
  var list = [];
  var agent = [];
  var picker = [];
  var map = {};
  var segments = {};
  var seglist = [];
  var loading = false;
  var updater = 0;
  var promise;
  function Load() {
    promise = API('webapp-api-admin?api=stockresponseall', null, 'ListStock', {}).init(function () {
      loading = true;
    }).then(function (result) {
      result.segments.forEach(function (segment) {
        segment.stocks.forEach(function (stock) {
          var group = segments[segment.id];
          if (!group) {
            group = { id: segment.id, stocks: [] };
            segments[segment.id] = group;
            seglist.push(group);
          }
          group.stocks.push(stock);
          map[stock.id] = stock;
          stock.segmentID = segment.id;
          list.push(stock);
        });
      });
      result.agent.forEach(function (stock) {
        map[stock.id] = stock;
        list.push(stock);
        agent.push(stock);
      });
      picker = [{ id: '', name: '-- None --', option: '-- None --' }].concat(list);
      updater++;
    }).catch(angular.noop);
  }
  return {
    list: function () {
      if (!loading) Load();
      return list;
    },
    agent: function () {
      if (!loading) Load();
      return agent;
    },
    picker: function () {
      if (!loading) Load();
      return picker;
    },
    segment: function (segmentID) {
      if (!loading) Load();
      if (segments[segmentID]) return segments[segmentID].stocks;
      return [];
    },
    segments: function () {
      if (!loading) Load();
      return seglist;
    },
    addToSegment: function (item, segmentID) {
      var segment = segments[segmentID];
      if (!segment) {
        segment = { id: segmentID, stocks: [] };
        segments[segmentID] = segment;
        seglist.push(segments[segmentID]);
      }
      segment.stocks.push(item);
      map[item.id] = item;
      list.push(item);
      updater++;
    },
    remove: function (item) {
      UpdateTable(list, item, 'id');
      UpdateTable(picker, item, 'id');
      seglist.forEach(function (segment) {
        UpdateTable(segment.stocks, item, 'id');
      });
      UpdateTable(agent, item, 'id');
      delete map[item.id];
      updater++;
    },
    'get': function (itemID) {
      if (!loading) Load();
      return map[itemID];
    },
    'then': function (func) {
      if (!loading) Load();
      return promise.then(func);
    },
    'catch': function (func) {
      if (!loading) Load();
      return promise.catch(func);
    },
    'finally': function (func) {
      if (!loading) Load();
      return promise.finally(func);
    },
    promise: function () {
      if (!loading) Load();
      return promise;
    },
    Watch: function () {
      return updater;
    }
  };
}]);
iServiceNg.factory('iService.contactsearchfields', ['iService.api.serial', function (API) {
  var list = [];
  var picker = [];
  var loading = false;
  var promise;
  function Load() {
    promise = API('webapp-api-admin?api=contactsearchfields', null, 'ListContactSearchFields', {}).init(function () {
      loading = true;
    }).then(function (result) {
      result.fields.forEach(function (field) {
        list.push(field);
      });
      picker = [{ id: '', name: '-- Choose --' }].concat(list);
    }).catch(angular.noop);
  }
  return {
    list: function () {
      if (!loading) Load();
      return list;
    },
    picker: function () {
      if (!loading) Load();
      return picker;
    },
    'then': function (func) {
      if (!loading) Load();
      return promise.then(func);
    },
    'catch': function (func) {
      if (!loading) Load();
      return promise.catch(func);
    },
    'finally': function (func) {
      if (!loading) Load();
      return promise.finally(func);
    },
    promise: function () {
      if (!loading) Load();
      return promise;
    }
  };
}]);

iServiceNg.controller('ControllerActionSteps', ['$scope', function ($scope) {
  function StepTypeName(wholeStep) {
    var names = Object.keys(wholeStep);
    for (var i = 0; i < names.length; i++) {
      if (names[i] === '$$hashKey' || wholeStep[names[i]] == null) continue;
      return names[i];
    }
  }
  function StepRealPart(wholeStep) {
    var name = StepTypeName(wholeStep);
    if (name) return wholeStep[name];
  }
  $scope.IncludePath = function (step) {
    var name = StepTypeName(step);
    if (name) return name + '.html';
    return 'emptyStep.html';
  };
  $scope.StepClass = function (step) {
    var name = StepTypeName(step);
    if (name) return name;
    return 'emptyStep';
  };
  $scope.AddStep = function (steps) {
    steps.push({});
  };
  $scope.StepMoveUp = function (steps, step) {
    for (var i = 1; i < steps.length; i++) {
      if (step !== steps[i]) continue;
      steps.splice(i, 1);
      steps.splice(i - 1, 0, step);
      return;
    }
  };
  $scope.StepMoveDown = function (steps, step) {
    for (var i = 0; i < steps.length - 1; i++) {
      if (step !== steps[i]) continue;
      steps.splice(i, 1);
      if (i === steps.length - 1) steps.push(step);
      else steps.splice(i + 1, 0, step);
      return;
    }
  };
  $scope.StepDelete = function (steps, step) {
    for (var i = 0; i < steps.length; i++) {
      if (step !== steps[i]) continue;
      steps.splice(i, 1);
      return;
    }
  };
}]);
function InstallControllerStep($scope) {
  $scope.wholeStep = $scope.step;
  var names = Object.keys($scope.wholeStep);
  for (var i = 0; i < names.length; i++) {
    if (names[i] === '$$hashKey' || $scope.wholeStep[names[i]] == null) continue;
    $scope.step = $scope.wholeStep[names[i]];
    break;
  }
}
iServiceNg.controller('ControllerStep', ['$scope', function ($scope) {
  InstallControllerStep($scope);
}]);
iServiceNg.controller('ControllerStepChoose', ['$scope', function ($scope) {
  $scope.wholeStep = $scope.step;
  $scope.stepType = 'choose';
  $scope.stepsOptions = [
    { name: "Choose A Type..", value: "choose", group: "" },
    { name: "Admin Forms", value: "adminForms", group: "Forms Actions" },
    { name: "Admin Stock Responses", value: "adminStockResponses", group: "Forms Actions" },
    { name: "Admin User Types", value: "adminUsertypes", group: "Forms Actions" },
    { name: "Admin Departments", value: "adminDepartments", group: "Forms Actions" },
    { name: "Admin Segments", value: "adminSegments", group: "Forms Actions" },
    { name: "Admin Interaction Properties", value: "adminInteractionProperties", group: "Forms Actions" },
    { name: "Admin Mailboxes", value: "adminMailboxes", group: "Forms Actions" },
    { name: "Admin Topics", value: "adminTopics", group: "Forms Actions" },
    { name: "Admin Auto Responses", value: "adminAutoresponses", group: "Forms Actions" },
    { name: "Admin Agents", value: "adminAgents", group: "Forms Actions" },
    { name: "Admin Massmail Lists", value: "adminMassLists", group: "Forms Actions" },
    { name: "Admin Massmail Campaigns", value: "adminMassCampaigns", group: "Forms Actions" },
    { name: "Admin Massmail Criterias", value: "adminMassCriterias", group: "Forms Actions" },
    { name: "Admin Massmail Mailings", value: "adminMassMailings", group: "Forms Actions" },
    { name: "Admin Massmail Messages", value: "adminMassMessages", group: "Forms Actions" },
    { name: "Admin Massmail Subscriptions", value: "adminSubscriptions", group: "Forms Actions" },
    { name: "Massmail Send", value: "mailingsend", group: "Forms Actions" },
    { name: "Admin Lists (aka Additional Details)", value: "adminLists", group: "Forms Actions" },
    { name: "Admin OpenID", value: "adminOpenID", group: "Forms Actions" },
    { name: "Admin Websites", value: "adminWebsites", group: "Forms Actions" },
    { name: "Admin Skills", value: "adminSkills", group: "Forms Actions" },
    { name: "Captcha Verify", value: "captchaVerify", group: "Forms Actions" },
    { name: "Chat Create", value: "chatCreate", group: "Forms Actions" },
    { name: "Chat End", value: "chatEnd", group: "Forms Actions" },
    { name: "Chat Forward", value: "chatForward", group: "Forms Actions" },
    { name: "Chat Get Next", value: "chatGetNext", group: "Forms Actions" },
    { name: "Chat Keep Alive", value: "chatKeepAlive", group: "Forms Actions" },
    { name: "Chat Send", value: "chatSend", group: "Forms Actions" },
    { name: "Chat Toggle Availability", value: "chatToggleAvailability", group: "Forms Actions" },
    { name: "Create Agent Email", value: "agentEmail", group: "Forms Actions" },
    { name: "Create Note", value: "note", group: "Forms Actions" },
    { name: "Create Ticket/Ask A Question", value: "ticket", group: "Forms Actions" },
    { name: "Find/Create Contact", value: "contactFindCreate", group: "Forms Actions" },
    { name: "Customer Info Set Details", value: "customerInfoDetails", group: "Forms Actions" },
    { name: "Interaction Set Properties", value: "interactionHistoryProperties", group: "Forms Actions" },
    { name: "Logout", value: "logout", group: "Forms Actions" },
    { name: "Message Queue Get Approve", value: "messageQueueGetApprove", group: "Forms Actions" },
    { name: "Message Queue Get Next", value: "messageQueueGetNext", group: "Forms Actions" },
    { name: "Message Queue Save Resolve", value: "messageQueueSaveResolve", group: "Forms Actions" },
    { name: "Message Queue Remove From My Queue", value: "messageQueueUnqueue", group: "Forms Actions" },
    { name: "Message Queue QA Reject", value: "messageQueueQAReject", group: "Forms Actions" },
    { name: "Message Queue Change Topic", value: "messageQueueChangeTopic", group: "Forms Actions" },
    { name: "Message Queue Mark As Spam", value: "messageQueueMarkSpam", group: "Forms Actions" },
    { name: "Message Queue Save Draft", value: "messageQueueSaveDraft", group: "Forms Actions" },
    { name: "Message Queue Assign To Agent", value: "messageQueueAssign", group: "Forms Actions" },
    { name: "Message Queue Forward To Segment", value: "messageQueueChangeSegment", group: "Forms Actions" },
    { name: "Message Queue Forward To External Agent", value: "messageQueueForwardExternal", group: "Forms Actions" },
    { name: "Message Queue Save Note", value: "messageQueueSaveNote", group: "Forms Actions" },
    { name: "Message Queue Send", value: "messageQueueSend", group: "Forms Actions" },
    { name: "My Account Subscriptions", value: "myAccountSubscriptions", group: "Forms Actions" },
    { name: "My Account Details", value: "myAccountDetails", group: "Forms Actions" },
    { name: "Password Reset Send", value: "passwordResetSend", group: "Forms Actions" },
    { name: "Password Reset Confirm", value: "passwordResetConfirm", group: "Forms Actions" },
    { name: "Find Answer Create", value: "findAnswerCreate", group: "Forms Actions" },
    { name: "Find Answer Modify", value: "findAnswerModify", group: "Forms Actions" },
    { name: "Find Answer Subscribe", value: "findAnswerSubscribe", group: "Forms Actions" },
    { name: "Find Answer Feedback", value: "findAnswerFeedback", group: "Forms Actions" },
    { name: "Find Answer Send Notification", value: "findAnswerSendNotification", group: "Forms Actions" },
    { name: "Find Answer Delete", value: "findAnswerDelete", group: "Forms Actions" },

    { name: "Find Answer Delete", value: "findAnswerDelete", group: "Filter Actions" }, 
    { name: "Change Topic", value: "changeTopic", group: "Filter Actions" }, 
    { name: "Filter", value: "filter", group: "Filter Actions" }, 
    { name: "Forward External", value: "forwardExternal", group: "Filter Actions" }, 
    { name: "Resolve Interaction", value: "interactionResolve", group: "Filter Actions" }, 
    { name: "Round Robin Assignment", value: "roundRobin", group: "Filter Actions" }, 
    { name: "Select Agents", value: "selectAgents", group: "Filter Actions" }, 
    { name: "Send Auto Response", value: "sendAutoResponse", group: "Filter Actions" }, 
    { name: "Set Interaction Property", value: "interactionPropertySet", group: "Filter Actions" },

    { name: "Resolve Interaction", value: "interactionResolve", group: "Alerts Actions" },
    { name: "Select Agents", value: "selectAgents", group: "Alerts Actions" },
    { name: "Send Auto Response", value: "sendAutoResponse", group: "Alerts Actions" },
    { name: "Set Interaction Property", value: "interactionPropertySet", group: "Alerts Actions" }

  ];
  $scope.SelectType = function (type) {
    if (type === 'choose') return;
    $scope.wholeStep[type] = {};
  };
}]);
iServiceNg.controller('ControllerStepChangeTopic', ['$scope', '$http', 'iService.topics', function ($scope, $http, topics) {
  InstallControllerStep($scope);
  $scope.topics = topics;
}]);
iServiceNg.controller('ControllerStepFilter', ['$scope', '$http', 'iService.filters', function ($scope, $http, filters) {
  InstallControllerStep($scope);
  $scope.filters = filters;
}]);
iServiceNg.controller('ControllerStepInteractionSetParent', ['$scope', '$http', 'iService.intProperties', function ($scope, $http, intProperties) {
  InstallControllerStep($scope);
  $scope.intProperties = intProperties;
  if (!$scope.step.changeAction) $scope.step.changeAction = {};
  if (!$scope.step.changeAction.steps) $scope.step.changeAction.steps = [];
}]);
iServiceNg.controller('ControllerStepInteractionPropertySet', ['$scope', '$http', 'iService.intProperties', function ($scope, $http, intProperties) {
  InstallControllerStep($scope);
  $scope.intProperties = intProperties;
}]);
iServiceNg.controller('ControllerStepRoundRobin', ['$scope', '$http', 'iService.contactProperties', 'iService.departments', function ($scope, $http, contactProperties, departments) {
  InstallControllerStep($scope);
  $scope.contactProperties = contactProperties;
  $scope.departments = departments;
}]);
iServiceNg.controller('ControllerStepSendAutoResponse', ['$scope', '$http', 'iService.segments', 'iService.mailboxes', 'iService.autoResponses', function ($scope, $http, segments, mailboxes, autoResponses) {
  InstallControllerStep($scope);
  $scope.segments = segments;
  $scope.mailboxes = mailboxes;
  $scope.autoResponses = autoResponses;
}]);
iServiceNg.controller('ControllerStepSelectAgent', ['$scope', '$http', 'iService.agents', function ($scope, $http, agents) {
  InstallControllerStep($scope);
  $scope.agents = agents;
}]);
iServiceNg.factory('iService.agentStatus', ['iService.api.serial', 'iService.agents', 'iService.topics', '$rootScope', '$q', function (API, agents, topics, $rootScope, $q) {
  var loading = false;
  var updater = 0;
  var promise;

  //don't do anything until everything loaded
  agents.then(function () {
    agents.list().forEach(function (agent) {
      agent.haveSkill = {};
      agent.allskills.forEach(function (skillID) {
        agent.haveSkill[skillID] = true;
      });
      agent.haveSegment = {};
      agent.allsegments.forEach(function (segmentID) {
        agent.haveSegment[segmentID] = true;
      });
      agent.haveTopic = {};
    });
    topics.then(function () {
      topics.list().forEach(function (topic) {
        agents.list().forEach(function (agent) {
          agent.haveTopic[topic.id] = HaveAccess(agent, topic);
        });
      });
      CountAgents();
      var feedback = websync.FeedbackObject($rootScope);
      function MessageReceive(msg) {
        var update = msg.getData();
        var agent = agents.get(update.contactID);
        if (!agent) return;
        agent.isAvailable = update.isAvailable;
        agent.isLoggedIn = update.isLoggedIn;
        CountAgents();
      }
      feedback.Unsubscribe = websync.subscribe('/agentstatus/' + iservice.loggedIn.tenantID, MessageReceive, feedback);
    });
  });

  function CountAgents() {
    topics.then(function () {
      agents.then(function () {
        topics.list().forEach(function (topic) {
          topic.numAvailable = 0;
          topic.numAgents = 0;
          agents.list().forEach(function (agent) {
            if (CanChat(agent, topic)) topic.numAvailable++;
            if (CanHandle(agent, topic)) topic.numAgents++;
          });
        });
      });
    });
    updater++;
  }
  function CanChat(agent, topic) {
    if (!agent.isAvailable) return false;
    return agent.haveTopic[topic.id];
  }
  function CanHandle(agent, topic) {
    if (!agent.isLoggedIn) return false;
    return agent.haveTopic[topic.id];
  }
  function HaveAccess(agent, topic) {
    if (!agent.haveSegment[topic.segmentID]) return false;
    if (!topic.skillIDs || !topic.skillIDs.length) return true;
    return !topic.skillIDs.find(function (skillID) {
      return !agent.haveSkill[skillID];
    });
  }
  function Load() {
    loading = true;
    agents.list();
    topics.list();
    promise = $q.all([agents.promise(), topics.promise()]);
  }
  return {
    list: function () {
      if (!loading) Load();
      return agents.list();
    },
    'get': function (agentID) {
      if (!loading) Load();
      return agents.get(agentID);
    },
    topic: function (topicID) {
      if (!loading) Load();
      return topics.get(topicID);
    },
    'then': function (func) {
      if (!loading) Load();
      return promise.then(func);
    },
    'catch': function (func) {
      if (!loading) Load();
      return promise.catch(func);
    },
    'finally': function (func) {
      if (!loading) Load();
      return promise.finally(func);
    },
    promise: function () {
      if (!loading) Load();
      return promise;
    },
    Watch: function () {
      return updater;
    }
  }
}]);
iServiceNg.controller('ControllerSelectTopic', ['$scope', '$http', '$q', '$rootScope', 'iService.topics', 'iService.agentStatus', function ($scope, $http, $q, $rootScope, topics, agents) {
  topics.then(function () {
    SaveTopics();
    $scope.$watch('limitSegmentId', function (newValue, oldValue) {
      if (newValue) {
        SaveTopics();
        if ($scope.selectID !== undefined && !FindArray($scope.topics, function (topic) { return topic.id === $scope.selectId; })) {
          $scope.selectId = '';
        }
      }
    });
  });
  agents.list();

  $scope.changeTopic = function (topic) {
    $scope.$parent.selectId = topic.id;
    $scope.onSelectTopic({ selected: topic });
  };

  function SaveTopics() {
    $scope.topics = [];
    topics.list().forEach(function (topic) {
      if (!$scope.limitSegmentId || '' + $scope.limitSegmentId == topic.segmentID) {
        $scope.topics.push(topic);
      }
      if ($scope.$parent.selectId === topic.id) {
        $scope.onSelectTopic({ selected: topic });
      }
    });
  }
}]);
iServiceNg.controller('ControllerSelectStock', ['$scope', '$http', '$q', '$rootScope', function ($scope, $http, $q, $rootScope) {
  if (iservice.selectui.stockResponses) {
    SaveResponses();
  }
  else {
    iservice.StockResponsesGetAll($http, function (data) {
      iservice.selectui.stockResponses = [];
      ForKeys(data.responsesAll, function (responses, segmentID) {
        ForArray(responses, function (stock) {
          stock.segmentID = segmentID;
          stock.segmentName = data.segments[segmentID];
          iservice.selectui.stockResponses.push(stock);
        });
      });
      $rootScope.$broadcast('stockresponses-loaded');
    }, $scope.idPrefix)();
  }
  $scope.changeStock = function (stock) {
    $scope.$parent.selectId = stock.id;
  };
  $scope.$on('stockresponses-loaded', function () {
    SaveResponses();
  });
  function SaveResponses() {
    $scope.responses = [];
    ForArray(iservice.selectui.stockResponses, function (stock) {
      if (!$scope.limitSegmentId || $scope.limitSegmentId === stock.segmentID) {
        $scope.responses.push(stock);
      }
    });
  }
  $scope.$watch('limitSegmentId', function (newValue, oldValue) {
    SaveResponses();
  });
}]);
iServiceNg.controller('ControllerBodySidebar', ['$scope', 'iService.segments', function ($scope, segments) {
  $scope.segments = segments;

  var currentTab = 'builtinforms';
  if (iservice.HaveRight('Tab.Top.AdminTools')) currentTab = 'builtinforms';
  else if (iservice.HaveRight('Tab.Admin.AgentEmailForm')) currentTab = 'myforms';

  $scope.TabClass = function (tab) {
    if (currentTab == tab) return 'active';
    return '';
  }
  $scope.PaneClass = function (tab) {
    if (currentTab == tab) return 'active';
    return '';
  }
  $scope.defaultSegmenttab = '';

  if (typeof iservice.HaveRight('Tab.Segments.Details') == 'undefined') {
    $scope.defaultSegmenttab = 'stock';
  }
  $scope.checkSegmentAccess = function (segmentID) {
    if (iservice.loggedIn.accessibleSegments.includes(segmentID)) return true;
    return false;
  }
  $scope.TabClick = function (tab) {
    currentTab = tab;
  }
}]);
iServiceNg.directive('dontValidate', [function () {
  return {
    restrict: 'A',
    require: 'ngModel',
    link: function (scope, elem, attrs, ngModelController) {
      Object.keys(ngModelController.$validators).forEach(function (key) {
        ngModelController.$validators[key] = function (v) {
          return true;
        }
      });
    }
  };
}]);
iServiceNg.directive('setFocus', ['$timeout', function ($timeout) {
  //
  // https://github.com/christurnbull/ng-set-focus
  //
  return {
    restrict: 'A',
    link: function (scope, element, attrs) {

      var delay = 300;

      // set focus on broadcast
      scope.$on(attrs.setFocus, function (e) {
        $timeout(function () {
          element[0].focus();
        }, delay);

      });

      // apply default focus after other events have complete
      $timeout(function () {
        if (attrs.hasOwnProperty('setFocusDefault')) {
          element[0].focus();
        }
      }, delay);

      // fix for default focus on iOS. Does not show keyboard
      element.on('touchstart', function (event) {
        element[0].blur();
      });

    }
  };
}]);
iServiceNg.controller('ControllerSidebarNewSegment', ['$scope', '$rootScope', 'iService.api.serial', 'iService.rootPath', 'iService.segments', function ($scope, $rootScope, API, rootPath, segments) {
  $scope.segments = segments;
  iservice.MessagesContainer($scope);

  $scope.newSegment = { name: '' };
  $scope.showNew = false;
  $scope.ShowNew = function (idPrefix) {
    $scope.showNew = !$scope.showNew;
    if ($scope.showNew) {
      $rootScope.$broadcast('new-segment-name');
    }
  }
  $scope.HideNew = function () {
    $scope.showNew = false;
  }
  $scope.CreateSegment = function (idPrefix) {
    API('webapp-api-admin?api=segments&mode=new', idPrefix, 'SegmentCreate', { details: $scope.newSegment }).init(function () {
      $scope.messages.Clear();
    }).then(function (result) {
      //add new row to the list.
      segments.add(result.row);
      $scope.messages.createSuccess = 'Segment added successfully.';
      $scope.newSegment = { name: '' };
      $scope.showNew = false;
      window.location.assign(rootPath + 'admin#/segment/' + result.row.id);
    }).catch(function (errors) {
      $scope.messages.createErrors = errors;
    });
  }
}]);

