﻿var iservice = window.iservice || {};
(function (iservice) {
  iservice.components = iservice.components || {};

  //component manage agent email form admin page
  iservice.components.adminManageAgentEmail = {
    name: 'vueapp-admin-manage-agent-email',
    template: '#vueapp-admin-manage-agent-email',
    route() {
      return {
        path: iservice.formPath + '/admin/agent-email-admin', priority: 300,
        myformsUrl: iservice.formPath + '/admin/agent-email-admin', myformsLinkLabel: 'Manage Agent Email Form', myformsLinkClass: 'fa icon-c icon-quick-agent-email',
        canAccess() {
          return iservice.HaveRight('Tab.Admin.Segments');
        },
        meta: { title: "iService | Manage Agent Email" }
      }
    },
    setup(props) {
      var agentEmailConfig = Vue.reactive({});
      var agentEmailData = Vue.reactive({ loading: true, mailboxesList: [], segmentsList: [], segmentID: '', configuredSegments: {}, currentSegment: '', OnSelectSegment: Function, mailboxMaplength: Function });
      var formActions = Vue.reactive({ Save: Function, errors: [], success: '', ClearTopic: Function });
      agentEmailData.loading = false;

      var loggedIn = iservice.LoggedInData();
      var featureConfig = Vue.reactive({ loading: true, config: {} });
      //agentemail.Init(featureConfig.config);
      iservice.API('webapp-api-admin?api=configuration', null, { application: 'agentapp', name: 'agentEmail' }).then(data => {
        if (data.items.length > 0) {
          featureConfig.config = data.items[0].data;

          //agentemail.Init(featureConfig.config, data.items[0].data);
        }
      }).catch(errors => {
        featureConfig.errors = errors;
      }).finally(() => {
        featureConfig.loading = false;
      });

      var topics = iservice.TopicData();
      if (!topics.loaded) {
        topics.Load();
      }
      var segments = iservice.SegmentData();
      if (!segments.loaded) {
        segments.Load();
      }
      Vue.watchEffect(function () {
        if (!segments.list.length) return;
        var accessibleSegments = loggedIn.accessibleSegments;
        accessibleSegments.forEach(function (segmentID) {
          var segment = segments.get(segmentID);
          segment.disabled = !segment.isStandard;
          if (!segment.hidden)
            agentEmailData.segmentsList.push(segment);
        });
        agentEmailData.segmentsList.sort((l, r) => { return segments.TreeSortCompare(l.id, r.id); });
      })

      var mailboxes = iservice.MailboxData();
      if (!mailboxes.loaded) {
        mailboxes.Load();
      }
      agentEmailData.mailboxesList = mailboxes.list;

      var emptySegment = Vue.reactive({
        mailingLists: [],
        mailboxMap: {},
        expectingResponse: false
      });
      agentEmailData.mailboxMaplength = function (obj) {
        var keys = Object.keys(obj);
        return keys.length;
      }
      Vue.watchEffect(() => {
        if (featureConfig.config) {
          agentEmailData.configuredSegments = featureConfig.config;
          segments.list.forEach(segment => {
            if (featureConfig.config[segment.id]) {
              var segment = featureConfig.config[segment.id];
              agentEmailConfig[segment.id] = segment;
              if (!agentEmailData.segmentID) {
                agentEmailData.currentSegment = segment;
                agentEmailData.segmentID = segment.id;
              }
            }
          });
        }
      });

      agentEmailData.OnSelectSegment = function () {
        var segmentID = agentEmailData.segmentID;
        var segment = agentEmailConfig[segmentID];
        if (!segment) {
          segment = Object.assign({}, emptySegment);
          agentEmailConfig[segmentID] = segment;
        }
        segment.id = segmentID;
        agentEmailData.currentSegment = segment;
      }

      formActions.Save = function () {
        if (agentEmailData.loading) return;
        agentEmailData.loading = true;
        var json = JSON.stringify(agentEmailConfig);
        iservice.API('webapp-api-admin?api=configurationsave', null, { application: 'agentapp', items: [{ name: 'agentEmail', websiteID: '', data: json }] }).then(data => {
          iservice.SuccessMessage('Configuration saved.');
        }).catch(errors => {
          formActions.errors = errors;
        }).finally(() => {
          formActions.working = false;
          agentEmailData.loading = false;
        });
      }
      formActions.ClearTopic = function (segment, topicID) {
        delete segment.mailboxMap[topicID];
      }
      formActions.AddList = function (segment) {
        segment.mailingLists.push({});
      }
      formActions.DeleteList = function (segment, $index) {
        segment.mailingLists.splice($index, 1);
      }
      return { agentEmailData, agentEmailConfig, formActions, topics, mailboxes, segments };
    }
  }

  var config = {
    installed: true
  }
  iservice.components.quickAgentEmail = {
    name: 'vueapp-quick-agent-email',
    template: '#vueapp-quick-agent-email',
    route() {
      return {
        path: iservice.formPath + '/quick-agent-email', priority: 200,
        menuLinkLabel: 'Quick Agent Email', menuLinkClass: 'fa icon-c icon-quick-agent-email',
        canAccess() {
          return config.installed && iservice.HaveRight('Tab.Top.CustomerInfo');
        },
        meta: { title: "iService | Quick Agent Email" }
      }
    },
    setup() {
      var agentEmailForm = Vue.reactive({ configuration: true, configObj: '', HaveRight: Function, topicSelected: Function, details: {}, currentSegment: {}, list: {}, showccline: false, addccline: Function, previousSegmentID: null, featureProps: {}, featureCaseProps: {} });
      
      var featureUploads = iservice.FeatureUploads();

      agentEmailForm.HaveRight = function () {
        return iservice.HaveRight('Tab.Top.CustomerInfo');
      }
      agentEmailForm.addccline = function () {
        agentEmailForm.showccline = true;
      }
      var featureConfig = Vue.reactive({ loading: true, config: {} });
      //agentemail.Init(featureConfig.config);
      iservice.API('webapp-api-admin?api=configuration', null, { application: 'agentapp', name: 'agentEmail' }).then(data => {
        if (data.items.length > 0) {
          featureConfig.config = data.items[0].data;
          agentEmailForm.configuration = true;
          agentEmailForm.configObj = featureConfig.config;
          //agentemail.Init(featureConfig.config, data.items[0].data);
        } else {
          agentEmailForm.configuration = false;
        }
      }).catch(errors => {
        featureConfig.errors = errors;
      }).finally(() => {
        featureConfig.loading = false;
      });

      var formActions = Vue.reactive({ tab: 'recipient_details', errors: [], success: '', selectionCC: '' });
      formActions.TabClass = function (tab) { return { 'active': tab == formActions.tab }; }
      formActions.TabClick = function (tab) {
        formActions.tab = tab;
        if (tab == 'properties') {

        }
      }
      var editor = Vue.ref(null);
      var anchorStocks = Vue.ref(null);
      var featureStockResponses = Vue.reactive({ filter: '', segmentID: '' });
      
      var topics = iservice.TopicData();
      if (!topics.loaded) {
        topics.Load();
      }
      var segments = iservice.SegmentData();
      if (!segments.loaded) {
        segments.Load();
      }
      var mailboxes = iservice.MailboxData();
      if (!mailboxes.loaded) {
        mailboxes.Load();
      }
      var featureTopicMenu = Vue.reactive({ options: [], topicID: '' });
      Vue.watchEffect(function () {
        if (!agentEmailForm.configObj) return;
        featureTopicMenu.options.length = 0;
        if (!segments.list.length) return;
        if (!topics.list.length) return;
        if (!mailboxes.list.length) return;
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            var isDisabled = false;
            var defaultMailbox = '';
            if (agentEmailForm.configObj[topic.segmentID]) {
              var segment = segments.get(topic.segmentID);
              defaultMailbox = mailboxes.get(segment.mailboxID).replyTo;
              if (!agentEmailForm.configObj[topic.segmentID].mailboxMap[topic.id]) isDisabled = true;
              group.options.push({ name: topic.name, id: topic.id, level: Math.min(2, topic.depth), isdisabled: isDisabled, defaultMailbox: defaultMailbox });
            }
          
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.id, r.id); });
          if (group.options.length < 1) return;
          featureTopicMenu.options.push(group);
        });
      })
      agentEmailForm.topicSelected = function () {
       
        if (!agentEmailForm.details.topicID) return;
        var topic = topics.get(agentEmailForm.details.topicID);
        var segmentID = topic.segmentID;
        if (!agentEmailForm.previousSegmentID || agentEmailForm.previousSegmentID != segmentID) {
          agentEmailForm.previousSegmentID = segmentID;
          featureStockResponses.segmentID = segmentID;
          agentEmailForm.currentSegment = agentEmailForm.configObj[segmentID];
          if (agentEmailForm.currentSegment && agentEmailForm.currentSegment.id) {
            agentEmailForm.details.expectingResponse = agentEmailForm.currentSegment.expectingResponse;
          }
        }
        startEmail();
        
      }
      featureStockResponses.Open = function () {
        featureStockResponses.Close = iservice.components.popupHolder.Open(anchorStocks, 'stock-responses-menu', false, featureStockResponses);
      }
      featureStockResponses.PasteVar = function (text) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteText(text);
      }
      featureStockResponses.PasteStock = function (html) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteHtml(html);
      }
      function startEmail() {
        iservice.API('webapp-api-custinfo?api=startagentemail', null, { topicID: agentEmailForm.details.topicID }).then(function (data) {
          agentEmailForm.featureProps = Vue.reactive({ label: 'Answer Properties', length: 0 });
          var props = data.properties.length ? data.properties[0].properties : [];
          var caseprops = data.caseProperties.length ? data.caseProperties[0].properties : [];
          iservice.EditableProperties(agentEmailForm.featureProps, props);

          agentEmailForm.featureCaseProps = Vue.reactive({ label: 'Case Properties', length: 0 });
          iservice.EditableProperties(agentEmailForm.featureCaseProps, caseprops);
          Vue.watchEffect(() => {
            agentEmailForm.featureCaseProps.length = agentEmailForm.featureCaseProps.props.length;
            agentEmailForm.featureProps.length = agentEmailForm.featureProps.props.length;
            formActions.haveProperties = agentEmailForm.featureCaseProps.props.length || agentEmailForm.featureProps.props.length;
          });
          Vue.watchEffect(() => { formActions.propertyMissingClass = { 'text-red': agentEmailForm.featureCaseProps.missingRequired || agentEmailForm.featureProps.missingRequired }; });
          Vue.watchEffect(() => { formActions.hasValues = agentEmailForm.featureCaseProps.hasValues || agentEmailForm.featureProps.hasValues; });
        }).catch(function (errors) {
          formActions.errors = errors;
        });
      }

      formActions.PasteList = function (list) {
        if (!list) return;
        var element = $('#cc');
        element.focus();
        if (formActions.selectionCC != null)
          restoreSelection(formActions.selectionCC);
        insertIntoTextarea(list, agentEmailForm.details);
      }
      formActions.ReadyPasteCC = function () {
        formActions.selectionCC = saveSelection();
      }
     
      function Clear() {
        agentEmailForm.details.topicID = null;
        agentEmailForm.details.destEmail = null;
        agentEmailForm.details.cc = null;
        agentEmailForm.list.selected = null;
        agentEmailForm.currentSegment = {}
        agentEmailForm.showccline = false;
        agentEmailForm.details.subject = null;
        agentEmailForm.details.bodyHtml = "";
        agentEmailForm.details.mailboxID = null;
        agentEmailForm.details.properties = null;
        agentEmailForm.details.caseProperties = null;
        agentEmailForm.featureProps.props = [];
        agentEmailForm.featureCaseProps.props = [];
        agentEmailForm.previousSegmentID = null;
        featureUploads.Reset();
        tinyMCE.activeEditor.setContent('');
      }
      formActions.handleTagRemove = function (option, event) {
        var cc = agentEmailForm.details.cc.split(',');
        var index = agentEmailForm.list.selected.indexOf(option.cc);
        var options = option.cc.split(',');

        if (index !== -1) {
          agentEmailForm.list.selected.splice(index, 1);

        }
        options.forEach(function (item) {
          var index2 = cc.indexOf(item);
          if (index2 !== -1) {
            cc.splice(index2, 1);
            agentEmailForm.details.cc = cc.join(',');
          }
        })
        
      }
      formActions.SendEmail = function () {
        var err = false;
        if (agentEmailForm.details.topicID == undefined || agentEmailForm.details.topicID == '') {
          formActions.errors = ['Topic cannot be blank.'];
          err = true;
          return false;
        }
        if (agentEmailForm.details.destEmail == undefined || agentEmailForm.details.destEmail == '') {
          formActions.errors= ['To email address cannot be blank.'];
          err = true;
          return false;
        }

        if (agentEmailForm.details.subject == '' || agentEmailForm.details.subject == undefined) {
          formActions.errors = ['Subject cannot be blank.'];
          err = true;
          return false;
        }

        if (agentEmailForm.details.bodyHtml == '' || agentEmailForm.details.bodyHtml == undefined) {
          formActions.errors = ['Body cannot be blank.'];
          err = true;
          return false;
        }

        if (err == false) {
          if (agentEmailForm.currentSegment && agentEmailForm.currentSegment.id) {
            agentEmailForm.details.mailboxID = agentEmailForm.currentSegment.mailboxMap[agentEmailForm.details.topicID];
          }
          agentEmailForm.details.properties = agentEmailForm.featureProps.out;
          agentEmailForm.details.caseProperties = agentEmailForm.featureCaseProps.out;
          agentEmailForm.details.addSubjectRefNum = true;
          iservice.API('webapp-api-custinfo?api=forwardemail', null, { details: agentEmailForm.details, email: agentEmailForm.details.destEmail }, featureUploads.files).then(function (data) {
            iservice.SuccessMessage("Email Sent Successfully");
            Clear();
          }).catch(function (errors) {
            formActions.errors = errors;
          });
        }
      }

      return { agentEmailForm, formActions, editor, featureStockResponses, featureTopicMenu, anchorStocks, featureUploads };
    }
  }
  function saveSelection(){
    if (window.getSelection) {
      var sel = window.getSelection();
      if (sel.getRangeAt && sel.rangeCount) {
        return sel.getRangeAt(0);
      }
    } else if (document.selection && document.selection.createRange) {
      return document.selection.createRange();
    }
    return null;
  }
  function insertIntoTextarea(text, formDetails) {
    if (!text) return;
    formDetails.cc = text.join(',');
  }
  function restoreSelection(range) {
    if (range) {
      if (window.getSelection) {
        var sel = window.getSelection();
        sel.removeAllRanges(); sel.addRange(range);
      } else if (document.selection && range.select) {
        range.select();
      }
    }
  }

})(iservice);