﻿var iservice = window.iservice || {};
(function (iservice) {
  iservice.components = iservice.components || {};

  iservice.components.commonAgentChat = {
    name: 'common-agent-chat',
    template: '#common-agent-chat',
    setup(props) {
      var Chat = Vue.reactive({ chatData: {}, chatClosetime: '', timeWait: '', timeChat: '' });
      var loggedIn = iservice.LoggedInData();
      var chats = iservice.ChatData();

      function HourMinSec(totalSeconds) {
        function twoDigits(val) {
          return (val < 10) ? '0' + val : val;
        }
        if (totalSeconds < 0) totalSeconds = -totalSeconds;
        if (totalSeconds > 35999999) totalSeconds = 35999999;
        var hours = Math.floor(totalSeconds / 3600);
        var minutes = Math.floor((totalSeconds - hours * 3600) / 60);
        var seconds = Math.floor(totalSeconds - hours * 3600 - minutes * 60);
        return (hours > 0) ? (hours + ':' + twoDigits(minutes) + ':' + twoDigits(seconds)) :
          (minutes + ':' + twoDigits(seconds));
      }
      Chat.websyncChat = {};

      Chat.showChat = function () {
        return iservice.HaveRight('Chat.Answer');
      };

      Chat.haveChats = function () {
        return loggedIn.chats.length > 0;
      };

      Chat.getChats = function () {
        iservice.data.loggedIn.chats.forEach(function (chat) {
          if (chat.startMs) return;
          chat.startMs = (new Date()).valueOf();
          chats.add(chat)
        })
        return iservice.data.loggedIn.chats;
      };
      Chat.updateTimer = setTimeout(function () {
        UpdateWait();
      }, 1000);
      function UpdateWait() {
        now = (new Date()).valueOf();
        Chat.getChats().forEach(function (item) {
          let chat = chats.get(item.id);
          if (chat) {
            var sinceMs = now - item.startMs;
            if (chat.status == 'ChatWaiting') {
              chat.timeWait = HourMinSec((chat.elapsedWaitMs + sinceMs) / 1000);
            }
            else {
              item.timeChat = HourMinSec((chat.elapsedChatMs + sinceMs) / 1000);
            }
          }
        });
        Chat.updateTimer = setTimeout(function () {
          UpdateWait();
        }, 1000);
      }
      /**
    * Convert URLs in a string to anchor buttons
    * @param {!string} string
    * @returns {!string}
    */
      Chat.URLify = function (string) {
        var urls = string.match(/(((ftp|https?):\/\/)[\-\w@:%_\+.~#?,&\/\/=]+)/g);
        if (urls) {
          urls.forEach(function (url) {
            string = string.replace(url, '<a target="_blank" href="' + url + '">' + url + "</a>");
          });
        }
        return string.replace("(", "<br/>(");
      }

      var activeChatsIdsStorage = iservice.activeChatsIdsStorage();
      Chat.selectChat = function (chat, force) {
        if (chat.isDeleted)
          return;
        chat.stockResponseGroups = Vue.reactive([]);
        chat.isActive = force || !chat.isActive;
        var activeIds = activeChatsIdsStorage.getData(),
          activeIdIndex = activeIds.indexOf(chat.id),
          arrayIsDirty = false;

        if (chat.isActive) {
          if (activeIdIndex < 0) {
            activeIds.push(chat.id);

            arrayIsDirty = true;
          }

          chat.chatData = {
            chatLines: []
          };

          iservice.API('webapp-api-agentchat', null, { 'api': 'details', 'chatID': chat.id, 'customerID': chat.customerID }).then(function (HttpResponse) {
            chat.custDetails = HttpResponse.custDetails;
            chat.chatData = HttpResponse.chatData;
          });

          if (!chat.stockResponseGroups.length) {
            iservice.API('dotnetapp-json-stockresponses', null, { 'segmentID': chat.segmentID }).then(function (HttpResponse) {
              HttpResponse.forEach(function (group) {
                chat.stockResponseGroups.push(group)
              });
            });
          }

          chat.unsubscribeChat = websync.subscribe('/agentchat/' + loggedIn.tenantID + '/' + chat.id, function (e) {
            chat.chatData.chatLines.push(e.getData());
          });
        }
        else {
          if (activeIdIndex >= 0) {
            activeIds.splice(activeIdIndex, 1);

            arrayIsDirty = true;
          }
          if (chat.unsubscribeChat)
            chat.unsubscribeChat();
        }

        if (arrayIsDirty)
          activeChatsIdsStorage.setData(activeIds);
      };

      function deleteChatObject(chat) {
        if (chat.removePromise)
          clearTimeout(chat.removePromise);

        loggedIn.chats = loggedIn.chats.filter(function (ch) {
          return ch.id !== chat.id;
        });
      }
      function removeChat(id) {
        var chat = loggedIn.chats.find(function (ch) {
          return ch.id === id;
        });

        if (!chat)
          return;

        deleteChatObject(chat);

        if (chat.unsubscribeChat)
          chat.unsubscribeChat();

        if (loggedIn.chats.length > 0 && !loggedIn.chats.some(function (ch) {
          return ch.isActive;
        })) {
          Chat.selectChat(loggedIn.chats[0]);
        }
        else
          activeChatsIdsStorage.clearData();
      }
      function chatInboxMessage(chat) {
        var existing = loggedIn.chats.find(function (ch) {
          return ch.id === chat.id;
        });

        if (!existing) {
          if (!chat.isDeleted) {
            chat.isActive = true;

            loggedIn.chats.push(chat);

            Chat.selectChat(chat, true);
          }

          return;
        }

        if (chat.isDeleted) {
          if (!chat.isActive) {
            existing.isDeleted = true;
            Chat.chatClosetime = 10;
            existing.removePromise = setTimeout(function () {
              existing.removePromise = null;

              removeChat(existing.id);
            }, 10000);
            setInterval(function () {
              Chat.chatClosetime = Chat.chatClosetime - 1;
              Chat.chatClosetimer = iservice.HourMinSec(Chat.chatClosetime)
            }, 1000);

          }
          else
            deleteChatObject(existing);
        }
        else {
          localStorage.setItem('lastMessageTime', new Date().toISOString());
          Object.assign(existing, chat);
        }
      }
      Vue.provide('removeChat', removeChat);
      Chat.chatAnswer = function () {
        iservice.API('webapp-api-agentchat?api=answer', null, {}).then(function (data) {
        }).finally(function () {
        });
      }

      function agentStatusMessage(message) {
        loggedIn.isAvailable = message.isAvailable;

        if (loggedIn.numChats === 0 && message.numChats > 0) {
          setTimeout(function () {
            if (loggedIn.isAvailable && loggedIn.isLoggedIn && iservice.HaveRight('Chat.Answer')) {

              var sound = new buzz.sound(rootPath +"sounds/2Rings", {
                formats: ["ogg", "mp3", "aac"], preload: true, autoplay: true, loop: false
              });

              sound.play()

              Push.create('iService Chat Request', {
                body: 'An iService chat is waiting to be answered',
                icon: rootPath +'images/iserviceicon.png',
                timeout: 30000,
                vibrate: [100, 100, 100],
                onClick: function () {
                  window.focus();

                  Chat.chatAnswer();

                  this.close();
                }
              });
            }
          }, 500);
        }

        loggedIn.numChats = message.numChats;
      }
      function agentMessageReceive(msg) {
        var message = msg.getData();
        if (message.agentStatus)
          agentStatusMessage(message.agentStatus);
        else if (message.chatInboxUpdate)
          chatInboxMessage(message.chatInboxUpdate);
      }
      function WebsyncSubscribeChat() {
        WebsyncCleanup();
        if (iservice.HaveRight('Chat.Answer')) {
          Chat.websyncChat.Unsubscribe = websync.subscribe('/agent/' + loggedIn.tenantID + '/' + loggedIn.contactID, agentMessageReceive, Chat.websyncChat);
          var activeChatsIds = activeChatsIdsStorage.getData();
          if (!activeChatsIds.length && loggedIn.chats.length > 0)
            Chat.selectChat(loggedIn.chats[0]);
          else {
            loggedIn.chats.forEach(function (ch) {
              if (activeChatsIds.indexOf(ch.id) >= 0)
                Chat.selectChat(ch, true);
            });
          }
        }
      }
      function WebsyncCleanup() {
        if (Chat.websyncChat.Unsubscribe) {
          Chat.websyncChat.Unsubscribe();
          Chat.websyncChat.Unsubscribe = null;
        }
        iservice.data.loggedIn.chats.forEach(function (ch) {
          if (ch.unsubscribeChat) {
            ch.unsubscribeChat();
            ch.unsubscribeChat = null;
          }
        });
      }
      WebsyncSubscribeChat();
      Vue.computed(function () {
        WebsyncSubscribeChat();
      })
      Chat.nameStyle = iservice.nameStyle;
      Chat.nameInitials = iservice.nameInitials;
      Chat.AgoDateFormat = function (date) {
        return moment(date).fromNow();
      }
      var chatUtilities = iservice.chatUtilities();
      
      Chat.Send = function (chat) {
        Chat.errors = [];
        var chatboxID = document.getElementById('chatBox' + chat.id);
        chat.message = chatboxID.innerHTML;
        chat.sendRunning = true;
        if (chat.message == undefined || chat.message == '') {
          Chat.errors= ["Please enter a message"];
          chat.sendRunning = false;
          return;
        }
        if (chat.isDeleted)
          return;
        var chatobject = {
          interactionID: chat.id,
          message: chatUtilities.setLinksTargetToBlank(chat.message)
        }
        iservice.API('webapp-api-agentchat?api=send', null, chatobject ).then(function (response) {
          chatboxID.innerText = '';
        }).catch(function (errors) {
          chat.sendRunning = false;
          Chat.errors = errors;
        }).finally(function () {
          chat.sendRunning = false;
        });
      };
      var chatFiltersStorage = iservice.chatFiltersStorage();
      Chat.filter = chatFiltersStorage.getData();
      Chat.filterChange = function () {
        chatFiltersStorage.setData(Chat.filter);
      };
      Chat.pasteStock = function (chat, response) {
        var inputArea = document.querySelector('#chatBox' + chat.id);
        if (!inputArea) return;
        inputArea.focus();
        inputArea.innerHTML = response.bodyHtml;
      }
      Chat.filterResponses = function (item) {
        var res = item.description.match(new RegExp(Chat.filter.tag, 'i'));
        var res2 = item.name.match(new RegExp(Chat.filter.search, 'i'));
        if (res && res2) {
          return true;
        }
        return false;
      }
      return { Chat }
    }
  }
  iservice.components.agentChatActionButtonsDesktop = {
    name: 'agent-chat-action-buttons-desktop',
    template: '#agent-chat-action-buttons-desktop',
    props: ['chat'],
    setup(props) {
      var anchorForward = Vue.ref(null);
      var feature = Vue.reactive({});
      var removeChat = Vue.inject('removeChat');
      var chat = Vue.computed(function () {
        return props.chat;
      })
      var featureForwardMenu = Vue.reactive({ chat: chat.value, removeChat: removeChat });

      featureForwardMenu.Open = function () {
        featureForwardMenu.Close = iservice.components.popupHolder.Open(anchorForward, 'agent-chat-menu-forward', true, featureForwardMenu);
      }

      var anchorTopic = Vue.ref(null);

      var featureTopicMenu = Vue.reactive({ chat: chat.value, removeChat: removeChat});

      featureTopicMenu.Open = function () {
        featureTopicMenu.Close = iservice.components.popupHolder.Open(anchorTopic, 'agent-chat-change-topic-menu', true, featureTopicMenu, null, "chat-view popup-holder");
      }

      feature.endChat = function (chat) {
        if (chat.isDeleted)
          return;
        chat.endRunning = true;
        iservice.API('webapp-api-agentchat?api=end', null, { interactionID: chat.id }).then(function () {
          removeChat(chat.id);
          chat.endRunning = false;
        });
      };
      return { anchorForward, featureForwardMenu, featureTopicMenu, anchorTopic, chat, feature }
    }
  }
  iservice.components.agentChatActionButtonsMobile = {
    name: 'agent-chat-action-buttons-mobile',
    template: '#agent-chat-action-buttons-mobile',
    props: ['chat'],
    setup(props) {
      var anchorForward = Vue.ref(null);
      var feature = Vue.reactive({});
      var removeChat = Vue.inject('removeChat');
      var chat = Vue.computed(function () {
        return props.chat;
      })
      var featureForwardMenu = Vue.reactive({ chat: chat.value, removeChat: removeChat });

      featureForwardMenu.Open = function () {
        featureForwardMenu.Close = iservice.components.popupHolder.Open(anchorForward, 'agent-chat-menu-forward', true, featureForwardMenu);
      }

      var anchorTopic = Vue.ref(null);

      var featureTopicMenu = Vue.reactive({ chat: chat.value, removeChat: removeChat });

      featureTopicMenu.Open = function () {
        featureTopicMenu.Close = iservice.components.popupHolder.Open(anchorTopic, 'agent-chat-change-topic-menu', true, featureTopicMenu);
      }

      feature.endChat = function (chat) {
        if (chat.isDeleted)
          return;
        chat.endRunning = true;
        iservice.API('webapp-api-agentchat?api=end', null, { interactionID: chat.id }).then(function () {
          removeChat(chat.id);
          chat.endRunning = false;
        });
      };
      return { anchorForward, featureForwardMenu, featureTopicMenu, anchorTopic, chat, feature }
    }

  }
  iservice.components.agentChatMenuForward = {
    name: 'agent-chat-menu-forward',
    template: '#agent-chat-menu-forward',
    props: ['params'],
    setup(props) {
      var loggedIn = iservice.LoggedInData();
      var agents = iservice.AgentData();
      var feature = Vue.computed(() => { return props.params; });

      if (!agents.loaded) agents.Load();
      var reassignStage = Vue.ref('closed');
      var reassignOptions = Vue.ref([]);
      var getChatAgents = function (chatID) {
        if (reassignStage.value == 'open') {
          reassignStage.value = 'closed';
        }
        else if (reassignStage.value == 'closed') {
          reassignStage.value = 'open';
        }
        iservice.API('webapp-api-agentchat?api=agentlist', null, { interactionID: chatID}).then(function (response) {
          reassignOptions.loading = false;
          response.forEach(function (agent) {
            if (agent.contactID == loggedIn.contactID) return;
            var agent = agents.get(agent.contactID);
            if (agent) {
              var name = agent.name || agent.email || ('Agent ' + agent.id);
              var option = {
                id: agent.id,
                name: name,
                email: agent.email,
                search: agent.name + '\n' + agent.email,
                disabled: !agent.isLoggedIn && !agent.isAvailable
              };
              reassignOptions.value.push(option);
            }
            reassignOptions.value.sort((l, r) => {
              return l.name.localeCompare(r.name);
            });
          })
        });
      }
      function assignedTo(chatID, assignToID) {
        iservice.API('webapp-api-agentchat?api=forward&mode=assign', null, { interactionID: chatID, assignToID: assignToID }).then(function () {
          iservice.SuccessMessage("Forward to agent successfully.");
          feature.value.removeChat(chatID);
        }).catch(function (errors) {

        }).finally(function () {

        })
      }
      var Unqueue = function (chat) {
        if (chat.isDeleted)
          return;
        iservice.API('webapp-api-agentchat?api=forward&mode=unqueue', null, { interactionID: chat.id }).then(function () {
          iservice.SuccessMessage("Unassigned successfully.");
          feature.value.removeChat(chat.id);
        }).catch(function (errors) {

        }).finally(function () {

        })
      };
      return { feature, reassignOptions, reassignStage, getChatAgents, assignedTo, Unqueue }
    }
  }
  iservice.components.agentChatMenuChangeTopic = {
    name: 'agent-chat-change-topic-menu',
    template: '#agent-chat-change-topic-menu',
    props: ['params'],
    setup(props) {
      var feature = Vue.computed(() => { return props.params; });
      var topics = iservice.TopicData();
      var segments = iservice.SegmentData();
      var agents = iservice.AgentData();

      function CanChat(agent, topic) {
        if (!agent.isAvailable) return false;
        return agent.haveTopic[topic.id];
      }
      function CanHandle(agent, topic) {
        if (!agent.isLoggedIn) return false;
        return agent.haveTopic[topic.id];
      }

      var topicGroups = Vue.computed(() => {
        var groups = [];
        if (!segments.loaded || !topics.loaded || !agents.loaded) {
          segments.Load(); topics.Load(); agents.Load();
        }
        if (!segments.list.length || !topics.list.length || !agents.list.length) return;
        topics.list.forEach(function (topic) {
          agents.list.forEach(function (agent) {
            topic.numAvailable = 0;
            topic.numAgents = 0;
            agents.list.forEach(function (agent) {
              agent.haveTopic = {}
              agent.haveTopic[topic.id] = iservice.HaveAccess(agent, topic);
              if (CanChat(agent, topic)) topic.numAvailable++;
              if (CanHandle(agent, topic)) topic.numAgents++;
            })
          })
        })
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            group.options.push({ name: topic.name, id: topic.id, level: Math.min(2, topic.depth), disabled: topic.numAvailable < 1 });
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.value, r.value); });
          if (group.options.length < 1) return;
          groups.push(group);
        });
        return groups;
      });
      var ChangeTopic = function (chat) {
        if (chat.isDeleted)
          return;
        iservice.API('webapp-api-agentchat?api=forward&mode=topic', null, { interactionID: chat.id, topicID: chat.topicID }).then(function () {
          iservice.SuccessMessage("Topic changed successfully.");
        }).catch(function (errors) {

        }).finally(function () {

        })
      };
      return { feature, topicGroups, ChangeTopic }
    }
  }

  iservice.components.chatView = {
    name: 'chat-view',
    template: '#chat-view',
    props: ['itemid'],
    emits: ['onLoadDone'],
    setup(props, context) {
      var Chat = Vue.reactive({ chatData: {}, chatClosetime: '' });
      var loggedIn = iservice.LoggedInData();
      Chat.websyncChat = {};

      Chat.getChats = Vue.computed(() => {
        let data = iservice.data.chats.list.find(x => x.id == props.itemid);
        return data ? [data] : [];
      });

      /**
    * Convert URLs in a string to anchor buttons
    * @param {!string} string
    * @returns {!string}
    */
      Chat.URLify = function (string) {
        var urls = string.match(/(((ftp|https?):\/\/)[\-\w@:%_\+.~#?,&\/\/=]+)/g);
        if (urls) {
          urls.forEach(function (url) {
            string = string.replace(url, '<a target="_blank" href="' + url + '">' + url + "</a>");
          });
        }
        return string.replace("(", "<br/>(");
      }

      var activeChatsIdsStorage = iservice.activeChatsIdsStorage();
      Chat.selectChat = function (chat, force, expand) {
        if (chat.isDeleted)
          return;

        if (expand) {
          chat.chatData = {
            chatLines: []
          };

          iservice.API('webapp-api-agentchat', null, { 'api': 'details', 'chatID': chat.id, 'customerID': chat.customerID }).then(function (HttpResponse) {
            chat.custDetails = HttpResponse.custDetails;
            chat.chatData = HttpResponse.chatData;
            context.emit('onLoadDone');
          });

          if (!chat.stockResponseGroups) {
            iservice.API('dotnetapp-json-stockresponses', null, { 'segmentID': chat.segmentID }).then(function (HttpResponse) {
              chat.stockResponseGroups = HttpResponse;
            });
          }

          chat.unsubscribeChat = websync.subscribe('/agentchat/' + loggedIn.tenantID + '/' + chat.id, function (e) {
            chat.chatData.chatLines.push(e.getData());
          });

          return;
        }
      };

      function deleteChatObject(chat) {
        if (chat.removePromise)
          clearTimeout(chat.removePromise);

        iservice.data.chats.list = iservice.data.chats.list.filter(function (ch) {
          return ch.id !== chat.id;
        });
      }
      function removeChat(id) {
        var chat = iservice.data.chats.list.find(function (ch) {
          return ch.id === id;
        });

        if (!chat)
          return;

        deleteChatObject(chat);

        if (chat.unsubscribeChat)
          chat.unsubscribeChat();

        if (iservice.data.chats.list.length > 0 && !iservice.data.chats.list.some(function (ch) {
          return ch.isActive;
        })) {
          Chat.selectChat(iservice.data.chats.list[0]);
        }
        else
          activeChatsIdsStorage.clearData();
      }
      
      Vue.provide('removeChat', removeChat);

      Chat.nameStyle = iservice.nameStyle;
      Chat.nameInitials = iservice.nameInitials;
      Chat.AgoDateFormat = function (date) {
        return moment(date).fromNow();
      }
      
      Vue.watchEffect(() => {
        if (props.itemid) {
          let logChat = iservice.data.chats.get(props.itemid);
          if (logChat) {
            Chat.selectChat(logChat, false, true);
          }
        }
      });
      
      return { Chat }
    }
  }
  iservice.components.chatViewLoader = {
    name: 'chat-view-loader',
    template: '#chat-view-loader',
    props: ['itemid', 'threads'],
    setup(props) {
      var loading = Vue.ref(true);
      var itemid = props.itemid;
      function onLoadDone() {
        loading.value = false;
      }

      return { loading, onLoadDone, itemid };
    }
  };
  iservice.components.manageChats = {
    name: 'vueapp-messagequeue-manage-chats',
    template: '#vueapp-messagequeue-manage-chats',
    route() { return { path: iservice.formPath + '/manage-chats', priority: 200, name: 'manage-chats', meta: { title: "iService | Manage Chats" } } },
    setup() {
      var segments = iservice.SegmentData();
      var topics = iservice.TopicData();
      var agents = iservice.AgentData();
      var loggedIn = iservice.LoggedInData();
      var chats = iservice.ChatData();
      var featureSearch = Vue.reactive({ working: true, chats: [], errors: [], success: null });
      Vue.watchEffect(() => {
        chats.showIds.forEach((id) => {
          let obj = chats.get(id);
          if (obj) InitChat(obj);
        });
        featureSearch.chats = chats.showIds;
      });

      var updateTimer = setTimeout(function () {
        UpdateWait();
      }, 1000);
      function UpdateWait() {
        var now = (new Date()).valueOf();
        featureSearch.chats.forEach(function (id) {
          if (chatsShown.value.includes(id)) {
            let chat = chats.get(id);
            if (chat) {
              var sinceMs = now - chat.startMs;
              
              if (chat.status == 'ChatWaiting') {
                chat.timeWait = HourMinSec((chat.elapsedWaitMs + sinceMs) / 1000);
              }
              else {
                chat.timeChat = HourMinSec((chat.elapsedChatMs + sinceMs) / 1000);
              }
            }
          }
        });
        updateTimer = setTimeout(function () {
          UpdateWait();
        }, 1000);
      }
      function HourMinSec(totalSeconds) {
        function twoDigits(val) {
          return (val < 10) ? '0' + val : val;
        }
        if (totalSeconds < 0) totalSeconds = -totalSeconds;
        if (totalSeconds > 35999999) totalSeconds = 35999999;
        var hours = Math.floor(totalSeconds / 3600);
        var minutes = Math.floor((totalSeconds - hours * 3600) / 60);
        var seconds = Math.floor(totalSeconds - hours * 3600 - minutes * 60);
        return (hours > 0) ? (hours + ':' + twoDigits(minutes) + ':' + twoDigits(seconds)) :
          (minutes + ':' + twoDigits(seconds));
      }
      chats.WhenDeleted(id => {
        var index = featureSearch.chats.indexOf(id);
        if (index == -1) return;
        featureSearch.chats.splice(index, 1);
      });
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }
      var featureChats = Vue.reactive({ showWaiting: true, showActive: true });
      var featureTopics = Vue.reactive({ showMine: true, segments: [], list: [], selectedMap: {}, collapsedMap: {}, selectAll: true, collapsed: false });
      var selectAll = featureTopics.selectAll;
      Vue.watchEffect(() => {
        selectAll = featureTopics.selectAll;
      });
      featureTopics.Collapse = function () {
        featureTopics.collapsed = !featureTopics.collapsed;
      }
      Vue.watchEffect(() => {
        featureTopics.collapsedClass = featureTopics.collapsed ? 'col-md-1 treeview-collapse' : 'col-md-4 treeview-expand';
        featureChats.collapsedClass = featureTopics.collapsed ? 'col-md-11 treeview-expand' : 'col-md-8';
      });
      var selectStartup = true;
      featureTopics.SelectAll = function () {
        selectStartup = false;
        featureTopics.list.forEach(topic => {
          featureTopics.selectedMap[topic.id] = featureTopics.selectAll;
        });
      }
      featureTopics.ShowMine = function () {
        selectStartup = false;
      }
      Vue.watchEffect(() => {
        if (!segments.loaded) segments.Load();
        if (!topics.loaded) topics.Load();
        if (!agents.loaded) agents.Load();
        featureTopics.segments.length = 0;
        var map = {};
        featureTopics.list.length = 0;
        segments.list.forEach(seg => {
          var segmentID = seg.id;
          var segment = Vue.reactive({
            name: seg.name,
            topics: []
          });
          topics.inSegment(segmentID).forEach(topic => {
            if (featureTopics.showMine) {
              var agentList = agents.forTopic(topic.id);
              var me = agentList.find(agentID => {
                var agent = agents.get(agentID);
                return agent.id == loggedIn.contactID;
              });
              if (!me) return;
            }
            var item = map[topic.id] = Vue.reactive({
              id: topic.id,
              name: topic.name,
              visibility: topic.visibility,
              children: []
            });
            featureTopics.list.push(item);
            if (selectStartup) featureTopics.selectedMap[item.id] = selectAll;
          });
          topics.inSegment(segmentID).forEach(topic => {
            var row = map[topic.id];
            if (!row) return;
            var parentID = topic.parentID;
            var parent = map[parentID];
            while (!parent) {
              if (!parentID) break;
              parentID = topics.get(parentID).parentID;
              parent = map[parentID];
            }
            (parent ? parent.children : segment.topics).push(row);
          });
          if (segment.topics.length < 1) return;
          featureTopics.segments.push(segment);
        });
        function SortRow(lrow, rrow) { return topics.TreeSortCompare(lrow.id, rrow.id); }
        featureTopics.segments.sort(SortRow);
        featureTopics.list.forEach(row => {
          row.children.sort(SortRow);
        });
      });
      Vue.watchEffect(() => {
        featureTopics.selectAll = false;
        for (var topic of featureTopics.list) {
          if (!featureTopics.selectedMap[topic.id]) return;
        }
        featureTopics.selectAll = true;
      });

      function ComputedStatusList(status) {
        return Vue.computed(() => {
          var chatTopic = {};
          featureTopics.list.forEach(topic => {
            chatTopic[topic.id] = [];
          });
          featureSearch.chats.forEach(id => {
            var chat = chats.get(id);
            if (!chat) return;
            var list = chatTopic[chat.topicID];
            if (!list) return;
            if (chat.status == status) list.push(id);
          });
          return chatTopic;
        });
      }
      var chatWaitingInTopic = ComputedStatusList('ChatWaiting');
      var chatInProgressInTopic = ComputedStatusList('ChatInProgress');
      var agentsForTopic = Vue.computed(() => {
        var forTopic = {};
        featureTopics.list.forEach(topic => {
          var item = forTopic[topic.id] = { count: 0, agentNameList: '' };
          var agentList = agents.forTopic(topic.id);
          var separator = '';
          agentList.forEach(agentID => {
            var agent = agents.get(agentID);
            if (!agent || !agent.isLoggedIn) return;
            item.count++;
            item.agentNameList += separator + agent.displayname;
            separator = ', ';
          });
        });
        return forTopic;
      });
      function CountDisplay(value) {
        if (Array.isArray(value)) value = value.length;
        if (!value || value < 1) return '--';
        if (value < 1000) return '' + value;
        return '1K+';
      }
      Vue.provide('selectedMap', featureTopics.selectedMap);
      Vue.provide('collapsedMap', featureTopics.collapsedMap);
      Vue.provide('chatWaitingInTopic', chatWaitingInTopic);
      Vue.provide('chatInProgressInTopic', chatInProgressInTopic);
      Vue.provide('agentsForTopic', agentsForTopic);
      Vue.provide('CountDisplay', CountDisplay);


      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        componentExpanded: 'chat-view-loader',
        filterLabel: 'Filter Chats',
        emptyMessage: 'No Chats',
        pageSize: 50,
        expandable: true,
        searching: false,
        initialSort: 'Wait Time',
        exportName: "Manage Chats"
      });
      tableConfig.searching = true;
      var inc = 0;
      var chatsShown = Vue.computed(() => {
        var list = [];
        featureTopics.list.forEach(topic => {
          if (featureTopics.selectedMap[topic.id]) {
            if (featureChats.showActive) {
              chatInProgressInTopic.value[topic.id].forEach(id => { list.push(id); });
            }
            if (featureChats.showWaiting) {
              chatWaitingInTopic.value[topic.id].forEach(id => { list.push(id); });
            }
          };
          inc++;
          if (inc === featureTopics.list.length) {
            tableConfig.searching = false;
          }
        });
        return list;
      });

      Vue.watchEffect(() => {
        tableConfig.columns = [
          Vue.reactive({
            name: 'expand', hidable: false,
            filterText: function () { },
            componentHead: 'iservice-table-head-select-all', componentCell: 'iservice-table-cell-expand-multiselect', shown: true, tdClass: 'col-icon-check expand-td'
          }),
          Vue.reactive({
            name: 'Subject', label: 'Subject',
            filterText: iservice.tables.SimpleFilterTextFunc(chats, 'subject'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Contact Name', label: 'Contact Name',
            filterText: iservice.tables.SimpleFilterTextFunc(chats, 'customerName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-interaction-customername-linked', shown: true
          }),
          Vue.reactive({
            name: 'Time In Chat', label: 'Time In Chat',
            filterText: iservice.tables.SimpleFilterTextFunc(chats, 'timeChat'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Wait Time', label: 'Wait Time',
            filterText: iservice.tables.SimpleFilterTextFunc(chats, 'timeWait'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Assigned To', label: 'Assigned To',
            filterText: iservice.tables.SimpleFilterTextFunc(chats, 'operatorName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Topic', label: 'Topic',
            filterText: iservice.tables.SimpleFilterTextFunc(chats, 'topicName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          })];
      });
      var selectedIDs = Vue.ref([]);
      var chatList = Vue.ref(null);

      var websyncData = {
        unsubForSegment: {},
        Clear() {
          for (var id in websyncData.unsubForSegment) {
            var unsub = websyncData.unsubForSegment[id];
            if (unsub) unsub();
            websyncData.unsubForSegment[id] = null;
          }
        }
      };
      Vue.onUnmounted(() => {
        if (updateTimer) {
          clearTimeout(updateTimer);
        }
        websyncData.Clear();
      });
      iservice.WatchLoginChange(() => {
        websyncData.Clear();
        for (var segmentID of loggedIn.accessibleSegments) {
          websyncData.unsubForSegment[segmentID] = websync.subscribe('/supervisechats/' + loggedIn.tenantID + '/' + segmentID, ManageChatsKeep);
        }
      });
      function InitChat(chat) {
        chat.startMs = (new Date()).valueOf();
        chat.timeWait = HourMinSec(chat.elapsedWaitMs / 1000);
        chat.timeChat = HourMinSec(chat.elapsedChatMs / 1000);
      }
      function ManageChatsKeep(chat) {
        let data = chat.getData().inboxUpdate;
        loadChats();
        var keep = (data.status == 'ChatWaiting' || data.status == 'ChatInProgress');
        return keep;
      }
      function loadChats() {
        iservice.API('webapp-api-mq-managechats?api=chatlist', null, null).then(data => {
          data.chats.forEach(chat => {
            let index = iservice.data.chats.list.findIndex(x => x.id == chat.id);
            if (index == -1) {
              InitChat(chat);
              chats.add(chat);
            } else {
              for (key in chat) {
                iservice.data.chats.map[chat.id][key] = chat[key];
              }
            }
          });
          iservice.data.chats.list.forEach(chat => {
            let isExist = data.chats.find(x => x.id == chat.id);
            if (!isExist) {
              delete iservice.data.chats.map[chat.id];
              iservice.data.chats.showIds = iservice.data.chats.showIds.filter(x => x.id != chat.id);
              iservice.data.chats.list = iservice.data.chats.list.filter(x => x.id != chat.id);
            }
          });
          if (data.chats.length == 0) {
            iservice.data.chats.map = {};
            iservice.data.chats.showIds = [];
            iservice.data.chats.list = [];
          }
        }).catch(errors => {
          featureSearch.errors = errors;
        }).finally(() => {
          featureSearch.working = false;
        });
      }
      loadChats();
      return { chatList, featureSearch, featureTopics, featureChats, chatsShown, tableConfig, selectedIDs, ListCollapsed };
    }
  }
})(iservice);