﻿var iservice = window.iservice || {};
(function (iservice) {
  iservice.components = iservice.components || {};
  iservice.tables = iservice.tables || {};

  iservice.components.iserviceMessageNotification = {
    name: 'iservice-message-notification',
    template: '#iservice-message-notification',
    props: ['feature'],
    setup(props) {
      var timeoutID;
      var errors = Vue.ref([]);
      var success = Vue.ref(null);
      Vue.watchEffect(() => {
        errors.value = props.feature.errors;
        success.value = props.feature.success;
        if (props.feature.success && props.feature.success != '') {
          setTimeout(() => {
            success.value = null;
            errors.value = null;
          }, 2000);
        }
        props.feature.errors = null;
        props.feature.success = null;
      });
      function Clear() {
        errors.value = null;
        success.value = null;
      }
      function Expand(error) {
        error.showDetails = !error.showDetails;
      }
      return { Clear, Expand, errors, success };
    }
  };
  iservice.components.popupHolder = {
    name: 'popup-holder',
    template: '#popup-holder',
    Open: function () {},
    setup() {
      var showing = Vue.ref(false);
      var alignOnLeft = Vue.ref(false);
      var componentContent = Vue.ref(null);

      var popupEl = Vue.ref(null);
      var anchorEl = null;
      var componentParams = Vue.ref(null);
      var style = Vue.ref({});
      function UpdatePosition() {
        if (!showing.value) return;
        if (!anchorEl || !anchorEl.value) {
          showing.value = false;
          return;
        }
        var anchorRect = anchorEl.value.getBoundingClientRect();
        if (!popupEl.value) return;
        var containerEl = popupEl.value.parentElement;
        if (!containerEl) return;
        var containerRect = containerEl.getBoundingClientRect();
        if (!popupEl.value) return;
        var popupRect = popupEl.value.getBoundingClientRect();

        if (alignOnLeft.value) {
          style.value.left = (anchorRect.left - containerRect.left) + 'px';
        }
        else {
          style.value.left = (anchorRect.right - containerRect.left - popupRect.width) + 'px';
        }
        style.value.top = (anchorRect.bottom - containerRect.top) + 'px';
      }
      var intervalID = setInterval(UpdatePosition, 33);

      iservice.components.popupHolder.Open = function (anchorRef, componentName, onLeft, params, customStyle, className) {
        if (anchorRef === anchorEl && showing.value == true) {
          showing.value = false;
          return;
        }
        showing.value = true;
        anchorEl = anchorRef;
        componentContent.value = componentName;
        alignOnLeft.value = onLeft;
        componentParams.value = params;
        style.value = Vue.reactive({
          position: 'absolute',
          'z-index': '2000',
          'max-height': '400px',
          'overflow-y': 'visible'
        });
        if (customStyle) Object.getOwnPropertyNames(customStyle).forEach(name => {
          style.value[name] = customStyle[name];
        });
        if (className) {
          popupEl.value.className = className;
        }
        UpdatePosition();
        return function () {
          if (anchorEl !== anchorRef) return;
          showing.value = false;
        }
      }

      Vue.onUnmounted(() => {
        clearInterval(intervalID);
        window.removeEventListener('click', ClickListener);
      });

      function ClickListener(e) {
        if (!showing.value) return;
        if (!popupEl.value) return;
        if (!anchorEl.value) return;
        if (!e.target) return;
        var box = e.target.getBoundingClientRect();
        if (box.width == 0 || box.height == 0) return;
        if (!popupEl.value.contains(e.target) && !anchorEl.value.contains(e.target)) {
          showing.value = false;
        }
      }
      window.addEventListener('click', ClickListener);

      return { showing, popupEl, componentContent, componentParams, style };
    }
  };
  iservice.components.loadingPlaceholder = {
    name: 'loading-placeholder',
    template: '#loading-placeholder',
    setup() {
      return { };
    }
  };
  iservice.components.propertyEdit = {
    name: 'property-edit',
    template: '#property-edit',
    props: ['property'],
    setup() {
      return { };
    }
  };
  iservice.components.propertyEditFloatingLabel = {
    name: 'property-edit-floating-label',
    template: '#property-edit-floating-label',
    props: ['property'],
    setup() {
      return { };
    }
  };
  iservice.components.propertyEditInputs = {
    name: 'property-edit-inputs',
    template: '#property-edit-inputs',
    props: ['property', 'value', 'i'],
    setup(props) {
      var control = Vue.ref(null);
      Vue.watchEffect(() => {
        if (!props.property.focusMe) return;
        props.property.focusMe = false;
        setTimeout(() => {
          if (!control || !control.value || !control.value.focus) return;
          control.value.focus();
        });
      });

      Vue.watchEffect(() => {
        if (props.property.def.hasDescriptions && props.property.def.descriptionChoices.length > 0) {
          if (!props.value.description) {
            props.value.description = props.property.def.descriptionChoices[0];
          }
        }
      });

      return { control };
    }
  };
  iservice.components.littleSpinner = {
    name: 'little-spinner',
    template: '#little-spinner',
    props: ['loading'],
    setup(props) {

      return { };
    }
  };
  iservice.components.bigSpinner = {
    name: 'big-spinner',
    template: '#big-spinner',
    props: ['loading'],
    setup(props) {

      return { };
    }
  };
  iservice.components.iserviceSwitch = {
    name: 'iservice-switch',
    template: '#iservice-switch',
    props: ['modelValue', 'disabled'],
    emits: ['changed', 'update:modelValue'],
    setup(props, context) {
      function Input(e) {
        context.emit('update:modelValue', e.target.checked);
        context.emit('changed', e.target.checked);
      }

      return { Input };
    }
  }
  iservice.components.interactionTypeDisplay = {
    name: 'interaction-type-display',
    template: '#interaction-type-display',
    props: ['itemid'],
    setup(props) {
      var interactions = iservice.InteractionData();

      var label = Vue.computed(() => {
        var int = interactions.get(props.itemid);
        if (!int) return '';
        return ' ' + int.type + ' ';
      });
      var style = Vue.computed(() => {
        var int = interactions.get(props.itemid);
        if (!int) return '';
        return iservice.InteractionTypeClass(int.type);
      });
      var icon = Vue.computed(() => {
        var int = interactions.get(props.itemid);
        if (!int) return '';
        return iservice.InteractionTypeIcon(int.type);
      });
      return { label, style, icon };
    }
  }
  iservice.components.interactionStatusDisplay = {
    name: 'interaction-status-display',
    template: '#interaction-status-display',
    props: ['itemid', 'status'],
    setup(props) {
      var interactions = iservice.InteractionData();

      var label = Vue.computed(() => {
        if (props.status) return props.status;
        if (props.itemid) {
          var int = interactions.get(props.itemid);
          if (!int) return '';
          return int.status;
        }
      });
      var style = Vue.computed(() => {
        if (props.status) {
          return iservice.InteractionStatusClass(props.status);
        }
        if (props.itemid) {
          var int = interactions.get(props.itemid);
          if (!int) return '';
          return iservice.InteractionStatusClass(int.status);
        }
      });
      return { label, style };
    }
  }
  iservice.components.interactionCaseStatusDisplay = {
    name: 'interaction-case-status-display',
    template: '#interaction-case-status-display',
    props: ['itemid', 'status'],
    setup(props) {
      var interactions = iservice.InteractionData();

      var label = Vue.computed(() => {
        if (props.status) return props.status;
        if (props.itemid) {
          var int = interactions.get(props.itemid);
          if (!int) return '';
          return int.currentStatus;
        }
      });
      var style = Vue.computed(() => {
        if (props.status) {
          return iservice.statuses.colorClass(props.status);
        }
        if (props.itemid) {
          var int = interactions.get(props.itemid);
          if (!int) return '';
          return int.statusClass;
        }
      });
      return { label, style };
    }
  }
  iservice.components.iserviceTinymceEditor = {
    //example <tinymce-editor v-model=""></tinymce-editor>
    // major change for syntax in vue3 for v-model with custom component https://v3.vuejs.org/guide/migration/v-model.html#migration-strategy
    name: 'tinymce-editor',
    props: ['modelValue'],
    emits: ['update:modelValue'],
    template: '#tinymce-template',
    setup(props, context) {
      var containerdiv = Vue.ref(null);
      var editor = null;
      var allowUndo = true;

      var lastUpdated = props.modelValue;
      function updateValue(value) {
        context.emit('update:modelValue', value);
        lastUpdated = value;
      }
      Vue.watchEffect(() => {
        if (props.modelValue == lastUpdated) return;
        if (!editor) return;
        editor.setContent(props.modelValue);
      });

      function PasteText(text) {
        if (!editor) return;
        editor.execCommand('mceReplaceContent', false, text);
      }
      function PasteHtml(html) {
        if (!editor) return;
        var endId = tinymce.DOM.uniqueId();
        var endSpan = '<span id="' + endId + '">&nbsp;</span>';
        editor.execCommand('mceFocus');
        editor.execCommand('mceInsertRawHTML', false, html + endSpan);
        var endEl = editor.dom.select('span#' + endId);
        if (endEl && endEl.length > 0) {
          editor.selection.select(endEl[0]);
          editor.execCommand('mceReplaceContent', false, '');
        }
      }

      Vue.onMounted(() => {
        var useDarkMode = window.matchMedia('(prefers-color-scheme: dark)').matches;
        tinymce.init({
          target: containerdiv.value.children[0],
          branding: false,
          height: 500,
          setup: function (e) {
            editor = e;
            editor.on('Change keyup undo', function (e) {
              updateValue(editor.getContent())
            });
            editor.on('BeforeAddUndo', function (e) {
              return allowUndo;
            });
          },
          init_instance_callback: function (editor) {
            var freeTiny = document.querySelector('.tox .tox-notification--in');
            if (freeTiny) freeTiny.style.display = 'none';
          },
          menubar: false,
          plugins: [
            'advlist autolink lists link image charmap print preview anchor',
            'searchreplace visualblocks code fullscreen',
            'insertdatetime media table paste code help wordcount'
          ],
          fontsize_formats: "8px 10px 12px 14px 18px 24px 36px",
          toolbar: 'undo redo | bold italic underline strikethrough | fontselect fontsizeselect formatselect | alignleft aligncenter alignright alignjustify | outdent indent |  numlist bullist checklist | forecolor backcolor casechange permanentpen formatpainter removeformat | pagebreak | charmap emoticons | fullscreen  preview save print | insertfile image media pageembed template link anchor codesample | a11ycheck ltr rtl | showcomments addcomment | code',
          autosave_ask_before_unload: true,
          autosave_interval: '30s',
          autosave_prefix: '{path}{query}-{id}-',
          autosave_restore_when_empty: false,
          autosave_retention: '2m',
          image_advtab: true,
          importcss_append: true,
          templates: [
            { title: 'New Table', description: 'creates a new table', content: '<div class="mceTmpl"><table width="98%%"  border="0" cellspacing="0" cellpadding="0"><tr><th scope="col"> </th><th scope="col"> </th></tr><tr><td> </td><td> </td></tr></table></div>' },
            { title: 'Starting my story', description: 'A cure for writers block', content: 'Once upon a time...' },
            { title: 'New list with dates', description: 'New List with dates', content: '<div class="mceTmpl"><span class="cdate">cdate</span><br /><span class="mdate">mdate</span><h2>My List</h2><ul><li></li><li></li></ul></div>' }
          ],
          template_cdate_format: '[Date Created (CDATE): %m/%d/%Y : %H:%M:%S]',
          template_mdate_format: '[Date Modified (MDATE): %m/%d/%Y : %H:%M:%S]',
          height: 600,
          image_caption: true,
          quickbars_selection_toolbar: 'bold italic | quicklink h2 h3 blockquote quickimage quicktable',
          noneditable_noneditable_class: 'mceNonEditable',
          toolbar_mode: 'sliding',
          spellchecker_ignore_list: ['Ephox', 'Moxiecode'],
          tinycomments_mode: 'embedded',
          content_style: '.mymention{ color: gray; }',
          // contextmenu: 'link image imagetools table configurepermanentpen',
          contextmenu: false,
          a11y_advanced_options: true,
          skin: useDarkMode ? 'oxide-dark' : 'oxide',
          paste_data_images: true,
          content_style: 'body { font-family:Helvetica,Arial,sans-serif; font-size:14px }'
        });
      });

      return { containerdiv, PasteText, PasteHtml };
    },
  };
  iservice.InteractionTypeClass = function (type) {
    switch (type) {
      case 'Agent Notification':
      case 'Message Acknowledgment':
        return 'bg-gray-disabled';
      case 'Contact Creation': 
      case 'External Agent Notification': 
      case 'Mass Mailing': 
      case 'Mass Mailing Article Update': 
      case 'Password Reset': 
      case 'Alert':
      case 'Private Email':
        return 'bg-gray-disabled';
      case 'Secure Response':
      case 'Agent Email':
      case 'Agent Response':
        return 'bg-green';
      case 'Customer Email':
      case 'Agent Ticket':
      case 'Customer Ticket':
        return 'bg-blue';
      case 'Chat':
        return 'bg-blue disabled';
      case 'Secure Email':
        return 'bg-green-active';
      case 'Public Note':
      case 'Private Note':
      case 'Resolve Note':
      case 'Unsent Response':
      case 'Find Answer Article':
        return 'bg-teal-active';
      case 'Secure Notification':
        return 'bg-gray-disabled';
    }
    return '';
  }
  iservice.InteractionTypeIcon = function (type) {
    switch (type) {
      case 'Agent Notification':
        return 'icon-c icon-agent-notification-2';
      case 'Message Acknowledgment':
        return 'fa-reply';
      case 'Contact Creation':
        return 'fa-address-book-o';
      case 'External Agent Notification':
        return 'icon-c icon-external-notification-1';
      case 'Mass Mailing':
      case 'Customer Email':
        return 'fa-envelope';
      case 'Mass Mailing Article Update':
        return 'icon-c icon-find-answer-update';
      case 'Password Reset':
        return 'fa-key';
      case 'Alert':
        return 'fa-exclamation-triangle';
      case 'Private Email':
      case 'Secure Email':
        return 'icon-c icon-secure-email';
      case 'Secure Response':
        return 'icon-c icon-secure-response';
      case 'Agent Email':
        return 'icon-c icon-agent-email-3';
      case 'Agent Response':
        return 'icon-c icon-agent-response';
      case 'Agent Ticket':
        return 'fa-ticket';
      case 'Customer Ticket':
        return 'icon-c icon-customer-ticket';
      case 'Chat':
        return 'fa-comments';
      case 'Public Note':
        return 'fa-file-text-o';
      case 'Private Note':
        return 'icon-c icon-public-note';
      case 'Resolve Note':
        return 'icon-c icon-resolve-note';
      case 'Unsent Response':
        return 'icon-c icon-unsent-response';
      case 'Find Answer Article':
        return 'fa-search';
      case 'Secure Notification':
        return 'fa-bell';
    }
    return '';
  }
  iservice.InteractionStatusClass = function (status) {
    switch (status) {
      case 'ChatInProgress': return 'bg-green label';
      case 'Sent': return 'bg-green-active label';
      case 'Unsent': return 'bg-red label';
      case 'Unassigned': return 'bg-red label';
      case 'Assigned': return 'label-info';
      case 'PrivateNote': return 'bg-green-disabled label';
      case 'PublicNote': return 'bg-green-disabled label';
      case 'Resolved': return 'bg-green-active label';
      case 'SentWaitingReply': return 'bg-green-disabled label';
      case 'Queued': return 'bg-red-disabled label';
      case 'ChatWaiting': return 'bg-red-disabled label';
      case 'Ignored': return 'bg-red label';
      case 'Pending': return 'bg-yellow-disabled label';
    }
    return '';
  }
  iservice.RowColorClass = function (type) {
    switch (type) {
      case 'Agent Email': return 'bg-answers-sent';
      case 'Agent Response': return 'bg-answers-sent';
      case 'Message Acknowledgment': return 'bg-answers-sent';
      case 'Secure Email': return 'bg-answers-sent';
      case 'Note': return 'bg-answers';
      case 'Public Note': return 'bg-answers';
      case 'Private Note': return 'bg-answers';
      case 'Resolve Note': return 'bg-answers';
      case 'Unsent Response': return 'bg-answers';
      case 'Find Answer': return 'bg-answers';
      case 'Ticket': return 'bg-questions';
      case 'Ask A Question': return 'bg-questions';
      case 'Customer Email': return 'bg-questions';
      case 'Agent Ticket': return 'bg-questions';
      case 'Customer Ticket': return 'bg-questions';
      case 'Chat': return 'bg-questions';
      case 'Agent Notification':
      case 'Message Acknowledgment':
      case 'External Agent Notification':
      case 'Secure Notification': return 'bg-notification';
    }
    return '';
  }
  iservice.types = {
    'Customer Email': { isAnswer: false, isQuestion: true, isNotification: false },
    'Private Email': { isAnswer: false, isQuestion: true, isNotification: false },
    'Agent Response': { isAnswer: true, isQuestion: false, isNotification: false },
    'Agent Email': { isAnswer: true, isQuestion: false, isNotification: false },
    'Alert': { isAnswer: false, isQuestion: false, isNotification: true },
    'Agent Notification': { isAnswer: false, isQuestion: false, isNotification: true },
    'External Agent Notification': { isAnswer: false, isQuestion: false, isNotification: true },
    'Password Reset': { isAnswer: false, isQuestion: false, isNotification: true },
    'Message Acknowledgment': { isAnswer: false, isQuestion: false, isNotification: true },
    'Secure Notification': { isAnswer: false, isQuestion: false, isNotification: true },
    'Customer Ticket': { isAnswer: false, isQuestion: true, isNotification: false },
    'Ask A Question': { isAnswer: false, isQuestion: true, isNotification: false },
    'Agent Ticket': { isAnswer: false, isQuestion: true, isNotification: false },
    'Ticket': { isAnswer: false, isQuestion: true, isNotification: false },
    'Unsent Response': { isAnswer: true, isQuestion: false, isNotification: false },
    'Secure Email': { isAnswer: true, isQuestion: false, isNotification: false },
    'Secure Response': { isAnswer: true, isQuestion: false, isNotification: false },
    'Private Note': { isAnswer: true, isQuestion: false, isNotification: false },
    'Public Note': { isAnswer: true, isQuestion: false, isNotification: false },
    'Find Answer Article': { isAnswer: false, isQuestion: false, isNotification: false },
    'Contact Creation': { isAnswer: false, isQuestion: false, isNotification: true },
    'Resolve Note': { isAnswer: true, isQuestion: false, isNotification: false },
    'Mass Mailing Article Update': { isQuestion: false, isNotification: true },
    'Mass Mailing': { isAnswer: true, isQuestion: false, isNotification: false },
    'Chat': { isAnswer: false, isQuestion: true, isNotification: false },
    IsNotification: function (t) {
      var type = iservice.types[t];
      if (!type) return false;
      return type.isNotification;
    },
    IsQuestion: function (t) {
      var type = iservice.types[t];
      if (!type) return false;
      return type.isQuestion;
    },
    IsAnswer: function (t) {
      var type = iservice.types[t];
      if (!type) return false;
      return type.isAnswer;
    }
  };
  iservice.statuses = {
    'Unassigned': { isOpen: true, label: 'Open', className: 'bg-red-disabled label'},
    'Queued': { isOpen: true, label: 'Open', className: 'bg-red-disabled label' },
    'Ignored': { isOpen: false, label: 'Ignored', className: 'bg-red label' },
    'Resolved': { isOpen: false, label: 'Resolved', className: 'bg-green-active label' },
    'Pending': { isOpen: true, label: 'Pending', className: 'bg-yellow-disabled label' },
    'ResolvedArchived': { isOpen: false, label: 'Resolved', className: 'bg-green-active label' },
    'ChatWaiting': { isOpen: true, label: 'ChatWaiting', className: 'bg-red-disabled label' },
    'ChatInProgress': { isOpen: true, label: 'ChatInProgress', className: 'bg-green label' },
    'Unsent': { isOpen: true, label: 'Unsent', className: 'bg-red label' },
    'PrivateNote': { isOpen: false, label: 'Resolved', className: 'bg-green-active label' },
    'PublicNote': { isOpen: false, label: 'Resolved', className: 'bg-green-active label' },
    'Sent': { isOpen: false, label: 'Resolved', className: 'bg-green-active label' },
    'SentWaitingReply': { isOpen: false, label: 'Resolved', className: 'bg-green-active label' },
    'SentArchived': { isOpen: false, label: 'Resolved', className: 'bg-green-active label' },
    isOpen: function (s) {
      var status = iservice.statuses[s];
      if (!status || status.isOpen === undefined) return false;
      return status.isOpen;
    },
    label: function (s) {
      var status = iservice.statuses[s];
      if (!status || status.isOpen === undefined) return 'Resolved';
      return status.label;
    },
    colorClass: function (s) {
      var status = iservice.statuses[s];
      if (!status || status.isOpen === undefined) return '';
      return status.className;
    }
  };

})(iservice);