﻿var iservice = window.iservice || {};
(function (iservice) {
  iservice.components = iservice.components || {};

  iservice.components.customerContactSearch = {
    name: 'vueapp-custinfo-search',
    template: '#vueapp-custinfo-search',
    route() { return { path: iservice.formPath + '/search-contacts', priority: 200, meta: { title: "iService | Search Contacts" } } },
    setup() {
      var router = VueRouter.useRouter();
      var contacts = iservice.ContactsData();
      var contactProperties = iservice.ContactPropertyData();
     
      var fieldGroups = Vue.reactive({});
      fieldGroups.searchGroups = [[{
        where: 'anywhere',
        field: 'login',
        text: ''
      }]];
      var maxFields = 10;
      var currentQuery = Vue.reactive({})
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }
      var tableColumns = Vue.computed(() => {
        var columns = [
          Vue.reactive({
            name: 'expand', hidable: false,
            filterText: function () { },
            componentHead: 'iservice-table-head-empty', componentCell: 'iservice-table-cell-expand', shown: true, tdClass: 'expand-td'
          }),
          Vue.reactive({
            name: 'Name', label: 'Name',
            filterText: iservice.tables.SimpleFilterTextFunc(contacts, 'name'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-contact-name-linked', shown: true
          }),
          Vue.reactive({
            name: 'Email', label: 'Email',
            filterText: iservice.tables.SimpleFilterTextFunc(contacts, 'email'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-contact-email-linked', shown: true
          }),
          Vue.reactive({
            name: 'Company', label: 'Company',
            filterText: iservice.tables.SimpleFilterTextFunc(contacts, 'company'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Phone', label: 'Phone',
            filterText: iservice.tables.SimpleFilterTextFunc(contacts, 'phone'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          })];
          if (!contactProperties.loaded) {
            contactProperties.Load();
              return columns;
        }
        var inc = 1;
        contactProperties.list.forEach(prop => {
          var shown = false
          if (prop.name == 'First Name' || prop.name == 'Last Name') shown = true
          if (prop.name != 'Empty Inbox When Logged Out' && prop.name != 'Receive Notifications' && prop.name != 'Inactivity Timeout Minutes' && prop.name != 'Notification Address' && prop.name != 'Address') {
            var column = Vue.reactive({
              name: 'contact-property-' + prop.id,
              label: prop.name,
              filterText: iservice.tables.ContactPropValueFilterTextFunc(prop.id),
              componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-contact-property',
              shown: shown,
              segmentName: prop.segmentName,
              propertyID: prop.id,
              queryString: 'cv' + inc,
              var: 'property' + prop.id
            });
            columns.push(column);
          }
            inc++;
        });
        var Query = router.currentRoute.value.query;
        //enable disable column visibility on basis of query strings
        for (var i = 0; i < columns.length; i++) {
          if (typeof columns[i].queryString != 'undefined') {
            if (Query.hasOwnProperty(columns[i].queryString)) {
              columns[i].shown = true;
            }
          }
        }
          return columns;
      });
     
      
      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        componentExpanded: 'contact-view-loader',
        filterLabel: 'Filter Results',
        emptyMessage: 'No contacts',
        pageSize: 5,
        expandable: true,
        searching: false,
        initialSort: 'Name',
        columns: tableColumns.value,
        exportName: "Search Contacts"
      });
      Vue.watchEffect(() => {
        tableConfig.columns = tableColumns.value;
      });
      var IDs = Vue.reactive([]);

      var formFeatures = Vue.reactive({ totalResults: 0, showPreviewDialer: false, stockID: null, StartPreviewDialer: false, stockResponses: [], topicsList: [] })
      formFeatures.stockresponses = iservice.StockResponsesData();
      var topics = iservice.TopicData();
      var segments = iservice.SegmentData();
     
      if (!topics.loaded || !segments.loaded) {
        topics.Load(); segments.Load();
      }
      if (!formFeatures.stockresponses.loaded) {
        formFeatures.stockresponses.Load();
      }

      formFeatures.topicsList = Vue.computed(() => {
        var groups = [];
        if (!segments.loaded || !topics.loaded) {
          segments.Load(); topics.Load();
        }
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            group.options.push({ name: topic.name, value: topic.id, level: Math.min(2, topic.depth) });
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.value, r.value); });
          if (group.options.length < 1) return;
          groups.push(group);
        });
        return groups;
      });

      formFeatures.togglePreviewDialer = function () {
        formFeatures.showPreviewDialer = !formFeatures.showPreviewDialer;
      }
     
      formFeatures.loadStockResponses = function (topicID) {
        if (!topicID) return;
        if (!formFeatures.stockresponses) return;
        var topic = topics.get(topicID);
        formFeatures.stockResponses = [{ id: "", name: "--no script needed--" }];
        var stockresponse = formFeatures.stockresponses.inSegment(topic.segmentID);
        if (stockresponse.length > 0) {
          stockresponse.forEach((item) => {
            if (item.description.includes('preview') || item.description.includes('Preview')) {
              formFeatures.stockResponses.push(item);
            }
          });
        }
      }
      formFeatures.BeginPreviewDialer = function (pdStockresponse) {
        formFeatures.stockID = pdStockresponse;
        formFeatures.StartPreviewDialer = true;
      };

      var searchLists = Vue.reactive({ currentSearch: {} });
      searchLists.showWhereList = function (field) {
        return true;
      }
      searchLists.splitGroup = function (group, field) {
        fieldGroups.groups = [];
        for (var i = 0, len = fieldGroups.searchGroups.length; i < len; i++) {
          var cur = fieldGroups.searchGroups[i];

          if (cur !== group)
            fieldGroups.groups.push(cur);
          else {
            var partial = Vue.reactive([]);

            fieldGroups.groups.push(partial);

            for (var f = 0; f < group.length; f++) {
              partial.push(group[f]);

              if (group[f] === field) {
                partial = [];
                fieldGroups.groups.push(partial);
              }
            }
          }
        }
        fieldGroups.searchGroups = fieldGroups.groups;
      }

      searchLists.canAdd = function () {
        var count = 0;

        for (var i = 0, len = fieldGroups.searchGroups.length; i < len; i++)
          count += fieldGroups.searchGroups[i].length;
        return count < maxFields;
      }

      searchLists.canRemove = function () {
        return fieldGroups.searchGroups.length === 1 && fieldGroups.searchGroups[0].length === 1 ? false : true;
      }
      searchLists.addGroup = function () {
        if (!searchLists.canAdd())
          return;

        fieldGroups.searchGroups.push([{ where: 'anywhere', field: 'all', text: '' }]);
      }
      searchLists.removeField = function (field) {
        fieldGroups.groups = [];

        for (var g = 0; g < fieldGroups.searchGroups.length; g++) {
          var group = null;

          for (var f = 0, len = fieldGroups.searchGroups[g].length; f < len; f++) {
            var cur = fieldGroups.searchGroups[g][f];

            if (cur !== field) {
              if (!group)
                group = [];

              group.push(cur);
            }
          }

          if (group)
            fieldGroups.groups.push(group);
        }
        fieldGroups.searchGroups = fieldGroups.groups;
      }
      searchLists.mergeGroup = function (group) {
        fieldGroups.groups = [];

        for (var i = 0, len = fieldGroups.searchGroups.length; i < len; i++) {
          var cur = fieldGroups.searchGroups[i];

          fieldGroups.groups.push(cur);

          if (cur === group) {
            var next = fieldGroups.searchGroups[i + 1];
            for (var f = 0; f < next.length; f++)
              cur.push(next[f]);

            i++;
          }
        }
        fieldGroups.searchGroups = fieldGroups.groups;
      }
      formFeatures.showMoreToggle = function () {
        formFeatures.showmore = !formFeatures.showmore;
      }

      var queryParams = Vue.reactive({});
      formFeatures.Search = function (e) {
        queryParams = Vue.reactive({});
        var fieldNum = 1, lastOp;
        for (var i = 0, groups = fieldGroups.searchGroups.length; i < groups; i++) {
          var group = fieldGroups.searchGroups[i];
          for (var j = 0, fields = group.length; j < fields; j++) {
            var field = group[j];

            if (lastOp)
              queryParams['op' + (fieldNum - 1)] = lastOp;

            queryParams['w' + fieldNum] = field.where;
            queryParams['f' + fieldNum] = field.field;
            queryParams['t' + fieldNum] = field.text;
            lastOp = 'or';

            fieldNum++;
          }

          lastOp = 'and';
        }
        router.push({ query: queryParams });
        const isEqual = (...objects) => objects.every(obj => JSON.stringify(obj) === JSON.stringify(objects[0]));
        if (isEqual(searchLists.currentSearch, currentQuery)) {
          doSearch(); //search callback
        } else {
          //ChangeLocation(this.route.current.params);
          doSearch();
        }


      }
      initialize();
      formFeatures.loading = true;
      function doSearch() {

        formFeatures.param = {
          groups: []
        };

        collectParameters(formFeatures.param)

        tableConfig.searching = true;
        iservice.API('webapp-api-custinfo?api=search', null, { fields: formFeatures.param }).then(data => {
          IDs.length = 0;
          formFeatures.totalResults = data.contacts.length;
          data.contacts.forEach(item => {
            var id = item.id;
            contacts.add(item);
            IDs.push(id);
          });
          formFeatures.firstContactID = IDs.length > 0 ? IDs[0] : "";
          localStorage.setItem('previewDialerContacts', JSON.stringify(IDs));
        }).finally(() => {
          tableConfig.searching = false;
          formFeatures.loading = false;
        });
      }
      function Clear() {
        formFeatures.totalResults = null;
      }
      function collectParameters(params) {
        for (var g = 0, numGroup = fieldGroups.searchGroups.length; g < numGroup; g++) {
          var group = Vue.reactive({
            fields: []
          });

          params.groups.push(group);

          for (var f = 0, numField = fieldGroups.searchGroups[g].length; f < numField; f++) {
            var field = fieldGroups.searchGroups[g][f];

            group.fields.push({
              where: field.where,
              fieldID: field.field,
              searchString: field.text
            });
          }
        }
      }
      function initialize() {
        currentQuery = router.currentRoute.value.query;
        if (currentQuery.f1 !== undefined) {
          fieldGroups.searchGroups = Vue.reactive([[]]);

          var group = fieldGroups.searchGroups[0];

          for (var i = 1; i <= maxFields; i++) {
            var w = currentQuery['w' + i],
              f = currentQuery['f' + i],
              t = currentQuery['t' + i],
              op = currentQuery['op' + i],
              field = {
                where: w !== undefined ? w : 'anywhere',
                field: f !== undefined ? f : 'login',
                text: t !== undefined ? t : ''
              };

            field.where = field.where.startsWith('ent') ? 'entire'
              : field.where.startsWith('s') ? 'start'
                : field.where.startsWith('end') ? 'end' : 'anywhere';

            group.push(field);

            searchLists.currentSearch['w' + i] = w;
            searchLists.currentSearch['f' + i] = f;
            searchLists.currentSearch['t' + i] = t;

            if (op === undefined || i === maxFields)
              break;

            searchLists.currentSearch['op' + i] = op;

            if (op.startsWith('a')) {
              group = [];

              fieldGroups.searchGroups.push(group);
            }
          }

          doSearch();
        }
      }
      formFeatures.Reset = function () {
        router.replace({ query: {} });
        setTimeout(() => router.go(), 200);
      }
      return { fieldGroups, maxFields, searchLists, ListCollapsed, formFeatures, tableConfig, IDs, Clear  }
    }
  }

  iservice.components.customerContact = {
    name: 'vueapp-custinfo-contact',
    template: '#vueapp-custinfo-contact',
    route() { return { path: iservice.formPath + '/contact/:id/:op/:op2?', priority: 200 } },
    setup() {
      var router = VueRouter.useRouter();
      var selectedID = Vue.ref(null);
      var contactIds = [];
      Select(router.currentRoute._value.params);
      function Select(params) {
        selectedID.id = params.id;
        selectedID.tab = params.op;
        selectedID.actionTab = params.op2;
      }
      var selectedIDs = Vue.computed(() => {
        if (selectedID.id) return [{ id: selectedID.id, tab: selectedID.tab, actionTab: selectedID.actionTab }];
        return [];
      });

      Vue.watchEffect(() => {
        contactIds = selectedIDs.value;
      });
      Vue.watchEffect(() => {
        Select(router.currentRoute._value.params);
      });

      return { router, selectedIDs, contactIds }
    }
  }
  iservice.components.customerContactOnce = {
    name: 'vueapp-custinfo-contact-once',
    template: '#vueapp-custinfo-contact-once',
    props: ['contactId', 'tab', 'actionTab'],
    setup(props, context) {
      var router = VueRouter.useRouter();
      var formFeatures = Vue.reactive({});
      var contactId = props.contactId;
      formFeatures.contactId = props.contactId;
      function PrintUtcDate(date) {
        return moment(date).format('YYYY-MM-DD hh:mm:ss A');
      }
      formFeatures.title = "Contact -";
      formFeatures.activeTabName = props.tab;
      formFeatures.actionTab = props.actionTab;
      formFeatures.changeTab = function (tab) {
        formFeatures.activeTabName = tab;
        router.push({ params: { op: tab, op2: null } });
      }
      formFeatures.ownAccount = props.contactId === loggedIn.contactID;
      formFeatures.currentCustomerName = formFeatures.ownAccount ? loggedIn.contactName : loggedIn.logins[0].name;

      return { formFeatures, contactId, PrintUtcDate }
    }
  }
  iservice.components.customerContactDetails = {
    name: 'vueapp-contact-details',
    template: '#vueapp-contact-details',
    props: ["feature", "load"],
    route() { return { path: iservice.formPath + '/new-contact', priority: 200, meta: { title: "iService | New Contact" } } },
    setup(props, context) {
      var featureLoad = Vue.reactive({ loaded: false, working: false });
      Vue.watchEffect(() => {
        if (!props.load || featureLoad.loaded || featureLoad.working) return;
        featureLoad.working = true;
      });
      
      var router = VueRouter.useRouter();
      var fields = Vue.reactive({
        createNew: true
      });
      var scope = Vue.reactive({
        loading: false,
        segments: null,
        collapsedSegments: [],
        errors: [],
        agents: null,
        agentsPropperties: [],
        title: "Create New Contact"
      });
      var allProps = Vue.reactive({});
      var formFeatures = Vue.reactive({});
      var agents = iservice.AgentData();
      var contactProperties = iservice.ContactPropertyData();
      var segments = iservice.SegmentData();
      if (!agents.loaded) {
        agents.Load();
      }
      if (!segments.loaded) {
        segments.Load();
      }
      if (props.feature && props.feature.contactId) {
        fields.createNew = false;
      }

      formFeatures.agentProperties = Vue.computed(() => {
        return scope.agentsPropperties;
      });
      formFeatures.isCollapsed = Vue.computed(() => {
        return function (segment) {
          return scope.collapsedSegments && scope.collapsedSegments.filter(x => x == segment).length > 0;
        }
      });
      formFeatures.toggleSegmentCollapsedState = Vue.computed(() => {
        return function (segment) {
          var index = scope.collapsedSegments.findIndex(x => x == segment);
          if (index == -1) {
            scope.collapsedSegments.push(segment);
          } else {
            scope.collapsedSegments.splice(index, 1);
          }
        }
      });
      formFeatures.isAgent = Vue.computed(() => {
        var agent = agents.list.filter(x => x.id == (props.feature ? props.feature.contactId : null)).length > 0;
        var isUs = (props.feature ? props.feature.contactId : null) == loggedIn.contactID;
        return agent && isUs;
      });
      var contactProps = Vue.computed(() => {
        var list = [];
        if (!contactProperties.loaded) {
          contactProperties.Load();
          return list;
        }
        contactProperties.list.forEach(contProp => {
          let item = allProps && allProps.propMap && allProps.propMap[contProp.id];
          if (item) {
            item["purpose"] = contProp.purpose;
            item["isAgentSetting"] = contProp.purpose == 'Agent';
            list.push(item);
          }
        });
        return list;
      });

      Vue.watchEffect(() => {
        scope.agentsPropperties = contactProps.value;
      });

      formFeatures.addLogins = function (seg) {
        seg.logins.push({ name: '', type: 'Email' });
      }
      formFeatures.removeLogins = function (seg, index) {
        seg.logins.splice(index, 1);
        if (seg.logins.length == 0) {
          seg.logins.push({ name: '', type: '' });
        }
      }
      formFeatures.CanModifyLogins = function (groups) {
        for (var i = 0; i < groups.length; i++) if (groups[i].logins && groups[i].logins.length && groups[i].canModifyLogins) return true;
        return false;
      }
      formFeatures.CollectLogins = function (groups) {
        var logins = [];
        for (var g = 0; g < groups.length; g++) if (groups[g].logins) for (var l = 0; l < groups[g].logins.length; l++) {
          var source = groups[g].logins[l];
          if (!source.name) continue;
          logins.push(source);
        }
        return logins;
      }
      formFeatures.processLogin = function (data) {
        if (data.errors && data.errors.length) {
          scope.errors = data.errors;
          return;
        }
        if (loggedIn.contactID === data.loggedIn.contactID)
          return;

        window.location.reload();
      }
      formFeatures.saveClick = function () {
        if (allProps.out.filter(x => x.values.length > 0).length == 0 && formFeatures.CollectLogins(scope.segments).length == 0) {
          scope.errors = ['You must enter at least one contact property value before creating a contact.‏'];
          return;
        }
        if (scope.membership.length == 0) {
          scope.errors = ['Must specify membership in at least one segment.‏'];
          return;
        }

        const properties = {
          properties: allProps.out,
          setLogins: formFeatures.CanModifyLogins(scope.segments),
          logins: formFeatures.CollectLogins(scope.segments)
        };

        scope.loading = true;
        scope.success = null;
        scope.errors = [];
        if (fields.createNew) {
          let contactId = null;
          iservice.API('webapp-api-custinfo?api=newcontact', null,
            {
              details: properties,
              membership: scope.membership
            }
          ).then(data => {
            if (data.errors && data.errors.length) {
              scope.errors = data.errors;
              return;
            }
            iservice.SuccessMessage('Created Successfully');
            contactId = data.contactID;
          }).catch(errors => {
            scope.errors = errors;
          }).finally(() => {
            scope.loading = false;
            if (contactId) {
              setTimeout(function () {
                router.push({ path: `/app/contact/${contactId}/summary` });
              }, 500);
            }
          });

        } else {
          iservice.API('webapp-api-custinfo?api=setdetails', null,
            {
              contactID: props.feature.contactId,
              details: properties,
              membership: scope.membership
            }
          ).then(data => {
            if (data.errors && data.errors.length) {
              scope.errors = data.errors;
              return;
            }
            iservice.SuccessMessage('Saved Successfully');
            if (props.feature.ownAccount) {
              iservice.API('webapp-api-login?api=loggedin', null, null).then(({ status, data }) => {
                if (status === 200) {
                  formFeatures.ProcessLogin(data);

                  props.feature.currentCustomerName = data.contactName;
                }
              }).catch(errors => {
                scope.errors = errors;
              });
            }
            else {
              iservice.API('webapp-api-custinfo?api=name', null, {
                contactID: props.feature.contactId
              }).then(({ status, data }) => {
                if (status === 200)
                  props.feature.currentCustomerName = data;
              }).catch(errors => {
                scope.errors = errors;
              });
            }
          }).catch(errors => {
            scope.errors = errors;
          }).finally(() => {
            scope.loading = false;
          });
        }
      }
      formFeatures.onReset = function () {
        onInIt();
      }
      scope.loading = true;
      Vue.watchEffect(() => { scope.segmentList = segments.list; });
      Vue.watchEffect(() => {
        if ((featureLoad.loaded || !featureLoad.working) && !fields.createNew) return;
        onInIt();
      })

      function onInIt() {
        scope.loading = true;
        scope.segments = null;
        scope.errors = [];
        iservice.API('webapp-api-custinfo?api=details', null, { contactID: fields.createNew ? "" : props.feature.contactId }).then(data => {
          allProps = null;
          allProps = Vue.reactive({});
          iservice.EditableProperties(allProps, data.properties);

          data.properties.forEach(segment => {
            segment.leftProperties = [];
            segment.rightProperties = [];
            const totalHeight = segment.properties.reduce((prev, prop) => ((prop.values.length > 0 ? prop.values.length : 1) * (prop.isMultiLine || prop.allowMultiple ? 2 : 1)) + prev, 0);
            let leftHeight = 0;
            segment.properties.forEach(property => {
              let prop = allProps.propMap[property.propertyID];
              //prop.def.isRequired = true;

              if (leftHeight <= (totalHeight / 2)) {
                if (prop.def.name == "First Name") {
                  if (prop.def.values.length > 0 && prop.def.values[0].value != '') {
                    props.feature.title = prop.def.values[0].value;
                  } else {
                    if (segment.logins.length) {
                      if (segment.logins[0].name != '') {
                        props.feature.title = 'Contact - ' + segment.logins[0].name;
                      }
                    }
                  }
                }
                if (prop.def.name == "Middle Initial") {
                  if (prop.def.values.length > 0 && prop.def.values[0].value != '') {
                    props.feature.title = props.feature.title + ' ' + prop.def.values[0].value;
                  }
                }
                if (prop.def.name == "Last Name") {
                  if (prop.def.values.length > 0 && prop.def.values[0].value != '') {
                    props.feature.title = 'Contact - ' + props.feature.title + ' ' + prop.def.values[0].value;
                  }
                }
                segment.leftProperties.push(prop);
              }
              else {
                segment.rightProperties.push(prop);
              }
              leftHeight = leftHeight + (prop.values.length * (prop.def.isMultiLine || prop.def.allowMultiple ? 2 : 1));
            });
          });

          scope.membership = data.membership || iservice.loggedIn.accessibleSegments;
          scope.segments = data.properties;

          const globalSegment = data.properties[0] || {};
          const isThereAnySegmentWithLogins = data.properties.some(function (seg) {
            return seg.logins && seg.logins.length;
          });

          if (!isThereAnySegmentWithLogins) {
            globalSegment.logins = [{
              name: '',
              type: 'Email'
            }];
          }
          scope.segments.sort((a, b) => {
            if (a.name == globalSegment.name)
              return -1;
            else if (b.name == globalSegment.name)
              return 1;

            return a.name.localeCompare(b.name);
          });
          globalSegment.name = 'Contact Information';
          
        }).finally(() => {
          scope.loading = false;
          for (id in allProps.propMap) {
            let contProp = contactProperties.get(id);
            if (contProp) {
              allProps.propMap[id]["purpose"] = contProp.purpose;
              allProps.propMap[id]["isAgentSetting"] = contProp.purpose == 'Agent';
              if (allProps.propMap[id]["isAgentSetting"] == true) {
                let index = formFeatures.agentProperties.findIndex(x => x.def.propertyID == id);
                if (index != -1) {
                  formFeatures.agentProperties[index] = allProps.propMap[id];
                }
              }
            }
          }
        });
      }
      return { formFeatures, scope, allProps, fields, agents, contactProperties, router, featureLoad }
    }
  }
  iservice.components.customerContactSummary = {
    name: 'vueapp-custinfo-summary',
    template: '#vueapp-custinfo-summary',
    props: ["feature", "load"],
    setup(props, context) {
      var featureLoad = Vue.reactive({ loaded: false, working: false });
      Vue.watchEffect(() => {
        if (!props.load || featureLoad.loaded || featureLoad.working) return;
        featureLoad.working = true;
      });
      var router = VueRouter.useRouter();
      var formFeatures = Vue.reactive({ showAlert: false});
      var contactId = props.feature.contactId;
      var now = new Date();
      var aWeekAgo = new Date();
      aWeekAgo.setDate(aWeekAgo.getDate() - 7);
      var afterDate = moment(aWeekAgo).format('YYYY-MM-DD hh:mm');
      var beforeDate = moment(now).format('YYYY-MM-DD hh:mm');
      var rootPath = window.location.origin;
      initialize();
      formFeatures.tabUrl = function (componentName) {
        let component = router.options.routes.find(x => x.component.name == componentName);
        if (component) {
          return component.path;
        } else { return ""; }
      }
      formFeatures.PrintDate = function (date, format) {
        return moment(date).format(format);
      }
      
      formFeatures.getIntIconClass = iservice.interactionTypeClasses;
      formFeatures.getTypeLabels = iservice.interactionTypeLabels;

      formFeatures.getURL = Vue.computed(() => {
        return function (key) {
          switch (key) {
            case "interactionPastWeekUrl":
              return `${rootPath}${formFeatures.tabUrl('vueapp-messagequeue-search').replace(':id', contactId)}?before=${beforeDate}&after=${afterDate}&w1=entire&f1=ownerid&t1=${contactId}&op1=and&w2=entire&f2=type&t2=Customer%20Email&op2=or&w3=anywhere&f3=type&t3=Ticket&op3=or&w4=anywhere&f4=type&t4=Ask%20A%20Question&op4=or&w5=anywhere&f5=type&t5=Chat`;
              break;
            case "unassignedUrl":
              return `${rootPath}${formFeatures.tabUrl('vueapp-messagequeue-search').replace(':id', contactId)}?w1=entire&f1=ownerid&t1=${contactId}&op1=and&w2=anywhere&f2=status&t2=1`;
              break;
            case "assignedUrl":
              return `${rootPath}${formFeatures.tabUrl('vueapp-messagequeue-search').replace(':id', contactId)}?w1=entire&f1=ownerid&t1=${contactId}&op1=and&w2=anywhere&f2=status&t2=2`;
              break;
            case "viewAllUrl":
              return `${rootPath}${formFeatures.tabUrl('vueapp-messagequeue-search').replace(':id', contactId)}?w1=entire&f1=ownerid&t1=${contactId}&op1=and&w2=entire&f2=status&t2=1&op2=or&w3=entire&f3=status&t3=2`;
              break;
            case "viewAllHistoryUrl":
              return rootPath + formFeatures.tabUrl('vueapp-custinfo-contact').replace(':id', contactId).replace(':op', "history").replace('/:op2?', "");
              break;
            case "lastAgnetInteractionUrl":
              return `${rootPath}${formFeatures.tabUrl('vueapp-messagequeue-search').replace(':id', contactId)}?w1=entire&f1=ID&t1=${formFeatures.lastInteractionid}`;
              break;
            default:
              return "";
          }
        }
      });
      formFeatures.getMessageLink = function (id) {
        return `${rootPath}${formFeatures.tabUrl('vueapp-messagequeue-search').replace(':id', contactId)}?w1=entire&f1=ID&t1=${id}`;
      }
      formFeatures.amTimeAgo = function (date) {
        return moment(date).fromNow();
      }
      function checkEmail(emailType) {
        if (emailType == 'Email') return true;
        return false;
      }
      formFeatures.AlertClear = function () {
        formFeatures.showAlert = false;
      }
      formFeatures.emailType = '';
      formFeatures.showpanel = function (component) {
        if (component == 'iservice-agent-email' || component == 'iservice-agent-ticket') {
          var emailExists = checkEmail(formFeatures.emailType);
          if (!emailExists) {
            formFeatures.showAlert = true;
            return;
          }
        }
        formFeatures.component = component;
        router.push({ params: { op2: component.replace("iservice-", "") } });
      }
      formFeatures.Cancel = function () {
        router.push({ params: { op2: null } });
        formFeatures.component = null;
      }
      function initialize() {
        let param = {
          id: contactId,
          unresolvedQuestionsParams: {
            groups: [{
              fields: [{
                where: 'entire',
                fieldID: 'ownerid',
                searchString: '' + contactId
              }]
            }, {
              fields: [{
                where: 'entire',
                fieldID: 'status',
                searchString: '2' // Queued
              }, {
                where: 'entire',
                fieldID: 'status',
                searchString: '1' // Unassigned
              }]
            }]
          },
          unassignedQuestionsParams: {
            groups: [{
              fields: [{
                where: 'entire',
                fieldID: 'ownerid',
                searchString: '' + contactId
              }]
            }, {
              fields: [{
                where: 'entire',
                fieldID: 'status',
                searchString: '1' // Unassigned
              }]
            }]
          },
          interactionsPastWeekParams: {
            groups: [{
              fields: [{
                where: 'entire',
                fieldID: 'ownerid',
                searchString: '' + contactId
              }]
            },
            {
              fields: [{
                where: 'entire',
                fieldID: 'type',
                searchString: 'Customer Email'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Ticket'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Ask A Question'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Agent Email'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Agent Response'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Chat'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Note'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Private Note'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Secure Email'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Secure Response'
              }]
            }],
            timeStartUtc: moment.utc(aWeekAgo).format(),
            timeEndUtc: moment.utc(now).format()
          },
          lastUpdatesParams: {
            groups: [{
              fields: [{
                where: 'entire',
                fieldID: 'ownerid',
                searchString: '' + contactId
              }]
            }, {
              fields: [{
                where: 'entire',
                fieldID: 'type',
                searchString: 'Customer Email'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Ticket'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Ask A Question'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Agent Email'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Agent Response'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Chat'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Note'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Private Note'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Secure Email'
              }, {
                where: 'entire',
                fieldID: 'type',
                searchString: 'Secure Response'
              }]
            }]
          }
        }
        if (props.feature.actionTab) {
          formFeatures.component = "iservice-" + props.feature.actionTab;
        } else {
          formFeatures.component = null;
          Vue.watchEffect(function () {
            if (featureLoad.loaded || !featureLoad.working) return;
            loadContactData(param);
          })
          
        }
      }
      function loadContactData(param) {
        formFeatures.loading = true;
        iservice.API('custinfo-summary-data', null, param).then(data => {
          formFeatures.name = data.name;
          formFeatures.jobTitle = data.jobTitle;
          formFeatures.company = data.company;
          formFeatures.email = data.email;
          formFeatures.phone = data.phone;
          data.logins.forEach(function (login) {
            if (login.type == 'Email')
              formFeatures.emailType = 'Email';
            else if (login.type == 'Bounced')
              formFeatures.bouncedEmail = login.name;
          })
          formFeatures.unresolvedQuestions = data.unresolvedQuestions;
          formFeatures.lastUpdates = data.lastUpdates;
          formFeatures.lastUpdates.sort(function (l, r) {
            return r.date.localeCompare(l.date);
          });
          formFeatures.nameStyle = iservice.nameStyle(formFeatures.name);
          formFeatures.nameInitials = iservice.nameInitials(formFeatures.name);

          formFeatures.unresolvedQuestionsCount = data.unresolvedQuestionsCount ? data.unresolvedQuestionsCount : 0;
          formFeatures.unassignedQuestionsCount = data.unassignedQuestionsCount ? data.unassignedQuestionsCount : 0;
          formFeatures.assignedQuestionsCount = formFeatures.unresolvedQuestionsCount - formFeatures.unassignedQuestionsCount;
          formFeatures.interactionsPastWeekCount = data.interactionsPastWeekCount ? data.interactionsPastWeekCount : 0;
          for (var i = 0; i < formFeatures.lastUpdates.length; i++) {
            formFeatures.lastAgent = formFeatures.lastUpdates[i].agent;
            formFeatures.lastInteractionid = formFeatures.lastUpdates[i].id;
            if (formFeatures.lastAgent) break;
          }

          if (!formFeatures.lastAgent) formFeatures.lastAgent = 'N/A';
          props.feature.title = 'Contact - ' + formFeatures.name;

        }).finally(() => {
          formFeatures.loading = false;
        });
      }
      return { formFeatures, contactId, router }
    }
  }
  iservice.components.customerContactSubscriptions = {
    name: 'vueapp-custinfo-subscriptions',
    template: '#vueapp-custinfo-subscriptions',
    props: ["feature", "load"],
    setup(props, context) {
      var featureLoad = Vue.reactive({ loaded: false, working: false });
      Vue.watchEffect(() => {
        if (!props.load || featureLoad.loaded || featureLoad.working) return;
        featureLoad.working = true;
      });

      var router = VueRouter.useRouter();
      var formFeatures = Vue.reactive({});
      formFeatures.articles = Vue.reactive([]);
      formFeatures.campaigns = Vue.reactive([]);
      formFeatures.lists = Vue.reactive([]);
      var contactId = props.feature.contactId;
      initialize();
      
      formFeatures.PrintDate = function (date, format) {
        return moment(date).format(format);
      }
      formFeatures.onSaveClick = function () {
        let param = {
          contactID: contactId,
          subscriptions: {
            listIDs: [],
            optoutIDs: [],
            articleIDs: []
          }
        };

        if (formFeatures.lists.length > 0) {
          param.subscriptions.listIDs = [...formFeatures.lists.filter(x => x.isOnList == true).map(x => x.listID)];
          param.subscriptions.optoutIDs = [...formFeatures.lists.filter(x => x.isOptedOut == true).map(x => x.listID)];
        }
        if (formFeatures.campaigns.length > 0) {
          param.subscriptions.listIDs = [...param.subscriptions.listIDs,...formFeatures.campaigns.filter(x => x.isOnList == true).map(x => 'campaign-' + x.campaignID)];
          param.subscriptions.optoutIDs = [...param.subscriptions.optoutIDs,...formFeatures.campaigns.filter(x => x.isOptedOut == true).map(x => 'campaign-' + x.campaignID)];
        }
        if (formFeatures.articles.length > 0) {
          param.subscriptions.articleIDs = [...formFeatures.articles.filter(x => x.subscribed == true).map(x => x.articleID)];
        }

        formFeatures.loading = true;
        Vue.watchEffect(function () {
          if (featureLoad.loaded || !featureLoad.working) return;
          iservice.API('webapp-api-custinfo?api=setsubscriptions', null, param).then(data => {
            iservice.SuccessMessage('Subscriptions saved successfully');
          }).catch(errors => {
            formFeatures.errors = errors;
          }).finally(() => {
            formFeatures.loading = false;
          });
        })

      }
      function initialize() {
        let param = {
          contactID: contactId
        }
        Vue.watchEffect(function () {
          if (featureLoad.loaded || !featureLoad.working) return;
          loadSubscriptionsData(param);
        })
      }
      function loadSubscriptionsData(param) {
        formFeatures.loading = true;
        iservice.API('webapp-api-custinfo?api=subscriptions', null, param).then(data => {
          formFeatures.hasData = data.subscriptions ? true : false;
          formFeatures.articles = data.subscriptions.articles;
          formFeatures.campaigns = data.subscriptions.campaigns;
          formFeatures.lists = data.subscriptions.lists;
        }).finally(() => {
          formFeatures.loading = false;
          formFeatures.isLoaded = true;
        });
      }
      return { formFeatures, contactId, router}
    }
  }
  iservice.components.interactionContactHistory = {
    name: 'contact-history',
    template: '#contact-history',
    props: ['itemid', 'load'],
    setup(props) {
      var featureLoad = Vue.reactive({ loaded: false, working: false });
      Vue.watchEffect(() => {
        if (!props.load || featureLoad.loaded || featureLoad.working) return;
        featureLoad.working = true;
      });
      var ActiveTab = Vue.reactive({ caseView: true }); //default active tab
      var tabFeatures = Vue.reactive({ showNotifications: false, originalThreadChildren: [], filterThreadChildren: [],errors: [], success: '', totalMessages: '', IDs: [], CaseIDs: []});
      var interactions = iservice.InteractionData();
      //toggle tab view
      function toggleView() {
        ActiveTab.caseView = !ActiveTab.caseView;
      }
      tabFeatures.showMoreToggle = function () {
        tabFeatures.showmore = !tabFeatures.showmore;
      }
      var contactID = props.itemid;
      function filterNotificationInteraction(i) {
        return !iservice.types.IsNotification(i.type);
      }
      var tableColumnsChrono = Vue.computed(() => {
        var columns = [
          Vue.reactive({
            name: 'expand', hidable: false,
            filterText: function () { },
            componentHead: 'iservice-table-head-empty', componentCell: 'iservice-table-cell-expand', shown: true, tdClass: 'expand-td'
          }),
          Vue.reactive({
            name: 'Subject', label: 'Subject',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'subject'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Segment', label: 'Segment',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'segmentName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: false
          }),
          Vue.reactive({
            name: 'Topic', label: 'Topic',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'topicName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Status', label: 'Status',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'status'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'interaction-status-display', shown: true
          }),
          Vue.reactive({
            name: 'Type', label: 'Type',
            filterText: iservice.tables.SimpleFilterTextFunc(iservice.data.interactions, 'type'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'interaction-type-display', shown: true
          }),
          Vue.reactive({
            name: 'Agent', label: 'Agent',
            filterText: iservice.tables.SimpleFilterTextFunc(iservice.data.interactions, 'agentName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-interaction-customername-linked', shown: true
          }),
          Vue.reactive({
            name: 'Date', label: 'Date', sortIncreasing: false,
            filterText: iservice.tables.DateFilterTextFunc(interactions, 'date'), sortText: iservice.tables.DateSortTextFunc(interactions, 'date'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-interaction-date', shown: true
          })
        ];
        return columns;
      });
      var tableColumnsCase = Vue.computed(() => {
        var columns = [
          Vue.reactive({
            name: 'expand', hidable: false,
            filterText: function () { },
            componentHead: 'iservice-table-head-empty', componentCell: 'iservice-table-cell-expand', shown: true, tdClass: 'expand-td'
          }),
          Vue.reactive({
            name: 'Subject', label: 'Subject',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'subject'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Segment', label: 'Segment',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'segmentName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: false
          }),
          Vue.reactive({
            name: 'Case Topic', label: 'Case Topic',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'topicName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Case Status', label: 'Case Status',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'status'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'interaction-case-status-display', shown: true
          }),
          Vue.reactive({
            name: 'Resolved By', label: 'Resolved By',
            filterText: iservice.tables.SimpleFilterTextFunc(iservice.data.interactions, 'resolvedByName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Date', label: 'Date', sortIncreasing: false,
            filterText: iservice.tables.DateFilterTextFunc(interactions, 'date'), sortText: iservice.tables.DateSortTextFunc(interactions, 'date'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-interaction-date', shown: true
          })
        ];
        return columns;
      });
      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        componentExpanded: ActiveTab.caseView ? 'case-view-threads' : 'interaction-view-loader',
        filterLabel: 'Filter History',
        emptyMessage: 'No Interactions',
        pageSize: 5,
        expandable: true,
        searching: false,
        isCaseView: ActiveTab.caseView ? true : false,
        initialSort: 'Date',
        exportName: ActiveTab.caseView  ? "Case History" : 'Chronological History',
        columns: ActiveTab.caseView ? tableColumnsCase.value : tableColumnsChrono.value
      });
      Vue.watchEffect(() => {
        tableConfig.columns = ActiveTab.caseView ? tableColumnsCase.value : tableColumnsChrono.value
        tableConfig.componentExpanded = ActiveTab.caseView ? 'case-view-threads' : 'interaction-view-loader';
        tableConfig.exportName = ActiveTab.caseView ? "Case History" : 'Chronological History';
      });
      tableConfig.searching = true;
      Vue.watchEffect(() => {
        if (featureLoad.loaded || !featureLoad.working) return;
        iservice.API('webapp-api-mq-search?api=search', null, {
          fields: {
            groups: [{
              fields: [{
                where: 'entire',
                fieldID: 'ownerid',
                searchString: contactID
              }]
            }]
          },
          pageNum: 0,
          perPage: 2500
        }).then(function (data) {
          tabFeatures.IDs.length = 0;
          tabFeatures.CaseIDs.length = 0;
          var map = Vue.reactive({});
          var cases = Vue.reactive([]);
          var root = '';
          if (data.interactions) {
            //construct case view from all interactions
            
            tabFeatures.totalMessages = data.interactions.length;

            data.interactions.forEach(function (row) {
              if (row.type == 'Auto Response') row.type = 'Message Acknowledgment';
              if (row.type == 'Find Answer') row.type = 'Knowledge Base Article';
              if (row.type == 'Notification') row.type = 'Secure Notification';
              if (row.type == 'Ticket') row.type = 'Agent Ticket';
              if (row.type == 'Ask A Question') row.type = 'Customer Ticket';

              tabFeatures.chronologicalViewData = data.interactions;
              tabFeatures.chronologicalViewDataFiltered = tabFeatures.chronologicalViewData.filter(filterNotificationInteraction);

              //create lookup map
              map[row.id] = row;
              
            });
            data.interactions.forEach(function (row) {
              //put children into thread roots
              root = map[row.threadID];
              if (!root) return;
              if (!root.threadChildren) {
                root.threadChildren = [];
                cases.push(root);
                root.caseRecord = {};
              }
              if (root == row) return;
              root.threadChildren.push(row);
            });
            
          }
          tableConfig.threadChildren = Vue.reactive([]);
          cases.sort((a, b) => new Date(b.date).getTime() - new Date(a.date).getTime());
          //sort case children and compute case status
          cases.forEach(function (root) {
            root.currentStatus = iservice.statuses.label(root.status);
            root.statusClass = iservice.statuses.colorClass(root.status);
            root.threadChildren.forEach(function (child) {
              if (iservice.statuses.isOpen(child.status)) {
                root.currentStatus = iservice.statuses.label(child.status);
                root.statusClass = iservice.statuses.colorClass(child.status);
              }
            });
            root.threadChildren.sort((a, b) => new Date(a.date).getTime() - new Date(b.date).getTime());
            var threads = [];
            var filterThreads = [];
            root.threadChildren.forEach(function (item) {
              threads.push(item.id);
              if (!iservice.types.IsNotification(item.type)) {
                filterThreads.push(item.id);
              }
            })
            tableConfig.threadChildren[root.id] = threads;
            tabFeatures["originalThreadChildren"][root.id] = threads;
            tabFeatures["filterThreadChildren"][root.id] = filterThreads;
          });          
          tabFeatures.caseViewData = cases;
          tabFeatures.caseViewDataFiltered = cases.filter(filterNotificationInteraction);
          Vue.watchEffect(function () {
            var chronoData = tabFeatures.showNotifications ? tabFeatures.chronologicalViewData : tabFeatures.chronologicalViewDataFiltered;
            var caseData = tabFeatures.showNotifications ? tabFeatures.caseViewData : tabFeatures.caseViewDataFiltered;
            tabFeatures.IDs = []
            tabFeatures.CaseIDs = []
            for (id in tabFeatures.originalThreadChildren) {
              tableConfig.threadChildren[id] = tabFeatures.showNotifications ? tabFeatures.originalThreadChildren[id] : tabFeatures.filterThreadChildren[id];
            }
            chronoData.forEach(function (row) {
              var id = row.id;
              interactions.add(row);
              tabFeatures.IDs.push(id);
            });
            caseData.forEach(function (root) {
              var id = root.id;
              interactions.add(root);
              tabFeatures.CaseIDs.push(id);
            });
          })
        }).catch(function (errors) {
          tabFeatures.errors = errors;
        }).finally(() => {
          tableConfig.searching = false;
        });
      });

      return { toggleView, ActiveTab, tabFeatures, tableConfig, featureLoad };
    }
  };
  iservice.components.caseHistoryViewLoader = {
    name: 'case-history-view-loader',
    template: '#case-history-view-loader',
    props: ['itemid', 'threadID'],
    setup(props) {
      var threadID = Vue.computed(function () {
        return props.threadID;
      });
      var interactions = iservice.InteractionData();
      var interaction = Vue.computed(() => {
        return interactions.get(props.itemid);
      });
      var showDetails = Vue.ref(false);
      function Expand() {
        showDetails.value = !showDetails.value;
      }
      var rowClass = Vue.computed(() => {
        if (!interaction.value) return '';
        return iservice.RowColorClass(interaction.value.type);
      });
      var subject = Vue.computed(() => {
        if (!interaction.value) return '';
        return interaction.value.subject;
      });
      var topicName = Vue.computed(() => {
        if (!interaction.value) return '';
        return interaction.value.topicName;
      });
      var resolvedByName = Vue.computed(() => {
        if (!interaction.value) return '';
        return interaction.value.resolvedByName;
      });
      var date = Vue.computed(() => {
        if (!interaction.value) return '';
        var date = interaction.value.date;
        if (!date || date == '') return '';
        return moment(date).format('YYYY-MM-DD hh:mm:ss A');
      });

      return { showDetails, Expand, rowClass, subject, topicName, resolvedByName, date, threadID };
    }
  };
  iservice.components.contactViewLoader = {
    name: 'contact-view-loader',
    template: '#contact-view-loader',
    props: ['itemid', 'type'],
    setup(props) {
      var contacts = iservice.ContactsData();
      var contactID = Vue.computed(function () {
        return props.itemid;
      })
      function checkEmail(logins) {
        var response = false
        logins.forEach(function (item) {
          if (item.type == 'Email') {
            response = true;
          }
        });
        return response;
      }
      var formFeatures = Vue.reactive({ component: '', type: 'contact', showAlert: false });
      formFeatures.AlertClear = function () {
        formFeatures.showAlert = false;
      }
      formFeatures.Click = function (component) {
        if (component == 'iservice-agent-email' || component == 'iservice-agent-ticket') {
          var contact = Vue.computed(function () {
            return contacts.get(props.itemid);
          });
          var emailExists = checkEmail(contact.value.logins);
          if (!emailExists) {
            formFeatures.showAlert = true;
            return;
          }
        }

        formFeatures.component = component;
      }


      function StopAction() {
        formFeatures.component = '';
      }
      return { formFeatures, StopAction, contactID };
    }
  };
  iservice.components.iserviceAgentEmail = {
    name: 'iservice-agent-email',
    template: '#iservice-agent-email',
    props: ['itemid', 'type', 'threadID'],
    emits: ['cancel'],
    setup(props, context) {
      var config = Vue.inject('config');
      var topics = iservice.TopicData();
      var segments = iservice.SegmentData();
      var mailboxes = iservice.MailboxData();
      var interactions = iservice.InteractionData();
      var loggedIn = iservice.LoggedInData();
      var threads = props.threads;
      var formFeatures = Vue.reactive({ tab: 'agent_email_details', success: '', loading: false, mailboxes: [], addresses: [], featureProps: {}, featureCaseProps: {}, processing: false, type: '' });
      formFeatures.details = { topicID: '', mailboxID: '', destEmail: '', cc: '', bcc: '', subject: '', bodyHtml: '', note: '', segmentID: '' }
      formFeatures.errors = [];
      var contactID = '';
      var interactionID = '';
      var interactionTopic = '';
      formFeatures.type = props.type;
      if (props.type == 'contact') {
        contactID = props.itemid;
      }
      if (props.type == 'interaction') {
        interactionID = props.itemid;
        var interaction = interactions.get(interactionID);
        formFeatures.details.segmentID = interaction.segmentID;
        formFeatures.details.cc = interaction.cc;
        formFeatures.details.bcc = interaction.bcc;
        formFeatures.details.subject = interaction.subjectBare;
        formFeatures.details.note = interaction.note;
        interactionTopic = interaction.topicID;
        contactID = interaction.customerID;
      }

      formFeatures.TabClass = function (tab) { return { 'active': tab == formFeatures.tab }; }
      formFeatures.TabClick = function (tab) {
        formFeatures.tab = tab;
      }

      var anchorSend = Vue.ref(null);
      var featureTopicMenu = Vue.reactive({ options: [] });
      Vue.watchEffect(() => {
        featureTopicMenu.options.length = 0;
        featureTopicMenu.loading = true;
        if (!segments.loaded || !topics.loaded) {
          segments.Load(); topics.Load();
          return;
        }
        featureTopicMenu.loading = false;
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            group.options.push({ name: topic.name, id: topic.id, level: Math.min(2, topic.depth) });
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.value, r.value); });
          if (group.options.length < 1) return;
          featureTopicMenu.options.push(group);
        });
      });
      var featureSendMenu = Vue.reactive({ addSubjectRefNum: true, secureOptions: [], notificationID: '' });
      featureSendMenu.Open = function () {
        featureSendMenu.Close = iservice.components.popupHolder.Open(anchorSend, 'agent-email-menu-send', true, featureSendMenu);
      }
      var featureUploads = iservice.FeatureUploads();
      var segmentOptions = Vue.ref([]);
      var selectMailbox = Vue.ref(null);

      formFeatures.loading = true;
      function startEmail() {
        formFeatures.processing = true;
        if (formFeatures.details.topicID) {
          var topic = topics.get(formFeatures.details.topicID);
          formFeatures.details.segmentID = topic.segmentID;
        }
        iservice.API('webapp-api-history?api=startagentemail', null, {
          topicID: formFeatures.details.topicID ? formFeatures.details.topicID : "",
          interactionID: interactionID ? interactionID : "",
          segmentID: formFeatures.details.segmentID ? formFeatures.details.segmentID : "",
          customerID: contactID ? contactID : ""
        }).then(function (data) {

          var mailboxIDs = [];
          data.mailboxes.forEach(function (item) {
            mailboxIDs.push(item.id);
          });
          Vue.watchEffect(() => {
            segmentOptions.value.length = 0;
            segmentOptions.loading = true;
            if (!mailboxes.loaded || !segments.loaded) {
              mailboxes.Load(); segments.Load(); return;
            }
            segmentOptions.loading = false;
            segments.list.forEach(seg => {
              var group = { id: seg.id, label: seg.name, options: [] };
              mailboxes.inSegment(seg.id).forEach(mailbox => {
                if (mailboxIDs.includes(mailbox.id)) {
                  var option = {
                    value: mailbox.id,
                    name: mailbox.name,
                    email: mailbox.replyTo,
                    search: mailbox.name + '\n' + mailbox.replyTo
                  };
                  group.options.push(option);
                }
              });
              if (group.options.length < 1) return;
              group.options.sort((l, r) => { return mailboxes.SortCompare(l.value, r.value); });
              segmentOptions.value.push(group);
            });
            segmentOptions.value.sort((l, r) => { return segments.SortCompare(l.id, r.id); });
            var segment = segments.get(formFeatures.details.segmentID);
            if (segment) {
              formFeatures.details.defaultMailboxID = segment.mailboxID;
              if (!selectMailbox.value) return;
              var mailbox = mailboxes.get(segment.mailboxID);
              if (mailbox) {
                var option = {
                  value: mailbox.id,
                  name: mailbox.name,
                  email: mailbox.replyTo,
                  search: mailbox.name + '\n' + mailbox.replyTo
                };
                selectMailbox.value.select(option);
              }
            }
          });
          formFeatures.addresses = data.addresses;

          formFeatures.details.destEmail = data.addresses.length ? data.addresses[0] : '';
          var answerIsInline = Vue.reactive({});
          let quotedBody = data.quotedBody;
          if (data.quotedBody.startsWith("<div>")) {
            quotedBody = quotedBody.substring(0, 5) + "<br><br>" + quotedBody.substring(5, quotedBody.length);
          }
          formFeatures.details.bodyHtml = iservice.FindInlineAttachments(quotedBody, data.attachments, answerIsInline);

          featureSendMenu.secureOptions = data.notifications.filter(response => response.segmentID === formFeatures.details.segmentID && response.isSecureNotification);
          if (formFeatures.details.notificationID && featureSendMenu.secureOptions.length)
            formFeatures.details.notificationID = featureSendMenu.secureOptions[featureSendMenu.secureOptions.length - 1].id;

          formFeatures.featureProps = Vue.reactive({ label: 'Answer Properties', length: 0 });
          var intProps = data.interactionProperties.length ? data.interactionProperties[0].properties : [];
          iservice.EditableProperties(formFeatures.featureProps, intProps);

          formFeatures.featureCaseProps = Vue.reactive({ label: 'Case Properties', length: 0 });
          var caseProps = data.caseProperties.length ? data.caseProperties[0].properties : [];
          iservice.EditableProperties(formFeatures.featureCaseProps, caseProps);
          Vue.watchEffect(function () {
            formFeatures.featureCaseProps.length = formFeatures.featureCaseProps.props.length;
            formFeatures.featureProps.length = formFeatures.featureProps.props.length;
            formFeatures.haveProperties = formFeatures.featureCaseProps.props.length || formFeatures.featureProps.props.length;
          });
          Vue.watchEffect(function () {
            formFeatures.propertyMissingClass = { 'text-red': formFeatures.featureCaseProps.missingRequired || formFeatures.featureProps.missingRequired };
          });
          Vue.watchEffect(function () {
            formFeatures.hasValues = formFeatures.featureCaseProps.hasValues || formFeatures.featureProps.hasValues;
          });
        }).catch(function (errors) {
          formFeatures.errors = errors;
        }).finally(function () {
          formFeatures.loading = false;
          formFeatures.processing = false;
        });
      }
      startEmail();

      var editor = Vue.ref(null);
      var anchorStocks = Vue.ref(null);
      var segmentID = '';
      if (props.type == 'contact') {
        segmentID = loggedIn.accessibleSegments[0];
      } else {
        segmentID = formFeatures.details.segmentID;
      }
      var featureStockResponses = Vue.reactive({ filter: '', segmentID: segmentID });
      Vue.watchEffect(function () {
        if (!formFeatures.details.topicID) return;
        var topic = topics.get(formFeatures.details.topicID);
        featureStockResponses.segmentID = topic.segmentID;
      })
      featureStockResponses.Open = function () {
        featureStockResponses.Close = iservice.components.popupHolder.Open(anchorStocks, 'stock-responses-menu', false, featureStockResponses);
      }
      featureStockResponses.PasteVar = function (text) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteText(text);
      }
      featureStockResponses.PasteStock = function (html) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteHtml(html);
      }
      function Cancel() {
        context.emit('cancel');
      }
      function CheckProperties(feature, label) {
        for (var prop of feature.props) {
          if (!prop.missingRequired) continue;
          prop.focusMe = true;
          formFeatures.TabClick('properties');
          formFeatures.errors = ['Must set value for ' + label + ' property ' + prop.def.name + '.'];
          return false;
        }
        return true;
      }
      var errors = Vue.reactive({});
      featureSendMenu.Send = function (action) {
        formFeatures.errors = [];
        if (props.type == 'contact') {
          if (!formFeatures.details.topicID) {
            formFeatures.errors.push({ msg: 'Please select a topic.', details: { description: 'It looks like you are missing the topic of your email. Please select a topic before sending.' } });
            errors.topicID = true;
          }
        }
        if (props.type == 'interaction')
          formFeatures.details.topicID = interactionTopic;

        if (!formFeatures.details.mailboxID) {
          formFeatures.errors.push({ msg: 'Please enter a From email address.', details: { description: 'It looks like you are missing From email address. Please select a From email before sending.' } });
          errors.mailboxID = true;
        }

        if (!formFeatures.details.destEmail) {
          formFeatures.errors.push({ msg: 'Please enter a To email address.', details: { description: 'It looks like you are missing To email address. Please select a To email before sending.' } });
          errors.destEmail = true;
        }
        if (!formFeatures.details.subject) {
          formFeatures.errors.push({ subject: true, msg: 'Please enter a subject for your email.', details: { description: 'Every interaction must have a subject, but it looks like one is missing here. Please enter a subject for your email before sending.' } });
          errors.subject = true;
        }
        if (!formFeatures.details.bodyHtml) {
          formFeatures.errors.push({ msg: 'Your email doesn\'t have a body.', details: { description: 'It looks like you are missing the body of this email. Please type your message in the editor before sending.' } });
          errors.bodyHtml = true;
        }
        if (formFeatures.errors.length) return;
        if (formFeatures.featureProps.props) {
          if (!CheckProperties(formFeatures.featureProps, 'answer')) return;
        }
        if (formFeatures.featureCaseProps.props) {
          if (!CheckProperties(formFeatures.featureCaseProps, 'case')) return;
        }
        formFeatures.processing = true;
        formFeatures.details.action = action;
        formFeatures.details.expectingResponse = featureSendMenu.expectingResponse;
        formFeatures.details.addSubjectRefNum = featureSendMenu.addSubjectRefNum;
        formFeatures.details.notificationID = featureSendMenu.notificationID;
        formFeatures.details.inResponseToID = interactionID;
        formFeatures.details.properties = formFeatures.featureProps.out;
        formFeatures.details.caseProperties = formFeatures.featureCaseProps.out;
        formFeatures.details.contactID = contactID;
        iservice.API('webapp-api-custinfo?api=createemail', null, { details: formFeatures.details }, featureUploads.files).then(function (data) {
          interactions.add(data.row);
          config.threadChildren[props.threadID].push(data.interactionID);
          if(action == 2)
            iservice.SuccessMessage("Secure email created successfully.");
          else
            iservice.SuccessMessage("Agent email created successfully.");
          Cancel();
        }).catch(function (errors) {
          formFeatures.errors = errors;
        }).finally(function () {
          formFeatures.processing = false;
        });
      }
      formFeatures.OnTopicSelect = function () {
        if (!formFeatures.details.topicID) return;
        startEmail();
      }
      return { formFeatures, contactID, featureSendMenu, anchorSend, featureTopicMenu, featureUploads, editor, anchorStocks, featureStockResponses, Cancel, segmentOptions, selectMailbox, errors };
    }
  };
  iservice.components.iserviceAgentNote = {
    name: 'iservice-agent-note',
    template: '#iservice-agent-note',
    props: ['itemid', 'type', 'threadID'],
    emits: ['cancel'],
    setup(props, context) {
      var config = Vue.inject('config');
      var topics = iservice.TopicData();
      var segments = iservice.SegmentData();
      var interactions = iservice.InteractionData();

      var formFeatures = Vue.reactive({ tab: 'note_details', errors: [], success: '', loading: false, mailboxes: [], addresses: [], featureProps: {}, featureCaseProps: {}, processing: false, type: '' });
      formFeatures.details = { topicID: '', subject: '', bodyHtml: '', note: '', segmentID: '', inResponseToID: '', addSubjectRefNum: true }

      var contactID = '';
      var interactionID = '';
      var interactionTopic = '';
      formFeatures.type = props.type;
      if (props.type == 'contact') {
        contactID = props.itemid;
      }
      if (props.type == 'interaction') {
        interactionID = props.itemid;
        var interaction = interactions.get(interactionID);
        formFeatures.details.segmentID = interaction.segmentID;
        interactionTopic = interaction.topicID;
        contactID = interaction.customerID;
      }
      var featureTopicMenu = Vue.reactive({ options: [] });
      Vue.watchEffect(() => {
        featureTopicMenu.options.length = 0;
        featureTopicMenu.loading = true;
        if (!segments.loaded || !topics.loaded) {
          segments.Load(); topics.Load();
          return;
        }
        featureTopicMenu.loading = false;
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            group.options.push({ name: topic.name, id: topic.id, level: Math.min(2, topic.depth) });
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.value, r.value); });
          if (group.options.length < 1) return;
          featureTopicMenu.options.push(group);
        });
      });
      formFeatures.TabClass = function (tab) { return { 'active': tab == formFeatures.tab }; }
      formFeatures.TabClick = function (tab) {
        formFeatures.tab = tab;
      }

      var anchorSend = Vue.ref(null);
      var featureUploads = iservice.FeatureUploads();

      formFeatures.loading = true;
      function startNote() {
        formFeatures.processing = true;
        if (formFeatures.details.topicID) {
          var topic = topics.get(formFeatures.details.topicID);
          formFeatures.details.segmentID = topic.segmentID;
        }
        iservice.API('webapp-api-history?api=startnote', null, {
          topicID: formFeatures.details.topicID ? formFeatures.details.topicID : "",
          interactionID: interactionID ? interactionID : "",
          segmentID: formFeatures.details.segmentID ? formFeatures.details.segmentID : ""
        }).then(function (data) {

          formFeatures.featureProps = Vue.reactive({ label: 'Answer Properties', length: 0 });
          var intProps = data.interactionProperties.length ? data.interactionProperties[0].properties : [];
          iservice.EditableProperties(formFeatures.featureProps, intProps);

          formFeatures.featureCaseProps = Vue.reactive({ label: 'Case Properties', length: 0 });
          var caseProps = data.caseProperties.length ? data.caseProperties[0].properties : [];
          iservice.EditableProperties(formFeatures.featureCaseProps, caseProps);
          Vue.watchEffect(function () {
            formFeatures.featureCaseProps.length = formFeatures.featureCaseProps.props.length;
            formFeatures.featureProps.length = formFeatures.featureProps.props.length;
            formFeatures.haveProperties = formFeatures.featureCaseProps.props.length || formFeatures.featureProps.props.length;
          });
          Vue.watchEffect(function () {
            formFeatures.propertyMissingClass = { 'text-red': formFeatures.featureCaseProps.missingRequired || formFeatures.featureProps.missingRequired };
          });
          Vue.watchEffect(function () {
            formFeatures.hasValues = formFeatures.featureCaseProps.hasValues || formFeatures.featureProps.hasValues;
          });
        }).catch(function (errors) {
          formFeatures.errors = errors;
        }).finally(function () {
          formFeatures.loading = false;
          formFeatures.processing = false;
        });
      }
      startNote();

      var editor = Vue.ref(null);
      var anchorStocks = Vue.ref(null);
      var featureStockResponses = Vue.reactive({ filter: '', segmentID: formFeatures.details.segmentID });
      Vue.watchEffect(function () {
        if (!formFeatures.details.topicID) return;
        var topic = topics.get(formFeatures.details.topicID);
        featureStockResponses.segmentID = topic.segmentID;
      })
      featureStockResponses.Open = function () {
        featureStockResponses.Close = iservice.components.popupHolder.Open(anchorStocks, 'stock-responses-menu', false, featureStockResponses);
      }
      featureStockResponses.PasteVar = function (text) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteText(text);
      }
      featureStockResponses.PasteStock = function (html) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteHtml(html);
      }
      function Cancel() {
        context.emit('cancel');
      }
      function CheckProperties(feature, label) {
        
        for (var prop of feature.props) {
          if (!prop.missingRequired) continue;
          prop.focusMe = true;
          formFeatures.TabClick('properties');
          formFeatures.errors = ['Must set value for ' + label + ' property ' + prop.def.name + '.'];
          return false;
        }
        return true;
      }
      var errors = Vue.reactive({});
      formFeatures.createNote = function (action) {
        formFeatures.errors = [];
        if (props.type == 'contact') {
          if (!formFeatures.details.topicID) {
            formFeatures.errors.push({ msg: 'Please select a topic.', details: { description: 'It looks like you are missing the topic of your email. Please select a topic before sending.' } });
            errors.topicID = true;
          }
        }
        if (props.type == 'interaction')
          formFeatures.details.topicID = interactionTopic;


        if (!formFeatures.details.subject) {
          formFeatures.errors.push({ subject: true, msg: 'Please enter a subject for your email.', details: { description: 'Every interaction must have a subject, but it looks like one is missing here. Please enter a subject for your email before sending.' } });
          errors.subject = true;
        }
        if (!formFeatures.details.bodyHtml) {
          formFeatures.errors.push({ msg: 'Your email doesn\'t have a body.', details: { description: 'It looks like you are missing the body of this email. Please type your message in the editor before sending.' } });
          errors.bodyHtml = true;
        }
        if(formFeatures.errors.length) return;
        if (formFeatures.featureProps.props) {
          if (!CheckProperties(formFeatures.featureProps, 'answer')) return;
        }
        if (formFeatures.featureCaseProps.props) {
          if (!CheckProperties(formFeatures.featureCaseProps, 'case')) return;
        }
        formFeatures.processing = true;
        formFeatures.details.action = action;
        formFeatures.details.inResponseToID = interactionID;
        formFeatures.details.properties = formFeatures.featureProps.out;
        formFeatures.details.caseProperties = formFeatures.featureCaseProps.out;
        formFeatures.details.contactID = contactID;
        iservice.API('webapp-api-custinfo?api=createnote', null, { details: formFeatures.details }, featureUploads.files).then(function (data) {
          interactions.add(data.row);
          config.threadChildren[props.threadID].push(data.interactionID);
          iservice.SuccessMessage('Note created successfully');
          Cancel();
        }).catch(function (errors) {
          formFeatures.errors = errors;
        }).finally(function () {
          formFeatures.processing = false;
        });
      }
      formFeatures.OnTopicSelect = function () {
        if (!formFeatures.details.topicID) return;
        startNote();
      }
      return { formFeatures, contactID, anchorSend, featureUploads, editor, anchorStocks, featureStockResponses, Cancel, featureTopicMenu, errors };
    }
  };
  iservice.components.iserviceAgentTicket = {
    name: 'iservice-agent-ticket',
    template: '#iservice-agent-ticket',
    props: ['itemid', 'type', 'threadID'],
    emits: ['cancel'],
    setup(props, context) {
      var config = Vue.inject('config');
      var topics = iservice.TopicData();
      var segments = iservice.SegmentData();
      var interactions = iservice.InteractionData();
      var agents = iservice.AgentData();

      var formFeatures = Vue.reactive({ tab: 'ticket_details', errors: [], success: '', loading: false, mailboxes: [], addresses: [], featureProps: {}, featureCaseProps: {}, processing: false, type: '' });
      formFeatures.details = { topicID: '', destEmail: '', subject: '', bodyHtml: '', note: '', segmentID: '', assignToAgentID: '', inResponseToID: '', addSubjectRefNum: true }

      var contactID = '';
      var interactionID = '';
      var interactionTopic = '';
      formFeatures.type = props.type;
      if (props.type == 'contact') {
        contactID = props.itemid;
      }
      if (props.type == 'interaction') {
        interactionID = props.itemid;
        var interaction = interactions.get(interactionID);
        interactionTopic = interaction.topicID;
        contactID = interaction.customerID;
      }
      Vue.watchEffect(() => {
        if (props.type == 'interaction') {
          interactionID = props.itemid;
          var interaction = interactions.get(interactionID);
          formFeatures.details.segmentID = interaction.segmentID;
        }
        else {
          var topic = topics.get(formFeatures.details.topicID);
          formFeatures.details.segmentID = topic?.segmentID;
        }
      });
      var featureTopicMenu = Vue.reactive({ options: [] });

      var featureAgents = Vue.reactive({});
      featureAgents.agentsList = [];

      Vue.watchEffect(() => {
        featureTopicMenu.options.length = 0;
        featureTopicMenu.loading = true;
        if (!segments.loaded || !topics.loaded || !agents.loaded) {
          segments.Load(); topics.Load(); agents.Load();
          return;
        }
        featureTopicMenu.loading = false;
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            group.options.push({ name: topic.name, id: topic.id, level: Math.min(2, topic.depth) });
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.value, r.value); });
          if (group.options.length < 1) return;
          featureTopicMenu.options.push(group);
        });
        var agentIDs = iservice.HaveRight('Tab.MessageQueue.Supervise') ? agents.forSegment(formFeatures.details.segmentID) : agents.forTopic(formFeatures.details.topicID ? formFeatures.details.topicID : interactionTopic);
        featureAgents.agentsList = [];
        agentIDs.forEach(function(agentID){
          var agent = agents.get(agentID);
          var name = agent.name || agent.email || ('Agent ' + agent.id);
          var option = {
            id: agentID,
            name: name,
            email: agent.email,
            search: agent.name + '\n' + agent.email,
            disabled: !agent.isLoggedIn && agent.doEmpty,
            isaway: !agent.isLoggedIn && !agent.doEmpty
          };
          featureAgents.agentsList.push(option);
        });
        featureAgents.agentsList.sort((l, r) => {
          return l.name.localeCompare(r.name);
        });
      
      });
      formFeatures.TabClass = function (tab) { return { 'active': tab == formFeatures.tab }; }
      formFeatures.TabClick = function (tab) {
        formFeatures.tab = tab;
      }

      var anchorSend = Vue.ref(null);
      var featureUploads = iservice.FeatureUploads();

      formFeatures.loading = true;
      function startTicket() {
        formFeatures.processing = true;
        if (formFeatures.details.topicID) {
          var topic = topics.get(formFeatures.details.topicID);
          formFeatures.details.segmentID = topic.segmentID;
        }
        iservice.API('webapp-api-history?api=startticket', null, {
          topicID: formFeatures.details.topicID ? formFeatures.details.topicID : interactionTopic,
          interactionID: interactionID ? interactionID : "",
          segmentID: formFeatures.details.segmentID ? formFeatures.details.segmentID : "",
          customerID: contactID ? contactID : ""
        }).then(function (data) {

          formFeatures.addresses = data.addresses;
          formFeatures.details.destEmail = data.addresses.length ? data.addresses[0] : '';
          topicSelectedOperationHandler();
          formFeatures.featureProps = Vue.reactive({ label: 'Answer Properties', length: 0 });
          var intProps = data.interactionProperties.length ? data.interactionProperties[0].properties : [];
          iservice.EditableProperties(formFeatures.featureProps, intProps);

          formFeatures.featureCaseProps = Vue.reactive({ label: 'Case Properties', length: 0 });
          var caseProps = data.caseProperties.length ? data.caseProperties[0].properties : [];
          iservice.EditableProperties(formFeatures.featureCaseProps, caseProps);
          Vue.watchEffect(function () {
            formFeatures.featureCaseProps.length = formFeatures.featureCaseProps.props.length;
            formFeatures.featureProps.length = formFeatures.featureProps.props.length;
            formFeatures.haveProperties = formFeatures.featureCaseProps.props.length || formFeatures.featureProps.props.length;
          });
          Vue.watchEffect(function () {
            formFeatures.propertyMissingClass = { 'text-red': formFeatures.featureCaseProps.missingRequired || formFeatures.featureProps.missingRequired };
          });
          Vue.watchEffect(function () {
            formFeatures.hasValues = formFeatures.featureCaseProps.hasValues || formFeatures.featureProps.hasValues;
          });
        }).catch(function (errors) {
          formFeatures.errors = errors;
        }).finally(function () {
          formFeatures.loading = false;
          formFeatures.processing = false;
        });
      }
      startTicket();

      var topicSelectedOperationHandler = function () {
        if (!topics.list.length) return;
        formFeatures.featureProps = Vue.reactive({ label: 'Question Properties', length: 0 });
        var topic = topics.get(formFeatures.details.topicID ? formFeatures.details.topicID : interactionTopic);
        
        var intProps = topic.questionProperties.length ? topic.questionProperties[0].properties : [];
        iservice.EditableProperties(formFeatures.featureProps, intProps);

        formFeatures.featureCaseProps = Vue.reactive({ label: 'Case Properties', length: 0 });
        var caseProps = topic.caseProperties.length ? topic.caseProperties[0].properties : [];
        iservice.EditableProperties(formFeatures.featureCaseProps, caseProps);
        Vue.watchEffect(function () {
          formFeatures.featureCaseProps.length = formFeatures.featureCaseProps.props.length;
          formFeatures.featureProps.length = formFeatures.featureProps.props ? formFeatures.featureProps.props.length : 0;
          formFeatures.haveProperties = formFeatures.featureCaseProps.props.length || (formFeatures.featureProps.props ? formFeatures.featureProps.props.length : 0);
        });
        Vue.watchEffect(function () {
          formFeatures.propertyMissingClass = { 'text-red': formFeatures.featureCaseProps.missingRequired || formFeatures.featureProps.missingRequired };
        });
        Vue.watchEffect(function () {
          formFeatures.hasValues = formFeatures.featureCaseProps.hasValues || formFeatures.featureProps.hasValues;
        });
        var agentIDs = iservice.HaveRight('Tab.MessageQueue.Supervise') ? agents.forSegment(formFeatures.details.segmentID) : agents.forTopic(formFeatures.details.topicID ? formFeatures.details.topicID : interactionTopic);
        featureAgents.agentsList = [];
        agentIDs.forEach(agentID => {
          var agent = agents.get(agentID);
          var name = agent.name || agent.email || ('Agent ' + agent.id);
          var option = {
            id: agentID,
            name: name,
            email: agent.email,
            search: agent.name + '\n' + agent.email,
            disabled: !agent.isLoggedIn && agent.doEmpty,
            isaway: !agent.isLoggedIn && !agent.doEmpty
          };
          featureAgents.agentsList.push(option);
        });
        featureAgents.agentsList.sort((l, r) => {
          return l.name.localeCompare(r.name);
        });
        formFeatures.details.assignToAgentID = '';
        formFeatures.details.mailboxID = '';
      };
      var editor = Vue.ref(null);
      var anchorStocks = Vue.ref(null);
      var featureStockResponses = Vue.reactive({ filter: '', segmentID: formFeatures.details.segmentID });
      Vue.watchEffect(function () {
        if (!formFeatures.details.topicID) return;
        var topic = topics.get(formFeatures.details.topicID);
        featureStockResponses.segmentID = topic?.segmentID;
      })
      featureStockResponses.Open = function () {
        featureStockResponses.Close = iservice.components.popupHolder.Open(anchorStocks, 'stock-responses-menu', false, featureStockResponses);
      }
      featureStockResponses.PasteVar = function (text) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteText(text);
      }
      featureStockResponses.PasteStock = function (html) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteHtml(html);
      }
      function Cancel() {
        context.emit('cancel');
      }
      function CheckProperties(feature, label) {
        for (var prop of feature.props) {
          if (!prop.missingRequired) continue;
          prop.focusMe = true;
          formFeatures.TabClick('properties');
          formFeatures.errors = ['Must set value for ' + label + ' property ' + prop.def.name + '.'];
          return false;
        }
        return true;
      }
      var errors = Vue.reactive({});
      formFeatures.createTicket = function () {
        formFeatures.errors = [];
        if (props.type == 'contact') {
          if (!formFeatures.details.topicID) {
            formFeatures.errors.push({ msg: 'Please select a topic.', details: { description: 'It looks like you are missing the topic of your email. Please select a topic before sending.' } });
            errors.topicID = true;
          }
        }
        if (props.type == 'interaction')
          formFeatures.details.topicID = interactionTopic;

        if (!formFeatures.details.destEmail) {
          formFeatures.errors.push({ msg: 'Please enter a From email address.', details: { description: 'It looks like you are missing From email address. Please select a From email before sending.' } });
          errors.destEmail = true;
        }
        if (!formFeatures.details.subject) {
          formFeatures.errors.push({ subject: true, msg: 'Please enter a subject for your email.', details: { description: 'Every interaction must have a subject, but it looks like one is missing here. Please enter a subject for your email before sending.' } });
          errors.subject = true;
        }
        if (!formFeatures.details.bodyHtml) {
          formFeatures.errors.push({ msg: 'Your email doesn\'t have a body.', details: { description: 'It looks like you are missing the body of this email. Please type your message in the editor before sending.' } });
          errors.bodyHtml = true;
        }
        if (formFeatures.errors.length) return;
        /*if (formFeatures.featureProps.props) {
          if (!CheckProperties(formFeatures.featureProps, 'answer')) return;
        }
        if (formFeatures.featureCaseProps.props) {
          if (!CheckProperties(formFeatures.featureCaseProps, 'case')) return;
        }*/
        formFeatures.processing = true;
        formFeatures.details.action = 5;
        formFeatures.details.inResponseToID = interactionID;
        formFeatures.details.properties = formFeatures.featureProps.out;
        formFeatures.details.caseProperties = formFeatures.featureCaseProps.out;
        formFeatures.details.contactID = contactID;
        iservice.API('webapp-api-custinfo?api=createticket', null, { details: formFeatures.details }, featureUploads.files).then(function (data) {
          interactions.add(data.row);
          config.threadChildren[props.threadID].push(data.interactionID);
          iservice.SuccessMessage('Ticket created successfully.');
          Cancel();
        }).catch(function (errors) {
          formFeatures.errors = errors;
        }).finally(function () {
          formFeatures.processing = false;
        });
      }
      formFeatures.OnTopicSelect = function () {
        if (!formFeatures.details.topicID) return;
        topicSelectedOperationHandler();
      }
      return { formFeatures, contactID, anchorSend, featureUploads, editor, anchorStocks, featureStockResponses, Cancel, featureTopicMenu, errors, featureAgents };
    }
  };
  iservice.components.myStockResponses = {
    name: 'vueapp-my-stock-responses',
    template: '#vueapp-my-stock-responses',
    route() { return { path: iservice.formPath + '/my-stock-responses', priority: 200, name: 'my-stock-responses', meta: { title: "iService | My Stock Responses" } } },
    setup() {
      var loggedIn = iservice.LoggedInData();
      var stockResponses = iservice.MyStockResponsesData();
      var selectedID = Vue.ref(null);
      var formFeatures = Vue.reactive({});
      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        filterLabel: 'Filter Stock Responses',
        emptyMessage: 'No Stock Responses',
        pageSize: 5,
        expandable: true,
        searching: false,
        initialSort: 'ID',
        exportName: "My Stock Responses"
      });
      var ListCollapsed = Vue.reactive({ isListCollapsed: true });

      function Select(id) {
        if (id && id > 0) {
          formFeatures.isEdit = true;
        } else { formFeatures.isEdit = false; }
        selectedID.oldValue = id;
        selectedID.value = id;
      }
      function Loading(val) {
        formFeatures.loading = val;
      }
      function Cancel() {
        Select(null);
      }
      function Notify(msg) {
        formFeatures.success = msg;
      }

      var stockResponsesShown = Vue.computed(() => {
        var list = [];
        stockResponses.list.forEach(x => {
          list.push(x.id);
        });
        return list;
      });
      var selectedIDs = Vue.computed(() => {
        if (selectedID.value != null) return [selectedID.value];
        return [];
      });
      
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }

      Vue.watchEffect(() => {
        tableConfig.columns = [
          Vue.reactive({
            name: 'ID', label: 'ID',
            type: 'number',
            filterText: iservice.tables.SimpleFilterTextFunc(stockResponses, 'id'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Name', label: 'Name',
            filterText: iservice.tables.SimpleFilterTextFunc(stockResponses, 'name'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Description', label: 'Description',
            filterText: iservice.tables.SimpleFilterTextFunc(stockResponses, 'description'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          })];
      });
      Loading(true);
      iservice.API('webapp-api-admin?api=agentdetails', null, { agentID: loggedIn.contactID }).then(function (data) {
        formFeatures.details = data.details;
        data.details.stockResponses.forEach(function (response) {
          stockResponses.add(response);
        });
      }).catch(errors => {
        formFeatures.errors = errors;
      }).finally(() => {
        Loading(false);
      });

      formFeatures.delete = function () {
        if (selectedID.value) {
          Loading(true);
          formFeatures.deleteClicked = false;
          formFeatures.isEdit = false;
          let param = { stockID: selectedID.value };
          iservice.API('webapp-api-admin?api=stockresponses&mode=delete', null, param).then(function (response) {
            let index = stockResponses.list.findIndex(x => x.id == param.stockID);
            if (index >= 0) {
              stockResponses.list.splice(index, 1);
            }
            selectedID.value = null;
            iservice.SuccessMessage("Stock Response deleted successfully.");
          }).catch(errors => {
            formFeatures.errors = errors;
          }).finally(() => {
            Loading(false);
          });
        }
      }
      
      return { tableConfig, formFeatures, selectedID, selectedIDs, Select, ListCollapsed, stockResponsesShown, Loading, Cancel, Notify};
    }
  }

  iservice.components.myStockResponseEdit = {
    name: 'vueapp-my-stock-response-edit',
    template: '#vueapp-my-stock-response-edit',
    props: ['id'],
    setup(props, context) {
      var loggedIn = iservice.LoggedInData();
      var formFeatures = Vue.reactive({ errors: null, success: null});
      var stockResponses = iservice.MyStockResponsesData();
      var stockResponseName = Vue.ref(null);
      var editor = Vue.ref(null);
      var anchorStocks = Vue.ref(null);
      var featureStockResponses = Vue.reactive({ filter: '', segmentID: formFeatures.details && formFeatures.details.segmentIDs[0] });
      featureStockResponses.Open = function () {
        featureStockResponses.Close = iservice.components.popupHolder.Open(anchorStocks, 'stock-responses-menu', false, featureStockResponses);
      }
      formFeatures.stockResponse = {
        contactID: loggedIn.contactID,
        name: null,
        description: "",
        bodyHtml: "",
        id: 0,
        segmentID: null
      };
      Vue.watchEffect(() => {
        if (props.id != 0) {
          context.emit('loading', true);
          iservice.API('webapp-api-admin?api=stockresponsedetails', null, { stockID: props.id }).then(function (data) {
            formFeatures.stockResponse["id"] = data.details["id"];
            formFeatures.stockResponse["name"] = data.details["name"];
            formFeatures.stockResponse["description"] = data.details["description"];
            formFeatures.stockResponse["bodyHtml"] = data.details["bodyHtml"];
            formFeatures.stockResponse["segmentID"] = data.details["segmentID"];
          }).catch(errors => {
            formFeatures.errors = errors;
          }).finally(() => {
            context.emit('loading', false);
          });
        }
      });

      formFeatures.save = function () {
        context.emit('loading', true);
        let param = { details: formFeatures.stockResponse};
        if (!formFeatures.stockResponse.name) {
          formFeatures.errors = ["Stock response name is Required."];
          formFeatures.hasRequiredErrors = true;
          stockResponseName.value.focus();
          return;
        }
        let mode = "";
        if (formFeatures.stockResponse.id == 0) {
          delete param.details.id;
          delete param.details.segmentID;
          mode = "new";
        } else {
          param["stockID"] = formFeatures.stockResponse.id;
          mode = "modify";
        }
        iservice.API('webapp-api-admin?api=stockresponses&mode=' + mode, null, param).then(function (response) {
          context.emit('select', response.row.id);
          if (mode == "new") {
            stockResponses.add(response.row);
            iservice.SuccessMessage('Stock Response Created Successfully.');
          } else {
            iservice.SuccessMessage('Stock Response Updated Successfully.');
          }
          context.emit('notify', formFeatures.success);
        }).catch(errors => {
          formFeatures.errors = errors;
        }).finally(() => {
          context.emit('loading', false);
        });
      }

      formFeatures.cancel = function () {
        context.emit('cancel');
      }
      
      return { formFeatures, featureStockResponses, editor, anchorStocks, stockResponses, stockResponseName };
    }
  }
})(iservice);