﻿var iservice = window.iservice || {};
(function (iservice) {
  iservice.data = {};

  iservice.Messages = function () {
    if (iservice.data.messages) return iservice.data.messages;

    var messages = iservice.data.messages = Vue.reactive({});

    var channels = {};
    function New() {
      return { listeners: [], queue: [] };
    }
    messages.Listen = function (name, callback) {
      var channel = channels[name];
      if (!channel) {
        channel = channels[name] = New();
      }
      channel.listeners.push(callback);
      Vue.onUnmounted(function () {
        var index = channel.listeners.indexOf(callback);
        if (index == -1) return;
        channel.listeners.splice(index, 1);
      });
      if (channel.queue.length > 0) {
        channel.queue.forEach(param => {
          callback(param);
        });
        channel.queue.length = 0;
      }
    }
    messages.Send = function (name, param) {
      var channel = channels[name];
      if (!channel) {
        channel = channels[name] = New();
      }
      if (channel.listeners.length > 0) channel.listeners.forEach(func => {
        func(param);
      });
      else channel.queue.push(param);
    }

    return messages;
  };

  iservice.activeChatsIdsStorage = function () {
    var key = 'activeChatsIdsStorage-contact' + iservice.data.loggedIn.contactID;

    return {
      getData: function () {
        var stateObj = JSON.parse(window.localStorage.getItem(key));

        return stateObj ? stateObj : [];
      },
      setData: function (data) {
        window.localStorage.setItem(key, JSON.stringify(data));
      },
      clearData: function () {
        window.localStorage.removeItem(key);
      }
    };
  }
  iservice.chatFiltersStorage = function () {
    var key = 'chatFiltersStorage-contact' + iservice.data.loggedIn.contactID;

    return {
      getData: function () {
        var stateObj = JSON.parse(window.localStorage.getItem(key));

        return stateObj ? stateObj : { tag: 'chat', search: '' };
      },
      setData: function (data) {
        window.localStorage.setItem(key, JSON.stringify(data));
      }
    };
  }
  iservice.chatUtilities = function () {
    var addTargetRegEx = /(<a)/ig;

    return {
      setLinksTargetToBlank: function (markup) {
        return markup.replace(addTargetRegEx, '$1 target="_blank"');
      }
    };
  }
  iservice.LoggedInData = function () {
    if (iservice.data.loggedIn) return iservice.data.loggedIn;
    
    var loggedIn = iservice.data.loggedIn = Vue.reactive(window.loggedIn);
    var accessRightMap = Vue.computed(() => {
      var map = {};
      loggedIn.accessRights.forEach(function (right) { map[right] = true; });
      return map;
    });
    iservice.HaveRight = function (right) {
      return (right && accessRightMap.value) ? accessRightMap.value[right] : false;
    }
    var segmentAccessMap = Vue.computed(() => {
      var map = {};
      loggedIn.accessibleSegments.forEach(id => {
        map[id] = true;
      });
      return map;
    });
    iservice.HaveSegment = function (segmentID) {
      return (segmentID && segmentAccessMap.value) ? segmentAccessMap.value[segmentID] : false;
    }

    loggedIn.Update = function (data) {
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) loggedIn[dest] = data[src]; }
      ['accessRights', 'accessibleSegments', 'broadcastMessage', 'chats', 'contactID', 'contactName', 'isAvailable',
       'isLoggedIn', 'isPasswordResetConfigured', 'logins', 'numChats', 'spamPolicyUrl', 'tenantID', 'sid'
      ].forEach(member => {
        copy(member);
      });
      if (loggedIn.isLoggedIn) {
        loggedIn.localLogout = false;
      }
    }

    loggedIn.Login = function (feature, e, loginName, password) {
      if (feature.working) return;
      feature.errors = null;
      feature.success = null;
      if (emptyStr(loginName) || emptyStr(password)) {
        feature.errors = ['It looks like you are missing some values. Please enter both a Login Name and Password and click the Login button.'];
        return;
      }
      feature.working = true;
      iservice.API('webapp-api-login?api=login', e, { emailAddress: loginName, password: password }).then(function (data) {
        if (!data.loggedIn || data.loggedIn.isLoggedIn != true) return Promise.reject('Login failed.');
        data.loggedIn.sid = data.sid;
        loggedIn.Update(data.loggedIn);
      }).catch(function (errors) {
        feature.errors = errors;
      }).finally(function () {
        feature.working = false;
      });
    }

    loggedIn.Logout = function (feature) {
      if (feature.working) return;
      loggedIn.localLogout = true;
      feature.working = true;
      iservice.API('webapp-api-login?api=logout', null, {}).then(function (data) {
        loggedIn.Update(data.loggedIn);
      }).finally(function () {
        feature.working = false;
      });
    }

    return loggedIn;
  }
  iservice.InteractionData = function () {
    if (iservice.data.interactions) return iservice.data.interactions;

    var interactions = iservice.data.interactions = Vue.reactive({});
    interactions.map = {};
    interactions.add = function (data) {
      var id = data.id;
      var obj = interactions.map[id];
      if (!obj) {
        obj = interactions.map[id] = Vue.reactive({ id: id, tags: {}, attachments: [], intProps: [], caseProps: [], valuesMap: {}, statusAudit: [] });
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['type', 'status', 'segmentID', 'currentStatus', 'columnColor', 'statusClass', 'segmentName', 'date', 'topicID', 'topicName', 'threadID', 'subject', 'subjectBare', 'customerID', 'customerName', 'customerLogins', 'fromAddress',
       'assignedToID', 'assignedToName', 'resolvedByName', 'approvedByName', 'agentName', 'bodyHtml', 'bodyPlain', 'headerFrom', 'headerTo', 'cc', 'bcc', 'note', 'properties'
      ].forEach(member => {
        copy(member);
      });
      if (data.tags) {
        obj.tags = {};
        data.tags.forEach((tag) => { obj.tags[tag] = true; });
      }
      function copytype(src) {
        if (!src) src = 'type';
        if (data[src] === undefined) return;
        obj.type = data[src];
        switch (obj.type) {
          case 'Auto Response': obj.type = 'Message Acknowledgment'; break;
          case 'Find Answer': obj.type = 'Find Answer Article'; break;
          case 'Notification': obj.type = 'Secure Notification'; break;
          case 'Ticket': obj.type = 'Agent Ticket'; break;
          case 'Ask A Question': obj.type = 'Customer Ticket'; break;
          case 'Note': obj.type = 'Public Note'; break;
        }
      }
      copytype();
      if (data.attachments && data.attachments.length !== undefined) {
        obj.attachments.length = 0;
        data.attachments.forEach(attach => {
          obj.attachments.push(Vue.reactive(attach));
        });
      }
      iservice.CopyProps(obj.intProps, data.interactionProperties, obj.valuesMap);
      iservice.CopyProps(obj.caseProps, data.caseProperties, obj.valuesMap);
      if (data.properties) for (var prop of data.properties) {
        obj.valuesMap[prop.propertyID] = prop.values;
      }
      if (data.statusAudit && data.statusAudit.length !== undefined) {
        obj.statusAudit.length = 0;
        data.statusAudit.forEach(status => {
          obj.statusAudit.push({ date: moment(status.date).format('YYYY-MM-DD hh:mm:ss A'), modifierName: status.modifierName, statusName: status.statusName, action: status.action, assignedToName: status.assignedToName, topicName: status.topicName });
        });
      }
    }
    interactions.get = function (id) {
      return interactions.map[id];
    }

    var deleteCallbacks = [];
    interactions.WhenDeleted = function (callback, noUnmount) {
      deleteCallbacks.push(callback);
      if (!noUnmount) Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    interactions.deleted = function (id) {
      if (!interactions.map[id]) return;
      deleteCallbacks.forEach(func => {
        func(id);
      });
      delete interactions.map[id];
    }

    return interactions;
  }
  iservice.SmtpOutData = function () {
    if (iservice.data.smtpout) return iservice.data.smtpout;

    var smtpout = iservice.data.smtpout = Vue.reactive({});
    smtpout.map = Vue.reactive({});
    smtpout.add = function (data) {
      if (data.isDeleted) return smtpout.deleted(data);
      var id = data.messageID;
      var obj = smtpout.map[id];
      if (!obj) {
        obj = smtpout.map[id] = Vue.reactive({ id: id });
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['interactionID', 'contactID', 'to', 'subject', 'intendedUtc', 'whenUtc', 'numberErrors'
      ].forEach(member => {
        copy(member);
      });
      if (data.tags) {
        obj.tags = {};
        data.tags.forEach((tag) => { obj.tags[tag] = true; });
      }
    }
    smtpout.get = function (id) {
      return smtpout.map[id];
    }

    smtpout.deleted = function (data) {
      var id = data.messageID;
      if (!smtpout.map[id]) return;
      delete smtpout.map[id];
    }
    smtpout.clear = function () {
      for (var name in smtpout.map) {
        delete smtpout.map[name];
      }
    }

    return smtpout;
  }
  iservice.ArticleData = function () {
    if (iservice.data.articles) return iservice.data.articles;

    var articles = iservice.data.articles = Vue.reactive({});
    articles.map = {};

    articles.add = function (data) {
      var id = data.id;
      var obj = articles.map[id];
      if (!obj) {
        obj = articles.map[id] = Vue.reactive({ id: id, subject: '', date: '', rating: '', topicID: '', topicName: '', viewCount: '', public: '', properties: [], propertiesEdit: [], attachments: [], questionHtml: '', answerHtml: '', creatorName: '', isSubscribed: '', isPublic: '' });
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['subject', 'date', 'rating', 'topicID', 'topicName', 'viewCount', 'public', 'properties', 'propertiesEdit', 'attachments', 'questionHtml', 'answerHtml', 'creatorName', 'isSubscribed', 'isPublic'].forEach(member => {
        copy(member);
      });
    }
    articles.get = function (id) {
      return articles.map[id];
    }

    var deleteCallbacks = [];
    articles.WhenDeleted = function (callback) {
      deleteCallbacks.push(callback);
      Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    articles.deleted = function (id) {
      if (!articles.map[id]) return;
      deleteCallbacks.forEach(func => {
        func(id);
      });
      delete articles.map[id];
    }

    return articles;
  }
  iservice.IntPropertyData = function () {
    if (iservice.data.intProperties) return iservice.data.intProperties;

    var properties = iservice.data.intProperties = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      properties.map = Vue.reactive({});
      properties.list.length = 0;
      properties.loaded = false;
      properties.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    properties.Load = function () {
      if (properties.working || properties.loaded) return;
      properties.working = true;
    }
    Vue.watchEffect(() => {
      if (properties.loaded || !properties.working) return;
      iservice.API('webapp-api-admin?api=intpropertiesall', null, {}).then(data => {
        data.intProperties.forEach(properties.add);
      }).catch(errors => {

      }).finally(() => {
        properties.working = false;
        properties.loaded = true;
      });
    });

    var segments = Vue.computed(() => {
      var map = {};
      properties.list.forEach(prop => {
        if (!prop.segmentID) return;
        var segment = map[prop.segmentID];
        if (!segment) {
          segment = map[prop.segmentID] = [];
        }
        segment.push(prop);
      });
      return map;
    });
    properties.inSegment = function (id) {
      return segments.value[id] || [];
    }

    properties.add = function (data) {
      var id = data.id;
      if (!id) return;
      var obj = properties.map[id];
      if (!obj) {
        obj = properties.map[id] = {};
        properties.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'option', 'isRequired', 'isDate', 'type', 'isInteger', 'isNumber', 'isShared', 'isViewable', 'isEditable', 'segmentID',
       'segmentName', 'purpose', 'hasDescriptions', 'isMultiLine', 'allowMultiple'].forEach(member => {
        copy(member);
      });
    }
    properties.get = function (id) {
      return properties.map[id];
    }
    properties.getByName = function (name) {
      for (var i in properties.map) {
        if (properties.map[i].name == name) return properties.map[i];
      }
      return null;
    }

    return properties;
  }
  iservice.ContactsData = function () {
    if (iservice.data.contacts) return iservice.data.contacts;

    var contacts = iservice.data.contacts = Vue.reactive({});
    contacts.map = {};
    contacts.add = function (data) {
      var id = data.id;
      var obj = contacts.map[id];
      if (!obj) {
        contacts.map[id] = Vue.reactive({ id: id, tags: {}, valuesMap: {}});
        obj = contacts.map[id];
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'logins', 'firstname', 'lastname', 'address1', 'address2', 'customertype', 'postalcode', 'city', 'country', 'state',
        'email', 'phone', 'company', 'address', 'propertySummary'
      ].forEach(member => {
        copy(member);
      });
      if (data.propertySummary) {
        for (var prop of data.propertySummary) {
          obj.valuesMap[prop.propertyID] = prop.values;
        }
      }
    }
    contacts.get = function (id) {
      return contacts.map[id];
    }
    return contacts;
  }
  iservice.ContactPropertyData = function () {
    if (iservice.data.contactProperties) return iservice.data.contactProperties;

    var properties = iservice.data.contactProperties = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      properties.map = Vue.reactive({});
      properties.list.length = 0;
      properties.loaded = false;
      properties.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    properties.Load = function () {
      if (properties.working || properties.loaded) return;
      properties.working = true;
    }
    Vue.watchEffect(() => {
      if (properties.loaded || !properties.working) return;
      iservice.API('webapp-api-admin?api=contactpropertiesall', null, {}).then(data => {
        data.contactproperties.forEach(properties.add);
      }).catch(errors => {

      }).finally(() => {
        properties.working = false;
        properties.loaded = true;
      });
    });

    var segments = Vue.computed(() => {
      var map = {};
      properties.list.forEach(prop => {
        if (!prop.segmentID) return;
        var segment = map[prop.segmentID];
        if (!segment) {
          segment = map[prop.segmentID] = [];
        }
        segment.push(prop);
      });
      return map;
    });
    properties.inSegment = function (id) {
      return segments.value[id] || [];
    }

    properties.add = function (data) {
      var id = data.id;
      if (!id) return;
      var obj = properties.map[id];
      if (!obj) {
        obj = properties.map[id] = {};
        properties.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'option', 'type', 'isDate', 'isInteger', 'isNumber', 'isShared', 'isViewable', 'isEditable',
       'purpose', 'segmentID', 'segmentName', 'hasDescriptions', 'isMultiLine', 'allowMultiple'].forEach(member => {
        copy(member);
      });
    }
    properties.get = function (id) {
      return properties.map[id];
    }

    return properties;
  }
  iservice.AutoResponseData = function () {
    if (iservice.data.autoResponses) return iservice.data.autoResponses;

    var autoResponses = iservice.data.autoResponses = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      autoResponses.map = Vue.reactive({});
      autoResponses.list.length = 0;
      autoResponses.loaded = false;
      autoResponses.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    autoResponses.Load = function () {
      if (autoResponses.working || autoResponses.loaded) return;
      autoResponses.working = true;
    }
    Vue.watchEffect(() => {
      if (autoResponses.loaded || !autoResponses.working) return;
      iservice.API('webapp-api-admin?api=notificationsall', null, {}).then(data => {
        data.notifications.forEach(autoResponses.add);
        data.externalNotify.forEach(ext => {
          var response = autoResponses.get(ext.id);
          if (!response) return;
          response.forExternal = true;
        });
      }).catch(errors => {

      }).finally(() => {
        autoResponses.working = false;
        autoResponses.loaded = true;
      });
    });

    var segments = Vue.computed(() => {
      var map = {};
      autoResponses.list.forEach(response => {
        if (!response.segmentID) return;
        var segment = map[response.segmentID];
        if (!segment) {
          segment = map[response.segmentID] = [];
        }
        segment.push(response);
      });
      return map;
    });
    autoResponses.inSegment = function (id) {
      return segments.value[id] || [];
    }
    autoResponses.SortCompare = function (lid, rid) {
      var l = autoResponses.get(lid), r = autoResponses.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }

    autoResponses.add = function (data) {
      var id = data.id;
      if (!id) return;
      var obj = autoResponses.map[id];
      if (!obj) {
        obj = autoResponses.map[id] = {};
        autoResponses.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'segmentID', 'segmentName', 'listNames', 'description', 'subject', 'bodyHtml', 'noReply'].forEach(member => {
        copy(member);
      });
      if (data.lists && data.lists.length !== undefined) {
        obj.lists = {};
        data.lists.forEach(list => {
          obj.lists[list] = true;
        });
      }
    }
    autoResponses.get = function (id) {
      return autoResponses.map[id];
    }

    return autoResponses;
  }
  iservice.ServiceLevelData = function () {
    if (iservice.data.servicelevels) return iservice.data.servicelevels;

    var servicelevels = iservice.data.servicelevels = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      servicelevels.map = Vue.reactive({});
      servicelevels.list.length = 0;
      servicelevels.loaded = false;
      servicelevels.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    servicelevels.Load = function () {
      if (servicelevels.working || servicelevels.loaded) return;
      servicelevels.working = true;
    }
    Vue.watchEffect(() => {
      if (servicelevels.loaded || !servicelevels.working) return;
      iservice.API('webapp-api-admin?api=servicelevelall', null, {}).then(data => {
        data.servicelevels.forEach(servicelevels.add);
      }).catch(errors => {

      }).finally(() => {
        servicelevels.working = false;
        servicelevels.loaded = true;
      });
    });

    servicelevels.SortCompare = function (lid, rid) {
      var l = servicelevels.get(lid), r = servicelevels.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }

    servicelevels.add = function (data) {
      var id = data.id;
      if (!id) return;
      var obj = servicelevels.map[id];
      if (!obj) {
        obj = servicelevels.map[id] = {};
        servicelevels.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'minutes', 'intPropertyID', 'intPropertyValue', 'contactPropertyID', 'contactPropertyValue'].forEach(member => {
        copy(member);
      });
      if (data.lists && data.lists.length !== undefined) {
        obj.lists = {};
        data.lists.forEach(list => {
          obj.lists[list] = true;
        });
      }
    }
    servicelevels.get = function (id) {
      return servicelevels.map[id];
    }

    return servicelevels;
  }
  iservice.SegmentData = function () {
    if (iservice.data.segments) return iservice.data.segments;

    var segments = iservice.data.segments = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      segments.map = Vue.reactive({});
      segments.list.length = 0;
      segments.loaded = false;
      segments.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    segments.Load = function () {
      if (segments.working || segments.loaded) return;
      segments.working = true;
    }
    Vue.watchEffect(() => {
      if (segments.loaded || !segments.working) return;
      iservice.API('webapp-api-admin?api=segmentsall', null, {}).then(data => {
        data.segments.forEach(segments.add);
      }).catch(errors => {

      }).finally(() => {
        segments.working = false;
        segments.loaded = true;
      });
    });

    var segmentPaths = Vue.computed(() => {
      function PathName(segment) {
        if (!segment) return '';
        if (segment.hidden) return '';
        var parent = segments.get(segment.parentID);
        return PathName(parent) + '!@#$' + segment.name;
      }
      var paths = {};
      segments.list.forEach(segment => {
        paths[segment.id] = PathName(segment);
      });
      return paths;
    });
    segments.TreeSortCompare = function (lid, rid) {
      var lname = segmentPaths.value[lid], rname = segmentPaths.value[rid];
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }
    segments.SortCompare = function (lid, rid) {
      var l = segments.get(lid), r = segments.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }

    segments.add = function (data) {
      var id = data.id;
      var obj = segments.map[id];
      if (!obj) {
        segments.map[id] = {};
        obj = segments.map[id];
        segments.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'parentID', 'option', 'isStandard', 'hidden', 'mailboxID'].forEach(member => {
        copy(member);
      });
    }
    segments.get = function (id) {
      return segments.map[id];
    }

    return segments;
  }
  iservice.TopicData = function () {
    if (iservice.data.topics) return iservice.data.topics;

    var topics = iservice.data.topics = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      topics.map = Vue.reactive({});
      topics.list.length = 0;
      topics.loaded = false;
      topics.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    topics.Load = function () {
      if (topics.working || topics.loaded) return;
      topics.working = true;
    }
    Vue.watchEffect(() => {
      if (topics.loaded || !topics.working) return;
      iservice.API('webapp-api-admin?api=topicsall', null, {}).then(data => {
        data.topics.forEach(topics.add);
      }).catch(errors => {

      }).finally(() => {
        topics.working = false;
        topics.loaded = true;
      });
    });

    var segments = Vue.computed(() => {
      var map = {};
      topics.list.forEach(topic => {
        if (!topic.segmentID) return;
        var segment = map[topic.segmentID];
        if (!segment) {
          segment = map[topic.segmentID] = [];
        }
        segment.push(topic);
      });
      return map;
    });
    topics.inSegment = function (id) {
      return segments.value[id] || [];
    }

    var topicDepths = Vue.ref({});
    Vue.watchEffect(() => {
      function Depth(topic) {
        if (!topic) return 0;
        var parent = topics.get(topic.parentID);
        if (!parent) return 0;
        return Depth(parent) + 1;
      }
      topics.list.forEach(topic => {
        var depth = Depth(topic);
        topic.depth = depth;
        topicDepths.value[topic.id] = Depth(topic);
      });
    });

    var topicPaths = Vue.computed(() => {
      function PathName(topic) {
        if (!topic) return '';
        var parent = topics.get(topic.parentID);
        if (!parent) {
          var segment = iservice.data.segments.get(topic.segmentID);
          return (segment ? segment.name : '') + '!@#$' + topic.name;
        }
        return PathName(parent) + '!@#$' + topic.name;
      }
      var paths = {};
      topics.list.forEach(topic => {
        paths[topic.id] = PathName(topic);
      });
      return paths;
    });
    topics.TreeSortCompare = function (lid, rid) {
      var lname = topicPaths.value[lid], rname = topicPaths.value[rid];
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }
    topics.SortCompare = function (lid, rid) {
      var l = topics.get(lid), r = topics.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }

    topics.add = function (data) {
      var id = data.id;
      var obj = topics.map[id];
      if (!obj) {
        obj = topics.map[id] = Vue.reactive({ id: id, questionProps: [], caseProps: [], answerProps: [] });
        topics.list.push(obj);
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'parentID', 'autoResponseID', 'neverAutoResponse', 'agentNotifyID', 'neverAgentNotify', 'askAQuestionFilterID', 'visibility', 'skillNames',
       'serviceLevelID', 'prefix', 'option', 'padding', 'segmentID', 'segmentName', 'propertyIDs', 'propertyNames',
       'caseProperties', 'questionProperties', 'answerProperties', 'faProperties'].forEach(member => {
        copy(member);
      });
      copy('skills', 'skillIDs');
      iservice.CopyProps(obj.caseProps, data.caseProperties);
      iservice.CopyProps(obj.questionProps, data.questionProperties);
      iservice.CopyProps(obj.answerProps, data.answerProperties);
    }
    topics.get = function (id) {
      return topics.map[id];
    }
    topics.remove = function (id) {
      topics.list = topics.list.filter(function (list) { return list.id != id; })
      delete topics.map[id];
    }

    return topics;
  }
  iservice.TopicDeletedData = function () {
    if (iservice.data.topicsDeleted) return iservice.data.topicsDeleted;

    var topicsDeleted = iservice.data.topicsDeleted = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      topicsDeleted.map = Vue.reactive({});
      topicsDeleted.list.length = 0;
      topicsDeleted.loaded = false;
      topicsDeleted.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    topicsDeleted.Load = function () {
      if (topicsDeleted.working || topicsDeleted.loaded) return;
      topicsDeleted.working = true;
    }
    Vue.watchEffect(() => {
      if (topicsDeleted.loaded || !topicsDeleted.working) return;
      iservice.API('webapp-api-admin?api=topicsdeletedall', null, {}).then(data => {
        data.topics.forEach(topicsDeleted.add);
      }).catch(errors => {

      }).finally(() => {
        topicsDeleted.working = false;
        topicsDeleted.loaded = true;
      });
    });

    var segments = Vue.computed(() => {
      var map = {};
      topicsDeleted.list.forEach(topic => {
        if (!topic.segmentID) return;
        var segment = map[topic.segmentID];
        if (!segment) {
          segment = map[topic.segmentID] = [];
        }
        segment.push(topic);
      });
      return map;
    });
    topicsDeleted.inSegment = function (id) {
      return segments.value[id] || [];
    }

    var topicDepths = Vue.ref({});
    Vue.watchEffect(() => {
      function Depth(topic) {
        if (!topic) return 0;
        var parent = topicsDeleted.get(topic.parentID);
        if (!parent) return 0;
        return Depth(parent) + 1;
      }
      topicsDeleted.list.forEach(topic => {
        var depth = Depth(topic);
        topic.depth = depth;
        topicDepths.value[topic.id] = Depth(topic);
      });
    });

    var topicPaths = Vue.computed(() => {
      function PathName(topic) {
        if (!topic) return '';
        var parent = topicsDeleted.get(topic.parentID);
        if (!parent) {
          var segment = iservice.data.segments.get(topic.segmentID);
          return (segment ? segment.name : '') + '!@#$' + topic.name;
        }
        return PathName(parent) + '!@#$' + topic.name;
      }
      var paths = {};
      topicsDeleted.list.forEach(topic => {
        paths[topic.id] = PathName(topic);
      });
      return paths;
    });
    topicsDeleted.TreeSortCompare = function (lid, rid) {
      var lname = topicPaths.value[lid], rname = topicPaths.value[rid];
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }
    topicsDeleted.SortCompare = function (lid, rid) {
      var l = topicsDeleted.get(lid), r = topicsDeleted.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }

    topicsDeleted.add = function (data) {
      var id = data.id;
      var obj = topicsDeleted.map[id];
      if (!obj) {
        obj = topicsDeleted.map[id] = Vue.reactive({ id: id, questionProps: [], caseProps: [], answerProps: [] });
        topicsDeleted.list.push(obj);
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'parentID', 'autoResponseID', 'neverAutoResponse', 'agentNotifyID', 'neverAgentNotify', 'askAQuestionFilterID', 'visibility', 'skillNames',
       'serviceLevelID', 'prefix', 'option', 'padding', 'segmentID', 'segmentName', 'propertyIDs', 'propertyNames',
       'caseProperties', 'questionProperties', 'answerProperties', 'faProperties'].forEach(member => {
        copy(member);
      });
      copy('skills', 'skillIDs');
      iservice.CopyProps(obj.caseProps, data.caseProperties);
      iservice.CopyProps(obj.questionProps, data.questionProperties);
      iservice.CopyProps(obj.answerProps, data.answerProperties);
    }
    topicsDeleted.get = function (id) {
      return topicsDeleted.map[id];
    }
    topicsDeleted.remove = function (id) {
      topicsDeleted.list = topicsDeleted.list.filter(function (list) { return list.id != id; })
      delete topicsDeleted.map[id];
    }

    return topicsDeleted;
  }
  iservice.CustomerTopicData = function () {
    if (iservice.data.customertopics) return iservice.data.customertopics;

    var customertopics = iservice.data.customertopics = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      customertopics.map = Vue.reactive({});
      customertopics.list.length = 0;
      customertopics.loaded = false;
      customertopics.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    customertopics.Load = function () {
      if (customertopics.working || customertopics.loaded) return;
      customertopics.working = true;
    }
    Vue.watchEffect(() => {
      if (customertopics.loaded || !customertopics.working) return;
      iservice.API('webapp-api-findanswer?api=topics', null, {}).then(data => {
        data.topics.forEach(customertopics.add);
      }).catch(errors => {

      }).finally(() => {
        customertopics.working = false;
        customertopics.loaded = true;
      });
    });

    var segments = Vue.computed(() => {
      var map = {};
      customertopics.list.forEach(topic => {
        if (!topic.segmentID) return;
        var segment = map[topic.segmentID];
        if (!segment) {
          segment = map[topic.segmentID] = [];
        }
        segment.push(topic);
      });
      return map;
    });
    customertopics.inSegment = function (id) {
      return segments.value[id] || [];
    }

    var topicDepths = Vue.ref({});
    Vue.watchEffect(() => {
      function Depth(topic) {
        if (!topic) return 0;
        var parent = customertopics.get(topic.parentID);
        if (!parent) return 0;
        return Depth(parent) + 1;
      }
      customertopics.list.forEach(topic => {
        var depth = Depth(topic);
        topic.depth = depth;
        topicDepths.value[topic.id] = Depth(topic);
      });
    });

    var topicPaths = Vue.computed(() => {
      function PathName(topic) {
        if (!topic) return '';
        var parent = customertopics.get(topic.parentID);
        if (!parent) {
          var segment = iservice.data.segments.get(topic.segmentID);
          return (segment ? segment.name : '') + '!@#$' + topic.name;
        }
        return PathName(parent) + '!@#$' + topic.name;
      }
      var paths = {};
      customertopics.list.forEach(topic => {
        paths[topic.id] = PathName(topic);
      });
      return paths;
    });
    customertopics.TreeSortCompare = function (lid, rid) {
      var lname = topicPaths.value[lid], rname = topicPaths.value[rid];
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }
    customertopics.SortCompare = function (lid, rid) {
      var l = customertopics.get(lid), r = customertopics.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }

    customertopics.add = function (data) {
      var id = data.id;
      var obj = customertopics.map[id];
      if (!obj) {
        obj = customertopics.map[id] = Vue.reactive({ id: id, questionProps: [], caseProps: [], answerProps: [] });
        customertopics.list.push(obj);
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'parentID', 'count', 'countRecurse', 'visibility', 'prefix', 'option', 'segmentName', 'isAvailable', 'segmentID'].forEach(member => {
          copy(member);
        });
      copy('skills', 'skillIDs');
      iservice.CopyProps(obj.caseProps, data.caseProperties);
      iservice.CopyProps(obj.questionProps, data.questionProperties);
      iservice.CopyProps(obj.answerProps, data.answerProperties);
    }
    customertopics.get = function (id) {
      return customertopics.map[id];
    }
    customertopics.remove = function (id) {
      customertopics.list = customertopics.list.filter(function (list) { return list.id != id; })
      delete customertopics.map[id];
    }

    return customertopics;
  }
  iservice.AgentData = function () {
    if (iservice.data.agents) return iservice.data.agents;

    var agents = iservice.data.agents = Vue.reactive({ loaded: false, working: false, list: [], data: [] });
    var websyncUnsub = null;
    function Clear() {
      agents.map = Vue.reactive({});
      agents.list.length = 0;
      agents.data.length = 0;
      agents.loaded = false;
      agents.working = false;

      if (websyncUnsub) websyncUnsub();
      websyncUnsub = null;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    agents.Load = function () {
      if (agents.working || agents.loaded) return;
      agents.working = true;

      if (websyncUnsub) websyncUnsub();
      websyncUnsub = websync.subscribe('/agentstatus/' + iservice.data.loggedIn.tenantID, msg => {
        var data = msg.getData();
        if (!data || !data.id) return;
        iservice.data.agents.add(data);
      });
    }
    Vue.watchEffect(() => {
      if (agents.loaded || !agents.working) return;
      iservice.API('webapp-api-admin?api=agentsall', null, {}).then(data => {
        data.agents.forEach(agents.add);
        agents.data = { agentsChat: data.agentsChat, agentsMax: data.agentsMax, agentsMaxChat: data.agentsMaxChat, agentsNum: data.agentsNum };
      }).catch(errors => {

      }).finally(() => {
        agents.working = false;
        agents.loaded = true;
      });
    });

    agents.add = function (data) {
      var id = data.id;
      var obj = agents.map[id];
      if (!obj) {
        agents.map[id] = { haveSkill: {}, haveSegment: {} };
        obj = agents.map[id];
        agents.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['email', 'name', 'displayname', 'typeID', 'segments', 'skills', 'departments', 'doReceive', 'isLoggedIn',
       'isAvailable', 'activity', 'doEmpty', 'timeout', 'allskills', 'windowsLoginName', 'allsegments', 'canchat'].forEach(member => {
        copy(member);
       });
      if (data.allskills && data.allskills.length !== undefined) {
        obj.haveSkill = {};
        data.allskills.forEach(id => {
          obj.haveSkill[id] = true;
        });
      }
      if (data.allsegments && data.allsegments.length !== undefined) {
        obj.haveSegment = {};
        data.allsegments.forEach(id => {
          obj.haveSegment[id] = true;
        });
      }
    }
    agents.get = function (id) {
      return agents.map[id];
    }

    var agentsForSegment = Vue.computed(() => {
      if (!iservice.data.segments.loaded) {
        iservice.data.segments.Load();
        return {};
      }
      var segments = {};
      iservice.data.segments.list.forEach(s => {
        var segment = { id: s.id, agents: [] };
        segments[s.id] = segment;
        agents.list.forEach(agent => {
          if (!agent.haveSegment[s.id]) return;
          segment.agents.push(agent.id);
        });
      });
      return segments;
    });
    agents.forSegment = function (segmentID) {
      var segment = agentsForSegment.value[segmentID];
      return segment ? segment.agents : [];
    }

    agents.forInteraction = function (intID) {
      if (!iservice.data.skills.loaded) {
        iservice.data.skills.Load();
        return [];
      }
      var list = [];
      var skills = iservice.data.skills.forInteraction(intID) || [];
      agents.list.forEach(agent => {
        var int = iservice.data.interactions.get(intID);
        if (!int) return;
        if (!agent.haveSegment[int.segmentID]) return;
        if (skills.length < 1) {
          list.push(agent.id);
        }
        else {
          for (var skillID of skills) {
            if (!agent.haveSkill[skillID]) return;
          }
          list.push(agent.id);
        }
      });
      return list;
    }

    var agentsForTopic = Vue.computed(() => {
      var topics = {};
      iservice.data.topics.list.forEach(t => {
        var topic = { id: t.id, agents: [] };
        topics[t.id] = topic;
        agents.list.forEach(agent => {
          if (!agent.haveSegment[t.segmentID]) return;
          for (var skillID of t.skills) {
            if (!agent.haveSkill[skillID]) return;
          }
          topic.agents.push(agent.id);
        });
      });
      return topics;
    });
    agents.forTopic = function (topicID) {
      var topic = agentsForTopic.value[topicID];
      if (!topic) return [];
      return topic.agents;
    }

    return agents;
  }
  iservice.StockResponsesData = function () {
    if (iservice.data.stockresponses) return iservice.data.stockresponses;

    var stockresponses = iservice.data.stockresponses = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      stockresponses.map = Vue.reactive({});
      stockresponses.list.length = 0;
      stockresponses.loaded = false;
      stockresponses.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    stockresponses.Load = function () {
      if (stockresponses.working || stockresponses.loaded) return;
      stockresponses.working = true;
    }
    Vue.watchEffect(() => {
      if (stockresponses.loaded || !stockresponses.working) return;
      iservice.API('webapp-api-admin?api=stockresponseall', null, {}).then(data => {
        data.segments.forEach(segment => {
          segment.stocks.forEach(stockresponses.add);
        });
        data.agent.forEach(stockresponses.add);
      }).catch(errors => {

      }).finally(() => {
        stockresponses.working = false;
        stockresponses.loaded = true;
      });
    });

    var segments = Vue.computed(() => {
      var map = {};
      stockresponses.list.forEach(stock => {
        if (!stock.segmentID) return;
        var segment = map[stock.segmentID];
        if (!segment) {
          segment = map[stock.segmentID] = [];
        }
        segment.push(stock);
      });
      return map;
    });
    stockresponses.inSegment = function (id) {
      return segments.value[id] || [];
    }

    var mine = Vue.computed(() => {
      var list = [];
      stockresponses.list.forEach(stock => {
        if (stock.segmentID) return;
        list.push(stock);
      });
      return list;
    });
    stockresponses.ofMine = function () {
      return mine.value;
    }

    stockresponses.add = function (data) {
      var id = data.id;
      var obj = stockresponses.map[id];
      if (!obj) {
        stockresponses.map[id] = {};
        obj = stockresponses.map[id];
        stockresponses.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['segmentID', 'name', 'description', 'bodyHtml', 'bodyPreview'].forEach(member => {
        copy(member);
      });
    }
    stockresponses.get = function (id) {
      return stockresponses.map[id];
    }

    return stockresponses;
  }
  iservice.WebsitesData = function () {
    if (iservice.data.websites) return iservice.data.websites;

    var websites = iservice.data.websites = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      websites.map = Vue.reactive({});
      websites.list.length = 0;
      websites.loaded = false;
      websites.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    websites.Load = function () {
      if (websites.working || websites.loaded) return;
      websites.working = true;
    }
    Vue.watchEffect(() => {
      if (websites.loaded || !websites.working) return;
      iservice.API('webapp-api-admin?api=websitesall', null, {}).then(data => {
        data.websites.forEach(websites.add);
      }).catch(errors => {

      }).finally(() => {
        websites.working = false;
        websites.loaded = true;
      });
      
    });

    websites.add = function (data) {
      var id = data.id;
      var obj = websites.map[id];
      if (!obj) {
        websites.map[id] = {};
        obj = websites.map[id];
        websites.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'domainsnames', 'segmentnames'].forEach(member => {
        copy(member);
      });
    }
    websites.get = function (id) {
      return websites.map[id];
    }
    return websites;
  }
  iservice.FormsData = function () {
    if (iservice.data.forms) return iservice.data.forms;

    var forms = iservice.data.forms = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      forms.map = Vue.reactive({});
      forms.list.length = 0;
      forms.loaded = false;
      forms.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    forms.Load = function () {
      if (forms.working || forms.loaded) return;
      forms.working = true;
    }
    Vue.watchEffect(() => {
      if (forms.loaded || !forms.working) return;
      iservice.API('webapp-api-admin?api=formall', null, {}).then(data => {
        data.forms.forEach(forms.add);
      }).catch(errors => {

      }).finally(() => {
        forms.working = false;
        forms.loaded = true;
      });
      
    });

    forms.add = function (data) {
      var id = data.id;
      var obj = forms.map[id];
      if (!obj) {
        forms.map[id] = {};
        obj = forms.map[id];
        forms.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'description', 'modifierName', 'modifyTime'].forEach(member => {
        copy(member);
      });
    }
    forms.get = function (id) {
      return forms.map[id];
    }
    return forms;
  }
  iservice.FormsBuiltinData = function () {
    if (iservice.data.formsbuiltin) return iservice.data.formsbuiltin;

    var formsbuiltin = iservice.data.formsbuiltin = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      formsbuiltin.map = Vue.reactive({});
      formsbuiltin.list.length = 0;
      formsbuiltin.loaded = false;
      formsbuiltin.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    formsbuiltin.Load = function () {
      if (formsbuiltin.working || formsbuiltin.loaded) return;
      formsbuiltin.working = true;
    }
    Vue.watchEffect(() => {
      if (formsbuiltin.loaded || !formsbuiltin.working) return;
      iservice.API('webapp-api-admin?api=formbuiltinall', null, {}).then(data => {
        data.forms.forEach(formsbuiltin.add);
      }).catch(errors => {

      }).finally(() => {
        formsbuiltin.working = false;
        formsbuiltin.loaded = true;
      });
      
    });

    formsbuiltin.add = function (data) {
      var id = data.id;
      var obj = formsbuiltin.map[id];
      if (!obj) {
        formsbuiltin.map[id] = {};
        obj = formsbuiltin.map[id];
        formsbuiltin.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'description'].forEach(member => {
        copy(member);
      });
    }
    formsbuiltin.get = function (id) {
      return formsbuiltin.map[id];
    }
    return formsbuiltin;
  }
  iservice.UserTypesData = function () {
    if (iservice.data.usertypes) return iservice.data.usertypes;

    var usertypes = iservice.data.usertypes = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      usertypes.map = Vue.reactive({});
      usertypes.list.length = 0;
      usertypes.loaded = false;
      usertypes.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    usertypes.Load = function () {
      if (usertypes.working || usertypes.loaded) return;
      usertypes.working = true;
    }
    Vue.watchEffect(() => {
      if (usertypes.loaded || !usertypes.working) return;
      iservice.API('webapp-api-admin?api=usertypesall', null, {}).then(data => {
        data.usertypes.forEach(usertypes.add);
      }).catch(errors => {

      }).finally(() => {
        usertypes.working = false;
        usertypes.loaded = true;
      });

    });
    usertypes.SortCompare = function (lid, rid) {
      var l = usertypes.get(lid), r = usertypes.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }
    usertypes.add = function (data) {
      var id = data.id;
      var obj = usertypes.map[id];
      if (!obj) {
        usertypes.map[id] = {};
        obj = usertypes.map[id];
        usertypes.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'count'].forEach(member => {
        copy(member);
      });
    }
    usertypes.get = function (id) {
      return usertypes.map[id];
    }
    var deleteCallbacks = [];
    usertypes.WhenDeleted = function (callback) {
      deleteCallbacks.push(callback);
      Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    usertypes.deleted = function (id) {
      if (!usertypes.map[id]) return;
      deleteCallbacks.forEach(func => {
        func(id);
      });
      delete usertypes.map[id];
    }
    return usertypes;
  }
  iservice.DepartmentsData = function () {
    if (iservice.data.departments) return iservice.data.departments;

    var departments = iservice.data.departments = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      departments.map = Vue.reactive({});
      departments.list.length = 0;
      departments.loaded = false;
      departments.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    departments.Load = function () {
      if (departments.working || departments.loaded) return;
      departments.working = true;
    }
    Vue.watchEffect(() => {
      if (departments.loaded || !departments.working) return;
      iservice.API('webapp-api-admin?api=departmentsall', null, {}).then(data => {
        data.departments.forEach(departments.add);
      }).catch(errors => {

      }).finally(() => {
        departments.working = false;
        departments.loaded = true;
      });

    });
    departments.SortCompare = function (lid, rid) {
      var l = departments.get(lid), r = departments.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }
    departments.add = function (data) {
      var id = data.id;
      var obj = departments.map[id];
      if (!obj) {
        departments.map[id] = {};
        obj = departments.map[id];
        departments.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'description', 'segmentnames', 'skillnames'].forEach(member => {
        copy(member);
      });
    }
    departments.get = function (id) {
      return departments.map[id];
    }
    var deleteCallbacks = [];
    departments.WhenDeleted = function (callback) {
      deleteCallbacks.push(callback);
      Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    departments.deleted = function (id) {
      if (!departments.map[id]) return;
      deleteCallbacks.forEach(func => {
        func(id);
      });
      delete departments.map[id];
    }
    return departments;
  }
  iservice.MasslistData = function () {
    if (iservice.data.masslists) return iservice.data.masslists;

    var masslists = iservice.data.masslists = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      masslists.map = Vue.reactive({});
      masslists.list.length = 0;
      masslists.loaded = false;
      masslists.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    masslists.Load = function () {
      if (masslists.working || masslists.loaded) return;
      masslists.working = true;
    }
    Vue.watchEffect(() => {
      if (masslists.loaded || !masslists.working) return;
      iservice.API('webapp-api-admin?api=masslistall', null, {}).then(data => {
        data.masslists.forEach(masslists.add);
      }).catch(errors => {

      }).finally(() => {
        masslists.working = false;
        masslists.loaded = true;
      });
      
    });
    masslists.SortCompare = function (lid, rid) {
      var l = masslists.get(lid), r = masslists.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }
    masslists.add = function (data) {
      var id = data.id;
      var obj = masslists.map[id];
      if (!obj) {
        masslists.map[id] = {};
        obj = masslists.map[id];
        masslists.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'description', 'segments', 'shown', 'subCount', 'unsubCount', 'lastMail'].forEach(member => {
        copy(member);
      });
    }
    masslists.get = function (id) {
      return masslists.map[id];
    }
    var deleteCallbacks = [];
    masslists.WhenDeleted = function (callback) {
      deleteCallbacks.push(callback);
      Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    masslists.deleted = function (id) {
      if (!masslists.map[id]) return;
      deleteCallbacks.forEach(func => {
        func(id);
      });
      delete masslists.map[id];
    }
    return masslists;
  }
  iservice.MassMessagesData = function () {
    if (iservice.data.massmessages) return iservice.data.massmessages;

    var massmessages = iservice.data.massmessages = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      massmessages.map = Vue.reactive({});
      massmessages.list.length = 0;
      massmessages.loaded = false;
      massmessages.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    massmessages.Load = function () {
      if (massmessages.working || massmessages.loaded) return;
      massmessages.working = true;
    }
    Vue.watchEffect(() => {
      if (massmessages.loaded || !massmessages.working) return;
      iservice.API('webapp-api-admin?api=massmessageall', null, {}).then(data => {
        data.massmessages.forEach(massmessages.add);
      }).catch(errors => {

      }).finally(() => {
        massmessages.working = false;
        massmessages.loaded = true;
      });
      
    });
    massmessages.SortCompare = function (lid, rid) {
      var l = massmessages.get(lid), r = massmessages.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }
    massmessages.add = function (data) {
      var id = data.id;
      var obj = massmessages.map[id];
      if (!obj) {
        massmessages.map[id] = {};
        obj = massmessages.map[id];
        massmessages.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'description', 'topicID', 'subject', 'addRefNum', 'lastMail'].forEach(member => {
        copy(member);
      });
    }
    massmessages.get = function (id) {
      return massmessages.map[id];
    }
    var deleteCallbacks = [];
    massmessages.WhenDeleted = function (callback) {
      deleteCallbacks.push(callback);
      Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    massmessages.deleted = function (id) {
      if (!massmessages.map[id]) return;
      deleteCallbacks.forEach(func => {
        func(id);
      });
      delete massmessages.map[id];
    }
    return massmessages;
  }
  iservice.MassMailingsData = function () {
    if (iservice.data.massmailings) return iservice.data.massmailings;

    var massmailings = iservice.data.massmailings = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      massmailings.map = Vue.reactive({});
      massmailings.list.length = 0;
      massmailings.loaded = false;
      massmailings.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    massmailings.Load = function () {
      if (massmailings.working || massmailings.loaded) return;
      massmailings.working = true;
    }
    Vue.watchEffect(() => {
      if (massmailings.loaded || !massmailings.working) return;
      iservice.API('webapp-api-admin?api=massmailingall', null, {}).then(data => {
        data.massmailings.forEach(massmailings.add);
      }).catch(errors => {

      }).finally(() => {
        massmailings.working = false;
        massmailings.loaded = true;
      });
      
    });
    massmailings.SortCompare = function (lid, rid) {
      var l = massmailings.get(lid), r = massmailings.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }
    massmailings.add = function (data) {
      var id = data.id;
      var obj = massmailings.map[id];
      if (!obj) {
        massmailings.map[id] = {};
        obj = massmailings.map[id];
        massmailings.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'description', 'topicID', 'listID', 'messageID', 'mailboxID', 'lastMail'].forEach(member => {
        copy(member);
      });
    }
    massmailings.get = function (id) {
      return massmailings.map[id];
    }
    var deleteCallbacks = [];
    massmailings.WhenDeleted = function (callback) {
      deleteCallbacks.push(callback);
      Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    massmailings.deleted = function (id) {
      if (!massmailings.map[id]) return;
      deleteCallbacks.forEach(func => {
        func(id);
      });
      delete massmailings.map[id];
    }
    return massmailings;
  }
  iservice.MassCampaignsData = function () {
    if (iservice.data.masscampaigns) return iservice.data.masscampaigns;

    var masscampaigns = iservice.data.masscampaigns = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      masscampaigns.map = Vue.reactive({});
      masscampaigns.list.length = 0;
      masscampaigns.loaded = false;
      masscampaigns.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    masscampaigns.Load = function () {
      if (masscampaigns.working || masscampaigns.loaded) return;
      masscampaigns.working = true;
    }
    Vue.watchEffect(() => {
      if (masscampaigns.loaded || !masscampaigns.working) return;
      iservice.API('webapp-api-admin?api=masscampaignall', null, {}).then(data => {
        data.masscampaigns.forEach(masscampaigns.add);
      }).catch(errors => {

      }).finally(() => {
        masscampaigns.working = false;
        masscampaigns.loaded = true;
      });
      
    });
    masscampaigns.SortCompare = function (lid, rid) {
      var l = masscampaigns.get(lid), r = masscampaigns.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }
    masscampaigns.add = function (data) {
      var id = data.id;
      var obj = masscampaigns.map[id];
      if (!obj) {
        masscampaigns.map[id] = {};
        obj = masscampaigns.map[id];
        masscampaigns.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'description', 'segments', 'shown', 'subCount', 'unsubCount'].forEach(member => {
        copy(member);
      });
    }
    masscampaigns.get = function (id) {
      return masscampaigns.map[id];
    }
    var deleteCallbacks = [];
    masscampaigns.WhenDeleted = function (callback) {
      deleteCallbacks.push(callback);
      Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    masscampaigns.deleted = function (id) {
      if (!masscampaigns.map[id]) return;
      deleteCallbacks.forEach(func => {
        func(id);
      });
      delete masscampaigns.map[id];
    }
    return masscampaigns;
  }
  iservice.MassCriteriaData = function () {
    if (iservice.data.masscriteria) return iservice.data.masscriteria;

    var masscriteria = iservice.data.masscriteria = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      masscriteria.map = Vue.reactive({});
      masscriteria.list.length = 0;
      masscriteria.loaded = false;
      masscriteria.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    masscriteria.Load = function () {
      if (masscriteria.working || masscriteria.loaded) return;
      masscriteria.working = true;
    }
    Vue.watchEffect(() => {
      if (masscriteria.loaded || !masscriteria.working) return;
      iservice.API('webapp-api-admin?api=masscriteriaall', null, {}).then(data => {
        data.masscriterias.forEach(masscriteria.add);
      }).catch(errors => {

      }).finally(() => {
        masscriteria.working = false;
        masscriteria.loaded = true;
      });
      
    });
    masscriteria.SortCompare = function (lid, rid) {
      var l = masscriteria.get(lid), r = masscriteria.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }
    masscriteria.add = function (data) {
      var id = data.id;
      var obj = masscriteria.map[id];
      if (!obj) {
        masscriteria.map[id] = {};
        obj = masscriteria.map[id];
        masscriteria.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'description', 'segments'].forEach(member => {
        copy(member);
      });
    }
    masscriteria.get = function (id) {
      return masscriteria.map[id];
    }
    var deleteCallbacks = [];
    masscriteria.WhenDeleted = function (callback) {
      deleteCallbacks.push(callback);
      Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    masscriteria.deleted = function (id) {
      if (!masscriteria.map[id]) return;
      deleteCallbacks.forEach(func => {
        func(id);
      });
      delete masscriteria.map[id];
    }
    return masscriteria;
  }

  iservice.QuotesData = function () {
    if (iservice.data.quotes) return iservice.data.quotes;

    var quotes = iservice.data.quotes = Vue.reactive({});
    quotes.map = {};
    quotes.add = function (data) {
      var id = data.id;
      var obj = quotes.map[id];
      if (!obj) {
        quotes.map[id] = Vue.reactive({ id: id, segment: "", quotetopic: "", followuptopic: "", stockresponses: "" });
        obj = quotes.map[id];
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['segment', 'quotetopic', 'followuptopic', 'stockresponses'].forEach(member => {
        copy(member);
      });
      if (data.tags) {
        obj.tags = {};
        data.tags.forEach((tag) => { obj.tags[tag] = true; });
      }
    }
    quotes.get = function (id) {
       return quotes.map[id];
    }

    var deleteCallbacks = [];
    quotes.WhenDeleted = function (callback) {
      deleteCallbacks.push(callback);
      Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    quotes.deleted = function (id) {
      if (!quotes.map[id]) return;
      deleteCallbacks.forEach(func => {
        func(id);
      });
      delete quotes.map[id];
    }
    return quotes;
  }
  iservice.PlaceholdersData = function () {
    if (iservice.data.placeholders) return iservice.data.placeholders;

    var placeholders = iservice.data.placeholders = Vue.reactive({});
    placeholders.map = {};
    placeholders.add = function (data) {
      var name = data.name;
      var obj = placeholders.map[name];
      if (!obj) {
        placeholders.map[name] = Vue.reactive({ name: "", value: "", defaultValue: "" });
        obj = placeholders.map[name];
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'value', 'defaultValue'].forEach(member => {
        copy(member);
      });
      if (data.tags) {
        obj.tags = {};
        data.tags.forEach((tag) => { obj.tags[tag] = true; });
      }
    }
    placeholders.get = function (name) {
      return placeholders.map[name];
    }

    return placeholders;
  }
  iservice.SkillData = function () {
    if (iservice.data.skills) return iservice.data.skills;

    var skills = iservice.data.skills = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      skills.map = Vue.reactive({});
      skills.list.length = 0;
      skills.loaded = false;
      skills.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    skills.Load = function () {
      if (skills.working || skills.loaded) return;
      skills.working = true;
    }
    Vue.watchEffect(() => {
      if (skills.loaded || !skills.working) return;
      iservice.API('webapp-api-admin?api=skillsall', null, {}).then(data => {
        data.skills.forEach(skills.add);
      }).catch(errors => {

      }).finally(() => {
        skills.working = false;
        skills.loaded = true;
      });
      
    });

    skills.add = function (data) {
      var id = data.id;
      var obj = skills.map[id];
      if (!obj) {
        skills.map[id] = {};
        obj = skills.map[id];
        skills.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'propertyID', 'propertyValue'].forEach(member => {
        copy(member);
      });
    }
    skills.get = function (id) {
      return skills.map[id];
    }

    skills.forInteraction = function (intID) {
      if (!iservice.data.topics.loaded) {
        iservice.data.topics.Load();
        return [];
      }
      var int = iservice.data.interactions.get(intID);
      if (!int) return [];
      var skillIDs = [];
      var found = {};
      var topic = iservice.data.topics.get(int.topicID);
      if (topic) topic.skills.forEach(id => {
        if (found[id]) return;
        found[id] = true;
        skillIDs.push(id);
      });
      skills.list.forEach(skill => {
        if (found[skill.id]) return;
        if (!skill.propertyID) return;
        if (!iservice.InteractionHasPropValue(int, skill.propertyID, skill.propertyValue)) return;
        found[skill.id] = true;
        skillIDs.push(skill.id);
      });
      return skillIDs;
    }

    return skills;
  }
  iservice.MailboxData = function () {
    if (iservice.data.mailboxes) return iservice.data.mailboxes;

    var mailboxes = iservice.data.mailboxes = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      mailboxes.map = Vue.reactive({});
      mailboxes.list.length = 0;
      mailboxes.loaded = false;
      mailboxes.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    mailboxes.Load = function () {
      if (mailboxes.working || mailboxes.loaded) return;
      mailboxes.working = true;
    }
    Vue.watchEffect(() => {
      if (mailboxes.loaded || !mailboxes.working) return;
      iservice.API('webapp-api-admin?api=mailboxesall', null, {}).then(data => {
        data.mailboxes.forEach(mailboxes.add);
      }).catch(errors => {

      }).finally(() => {
        mailboxes.working = false;
        mailboxes.loaded = true;
      });      
    });

    var segments = Vue.computed(() => {
      var map = {};
      mailboxes.list.forEach(mailbox => {
        if (!mailbox.segmentID) return;
        var segment = map[mailbox.segmentID];
        if (!segment) {
          segment = map[mailbox.segmentID] = [];
        }
        segment.push(mailbox);
      });
      return map;
    });
    mailboxes.inSegment = function (id) {
      return segments.value[id] || [];
    }

    mailboxes.SortCompare = function (lid, rid) {
      var l = mailboxes.get(lid), r = mailboxes.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }

    mailboxes.add = function (data) {
      var id = data.id;
      var obj = mailboxes.map[id];
      if (!obj) {
        mailboxes.map[id] = {};
        obj = mailboxes.map[id];
        mailboxes.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'segmentName', 'segmentID', 'displayName', 'replyTo', 'enabled', 'option'].forEach(member => {
        copy(member);
      });
    }
    mailboxes.get = function (id) {
      return mailboxes.map[id];
    }

    return mailboxes;
  }
  iservice.MailboxAdminData = function () {
    if (iservice.data.mailboxesadmin) return iservice.data.mailboxesadmin;

    var mailboxesadmin = iservice.data.mailboxesadmin = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      mailboxesadmin.map = Vue.reactive({});
      mailboxesadmin.list.length = 0;
      mailboxesadmin.loaded = false;
      mailboxesadmin.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    mailboxesadmin.Load = function () {
      if (mailboxesadmin.working || mailboxesadmin.loaded) return;
      mailboxesadmin.working = true;
    }
    Vue.watchEffect(() => {
      if (mailboxesadmin.loaded || !mailboxesadmin.working) return;
      iservice.API('webapp-api-admin?api=mailboxesadminlist', null, {}).then(data => {
        data.mailboxes.forEach(mailboxesadmin.add);
      }).catch(errors => {

      }).finally(() => {
        mailboxesadmin.working = false;
        mailboxesadmin.loaded = true;
      });      
    });

    var segments = Vue.computed(() => {
      var map = {};
      mailboxesadmin.list.forEach(mailbox => {
        if (!mailbox.segmentID) return;
        var segment = map[mailbox.segmentID];
        if (!segment) {
          segment = map[mailbox.segmentID] = [];
        }
        segment.push(mailbox);
      });
      return map;
    });
    mailboxesadmin.inSegment = function (id) {
      return segments.value[id] || [];
    }

    mailboxesadmin.SortCompare = function (lid, rid) {
      var l = mailboxesadmin.get(lid), r = mailboxesadmin.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }

    mailboxesadmin.add = function (data) {
      var id = data.id;
      var obj = mailboxesadmin.map[id];
      if (!obj) {
        mailboxesadmin.map[id] = {};
        obj = mailboxesadmin.map[id];
        mailboxesadmin.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'segmentName', 'segmentID', 'displayName', 'replyTo', 'enabled', 'option', 'topicID', 'filterID', 'inProtocol', 'inServer', 'inPort', 'inEncrypt', 'inUsername', 'inPassword', 'outProtocol', 'outServer', 'outPort', 'outEncrypt', 'outUsername', 'outPassword', 'ewsAppID', 'ewsSecret', 'ewsTenantID', 'errorCount'].forEach(member => {
        copy(member);
      });
    }
    mailboxesadmin.get = function (id) {
      return mailboxesadmin.map[id];
    }

    return mailboxesadmin;
  }
  iservice.FilterData = function () {
    if (iservice.data.filters) return iservice.data.filters;

    var filters = iservice.data.filters = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      filters.map = Vue.reactive({});
      filters.list.length = 0;
      filters.loaded = false;
      filters.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    filters.Load = function () {
      if (filters.working || filters.loaded) return;
      filters.working = true;
    }
    Vue.watchEffect(() => {
      if (filters.loaded || !filters.working) return;
      iservice.API('webapp-api-admin?api=filtersall', null, {}).then(data => {
        data.filters.forEach(filters.add);
      }).catch(errors => {

      }).finally(() => {
        filters.working = false;
        filters.loaded = true;
      });      
    });

    var segments = Vue.computed(() => {
      var map = {};
      filters.list.forEach(filter => {
        if (!filter.segmentID) return;
        var segment = map[filter.segmentID];
        if (!segment) {
          segment = map[filter.segmentID] = { all: [], simple: [], groups: [] };
        }
        segment.all.push(filter);
        if (filter.isGroup) segment.groups.push(filter);
        else segment.simple.push(filter);
      });
      return map;
    });
    filters.inSegment = function (id) {
      var segment = segments.value[id];
      return segment ? segment.all : [];
    }
    filters.segmentSimple = function (id) {
      var segment = segments.value[id];
      return segment ? segment.simple : [];
    }
    filters.segmentGroups = function (id) {
      var segment = segments.value[id];
      return segment ? segment.groups : [];
    }

    filters.SortCompare = function (lid, rid) {
      var l = filters.get(lid), r = filters.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }

    filters.add = function (data) {
      var id = data.id;
      var obj = filters.map[id];
      if (!obj) {
        filters.map[id] = {};
        obj = filters.map[id];
        filters.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'option', 'isGroup', 'segmentID', 'segmentName', 'description', 'expression',
        'matchSubject', 'matchBody', 'matchTo', 'matchFrom', 'matchContactProperty', 'matchAttachments', 'action', 'runAllChildren', 'childIDs'].forEach(member => {
        copy(member);
      });
    }
    filters.get = function (id) {
      return filters.map[id];
    }

    return filters;
  }
  iservice.MailboxErrosData = function () {
    if (iservice.data.mailboxerrors) return iservice.data.mailboxerrors;

    var mailboxerrors = iservice.data.mailboxerrors = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      mailboxerrors.map = Vue.reactive({});
      mailboxerrors.list.length = 0;
      mailboxerrors.loaded = false;
      mailboxerrors.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    mailboxerrors.Load = function () {
      if (mailboxerrors.working || mailboxerrors.loaded) return;
      mailboxerrors.working = true;
    }

    mailboxerrors.add = function (data) {
      var id = data.timeUTC;
      var obj = mailboxerrors.map[id];
      if (!obj) {
        mailboxerrors.map[id] = {};
        obj = mailboxerrors.map[id];
        mailboxerrors.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['timeUTC', 'timeStamp', 'message'].forEach(member => {
        copy(member);
      });
    }
    mailboxerrors.get = function (id) {
      return mailboxerrors.map[id];
    }

    return mailboxerrors;
  }
  iservice.PropertyTypeData = function () {
    if (iservice.data.propertytypes) return iservice.data.propertytypes;

    var propertytypes = iservice.data.propertytypes = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      propertytypes.map = Vue.reactive({});
      propertytypes.list.length = 0;
      propertytypes.loaded = false;
      propertytypes.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    propertytypes.Load = function () {
      if (propertytypes.working || propertytypes.loaded) return;
      propertytypes.working = true;
    }
    Vue.watchEffect(() => {
      if (propertytypes.loaded || !propertytypes.working) return;
      iservice.API('webapp-api-admin?api=propertytypes', null, {}).then(data => {
        data.propertyTypes.forEach(propertytypes.add);
      }).catch(errors => {

      }).finally(() => {
        propertytypes.working = false;
        propertytypes.loaded = true;
      });      
    });

    propertytypes.SortCompare = function (lid, rid) {
      var l = propertytypes.get(lid), r = propertytypes.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }

    propertytypes.add = function (data) {
      var id = data.typeID;
      var obj = propertytypes.map[id];
      if (!obj) {
        propertytypes.map[id] = {};
        obj = propertytypes.map[id];
        propertytypes.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'typeID'].forEach(member => {
        copy(member);
      });
    }
    propertytypes.get = function (id) {
      return propertytypes.map[id];
    }

    return propertytypes;
  }
  iservice.MyQueueData = function () {
    if (iservice.data.mq) return iservice.data.mq;

    var mq = iservice.data.mq = Vue.reactive({ loaded: false, working: true });
    var inbox = mq.inbox = Vue.reactive([]);
    var subbox = mq.subbox = Vue.reactive({});
    var websyncUnsub = null;
    function Clear() {
      inbox.length = 0;
      for (var name in subbox) subbox[name].length = 0;
      mq.numCanApprove = 0;
      mq.numCanGet = 0;
      mq.loaded = false;
      mq.working = true;

      if (websyncUnsub) websyncUnsub();
      websyncUnsub = null;
      if (!emptyStr(iservice.data.loggedIn.contactID)) {
        websyncUnsub = websync.subscribe('/agent/' + iservice.data.loggedIn.tenantID + '/' + iservice.data.loggedIn.contactID, AgentInboxMessage);
      }
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    Vue.watchEffect(() => {
      if (mq.loaded || !mq.working) return;
      iservice.API('webapp-api-mq-myqueue?api=inboxes', null, {}).then(data => {
        mq.numCanGet = data.numCanGet;
        mq.numCanApprove = data.numCanApprove;
        inbox.length = 0;
        data.inbox.forEach(item => {
          iservice.data.interactions.add(item);
          inbox.push(item.id);
        });
      }).catch(errors => {

      }).finally(() => {
        mq.working = false;
        mq.loaded = true;
      });
    });

    var loading = Vue.ref(true);
    setTimeout(() => {
      loading.value = false;
    });
    Vue.watchEffect(() => {
      if (loading.value) return;
      if (iservice.data.plugins) for (var plugin of iservice.data.plugins.list) {
        var boxName = plugin.inboxComponent;
        if (!subbox[boxName]) subbox[boxName] = Vue.reactive([]);
        subbox[boxName].length = 0;
      }
      inbox.forEach((id) => {
        if (emptyStr(id)) return;
        var plugin = iservice.data.plugins.FindPlugin(id);
        var boxName = plugin.inboxComponent || 'messageQueueMyInbox';
        var box = subbox[boxName];
        if (!box) {
          box = Vue.reactive([]);
          subbox[boxName] = box;
        }
        box.push(id);
      });
    });

    function AgentInboxMessage(msg) {
      var data = msg.getData().inboxUpdate;
      if (!data || !data.id) return;
      iservice.data.interactions.add(data);
      var ours = data.assignedToID == iservice.data.loggedIn.contactID;
      var inboxPos = inbox.indexOf(data.id);
      if (inboxPos != -1 && !ours) {
        inbox.splice(inboxPos, 1);
      }
      else if (inboxPos == -1 && ours) {
        inbox.push(data.id);
      }
    }

    iservice.data.interactions.WhenDeleted(id => {
      var index = inbox.indexOf(id);
      if (index == -1) return;
      inbox.splice(index, 1);
    }, true);

    function Redirect(id) {
      var router = iservice.router;
      var plugin = iservice.data.plugins.FindPlugin(id) || iservice.data.plugins.normal;
      var boxName = plugin.inboxComponent || 'messageQueueMyInbox';
      var component = iservice.components[boxName] || iservice.components.messageQueueMyInbox;
      var route = component.route ? component.route() : iservice.components.messageQueueMyInbox.route();
      router.push(route.path);
      iservice.data.messages.Send(boxName, id);
    }
    mq.GetNext = function (feature, e, router) {
      feature.working = true;
      feature.errors = null;
      feature.success = null;
      iservice.API('webapp-api-mq-myqueue?api=getnext', e, {}).then(function (data) {
        mq.numCanGet = data.numCanGet;
        mq.numCanApprove = data.numCanApprove;
        if (emptyStr(data.id)) return;
        iservice.data.interactions.add(data.details);
        var inboxPos = inbox.indexOf(data.id);
        if (inboxPos == -1) {
          inbox.push(data.id);
        }
        Redirect(data.id);
      }).catch(function (errors) {
        feature.errors = errors
      }).finally(function () {
        feature.working = false;
      });
    }
    mq.GetApprove = function (feature, e) {
      feature.working = true;
      feature.errors = null;
      feature.success = null;
      iservice.API('webapp-api-mq-myqueue?api=getapprove', e, {}).then(function (data) {
        mq.numCanGet = data.numCanGet;
        mq.numCanApprove = data.numCanApprove;
        if (emptyStr(data.id)) return;
        iservice.data.interactions.add(data.details);
        var inboxPos = inbox.indexOf(data.id);
        if (inboxPos == -1) {
          inbox.push(data.id);
        }
        Redirect(data.id);
      }).catch(function (errors) {
        feature.errors = errors;
      }).finally(function () {
        feature.working = false;
      });
    }

    return mq;
  }
  iservice.Plugins = function () {
    if (iservice.data.plugins) return iservice.data.plugins;

    var normal = Vue.reactive({
      priority: 0,
      func: function (interaction) { return true; },
      detailsComponent: 'interaction-view',
      answerComponent: 'interaction-answer',
      inboxComponent: 'messageQueueMyInbox'
    });
    var approve = Vue.reactive({
      priority: 10,
      func: function (interaction) { return interaction.status == 'Pending'; },
      detailsComponent: 'interaction-view',
      answerComponent: 'interaction-answer',
      inboxComponent: 'messageQueueMyApprove'
    });
    var pending = Vue.reactive({
      priority: 10,
      func: function (interaction) { return interaction.status == 'SentWaitingReply'; },
      detailsComponent: 'interaction-view',
      answerComponent: 'interaction-answer',
      inboxComponent: 'messageQueueMyPending'
    });
    var chats = Vue.reactive({
      priority: 20,
      func: function (interaction) { return interaction.status == 'ChatInProgress'; },
      detailsComponent: 'interaction-view',
      answerComponent: 'interaction-answer',
      inboxComponent: 'messageQueueChats'
    });
    var plugins = iservice.data.plugins = Vue.reactive({
      list: [normal, approve, pending, chats],
      normal
    });
    plugins.FindPlugin = function (id) {
      var interaction = iservice.data.interactions.get(id);
      if (!interaction) return normal;
      var best = null;
      plugins.list.forEach(plugin => {
        if (!plugin.func) return;
        if (best != null && best.priority > plugin.priority) return;
        if (!plugin.func(interaction)) return;
        best = plugin;
      });
      return best ? best : normal;
    }

    return plugins;
  };
  iservice.VariablePickerData = function () {
    if (iservice.data.variablePicker) return iservice.data.variablePicker;

    var variablePicker = iservice.data.variablePicker = Vue.reactive({ loaded: false, working: false, sections: [] });
    function Clear() {
      variablePicker.map = Vue.reactive({});
      variablePicker.sections.length = 0;
      variablePicker.loaded = false;
      variablePicker.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
    variablePicker.Load = function () {
      if (variablePicker.working || variablePicker.loaded) return;
      variablePicker.working = true;
    }
    Vue.watchEffect(() => {
      if (variablePicker.loaded || !variablePicker.working) return;
      if (!iservice.data.segments.loaded || !iservice.data.intProperties.loaded || !iservice.data.contactProperties.loaded) {
        iservice.data.segments.Load(); iservice.data.intProperties.Load(); iservice.data.contactProperties.Load();
      }
      if (iservice.data.segments.loaded && iservice.data.intProperties.loaded && iservice.data.contactProperties.loaded) {
        variablePicker.working = false;
        variablePicker.loaded = true;
      }
    });

    Vue.watchEffect(() => {
      variablePicker.sections.length = 0;

      var contactVars = Vue.reactive({
        label: 'Contact',
        vars: [
          { label: 'Name', text: '$value -customer(name)$' }
        ],
        segments: []
      });
      variablePicker.sections.push(contactVars);
      iservice.data.segments.list.forEach(seg => {
        var segment = { label: seg.name, vars: [] };
        iservice.data.contactProperties.inSegment(seg.id).forEach(prop => {
          segment.vars.push({ label: prop.name, text: '$value -customer(property' + (prop.name.indexOf("'") >= 0 ? prop.id : "'" + prop.name + "'") + ')$' });
        });
        if (segment.vars.length > 0) contactVars.segments.push(segment);
      });

      var caseVars = Vue.reactive({
        label: 'Case',
        vars: [],
        segments: []
      });
      variablePicker.sections.push(caseVars);

      var questionVars = Vue.reactive({
        label: 'Question',
        vars: [
          { label: 'Email', text: '$value -answer(fromemail)$' }
        ],
        segments: []
      });
      variablePicker.sections.push(questionVars);

      var answerVars = Vue.reactive({
        label: 'Answer',
        vars: [
          { label: 'Original Body', text: '$value -answer(body)$' },
          { label: 'Original Date', text: '$value -answer(date)$' },
          { label: 'Original Time', text: '$value -answer(time)$' },
          { label: 'Quoted Question', text: '$value -answer(quotedbody)$' },
          { label: 'Reference Number', text: '$value -answer(ref)$' },
          { label: 'Subject', text: '$value -answer(subject)$' }
        ],
        segments: []
      });
      variablePicker.sections.push(answerVars);

      iservice.data.segments.list.forEach(seg => {
        var caseSeg = { label: seg.name, vars: [] };
        var questionSeg = { label: seg.name, vars: [] };
        var answerSeg = { label: seg.name, vars: [] };
        iservice.data.intProperties.inSegment(seg.id).forEach(prop => {
          var property = { label: prop.name };
          if (prop.purpose == 'Answer') {
            property.text = '$value -answer(property' + (prop.name.indexOf("'") >= 0 ? prop.id : "'" + prop.name + "'") + ')$';
            answerSeg.vars.push(property);
          }
          else if (prop.purpose == 'Question') {
            property.text = '$value -question(property' + (prop.name.indexOf("'") >= 0 ? prop.id : "'" + prop.name + "'") + ')$';
            questionSeg.vars.push(property);
          }
          else if (prop.purpose == 'Case') {
            property.text = '$value -case(property' + (prop.name.indexOf("'") >= 0 ? prop.id : "'" + prop.name + "'") + ')$';
            caseSeg.vars.push(property);
          }
        });
        if (caseSeg.vars.length > 0) caseVars.segments.push(caseSeg);
        if (questionSeg.vars.length > 0) questionVars.segments.push(questionSeg);
        if (answerSeg.vars.length > 0) answerVars.segments.push(answerSeg);
      });

      var links = Vue.reactive({
        label: 'Links',
        vars: [
          { label: 'Link', text: '$link changeme$' },
          { label: 'Link w/ Url', text: "$link changeme -url 'changeme'$" },
          { label: 'Link My Account Subscriptions', text: "$link myaccount -url 'Form.aspx?formID=setme'$" },
          { label: '<a></a> Link', text: "$linka changeme -text 'changeme'$" },
          { label: '<a></a> Link w/Url', text: "$linka changeme  -text 'changeme' -url 'changeme'$" },
          { label: '<a></a> Link My Account Subscriptions', text: "$linka myaccount -text 'changeme' -url 'Form.aspx?formID=setme'$" },
          { label: '<img /> Viewed Tracking Image', text: '$linkimg viewed$' }
        ],
        segments: []
      });
      variablePicker.sections.push(links);

      var time = Vue.reactive({
        label: 'Time',
        vars: [
          { label: 'Today', text: '$value -today$' }
        ],
        segments: []
      });
      variablePicker.sections.push(time);
    });

    return variablePicker;
  }
  iservice.AutocompleteData = function () {
    if (iservice.data.autocomplete) return iservice.data.autocomplete;

    var autocomplete = iservice.data.autocomplete = Vue.reactive(iservice.GetLocalStorageJson('autocomplete'));

    return autocomplete;
  };

  iservice.MyStockResponsesData = function () {
    if (iservice.data.myStockResponsesData) return iservice.data.myStockResponsesData;

    var myStockResponsesData = iservice.data.myStockResponsesData = Vue.reactive({ list: [] });
    function Clear() {
      myStockResponsesData.map = Vue.reactive({});
      myStockResponsesData.list.length = 0;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });
        
    myStockResponsesData.add = function (data) {
      var id = data.id;
      if (!id) return;
      var obj = myStockResponsesData.map[id];
      if (!obj) {
        obj = myStockResponsesData.map[id] = {};
        myStockResponsesData.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['id', 'name', 'description', 'contactID', 'bodyHtml', 'segmentID'].forEach(member => {
          copy(member);
        });
    }
    myStockResponsesData.get = function (id) {
      return myStockResponsesData.map[id];
    }

    return myStockResponsesData;
  }

  iservice.ListsData = function () {
    if (iservice.data.listsData) return iservice.data.listsData;

    var listsData = iservice.data.listsData = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      listsData.map = Vue.reactive({});
      listsData.list.length = 0;
      listsData.loaded = false;
      listsData.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });

    listsData.Load = function () {
      if (listsData.working || listsData.loaded) return;
      listsData.working = true;
    }
    Vue.watchEffect(() => {
      if (listsData.loaded || !listsData.working) return;
      iservice.API('webapp-api-admin?api=listsall', null, {}).then(data => {
        data.lists.forEach(listsData.add);
      }).catch(errors => {

      }).finally(() => {
        listsData.working = false;
        listsData.loaded = true;
      });

    });
    listsData.SortCompare = function (lid, rid) {
      var l = listsData.get(lid), r = listsData.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }

    listsData.add = function (data) {
      var id = data.id;
      if (!id) return;
      var obj = listsData.map[id];
      if (!obj) {
        obj = listsData.map[id] = {};
        listsData.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'description'].forEach(member => {
        copy(member);
      });
    }
    listsData.get = function (id) {
      return listsData.map[id];
    }

    var deleteCallbacks = [];
    listsData.WhenDeleted = function (callback) {
      deleteCallbacks.push(callback);
      Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    listsData.deleted = function (id) {
      if (!departments.map[id]) return;
      listsData.forEach(func => {
        func(id);
      });
      delete listsData.map[id];
    }

    return listsData;
  }
  iservice.ChatData = function () {
    if (iservice.data.chats) return iservice.data.chats;

    var chats = iservice.data.chats = Vue.reactive({ showIds: [], list: []});
    chats.map = {};
    chats.add = function (data) {
      var id = data.id;
      var obj = chats.map[id];
      if (!obj) {
        chats.showIds.push(id);
        obj = chats.map[id] = Vue.reactive({ id: id, customerID: '', customerName: '', elapsedChatMs: '', elapsedWaitMs: '', operatorName: '', status: '', listSubject: '', subject: '', topicID: '', topicName: '', startMs: '', timeChat: '', timeWait: '', chatData: '', unsubscribeChat: '', custDetails: '' });
        chats.list.push(obj);
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['customerID', 'customerName', 'elapsedChatMs', 'elapsedWaitMs', 'operatorName', 'status', , 'listSubject', 'subject', 'topicID', 'topicName', 'startMs', 'timeChat', 'timeWait', 'chatData', 'unsubscribeChat','custDetails'].forEach(member => {
        copy(member);
      });
    }
    chats.get = function (id) {
      return chats.map[id];
    }
    function Clear() {
      chats.map = {};
      chats.list = [];
      chats.showIds = [];
    }
    iservice.WatchLoginChange(() => {
      Clear();
    });
    
    var deleteCallbacks = [];
    chats.WhenDeleted = function (callback) {
      deleteCallbacks.push(callback);
      Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    chats.deleted = function (id) {
      if (!chats.map[id]) return;
      deleteCallbacks.forEach(func => {
        func(id);
      });
      delete chats.map[id];
    }

    return chats;
  }
  iservice.AlertsData = function () {
    if (iservice.data.alertsData) return iservice.data.alertsData;

    var alertsData = iservice.data.alertsData = Vue.reactive({ loaded: false, working: false, list: [] });
    function Clear() {
      alertsData.map = Vue.reactive({});
      alertsData.list.length = 0;
      alertsData.loaded = false;
      alertsData.working = false;
    }
    Clear();
    iservice.WatchLoginChange(() => {
      Clear();
    });

    alertsData.Load = function () {
      if (alertsData.working || alertsData.loaded) return;
      alertsData.working = true;
    }
    Vue.watchEffect(() => {
      if (alertsData.loaded || !alertsData.working) return;
      iservice.API('webapp-api-admin?api=alertall', null, {}).then(data => {
        data.alerts.forEach(alertsData.add);
      }).catch(errors => {

      }).finally(() => {
        alertsData.working = false;
        alertsData.loaded = true;
      });

    });
    alertsData.SortCompare = function (lid, rid) {
      var l = alertsData.get(lid), r = alertsData.get(rid);
      if (!l && !r) return 0; if (!r) return -1; if (!l) return 1;
      var lname = l.name, rname = r.name;
      if (!lname && !rname) return 0; if (!rname) return -1; if (!lname) return 1;
      return lname.localeCompare(rname);
    }

    alertsData.add = function (data) {
      var id = data.id;
      if (!id) return;
      var obj = alertsData.map[id];
      if (!obj) {
        obj = alertsData.map[id] = {};
        alertsData.list.push(obj);
        obj.id = id;
      }
      function copy(dest, src) { if (!src) src = dest; if (data[src] !== undefined) obj[dest] = data[src]; }
      ['name', 'enabled', 'frequencyMinutes', 'ageMinutes', 'topicIDs', 'action'].forEach(member => {
        copy(member);
      });
    }
    alertsData.get = function (id) {
      return alertsData.map[id];
    }

    var deleteCallbacks = [];
    alertsData.WhenDeleted = function (callback) {
      deleteCallbacks.push(callback);
      Vue.onUnmounted(function () {
        var index = deleteCallbacks.indexOf(callback);
        if (index == -1) return;
        deleteCallbacks.splice(index, 1);
      });
    }
    alertsData.deleted = function (id) {
      if (!departments.map[id]) return;
      alertsData.forEach(func => {
        func(id);
      });
      delete alertsData.map[id];
    }

    return alertsData;
  }
  //instantiate all modules, so watchers are not scoped to any single component
  iservice.Messages();
  iservice.LoggedInData();
  iservice.InteractionData();
  iservice.SmtpOutData();
  iservice.MyQueueData();
  iservice.SegmentData();
  iservice.IntPropertyData();
  iservice.ContactPropertyData();
  iservice.AutoResponseData();
  iservice.TopicData();
  iservice.TopicDeletedData();
  iservice.AgentData();
  iservice.StockResponsesData();
  iservice.WebsitesData();
  iservice.SkillData();
  iservice.MailboxData();
  iservice.Plugins();
  iservice.VariablePickerData();
  iservice.AutocompleteData();
  iservice.MyStockResponsesData();
  iservice.DepartmentsData();
  iservice.FilterData();
  iservice.ServiceLevelData();
  iservice.PropertyTypeData();
  iservice.MailboxErrosData();
  iservice.ListsData();
  iservice.activeChatsIdsStorage();
  iservice.chatUtilities();
  iservice.chatFiltersStorage();
  iservice.MailboxAdminData();
  iservice.MassCriteriaData();
  iservice.MassMessagesData();
  iservice.MasslistData();
  iservice.MassCampaignsData();
  iservice.MassMailingsData();
  iservice.UserTypesData();
  iservice.ArticleData();
  iservice.WebsitesData();
  iservice.QuotesData();
  iservice.PlaceholdersData();
  iservice.CustomerTopicData();
  iservice.AlertsData();
  iservice.FormsData();
  iservice.FormsBuiltinData();
})(iservice);