﻿var iservice = window.iservice || {};
(function (iservice) {
  iservice.components = iservice.components || {};

  iservice.components.dashboard = {
    name: 'vueapp-dashboard',
    template: '#vueapp-dashboard',
    defaultConfig: [
      { cells: [{ class: 'col-md-12', component: 'dashboard-recent-interactions' }] },
      { cells: [{ class: 'col-md-8', component: 'dashboard-my-answers-this-week' }, { class: 'col-md-4', component: 'dashboard-get-support' }] }
    ],
    route() {
      return { path: iservice.formPath, priority: 100, meta: { title: 'iService | My Dashboard' } } },
    setup() {
      var rows = Vue.reactive([]);
      var config = iservice.config?.dashboard;
      if (!config) config = iservice.components.dashboard.defaultConfig;
      config.forEach(row => { rows.push(row); });

      return { rows };
    }
  };

  iservice.components.dashboardRecentInteractions = {
    name: 'dashboard-recent-interactions',
    template: '#dashboard-recent-interactions',
    setup() {
      var loggedIn = iservice.LoggedInData();
      var interactions = iservice.InteractionData();

      var config = Vue.reactive({
        componentControls: 'iservice-table-controls-recent',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        componentExpanded: 'interaction-view-loader',
        filterLabel: 'Filter Interactions',
        emptyMessage: 'No Interactions',
        pageSize: 5,
        expandable: true,
        searching: false,
        numDays: iservice.GetLocalStoragePositiveInt('defaultPastdayValue'),
        initialSort: 'Date',
        exportName: "My Recent Interactions"
      });
      Vue.watchEffect(() => {
        config.columns = [
          Vue.reactive({
            name: 'expand', hidable: false,
            filterText: function () { },
            componentHead: 'iservice-table-head-empty', componentCell: 'iservice-table-cell-expand', shown: true, tdClass: 'expand-td'
          }),
          Vue.reactive({
            name: 'Interaction Type', label: 'Interaction Type',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'type'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'interaction-type-display', shown: true
          }),
          Vue.reactive({
            name: 'Subject', label: 'Subject',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'subject'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Contact Email', label: 'Contact Email',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'fromAddress'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-interaction-from-linked', shown: true
          }),
          Vue.reactive({
            name: 'Date', label: 'Date', sortIncreasing: false,
            filterText: iservice.tables.DateFilterTextFunc(interactions, 'date'), sortText: iservice.tables.DateSortTextFunc(interactions, 'date'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-interaction-date', shown: true
          })];
      })

      var numDays = Vue.ref(1);
      Vue.watchEffect(() => {
        var days = config.numDays;
        if (days && !isNaN(parseInt(days))) {
          days = Math.floor(Math.abs(parseInt(days)));
          if (days > 0) {
            numDays.value = config.numDays = days;
            localStorage.setItem('defaultPastdayValue', days);
          }
        }
      });

      var currentDays = Vue.ref(0);
      var IDs = Vue.reactive([]);
      var delayTimer;
      Vue.watchEffect(() => {
        if (config.searching) return;
        var now = moment.utc().toISOString();
        var past = moment.utc().subtract(numDays.value, 'd').toISOString();
        if (delayTimer) clearTimeout(delayTimer);
        delayTimer = setTimeout(function () {
          if (currentDays.value == numDays.value) return;
          config.searching = true;
          currentDays.value = numDays.value;
          var fields = {
            groups: [{ fields: [{ where: 'anywhere', fieldID: 'resolve', searchString: loggedIn.contactID }] }],
            timeEndUtc: now,
            timeStartUtc: past
          };
          iservice.API('webapp-api-mq-search?api=search', null, { fields, pageNum: 0, perPage: 1000 }).then(data => {
            IDs.length = 0;
            data.interactions.forEach(item => {
              var id = item.id;
              interactions.add(item);
              IDs.push(id);
            });
          }).finally(() => {
            config.searching = false;
          });
        }, 250);
      });

      interactions.WhenDeleted(id => {
        var index = IDs.indexOf(id);
        if (index == -1) return;
        IDs.splice(index, 1);
      });

      return { config, IDs };
    }
  };

  iservice.components.iserviceTableControlsRecent = {
    name: 'iservice-table-controls-recent',
    template: '#iservice-table-controls-recent',
    setup() {

      return {}
    }
  };

  iservice.components.dashboardMyAnswersThisWeek= {
    name: 'dashboard-my-answers-this-week',
    template: '#dashboard-my-answers-this-week',
    setup() {
      var loggedIn = iservice.LoggedInData();
      var interactions = Vue.reactive({});
      var formFeatures = Vue.reactive({});
      var v3Chart = Vue.ref(null);
      var now = new Date(), aWeekAgo = new Date();
      aWeekAgo.setDate(aWeekAgo.getDate() - 7);
      var defaultcount = [0, 0, 0, 0, 0, 0, 0];
      var dates = Last7Days().reverse();
      var config = { labels: dates, datasets: [] };
      
      function checkPrivateNote(msg) {
        return msg.type == 'Private Note';
      }
      function checkAgentResponse(msg) {
        return msg.type == 'Agent Response';
      }
      function checkAgentEmail(msg) {
        return msg.type == 'Agent Email';
      }
      function checkResolveNote(msg) {
        return msg.type == 'Resolve Note';
      }
      function checkPublicNote(msg) {
        return msg.type == 'Note';
      }
      function formatDate(date) {
        date = new Date(date);
        var dd = date.getDate();
        var mm = date.getMonth() + 1;
        var yyyy = date.getFullYear();
        if (dd < 10) { dd = '0' + dd }
        if (mm < 10) { mm = '0' + mm }
        date = mm + '/' + dd + '/' + yyyy;
        return date
      }
      function Last7Days() {
        var result = [];
        for (var i = 0; i < 7; i++) {
          var d = new Date();
          d.setDate(d.getDate() - i);
          result.push(formatDate(d))
        }

        return result;
      }
      function formatDateWithoutYear(date) {
        date = new Date(date);
        var dd = date.getDate();
        var mm = date.getMonth() + 1;
        var yyyy = date.getFullYear();
        if (dd < 10) { dd = '0' + dd }
        if (mm < 10) { mm = '0' + mm }
        date = mm + '/' + dd;
        return date
      }
      function Last7DayswithoutYear() {
        var result = [];
        for (var i = 0; i < 7; i++) {
          var d = new Date();
          d.setDate(d.getDate() - i);
          result.push(formatDateWithoutYear(d))
        }

        return result;
      }

      
      function init() {
        let param = {
          fields: {
            groups: [
              {
                fields: [{
                  where: 'anywhere',
                  fieldID: 'resolve',
                  searchString: loggedIn.contactID
                }]
              },
              {
                fields: [{
                  where: 'entire',
                  fieldID: 'not-findanswers',
                  searchString: ""
                }]
              },
              {
                fields: [
                  {
                    where: 'anywhere',
                    fieldID: 'type',
                    searchString: "Private Note"
                  },
                  {
                    where: 'anywhere',
                    fieldID: 'type',
                    searchString: "Note"
                  },
                  {
                    where: 'anywhere',
                    fieldID: 'type',
                    searchString: "Agent Response"
                  },
                  {
                    where: 'anywhere',
                    fieldID: 'type',
                    searchString: "Agent Email"
                  },
                  {
                    where: 'anywhere',
                    fieldID: 'type',
                    searchString: "Resolve Note"
                  }
                ]
              }
            ],
            timeStartUtc: moment.utc(aWeekAgo).format(),
            timeEndUtc: moment.utc(now).format()
          },
          pageNum: 0,
          perPage: 1000
        };
        formFeatures.loading = true;
        iservice.API('webapp-api-mq-search?api=search', null, param).then(data => {
          interactions.data = data.interactions;
        }).finally(() => {
          formFeatures.loading = false;
          formFeatures.loaded = true;
        });
      }
      init();
      
      Vue.watchEffect(() => {
        let data = interactions.data;
        if (data) {
          formFeatures.LastWeekdates = Last7DayswithoutYear().reverse();
          var privateNotes = data.filter(checkPrivateNote);
          var publicNotes = data.filter(checkPublicNote);
          var agentEmails = data.filter(checkAgentEmail);
          var resolved = data.filter(checkResolveNote);
          var agentresponses = data.filter(checkAgentResponse);

          var pNotes = [0, 0, 0, 0, 0, 0, 0];
          var publicNote = [0, 0, 0, 0, 0, 0, 0];
          var agentEmail = [0, 0, 0, 0, 0, 0, 0];
          var resolve = [0, 0, 0, 0, 0, 0, 0];
          var agentresponse = [0, 0, 0, 0, 0, 0, 0];
          if (privateNotes.length) {
            privateNotes.forEach(function (row) {
              dates.forEach(function (date, key) {
                pNotes[key] = formatDate(row.date) == date ? pNotes[key] + 1 : pNotes[key];
              });
            });
          }

          if (publicNotes.length) {
            publicNotes.forEach(function (row) {
              dates.forEach(function (date, key) {
                publicNote[key] = formatDate(row.date) == date ? publicNote[key] + 1 : publicNote[key];
              });
            });
          }

          if (agentEmails.length) {
            agentEmails.forEach(function (row) {
              dates.forEach(function (date, key) {
                agentEmail[key] = formatDate(row.date) == date ? agentEmail[key] + 1 : agentEmail[key];
              });
            });
          }
          if (resolved.length) {
            resolved.forEach(function (row) {
              dates.forEach(function (date, key) {
                resolve[key] = formatDate(row.date) == date ? resolve[key] + 1 : resolve[key];
              });
            });
          }
          if (agentresponses.length) {
            agentresponses.forEach(function (row) {
              dates.forEach(function (date, key) {
                agentresponse[key] = formatDate(row.date) == date ? agentresponse[key] + 1 : agentresponse[key];
              });
            });
          }
          formFeatures.privateNotes = pNotes.length ? pNotes : defaultcount;
          formFeatures.publicNote = publicNote.length ? publicNote : defaultcount;
          formFeatures.agentEmails = agentEmail.length ? agentEmail : defaultcount;
          formFeatures.resolved = resolve.length ? resolve : defaultcount;
          formFeatures.agentresponses = agentresponse.length ? agentresponse : defaultcount;

          config.datasets = [
            { label: "Private Note", data: pNotes.length ? pNotes : defaultcount, backgroundColor: "#2672EC", hoverBackgroundColor: "#2672EC", hoverBorderWidth: 0, },
            { label: "Public Note", data: publicNote.length ? publicNote : defaultcount, backgroundColor: "#4BACC6", hoverBackgroundColor: "#4BACC6", hoverBorderWidth: 0, },
            { label: "Agent Email", data: agentEmail.length ? agentEmail : defaultcount, backgroundColor: "#F79646", hoverBackgroundColor: "#F79646", hoverBorderWidth: 0, },
            { label: "Resolved", data: resolve.length ? resolve : defaultcount, backgroundColor: "#9BBB59", hoverBackgroundColor: "#9BBB59", hoverBorderWidth: 0, },
            { label: "Agent Response", data: agentresponse.length ? agentresponse : defaultcount, backgroundColor: "#C8A6F2", hoverBackgroundColor: "#C8A6F2", hoverBorderWidth: 0, },
          ];

          var zeroPrivateNotes = formFeatures.privateNotes.every(item => item === 0);
          var zeroprivateNotes = formFeatures.publicNote.every(item => item === 0);
          var zeroagentEmails = formFeatures.agentEmails.every(item => item === 0);
          var zeroresolved = formFeatures.resolved.every(item => item === 0);
          var zeroagentresponses = formFeatures.agentresponses.every(item => item === 0);

          if (zeroPrivateNotes && zeroprivateNotes && zeroagentEmails && zeroresolved && zeroagentresponses) {
            config.datasets = [];
          }
          formFeatures.barChart = {
            type: "bar",
            options: {
              min: 0,
              max: 100,
              responsive: true,
              plugins: {
                legend: {
                  position: "top",
                  labels: { boxWidth: 13 }
                }
              },
              scales: {
                x: {
                  stacked: true,
                  grid: {
                    display: false
                  }
                },
                y: {
                  stacked: true,
                  ticks: {
                    beginAtZero: true,
                    callback: function (value, index, values) {
                      if (Math.floor(value) === value) {
                        return value;
                      }
                    }
                  }
                }
              },
            },
            data: {
              labels: config.labels,
              datasets: config.datasets,
            },
          };
        }
      });
      Vue.watchEffect(() => {
        var chart = v3Chart.value;
        if (!chart) return;
        if (chart.data.datasets.length === 0) {
          // No data is present
          var ctx = chart.chartJSState.chart.ctx;
          var width = chart.chartJSState.chart.width;
          var height = chart.chartJSState.chart.height
          chart.chartJSState.chart.clear();

          ctx.save();
          ctx.textAlign = 'center';
          ctx.textBaseline = 'middle';
          ctx.font = "16px normal 'Helvetica Nueue'";
          ctx.fillText('There is no data to display for this period.', width / 2, height / 2);
          ctx.restore();
        }
      });
      return { formFeatures, v3Chart };
    }
  };

  iservice.components.dashboardGetSupport = {
    name: 'dashboard-get-support',
    template: '#dashboard-get-support',
    setup() {

      return {};
    }
  };

})(iservice);