﻿var iservice = window.iservice || {};
(function (iservice) {
  iservice.components = iservice.components || {};

  iservice.components.adminFormsPage = {
    name: 'vueapp-admin-forms',
    template: '#vueapp-admin-forms',
    route() { return { path: iservice.formPath + '/admin/form/:id?', priority: 300, meta: { title: "iService | Forms" }, name: "form" } },
    props: [],
    setup(props) {

      //router object declartion
      var router = VueRouter.useRouter();

      //form data and features object decralation
      var formFeatures = Vue.reactive({ errors: [], success: '', ChangeState: Function, ShowNew: false, ShowDelete: false, ShowDetails: false, ShowDeleteButton: false, Create: Function, Delete: Function, CancelDelete: Function, Save: Function, shownToggleRunning: false, createnew: false, deleteStart: false, segmentOptions: [] });

      formFeatures.details = {}
      formFeatures.wizard = {};

      formFeatures.details.effects = {}

      var selectedID = Vue.ref(null);
      function Select(id) {
        selectedID.value = id;
        formFeatures.ShowNew = false;
        formFeatures.ShowWizard = false;
        router.push({ params: { id } }); //update route paramater on selecting table row 
      }
      var selectedIDs = Vue.computed(() => {
        if (selectedID.value) return [selectedID.value];
        return [];
      });

      formFeatures.GroupButtonClass = function (group) {
        return router.currentRoute.value.name == group ? 'active btn-primary' : 'btn-default';
      }
      formFeatures.ChangeGroup = function (group) {
        return router.push({ name: group });
      }


      //form state change on different actions
      formFeatures.ChangeState = function (state = 'new', details = '') {
        formFeatures.ShowNew = state == 'new' ? true : false;
        formFeatures.ShowWizard = state == 'wizard' ? true : false;
        formFeatures.ShowDelete = state == 'delete' ? true : false;

        formFeatures.ShowDetails = state == 'details' ? true : false;
        if (state == 'details') {
          if (details) {
            var id = details.id;
            selectedID.value = id;
            router.push({ params: { id: id } });
          }
        }
        if (state == 'new') {
          formFeatures.ShowDeleteButton = false;
          formFeatures.details = { actions: [] }
        }
        if (state == 'wizard') {
          formFeatures.ShowDeleteButton = false;
          formFeatures.wizard = {}
          formFeatures.wizard.details = {};
          formFeatures.wizard.step = 1;
        }
        if (state == 'cancel') {
          formFeatures.ShowDeleteButton = false;
        }
      }
      formFeatures.previewUrl = function (formid) {
        return rootPath + 'f/' + formid;
      };
      //page access have right 
      formFeatures.haveRight = iservice.HaveRight('Tab.Admin.Forms');


      //load all forms
      var forms = iservice.FormsData();
      if (!forms.loaded) {
        forms.Load();
      }


      //table list collapse toggle
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }
      //table columns declaration
      var tableColumns = Vue.computed(() => {
        var columns = [
          Vue.reactive({
            name: 'ID', label: 'ID',
            type: 'number',
            filterText: iservice.tables.SimpleFilterTextFunc(forms, 'id'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Name', label: 'Name',
            filterText: iservice.tables.SimpleFilterTextFunc(forms, 'name'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Description', label: 'Description',
            filterText: iservice.tables.SimpleFilterTextFunc(forms, 'description'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Modifier', label: 'Modifier',
            filterText: iservice.tables.SimpleFilterTextFunc(forms, 'modifierName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Modify Time', label: 'Modify Time',
            filterText: iservice.tables.DateFilterTextFunc(forms, 'modifyTime'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          })];

        return columns;
      });

      //table configuration
      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        filterLabel: 'Filter Forms',
        emptyMessage: 'No forms',
        pageSize: 5,
        expandable: true,
        searching: false,
        initialSort: 'Name',
        columns: tableColumns.value,
        exportName: "Admin Forms"
      });

      formFeatures.componentProps = {
        forms: forms,
        ChangeState: formFeatures.ChangeState
      }
      //table data ids array
      formFeatures.IDs = [];
      var itemsProcessed = 0;
      tableConfig.searching = true;

      //load table records by pushing ids 
      Vue.watchEffect(function () {
        if (formFeatures.createnew) return;
        if (!forms.list) return;
        if (formFeatures.deleteStart) return;
        formFeatures.IDs = []
        forms.list.forEach(function (form) {
          var id = form.id;
          forms.add(form);
          formFeatures.IDs.push(id);
          itemsProcessed++;
          if (itemsProcessed == forms.list.length) {
            tableConfig.searching = false;
          }
        });
        formFeatures.componentProps.IDs = formFeatures.IDs;
      });

      Vue.watchEffect(function () {
        if (formFeatures.ShowNew || formFeatures.ShowWizard) {
          var id = '';
          selectedID.value = '';
          router.push({ params: { id: id } });
        }
      })

      //update record
      formFeatures.Save = function () {
        formFeatures.errors = [];
        formFeatures.formatPayloadAction();
        iservice.API('webapp-api-admin?api=forms&mode=modify', null, { formID: selectedID.value, details: formFeatures.details }).then(function (result) {
          forms.add(result.row);
          formFeatures.formatDisplayAction(result.details.actions);
          iservice.SuccessMessage('Form updated successfully.');

        }).catch(function (errors) {
          formFeatures.formatDisplayAction(formFeatures.details.actions);
          formFeatures.errors = errors;
        });
      }
      //check route params change effect and load details
      Vue.watchEffect(() => {
        if (formFeatures.ShowNew || formFeatures.ShowWizard) return;
        if (router.currentRoute.value.params.id) {
          formFeatures.ShowDeleteButton = true;
          formFeatures.loading = true;
          iservice.API('webapp-api-admin?api=formdetails', null, { formID: router.currentRoute.value.params.id }).then(function (result) {
            selectedID.value = router.currentRoute.value.params.id;
            formFeatures.details = result.details;
            formFeatures.formatDisplayAction(result.details.actions);
            formFeatures.ChangeState('details');
            formFeatures.loading = false;
          }).then(function (result) {

          }).catch(function (errors) {

          }).finally(function () {

          });
        } else {
          selectedID.value = '';
          formFeatures.ShowDeleteButton = false;
          formFeatures.ShowNew = false;
          formFeatures.ShowWizard = false;
          formFeatures.ShowDetails = false;
        }
      });
      formFeatures.error = {};

      //create new record
      formFeatures.Create = function () {
        formFeatures.errors = [];
        formFeatures.deleteStart = false;
        formFeatures.createnew = true;
        if (!formFeatures.details.name) {
          formFeatures.errors.push({ msg: 'Please enter name.', details: { description: '' } });
          formFeatures.error.name = true;
        }

        if (formFeatures.errors.length) return;
        
        formFeatures.details.actions.forEach(function (action) {
          var steps = {};
          action.submitAction.steps.forEach(function (step, key) {
            step.stepType = step.stepType.replace('actionstep-', '');
            steps[step.stepType] = {};
            for (var key in step) {
              if (step.hasOwnProperty(key)) {
                if (key !== 'stepType')
                  steps[step.stepType][key] = step[key];
              }
            }
          });
          action.submitAction.steps.forEach(function (stp) {
            for (const property in steps) {
              if (stp.stepType == property) {
                stp[property] = steps[property];
              }
            }
          });
        });
        iservice.API('webapp-api-admin?api=forms&mode=new', null, { details: formFeatures.details }).then(function (result) {
          //add new row to the list.
          forms.add(result.row);
          formFeatures.IDs.push(result.row.id);
          formFeatures.ChangeState('details', result.row);
          iservice.SuccessMessage('Form added successfully.');
          formFeatures.formatDisplayAction(result.details.actions);
        }).catch(function (errors) {
          formFeatures.errors = errors;
        });
      }

      //delete record
      formFeatures.Delete = function () {
        formFeatures.deleteStart = true;
        iservice.API('webapp-api-admin?api=forms&mode=delete', null, { formID: selectedID.value }).then(function (result) {

          formFeatures.ChangeState('cancel');
          formFeatures.ShowDeleteButton = false;
          const index = formFeatures.IDs.indexOf(selectedID.value);
          if (index > -1) {
            formFeatures.IDs.splice(index, 1);
          }
          selectedID.value = '';
          var id = '';
          router.push({ params: { id: id } })
          iservice.SuccessMessage('Form deleted successfully.');
        }).catch(function (errors) {
          formFeatures.errors = errors;
        }).finally(function () {

        });
      }

      var editor = Vue.ref(null);
      var anchorStocks = Vue.ref(null);
      var featureFormVariables = Vue.reactive({ filter: '' });
      featureFormVariables.Open = function () {
        featureFormVariables.Close = iservice.components.popupHolder.Open(anchorStocks, 'admin-form-variables', false, featureFormVariables);
      }
      featureFormVariables.PasteVar = function (text) {
        if (!editor || !editor.value) return;
        if (featureFormVariables.Close) featureFormVariables.Close();
        iservice.insertAtCaret('bodyarea', text);
      }
      formFeatures.AddAction = function (actions) {
        actions.push({
          checkInputID: null,
          checkValue: null,
          submitAction: {
            steps: []
          }
        });
      }
      formFeatures.ActionDeleteStart = function (action) {
        action.deleting = true;
      }
      formFeatures.ActionDeleteConfirm = function (actions, action) {
        for (var i = 0; i < actions.length; i++) {
          if (action !== actions[i]) continue;
          actions.splice(i, 1);
          return;
        }
      }
      formFeatures.ActionDeleteCancel = function (action) {
        action.deleting = false;
      }
      formFeatures.AddStep = function (steps) {
        steps.push({});
      };
      formFeatures.StepMoveUp = function (actions, step) {
        let stepType = step.stepType.replace("actionstep-", "");
        let index = formFeatures.steps.findIndex(x => x.hasOwnProperty(stepType) && x[stepType] !== null);
        if (index !== -1) {
          let obj = formFeatures.steps[index];
          formFeatures.steps.splice(index, 1);
          formFeatures.steps.splice(index - 1, 0, obj);
        }
        for (var i = 1; i < actions.length; i++) {
          if (step !== actions[i]) continue;
          actions.splice(i, 1);
          actions.splice(i - 1, 0, step);
          return;
        }
        
      };
      formFeatures.StepMoveDown = function (actions, step) {
        let stepType = step.stepType.replace("actionstep-", "");
        let index = formFeatures.steps.findIndex(x => x.hasOwnProperty(stepType) && x[stepType] !== null);
        if (index !== -1) {
          let obj = formFeatures.steps[index];
          formFeatures.steps.splice(index, 1);
          if (index === formFeatures.steps.length - 1) formFeatures.steps.push(obj);
          else formFeatures.steps.splice(index + 1, 0, obj);
        }
        for (var i = 0; i < actions.length - 1; i++) {
          if (step !== actions[i]) continue;
          actions.splice(i, 1);
          if (i === actions.length - 1) actions.push(step);
          else actions.splice(i + 1, 0, step);
          return;
        }
      };
      formFeatures.StepDelete = function (actions, step) {
        let stepType = step.stepType.replace("actionstep-", "");
        let index = formFeatures.steps.findIndex(x => x.hasOwnProperty(stepType) && x[stepType] !== null);
        if (index !== -1) formFeatures.steps.splice(index, 1);
        for (var i = 0; i < actions.length; i++) {
          if (step !== actions[i]) continue;
          actions.splice(i, 1);
          return;
        }
      };
      formFeatures.stepsOptions = iservice.FormsstepsOptions;

      formFeatures.getStepForm = function (step) {
        step.stepType = 'actionstep-' + step.stepType;
      }
      formFeatures.WizStepClass = function (num) {
        if (num == formFeatures.wizard.step) return 'btn-primary';
        return 'btn-default';
      }
      formFeatures.WizardBodyUrl = function () {
        if (formFeatures.wizard.step == 1) return 'admin-form-wizard-body-type';
        if (formFeatures.wizard.step == 2 && formFeatures.wizard.type == 'contact') return 'admin-form-wizard-body-contact';
        if (formFeatures.wizard.step == 2 && formFeatures.wizard.type == 'ticket') return 'admin-form-wizard-body-ticket-topic';
        if (formFeatures.wizard.step == 3) return 'admin-form-wizard-body-ticket-contactproperties';
        if (formFeatures.wizard.step == 4) return 'admin-form-wizard-body-ticket-intproperties';
        return '';
      }
      formFeatures.formatDisplayAction = function (actions) {
        formFeatures.steps = [];
        actions.forEach(function (action) {
          action.submitAction.steps.forEach(function (step) {
            formFeatures.steps.push(step);
          })
        })
        formFeatures.details.actions.forEach(function (action) {
          var steps = [];
          action.submitAction.steps.forEach(function (step) {
            for (let key in step) {
              if (step[key] !== null) {
                var obj = {
                  stepType: 'actionstep-' + key
                }
                for (let key2 in step[key]) {
                  obj[key2] = step[key][key2];
                }
                steps.push(obj);
              }
            }
          });
          action.submitAction.steps = steps;
        });
      }
      formFeatures.formatPayloadAction = function () {
        formFeatures.details.actions.forEach(function (action) {
          action.submitAction.steps.forEach(function (step, key) {
            var steps = {};
            var stepExists = false;
            step.stepType = step.stepType.replace('actionstep-', '');
            
            formFeatures.steps.forEach(function (singlestep, key) {
              if (singlestep[step.stepType] && singlestep[step.stypetype] !== null) {
                stepExists = true
                for (var key in step) {
                  if (step.hasOwnProperty(key)) {
                    if (key !== 'stepType')
                      singlestep[step.stepType][key] = step[key];
                  }
                }
              }
            })
            if (!stepExists) {
              steps[step.stepType] = {};
              for (var key in step) {
                if (step.hasOwnProperty(key)) {
                  if (key !== 'stepType')
                    steps[step.stepType][key] = step[key];
                }
              }
              let obj = formFeatures.steps.length > 0 ? Object.assign({}, formFeatures.steps[0]) : {};
              for (key in obj) {
                obj[key] = null;
              }
              obj[step.stepType] = steps[step.stepType];
              formFeatures.steps.push(obj);
            }
            action.submitAction.steps = [];
            formFeatures.steps.forEach(function (step) {
              formFeatures.details.actions.forEach(function (action) {
                action.submitAction.steps.push(step);
              });
            });
          });
        });

        formFeatures.details.actions.forEach(function (action) {
          var newArray = action.submitAction.steps.filter(element => {
            if (Object.keys(element).length !== 0) {
              return true;
            }
            return false;
          });
          action.submitAction.steps = newArray;
        });
      }
      return { formFeatures, tableConfig, Select, selectedIDs, selectedID, ListCollapsed, featureFormVariables, editor, anchorStocks };
    }
  };

  iservice.components.adminFormBuiltIn = {
    name: 'vueapp-forms-built-in',
    template: '#vueapp-admin-forms-built-in',
    route() { return { path: iservice.formPath + '/admin/formbuilin/:id?', priority: 300, meta: { title: "iService | Built-in Forms" }, name: "formbuiltin" } },
    props: [],
    setup(props) {
      //router object declartion
      var router = VueRouter.useRouter();

      //form data and features object decralation
      var formFeatures = Vue.reactive({ errors: [], success: '', ChangeState: Function, ShowNew: false, ShowDelete: false, ShowDetails: false, ShowDeleteButton: false, Create: Function, Delete: Function, CancelDelete: Function, Save: Function, shownToggleRunning: false, createnew: false, deleteStart: false, segmentOptions: [] });

      formFeatures.details = {}

      formFeatures.previewUrl = function (formid) {
        return rootPath + 'f/' + formid;
      };

      formFeatures.details.effects = {}
      var selectedID = Vue.ref(null);
      //table row select callback
      function Select(id) {
        selectedID.value = id;
        formFeatures.ShowNew = false;
        router.push({ params: { id: id } }); //update route paramater on selecting table row 
      }
      var selectedIDs = Vue.computed(() => {
        if (selectedID.value) return [selectedID.value];
        return [];
      });

      formFeatures.GroupButtonClass = function (group) {
        return router.currentRoute.value.name == group ? 'active btn-primary' : 'btn-default';
      }
      formFeatures.ChangeGroup = function (group) {
        return router.push({ name: group });
      }

      //form state change on different actions
      formFeatures.ChangeState = function (state = 'new', details = '') {
        formFeatures.ShowNew = state == 'new' ? true : false;
        formFeatures.ShowDelete = state == 'delete' ? true : false;

        formFeatures.ShowDetails = state == 'details' ? true : false;
        if (state == 'details') {
          if (details) {
            var id = details.id;
            selectedID.value = id;
            router.push({ params: { id: id } });
          }
        }
        if (state == 'new') {
          formFeatures.details = {}
        }
        if (state == 'cancel') {
          formFeatures.ShowDeleteButton = false;
        }
      }
      //page access have right 
      formFeatures.haveRight = iservice.HaveRight('Tab.Admin.Forms');

      //load all forms
      var forms = iservice.FormsBuiltinData();
      if (!forms.loaded) {
        forms.Load();
      }
      //table list collapse toggle
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }
      //table columns declaration
      var tableColumns = Vue.computed(() => {
        var columns = [
          Vue.reactive({
            name: 'Name', label: 'Form Name',
            filterText: iservice.tables.SimpleFilterTextFunc(forms, 'name'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Description', label: 'Description',
            filterText: iservice.tables.SimpleFilterTextFunc(forms, 'description'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          })];

        return columns;
      });

      //table configuration
      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        filterLabel: 'Filter Forms',
        emptyMessage: 'No forms',
        pageSize: 5,
        expandable: true,
        searching: false,
        initialSort: 'Name',
        columns: tableColumns.value,
        exportName: "Admin Forms"
      });

      //table data ids array
      formFeatures.IDs = [];
      var itemsProcessed = 0;
      tableConfig.searching = true;

      //load table records by pushing ids 
      Vue.watchEffect(function () {
        if (formFeatures.createnew) return;
        if (!forms.list) return;
        if (formFeatures.deleteStart) return;
        formFeatures.IDs = []
        forms.list.forEach(function (form) {
          var id = form.id;
          forms.add(form);
          formFeatures.IDs.push(id);
          itemsProcessed++;
          if (itemsProcessed == forms.list.length) {
            tableConfig.searching = false;
          }
        });
      });

      //check route params change effect and load details
      Vue.watchEffect(() => {
        if (formFeatures.ShowNew) return;
        if (router.currentRoute.value.params.id) {
          formFeatures.ShowDeleteButton = true;
          formFeatures.loading = true;
          iservice.API('webapp-api-admin?api=formdetails', null, { formID: router.currentRoute.value.params.id }).then(function (result) {
            selectedID.value = router.currentRoute.value.params.id;
            formFeatures.details = result.details;
            formFeatures.ChangeState('details');
            formFeatures.loading = false;
          }).then(function (result) {

          }).catch(function (errors) {

          }).finally(function () {

          });
        } else {
          selectedID.value = '';
          formFeatures.ChangeState('cancel');
          formFeatures.ShowDeleteButton = false;
          formFeatures.ShowNew = false;
          formFeatures.ShowDetails = false;
        }
      });
      var editor = Vue.ref(null);
      var anchorStocks = Vue.ref(null);
      var featureFormVariables = Vue.reactive({ filter: '' });
      featureFormVariables.Open = function () {
        featureFormVariables.Close = iservice.components.popupHolder.Open(anchorStocks, 'admin-form-variables', false, featureFormVariables);
      }
      featureFormVariables.PasteVar = function (text) {
        if (!editor || !editor.value) return;
        if (featureFormVariables.Close) featureFormVariables.Close();
        iservice.insertAtCaret('bodyarea', text);
      }
      return { formFeatures, tableConfig, Select, selectedIDs, selectedID, ListCollapsed, editor, anchorStocks, featureFormVariables };
    }
  }

  iservice.components.adminFormWizardBodyType = {
    name: 'admin-form-wizard-body-type',
    template: '#admin-form-wizard-body-type',
    props: ['wizard', 'properties'],
    setup(props) {
      var feature = Vue.reactive({});
      var properties = Vue.computed(function () {
        return props.properties;
      })
      var wizard = Vue.computed(function () {
        return props.wizard;
      });
      feature.TypeNext = function () {
        if (!wizard.value.type) return;
        if (!wizard.value.details.name) return;
        wizard.value.step = 2;
      }
      return { wizard, feature, properties };
    }
  }
  iservice.components.adminFormWizardBodyContact = {
    name: 'admin-form-wizard-body-contact',
    template: '#admin-form-wizard-body-contact',
    props: ['wizard', 'properties'],
    setup(props) {
      var contactProperties = iservice.ContactPropertyData();
      if (!contactProperties.loaded) {
        contactProperties.Load();
      }
      var properties = Vue.computed(function () {
        return props.properties;
      });
      var feature = Vue.reactive({ errors: [] });
      var wizard = Vue.computed(function () {
        return props.wizard;
      });
      wizard.value.contactProperties = Vue.computed(function () {
        var contactProps = [];
        feature.proploading = true;
        contactProps.push({ name: 'Email Address', selected: true, required: true, optional: false, describe: false, input: "$input -email -id'email'{req}$", inputConfirm: "$input -email -id'emailconfirm' -confirm'email'$" });
        contactProps.push({ name: 'Password', optional: true, describe: false, input: "$input -password -id'password'{req}$", inputConfirm: "$input -password -id'passwordconfirm' -confirm'password'$" });
        contactProperties.list.forEach(function (prop) {
          var row = iservice.MakePropertyRow(prop, function (prop, id) { return ' -contactProperty' + prop.id + "'" + id + "'"; });
          contactProps.push(row);
        });
        feature.proploading = false;
        return contactProps;
      });
      feature.Back = function () {
        wizard.value.step = wizard.value.step - 1;
      }
      function WritePropertyHtml(prop) {
        var body = '      ' + prop.name + ': ';
        if (prop.describe) body += prop.inputDescribe;
        if (prop.multiline) body += '<br>\r\n      ';
        body += prop.input.replace('{req}', (prop.required ? ' -required' : '')) + '<br>\r\n';
        if (!prop.unconfirm && prop.confirm) {
          body += '      Confirm ' + prop.name + ': ';
          if (prop.describe) body += prop.inputDescribeConfirm;
          if (prop.multiline) body += '<br>\r\n      ';
          body += prop.inputConfirm + '<br>\r\n';
        } return body;
      }
      feature.ContactFinish = function () {
        var body = '<html>\r\n  <body>\r\n    <form method="POST">\r\n';
        wizard.value.contactProperties.forEach(function (prop) {
          if (!prop.selected) return;
          body += WritePropertyHtml(prop);
        });
        body += '      <input type="submit" id="ok" name="ok" value="Create/Update Contact" /><br>\r\n';
        body += '      <div style="background-color:yellow;">$errormessage$</div>\r\n';
        body += '    </form>\r\n  </body>\r\n</html>';
        wizard.value.details.body = body;
        wizard.value.details.actions = [{ submitAction: { steps: [{ contactFindCreate: { emailSearch: "$form -id'email'$", ignoreBlankFields: true, resultName: 'contactID' } }] } }];
        iservice.API('webapp-api-admin?api=forms&mode=new', null, { details: wizard.value.details }).then(function (result) {
          //add new row to the list.
          properties.value.forms.add(result.row);
          properties.value.IDs.push(result.row.id);
          properties.value.ChangeState('details', result.row);
          iservice.SuccessMessage('Form added successfully.');
        }).catch(function (errors) {
          feature.errors = errors;
        });
      }
      return { wizard, feature, properties };
    }
  }
  iservice.components.adminFormWizardBodyTicketTopic = {
    name: 'admin-form-wizard-body-ticket-topic',
    template: '#admin-form-wizard-body-ticket-topic',
    props: ['wizard', 'properties'],
    setup(props) {
      var topics = iservice.TopicData();
      var segments = iservice.SegmentData();
      var feature = Vue.reactive({});
      var properties = Vue.computed(function () {
        return props.properties;
      });
      var wizard = Vue.computed(function () {
        return props.wizard;
      });
      feature.Back = function () {
        wizard.value.step = wizard.value.step - 1;
      }
      var topicGroups = Vue.computed(() => {
        var groups = [];
        if (!segments.loaded || !topics.loaded) {
          segments.Load(); topics.Load();
        }
        if (!topics.list.length) feature.topicLoading = true; else feature.topicLoading = false;
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            group.options.push({ name: topic.name, id: topic.id, level: Math.min(2, topic.depth) });
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.value, r.value); });
          if (group.options.length < 1) return;
          groups.push(group);
        });
        return groups;
      });
      feature.TicketTopicNext = function () {
        if (!wizard.value.topicID) return;
        wizard.value.step = 3;
      }
      return { wizard, feature, topicGroups, properties };
    }
  }
  iservice.components.adminFormWizardBodyTicketContactProps = {
    name: 'admin-form-wizard-body-ticket-contactproperties',
    template: '#admin-form-wizard-body-ticket-contactproperties',
    props: ['wizard', 'properties'],
    setup(props) {
      var contactProperties = iservice.ContactPropertyData();
      if (!contactProperties.loaded) {
        contactProperties.Load();
      }
      var feature = Vue.reactive({});
      var properties = Vue.computed(function () {
        return props.properties;
      });
      var wizard = Vue.computed(function () {
        return props.wizard;
      });
      wizard.value.contactProperties = Vue.computed(function () {
        if (!contactProperties.loaded) {
          contactProperties.Load();
        }
        var contactProps = [];
        feature.proploading = true;
        contactProps.push({ name: 'Email Address', selected: true, required: true, optional: false, describe: false, input: "$input -email -id'email'{req}$", inputConfirm: "$input -email -id'emailconfirm' -confirm'email'$" });
        contactProps.push({ name: 'Password', optional: true, describe: false, input: "$input -password -id'password'{req}$", inputConfirm: "$input -password -id'passwordconfirm' -confirm'password'$" });
        contactProperties.list.forEach(function (prop) {
          var row = iservice.MakePropertyRow(prop, function (prop, id) { return ' -contactProperty' + prop.id + "'" + id + "'"; });

          contactProps.push(row);
        });
        feature.proploading = false;
        return contactProps;
      });
      feature.Back = function () {
        wizard.value.step = wizard.value.step - 1;
      }
      feature.TicketContactNext = function () {
        wizard.value.step = 4;
      }
      return { wizard, feature, properties };
    }
  }
  iservice.components.adminFormWizardBodyTicketIntProps = {
    name: 'admin-form-wizard-body-ticket-intproperties',
    template: '#admin-form-wizard-body-ticket-intproperties',
    props: ['wizard', 'properties'],
    setup(props) {
      var intProperties = iservice.IntPropertyData();
      if (!intProperties.loaded) {
        intProperties.Load();
      }
      var properties = Vue.computed(function () {
        return props.properties;
      });
      var feature = Vue.reactive({ errors: [] });
      var wizard = Vue.computed(function () {
        return props.wizard;
      });
      feature.Back = function () {
        wizard.value.step = wizard.value.step - 1;
      }

      wizard.value.intProperties = Vue.computed(function () {
        var intProps = [];
        feature.proploading = true;
        intProps.push({ name: 'Subject', selected: true, required: true, optional: false, describe: false, unconfirm: true, input: "$input -id'subject'{req}$" });
        intProps.push({ name: 'Body', selected: true, required: true, optional: false, describe: false, unconfirm: true, input: "$input -id'body'{req} -textarea$" });
        intProperties.list.forEach(function (prop) {
          if (prop.purpose != 'Answer') {
            var row = iservice.MakePropertyRow(prop, function (prop, id) { return ' -interactionProperty' + prop.id + "'" + id + "'"; });
            intProps.push(row);
          }
        });
        feature.proploading = false;
        return intProps;
      });
      function WritePropertyHtml(prop) {
        var body = '      ' + prop.name + ': ';
        if (prop.describe) body += prop.inputDescribe;
        if (prop.multiline) body += '<br>\r\n      ';
        body += prop.input.replace('{req}', (prop.required ? ' -required' : '')) + '<br>\r\n';
        if (!prop.unconfirm && prop.confirm) {
          body += '      Confirm ' + prop.name + ': ';
          if (prop.describe) body += prop.inputDescribeConfirm;
          if (prop.multiline) body += '<br>\r\n      ';
          body += prop.inputConfirm + '<br>\r\n';
        } return body;
      }
      feature.TicketFinish = function () {
        var body = '<html>\r\n  <body>\r\n    <form method="POST">\r\n';
        wizard.value.contactProperties.forEach(function (prop) {
          if (!prop.selected) return;
          body += WritePropertyHtml(prop);
        });
        wizard.value.intProperties.forEach(function (prop) {
          if (!prop.selected) return;
          body += WritePropertyHtml(prop);
        });
        body += '      <input type="submit" id="ok" name="ok" value="Submit Question" /><br>\r\n';
        body += '      <div style="background-color:yellow;">$errormessage$</div>\r\n';
        body += '    </form>\r\n  </body>\r\n</html>';
        wizard.value.details.body = body;
        wizard.value.details.actions = [{
          submitAction: {
            steps:
              [{ contactFindCreate: { emailSearch: "$form -id'email'$", ignoreBlankFields: true, resultName: 'contactID' } },
              { ticket: { isTicketNotAskAQuestion: false, contactID: "$result -id'contactID'$", emailAddress: "$form -id'email'$", topicID: '' + wizard.value.topicID, subject: "$form -id'subject'$", bodyHtml: "$form -id'body'$", resultName: 'interactionID' } }]
          }
        }];
        iservice.API('webapp-api-admin?api=forms&mode=new', null, { details: wizard.value.details }).then(function (result) {
          //add new row to the list.
          properties.value.forms.add(result.row);
          properties.value.IDs.push(result.row.id);
          properties.value.ChangeState('details', result.row);
          iservice.SuccessMessage('Form added successfully.');
        }).catch(function (errors) {
          feature.errors = errors;
        });
      }
      return { wizard, feature, properties };
    }
  }
  iservice.components.AdminFormVariables = {
    name: 'admin-form-variables',
    template: '#admin-form-variables',
    props: ['params'],
    setup(props) {
      var intProperties = iservice.IntPropertyData();
      var contactProperties = iservice.ContactPropertyData();
      if (!intProperties.loaded || !contactProperties.loaded) {
        intProperties.Load(); contactProperties.Load();
      }
      var feature = Vue.computed(() => { return props.params; });

      var segmentID = 0;
      var contactPropsGroup = Vue.computed(function () {
        if (!contactProperties.loaded) {
          contactProperties.Load();
        }
        var groupContact = {
          name: 'Contact',
          items: [
            {
              text: 'Email Address', v: "$input -email -id'enteruniquenamehere'$",
              subs: [
                { text: 'Confirm', v: "$input -email -id'enteruniquenamehere' -confirm'uniquenameofwhatthisconfirms'$" },
                { text: 'Self List', v: "$input -email(self) -id'enteruniquenamehere'$" }]
            },
            {
              text: 'Password', v: "$input -password -id'enteruniquenamehere'$",
              subs: [
                { text: 'Confirm', v: "$input -password -id'enteruniquenamehere' -confirm'uniquenameofwhatthisconfirms'$" }]
            }]
        };
        contactProperties.list.forEach(function (prop) {
          var item = { text: prop.name, v: "$input -id'enteruniquenamehere' -contactProperty" + prop.id + "'" + prop.name.replace(/\W/g, '') + "'$" };
          if (segmentID != prop.segmentID) {
            item.segmentName = prop.segmentName;
            segmentID = prop.segmentID;
          }
          if (prop.hasDescriptions) item.subs = [
            { text: 'Desc', v: "$input -contactProperty" + prop.id + "'" + prop.name.replace(/\W/g, '') + "' -id'enteruniquenamehere' -description'uniquenameofwhatthisdescribes'$" }];
          groupContact.items.push(item);
        });
        return groupContact;
      });
      segmentID = 0;

      var intPropsGroup = Vue.computed(function () {
        if (!intProperties.loaded) {
          intProperties.Load();
        }
        var groupInt = {
          name: 'Interaction',
          items: []
        };
        intProperties.list.forEach(function (prop) {
          var item = {
            text: prop.name, v: "$input -id'enteruniquenamehere' -interactionProperty" + prop.id + "'" + prop.name.replace(/\W/g, '') + "'$", purpose: prop.purpose
          };
          if (segmentID != prop.segmentID) {
            item.segmentName = prop.segmentName;
            segmentID = prop.segmentID;
          }
          if (prop.hasDescriptions) item.subs = [
            { text: 'Desc', v: "$input -interactionProperty" + prop.id + "'" + prop.name.replace(/\W/g, '') + "' -id'enteruniquenamehere' -description'uniquenameofwhatthisdescribes'$" }];
          groupInt.items.push(item);
        });
        return groupInt;
      });

      var variableGroups = Vue.computed(function () {
        return [
          {
            name: 'Basic',
            items: [
              { text: 'Any Field', v: "$input -id'enteruniquenamehere'$" },
              { text: 'Any Text Area', v: "$input -textarea -id'enteruniquenamehere'$" },
              { text: 'Form Login', v: "$loginname -id'enterniquenamehere'$" },
              { text: 'Form Password', v: "$loginpassword -id'enterniquenamehere'$" },
              { text: 'Error Message', v: "$errormessage$" },
              { text: 'Form ID', v: "$value -formID$" }
            ]
          },
          contactPropsGroup.value,
          intPropsGroup.value,
          {
            name: 'Conditionals',
            items: [
              {
                text: 'Logged In', v: "$if -loggedin$\r\n$endif$\r\n",
                subs: [
                  { text: 'With Else', v: "$if -loggedin$\r\n$else$\r\n$endif$\r\n" }]
              },
              {
                text: 'Post', v: "$if -ispostback$\r\n$endif$\r\n",
                subs: [
                  { text: 'With Else', v: "$if -ispostback$\r\n$else$\r\n$endif$\r\n" }]
              },
              {
                text: 'Submit Success', v: "$if -submitsuccess$\r\n$endif$\r\n",
                subs: [
                  { text: 'With Else', v: "$if -submitsuccess$\r\n$else$\r\n$endif$\r\n" }]
              }]
          },
          {
            name: 'Logged In',
            items: [
              { text: 'User Name', v: "$value -loggedin(name)$" },
              { text: 'User Contact ID', v: "$value -loggedin(contactid)$" },
              { text: 'Session ID', v: "$value -loggedin(sessionID)$" }]
          },
          {
            name: 'My Account',
            items: [
              { text: 'Mailing Lists', v: "$if -myaccountlists$\r\n$repeat -myaccountlists$\r\n  $value -myaccountlist(id)$\r\n  $value -myaccountlist(name)$\r\n  $value -myaccountlist(description)$\r\n  $input -id'enteruniquenamehere' -myaccount(list)$\r\n$endrepeat$\r\n$else$\r\nNo Mailing Lists\r\n$endif$\r\n" },
              { text: 'Campaigns', v: "$if -myaccountcampaigns$\r\n$repeat -myaccountcampaigns$\r\n  $value -myaccountcampaign(id)$\r\n  $value -myaccountcampaign(name)$\r\n  $value -myaccountcampaign(description)$\r\n  $input -id'enteruniquenamehere' -myaccount(campaign)$\r\n$endrepeat$\r\n$else$\r\nNo Campaigns\r\n$endif$\r\n" },
              { text: 'Articles', v: "$if -myaccountarticles$\r\n$repeat -myaccountarticles$\r\n  $value -myaccountarticle(id)$\r\n  $value -myaccountarticle(subject)$\r\n  $value -myaccountarticle(topic)$\r\n  $input -id'enteruniquenamehere' -myaccount(article)$\r\n$endrepeat$\r\n$else$\r\nNo Article Subscriptions\r\n$endif$\r\n" }]
          },
          {
            name: 'Ask A Question',
            items: [
              { text: 'Public Topics', v: "$input -id'enteruniquenamehere' -topics$" }]
          },
          {
            name: 'Find Answers',
            items: [
              { text: 'Topics - Flat', v: "$repeat -topics(findanswer)$\r\n  $value -topic(id)$\r\n  $value -topic(parentid)$\r\n  $value -topic(name)$\r\n  $value -topic(messagecount)$\r\n  $value -topic(segmentid)$\r\n  $value -topic(segmentname)$\r\n  $value -topic(visibility)$\r\n$endrepeat$\r\n" },
              { text: 'Topics - Tree', v: "$repeat -topictree(findanswer)$\r\n  $value -topic(id)$\r\n  $value -topic(parentid)$\r\n  $value -topic(name)$\r\n  $value -topic(messagecount)$\r\n  $value -topic(segmentid)$\r\n  $value -topic(segmentname)$\r\n  $value -topic(visibility)$\r\n  $if -topictree(findanswer)$\r\n    $repeat -topictree(findanswer)$\r\n      $value -topic(id)$\r\n      $value -topic(parentid)$\r\n      $value -topic(name)$\r\n      $value -topic(messagecount)$\r\n      $value -topic(segmentid)$\r\n      $value -topic(segmentname)$\r\n      $value -topic(visibility)$\r\n    $endrepeat$\r\n  $endif$\r\n$endrepeat$\r\n" },
              { text: 'Article List For Topic', v: "$if -articlelist(topic) -Ptopic'nameofinputwithtopicID' -Ppagenum'1' -Pperpage'100' -Psort'DATE_REVERSE'$\r\n$repeat -articlelist(topic) -Ptopic'nameofinputwithtopicID' -Ppagenum'1' -Pperpage'100' -Psort'DATE_REVERSE'$\r\n  $value -articlelist(id)$\r\n  $value -articlelist(subject)$\r\n  $value -articlelist(date)$\r\n  $value -articlelist(topicid)$\r\n  $value -articlelist(topicname)$\r\n  $value -articlelist(rating)$\r\n  $value -articlelist(viewcount)$\r\n  $value -articlelist(public)$\r\n$endrepeat$\r\n$else$\r\nNo Articles\r\n$endif$\r\n" },
              { text: 'Article Search Results In Topic', v: "$if -articlelist(search) -Ptopic'nameofinputwithtopicID' -Psearchid'nameofinputwithsearchterms' -Ppagenum'1' -Pperpage'100' -Psort'DATE_REVERSE'$\r\n$repeat -articlelist(search) -Ptopic'nameofinputwithtopicID' -Psearchid'nameofinputwithsearchterms' -Ppagenum'1' -Pperpage'100' -Psort'DATE_REVERSE'$\r\n  $value -articlelist(id)$\r\n  $value -articlelist(subject)$\r\n  $value -articlelist(date)$\r\n  $value -articlelist(topicid)$\r\n  $value -articlelist(topicname)$\r\n  $value -articlelist(rating)$\r\n  $value -articlelist(viewcount)$\r\n  $value -articlelist(public)$\r\n$endrepeat$\r\n$else$\r\nNo Articles\r\n$endif$\r\n" },
              { text: 'Article Data', v: "$if -article -Pid'nameofinputwitharticleID'$\r\n  $value -article(id) -Pid'nameofinputwitharticleID'$\r\n  $value -article(subject)$\r\n  $value -article(date)$\r\n  $value -article(question)$\r\n  $value -article(answer)$\r\n  $value -article(public)$\r\n  $value -article(segmentid)$\r\n  $value -article(segmentname)$\r\n  $value -article(topicid)$\r\n  $value -article(topicname)$\r\n  $value -article(viewcount)$\r\n  $value -article(rating)$\r\n  $value -article(myrating)$\r\n  $value -article(creatorname)$\r\n  $if -attachments -Pid'nameofinputwitharticleID'$\r\n    $repeat -attachments -Pid'nameofinputwitharticleID'$\r\n      $value -attachment(id)$\r\n      $value -attachment(ownerid)$\r\n      $value -attachment(name)$\r\n    $endrepeat$\r\n  $else$\r\n    No attachments.\r\n  $endif$\r\n$else$\r\nNo Article Found\r\n$endif$\r\n" }]
          }];
      })

      return { feature, variableGroups }
    }
  }
  iservice.components.formInteractionPropertySet = {
    name: 'actionstep-interactionPropertySet',
    template: '#interactionPropertySet',
    props: ['action'],
    setup(props) {
      var intProperties = iservice.IntPropertyData();
      if (!intProperties.loaded) {
        intProperties.Load();
      }
      var properties = Vue.computed(function () {
        if (!intProperties.loaded) {
          intProperties.Load();
        }
        var props = [];
        intProperties.list.forEach(function (property) {
          props.push(property);
        });
        return props;
      })
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature, properties };
    }
  }
  iservice.components.forminteractionSetParent = {
    name: 'actionstep-interactionSetParent',
    template: '#interactionSetParent',
    props: ['action'],
    setup(props) {
      var intProperties = iservice.IntPropertyData();
      if (!intProperties.loaded) {
        intProperties.Load();
      }
      var properties = Vue.computed(function () {
        if (!intProperties.loaded) {
          intProperties.Load();
        }
        var props = [];
        intProperties.list.forEach(function (property) {
          props.push(property);
        });
        return props;
      })
      var feature = Vue.reactive({});
      feature.action = props.action;
      if (!feature.action.changeAction)
        feature.action.changeAction = { steps: [] };

      var steps = [];
      feature.action.changeAction.steps.forEach(function (step) {
        for (let key in step) {
          if (step[key] !== null) {
            var obj = {
              stepType: 'actionstep-' + key
            }
            for (let key2 in step[key]) {
              obj[key2] = step[key][key2];
            }
            steps.push(obj);
          }
        }
      });
      feature.action.changeAction.steps = steps;
      
      
      feature.AddAction = function () {
        feature.action.changeAction.steps.push({});
      };
      feature.ActionMoveUp = function (actions, action) {
        for (var i = 1; i < actions.length; i++) {
          if (action !== actions[i]) continue;
          actions.splice(i, 1);
          actions.splice(i - 1, 0, action);
          return;
        }
      };
      feature.ActionMoveDown = function (actions, action) {
        for (var i = 0; i < actions.length - 1; i++) {
          if (action !== actions[i]) continue;
          actions.splice(i, 1);
          if (i === actions.length - 1) actions.push(action);
          else actions.splice(i + 1, 0, action);
          return;
        }
      };
      feature.ActionDelete = function (actions, action) {
        for (var i = 0; i < actions.length; i++) {
          if (action !== actions[i]) continue;
          actions.splice(i, 1);
          return;
        }
      };
      feature.getStepForm = function (action) {
        action.stepType = 'actionstep-' + action.stepType;
      }
      feature.stepsOptions = iservice.FiltersStepsOptions;
      return { feature, properties };
    }
  }
  iservice.components.formAdminForms = {
    name: 'actionstep-adminForms',
    template: '#adminForms',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formAdminStockResponses = {
    name: 'actionstep-adminStockResponses',
    template: '#adminStockResponses',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formAdminUsertypes = {
    name: 'actionstep-adminUsertypes',
    template: '#adminUsertypes',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formAdminDepartments = {
    name: 'actionstep-adminDepartments',
    template: '#adminDepartments',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formAdminSegments = {
    name: 'actionstep-adminSegments',
    template: '#adminSegments',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formAdminInteractionProperties = {
    name: 'actionstep-adminInteractionProperties',
    template: '#adminInteractionProperties',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formadminMailboxes = {
    name: 'actionstep-adminMailboxes',
    template: '#adminMailboxes',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formAdminTopics = {
    name: 'actionstep-adminTopics',
    template: '#adminTopics',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formAdminAutoresponses = {
    name: 'actionstep-adminAutoresponses',
    template: '#adminAutoresponses',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formadminAgents = {
    name: 'actionstep-adminAgents',
    template: '#adminAgents',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formAdminMassLists = {
    name: 'actionstep-adminMassLists',
    template: '#adminMassLists',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formadminMassCampaigns = {
    name: 'actionstep-adminMassCampaigns',
    template: '#adminMassCampaigns',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formadminMassCriterias = {
    name: 'actionstep-adminMassCriterias',
    template: '#adminMassCriterias',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formadminMassMailings = {
    name: 'actionstep-adminMassMailings',
    template: '#adminMassMailings',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formadminMassMessages = {
    name: 'actionstep-adminMassMessages',
    template: '#adminMassMessages',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formadminSubscriptions = {
    name: 'actionstep-adminSubscriptions',
    template: '#adminSubscriptions',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmailingsend = {
    name: 'actionstep-mailingsend',
    template: '#mailingsend',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formadminLists = {
    name: 'actionstep-adminLists',
    template: '#adminLists',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formadminOpenID = {
    name: 'actionstep-adminOpenID',
    template: '#adminOpenID',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formadminWebsites = {
    name: 'actionstep-adminWebsites',
    template: '#adminWebsites',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formadminSkills = {
    name: 'actionstep-adminSkills',
    template: '#adminSkills',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formcaptchaVerify = {
    name: 'actionstep-captchaVerify',
    template: '#captchaVerify',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formchatEnd = {
    name: 'actionstep-chatEnd',
    template: '#chatEnd',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formchatForward = {
    name: 'actionstep-chatForward',
    template: '#chatForward',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formchatGetNext = {
    name: 'actionstep-chatGetNext',
    template: '#chatGetNext',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formchatKeepAlive = {
    name: 'actionstep-chatKeepAlive',
    template: '#chatKeepAlive',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formchatCreate = {
    name: 'actionstep-chatCreate',
    template: '#chatCreate',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formchatSend = {
    name: 'actionstep-chatSend',
    template: '#chatSend',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formchatToggleAvailability = {
    name: 'actionstep-chatToggleAvailability',
    template: '#chatToggleAvailability',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formagentEmail = {
    name: 'actionstep-agentEmail',
    template: '#agentEmail',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formnote = {
    name: 'actionstep-note',
    template: '#note',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formticket = {
    name: 'actionstep-ticket',
    template: '#ticket',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formcontactFindCreate = {
    name: 'actionstep-contactFindCreate',
    template: '#contactFindCreate',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formcustomerInfoDetails = {
    name: 'actionstep-customerInfoDetails',
    template: '#customerInfoDetails',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.forminteractionHistoryProperties = {
    name: 'actionstep-interactionHistoryProperties',
    template: '#interactionHistoryProperties',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formlogout = {
    name: 'actionstep-logout',
    template: '#logout',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueGetApprove = {
    name: 'actionstep-messageQueueGetApprove',
    template: '#messageQueueGetApprove',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueGetNext = {
    name: 'actionstep-messageQueueGetNext',
    template: '#messageQueueGetNext',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueSaveResolve = {
    name: 'actionstep-messageQueueSaveResolve',
    template: '#messageQueueSaveResolve',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueUnqueue = {
    name: 'actionstep-messageQueueUnqueue',
    template: '#messageQueueUnqueue',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueQAReject = {
    name: 'actionstep-messageQueueQAReject',
    template: '#messageQueueQAReject',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueChangeTopic = {
    name: 'actionstep-messageQueueChangeTopic',
    template: '#messageQueueChangeTopic',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueMarkSpam = {
    name: 'actionstep-messageQueueMarkSpam',
    template: '#messageQueueMarkSpam',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueSaveDraft = {
    name: 'actionstep-messageQueueSaveDraft',
    template: '#messageQueueSaveDraft',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueAssign = {
    name: 'actionstep-messageQueueAssign',
    template: '#messageQueueAssign',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueChangeSegment = {
    name: 'actionstep-messageQueueChangeSegment',
    template: '#messageQueueChangeSegment',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueForwardExternal = {
    name: 'actionstep-messageQueueForwardExternal',
    template: '#messageQueueForwardExternal',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueSaveNote = {
    name: 'actionstep-messageQueueSaveNote',
    template: '#messageQueueSaveNote',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmessageQueueSend = {
    name: 'actionstep-messageQueueSend',
    template: '#messageQueueSend',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmyAccountSubscriptions = {
    name: 'actionstep-myAccountSubscriptions',
    template: '#myAccountSubscriptions',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formmyAccountDetails = {
    name: 'actionstep-myAccountDetails',
    template: '#myAccountDetails',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formpasswordResetSend = {
    name: 'actionstep-passwordResetSend',
    template: '#passwordResetSend',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formpasswordResetConfirm = {
    name: 'actionstep-passwordResetConfirm',
    template: '#passwordResetConfirm',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formfindAnswerCreate = {
    name: 'actionstep-findAnswerCreate',
    template: '#findAnswerCreate',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formfindAnswerModify = {
    name: 'actionstep-findAnswerModify',
    template: '#findAnswerModify',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formfindAnswerSubscribe = {
    name: 'actionstep-findAnswerSubscribe',
    template: '#findAnswerSubscribe',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formfindAnswerFeedback = {
    name: 'actionstep-findAnswerFeedback',
    template: '#findAnswerFeedback',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formfindAnswerSendNotification = {
    name: 'actionstep-findAnswerSendNotification',
    template: '#findAnswerSendNotification',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formfindAnswerDelete = {
    name: 'actionstep-findAnswerDelete',
    template: '#findAnswerDelete',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formfindAnswerDelete = {
    name: 'actionstep-findAnswerDelete',
    template: '#findAnswerDelete',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formchangeTopic = {
    name: 'actionstep-changeTopic',
    template: '#changeTopic',
    props: ['action'],
    setup(props) {
      var topics = iservice.TopicData();
      var segments = iservice.SegmentData();
      if (!topics.loaded)
        topics.Load();

      var topicGroups = Vue.computed(() => {
        var groups = [];
        if (!segments.loaded || !topics.loaded) {
          segments.Load(); topics.Load();
        }
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            group.options.push({ name: topic.name, value: topic.id, level: Math.min(2, topic.depth) });
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.value, r.value); });
          if (group.options.length < 1) return;
          groups.push(group);
        });
        return groups;
      });
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature, topicGroups };
    }
  }
  iservice.components.formfilter = {
    name: 'actionstep-filter',
    template: '#filterForm',
    props: ['action'],
    setup(props) {
      var filters = iservice.FilterData();
      if (!filters.loaded) {
        filters.Load();
      }
      var feature = Vue.reactive({});
      feature.action = props.action;
      var filterList = Vue.computed(function () {
        if (!filters.loaded) {
          filters.Load();
        }
        var filtersData = [];
        filters.list.forEach(function (filter) {
          filtersData.push(filter);
        });
        return filtersData;
      });
      return { feature, filterList };
    }
  }
  iservice.components.formforwardExternal = {
    name: 'actionstep-forwardExternal',
    template: '#forwardExternal',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.forminteractionResolve = {
    name: 'actionstep-interactionResolve',
    template: '#interactionResolve',
    props: ['action'],
    setup(props) {
      var feature = Vue.reactive({});
      feature.action = props.action;
      return { feature };
    }
  }
  iservice.components.formsendAutoResponse = {
    name: 'actionstep-sendAutoResponse',
    template: '#sendAutoResponse',
    props: ['action', 'type'],
    setup(props) {
      var segments = iservice.SegmentData();
      var mailboxes = iservice.MailboxData();
      var autoResponses = iservice.AutoResponseData();
      if (!segments.loaded || !mailboxes.loaded || !autoResponses.loaded) {
        segments.Load(); mailboxes.Load(); autoResponses.Load();
      }
      var type = Vue.computed(function () {
        return props.type;
      });
      function isHidden(segment) {
        return !segment.hidden;
      }
      var feature = Vue.reactive({});
      feature.action = props.action;
      var segmentsList = Vue.computed(function () {
        if (!segments.loaded) {
          segments.Load();
        }
        var segmentData = [];
        segments.list.forEach(function (segment) {
          segmentData.push(segment);
        });
        return segmentData;
      });
      var mailboxData = Vue.computed(function () {
        var mailboxArr = [];
        if (!mailboxes.loaded || !segments.loaded) {
          mailboxes.Load(); segments.Load(); return;
        }
        if (!segments.list) return;
        var segment = segments.get(feature.action.segmentID);
        if (!segment) return;
        var group = { id: segment.id, label: segment.name, options: [] };
        mailboxes.inSegment(segment.id).forEach(mailbox => {
          var option = {
            id: mailbox.id,
            name: mailbox.name,
            email: mailbox.replyTo,
            search: mailbox.name + '\n' + mailbox.replyTo
          };
          group.options.push(option);
        });
        if (group.options.length < 1) return;
        group.options.sort((l, r) => { return mailboxes.SortCompare(l.value, r.value); });
        mailboxArr.push(group);
        return mailboxArr;
      })
      var agentNotifications = Vue.computed(function () {

        var autoResponsesData = [];
        if (!segments.loaded) {
          segments.Load(); return;
        }
        if (!segments.list.length) return;
        var segment = segments.get(feature.action.segmentID);

        if (!segment) return;

        var group = { id: segment.id, label: segment.name, options: [] };
        autoResponses.inSegment(segment.id).forEach(function (response) {
          if (type.value == 'alert') {
            if (!response.listNames.includes('alert')) return;
          }
          var option = {
            value: response.id,
            name: response.name,
            label: '(' + response.segmentName + ') ' + response.name,
            search: response.name
          };
          group.options.push(option);
          if (group.options.length < 1) return;
          group.options.sort((l, r) => { return autoResponses.SortCompare(l.value, r.value); });
        });
        autoResponsesData.push(group);
        autoResponsesData.sort((l, r) => { return segments.SortCompare(l.id, r.id); });
        return autoResponsesData;
      })
      
      return { feature, segmentsList, mailboxes, agentNotifications, isHidden, mailboxData };
    }
  }
  iservice.components.formselectAgents = {
    name: 'actionstep-selectAgents',
    template: '#selectAgents',
    props: ['action'],
    setup(props) {
      var agents = iservice.AgentData();
      if (!agents.loaded) {
        agents.Load();
      }
      var feature = Vue.reactive({});
      feature.action = props.action;
      if (!feature.action.agentIDs) feature.action.agentIDs = [];
      var agentsList = Vue.computed(function () {
        if (!agents.loaded) {
          agents.Load();
        }
        var agentsData = [];
        agents.list.forEach(function (agent) {
          agentsData.push(agent);
        });
        return agentsData;
      });
      return { feature, agentsList };
    }
  }
  iservice.components.formroundRobin = {
    name: 'actionstep-roundRobin',
    template: '#roundRobin',
    props: ['action'],
    setup(props) {
      var departments = iservice.DepartmentsData();
      var contactProperties = iservice.ContactPropertyData();
      var lists = iservice.ListsData();

      if (!departments.loaded || !contactProperties) {
        departments.Load(); contactProperties.Load();
      }
      var feature = Vue.reactive({});
      feature.action = props.action;
      var departmentsList = Vue.computed(function () {
        if (!departments.loaded) {
          departments.Load();
        }
        var departmentData = [];
        departments.list.forEach(function (department) {
          departmentData.push(department);
        });
        return departmentData;
      });
      var boolType = Vue.computed(() => {
        lists.Load();
        var bool = lists.list.find(list => {
          return list.name == 'Bool';
        });
        if (!bool) return '';
        return 'detail' + bool.id;
      });
      var contactPropertiesList = Vue.computed(function () {
        if (!contactProperties) {
          contactProperties.Load();
        }
        var contactpropsData = [];
        contactProperties.list.forEach(function (contactprop) {
          if (contactprop.purpose != 'Agent') return;
          if (contactprop.segmentID == '1') return;
          if (contactprop.type != boolType.value) return;
          contactpropsData.push(contactprop);
        });
        return contactpropsData;
      });
      return { feature, departmentsList, contactPropertiesList };
    }
  }
})(iservice);