﻿var iservice = window.iservice || {};
(function (iservice) {
  iservice.components = iservice.components || {};

  iservice.components.interactionViewLoader = {
    name: 'interaction-view-loader',
    template: '#interaction-view-loader',
    props: ['itemid', 'threads', 'tableIDs', 'threadID'],
    setup(props) {
      var threadID = Vue.computed(function () {
        return props.threadID;;
      })
      var interactions = iservice.InteractionData();
      var plugins = iservice.Plugins();
      Vue.provide('interactionID', props.itemid);

      var tableIDs = Vue.computed(function () {
        return props.tableIDs;
      })
      var loading = Vue.ref(true);
      iservice.API('webapp-api-history?api=details', null, { interactionID: props.itemid }).then(data => {
        interactions.add(data);
        if (data.history && data.history.length) {
          data.history.forEach(item => {
            interactions.add(item);
          });
        }
      }).catch(errors => {

      }).finally(() => {
        loading.value = false;
      });

      var viewComponent = Vue.computed(() => {
        if (loading.value) return '';
        var plugin = plugins.FindPlugin(props.itemid);
        return plugin.detailsComponent;
      });

      return { loading, viewComponent, tableIDs, threadID };
    }
  };
  iservice.components.caseViewThreads = {
    name: 'case-view-threads',
    template: '#case-view-threads',
    props: ['itemid', 'threads'],
    setup(props) {
      Vue.provide('interactionID', props.itemid);
      var loading = Vue.ref(true);
      var feature = Vue.reactive({ errors: [], caseProps: { props: [] }, intProps: { props: [] }, threadID: '' });
      var threads = Vue.computed(function () {
        return props.threads;
      });
      iservice.API('webapp-api-history?api=details', null, { interactionID: props.itemid }).then(data => {
        var caseProps = data.caseProperties.length ? data.caseProperties[0].properties : [];
        var intProps = data.interactionProperties.length ? data.interactionProperties[0].properties : [];
        iservice.EditableProperties(feature.caseProps, caseProps);
        iservice.EditableProperties(feature.intProps, intProps);
        feature.threadID = data.threadID;

      }).catch(errors => {

      }).finally(() => {
        loading.value = false;
      });
      function saveProperties() {
        iservice.API('webapp-api-history?api=propertyset', null, { interactionID: props.itemid, values: feature.intProps.out, caseValues: feature.caseProps.out }).then(data => {
          iservice.SuccessMessage('Saved successfully');
        }).catch(errors => {
          feature.errors = errors;
        }).finally(() => {

        });
      }
      return { loading, feature, threads, saveProperties };
    }
  };
  iservice.components.iserviceForwardCopy = {
    name: 'iservice-agent-forward-email',
    template: '#iservice-agent-forward-email',
    props: ['itemid', 'type'],
    emits: ['cancel'],
    setup(props, context) {
      var topics = iservice.TopicData();
      var segments = iservice.SegmentData();
      var mailboxes = iservice.MailboxData();
      var interactions = iservice.InteractionData();

      var formFeatures = Vue.reactive({ tab: 'agent_email_details', errors: {}, success: '', loading: false, mailboxes: [], addresses: [], featureProps: {}, featureCaseProps: {}, processing: false, type: '' });
      formFeatures.details = { topicID: '', mailboxID: '', destEmail: '', cc: '', bcc: '', subject: '', bodyHtml: '', note: '', segmentID: '' }

      var interactionID = '';
      var interactionTopic = '';
      formFeatures.type = props.type;
      if (props.type == 'interaction') {
        interactionID = props.itemid;
        var interaction = interactions.get(interactionID);
        formFeatures.details.segmentID = interaction.segmentID;
        formFeatures.details.cc = interaction.cc;
        formFeatures.details.bcc = interaction.bcc;
        formFeatures.details.subject = 'Fwd: '+interaction.subject;
        formFeatures.details.note = interaction.note;
        interactionTopic = interaction.topicID;
      }

      formFeatures.TabClass = function (tab) { return { 'active': tab == formFeatures.tab }; }
      formFeatures.TabClick = function (tab) {
        formFeatures.tab = tab;
      }

      var featureUploads = iservice.FeatureUploads();
      var segmentOptions = Vue.ref([]);
      var selectMailbox = Vue.ref(null);
      formFeatures.loading = true;
      function startEmail() {
        formFeatures.processing = true;
        if (formFeatures.details.topicID) {
          var topic = topics.get(formFeatures.details.topicID);
          formFeatures.details.segmentID = topic.segmentID;
        }
        iservice.API('webapp-api-history?api=startforwardemail', null, {
          interactionID: interactionID ? interactionID : "",
          segmentID: formFeatures.details.segmentID ? formFeatures.details.segmentID : ""
        }).then(function (data) {

          var mailboxIDs = [];
          data.mailboxes.forEach(function (item) {
            mailboxIDs.push(item.id);
          });
          Vue.watchEffect(() => {
            segmentOptions.value.length = 0;
            segmentOptions.loading = true;
            if (!mailboxes.loaded || !segments.loaded) {
              mailboxes.Load(); segments.Load(); return;
            }
            segmentOptions.loading = false;
            segments.list.forEach(seg => {
              var group = { id: seg.id, label: seg.name, options: [] };
              mailboxes.inSegment(seg.id).forEach(mailbox => {
                if (mailboxIDs.includes(mailbox.id)) {
                  var option = {
                    value: mailbox.id,
                    name: mailbox.name,
                    email: mailbox.replyTo,
                    search: mailbox.name + '\n' + mailbox.replyTo
                  };
                  group.options.push(option);
                }
              });
              if (group.options.length < 1) return;
              group.options.sort((l, r) => { return mailboxes.SortCompare(l.value, r.value); });
              segmentOptions.value.push(group);
            });
            segmentOptions.value.sort((l, r) => { return segments.SortCompare(l.id, r.id); });
            var segment = segments.get(formFeatures.details.segmentID);
            formFeatures.details.defaultMailboxID = segment.mailboxID;
            if (!selectMailbox.value) return;
            var mailbox = mailboxes.get(segment.mailboxID);
            var option = {
              value: mailbox.id,
              name: mailbox.name,
              email: mailbox.replyTo,
              search: mailbox.name + '\n' + mailbox.replyTo
            };
            selectMailbox.value.select(option);
          });
          var answerIsInline = Vue.reactive({});
          let quotedBody = data.quotedBody;
          if (data.quotedBody.startsWith("<div>")) {
            quotedBody = quotedBody.substring(0, 5) + "<br><br>" + quotedBody.substring(5, quotedBody.length);
          }
          formFeatures.details.bodyHtml = iservice.FindInlineAttachments(quotedBody, data.attachments, answerIsInline);
          var segment = segments.get(formFeatures.details.segmentID);
          formFeatures.details.defaultMailboxID = segment.mailboxID;

          formFeatures.featureProps = Vue.reactive({ label: 'Answer Properties', length: 0 });
          var intProps = data.interactionProperties.length ? data.interactionProperties[0].properties : [];
          if (data.interactionProperties.length) iservice.EditableProperties(formFeatures.featureProps, intProps);

          formFeatures.featureCaseProps = Vue.reactive({ label: 'Case Properties', length: 0 });
          var caseProps = data.caseProperties.length ? data.caseProperties[0].properties : [];
          iservice.EditableProperties(formFeatures.featureCaseProps, caseProps);
          Vue.watchEffect(function () {
            formFeatures.featureCaseProps.length = formFeatures.featureCaseProps.props.length;
            formFeatures.featureProps.length = formFeatures.featureProps.props.length;
            formFeatures.haveProperties = formFeatures.featureCaseProps.props.length || formFeatures.featureProps.props.length;
          });
          Vue.watchEffect(function () {
            formFeatures.propertyMissingClass = { 'text-red': formFeatures.featureCaseProps.missingRequired || formFeatures.featureProps.missingRequired };
          });
          Vue.watchEffect(function () {
            formFeatures.hasValues = formFeatures.featureCaseProps.hasValues || formFeatures.featureProps.hasValues;
          });
        }).catch(function (errors) {
          formFeatures.errors = errors;
        }).finally(function () {
          formFeatures.loading = false;
          formFeatures.processing = false;
        });
      }
      startEmail();

      var editor = Vue.ref(null);
      var anchorStocks = Vue.ref(null);
      var featureStockResponses = Vue.reactive({ filter: '', segmentID: formFeatures.details.segmentID });
      Vue.watchEffect(function () {
        if (!formFeatures.details.topicID) return;
        var topic = topics.get(formFeatures.details.topicID);
        featureStockResponses.segmentID = topic.segmentID;
      })
      featureStockResponses.Open = function () {
        featureStockResponses.Close = iservice.components.popupHolder.Open(anchorStocks, 'stock-responses-menu', false, featureStockResponses);
      }
      featureStockResponses.PasteVar = function (text) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteText(text);
      }
      featureStockResponses.PasteStock = function (html) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteHtml(html);
      }
      function Cancel() {
        context.emit('cancel');
      }
      function CheckProperties(feature, label) {
        for (var prop of feature.props) {
          if (!prop.missingRequired) continue;
          prop.focusMe = true;
          formFeatures.TabClick('properties');
          formFeatures.errors = ['Must set value for ' + label + ' property ' + prop.def.name + '.'];
          return false;
        }
        return true;
      }
      var errors = Vue.reactive({});
      formFeatures.Send = function (action) {
        formFeatures.errors = [];
        if (props.type == 'interaction')
          formFeatures.details.topicID = interactionTopic;

        if (!formFeatures.details.mailboxID) {
          formFeatures.errors.push({ msg: 'Please enter a From email address.', details: { description: 'It looks like you are missing From email address. Please select a From email before sending.' } });
          errors.mailboxID = true;
        }

        if (!formFeatures.details.destEmail) {
          formFeatures.errors.push({ msg: 'Please enter a To email address.', details: { description: 'It looks like you are missing To email address. Please select a To email before sending.' } });
          errors.destEmail = true;
        }
        if (!formFeatures.details.subject) {
          formFeatures.errors.push({ subject: true, msg: 'Please enter a subject for your email.', details: { description: 'Every interaction must have a subject, but it looks like one is missing here. Please enter a subject for your email before sending.' } });
          errors.subject = true;
        }
        if (!formFeatures.details.bodyHtml) {
          formFeatures.errors.push({ msg: 'Your email doesn\'t have a body.', details: { description: 'It looks like you are missing the body of this email. Please type your message in the editor before sending.' } });
          errors.bodyHtml = true;
        }
        if (formFeatures.errors.length) return;
        if (formFeatures.featureProps.props) {
          if (!CheckProperties(formFeatures.featureProps, 'answer')) return;
        }
        if (formFeatures.featureCaseProps.props) {
          if (!CheckProperties(formFeatures.featureCaseProps, 'case')) return;
        }
        formFeatures.processing = true;
        formFeatures.details.action = action;
        formFeatures.details.inResponseToID = interactionID;
        formFeatures.details.properties = formFeatures.featureProps.out;
        formFeatures.details.caseProperties = formFeatures.featureCaseProps.out;
        iservice.API('webapp-api-custinfo?api=forwardemail', null, { details: formFeatures.details, email: formFeatures.details.destEmail}, featureUploads.files).then(function (data) {
          Cancel();
          iservice.SuccessMessage('Email sent.');
        }).catch(function (errors) {
          formFeatures.errors = errors;
        }).finally(function () {
          formFeatures.processing = false;
        });
      }
      formFeatures.OnTopicSelect = function () {
        if (!formFeatures.details.topicID) return;
        startEmail();
      }
      return {
        formFeatures, featureUploads, editor, anchorStocks, featureStockResponses, Cancel, segmentOptions, selectMailbox, errors  };
    }
  };
  iservice.components.interactionView = {
    name: 'interaction-view',
    template: '#interaction-view',
    props: ['itemid', 'type', 'tableIDs', 'threadID'],
    setup(props) {
      var loggedIn = iservice.LoggedInData();
      var interactions = iservice.InteractionData();
      var messages = iservice.Messages();
      var threadID = Vue.computed(function () {
        return props.threadID;
      });
      var interaction = Vue.computed(function () {
        return interactions.get(props.itemid);
      });
      var contactID = interaction.value.customerID;
      function checkEmail(logins) {
        var response = false
        logins.forEach(function (item) {
          if (item.type == 'Email') {
            response = true;
          }
        });
        return response;
      }
      var ViewOperation = Vue.reactive({ component: '', type: 'interaction', showAlert: false });
      ViewOperation.AlertClear = function () {
        ViewOperation.showAlert = false;
      }
      ViewOperation.Click = function (component) {
        if (component == 'iservice-agent-email' || component == 'iservice-agent-ticket' || component == 'iservice-agent-forward-email') {
          var emailExists = checkEmail(interaction.value.customerLogins);
          if (!emailExists) {
            ViewOperation.showAlert = true;
            return;
          }
        }
        ViewOperation.component = component;
      }
      var actionComponent = Vue.ref('');
      function StopAction() {
        actionComponent.value = '';
        ViewOperation.component = '';
      }


      var featureAction = Vue.reactive({});

      var displayHtml = Vue.ref('');
      var isInline = Vue.reactive({});
      displayHtml.value = iservice.FindInlineAttachments(interaction.value.bodyHtml, interaction.value.attachments, isInline);
      var realAttachments = Vue.ref(iservice.DownloadableFiles(interaction.value.attachments, isInline));

      var featureDetails = Vue.reactive({ tab: 'details' });
      Vue.watchEffect(() => { featureDetails.haveNote = interaction.value && interaction.value.note && interaction.value.note != ''; })
      featureDetails.TabClass = function (tab) { return { 'active': tab == featureDetails.tab }; }
      featureDetails.TabClick = function (tab) { featureDetails.tab = tab; }

      var actionMenuRef = Vue.ref(null);
      var featureActionMenu = Vue.reactive({ showPlain: false, showCase: false, showStatusAudit: false });
      Vue.watchEffect(() => { featureActionMenu.interaction = interaction.value; });
      Vue.watchEffect(() => { featureActionMenu.showDelete = iservice.HaveRight('Tab.MessageQueue.Delete'); });
      featureActionMenu.Click = function () {
        iservice.components.popupHolder.Open(actionMenuRef, 'interaction-view-action-menu', false, featureActionMenu);
      }
      featureActionMenu.Delete = function (e) {
        if (featureActionMenu.deleting) return;
        featureActionMenu.errors = featureActionMenu.success = null;
        featureActionMenu.deleting = true;
        iservice.API('webapp-api-history?api=delete', e, { interactionID: props.itemid }).then(data => {
          interactions.deleted(props.itemid);
          iservice.SuccessMessage('Interaction deleted successfully.');
        }).catch(errors => {
          featureActionMenu.errors = errors;
        }).finally(() => {
          featureActionMenu.deleting = true;
        });
      }

      var featureProps = Vue.reactive({});
      Vue.watchEffect(() => {
        if (!interaction.value) return;
        featureProps.label = iservice.types.IsQuestion(interaction.value.type) ? 'Question Properties' : (iservice.types.IsAnswer(interaction.value.type) ? 'Answer Properties' : 'Interaction Properties');
      });
      iservice.EditableProperties(featureProps, interaction.value.intProps);

      var featureCaseProps = Vue.reactive({ label: 'Case Properties'});
      iservice.EditableProperties(featureCaseProps, interaction.value.caseProps);

      var featurePropertySet = Vue.reactive({ working: false });
      Vue.watchEffect(() => { featurePropertySet.shown = iservice.HaveRight('Tab.CustomerInfo.ModifyHistory') && (featureProps.props.length || featureCaseProps.props.length); });
      featurePropertySet.Click = function (e) {
        if (featurePropertySet.working) return;
        featurePropertySet.errors = featurePropertySet.success = null;
        featurePropertySet.working = true;
        iservice.API('webapp-api-history?api=propertyset', e, { interactionID: props.itemid, values: featureProps.out, caseValues: featureCaseProps.out }).then(data => {
          interactions.add(data);
          featureProps.Reload();
          featureCaseProps.Reload();
          iservice.SuccessMessage('Saved successfully');
        }).catch(errors => {
          featurePropertySet.errors = errors;
        }).finally(() => {
          featurePropertySet.working = false;
        });
      }

      Vue.watchEffect(() => { featureDetails.haveProperties = featureCaseProps.props.length || featureProps.props.length; });
      Vue.watchEffect(() => { featureDetails.propertyMissingClass = { 'text-red': featureCaseProps.missingRequired || featureProps.missingRequired }; });
      Vue.watchEffect(() => { featureDetails.hasValues = featureCaseProps.hasValues || featureProps.hasValues; });

      var featureAnswer = Vue.reactive({});
      var canAnswer = Vue.computed(() => {
        if (!interaction.value) return false;
        if (interaction.value.type == 'Unsent Response' || interaction.value.type == 'Chat') return false;
        if (interaction.value.assignedToID != loggedIn.contactID) return false;
        if (interaction.value.status == 'Unassigned' || interaction.value.status == 'Queued' || interaction.value.status == 'Pending' || interaction.value.status == 'SentWaitingReply') return true;
        return false;
      });
      Vue.watchEffect(() => {
        featureAnswer.shown = canAnswer.value;
      });
      featureAnswer.Click = function (e) {
        actionComponent.value = 'interaction-answer-loader';
      }

      var featureGetThis = Vue.reactive({});
      var canGetThis = Vue.computed(() => {
        if (!interaction.value) return false;
        if (interaction.value.type == 'Unsent Response' || interaction.value.type == 'Chat') return false;
        if (!iservice.HaveRight('Tab.MessageQueue.Supervise')) return false;
        if (interaction.value.assignedToID == loggedIn.contactID) return false;
        if (interaction.value.status == 'Unassigned' || interaction.value.status == 'Queued' || interaction.value.status == 'SentWaitingReply') return true;
        if (interaction.value.status == 'Pending' && iservice.HaveRight('Tab.MessageQueue.Approve')) return true;
        return false;
      });
      Vue.watchEffect(() => {
        featureGetThis.shown = canGetThis.value;
      });
      featureGetThis.Click = function (e) {
        if (featureGetThis.working) return;
        featureGetThis.working = true;
        featureAction.errors = featureAction.success = null;
        iservice.API('webapp-api-mq-myqueue?api=assign', e, { interactionID: interaction.value.id, interactionIDs: [interaction.value.id], answer: { isMassUpdate: true, reassignAgentID: loggedIn.contactID } }).then(data => {
          interactions.add(data.question.details);
          interactions.add(data.response.details);
          interactions.add(data.thread.details);
          actionComponent.value = 'interaction-answer-loader';
        }).catch(errors => {
          featureAction.errors = errors;
        }).finally(() => {
          featureGetThis.working = false;
        });
      }

      return { actionComponent, interaction, StopAction, realAttachments, displayHtml, featureDetails, featureProps, featureCaseProps, featureActionMenu, actionMenuRef, featurePropertySet, featureGetThis, featureAnswer, featureAction, ViewOperation, contactID, threadID };
    }
  }
  iservice.components.intearctionDetailsPanel = {
    name: 'interaction-details-panel',
    template: '#interaction-details-panel',
    props: ['itemid'],
    setup(props) {
      var interactions = iservice.InteractionData();

      var interaction = Vue.computed(function () {
        return interactions.get(props.itemid);
      });

      var headerFrom = Vue.computed(() => {
        if (!interaction.value) return '';
        return interaction.value.headerFrom;
      });
      var headerTo = Vue.computed(() => {
        if (!interaction.value) return '';
        var type = interaction.value.type;
        if (type == 'Agent Ticket' || type == 'Ask A Question' || type == 'Customer Ticket' || type == 'Ticket' || type == 'Chat') {
          return interaction.value.segmentName + ' / ' + interaction.value.topicName;
        }
        return interaction.value.headerTo;
      });
      var headerCC = Vue.computed(() => {
        if (!interaction.value) return '';
        return interaction.value.cc;
      });
      var headerBCC = Vue.computed(() => {
        if (!interaction.value) return '';
        return interaction.value.bcc;
      });

      var attachmentURL = Vue.computed(() => {
        return function (intId, attId) {
          var roothPath = window.location.origin;
          return roothPath + '/File.aspx?interactionID=' + intId + '&fileID=' + attId;
        }
      });

      var userName = Vue.ref('');
      var userType = Vue.ref('');
      var userImage = Vue.ref('');
      Vue.watchEffect(() => {
        if (!interaction.value) return;
        var type = interaction.value.type;
        if (type == 'Agent Email' || type == 'Agent Response' || type == 'Secure Email' || type == 'Private Note' || type == 'Public Note' || type == 'Resolve Note' || type == 'Unsent Response' || type == 'Find Answer Article') {
          userType.value = 'Agent';
          userImage.value = rootPath + 'images/agent.png';
          userName.value = interaction.value.resolvedByName;
        }
        else if (type == 'Customer Email' || type == 'Agent Ticket' || type == 'Customer Ticket' || type == 'Chat') {
          userType.value = 'Customer';
          userImage.value = rootPath + 'images/customer.png';
          userName.value = interaction.value.customerName;
        }
        else {
          userType.value = 'System';
          userImage.value = rootPath + 'images/system.png';
          userName.value = '';
        }
      });

      var isInline = Vue.reactive({});
      iservice.FindInlineAttachments(interaction.value.bodyHtml, interaction.value.attachments, isInline);
      var realAttachments = Vue.ref(iservice.DownloadableFiles(interaction.value.attachments, isInline));
      var haveEmailDownload = iservice.HaveEmailDownload(interaction.value.attachments);

      return { interaction, headerFrom, headerTo, headerCC, headerBCC, userName, userType, userImage, realAttachments, haveEmailDownload, attachmentURL };
    }
  };
  iservice.components.interactionViewActionMenu = {
    name: 'interaction-view-action-menu',
    template: '#interaction-view-action-menu',
    props: ['params'],
    setup(props) {
      var feature = Vue.computed(() => { return props.params; });
      var printUrl = Vue.computed(() => {
        if (!props.params.interaction) return '';
        return rootPath + 'f/interaction-print?interactionId=' + props.params.interaction.id;
      });

      var deleteStarted = Vue.ref(false);

      return { feature, printUrl, deleteStarted };
    }
  };
  iservice.components.fullCaseView = {
    name: 'full-case-view',
    template: '#full-case-view',
    props: ['threadid'],
    emits: ['close'],
    setup(props, context) {
      var interactions = iservice.InteractionData();
      var featureCase = Vue.reactive({ working: true, caseIDs: [] });
      featureCase.Close = function () {
        context.emit('close');
      }

      iservice.API('webapp-api-history?api=children', null, { threadID: props.threadid }).then(data => {
        interactions.add(data.details);
        featureCase.caseIDs.push(data.details.id);
        var root = interactions.get(data.details.id);
        iservice.EditableProperties(featureCase, root.caseProps);

        data.children.sort((a, b) => new Date(a.date) - new Date(b.date));
        data.children.forEach(child => {
          interactions.add(child);
          featureCase.caseIDs.push(child.id);
        });
        //featureCase.caseIDs.sort(iservice.SortID);
      }).catch(errors => {
        
      }).finally(() => {
        featureCase.working = false;
      });

      var featureSave = Vue.reactive({ working: false });
      featureSave.Save = function (e) {
        if (featureSave.working) return;
        featureSave.errors = featureSave.success = null;
        featureSave.working = true;
        iservice.API('webapp-api-history?api=propertyset', e, { interactionID: props.threadid, caseValues: featureCase.out }).then(data => {
          interactions.add(data);
          featureCase.Reload();
          iservice.SuccessMessage('Saved successfully');
        }).catch(errors => {
          featureSave.errors = errors;
        }).finally(() => {
          featureSave.working = false;
        });
      }

      return { featureCase, featureSave };
    }
  }
  iservice.components.caseviewRow = {
    name: 'case-view-row',
    template: '#case-view-row',
    props: ['itemid'],
    setup(props) {
      var interactions = iservice.InteractionData();

      var interaction = Vue.computed(() => {
        return interactions.get(props.itemid);
      });
      var showDetails = Vue.ref(false);
      function Expand() {
        showDetails.value = !showDetails.value;
      }
      var rowClass = Vue.computed(() => {
        if (!interaction.value) return '';
        return iservice.RowColorClass(interaction.value.type);
      });
      var subject = Vue.computed(() => {
        if (!interaction.value) return '';
        return interaction.value.subject;
      });
      var topicName = Vue.computed(() => {
        if (!interaction.value) return '';
        return interaction.value.topicName;
      });
      var resolvedByName = Vue.computed(() => {
        if (!interaction.value) return '';
        return interaction.value.resolvedByName;
      });
      var date = Vue.computed(() => {
        if (!interaction.value) return '';
        var date = interaction.value.date;
        if (!date || date == '') return '';
        return moment(date).format('YYYY-MM-DD hh:mm:ss A');
      });

      return { showDetails, Expand, rowClass, subject, topicName, resolvedByName, date };
    }
  };
  iservice.components.interactionAnswerLoader = {
    name: 'interaction-answer-loader',
    template: '#interaction-answer-loader',
    props: ['itemid', 'load', 'cancelable', 'tableIDs'],
    emits: ['cancel'],
    setup(props, context) {
      var interactions = iservice.InteractionData();
      var plugins = iservice.Plugins();
      var apiData = Vue.ref(null);
      function Cancel() {
        context.emit('cancel');
      }
      var tableIDs = Vue.computed(function () {
        return props.tableIDs;
      });

      var featureLoad = Vue.reactive({ loaded: false, working: false });
      Vue.watchEffect(() => {
        var wantLoad = props.load === undefined || props.load;
        if (!wantLoad || featureLoad.loaded || featureLoad.working) return;
        featureLoad.working = true;
      });
      Vue.watchEffect(() => {
        if (featureLoad.loaded || !featureLoad.working) return;
        featureLoad.errors = null;
        var segmentID = interactions.get(props.itemid).segmentID;
        iservice.API('webapp-api-mq-myqueue?api=details2', null, { interactionID: props.itemid, segmentID: segmentID }).then(function (data) {
          interactions.add(data.details);
          interactions.add(data.thread);
          data.thread.children.forEach(child => {
            interactions.add(child);
          });
          apiData.value = data;
        }).catch(errors => {
          featureLoad.errors = errors;
          Cancel();
        }).finally(() => {
          featureLoad.working = false;
          featureLoad.loaded = true;
        });
      });

      var componentAnswer = Vue.computed(() => {
        if (!featureLoad.loaded) return '';
        var plugin = plugins.FindPlugin(props.itemid);
        return plugin.answerComponent;
      });

      return { componentAnswer, apiData, featureLoad, Cancel, tableIDs };
    }
  }
  iservice.components.interactionAnswer = {
    name: 'interaction-answer',
    template: '#interaction-answer',
    props: ['itemid', 'load', 'cancelable', 'data', 'isquote', 'quotetoggle'],
    emits: ['cancel'],
    setup(props, context) {
      var interactions = iservice.InteractionData();
      var mailboxes = iservice.MailboxData();
      var interaction = interactions.get(props.itemid);
      var interactionRef = Vue.ref(interaction);
      var quotetoggle = props.quotetoggle;
      var isQuote = props.isquote ? props.isquote : false;
      function Cancel() {
        context.emit('cancel');
      }
      var intProperties = iservice.IntPropertyData();
      if (!intProperties.loaded) {
        intProperties.Load();
      }
      var questionDetails = Vue.ref(props.data.details);  
      var answer = Vue.reactive({
        subject: questionDetails.value.responseSubject,
        mailboxID: questionDetails.value.defaultMailboxID,
        destEmail: questionDetails.value.fromAddress,
        destCC: '',
        destBCC: '',
        topicID: questionDetails.value.topicID,
        responseNote: questionDetails.value.note,
        bodyHtml: '',
        supressSubjectRef: false
      });

      var featureCaseProps = Vue.reactive({});
      iservice.EditableProperties(featureCaseProps, interaction.caseProps, out => { answer.threadProperties = out; });
      var featureQuestionProps = Vue.reactive({});
      iservice.EditableProperties(featureQuestionProps, interaction.intProps, out => { answer.messageProperties = out; });
      var featureAnswerProps = Vue.reactive({ responseProps: [] });
      iservice.CopyProps(featureAnswerProps.responseProps, questionDetails.value.responseProperties);
      iservice.EditableProperties(featureAnswerProps, featureAnswerProps.responseProps, out => { answer.answerProperties = out; });

      var questionIsInline = Vue.reactive({});
      iservice.FindInlineAttachments(interaction.bodyHtml, interaction.attachments, questionIsInline);
      var questionAttachments = Vue.ref(iservice.DownloadableFiles(interaction.attachments, questionIsInline));
      var haveEmailDownload = iservice.HaveEmailDownload(interaction.attachments);

      var bodyHtmlEdit = Vue.ref('');
      var answerIsInline = Vue.reactive({});
      bodyHtmlEdit.value = iservice.FindInlineAttachments(questionDetails.value.responseHtml, interaction.attachments, answerIsInline);
      bodyHtmlEdit.value = (bodyHtmlEdit.value).replace("<br/><br/><div>", "<div><br/><br/>");
      bodyHtmlEdit.value = iservice.FindInlineAttachments(bodyHtmlEdit.value, questionDetails.value.responseAttachments, answerIsInline);
      var answerAttachments = Vue.ref(iservice.DownloadableFiles(questionDetails.value.responseAttachments, answerIsInline));
      Vue.watchEffect(() => {
        var keep = [];
        questionDetails.value.responseAttachments.forEach(attach => {
          if (answerIsInline[attach.attachmentID] && bodyHtmlEdit.value.indexOf(iservice.FileReference(attach)) != -1) keep.push(attach.attachmentID);
        });
        answerAttachments.value.forEach(attach => {
          keep.push(attach.attachmentID);
        });
        answer.attachments = keep;
      });
      function AttachmentRemove(attach) {
        var pos = answerAttachments.value.indexOf(attach);
        if (pos == -1) return;
        answerAttachments.value.splice(pos, 1);
      }
      Vue.watchEffect(() => {
        bodyHtmlEdit.value = (bodyHtmlEdit.value).replace("<br /><br /></p>", "</p>").replace("<br><br></p>", "</p>");
        answer.bodyHtml = iservice.RestoreInlineAttachments(bodyHtmlEdit.value, interaction.attachments);
        answer.bodyHtml = iservice.RestoreInlineAttachments(answer.bodyHtml, questionDetails.value.responseAttachments);
      });
      var featureUploads = iservice.FeatureUploads();

      var inputBCC = Vue.ref(null);
      var inputCC = Vue.ref(null);
      var featureCC = Vue.reactive({ showBcc: false, anchor: inputCC });
      featureCC.ShowBCC = function () {
        featureCC.showBcc = true;
        setTimeout(() => { inputBCC.value.focus(); });
      }
      featureCC.ApplyReCC = function () {
        answer.destCC = questionDetails.value.reCC;
      }
      featureCC.Focus = function () {
        
      }
      featureCC.Change = function () {
        if (featureCC.Close) return;
        //featureCC.Close = iservice.components.popupHolder.Open(inputCC, 'address-autocomplete', true, featureCC);
      }
      featureCC.Blur = function () {
        //if (featureCC.Close) featureCC.Close();
      }
      featureCC.Complete = function (to) {
        answer.destCC = to;
      }

      var featureAction = Vue.reactive({});
      featureAction.SubmitAnswer = function (url, e, successMessage, actiontype = null, removeOnSuccess = false) {
        if (featureAction.working) return;
        featureAction.working = true;
        featureAction.errors = featureAction.success = null;
        //secure portal no notification resolve action
        if (actiontype == 'securePortalResolve') {
          if (answer.answerProperties) {
            answer.answerProperties.forEach(function (prop) {
              var property = intProperties.get(prop.propertyID);
              if (property.name == 'Deleted') prop.values = [{ value: "False" }];
              if (property.name == 'DoNotReply') prop.values = [{ value: "True" }];
              if (property.name == 'HasBeenRead') prop.values = [{ value: "True" }];
            });
          }
        }
        //secure portal with notification resolve action
        if (actiontype == 'secureNotificationResolve') {
          if (answer.answerProperties) {
            answer.answerProperties.forEach(function (prop) {
              var property = intProperties.get(prop.propertyID);
              if (property.name == 'Deleted') prop.values = [{ value: "True" }];
              if (property.name == 'DoNotReply') prop.values = [{ value: "True" }];
              if (property.name == 'HasBeenRead') prop.values = [{ value: "True" }];
            });
          }
        }
        //secure portal with notification send action
        if (actiontype == 'SendNotification' || actiontype == 'securePortalSaveNote') {
          if (answer.answerProperties) {
            answer.answerProperties.forEach(function (prop) {
              var property = intProperties.get(prop.propertyID);
              if (property.name == 'Deleted') prop.values = [{ value: "False" }];
              if (property.name == 'DoNotReply') prop.values = [{ value: "False" }];
              if (property.name == 'HasBeenRead') prop.values = [{ value: "False" }];
            });
          }
        }
         //secure portal no notification send action
        if (actiontype == 'securePortalSaveNote' || actiontype == 'securePortalSendResolve') {
          if (answer.answerProperties) {
            answer.answerProperties.forEach(function (prop) {
              var property = intProperties.get(prop.propertyID);
              if (property.name == 'Deleted') prop.values = [{ value: "False" }];
              if (property.name == 'DoNotReply') prop.values = [{ value: "False" }];
              if (property.name == 'HasBeenRead') prop.values = [{ value: "False" }];
            });
          }
        }
        iservice.API(url, e, { interactionID: interaction.id, interactionIDs: [interaction.id], answer }, featureUploads.files).then(data => {
          iservice.SuccessMessage(successMessage);
          featureUploads.Reset();
          if (data.thread) interactions.add(data.thread.row);
          if (data.thread) interactions.add(data.thread.details);
          if (data.question) interactions.add(data.question.row);
          if (data.question) interactions.add(data.question.details);
          if (data.response) interactions.add(data.response.row);
          if (data.response) interactions.add(data.response.details);
          if (data.details.assignedToID == loggedIn.contactID) {
            interactions.add(data.row);
            interactions.add(data.details);
            if (data.details) {
              props.data.details = data.details;
              questionDetails.value = props.data.details;
            }
            featureCaseProps.Reload();
            featureQuestionProps.Reload();
            iservice.CopyProps(featureAnswerProps.responseProps, questionDetails.value.responseProperties);
            featureAnswerProps.Reload();
            answer.subject = questionDetails.value.responseSubject;
            answer.mailboxID = questionDetails.value.defaultMailboxID;
            answer.destEmail = questionDetails.value.fromAddress;
            answer.responseNote = questionDetails.value.note;
            answerIsInline = Vue.reactive({});
            bodyHtmlEdit.value = iservice.FindInlineAttachments(questionDetails.value.responseHtml, interaction.attachments, answerIsInline);
            bodyHtmlEdit.value = iservice.FindInlineAttachments(bodyHtmlEdit.value, questionDetails.value.responseAttachments, answerIsInline);
            answerAttachments.value = iservice.DownloadableFiles(questionDetails.value.responseAttachments, answerIsInline);
            actionButtons.value.forEach(feature => { if (feature.Reset) fetaure.Reset(); });
          }
          if (removeOnSuccess) {
            iservice.data.mq.numCanGet = data.numCanGet;
            iservice.data.mq.numCanApprove = data.numCanApprove;
          }
        }).catch(errors => {
          featureAction.errors = errors;
          featureAction.working = false;
        }).finally(() => {
          featureAction.working = false;
        });
      }
      function CheckProperties(feature, label) {
        for (var prop of feature.props) {
          if (!prop.missingRequired) continue;
          prop.focusMe = true;
          featureTabs.TabClick('property');
          featureAction.errors = ['Must set value for ' + label + ' property ' + prop.def.name + '.'];
          return false;
        }
        return true;
      }
      featureAction.CheckAnswerProperties = function () {
        return CheckProperties(featureAnswerProps, 'answer');
      }
      featureAction.CheckQuestionProperties = function () {
        return CheckProperties(featureQuestionProps, 'question');
      }
      featureAction.CheckCaseProperties = function () {
        return questionDetails.value.caseOpenCount > 1 || CheckProperties(featureCaseProps, 'case');
      }

      var actionButtons = Vue.computed(() => {
        var buttons = [];
        if (!iservice.config.answerActions) {
          iservice.config.answerActions = iservice.components.adminVueappCustomizeActions.buttonsStandard;
        }
        iservice.config.answerActions.forEach(name => {
          var component = iservice.components[name];
          if (!component || !component.answerAction) return;
          var feature = component.answerAction(interactionRef, answer, featureAction);
          if (!feature.Visible()) return;
          buttons.push({ component, feature });
        });
        return buttons;
      });

      var featureTabs = Vue.reactive({ tab: 'address', haveProps: false });
      Vue.watchEffect(() => {
        featureTabs.haveProps = featureCaseProps.props.length > 0 || featureQuestionProps.props.length > 0 || featureAnswerProps.props.length > 0;
        featureTabs.havePropValues = featureCaseProps.hasValues || featureQuestionProps.hasValues || featureAnswerProps.hasValues
        featureTabs.havePropRequired = featureCaseProps.missingRequired || featureQuestionProps.missingRequired || featureAnswerProps.missingRequired;
        featureTabs.propertyIconClass = { 'text-red': featureTabs.havePropRequired };
      });
      featureTabs.TabClass = function (tab) { return { 'active': tab == featureTabs.tab, 'text-red': tab == 'property' && featureTabs.havePropRequired }; }
      featureTabs.PaneClass = function (tab) { return { 'show active': tab == featureTabs.tab }; }
      featureTabs.TabClick = function (tab) {
        featureTabs.tab = tab;
      }

      var addressFromSelect = Vue.ref(null);
      var featureAddresses = Vue.reactive({ mailboxes: [], mailbox: null });
      var fromMailboxList = Vue.computed(() => {
        var list = [];
        mailboxes.Load();
        var foundSelected = false;
        var foundDefault = false;
        mailboxes.inSegment(interaction.segmentID).forEach(mailbox => {
          var option = Vue.reactive({ id: mailbox.id, name: mailbox.name, replyTo: mailbox.replyTo, search: mailbox.name + '\n' + mailbox.replyTo });
          list.push(option);
          if (mailbox.id == answer.mailboxID) foundSelected = true;
          if (mailbox.id == questionDetails.value.defaultMailboxID) foundDefault = true;
        });
        if (!foundSelected) {
          answer.mailboxID = foundDefault ? questionDetails.value.defaultMailboxID : '';
        }
        list.sort((l, r) => { return mailboxes.SortCompare(l.id, r.id); });
        return list;
      });
      Vue.watchEffect(() => {
        featureAddresses.subjectRef = props.data.details.subjectRef;
      });

      var editor = Vue.ref(null);
      var anchorStocks = Vue.ref(null);
      var featureStockResponses = Vue.reactive({ filter: '', segmentID: interaction.segmentID });
      featureStockResponses.Open = function () {
        featureStockResponses.Close = iservice.components.popupHolder.Open(anchorStocks, 'stock-responses-menu', false, featureStockResponses);
      }
      featureStockResponses.PasteVar = function (text) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteText(text);
      }
      featureStockResponses.PasteStock = function (html) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteHtml(html);
      }

      var actionMenuRef = Vue.ref(null);
      var featureActionMenu = Vue.reactive({ showPlain: false, showCase: false, showStatusAudit: false });
      Vue.watchEffect(() => { featureActionMenu.interaction = interaction; });
      Vue.watchEffect(() => { featureActionMenu.showDelete = iservice.HaveRight('Tab.MessageQueue.Delete'); });
      featureActionMenu.Click = function () {
        iservice.components.popupHolder.Open(actionMenuRef, 'interaction-view-action-menu', false, featureActionMenu);
      }
      featureActionMenu.Delete = function (e) {
        if (featureActionMenu.deleting) return;
        featureActionMenu.errors = featureActionMenu.success = null;
        featureActionMenu.deleting = true;
        iservice.API('webapp-api-history?api=delete', e, { interactionID: props.itemid }).then(data => {
          interactions.deleted(props.itemid);
          iservice.SuccessMessage('Interaction deleted successfully.');
        }).catch(errors => {
          featureActionMenu.errors = errors;
        }).finally(() => {
          featureActionMenu.deleting = true;
        });
      }
      var BodyTabs = Vue.reactive({ tab: 'body' });

      BodyTabs.TabClass = function (tab) { return { 'active': tab == BodyTabs.tab } }
      BodyTabs.PaneClass = function (tab) { return { 'show active': tab == BodyTabs.tab }; }
      BodyTabs.TabClick = function (tab) {
        BodyTabs.tab = tab;
        if (tab == 'preview') {
          BodyTabs.loadingPreview = true;
          var submitData = {}
          submitData = $.extend(submitData, answer);

          submitData.threadProperties = featureCaseProps.out;
          submitData.messageProperties = featureQuestionProps.out;
          submitData.answerProperties = featureAnswerProps.out;

          iservice.API('webapp-api-mq-myqueue?api=savedraft', null, { interactionIDs: [props.itemid], answer: submitData, interactionID: props.itemid }, featureUploads.files).then(function (data) {
            iservice.SanitizeInlineImageAttachments(data.details, 'responseFinalHtml', data.response.details.attachments);
            BodyTabs.finalHtml = data.details.responseFinalHtml;
            BodyTabs.loadingPreview = false;
          }).catch(function (errors) {
            formFeatures.errors = errors;
            BodyTabs.loadingPreview = false;
          });
        }
      }
      function SaveProperties() {
        var submitData = {}
        submitData = $.extend(submitData, answer);

        submitData.threadProperties = featureCaseProps.out;
        submitData.messageProperties = featureQuestionProps.out;
        submitData.answerProperties = featureAnswerProps.out;

        iservice.API('webapp-api-mq-myqueue?api=savedraft', null, { interactionIDs: [props.itemid], answer: submitData, interactionID: props.itemid }, featureUploads.files).then(function (data) {
          iservice.SuccessMessage('Interaction updated successfully.');
        }).catch(function (errors) {
          formFeatures.errors = errors;
        });
      }
      return { answer, Cancel, questionDetails, bodyHtmlEdit, featureCC, inputBCC, inputCC, editor, addressFromSelect, featureAction, featureCaseProps, featureQuestionProps, featureAnswerProps, actionButtons, featureTabs, featureAddresses, fromMailboxList, featureStockResponses, featureUploads, anchorStocks, questionAttachments, haveEmailDownload, answerAttachments, AttachmentRemove, isQuote, quotetoggle, actionMenuRef, featureActionMenu, interaction, BodyTabs, SaveProperties};
    }
  };
  iservice.components.answerButtonStandardSend = {
    name: 'answer-button-standard-send',
    template: '#answer-button-standard-send',
    answerAction(interactionRef, answer, featureAction) {
      var featureSend = Vue.reactive({ isDisabled: false, addSubjectRef: true, interactionRef });
      featureSend.Visible = function () { return iservice.HaveRight('Tab.MessageQueue.Send'); }
      Vue.watchEffect(() => { featureSend.label = (interactionRef?.value?.status == 'Pending') ? 'Approve' : 'Send' });
      Vue.watchEffect(() => { answer.supressSubjectRef = !featureSend.addSubjectRef; });
      Vue.watchEffect(() => { answer.expectReply = featureSend.expectReply; });
      Vue.watchEffect(() => { answer.leaveOpen = featureSend.leaveOpen; });
      Vue.watchEffect(() => { answer.notificationID = featureSend.notificationID; });
      var anchorRef = {};
      featureSend.Open = function (anchor) {
        anchorRef.value = anchor;
        featureSend.Close = iservice.components.popupHolder.Open(anchorRef, 'answer-button-standard-send-menu', true, featureSend);
      }
      featureSend.Send = function (e) {
        if (featureSend.Close) featureSend.Close();
        if (!featureAction.CheckAnswerProperties()) return;
        if (!featureAction.CheckQuestionProperties()) return;
        if (!featureAction.CheckCaseProperties()) return;
        if (!answer.leaveOpen) featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=send', e, 'Answer sent.', true, true);
        else featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=sendleaveopen', e, 'Email sent.', true);
      }
      featureSend.SendSecure = function (e) {
        if (featureSend.Close) featureSend.Close();
        if (!featureAction.CheckAnswerProperties()) return;
        if (!featureAction.CheckQuestionProperties()) return;
        if (!featureAction.CheckCaseProperties()) return;
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=send', e, 'Secure answer sent.', true, true);
      }
      return featureSend;
    },
    props: ['feature'],
    setup(props) {
      var anchorSend = Vue.ref(null);

      return { anchorSend }
    }
  }
  iservice.components.answerButtonStandardSendMenu = {
    name: 'answer-button-standard-send-menu',
    template: '#answer-button-standard-send-menu',
    props: ['params'],
    setup(props) {
      var autoResponses = iservice.AutoResponseData();
      var feature = Vue.computed(() => { return props.params; });
      var showRefrenceMenu = iservice.HaveRight('Tab.MessageQueue.SuppressRef') ? iservice.HaveRight('Tab.MessageQueue.SuppressRef') : false;

      var secureStage = Vue.ref('closed');
      var secureOptions = Vue.ref([]);
      Vue.watchEffect(() => {
        secureOptions.value.length = 0;
        secureOptions.loading = true;
        if (!autoResponses.loaded) {
          autoResponses.Load();
          return;
        }
        secureOptions.loading = false;
        if (feature.value.interactionRef) autoResponses.inSegment(feature.value.interactionRef.segmentID).forEach(response => {
          if (!response.listNames.includes('securemessagenotification')) return;
          var option = Vue.reactive({ id: response.id, name: response.name });
          secureOptions.value.push(option);
        });
        if (secureStage.value == 'open' && secureOptions.value.length < 1) {
          secureStage.value = 'none';
        }
      });
      function SecureStart() {
        if (secureStage.value != 'closed') {
          secureStage.value = 'closed';
        }
        else if (secureStage.value == 'closed') {
          secureStage.value = 'open';
          if (secureOptions.value.length) {
            feature.value.notificationID = secureOptions.value[0].id;
          }
        }
      }

      return { feature, SecureStart, secureStage, secureOptions, showRefrenceMenu };
    }
  };
  iservice.components.answerButtonStandardSubmit = {
    name: 'answer-button-standard-submit',
    template: '#answer-button-standard-submit',
    answerAction(interactionRef, answer, featureAction) {
      var featureSubmit = Vue.reactive({ isDisabled: false });
      featureSubmit.Visible = function () { return !iservice.HaveRight('Tab.MessageQueue.Send'); }
      featureSubmit.Send = function (e) {
        if (!featureAction.CheckAnswerProperties()) return;
        if (!featureAction.CheckQuestionProperties()) return;
        if (!featureAction.CheckCaseProperties()) return;
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=send', e, 'Answer submitted.', true, true);
      }
      return featureSubmit;
    },
    props: ['feature'],
    setup(props) {

      return { }
    }
  }
  iservice.components.answerButtonStandardReject = {
    name: 'answer-button-standard-reject',
    template: '#answer-button-standard-reject',
    answerAction(interactionRef, answer, featureAction) {
      var featureReject = Vue.reactive({ isDisabled: false });
      featureReject.Visible = function () { return interactionRef?.value?.status == 'Pending' && iservice.HaveRight('Tab.MessageQueue.Approve'); }
      featureReject.Reject = function (e) {
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=qareject', e, 'Answer rejected.', true);
      }
      return featureReject;
    },
    props: ['feature'],
    setup(props) {

      return { }
    }
  }
  iservice.components.answerButtonStandardResolve = {
    name: 'answer-button-standard-resolve',
    template: '#answer-button-standard-resolve',
    answerAction(interactionRef, answer, featureAction) {
      var featureResolve = Vue.reactive({ isDisabled: false });
      featureResolve.Visible = function () { return iservice.HaveRight('Tab.MessageQueue.Send'); }
      featureResolve.Resolve = function (e) {
        if (!featureAction.CheckAnswerProperties()) return;
        if (!featureAction.CheckQuestionProperties()) return;
        if (!featureAction.CheckCaseProperties()) return;
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=saveresolve', e, 'Resolved.', true, true);
      }
      return featureResolve;
    },
    props: ['feature'],
    setup(props) {

      return { }
    }
  }
  iservice.components.answerButtonStandardForward = {
    name: 'answer-button-standard-forward',
    template: '#answer-button-standard-forward',
    answerAction(interactionRef, answer, featureAction) {
      var featureForward = Vue.reactive({ isDisabled: false, canAssignSelf: false, interactionRef, onLeft: true });
      featureForward.Visible = function () { return true; }
      Vue.watchEffect(() => { answer.reassignAgentID = featureForward.reassignAgentID; });
      Vue.watchEffect(() => { answer.notificationID = featureForward.notificationID; });
      Vue.watchEffect(() => { answer.forwardExternalAddress = featureForward.forwardExternalAddress; });
      Vue.watchEffect(() => { answer.forwardMailboxID = featureForward.forwardMailboxID; });
      var anchorRef = {};
      featureForward.Open = function (anchor) {
        anchorRef.value = anchor;
        featureForward.Close = iservice.components.popupHolder.Open(anchorRef, 'answer-button-standard-forward-menu', featureForward.onLeft, featureForward);
      }
      featureForward.ForwardExternal = function (e) {
        if (featureForward.Close) featureForward.Close();
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=forwardexternal', e, 'Reassigned to external agent.', false, true);
      }
      featureForward.ForwardSegment = function (e) {
        if (featureForward.Close) featureForward.Close();
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=changesegment', e, 'Forwarded.', true, true);
      }
      featureForward.ReassignAgent = function (e) {
        if (featureForward.Close) featureForward.Close();
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=assign', e, 'Reassigned.', false, true);
      }
      featureForward.UnassignQuestion = function (e) {
        if (featureForward.Close) featureForward.Close();
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=unqueue', e, 'Unassigned.', false, true);
      }
      return featureForward;
    },
    props: ['feature'],
    setup(props) {
      var anchorForward = Vue.ref(null);

      return { anchorForward }
    }
  }
  iservice.components.answerButtonStandardForwardMenu = {
    name: 'answer-button-standard-forward-menu',
    template: '#answer-button-standard-forward-menu',
    props: ['params'],
    setup(props) {
      var loggedIn = iservice.LoggedInData();
      var autoResponses = iservice.AutoResponseData();
      var mailboxes = iservice.MailboxData();
      var agents = iservice.AgentData();
      var segments = iservice.SegmentData();
      var feature = Vue.computed(() => { return props.params; });

      var externalStage = Vue.ref('closed');
      var externalOptions = Vue.ref([]);
      Vue.watchEffect(() => {
        externalOptions.value.length = 0;
        externalOptions.loading = true;
        if (!autoResponses.loaded) {
          autoResponses.Load();
          return;
        }
        externalOptions.loading = false;
        externalOptions.value = [{ id: "", name: "-- Select A Notification --" }];
        if (feature.value.interactionRef) autoResponses.inSegment(feature.value.interactionRef.segmentID).forEach(response => {
          if (!response.forExternal) return;
          var option = Vue.reactive({ id: response.id, name: response.name });
          externalOptions.value.push(option);
        });
        if (externalStage.value == 'open' && externalOptions.value.length < 1) {
          externalStage.value = 'none';
        }
      });
      function ExternalStart() {
        if (externalStage.value != 'closed') {
          externalStage.value = 'closed';
        }
        else if (externalStage.value == 'closed') {
          setTimeout(function () {
            $('.dropdown-forward.show').find('input.multiselect-search').focus();
          })
          externalStage.value = 'open';
          reassignStage.value = 'closed';
          segmentStage.value = 'closed';
          if (externalOptions.value.length) {
            //feature.value.notificationID = externalOptions.value[0].id;
            feature.value.notificationID = "";
          }
        }
      }

      var reassignStage = Vue.ref('closed');
      var reassignOptions = Vue.ref([]);
      Vue.watchEffect(() => {
        reassignOptions.value.length = 0;
        reassignOptions.loading = true;
        if (!agents.loaded) {
          agents.Load(); return;
        }
        if (!feature.value.interactionRef) return;
        reassignOptions.loading = false;
        var isSupervisor = iservice.HaveRight('Tab.MessageQueue.Supervise');
        var agentIDs = isSupervisor ? agents.forSegment(feature.value.interactionRef.segmentID) : agents.forInteraction(feature.value.interactionRef.id);
        agentIDs.forEach(agentID => {
          if (!feature.value.canAssignSelf && agentID == loggedIn.contactID) return;
          var agent = agents.get(agentID);
          var name = agent.name || agent.email || ('Agent ' + agent.id);
          var option = {
            id: agentID,
            name: name,
            email: agent.email,
            search: agent.name + '\n' + agent.email,
            disabled: !agent.isLoggedIn && agent.doEmpty,
            isavailable: (agent.isLoggedIn || isSupervisor) && !agent.doEmpty,
            isaway: !agent.isLoggedIn && !agent.doEmpty
          };
          reassignOptions.value.push(option);
        });
        reassignOptions.value.sort((l, r) => {
          return l.name.localeCompare(r.name);
        });
        if (reassignStage.value == 'open' && reassignOptions.value.length < 1) {
          reassignStage.value = 'none';
        }
      });
      function ReassignStart() {
        feature.value.reassignAgentID = '';
        if (reassignStage.value == 'open') {
          reassignStage.value = 'closed';
        }
        else if (reassignStage.value == 'closed') {
          setTimeout(function () {
            $('.dropdown-forward.show').find('input.multiselect-search').focus();
          })
          reassignStage.value = 'open';
          externalStage.value = 'closed';
          segmentStage.value = 'closed';
        }
      }

      var segmentStage = Vue.ref('closed');
      var segmentOptions = Vue.ref([]);
      Vue.watchEffect(() => {
        segmentOptions.value.length = 0;
        segmentOptions.loading = true;
        if (!mailboxes.loaded || !segments.loaded) {
          mailboxes.Load(); segments.Load(); return;
        }
        segmentOptions.loading = false;
        segments.list.forEach(seg => {
          var group = { id: seg.id, label: seg.name, options: [] };
          mailboxes.inSegment(seg.id).forEach(mailbox => {
            var option = {
              value: mailbox.id,
              name: mailbox.name,
              email: mailbox.replyTo,
              search: mailbox.name + '\n' + mailbox.replyTo
            };
            group.options.push(option);
          });
          if (group.options.length < 1) return;
          group.options.sort((l, r) => { return mailboxes.SortCompare(l.value, r.value); });
          segmentOptions.value.push(group);
        });
        segmentOptions.value.sort((l, r) => { return segments.SortCompare(l.id, r.id); });
      });
      function SegmentStart() {
        if (segmentStage.value == 'open') {
          segmentStage.value = 'closed';
        }
        else if (segmentStage.value == 'closed') {
          setTimeout(function () {
            $('.dropdown-forward.show').find('input.multiselect-search').focus();
          })
          segmentStage.value = 'open';
          reassignStage.value = 'closed';
          externalStage.value = 'closed';
        }
      }

      return { feature, ExternalStart, externalStage, ReassignStart, externalOptions, reassignStage, reassignOptions, SegmentStart, segmentStage, segmentOptions };
    }
  };
  iservice.components.answerButtonStandardSave = {
    name: 'answer-button-standard-save',
    template: '#answer-button-standard-save',
    answerAction(interactionRef, answer, featureAction) {
      var featureSave = Vue.reactive({ isDisabled: false });
      featureSave.Visible = function () { return true; }
      var anchorRef = {};
      featureSave.Open = function (anchor) {
        anchorRef.value = anchor;
        featureSave.Close = iservice.components.popupHolder.Open(anchorRef, 'answer-button-standard-save-menu', true, featureSave);
      }
      featureSave.SaveDraft = function (e) {
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=savedraft', e, 'Draft saved', false);
      }
      featureSave.PrivateNote = function (e) {
        if (featureSave.Close) featureSave.Close();
        if (!featureAction.CheckAnswerProperties()) return;
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=savenoteprivate', e, 'Note saved', true);
      }
      featureSave.PublicNote = function (e) {
        if (featureSave.Close) featureSave.Close();
        if (!featureAction.CheckAnswerProperties()) return;
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=savenotepublic', e, 'Note saved', true);
      }
      return featureSave;
    },
    props: ['feature'],
    setup(props) {
      var anchorSave = Vue.ref(null);

      return { anchorSave }
    }
  }
  iservice.components.answerButtonStandardSaveMenu = {
    name: 'answer-button-standard-save-menu',
    template: '#answer-button-standard-save-menu',
    props: ['params'],
    setup(props) {
      var feature = Vue.computed(() => { return props.params; });

      return { feature };
    }
  };
  iservice.components.answerButtonStandardTopic = {
    name: 'answer-button-standard-topic',
    template: '#answer-button-standard-topic',
    answerAction(interactionRef, answer, featureAction) {
      var segments = iservice.SegmentData();
      var topics = iservice.TopicData();

      var featureTopic = Vue.reactive({ isDisabled: false, options: [], topicID: '', onLeft: true });
      featureTopic.topicID = answer.topicID;
      featureTopic.Visible = function () { return true; }
      Vue.watchEffect(() => { answer.changeTopicID = featureTopic.topicID; });
      var anchorRef = {};
      featureTopic.Open = function (anchor) {
        anchorRef.value = anchor;
        setTimeout(function () {
          $('.dropdown-forward.show').find('input.multiselect-search').focus();
        })
        featureTopic.Close = iservice.components.popupHolder.Open(anchorRef, 'answer-button-standard-topic-menu', featureTopic.onLeft, featureTopic);
      }
      Vue.watchEffect(() => {
        featureTopic.options.length = 0;
        featureTopic.loading = true;
        if (!segments.loaded || !topics.loaded) {
          segments.Load(); topics.Load();
          return;
        }
        featureTopic.loading = false;
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            group.options.push({ name: topic.name, value: topic.id, level: Math.min(2, topic.depth) });
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.value, r.value); });
          if (group.options.length < 1) return;
          featureTopic.options.push(group);
        });
      });
      featureTopic.Select = function () {
        if (featureTopic.Close) featureTopic.Close();
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=changetopic', featureTopic.anchor, 'Topic changed', false);
      }
      featureTopic.Reset = function () {
        featureTopic.topicID = '';
      }
      return featureTopic;
    },
    props: ['feature'],
    setup(props) {
      var anchorTopic = Vue.ref(null);
      Vue.watchEffect(() => {
        props.feature.anchor = anchorTopic.value;
      });

      return { anchorTopic }
    }
  }
  iservice.components.answerButtonStandardTopicMenu = {
    name: 'answer-button-standard-topic-menu',
    template: '#answer-button-standard-topic-menu',
    props: ['params'],
    setup(props) {
      var feature = Vue.computed(() => { return props.params; });
      function Select() {
        setTimeout(() => {
          feature.value.Select();
        });
      }
      var select = Vue.ref(null);
      Vue.watchEffect(() => {
        if (!select.value) return;
        select.value.open();
      })

      return { feature, Select, select };
    }
  };
  iservice.components.answerButtonStandardSpam = {
    name: 'answer-button-standard-spam',
    template: '#answer-button-standard-spam',
    answerAction(interactionRef, answer, featureAction) {
      var featureSpam = Vue.reactive({ isDisabled: false });
      featureSpam.Visible = function () { return true; }
      featureSpam.MarkSpam = function (e) {
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=markspam', e, 'Resolved as spam.', true, true);
      }
      return featureSpam;
    },
    props: ['feature'],
    setup(props) {

      return {}
    }
  }
  iservice.components.answerButtonStandardPublicNote = {
    name: 'answer-button-standard-publicnote',
    template: '#answer-button-standard-publicnote',
    answerAction(interactionRef, answer, featureAction) {
      var featurePublicNote = Vue.reactive({ isDisabled: false });
      featurePublicNote.Visible = function () { return true; }
      featurePublicNote.PublicNote = function (e) {
        if (!featureAction.CheckAnswerProperties()) return;
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=savenotepublic', e, 'Note saved', true);
      }
      return featurePublicNote;
    },
    props: ['feature'],
    setup(props) {

      return {}
    }
  }
  iservice.components.answerButtonStandardPrivateNote = {
    name: 'answer-button-standard-privatenote',
    template: '#answer-button-standard-privatenote',
    answerAction(interactionRef, answer, featureAction) {
      var featurePrivateNote = Vue.reactive({ isDisabled: false });
      featurePrivateNote.Visible = function () { return true; }
      featurePrivateNote.PrivateNote = function (e) {
        if (!featureAction.CheckAnswerProperties()) return;
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=savenoteprivate', e, 'Note saved', true);
      }
      return featurePrivateNote;
    },
    props: ['feature'],
    setup(props) {

      return {}
    }
  }
  iservice.components.answerButtonStandardSaveDraft = {
    name: 'answer-button-standard-savedraft',
    template: '#answer-button-standard-savedraft',
    answerAction(interactionRef, answer, featureAction) {
      var featureSaveDraft = Vue.reactive({ isDisabled: false });
      featureSaveDraft.Visible = function () { return true; }
      featureSaveDraft.SaveDraft = function (e) {
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=savedraft', e, 'Draft saved', false);
      }
      return featureSaveDraft;
    },
    props: ['feature'],
    setup(props) {

      return {}
    }
  }
  iservice.components.answerButtonPortalSend = {
    name: 'answer-button-portal-send',
    template: '#answer-button-portal-send',
    answerAction(interactionRef, answer, featureAction) {
      var featureSend = Vue.reactive({ isDisabled: false });
      featureSend.Visible = function () { return true; }
      featureSend.Resolve = function (e) {
        if (!featureAction.CheckAnswerProperties()) return;
        if (!featureAction.CheckQuestionProperties()) return;
        if (!featureAction.CheckCaseProperties()) return;
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=saveresolve', e, 'Resolved.', true);
      }
      return featureSend;
    },
    props: ['feature'],
    setup(props) {
      var anchorSend = Vue.ref(null);

      return { anchorSend }
    }
  }
  iservice.components.answerButtonSecurePortalSend = {
    name: 'answer-button-secure-portal-send',
    template: '#answer-button-secure-portal-send',
    answerAction(interactionRef, answer, featureAction) {
      var featureSend = Vue.reactive({ isDisabled: false, addSubjectRef: true, interactionRef });
      featureSend.Visible = function () { return iservice.HaveRight('Tab.MessageQueue.Send'); }
      Vue.watchEffect(() => { featureSend.label = (interactionRef?.value?.status == 'Pending') ? 'Approve' : 'Send' });
      Vue.watchEffect(() => { answer.securePanelleaveOpen = featureSend.securePanelleaveOpen; });
      var anchorRef = {};
      featureSend.Open = function (anchor) {
        anchorRef.value = anchor;
        featureSend.Close = iservice.components.popupHolder.Open(anchorRef, 'answer-button-secure-portal-send-menu', true, featureSend);
      }
      featureSend.Send = function (e) {
        if (featureSend.Close) featureSend.Close();
        if (!featureAction.CheckAnswerProperties()) return;
        if (!featureAction.CheckQuestionProperties()) return;
        if (!featureAction.CheckCaseProperties()) return;
        if (!answer.securePanelleaveOpen) {
          featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=saveresolve', e, 'Question Resolved.', 'securePortalSendResolve', true);
        }
        else {
          featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=savenote', e, 'Note Saved.', 'securePortalSaveNote', false);
          featureSend.securePanelleaveOpen = false;
        }
      }
      return featureSend;
    },
    props: ['feature'],
    setup(props) {
      var anchorSend = Vue.ref(null);
      return { anchorSend }
    }
  }
  iservice.components.answerButtonSecureNotificationSend = {
    name: 'answer-button-secure-notification-send',
    template: '#answer-button-secure-notification-send',
    answerAction(interactionRef, answer, featureAction) {
      var featureSend = Vue.reactive({ isDisabled: false, addSubjectRef: true, interactionRef });
      featureSend.Visible = function () { return iservice.HaveRight('Tab.MessageQueue.Send'); }
      Vue.watchEffect(() => { featureSend.label = (interactionRef?.value?.status == 'Pending') ? 'Approve' : 'Send' });
      Vue.watchEffect(() => { answer.leaveOpen = featureSend.leaveOpen; });
      var anchorRef = {};
      featureSend.Open = function (anchor) {
        anchorRef.value = anchor;
        featureSend.Close = iservice.components.popupHolder.Open(anchorRef, 'answer-button-secure-notification-send-menu', true, featureSend);
      }
      var autoResponses = iservice.AutoResponseData();
      if (!autoResponses.loaded) {
        autoResponses.Load();
      }
      if (interactionRef) autoResponses.inSegment(interactionRef.segmentID).forEach(response => {
        if (!response.listNames.includes('securemessagenotification')) return;
        answer.notificationID = response.id;
      });
      featureSend.Send = function (e) {
        if (featureSend.Close) featureSend.Close();
        if (!featureAction.CheckAnswerProperties()) return;
        if (!featureAction.CheckQuestionProperties()) return;
        if (!featureAction.CheckCaseProperties()) return;
        answer.isPublic = true;
        if (!answer.leaveOpen) {
          featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=send', e, 'Secure Answer Notification Sent.', 'SendNotification', true);
        }
        else {
          featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=send', e, 'Secure Email Sent.', 'SecureEmail', false);
          featureSend.leaveOpen = false;
        }
      }
      return featureSend;
    },
    props: ['feature'],
    setup(props) {
      var anchorSend = Vue.ref(null);
      return { anchorSend }
    }
  }

  iservice.components.answerButtonSecurePortalResolve = {
    name: 'answer-button-secure-portal-resolve',
    template: '#answer-button-secure-portal-resolve',
    answerAction(interactionRef, answer, featureAction) {
      var featureResolve = Vue.reactive({ isDisabled: false });
      featureResolve.Visible = function () { return iservice.HaveRight('Tab.MessageQueue.Send'); }
      featureResolve.Resolve = function (e) {
        if (!featureAction.CheckAnswerProperties()) return;
        if (!featureAction.CheckQuestionProperties()) return;
        if (!featureAction.CheckCaseProperties()) return;
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=saveresolve', e, 'Question Resolved.', 'securePortalResolve', true,);
      }
      return featureResolve;
    },
    props: ['feature'],
    setup(props) {

      return {}
    }
  }
  iservice.components.answerButtonSecureNotificationResolve = {
    name: 'answer-button-secure-notification-resolve',
    template: '#answer-button-secure-notification-resolve',
    answerAction(interactionRef, answer, featureAction) {
      var featureResolve = Vue.reactive({ isDisabled: false });
      featureResolve.Visible = function () { return iservice.HaveRight('Tab.MessageQueue.Send'); }
      featureResolve.Resolve = function (e) {
        if (!featureAction.CheckAnswerProperties()) return;
        if (!featureAction.CheckQuestionProperties()) return;
        if (!featureAction.CheckCaseProperties()) return;
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=saveresolve', e, 'Question Resolved..', 'secureNotificationResolve', true,);
      }
      return featureResolve;
    },
    props: ['feature'],
    setup(props) {

      return {}
    }
  }
  iservice.components.answerButtonSecureSendMenu = {
    name: 'answer-button-secure-portal-send-menu',
    template: '#answer-button-secure-portal-send-menu',
    props: ['params'],
    setup(props) {
      var autoResponses = iservice.AutoResponseData();
      var feature = Vue.computed(() => { return props.params; });
      var showRefrenceMenu = iservice.HaveRight('Tab.MessageQueue.SuppressRef') ? iservice.HaveRight('Tab.MessageQueue.SuppressRef') : false;

      return { feature, showRefrenceMenu };
    }
  };
  iservice.components.answerButtonSecureNotificationSendMenu = {
    name: 'answer-button-secure-notification-send-menu',
    template: '#answer-button-secure-notification-send-menu',
    props: ['params'],
    setup(props) {
      var autoResponses = iservice.AutoResponseData();
      var feature = Vue.computed(() => { return props.params; });
      var showRefrenceMenu = iservice.HaveRight('Tab.MessageQueue.SuppressRef') ? iservice.HaveRight('Tab.MessageQueue.SuppressRef') : false;

      return { feature, showRefrenceMenu };
    }
  };
  iservice.components.answerButtonPortalForward = {
    name: 'answer-button-portal-forward',
    template: '#answer-button-portal-forward',
    answerAction(interactionRef, answer, featureAction) {
      var featureForward = Vue.reactive({ isDisabled: false, interactionRef });
      featureForward.Visible = function () { return true; }
      Vue.watchEffect(() => { answer.reassignAgentID = featureForward.reassignAgentID; });
      Vue.watchEffect(() => { answer.notificationID = featureForward.notificationID; });
      Vue.watchEffect(() => { answer.forwardMailboxID = featureForward.forwardMailboxID; });
      var anchorRef = {};
      featureForward.Open = function (anchor) {
        anchorRef.value = anchor;
        featureForward.Close = iservice.components.popupHolder.Open(anchorRef, 'answer-button-standard-forward-menu', true, featureForward);
      }
      featureForward.ForwardSegment = function (e) {
        if (featureForward.Close) featureForward.Close();
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=changesegment', e, 'Segment changed.', true, true);
      }
      featureForward.ReassignAgent = function (e) {
        if (featureForward.Close) featureForward.Close();
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=assign', e, 'Assignment changed.', false, true);
      }
      featureForward.UnassignQuestion = function (e) {
        if (featureForward.Close) featureForward.Close();
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=unqueue', e, 'Unassigned.', false, true);
      }
      return featureForward;
    },
    props: ['feature'],
    setup(props) {
      var anchorForward = Vue.ref(null);

      return { anchorForward }
    }
  }

  iservice.components.agentEmailMenuSend = {
    name: 'agent-email-menu-send',
    template: '#agent-email-menu-send',
    props: ['params'],
    setup(props) {
      var feature = Vue.computed(() => { return props.params; });
      var showRefrenceMenu = iservice.HaveRight('Tab.MessageQueue.SuppressRef') ? iservice.HaveRight('Tab.MessageQueue.SuppressRef') : false;
      var secureStage = Vue.ref('closed');
      if (feature.value.secureOptions.length)
        feature.value.notificationID = feature.value.secureOptions[0].id;
      function SecureStart() {
        if (secureStage.value != 'closed') {
          secureStage.value = 'closed';
        }
        else if (secureStage.value == 'closed') {
          secureStage.value = 'open';
          if (feature.value.secureOptions.length) {
            feature.value.notificationID = feature.value.secureOptions[0].id;
          }
        }
      }
      return { feature, showRefrenceMenu, secureStage, SecureStart};
    }
  };
  iservice.components.addressAutocomplete = {
    name: 'address-autocomplete',
    template: '#address-autocomplete',
    props: ['params'],
    setup(props) {
      var feature = Vue.computed(() => { return props.params; });
      var autocomplete = iservice.AutocompleteData();
      //var ours = autocomplete[];

      var width = Vue.computed(() => {
        if (feature.value.anchor) {
          return feature.value.anchor.clientWidth + 'px';
        }
        return '100px';
      });

      var items = Vue.computed(() => {
        var items = [];
        items.push('Item 1');
        items.push('Item 2');
        items.push('Item 3');
        return items;
      });

      return { items, width };
    }
  }
  iservice.components.stockResponsesMenu = {
    name: 'stock-responses-menu',
    template: '#stock-responses-menu',
    props: ['params'],
    setup(props) {
      var allVariables = iservice.VariablePickerData();
      var allStocks = iservice.StockResponsesData();
      var feature = Vue.computed(() => { return props.params; });

      var notificationTemplates = iservice.NotificationTemplates;
    
      var templatesData = Vue.reactive([]);
      notificationTemplates.forEach(function (template) {
        if (template['agentnotification'])
          templatesData.push({ label: "Agent Notification", templates: template['agentnotification'] });
        if (template['securenotification'])
          templatesData.push({ label: "Secure Notification", templates: template['securenotification'] });
        if (template['kbarticlenotification'])
          templatesData.push({ label: "KB Article Notification", templates: template['kbarticlenotification'] });
        if (template['passwordresetnotification'])
          templatesData.push({ label: "Password Reset Notification", templates: template['passwordresetnotification'] });
        if (template['accountcreation'])
          templatesData.push({ label: "Contact Creation", templates: template['accountcreation'] });
        if (template['topicautoresponse'])
          templatesData.push({ label: "Message Acknowledgement", templates: template['topicautoresponse'] });
        if (template['alert'])
          templatesData.push({ label: "Alert", templates: template['alert'] });
      });
      if (!feature.value.tab) feature.value.tab = 'stocks';
      var featureTab = Vue.reactive({ tab: 'stocks' });
      featureTab.TabClass = function (tab) {
        return tab == feature.value.tab ? 'active btn-primary' : '';
      }
      featureTab.TabClick = function (tab) {
        feature.value.tab = tab;
      }

      function PassedFilter(str) {
        if (!feature.value.filter || feature.value.filter.length < 1) return true;
        if (str.toLocaleLowerCase().indexOf(feature.value.filter.toLocaleLowerCase()) == -1) return false;
        return true;
      }
      function StockTitle(text) {
        if (!text) return '';
        if (text.length > 100) {
          return text.substring(0, 100) + '...';
        }
        return text;
      }
      var variables = Vue.computed(() => {
        if (!allVariables.loaded) allVariables.Load();
        var sections = [];
        allVariables.sections.forEach(section => {
          var filtered = {
            label: section.label,
            vars: [],
            segments: []
          };
          section.vars.forEach(v => {
            if (!PassedFilter(v.label)) return;
            filtered.vars.push(v);
          });
          section.segments.forEach(seg => {
            var segment = { label: seg.label, vars: [] };
            seg.vars.forEach(v => {
              if (!PassedFilter(v.label)) return;
              segment.vars.push(v);
            });
            if (segment.vars.length > 0) filtered.segments.push(segment);
          });
          if (filtered.vars.length > 0 || filtered.segments.length > 0) sections.push(filtered);
        });
        return sections;
      });
      var stocks = Vue.computed(() => {
        if (!allStocks.loaded) allStocks.Load();
        var segment = [];
        allStocks.inSegment(feature.value.segmentID).forEach(stock => {
          if (PassedFilter(stock.name) || PassedFilter(stock.description) || PassedFilter(stock.bodyPreview)) {
            segment.push({ id: stock.id, name: stock.name, title: StockTitle(stock.bodyPreview) });
          }
        });
        var agent = [];
        allStocks.ofMine().forEach(stock => {
          if (PassedFilter(stock.name) || PassedFilter(stock.description) || PassedFilter(stock.bodyPreview)) {
            agent.push({ id: stock.id, name: stock.name, title: StockTitle(stock.bodyPreview) });
          }
        });
        return { segment, agent };
      });
      var pasting = false;
      function PasteStock(item) {
        if (pasting) return;
        var stock = allStocks.get(item.id);
        if (!stock.bodyHtml) {
          pasting = item.working = true;
          iservice.API('webapp-api-mq-myqueue?api=stock', null, { stockID: item.id }).then(data => {
            allStocks.add(data);
            feature.value.PasteStock(stock.bodyHtml);
          }).finally(() => {
            pasting = item.working = false;
          });
        }
        else {
          feature.value.PasteStock(stock.bodyHtml);
        }
      }

      return { allVariables, allStocks, feature, featureTab, variables, stocks, PasteStock, templatesData };
    }
  };
  iservice.components.interactionTypeDisplay = {
    name: 'interaction-type-display',
    template: '#interaction-type-display',
    props: ['itemid'],
    setup(props) {
      var interactions = iservice.InteractionData();

      var label = Vue.computed(() => {
        var int = interactions.get(props.itemid);
        if (!int) return '';
        return ' ' + int.type + ' ';
      });
      var style = Vue.computed(() => {
        var int = interactions.get(props.itemid);
        if (!int) return '';
        return iservice.InteractionTypeClass(int.type);
      });
      var icon = Vue.computed(() => {
        var int = interactions.get(props.itemid);
        if (!int) return '';
        return iservice.InteractionTypeIcon(int.type);
      });
      return { label, style, icon };
    }
  }
  iservice.components.interactionStatusDisplay = {
    name: 'interaction-status-display',
    template: '#interaction-status-display',
    props: ['itemid', 'status'],
    setup(props) {
      var interactions = iservice.InteractionData();

      var label = Vue.computed(() => {
        if (props.status) return props.status;
        if (props.itemid) {
          var int = interactions.get(props.itemid);
          if (!int) return '';
          return int.status;
        }
      });
      var style = Vue.computed(() => {
        if (props.status) {
          return iservice.InteractionStatusClass(props.status);
        }
        if (props.itemid) {
          var int = interactions.get(props.itemid);
          if (!int) return '';
          return iservice.InteractionStatusClass(int.status);
        }
      });
      return { label, style };
    }
  }

})(iservice);
