﻿var iservice = window.iservice || {};
(function (iservice) {
  iservice.components = iservice.components || {};
  iservice.directives = iservice.directives || {};

  iservice.components.iserviceRoot = {
    name: 'iservice-root',
    template: '#iservice-root',
    setup(props) {
      var router = VueRouter.useRouter();
      var loggedIn = iservice.LoggedInData();
      return { loggedIn, router };
    }
  };
  iservice.components.logoutWarning = {
    name: 'logout-warning',
    template: '#logout-warning',
    setup(props) {
      var loggedIn = iservice.LoggedInData();

      var featureLogoutWarning = Vue.reactive({ loggedOut: false, countDown: false });
      featureLogoutWarning.Login = function () {
        startTime = null;
        featureLogoutWarning.loggedOut = false;
        featureLogoutWarning.countDown = false;
      }
      featureLogoutWarning.Continue = function () {
        iservice.API('webapp-api-login?api=loggedin').then(data => {
          loggedIn.Update(data);
          if (loggedIn.isLoggedIn) {
            startTime = null;
            featureLogoutWarning.loggedOut = false;
            featureLogoutWarning.countDown = false;
          }
        });
      }
      featureLogoutWarning.Logout = function () {
        if (featureLogoutWarning.working) return;
        loggedIn.localLogout = true;
        featureLogoutWarning.working = true;
        iservice.API('webapp-api-login?api=logout', null, {}).then(function (data) {
          loggedIn.Update(data.loggedIn);
          startTime = null;
          featureLogoutWarning.loggedOut = false;
          featureLogoutWarning.countDown = false;
        }).finally(function () {
          featureLogoutWarning.working = false;
        });
      }
      Vue.watchEffect(() => {
        var bodyEl = $('body')[0];
        if (featureLogoutWarning.showing) {
          bodyEl.classList.add('modal-open')
        }
        else {
          bodyEl.classList.remove('modal-open');
        }
      });

      var startTime;
      var timeLimit;
      function Countdown() {
        if (!startTime) return;
        var elapsed = moment().diff(startTime);
        var remainSecs = Math.max(Math.min(Math.trunc((timeLimit - elapsed) / 1000), 30), 0);
        if (!featureLogoutWarning.countDown && remainSecs <= 30) {
          featureLogoutWarning.countDown = true;
        }
        if (featureLogoutWarning.countDown && remainSecs > 30) {
          featureLogoutWarning.countDown = false;
        }
        if (featureLogoutWarning.countDown) {
          featureLogoutWarning.totalSecs = remainSecs;
        }

      }
      setInterval(Countdown, 100);

      function LoginUpdate(msg) {
        var message = msg.getData();
        if (message.inactivityUpdate != null) {
          timeLimit = message.inactivityUpdate.secondsRemaining * 1000;
          if (timeLimit > 30000) {
            startTime = null;
            featureLogoutWarning.countDown = false;
          }
          else {
            startTime = moment();
          }
        }
        else if (message.logoutUpdate != null) {
          if (!loggedIn.localLogout) featureLogoutWarning.loggedOut = true;
          if (loggedIn.isLoggedIn) {
            iservice.API('webapp-api-login?api=loggedin').then(data => {
              loggedIn.Update(data);
            });
          }
        }
      }

      var websyncUnsub = null;
      iservice.WatchLoginChange(() => {
        if (websyncUnsub) websyncUnsub();
        websyncUnsub = null;
        if (!emptyStr(loggedIn.contactID)) {
          websyncUnsub = websync.subscribe('/agent/' + loggedIn.tenantID + '/' + loggedIn.contactID, LoginUpdate);
        }
      });

      return { featureLogoutWarning };
    }
  };
  iservice.components.loggedinMain = {
    name: 'loggedin-main',
    template: '#loggedin-main',
    setup(props) {
      var myQueue = iservice.MyQueueData();
      var loggedIn = iservice.LoggedInData();
      var interactions = iservice.InteractionData();
      var segments = iservice.SegmentData();
      var messages = iservice.Messages();
      var router = VueRouter.useRouter();
      var plugins = iservice.Plugins();
      var topics = iservice.TopicData();

      var featureLogout = Vue.reactive({ working: false });

      var featureSidebar = Vue.reactive({ collapsed: false });
      featureSidebar.SidebarToggle = function () {
        featureSidebar.collapsed = !featureSidebar.collapsed;
      }
      Vue.watchEffect(() => {
        var bodyEl = $('body')[0];
        if (featureSidebar.collapsed) {
          bodyEl.classList.add('sidebar-collapse', 'sidebar-open');
        }
        else {
          bodyEl.classList.remove('sidebar-collapse', 'sidebar-open');
        }
        if (window.matchMedia("(max-width: 768px)").matches) {
          bodyEl.classList.remove('sidebar-collapse');
        }
      });
      

      var featureChat = Vue.reactive({ working: false, shown: false });
      Vue.watchEffect(() => { featureChat.shown = iservice.HaveRight('Chat.Answer'); });
      Vue.watchEffect(() => {
        featureChat.availableTitle = loggedIn.isAvailable ? 'Go Unavailable' : 'Go Available';
        featureChat.availableClass = loggedIn.isAvailable ? 'text-success' : 'text-danger';
        featureChat.availableText = loggedIn.isAvailable ? 'Available' : 'Unavailable';
        featureChat.takeClass = loggedIn.numChats > 0 ? 'blink' : '';
        featureChat.takeDisable = loggedIn.numChats < 1 || featureChat.working;
      });
      featureChat.Answer = function (e) {
        if (featureChat.working) return;
        featureChat.working = true;
        iservice.API('webapp-api-agentchat?api=answer', null, {}).then(function (data) {
          //loggedIn.Update(data);
        }).finally(function () {
          featureChat.working = false;
        });
      }

      featureChat.Available = function () {
        if (featureChat.working) return;
        featureChat.working = true;
        iservice.API('webapp-api-agentchat?api=toggleavail', null, {}).then(function (data) {
          loggedIn.Update(data);
        }).finally(function () {
          featureChat.working = false;
        });
      }

      var featureQA = Vue.reactive({ working: false, shown: false });
      Vue.watchEffect(() => { featureQA.shown = iservice.HaveRight('Tab.MessageQueue.Approve'); });
      Vue.watchEffect(() => { featureQA.menuclassInbox = router.currentRoute.value.path == iservice.components.messageQueueMyApprove.route().path ? 'active' : ''; });

      var featureMQ = Vue.reactive({ working: false, shown: false, open: true });
      Vue.watchEffect(() => { featureMQ.shown = iservice.HaveRight('Tab.Top.MessageQueue'); });
      Vue.watchEffect(() => { featureMQ.menuclassMain = { 'menu-open': featureMQ.open, 'menu-closed': !featureMQ.open }; });
      Vue.watchEffect(() => { featureMQ.menuStyleMain = { 'display': featureMQ.open ? 'block' : 'none' }; });
      Vue.watchEffect(() => { featureMQ.menuclassSearch = router.currentRoute.value.path == iservice.components.messageQueueSearch.route().path ? 'active' : ''; });
      Vue.watchEffect(() => { featureMQ.menuclassInbox = router.currentRoute.value.path == iservice.components.messageQueueMyInbox.route().path ? 'active' : ''; });
      Vue.watchEffect(() => { featureMQ.menuclassPending = router.currentRoute.value.path == iservice.components.messageQueueMyPending.route().path ? 'active' : '';});
      Vue.watchEffect(() => { featureMQ.countMain = myQueue.subbox.messageQueueMyInbox ? myQueue.subbox.messageQueueMyInbox.length : 0; })
      Vue.watchEffect(() => { featureMQ.countPending = myQueue.subbox.messageQueueMyPending ? myQueue.subbox.messageQueueMyPending.length : 0; })
      Vue.watchEffect(() => { featureMQ.countApprove = myQueue.subbox.messageQueueMyApprove ? myQueue.subbox.messageQueueMyApprove.length : 0; })
      featureMQ.Menu = function () { featureMQ.open = !featureMQ.open; }
      Vue.watchEffect(() => {
        var path = iservice.components.messageQueueMyInbox.route().path;
        featureMQ.classMyInbox = router.currentRoute.value.path == path ? 'active' : '';
      });

      var featureUserMenu = Vue.reactive({ working: false, shown: true, open: false });
      //Vue.watchEffect(() => { featureUserMenu.shown = iservice.HaveRight('Tab.Top.CustomerInfo'); });
      Vue.watchEffect(() => { featureUserMenu.menuclassMain = { 'show': featureUserMenu.open }; });
      featureUserMenu.Menu = function () { featureUserMenu.open = !featureUserMenu.open; }

      var featureProfile = Vue.reactive({ working: false, shown: false });
      Vue.watchEffect(() => { featureProfile.shown = iservice.HaveRight('Tab.Top.CustomerInfo'); });

      var featureChangePassword = Vue.reactive({ working: false, open: false });
      featureChangePassword.Menu = function () {
        featureChangePassword.open = !featureChangePassword.open;
        if (!featureChangePassword.open) return;
        featureChangePassword.errors = null;
        featureChangePassword.success = false;
        featureChangePassword.newPassword = featureChangePassword.confirmPassword = '';
        setTimeout(() => {
          if (newPasswordEl.value) newPasswordEl.value.focus();
        });
      }
      featureChangePassword.Submit = function (e) {
        if (featureChangePassword.working == true) return;
        featureChangePassword.errors = null;
        featureChangePassword.success = false;
        if (featureChangePassword.newPassword != featureChangePassword.confirmPassword) {
          featureChangePassword.errors = ['Confirm password did not match.'];
          return;
        }
        var properties = { properties: [{ propertyID: 'password', values: [{ value: featureChangePassword.newPassword }] }] };
        featureChangePassword.working = true;
        iservice.API('webapp-api-findanswer?api=mydetailsset', e, { details: properties }).then(function (data) {
          iservice.SuccessMessage('Password changed successfully.');
          featureChangePassword.open = false;
        }).catch(function (errors) {
          featureChangePassword.errors = errors;
        }).finally(function () {
          featureChangePassword.working = false;
        });
      }
      var newPasswordEl = Vue.ref(null);

      var featureCustinfo = Vue.reactive({ working: false, shown: false, search: '', open: true });
      Vue.watchEffect(() => { featureCustinfo.shown = iservice.HaveRight('Tab.Top.CustomerInfo'); });
      Vue.watchEffect(() => { featureCustinfo.menuclassMain = { 'menu-open': featureCustinfo.open, 'menu-closed': !featureCustinfo.open }; });
      Vue.watchEffect(() => { featureCustinfo.menuStyleMain = { 'display': featureCustinfo.open ? 'block' : 'none' }; });
      Vue.watchEffect(() => { featureCustinfo.menuclassSubNew = router.currentRoute.value.path == iservice.components.customerContactDetails.route().path ? 'active' : ''; });
      Vue.watchEffect(() => { featureCustinfo.menuclassSubSearch = router.currentRoute.value.path == iservice.components.customerContactSearch.route().path ? 'active' : ''; });
      featureCustinfo.Menu = function () { featureCustinfo.open = !featureCustinfo.open; }
      featureCustinfo.Search = function () {
        if (!featureCustinfo.search) return;
        window.location.href = iservice.formPath + '/search-contacts?w1=anywhere&f1=login&t1=' + featureCustinfo.search;
      }
      var featureQuotes = Vue.reactive({ working: false, shown: false, installed: false });
      Vue.watchEffect(() => { featureQuotes.countQuotes = myQueue.subbox.messageQueueOpenQuotes ? myQueue.subbox.messageQueueOpenQuotes.length : 0; })
      Vue.watchEffect(() => {
        if (iservice.config.quoteConfig && iservice.config.quoteConfig.topics.length > 0) {
          featureQuotes.installed = true;
        }
        featureQuotes.shown = iservice.HaveRight('Tab.Top.MessageQueue') && featureQuotes.installed;
      });
      Vue.watchEffect(() => { featureQuotes.menuclassInbox = router.currentRoute.value.path == iservice.components.messageQueueOpenQuotes.route().path ? 'active' : ''; });
      Vue.watchEffect(() => { featureQuotes.menuclassManage = router.currentRoute.value.path == iservice.components.messageQueueManageOpenQuotes.route().path ? 'active' : ''; });

      featureQuotes.MyQuotes = function () {
        var path = iservice.components.messageQueueOpenQuotes.route().path;
        if (router.currentRoute.value.path == path) return;
        router.push(path);
      }
      featureQuotes.ManageQuotes = function () {
        var path = iservice.components.messageQueueManageOpenQuotes.route().path;
        if (router.currentRoute.value.path == path) return;
        router.push(path);
      }

      var featureManage = Vue.reactive({ working: false, shown: false, open: true });
      Vue.watchEffect(() => { featureManage.shown = iservice.HaveRight('Tab.MessageQueue.Supervise'); });
      Vue.watchEffect(() => { featureManage.menuclassMain = { 'menu-open': featureManage.open, 'menu-closed': !featureManage.open }; });
      Vue.watchEffect(() => { featureManage.menuStyleMain = { 'display': featureManage.open ? 'block' : 'none' }; });
      Vue.watchEffect(() => { featureManage.menuclassSubMessages = router.currentRoute.value.path == iservice.components.manageMessages.route().path ? 'active' : ''; });
      Vue.watchEffect(() => { featureManage.menuclassKB = router.currentRoute.value.path == iservice.components.manageKBArticles.route().path ? 'active' : ''; });
      featureManage.Menu = function () { featureManage.open = !featureManage.open; }
     
      var featureManageChat = Vue.reactive({ working: false, shown: false });
      Vue.watchEffect(() => { featureManageChat.shown = iservice.HaveRight('Tab.MessageQueue.SuperviseChat'); });
      Vue.watchEffect(() => { featureManageChat.menuclassSubChats = router.currentRoute.value.path == iservice.components.manageChats.route().path ? 'active' : ''; });
      Vue.watchEffect(() => { featureManageChat.menuclassAgents = router.currentRoute.value.path == iservice.components.manageLoggedInChatAgents.route().path ? 'active' : ''; });
     
      var featureQuickEmail = Vue.reactive({ working: false, shown: false, installed: true });
      Vue.watchEffect(() => {
        if (iservice.config.agentEmail) {
          featureQuickEmail.installed = true;
        }
        featureQuickEmail.shown = iservice.HaveRight('Tab.Top.MessageQueue') && featureQuickEmail.installed;
      });
      Vue.watchEffect(() => { featureQuickEmail.menuclassSend = ''; });

      var featureCreditMemo = Vue.reactive({ working: false, shown: false, installed: creditmemo.installed });
      topics.Load();
      Vue.watchEffect(() => {
        if (!featureCreditMemo.installed) return;
        topics.Load();
        var topic = topics.get(iservice.config.creditMemo.topicIDPreApproval);
        let shown = iservice.HaveRight('Tab.Top.MessageQueue');
        if (topic) {
          var segmentID = topic.segmentID;
          shown = loggedIn.accessibleSegments.includes(segmentID);
        } else {
          shown = false;
        }
        featureCreditMemo.shown = shown;
      });
      Vue.watchEffect(() => { featureCreditMemo.menuclassSubmit = ''; });

      var featureCustom = Vue.reactive({ working: false, shown: false, open: true });
      Vue.watchEffect(() => { featureCustom.shown = featureCreditMemo.shown || featureQuickEmail.shown; });
      Vue.watchEffect(() => { featureCustom.menuclassMain = { 'active': false, 'menu-open': featureCustom.open, 'menu-closed': !featureCustom.open }; });
      Vue.watchEffect(() => { featureCustom.menuStyleMain = { 'display': featureCustom.open ? 'block' : 'none' }; });
      Vue.watchEffect(() => {
        featureCustom.links = [];
        Object.getOwnPropertyNames(iservice.components).forEach(name => {
          var component = iservice.components[name];
          if (!component.route) return;
          var link = component.route();
          if (!link.menuLinkLabel) return;
          if (link.canAccess && !link.canAccess()) return;
          link.Click = function () {
            if (router.currentRoute.value.path == link.path) return;
            router.push(link.path);
          }
          link.listClass = router.currentRoute.value.path == link.path ? 'active' : '';
          if (link.path.includes("/credit-memo-submit")) {
            link.hide = !featureCreditMemo.shown;
          }
          featureCustom.links.push(link);
        });
      });
      featureCustom.Menu = function () { featureCustom.open = !featureCustom.open; }

      var featureSettings = Vue.reactive({ shown: false, open: false, tab: 'builtinforms' });
      Vue.watchEffect(() => { featureSettings.shown = iservice.HaveRight('Tab.Top.AdminTools') || iservice.HaveRight('Tab.Top.AgentEmailForm'); });
      Vue.watchEffect(() => { featureSettings.tabshownBuiltin = iservice.HaveRight('Tab.Top.AdminTools'); });
      Vue.watchEffect(() => { featureSettings.tabshownMyforms = iservice.HaveRight('Tab.Top.AdminTools') || iservice.HaveRight('Tab.Top.AgentEmailForm'); });
      Vue.watchEffect(() => { featureSettings.tabshownTenant = iservice.HaveRight('Tab.Top.AdminTools'); });
      Vue.watchEffect(() => { featureSettings.showAlerts = iservice.HaveRight('Tab.Admin.Alerts'); });
      Vue.watchEffect(() => { featureSettings.showForms = iservice.HaveRight('Tab.Admin.Forms'); });
      Vue.watchEffect(() => { featureSettings.showSmtpout = iservice.HaveRight('Tab.Admin.SmtpOut'); });
      Vue.watchEffect(() => { featureSettings.showMonitor = iservice.HaveRight('Tab.Admin.Monitor'); });
      Vue.watchEffect(() => { featureSettings.showAgentEmailAdmin = iservice.HaveRight('Tab.Admin.Forms'); });
      Vue.watchEffect(() => { featureSettings.showSegmentAdmin = iservice.HaveRight('Tab.Admin.Segments'); });
      Vue.watchEffect(() => { featureSettings.showAgentAdmin = iservice.HaveRight('Tab.Admin.Agents'); });
      Vue.watchEffect(() => { featureSettings.showDepartmentAdmin = iservice.HaveRight('Tab.Admin.Departments'); });
      Vue.watchEffect(() => { featureSettings.showListsAdmin = iservice.HaveRight('Tab.Admin.AdditionalDetails'); });
      Vue.watchEffect(() => { featureSettings.showOpenIDAdmin = iservice.HaveRight('Tab.Admin.OpenID'); });
      Vue.watchEffect(() => { featureSettings.showServiceLevelAdmin = iservice.HaveRight('Tab.Admin.ServiceLevels'); });
      Vue.watchEffect(() => { featureSettings.showSkillAdmin = iservice.HaveRight('Tab.Admin.Skills'); });
      Vue.watchEffect(() => { featureSettings.showUsertypeAdmin = iservice.HaveRight('Tab.Admin.UserTypes'); });
      Vue.watchEffect(() => { featureSettings.showWebsiteAdmin = iservice.HaveRight('Tab.Admin.Websites'); });
      Vue.watchEffect(() => { featureSettings.menuclassMain = { 'control-sidebar-open': featureSettings.open }; });
      featureSettings.TabClass = function (tab) { return { 'active': tab == featureSettings.tab }; }
      featureSettings.TabClick = function (tab) { featureSettings.tab = tab; }
      featureSettings.Menu = function () {
        featureSettings.open = !featureSettings.open;
      }

      var featureMyForms = Vue.reactive({ links: [] });
      Vue.watchEffect(() => {
        featureMyForms.links.length = 0;
        for (var name in iservice.components) {
          var comp = iservice.components[name];
          if (!comp.route) continue;
          var route = comp.route();
          if (!route.myformsLinkLabel) continue;
          if (route.canAccess && !route.canAccess()) continue;
          featureMyForms.links.push(route);
        }
      });

      var featureSegmentCreate = Vue.reactive({ working: false, shown: false, open: false });
      Vue.watchEffect(() => { featureSegmentCreate.shown = iservice.HaveRight('Tab.UserTypes.CreateAdmin'); });
      featureSegmentCreate.Start = function () {
        featureSegmentCreate.open = !featureSegmentCreate.open;
        if (!featureSegmentCreate.open) return;
        featureSegmentCreate.errors = null;
        featureSegmentCreate.success = false;
        featureSegmentCreate.name = '';
        setTimeout(() => {
          if (segmentNameEl.value) segmentNameEl.value.focus();
        });
      }
      featureSegmentCreate.Create = function (e) {
        if (featureSegmentCreate.working) return;
        featureSegmentCreate.working = true;
        featureSegmentCreate.errors = null;
        featureSegmentCreate.success = false;
        iservice.API('webapp-api-admin?api=segments&mode=new', e, { details: { name: featureSegmentCreate.name } }).then(function (data) {
          segments.add(data.row);
          loggedIn.Update(data.loggedIn);
          iservice.SuccessMessage('Segment created.');
          featureSegmentCreate.open = false;
        }).catch(function (errors) {
          featureSegmentCreate.errors = errors;
        }).finally(function () {
          featureSegmentCreate.working = false;
        });
      }
      var segmentNameEl = Vue.ref(null);

      var featureMassmail = Vue.reactive({ shown: false });
      Vue.watchEffect(() => { featureMassmail.shown = iservice.HaveRight('Tab.Admin.Mailings'); });

      var nameInitials = Vue.computed(() => {
        function getParts(name) {
          if (name.indexOf(' ') >= 0)
            return name.split(' ');
          if (name.indexOf('@') >= 0)
            return getParts(name.split('@')[0]);
          if (name.indexOf('.') >= 0)
            return name.split('.');
          if (name.indexOf('_') >= 0)
            return name.split('_');
          return [name, ''];
        }
        var parts = getParts(loggedIn.contactName);
        return parts[0][0] + (parts[1][0] || '');
      });
      var nameStyle = Vue.computed(() => {
        function getHashCode(str) {
          // from https://stackoverflow.com/a/7616484/95970
          var hash = 0, i, chr;
          if (!str.length)
            return hash;
          for (i = 0; i < str.length; i++) {
            chr = str.charCodeAt(i);
            hash = (hash << 5) - hash + chr;
            hash |= 0; // Convert to 32bit integer
          }
          return hash;
        }
        function getDarkColor(seed) {
          const hexDigits = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'];
          // from https://stackoverflow.com/a/20114561/95970
          var color = '#';
          seed = Math.abs(seed);
          color += hexDigits[seed % 5];
          for (var i = 1; i <= 5; i++)
            color += hexDigits[seed * i % 15];
          return color;
        }        
        return { 'background-color': getDarkColor(getHashCode(loggedIn.contactName)) };
      });

      var adminSegmentList = Vue.computed(() => {
        if (!segments.loaded) {
          segments.Load();
          return [];
        }
        var list = [];
        var have = {};
        segments.list.forEach(segment => {
          if (segment.hidden) return;
          if (!iservice.HaveSegment(segment.id)) return;
          var item = { id: segment.id, name: segment.name };
          list.push(item);
          have[segment.id] = true;
        });
        list.sort(function (l, r) {
          return segments.TreeSortCompare(l.id, r.id);
        });
        list.forEach(item => {
          var segment = segments.get(item.id);
          var parent = segments.get(segment.parentID);
          item.style = { 'sub-segments': (parent && have[parent.id]) ? true : false };
        });
        return list;
      });
      var defaultSegmenttab = Vue.ref('');

      if (typeof iservice.HaveRight('Tab.Segments.Details') == 'undefined') {
        defaultSegmenttab.value = '/stock';
      }

      var featureGlobalSuccess = Vue.reactive({});
      iservice.SuccessMessage = function (message) {
        featureGlobalSuccess.success = message;
      }
      messages.Listen('interaction-deleted', (success) => {
        featureGlobalSuccess.success = success;
      });

      //var socket = new WebSocket('ws://localhost:2424/web.skt');
      iservice.featureSettings = featureSettings;
      iservice.featureSidebar = featureSidebar;
      return { loggedIn, myQueue, interactions, adminSegmentList, segmentNameEl, newPasswordEl, featureSidebar, featureLogout, featureChat, featureQA, featureMQ, featureUserMenu, featureProfile, featureChangePassword, featureCustinfo, featureQuotes, featureManage, featureManageChat, featureQuickEmail, featureCreditMemo, featureCustom, featureSettings, featureMyForms, featureMassmail, featureSegmentCreate, nameInitials, nameStyle, featureGlobalSuccess, router, defaultSegmenttab };
    }
  };
  iservice.components.vueappLogin = {
    name: 'vueapp-login',
    template: '#vueapp-login',
    setup(props) {
      var loggedIn = iservice.LoggedInData();
      var loginName = Vue.ref('');
      var password = Vue.ref('');
      var working = Vue.ref(false);

      var featureLogin = Vue.reactive({ working: false });
      featureLogin.Click = function (e) {
        loggedIn.Login(featureLogin, e, loginName.value, password.value);
      }

      var currentYear = Vue.ref('2021');
      var canAgentLogin = Vue.ref(false);

      return { loggedIn, featureLogin, loginName, password, currentYear, canAgentLogin, working };
    }
  };
  iservice.components.vueappForgotPassword = {
    name: 'vueapp-forgot-password',
    route() {
      return {
        path: iservice.formPath + '/forgot-password', priority: 300, name: 'forgot-password',
        meta: { title: "iService | Forgot Password" }
      }
    },
    template: '#vueapp-forgot-password',
    setup(props) {
      var loggedIn = iservice.LoggedInData();

      var feature = Vue.reactive({ working: false, showSuccess: false });
      feature.error = {};
      feature.Click = function (e) {

        feature.errors = [];
        if (!feature.email) {
          feature.errors = ['Email address is required.'];
          feature.error.email = true;
        }
        if (feature.errors.length) return;
        feature.loading = true;
        var protocol = document.location.protocol,
          host = document.location.host,
          guid = '$value - passwordreset(guid)$',
          guid = guid.replace('- passwordreset', '-passwordreset'),
          resetUrl = protocol + '//' + host + '/app/reset-password?confirm=' + guid;

        iservice.API('webapp-api-login?api=resetSend', null, { emailAddress: feature.email, resetUrl: resetUrl }).then(function () {
          feature.loading = false;
          feature.showSuccess = true;
        }).catch(function (errors) {
          feature.errors = errors;
        }).finally(function () {
          feature.loading = false;
        })

      }

      var currentYear = Vue.ref('2021');

      return { loggedIn, feature, currentYear };
    }
  };
  iservice.components.vueappResetPassword = {
    name: 'vueapp-reset-password',
    route() {
      return {
        path: iservice.formPath + '/reset-password/:id?', priority: 300, name: 'reset-password',
        meta: { title: "iService | Reset Password" }
      }
    },
    template: '#vueapp-reset-password',
    setup(props) {
      var loggedIn = iservice.LoggedInData();
      var router = VueRouter.useRouter();

      var feature = Vue.reactive({ working: false, showSuccess: false });
      feature.error = {};
      feature.Click = function (e) {
        feature.errors = [];
        if (!feature.newPassword) {
          feature.errors.push({ msg: 'New Password is required.', details: { description: '' } });
          feature.error.newPassword = true;
        }
        if (!feature.confirmPassword) {
          feature.errors.push({ msg: 'Confirm Password is required.', details: { description: '' } });
          feature.error.confirmPassword = true;
        }
       
        if (feature.errors.length) return;
        if (feature.confirmPassword !== feature.newPassword) {
          feature.errors = ['Confirm Password did not match.'];
        }
        if (feature.errors.length) return;
        feature.loading = true;
        iservice.API('webapp-api-login?api=resetconfirm', null, { confirmGuid: router.currentRoute.value.query.confirm, newPassword: feature.newPassword }).then(function () {
          feature.loading = false;
          feature.showSuccess = true;
        }).catch(function (errors) {
          feature.errors = errors;
        }).finally(function () {
          feature.loading = false;
        })

      }

      var currentYear = Vue.ref('2021');

      return { loggedIn, feature, currentYear };
    }
  };

  function GetSuffix(el) {
    var node = el.__vnode;
    if (!node) {
      return null;
    }
    if (!node.dirs || !node.dirs.length) return null;
    for (var dir of node.dirs) {
      if (dir.dir.name != 'id') continue;
      return dir.value;
    }
    return null;
  }
  function SetID(el, suffix) {
    var id = suffix;
    if (!el) return;
    var parent = el.parentElement;
    while (parent) {
      var prefix = GetSuffix(parent);
      if (prefix) id = prefix + '_' + id;
      parent = parent.parentElement;
    }
    el.id = id;
  }
  iservice.directives.iserviceIdPrefix = {
    name: 'id',
    mounted(el, binding, vnode) {
      SetID(el, binding.value);
    },
    updated(el, binding, vnode) {
      SetID(el, binding.value);
    }
  };
  iservice.directives.iserviceScrollToBottom = {
    name: 'bottomScroll',
    mounted(element, binding, vnode) {
      var elementID = vnode.props.id;
      var objDiv = document.getElementById(elementID);
      objDiv.scrollTop = objDiv.scrollHeight;
    },
    updated(element, binding, vnode) {
      var elementID = vnode.props.id;
      var objDiv = document.getElementById(elementID);
      objDiv.scrollTop = objDiv.scrollHeight;
    }
  };
  
  iservice.directives.iserviceTooltip = {
    name: 'tooltip',
    updateTooltip(el, { value, modifiers }) {
      value.displayArrow = true;
      if (typeof value === "string") {
        // we can pass either a string
        el.setAttribute("data-v-tooltip", value);

        // this check if when v-tooltip receives string with .arrow modifier
        if (modifiers.arrow) {
          el.style.setProperty("--v-tooltip-arrow-display", "inline");
        }
      } else {
        // or an object
        if (value.text) {
          el.setAttribute("data-v-tooltip", value.text);
        }
        if (value.displayArrow || modifiers.arrow) {
          // if there is a prop global: true then mutate the :root css variables
          // otherwise it adds given variables to the element, which makes it possible to be different than others
          const targetEl = value.global ? document.documentElement : el;
          targetEl.style.setProperty(
            "--v-tooltip-arrow-display",
            "inline"
          );
        }
        if (value.theme) {
          // if there is a prop global: true then mutate the :root css variables
          // otherwise it adds given variables to the element, which makes it possible to be different than others
          const targetEl = value.global ? document.documentElement : el;
          for (const [key, val] of Object.entries(value.theme)) {
            if (key === "placement") {
              switch (val) {
                case "top":
                  targetEl.style.setProperty(
                    "--v-tooltip-left",
                    "50%"
                  );
                  targetEl.style.setProperty(
                    "--v-tooltip-top",
                    "0%"
                  );
                  targetEl.style.setProperty(
                    "--v-tooltip-translate",
                    "translate(-50%, -110%)"
                  );
                  if (value.displayArrow || modifiers.arrow) {
                    targetEl.style.setProperty(
                      "--v-tooltip-arrow-border-color",
                      "var(--v-tooltip-background-color) transparent transparent transparent"
                    );
                    targetEl.style.setProperty(
                      "--v-tooltip-arrow-top",
                      "calc(var(--v-tooltip-top) - var(--v-tooltip-top-offset) + 8px)"
                    );
                  }
                  break;
                case "bottom":
                  targetEl.style.setProperty(
                    "--v-tooltip-left",
                    "50%"
                  );
                  targetEl.style.setProperty(
                    "--v-tooltip-top",
                    "100%"
                  );
                  targetEl.style.setProperty(
                    "--v-tooltip-translate",
                    "translate(-50%, 10%)"
                  );
                  if (value.displayArrow || modifiers.arrow) {
                    targetEl.style.setProperty(
                      "--v-tooltip-arrow-border-color",
                      "transparent transparent var(--v-tooltip-background-color) transparent"
                    );
                    targetEl.style.setProperty(
                      "--v-tooltip-arrow-top",
                      "calc(var(--v-tooltip-top) - var(--v-tooltip-top-offset) - 7px)"
                    );
                  }
                  break;
                case "left":
                  targetEl.style.setProperty(
                    "--v-tooltip-left",
                    "0%"
                  );
                  targetEl.style.setProperty(
                    "--v-tooltip-top",
                    "50%"
                  );
                  targetEl.style.setProperty(
                    "--v-tooltip-translate",
                    "translate(-110%, -50%)"
                  );
                  if (value.displayArrow || modifiers.arrow) {
                    targetEl.style.setProperty(
                      "--v-tooltip-arrow-border-color",
                      "transparent transparent transparent var(--v-tooltip-background-color)"
                    );
                    targetEl.style.setProperty(
                      "--v-tooltip-arrow-top",
                      "calc(var(--v-tooltip-top)"
                    );
                    targetEl.style.setProperty(
                      "--v-tooltip-arrow-left",
                      "calc( var(--v-tooltip-left) - var(--v-tooltip-left-offset) + 1.5px)"
                    );
                  }
                  break;
                case "right":
                  targetEl.classList.add("right-tooltip")
                  targetEl.style.setProperty(
                    "--v-tooltip-left",
                    "100%"
                  );
                  targetEl.style.setProperty(
                    "--v-tooltip-top",
                    "50%"
                  );
                  targetEl.style.setProperty(
                    "--v-tooltip-translate",
                    "translate(10%, -50%)"
                  );
                  if (value.displayArrow || modifiers.arrow) {
                    targetEl.style.setProperty(
                      "--v-tooltip-arrow-border-color",
                      "transparent var(--v-tooltip-background-color) transparent  transparent"
                    );
                    targetEl.style.setProperty(
                      "--v-tooltip-arrow-top",
                      "calc(var(--v-tooltip-top)"
                    );
                    targetEl.style.setProperty(
                      "--v-tooltip-arrow-left",
                      "calc( var(--v-tooltip-left) - var(--v-tooltip-left-offset) - 2px)"
                    );
                  }
                  break;
                default:
                  break;
              }
            } else if (key === "offset" && !value.global) {
              for (const direction of val) {
                if (direction === "left") {
                  targetEl.style.setProperty(
                    "--v-tooltip-left-offset",
                    `-${targetEl.scrollWidth -
                    targetEl.clientWidth
                    }px`
                  );
                } else if (direction === "right") {
                  targetEl.style.setProperty(
                    "--v-tooltip-left-offset",
                    `${targetEl.scrollWidth -
                    targetEl.clientWidth
                    }px`
                  );
                } else if (direction === "top") {
                  targetEl.style.setProperty(
                    "--v-tooltip-top-offset",
                    `-${targetEl.scrollHeight -
                    targetEl.clientHeight
                    }px`
                  );
                } else if (direction === "bottom") {
                  targetEl.style.setProperty(
                    "--v-tooltip-top-offset",
                    `${targetEl.scrollHeight -
                    targetEl.clientHeight
                    }px`
                  );
                }
              }
            } else {
              targetEl.style.setProperty(`--v-tooltip-${key}`, val);
            }
          }
        }
      }
    },
    // hooks
    mounted(el, { value, dir, modifiers }) {
      // v-tooltips with global prop won't show the tooltip
      // also object notation without text prop won't show neither
      if (typeof value === "object" && !value.global && value.text) {
        el.classList.add("data-v-tooltip");
      } else if (typeof value === "string") {
        el.classList.add("data-v-tooltip");
      }

      // to use functions in Vue's directives which are inside this object, we can't use this, we have to use dir, which is the directive object
      dir.updateTooltip(el, { value, modifiers });
    },
    updated(el, { value, dir, modifiers }) {
      dir.updateTooltip(el, { value, modifiers });
    },
  };


  //create vue app.
  var vueRoot = {
    data() { return {
      title: 'iService'
    }}
  };
  var app = Vue.createApp(vueRoot);

  iservice.components.iservicePopover = {
    name: 'link-popover',
    data() {
      return {
        showpopover: false,
        popoverOpts: {
          transitionName: 'popover',
          theme: ''
        },
        isInInfo: false
      }
    },
    methods: {
      hoverIn() {
        let vm = this
        this.timer = setTimeout(function () {
          vm.showPopover()
        }, 100)
      },

      hoverOut() {
        let vm = this
        clearTimeout(vm.timer)
        this.timer = setTimeout(function () {
          if (!vm.isInInfo) {
            vm.closePopover()
          }
        }, 200)
      },

      hoverInfo() {
        let vm = this
        vm.isInInfo = !vm.isInInfo;
        this.timer = setTimeout(function () {
          if (!vm.isInInfo) {
            vm.closePopover()
          } else {
            
            vm.showPopover()
          }
        }, 100)
      },

      hoverOutInfo() {
        this.isInInfo = false
        this.hoverOut()
      },

      showPopover: function () {
        this.showpopover = true
      },

      closePopover: function () {
        this.showpopover = false
      }
    },
    template: `
    <div class="popover-main" :class="popoverOpts ? popoverOpts.theme : ''">
      <span
        class="popover__text"
        v-on:click="hoverInfo"
      >
        <slot></slot>
      </span>
      <transition :name="popoverOpts ? popoverOpts.transitionName : 'popover'">
        <div class="popover__content"
          v-if="showpopover"
        >
          <slot name="content"></slot>
        </div>
      </transition>
    </div>
  `
  };

  //set up global rootPath so we can use it in all template html
  app.config.globalProperties.rootPath = window.rootPath;
  app.config.globalProperties.formPath = iservice.formPath;

  //add all of our directives.
  Object.getOwnPropertyNames(iservice.directives).forEach(name => {
    var directive = iservice.directives[name];
    app.directive(directive.name, directive);
  });

  //add all of our components.
  Object.getOwnPropertyNames(iservice.components).forEach(name => {
    var component = iservice.components[name];
    app.component(component.name, component);
  });
  app.component('Datepicker', Vue3DatePicker);
  app.component('Multiselect', VueformMultiselect);
  app.component('Vue3ChartJs', Vue3ChartJs);
  //create page routes
  var routes = [];
  Object.getOwnPropertyNames(iservice.components).forEach(name => {
    var component = iservice.components[name];
    if (!component.route) return;
    var route = component.route();
    route.component = component;
    routes.push(route);
  });
  routes.sort((l, r) => {
    if (l.priority < r.priority) return -1;
    if (l.priority > r.priority) return 1;
    return 0;
  });
  
  var router = iservice.router = VueRouter.createRouter({
    history: VueRouter.createWebHistory(),
    routes
  });

  app.use(router);
  router.beforeEach((toRoute, fromRoute, next) => {
    if (iservice.featureSettings) iservice.featureSettings.open = false;
    if (/Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test(navigator.userAgent)) {
      if (iservice.featureSidebar) iservice.featureSidebar.collapsed = false;
    }
    window.document.title = toRoute.meta && toRoute.meta.title ? toRoute.meta.title : 'iService';

    next();
  })

  //start vue
  app.mount('#iService');

})(iservice);