﻿var iservice = window.iservice || {};
(function (iservice) {
  iservice.components = iservice.components || {};

  iservice.components.messageQueueMyInbox = {
    name: 'vueapp-messagequeue-myinbox',
    template: '#vueapp-messagequeue-myinbox',
    route() {
      return { path: iservice.formPath + '/my-message-inbox', priority: 200, meta: {title : "iService | My Message Inbox"} } },
    setup() {
      var myQueue = iservice.MyQueueData();
      var interactions = iservice.InteractionData();
      var messages = iservice.Messages();
      var loggedIn = iservice.LoggedInData();
      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        filterLabel: 'Filter Inbox',
        emptyMessage: 'There are no messages in your inbox.',
        pageSize: 5,
        expandable: true,
        searching: false,
        initialSort: 'Date',
        exportName: "My Inbox Messages"
      });
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }
      Vue.watchEffect(() => {
        tableConfig.columns = [
          Vue.reactive({
            name: 'From', label: 'From',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'customerName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Email', label: 'Email',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'fromAddress'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: false
          }),
          Vue.reactive({
            name: 'Date', label: 'Date', sortIncreasing: false,
            filterText: iservice.tables.DateFromNowFilterTextFunc(interactions, 'date'), sortText: iservice.tables.DateSortTextFunc(interactions, 'date'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-date-from-now', shown: true
          }),
          Vue.reactive({
            name: 'Subject', label: 'Subject',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'subjectBare'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Segment', label: 'Segment',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'segmentName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Topic', label: 'Topic',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'topicName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          })];
      });

      var featureTabs = Vue.reactive({ tab: 'details' });
      featureTabs.TabClass = function (tab) { return { 'active': tab == featureTabs.tab }; }
      featureTabs.TabClick = function (tab) { featureTabs.tab = tab; }
      featureTabs.HaveRight = iservice.HaveRight('Tab.Top.MessageQueue');
      var inboxIDs = Vue.computed(() => {
        if (myQueue.subbox.messageQueueMyInbox) return myQueue.subbox.messageQueueMyInbox;
        return [];
      });

      var selectedID = Vue.ref(null);
      var feature = Vue.reactive({ contactId: '', ownAccount: '', currentCustomerName: ''  });
      function Select(id) {
        selectedID.value = id;
      }
      var selectedIDs = Vue.computed(() => {
        if (selectedID.value) return [selectedID.value];
        return [];
      });
      Vue.watchEffect(() => {
        if (!selectedID.value) return;
        var interaction = interactions.get(selectedID.value);
        if (interaction) {
          feature.contactId = interactions.get(selectedID.value).customerID;
          feature.currentCustomerName = interactions.get(selectedID.value).customerName;
          feature.ownAccount = interactions.get(selectedID.value).customerID == loggedIn.contactID;
        }
        var pos = myQueue.subbox.messageQueueMyInbox.indexOf(selectedID.value);
        if (pos != -1) return;
        selectedID.value = null;
      });

      messages.Listen('messageQueueMyInbox', id => {
        selectedID.value = id;
      });

      return { myQueue, tableConfig, selectedID, featureTabs, inboxIDs, selectedID, selectedIDs, Select, feature, ListCollapsed };
    }
  };
  iservice.components.messageQueueMyPending = {
    name: 'vueapp-messagequeue-mypending',
    template: '#vueapp-messagequeue-mypending',
    route() { return { path: iservice.formPath + '/my-pending-messages', priority: 300, meta: { title: "iService | My Pending Messages" } } },
    setup() {
      var myQueue = iservice.MyQueueData();
      var interactions = iservice.InteractionData();
      var messages = iservice.Messages();
      var loggedIn = iservice.LoggedInData();

      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        filterLabel: 'Filter Inbox',
        emptyMessage: 'There are no messages sent waiting reply in your inbox.',
        pageSize: 5,
        expandable: true,
        searching: false,
        initialSort: 'Date',
        exportName: "My Pending Messages"
      });
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }
      Vue.watchEffect(() => {
        tableConfig.columns = [
          Vue.reactive({
            name: 'From', label: 'From',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'customerName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Email', label: 'Email',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'fromAddress'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: false
          }),
          Vue.reactive({
            name: 'Date', label: 'Date', sortIncreasing: false,
            filterText: iservice.tables.DateFromNowFilterTextFunc(interactions, 'date'), sortText: iservice.tables.DateSortTextFunc(interactions, 'date'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-date-from-now', shown: true
          }),
          Vue.reactive({
            name: 'Subject', label: 'Subject',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'subjectBare'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Segment', label: 'Segment',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'segmentName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Topic', label: 'Topic',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'topicName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          })];
      });

      var featureTabs = Vue.reactive({ tab: 'details' });
      featureTabs.TabClass = function (tab) { return { 'active': tab == featureTabs.tab }; }
      featureTabs.TabClick = function (tab) { featureTabs.tab = tab; }
      featureTabs.HaveRight = iservice.HaveRight('Tab.Top.MessageQueue');
      var inboxIDs = Vue.computed(() => {
        if (myQueue.subbox.messageQueueMyPending) return myQueue.subbox.messageQueueMyPending;
        return [];
      });

      var selectedID = Vue.ref(null);
      var feature = Vue.reactive({ contactId: '', ownAccount: '', currentCustomerName: '' });
      function Select(id) {
        selectedID.value = id;
      }
      var selectedIDs = Vue.computed(() => {
        if (selectedID.value) return [selectedID.value];
        return [];
      });
      Vue.watchEffect(() => {
        if (!selectedID.value) return;
        var interaction = interactions.get(selectedID.value);
        if (interaction) {
          feature.contactId = interactions.get(selectedID.value).customerID;
          feature.currentCustomerName = interactions.get(selectedID.value).customerName;
          feature.ownAccount = interactions.get(selectedID.value).customerID == loggedIn.contactID;
        }
        var pos = myQueue.subbox.messageQueueMyPending.indexOf(selectedID.value);
        if (pos != -1) return;
        selectedID.value = null;
      });

      messages.Listen('messageQueueMyPending', id => {
        selectedID.value = id;
      });

      return { myQueue, tableConfig, inboxIDs, selectedID, featureTabs, selectedID, selectedIDs, Select, feature, ListCollapsed };
    }
  };
  iservice.components.messageQueueMyApprove = {
    name: 'vueapp-messagequeue-myapprove',
    template: '#vueapp-messagequeue-myapprove',
    route() { return { path: iservice.formPath + '/my-messages-to-approve', priority: 300, meta: { title: "iService | My Messages to Approve" } } },
    setup() {
      var myQueue = iservice.MyQueueData();
      var interactions = iservice.InteractionData();
      var messages = iservice.Messages();
      var loggedIn = iservice.LoggedInData();

      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        filterLabel: 'Filter Inbox',
        emptyMessage: 'There are no messages to approve in your inbox.',
        pageSize: 5,
        expandable: true,
        searching: false,
        initialSort: 'Date',
        exportName: "My Messages to Approve"
      });
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }
      Vue.watchEffect(() => {
        tableConfig.columns = [
          Vue.reactive({
            name: 'From', label: 'From',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'customerName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Email', label: 'Email',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'fromAddress'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: false
          }),
          Vue.reactive({
            name: 'Date', label: 'Date', sortIncreasing: false,
            filterText: iservice.tables.DateFromNowFilterTextFunc(interactions, 'date'), sortText: iservice.tables.DateSortTextFunc(interactions, 'date'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-date-from-now', shown: true
          }),
          Vue.reactive({
            name: 'Subject', label: 'Subject',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'subjectBare'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Segment', label: 'Segment',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'segmentName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Topic', label: 'Topic',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'topicName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          })];
      });

      var featureTabs = Vue.reactive({ tab: 'details' });
      featureTabs.TabClass = function (tab) { return { 'active': tab == featureTabs.tab }; }
      featureTabs.TabClick = function (tab) { featureTabs.tab = tab; }
      featureTabs.HaveRight = iservice.HaveRight('Tab.Top.MessageQueue');
      var inboxIDs = Vue.computed(() => {
        if (myQueue.subbox.messageQueueMyApprove) return myQueue.subbox.messageQueueMyApprove;
        return [];
      });

      var selectedID = Vue.ref(null);
      var feature = Vue.reactive({ contactId: '', ownAccount: '', currentCustomerName: '' });
      function Select(id) {
        selectedID.value = id;
      }
      var selectedIDs = Vue.computed(() => {
        if (selectedID.value) return [selectedID.value];
        return [];
      });
      Vue.watchEffect(() => {
        if (!selectedID.value) return;
        var interaction = interactions.get(selectedID.value);
        if (interaction) {
          feature.contactId = interactions.get(selectedID.value).customerID;
          feature.currentCustomerName = interactions.get(selectedID.value).customerName;
          feature.ownAccount = interactions.get(selectedID.value).customerID == loggedIn.contactID;
        }
        var pos = myQueue.subbox.messageQueueMyApprove.indexOf(selectedID.value);
        if (pos != -1) return;
        selectedID.value = null;
      });

      messages.Listen('messageQueueMyApprove', id => {
        selectedID.value = id;
      });

      return { myQueue, tableConfig, inboxIDs, selectedID, featureTabs, selectedID, selectedIDs, Select, feature, ListCollapsed};
    }
  };

  iservice.components.manageMessages = {
    name: 'vueapp-messagequeue-manage-messages',
    template: '#vueapp-messagequeue-manage-messages',
    route() { return { path: iservice.formPath + '/manage-messages', priority: 200, name: 'manage-messages', meta: { title: "iService | Manage Messages" } } },
    setup() {
      var segments = iservice.SegmentData();
      var topics = iservice.TopicData();
      var agents = iservice.AgentData();
      var loggedIn = iservice.LoggedInData();
      var interactions = iservice.InteractionData();
      var plugins = iservice.Plugins();

      var featureSearch = Vue.reactive({ working: true, interactions: [], alert: false, alertDetails: false, colorCodes: [] });
      featureSearch.AlertDetails = function () { featureSearch.alertDetails = !featureSearch.alertDetails; }
      featureSearch.AlertClear = function () { featureSearch.alert = false; }
      var search = { groups: [{ fields: [{ fieldID: 'status', searchString: 'Unassigned', where: 'anywhere' }, { fieldID: 'status', searchString: 'Queued', where: 'anywhere' }, { fieldID: 'status', searchString: 'Pending', where: 'anywhere' }] }] };
      iservice.API('webapp-api-mq-search?api=search', null, { fields: search, pageNum: 0, perPage: 2500 }).then(data => {
        data.interactions.forEach(int => {
          int.columnColor = '';
          interactions.add(int);
          featureSearch.interactions.push(int.id);
        });
        featureSearch.alert = data.resultCount >= 2500;
      }).catch(errors => {

      }).finally(() => {
        featureSearch.working = false;
      });

      interactions.WhenDeleted(id => {
        var index = featureSearch.interactions.indexOf(id);
        if (index == -1) return;
        featureSearch.interactions.splice(index, 1);
      });
      
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }
      var featureMessages = Vue.reactive({ showQuestions: true, showPending: false });
      var featureTopics = Vue.reactive({ showMine: true, segments: [], list: [], selectedMap: {}, collapsedMap: {}, selectAll: true, collapsed: false });
      var selectAll = featureTopics.selectAll;
      Vue.watchEffect(() => {
        selectAll = featureTopics.selectAll;
      });
      featureTopics.Collapse = function () {
        featureTopics.collapsed = !featureTopics.collapsed;
      }
      Vue.watchEffect(() => {
        featureTopics.collapsedClass = featureTopics.collapsed ? 'col-md-1 treeview-collapse' : 'col-md-4 treeview-expand';
        featureMessages.collapsedClass = featureTopics.collapsed ? 'col-md-11 treeview-expand' : 'col-md-8';
      });
      var selectStartup = true;
      featureTopics.SelectAll = function () {
        selectStartup = false;
        featureTopics.list.forEach(topic => {
          featureTopics.selectedMap[topic.id] = featureTopics.selectAll;
        });
      }
      featureTopics.ShowMine = function () {
        selectStartup = false;
      }
      Vue.watchEffect(() => {
        if (!segments.loaded) segments.Load();
        if (!topics.loaded) topics.Load();
        if (!agents.loaded) agents.Load();
        if (!topics.list.length) return;
        featureTopics.segments.length = 0;
        var map = {};
        featureTopics.list.length = 0;
        segments.list.forEach(seg => {
          var segmentID = seg.id;
          var segment = Vue.reactive({
            name: seg.name,
            topics: []
          });
          topics.inSegment(segmentID).forEach(topic => {
            if (featureTopics.showMine) {
              var agentList = agents.forTopic(topic.id);
              var me = agentList.find(agentID => {
                var agent = agents.get(agentID);
                return agent.id == loggedIn.contactID;
              });
              if (!me) return;
            }
            var item = map[topic.id] = Vue.reactive({
              id: topic.id,
              name: topic.name,
              visibility: topic.visibility,
              children: []
            });
            featureTopics.list.push(item);
            if (selectStartup) featureTopics.selectedMap[item.id] = selectAll;
          });
          topics.inSegment(segmentID).forEach(topic => {
            var row = map[topic.id];
            if (!row) return;
            var parentID = topic.parentID;
            var parent = map[parentID];
            while (!parent) {
              if (!parentID) break;
              parentID = topics.get(parentID).parentID;
              parent = map[parentID];
            }
            (parent ? parent.children : segment.topics).push(row);
          });
          if (segment.topics.length < 1) return;
          featureTopics.segments.push(segment);
        });
        function SortRow(lrow, rrow) { return topics.TreeSortCompare(lrow.id, rrow.id); }
        featureTopics.segments.sort(SortRow);
        featureTopics.list.forEach(row => {
          row.children.sort(SortRow);
        });
      });
      Vue.watchEffect(() => {
        featureTopics.selectAll = false;
        for (var topic of featureTopics.list) {
          if (!featureTopics.selectedMap[topic.id]) return;
        }
        featureTopics.selectAll = true;
      });

      function ComputedStatusList(status) {
        return Vue.computed(() => {
          var inTopic = {};
          featureTopics.list.forEach(topic => {
            inTopic[topic.id] = [];
          });
          featureSearch.interactions.forEach(id => {
            var int = interactions.get(id);
            if (!int) return;
            var list = inTopic[int.topicID];
            if (!list) return;
            if (int.status == status) list.push(id);
          });
          return inTopic;
        });
      }
      var queuedInTopic = ComputedStatusList('Queued');
      var unassignedInTopic = ComputedStatusList('Unassigned');
      var pendingInTopic = ComputedStatusList('Pending');
      var agentsForTopic = Vue.computed(() => {
        var forTopic = {};
        featureTopics.list.forEach(topic => {
          var item = forTopic[topic.id] = { count: 0, agentNameList: '' };
          var agentList = agents.forTopic(topic.id);
          var separator = '';
          agentList.forEach(agentID => {
            var agent = agents.get(agentID);
            if (!agent || !agent.isLoggedIn) return;
            item.count++;
            item.agentNameList += separator + agent.displayname;
            separator = ', ';
          });
        });
        return forTopic;
      });
      function CountDisplay(value) {
        if (Array.isArray(value)) value = value.length;
        if (!value || value < 1) return '--';
        if (value < 1000) return '' + value;
        return '1K+';
      }

      Vue.provide('selectedMap', featureTopics.selectedMap);
      Vue.provide('collapsedMap', featureTopics.collapsedMap);
      Vue.provide('queuedInTopic', queuedInTopic);
      Vue.provide('unassignedInTopic', unassignedInTopic);
      Vue.provide('pendingInTopic', pendingInTopic);
      Vue.provide('agentsForTopic', agentsForTopic);
      Vue.provide('CountDisplay', CountDisplay);


      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        componentExpanded: 'interaction-view-loader',
        filterLabel: 'Filter Messages',
        emptyMessage: 'No Interactions',
        pageSize: 50,
        expandable: true,
        searching: false,
        initialSort: 'Date',
        colorCodes: [],
        checkAllMode: 'visible',
        exportName: "Manage Messages",
        checkIDAllowed: function (id) {
          var int = interactions.get(id);
          if (!int) return false;
          var plugin = plugins.FindPlugin(id);
          if (!plugin) return false;
          return plugin.answerComponent == 'interaction-answer';
        }
      });

      iservice.API('webapp-api-admin?api=configuration', null, { application: 'agentapp', name: 'colorCodes' }).then(response => {
        tableConfig.colorCodes = response?.items?.[0]?.data;
        if (!tableConfig.colorCodes) tableConfig.colorCodes = [];
      })
      tableConfig.searching = true;
     
      var interactionsShown = Vue.computed(() => {
      
        if (!segments.loaded) segments.Load();
        if (!topics.loaded) topics.Load();
        if (!agents.loaded) agents.Load();
        if (!topics.list.length) return [];
        var list = [];
        featureTopics.list.forEach(topic => {
          if (!featureTopics.selectedMap[topic.id]) return;
          if (featureMessages.showQuestions) {
            queuedInTopic.value[topic.id].forEach(id => {
              list.push(id);
            });
            unassignedInTopic.value[topic.id].forEach(id => {
              list.push(id);
            });
          }

          if (featureMessages.showPending) {
            pendingInTopic.value[topic.id].forEach(id => {
              list.push(id);
            });
          }
        });
        tableConfig.searching = false
        return list;
      });

      Vue.watchEffect(() => {
        tableConfig.columns = [
          Vue.reactive({
            name: 'expand', hidable: false,
            filterText: function () { },
            componentHead: 'iservice-table-head-select-all', componentCell: 'iservice-table-cell-expand-multiselect', shown: true, tdClass: 'col-icon-check expand-td',
          }),
          Vue.reactive({
            name: 'Subject', label: 'Subject',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'subjectBare'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Contact Name', label: 'Contact Name',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'customerName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-interaction-customername-linked', shown: true
          }),
          Vue.reactive({
            name: 'Contact Email', label: 'Contact Email',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'fromAddress'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-interaction-from-linked', shown: false
          }),
          Vue.reactive({
            name: 'Date', label: 'Date', sortIncreasing: false,
            hasColor: true,
            filterText: iservice.tables.DateFromNowFilterTextFunc(interactions, 'date'),
            sortText: iservice.tables.DateSortTextFunc(interactions, 'date'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-date-from-now-manage', shown: true
          }),
          Vue.reactive({
            name: 'Assigned To', label: 'Assigned To',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'assignedToName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Topic', label: 'Topic',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'topicName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Segment', label: 'Segment',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'segmentName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: false
          })];
      });
      var selectedIDs = Vue.ref([]);
      var messageList = Vue.ref(null);

      var websyncData = {
        unsubForSegment: {},
        Clear() {
          for (var id in websyncData.unsubForSegment) {
            var unsub = websyncData.unsubForSegment[id];
            if (unsub) unsub();
            websyncData.unsubForSegment[id] = null;
          }
        }
      };
      Vue.onUnmounted(() => {
        websyncData.Clear();
      });
      iservice.WatchLoginChange(() => {
        websyncData.Clear();
        for (var segmentID of loggedIn.accessibleSegments) {
          websyncData.unsubForSegment[segmentID] = websync.subscribe('/managemsg/' + loggedIn.tenantID + '/' + segmentID, MessageReceive);
        }
      });
      function MessageReceive(msg) {
        var message = msg.getData();
        if (message.inboxUpdate != null) {
          MessageStatusUpdate(message.inboxUpdate);
        }
      }
      function MessageStatusUpdate(data) {
        interactions.add(data);
        var int = interactions.get(data.id);
        if (!int) return;
        var pos = featureSearch.interactions.indexOf(int.id);
        if (int.status == 'Queued' || int.status == 'Unassigned' || int.status == 'Pending') {
          if (pos == -1) featureSearch.interactions.push(int.id);
        }
        else {
          if (pos != -1) featureSearch.interactions.splice(pos, 1);
        }
      }

      var featureMassUpdate = Vue.reactive({ isCollapsed: true, tab: 'details', topics: [], showAlert: false  });
      featureMassUpdate.Active = function (tab) { return { 'active': tab == featureMassUpdate.tab } }
      featureMassUpdate.Tab = function (tab) { featureMassUpdate.tab = tab; }
      featureMassUpdate.Collapse = function () {
        if (featureMessages.showPending && featureMessages.showQuestions) {
          featureMassUpdate.showAlert = true;
          return;
        }
        featureMassUpdate.isCollapsed = !featureMassUpdate.isCollapsed;
      }
      featureMassUpdate.CancelAlert = function () {
        featureMassUpdate.showAlert = false;  
      }
      Vue.watchEffect(() => {
        featureMassUpdate.collapseClass = featureMassUpdate.isCollapsed ? '' : 'chevron-expand-indicator_expanded';
        featureMassUpdate.boxClass = featureMassUpdate.isCollapsed ? 'in collapse' : '';
      });

      var featureAnswerProps = Vue.reactive({ responseProps: [], setInAllTopics: false });
      iservice.EditableProperties(featureAnswerProps, featureAnswerProps.responseProps);
      Vue.watchEffect(() => {
        featureAnswerProps.responseProps.length = 0;
        featureMassUpdate.topics.length = 0;
        var topicIDs = {};
        var propIDs = {};
        selectedIDs.value.forEach(id => {
          var int = interactions.get(id);
          if (!int) return;
          if (topicIDs[int.topicID]) return;
          var topic = topics.get(int.topicID);
          if (!topic) return;
          topicIDs[topic.id] = topic;
          featureMassUpdate.topics.push(topic.id);
          topic.answerProperties.forEach(seg => {
            seg.properties.forEach(prop => {
              if (propIDs[prop.propertyID]) return;
              featureAnswerProps.responseProps.push(prop);
              propIDs[prop.propertyID] = prop;
            });
          });
        });
        featureAnswerProps.Reload(true);
      });
      Vue.watchEffect(() => {
        if (featureAnswerProps.props.length < 1) featureMassUpdate.tab = 'details';
      });
      var bodyHtmlEdit = Vue.ref('');
      var cc = Vue.ref('');
      var bcc = Vue.ref('');
      var subject = Vue.ref('');
      var answer = Vue.reactive({});
      Vue.watchEffect(() => {
        answer.isMassUpdate = true;
        answer.setInAllTopics = featureAnswerProps.setInAllTopics;
        if (featureMassUpdate.isCollapsed) {
          answer.answerProperties = [];
          answer.bodyHtml =  answer.cc = answer.bcc = answer.subject = '';
        }
        else {
          answer.answerProperties = featureAnswerProps.out;
          answer.setInAllTopics = featureAnswerProps.setInAllTopics;
          if (featureMassUpdate.showCC) answer.cc = cc.value;
          if (featureMassUpdate.showBCC) answer.bcc = bcc.value;
          answer.subject = subject.value;
          answer.bodyHtml = bodyHtmlEdit.value;
        }
      });
      var featureUploads = iservice.FeatureUploads();

      var featureAction = Vue.reactive({});
      featureAction.SubmitAnswer = function (url, e, successMessage, doReset, removeOnSuccess = false) {
        if (featureAction.working) return;
        featureAction.working = true;
        featureAction.errors = featureAction.success = null;
        var uploads = featureMassUpdate.isCollapsed ? null : featureUploads.files;
        iservice.API(url, e, { interactionIDs: selectedIDs.value, answer }, uploads).then(data => {
          featureUploads.Reset();
          iservice.SuccessMessage(successMessage);
          if (doReset) {
            messageList.value.UncheckAll();
            bodyHtmlEdit.value = cc.value = bcc.value = subject.value = '';
            featureAnswerProps.Reload();
          }
          if (removeOnSuccess) {
            iservice.data.mq.numCanGet = data.numCanGet;
            iservice.data.mq.numCanApprove = data.numCanApprove;
          }
        }).catch(errors => {
          featureAction.errors = errors;
        }).finally(() => {
          featureAction.working = false;
        });
      }
      function CheckProperties(feature, label) {
        for (var prop of feature.props) {
          if (!prop.missingRequired) continue;
          prop.focusMe = true;
          featureMassUpdate.Tab('props');
          featureAction.errors = ['Must set value for ' + label + ' property ' + prop.def.name + '.'];
          return false;
        }
        return true;
      }
      featureAction.CheckAnswerProperties = function () {
        return CheckProperties(featureAnswerProps, 'answer');
      }
      featureAction.CheckQuestionProperties = function () {
        return true;
      }
      featureAction.CheckCaseProperties = function () {
        return true;
      }
      var interactionRef = Vue.computed(() => {
        var id = selectedIDs.value.length > 0 ? selectedIDs.value[0] : '';
        return interactions.get(id);
      });
      var featureResolve = iservice.components.answerButtonStandardResolve.answerAction(interactionRef, answer, featureAction);
      var featureForward = iservice.components.answerButtonStandardForward.answerAction(interactionRef, answer, featureAction);
      var featureTopic = iservice.components.answerButtonStandardTopic.answerAction(interactionRef, answer, featureAction);
      var featureSpam = iservice.components.answerButtonStandardSpam.answerAction(interactionRef, answer, featureAction);
      var featureSend = iservice.components.answerButtonStandardSend.answerAction(interactionRef, answer, featureAction);
      var featurePrivateNote = iservice.components.answerButtonStandardPrivateNote.answerAction(interactionRef, answer, featureAction);
      var featurePublicNote = iservice.components.answerButtonStandardPublicNote.answerAction(interactionRef, answer, featureAction);
      var featureSaveDraft = iservice.components.answerButtonStandardSaveDraft.answerAction(interactionRef, answer, featureAction);
      Vue.watchEffect(() => {
        [featureResolve, featureForward, featureTopic, featureSpam, featureSend, featurePrivateNote, featurePublicNote, featureSaveDraft].forEach(feature => { feature.isDisabled = selectedIDs.value.length < 1; });
      });
      [featureResolve, featureForward, featureTopic, featureSpam].forEach(feature => { feature.class = 'btn-sm'; });
      featureForward.canAssignSelf = true;
      featureForward.onLeft = false;
      featureTopic.onLeft = false;

      var editor = Vue.ref(null);
      var anchorStocks = Vue.ref(null);
      var featureStockResponses = Vue.reactive({ filter: '', segmentID: '' });
      featureStockResponses.Open = function () {
        featureStockResponses.Close = iservice.components.popupHolder.Open(anchorStocks, 'stock-responses-menu', false, featureStockResponses);
      }
      featureStockResponses.PasteVar = function (text) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteText(text);
      }
      featureStockResponses.PasteStock = function (html) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteHtml(html);
      }

      return { messageList, featureSearch, featureTopics, featureMessages, interactionsShown, tableConfig, selectedIDs, featureMassUpdate, featureAnswerProps, editor, anchorStocks, featureStockResponses, bodyHtmlEdit, cc, bcc, subject, featureSend, featurePrivateNote, featurePublicNote, featureResolve, featureForward, featureTopic, featureSpam, featureSaveDraft, featureUploads, featureAction, ListCollapsed };
    }
  }
  iservice.components.manageChatsTopicRow = {
    name: 'manage-chats-topic-row',
    template: '#manage-chats-topic-row',
    props: ['topic'],
    setup(props) {
      var selectedMap = Vue.inject('selectedMap');
      var collapsedMap = Vue.inject('collapsedMap');
      var chatWaitingInTopic = Vue.inject('chatWaitingInTopic');
      var chatInProgressInTopic = Vue.inject('chatInProgressInTopic');
      var agentsForTopic = Vue.inject('agentsForTopic');
      var CountDisplay = Vue.inject('CountDisplay');

      var collapseClass = Vue.computed(() => {
        if (collapsedMap[props.topic.id]) return 'fa-plus-circle';
        return 'fa-minus-circle';
      });
      function CollapseClick() {
        collapsedMap[props.topic.id] = !collapsedMap[props.topic.id];
      }

      return { selectedMap, collapsedMap, collapseClass, CollapseClick, chatWaitingInTopic, chatInProgressInTopic, agentsForTopic, CountDisplay };
    }
  }
  iservice.components.manageKBArticles = {
    name: 'vueapp-messagequeue-manage-kbarticles',
    template: '#vueapp-messagequeue-manage-kbarticles',
    route() { return { path: iservice.formPath + '/knowledge-base', priority: 200, name: 'manage-kbarticles', meta: { title: "iService | Manage KB Articles" } } },
    setup() {
      var segments = iservice.SegmentData();
      var topics = iservice.TopicData();
      var agents = iservice.AgentData();
      var loggedIn = iservice.LoggedInData();
      var articles = iservice.ArticleData();

      var featureSearch = Vue.reactive({ working: true, articles: [], alert: false, alertDetails: false });
      featureSearch.AlertClear = function () { featureSearch.alert = false; }

      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }
      var featureUploads = iservice.FeatureUploads();
      featureSearch.details = {};
      function Clear() {
        featureUploads.files = [];
        featureSearch.details.subject = '';
        featureSearch.details.topicID = '';
        featureSearch.details.isPublic = '';
        featureSearch.details.questionHtml = '';
        featureSearch.details.answerHtml = '';
        featureSearch.errors = [];
        featureSearch.success = '';
      }
      Clear();
      featureSearch.HaveRight = function (action) {
        if (action == 'create') return iservice.HaveRight('Tab.FindAnswers.Create');
        else return false;
      }

      iservice.API('webapp-api-findanswer?api=manage', null, { pageNum: 1, perPage: 2500, recursive: true }).then(function(data){
        data.articles.forEach(function(article){
          articles.add(article);
          featureSearch.articles.push(article.id);
        });
      }).catch(errors => {

      }).finally(() => {
        featureSearch.working = false;
      });

      var featureMessages = Vue.reactive({ showQuestions: true, showPending: false });
      var featureTopics = Vue.reactive({ showMine: true, segments: [], list: [], selectedMap: {}, collapsedMap: {}, selectAll: true, collapsed: false });
      var selectAll = featureTopics.selectAll;
      Vue.watchEffect(() => {
        selectAll = featureTopics.selectAll;
      });
      featureTopics.Collapse = function () {
        featureTopics.collapsed = !featureTopics.collapsed;
      }
      Vue.watchEffect(() => {
        featureTopics.collapsedClass = featureTopics.collapsed ? 'col-md-1 treeview-collapse' : 'col-md-4 treeview-expand';
        featureMessages.collapsedClass = featureTopics.collapsed ? 'col-md-11 treeview-expand' : 'col-md-8';
      });
      var selectStartup = true;
      featureTopics.SelectAll = function () {
        selectStartup = false;
        featureTopics.list.forEach(topic => {
          featureTopics.selectedMap[topic.id] = featureTopics.selectAll;
        });
      }
      featureTopics.ShowMine = function () {
        selectStartup = false;
      }
      Vue.watchEffect(() => {
        if (!segments.loaded) segments.Load();
        if (!topics.loaded) topics.Load();
        if (!agents.loaded) agents.Load();
        featureTopics.segments.length = 0;
        var map = {};
        featureTopics.list.length = 0;
        segments.list.forEach(seg => {
          var segmentID = seg.id;
          var segment = Vue.reactive({
            name: seg.name,
            topics: []
          });
          topics.inSegment(segmentID).forEach(topic => {
            if (featureTopics.showMine) {
              var agentList = agents.forTopic(topic.id);
              var me = agentList.find(agentID => {
                var agent = agents.get(agentID);
                return agent.id == loggedIn.contactID;
              });
              if (!me) return;
            }
            var item = map[topic.id] = Vue.reactive({
              id: topic.id,
              name: topic.name,
              visibility: topic.visibility,
              children: []
            });
            featureTopics.list.push(item);
            if (selectStartup) featureTopics.selectedMap[item.id] = selectAll;
          });
          topics.inSegment(segmentID).forEach(topic => {
            var row = map[topic.id];
            if (!row) return;
            var parentID = topic.parentID;
            var parent = map[parentID];
            while (!parent) {
              if (!parentID) break;
              parentID = topics.get(parentID).parentID;
              parent = map[parentID];
            }
            (parent ? parent.children : segment.topics).push(row);
          });
          if (segment.topics.length < 1) return;
          featureTopics.segments.push(segment);
        });
        function SortRow(lrow, rrow) { return topics.TreeSortCompare(lrow.id, rrow.id); }
        featureTopics.segments.sort(SortRow);
        featureTopics.list.forEach(row => {
          row.children.sort(SortRow);
        });
      });
      Vue.watchEffect(() => {
        featureTopics.selectAll = false;
        for (var topic of featureTopics.list) {
          if (!featureTopics.selectedMap[topic.id]) return;
        }
        featureTopics.selectAll = true;
      });
      var featureTopicMenu = Vue.reactive({ options: [], topicID: '' });

      Vue.watchEffect(function () {
        if (!featureSearch.details.topicID) featureSearch.alert = true; else featureSearch.alert = false;
        featureTopicMenu.options.length = 0;
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            group.options.push({ name: topic.name, id: topic.id, level: Math.min(2, topic.depth) });
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.id, r.id); });
          if (group.options.length < 1) return;
          featureTopicMenu.options.push(group);
        });
      })
      function ComputedList() {
        return Vue.computed(() => {
          var inTopic = {};
          featureTopics.list.forEach(topic => {
            inTopic[topic.id] = [];
          });
          featureSearch.articles.forEach(id => {
            var int = articles.get(id);
            if (!int) return;
            var list = inTopic[int.topicID];
            if (!list) return;
            list.push(id);
          });
          return inTopic;
        });
      }
      function CountDisplay(value) {
        if (Array.isArray(value)) value = value.length;
        if (!value || value < 1) return 0;
        if (value < 1000) return '' + value;
        return '1K+';
      }
      Vue.provide('selectedMap', featureTopics.selectedMap);
      Vue.provide('collapsedMap', featureTopics.collapsedMap);
      Vue.provide('CountDisplay', CountDisplay);

      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        componentExpanded: 'kb-article-view-loader',
        filterLabel: 'Filter Articles',
        emptyMessage: 'No articles',
        pageSize: 50,
        expandable: true,
        searching: false,
        initialSort: 'Date',
        exportName: "Manage KB Articles"
      });
      tableConfig.searching = true;
      var articlesInTopic = ComputedList();
      featureSearch.articlesShown = [];
      var inc = 0;
      Vue.watchEffect(function () {
        if (!featureTopics.list.length) return;
        if (featureSearch.keywords) return;
        featureSearch.articlesShown = [];
        featureTopics.list.forEach(topic => {
          if (!featureTopics.selectedMap[topic.id]) return;
          articlesInTopic.value[topic.id].forEach(id => {
            featureSearch.articlesShown.push(id);
          });
          if (inc === featureTopics.list.length) {
            tableConfig.searching = false;
          }
          inc++;
        });
      })
      

      Vue.provide('articlesInTopic', articlesInTopic);
     
      Vue.watchEffect(() => {
        tableConfig.columns = [
          Vue.reactive({
            name: 'expand', hidable: false,
            filterText: function () { },
            showDelete: false,
            componentHead: 'iservice-table-head-empty', componentCell: 'iservice-table-cell-expand-row-edit-delete', shown: true, tdClass: 'expand-td'
          }),
          Vue.reactive({
            name: 'ID', label: 'ID',
            type: 'number',
            filterText: iservice.tables.SimpleFilterTextFunc(articles, 'id'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-article-simple', shown: false
          }),
          Vue.reactive({
            name: 'Subject', label: 'Subject',
            filterText: iservice.tables.SimpleFilterTextFunc(articles, 'subject'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-article-simple', shown: true
          }),
          Vue.reactive({
            name: 'Access Level', label: 'Access Level',
            filterText: iservice.tables.SimpleFilterTextFunc(articles, 'public'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-article-simple', shown: true
          }),
          Vue.reactive({
            name: 'Rating', label: 'Rating',
            filterText: iservice.tables.SimpleFilterTextFunc(articles, 'rating'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-article-simple', shown: true
          }),
          Vue.reactive({
            name: 'Views', label: 'Views',
            filterText: iservice.tables.SimpleFilterTextFunc(articles, 'viewCount'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-article-simple', shown: true
          }),
          Vue.reactive({
            name: 'Topic', label: 'Topic',
            filterText: iservice.tables.SimpleFilterTextFunc(articles, 'topicName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-article-simple', shown: true
          }),
          Vue.reactive({
            name: 'Date', label: 'Date', sortIncreasing: false,
            filterText: iservice.tables.DateFromNowFilterTextFunc(articles, 'date'), sortText: iservice.tables.DateSortTextFunc(articles, 'date'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-date-from-now-articles', shown: false
          }),
          Vue.reactive({
            name: 'delete', hidable: false,
            filterText: function () { },
            showDelete: true,
            componentHead: 'iservice-table-head-empty', componentCell: 'iservice-table-cell-article-simple', shown: true
          })];
      });

      featureSearch.openAddArticle = function () {
        featureSearch.ActionAddArticle = !featureSearch.ActionAddArticle;
      }

      var featureTabs = Vue.reactive({ tab: 'attachments' });
      featureTabs.TabClass = function (tab) { return { 'active': tab == featureTabs.tab }; }
      featureTabs.TabClick = function (tab) {
        featureTabs.tab = tab;
      }
      var featureQuestionTabs = Vue.reactive({ tab: 'body' });
      featureQuestionTabs.TabClass = function (tab) { return { 'active': tab == featureQuestionTabs.tab }; }
      featureQuestionTabs.TabClick = function (tab) {
        featureQuestionTabs.tab = tab;
      }
      var featureAnswerTabs = Vue.reactive({ tab: 'body' });
      featureAnswerTabs.TabClass = function (tab) { return { 'active': tab == featureAnswerTabs.tab }; }
      featureAnswerTabs.TabClick = function (tab) {
        featureAnswerTabs.tab = tab;
      }

      featureSearch.featureFAProps = Vue.reactive({ label: 'Article Properties' });
      featureSearch.topicSelected = function () {

        if (!featureSearch.details.topicID) return;
        var topic = topics.get(featureSearch.details.topicID);
        iservice.EditableProperties(featureSearch.featureFAProps, topic.faProperties);
        Vue.watchEffect(() => {
          featureSearch.haveProperties = featureSearch.featureFAProps.props.length;
        });
        Vue.watchEffect(() => { featureSearch.propertyMissingClass = { 'text-red': featureSearch.featureFAProps.missingRequired }; });
        Vue.watchEffect(() => { featureSearch.hasValues = featureSearch.featureFAProps.hasValues });
      }
      function CheckProperties(feature, label) {
        for (var prop of feature.props) {
          if (!prop.missingRequired) continue;
          prop.focusMe = true;
          featureTabs.TabClick('properties');
          featureSearch.errors = ['Must set value for ' + label + ' property ' + prop.def.name + '.'];
          featureSearch.startLoading = false;
          return false;
        }
        return true;
      }
      var errors = Vue.reactive({});
      featureSearch.addArticle = function () {
        featureSearch.errors = [];
        if (!featureSearch.details.subject) {
          featureSearch.errors.push({ msg: 'Please enter a Subject.' });
          errors.subject = true;
        }
        if (!featureSearch.details.topicID) {
          featureSearch.errors.push({ msg: 'Please select a Topic.' });
          errors.topicID = true;
        }
        if (!featureSearch.details.isPublic) {
          featureSearch.errors.push({ msg: 'Please select a Access Level.' });
          errors.isPublic = true;
        }
        if (!featureSearch.details.questionHtml) {
          featureSearch.errors.push({ msg: 'Please enter a Question Body.' });
          errors.questionHtml = true;
        }
        if (!featureSearch.details.answerHtml) {
          featureSearch.errors.push({ msg: 'Please enter a Answer Body.' });
          errors.answerHtml = true;
        }
        if (featureSearch.errors.length) return;
        featureSearch.startLoading = true;
        if (!CheckProperties(featureSearch.featureFAProps, 'article')) return;
        featureSearch.details.properties = featureSearch.featureFAProps.out;
        featureSearch.details.propertiesEdit = featureSearch.featureFAProps.out;
        iservice.API('webapp-api-findanswer?api=create', null, { details: featureSearch.details }, featureUploads.files).then((response) => {
          articles.add(response.row);
          featureSearch.articlesShown.push(response.row.id);
          iservice.SuccessMessage('Article added successfully.');
          featureSearch.openAddArticle();
          Clear();

        }).catch(function (errors) {
          featureSearch.errors = errors;
          featureSearch.startLoading = false;
        }).finally(function () {
          featureSearch.startLoading = false;
        });
        
      }
      featureSearch.UpdateRow = function (article, state, messages) {
        iservice.API('webapp-api-findanswer?api=details', null, {
          articleID: article.id
        }).then((data) => {
          var details = {};
          iservice.EditableProperties(featureSearch.featureFAProps, data.propertiesEdit);
          details.propertiesEdit = featureSearch.featureFAProps.out;
          details.properties = featureSearch.featureFAProps.out;
          var attachments = [];
          if (data.attachments) {
            data.attachments.forEach(function (attachment) {
              attachments.push(attachment.attachmentID);
            })
          }
          details.questionHtml = data.questionHtml;
          details.answerHtml = data.answerHtml;
          details.subject = article.editSubject;
          details.public = article.editpublic;
          details.attachments = attachments;
          details.topicID = data.topicID;
          details.isPublic = details.public == 'public' ? true : false;
          iservice.API('webapp-api-findanswer?api=modify', null, { articleID: article.id, details: details }, featureUploads.files).then(function(response){
            iservice.SuccessMessage("Article updated successfully.");

            state(article.id)
          }).catch(function (errors) {
            messages("", errors);
          });
        }).catch(function (errors) {

        });
      }
      featureSearch.deleteFunction = function (articleID) {
        iservice.API('webapp-api-findanswer?api=delete', null, { articleID: articleID }).then((data) => {
          const index = featureSearch.articlesShown.indexOf(articleID);
          if (index > -1) {
            featureSearch.articlesShown.splice(index, 1);
          }
          iservice.SuccessMessage("Article deleted successfully.");
        }).catch(function (errors) {
          featureSearch.errors = errors;
        });
      }
      var editor1 = Vue.ref(null);
      var anchorStocksQuestion = Vue.ref(null);
      var featureStockResponsesQuestion = Vue.reactive({ filter: '', segmentID: '', hidevariable: true  });
      Vue.watchEffect(function () {
        if (featureSearch.details.topicID) {
          var segmentID = topics.get(featureSearch.details.topicID).segmentID;
          featureStockResponsesQuestion.segmentID = segmentID
        } else {
          featureStockResponsesQuestion.isDisabled = true;
        }
      })
      featureStockResponsesQuestion.Open = function () {
        featureStockResponsesQuestion.Close = iservice.components.popupHolder.Open(anchorStocksQuestion, 'stock-responses-menu', false, featureStockResponsesQuestion);
      }
      featureStockResponsesQuestion.PasteVar = function (text) {
        if (!editor1 || !editor1.value) return;
        if (featureStockResponsesQuestion.Close) featureStockResponsesQuestion.Close();
        editor.value.PasteText(text);
      }
      featureStockResponsesQuestion.PasteStock = function (html) {
        if (!editor1 || !editor1.value) return;
        if (featureStockResponsesQuestion.Close) featureStockResponsesQuestion.Close();
        editor1.value.PasteHtml(html);
      }
      var editor2 = Vue.ref(null);
      var anchorStocksAnswer = Vue.ref(null);
      var featureStockResponsesAnswer = Vue.reactive({ filter: '', segmentID: '', hidevariable: true });
      Vue.watchEffect(function () {
        if (featureSearch.details.topicID) {
          var segmentID = topics.get(featureSearch.details.topicID).segmentID;
          featureStockResponsesAnswer.segmentID = segmentID
        } else {
          featureStockResponsesAnswer.isDisabled = true;
        }
      })
      featureStockResponsesAnswer.Open = function () {
        featureStockResponsesAnswer.Close = iservice.components.popupHolder.Open(anchorStocksAnswer, 'stock-responses-menu', false, featureStockResponsesAnswer);
      }
      featureStockResponsesAnswer.PasteVar = function (text) {
        if (!editor2 || !editor2.value) return;
        if (featureStockResponsesAnswer.Close) featureStockResponsesAnswer.Close();
        editor.value.PasteText(text);
      }
      featureStockResponsesAnswer.PasteStock = function (html) {
        if (!editor2 || !editor2.value) return;
        if (featureStockResponsesAnswer.Close) featureStockResponsesAnswer.Close();
        editor2.value.PasteHtml(html);
      }
      featureSearch.getSearchedArticles = function () {
        featureSearch.searchingData = true;
        featureSearch.articlesShown = [];
        iservice.API('webapp-api-findanswer?api=articles', null, { topicID: "", pageNum: 1, perPage: 1000, recursive: false, searchString: featureSearch.keywords }, null).then(function (data) {
          featureSearch.articlesShown = [];
          data.interactions.forEach((item) => {
            featureSearch.articlesShown.push(item.id);
          });

          featureSearch.searchingData = false;
        });
      }
      featureSearch.Reset = function () {
        featureSearch.keywords = '';
      }
      return { featureSearch, featureTopics, featureMessages, tableConfig, featureTopicMenu, featureUploads, featureTabs, featureQuestionTabs, featureAnswerTabs, anchorStocksQuestion, anchorStocksAnswer, editor1, editor2, featureStockResponsesQuestion, featureStockResponsesAnswer, errors, ListCollapsed };
    }
  }
  iservice.components.manageMessagesTopicRow = {
    name: 'manage-messages-topic-row',
    template: '#manage-messages-topic-row',
    props: ['topic'],
    setup(props) {
      var selectedMap = Vue.inject('selectedMap');
      var collapsedMap = Vue.inject('collapsedMap');
      var queuedInTopic = Vue.inject('queuedInTopic');
      var unassignedInTopic = Vue.inject('unassignedInTopic');
      var pendingInTopic = Vue.inject('pendingInTopic');
      var agentsForTopic = Vue.inject('agentsForTopic');
      var CountDisplay = Vue.inject('CountDisplay');

      var collapseClass = Vue.computed(() => {
        if (collapsedMap[props.topic.id]) return 'fa-plus-circle';
        return 'fa-minus-circle';
      });
      function CollapseClick() {
        collapsedMap[props.topic.id] = !collapsedMap[props.topic.id];
      }

      return { selectedMap, collapsedMap, collapseClass, CollapseClick, queuedInTopic, unassignedInTopic, pendingInTopic, agentsForTopic, CountDisplay };
    }
  }
  iservice.components.manageArticlesTopicRow = {
    name: 'manage-articles-topic-row',
    template: '#manage-articles-topic-row',
    props: ['topic'],
    setup(props) {
      var selectedMap = Vue.inject('selectedMap');
      var collapsedMap = Vue.inject('collapsedMap');
      var CountDisplay = Vue.inject('CountDisplay');
      var articlesInTopic = Vue.inject('articlesInTopic');

      var collapseClass = Vue.computed(() => {
        if (collapsedMap[props.topic.id]) return 'fa-plus-circle';
        return 'fa-minus-circle';
      });
      function CollapseClick() {
        collapsedMap[props.topic.id] = !collapsedMap[props.topic.id];
      }

      return { selectedMap, collapsedMap, collapseClass, CollapseClick, CountDisplay, articlesInTopic };
    }
  }

  iservice.components.manageLoggedInChatAgents = {
    name: 'vueapp-templates-messagequeue-manage-chatagents',
    template: '#vueapp-templates-messagequeue-manage-chatagents',
    route() { return { path: iservice.formPath + '/logged-in-chat-agents', priority: 200, name: 'manage-chatagents', meta: { title: "iService | Logged In Chat Agents" } } },
    setup() {
      var formFeatures = Vue.reactive({ errors: [], success: ''});
      var agents = iservice.AgentData();
      var loggedIn = iservice.LoggedInData();
      formFeatures.HaveRight = iservice.HaveRight('Tab.MessageQueue.SuperviseChat');
      if (!agents.loaded) {
        agents.Load();
      }
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }
      var tableColumns = Vue.computed(() => {
        var columns = [
          Vue.reactive({
            name: 'Available', label: 'Available',
            filterText: iservice.tables.SimpleFilterTextFunc(agents, 'isAvailable'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-available', shown: true
          }),
          Vue.reactive({
            name: 'Agent Name', label: 'Agent Name',
            filterText: iservice.tables.SimpleFilterTextFunc(agents, 'displayname'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Assigned', label: 'Assigned',
            filterText: iservice.tables.SimpleFilterTextFunc(agents, 'countAssignedString'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Completed Today', label: 'Completed Today',
            filterText: iservice.tables.SimpleFilterTextFunc(agents, 'countResolvedString'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: "Today's Activity:", label: "Today's Activity:",
            filterText: iservice.tables.SimpleFilterTextFunc(agents, 'countAssigned'),
            isTodayActivity: true,
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-today-activity', shown: true
          })
        ];

        return columns;
      });
      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        filterLabel: 'Filter Agents',
        emptyMessage: 'No logged in Agents.',
        pageSize: 5,
        expandable: true,
        searching: false,
        initialSort: 'Agent Name',
        exportName: "Logged in Chat Agents",
        notclickable: true,
        columns: tableColumns.value
      });

      var websyncData = {
        unsubForSegment: {},
        Clear() {
          for (var id in websyncData.unsubForSegment) {
            var unsub = websyncData.unsubForSegment[id];
            if (unsub) unsub();
            websyncData.unsubForSegment[id] = null;
          }
        }
      };
      Vue.onUnmounted(() => {
        websyncData.Clear();
      });
      iservice.WatchLoginChange(() => {
        websyncData.Clear();
        for (var segmentID of loggedIn.accessibleSegments) {
          websyncData.unsubForSegment[segmentID] = websync.subscribe('/supervisechats/' + loggedIn.tenantID + '/' + segmentID, ManageChatsKeep);
        }
      });
      function ManageChatsKeep(chat) {
        if (chat.getData().inboxUpdate.type == 'Chat') {
          loadData();
          return true;
        }
        return false;
      }
      Vue.watchEffect(function () {
        let data = loggedIn.chats;
        loadData();
      });
      var IDs = Vue.reactive([]);
      var itemsProcessed = 0;
      tableConfig.searching = true;
      var loaded = false;
      Vue.watchEffect(function () {
        if (!agents.list.length || !formFeatures.chats) return;
        if (loaded) return;
        agents.list.forEach(function (agent) {
          var id = agent.id;
          agent["countAssigned"] = 0;
          agent["countResolved"] = 0;
          agent["countAssignedString"] = agent.countAssigned.toString();
          agent["countResolvedString"] = agent.countResolved.toString();
          agents.add(agent);
          if (agent.isLoggedIn == true) {
            IDs.push(id);
          }
          itemsProcessed++;
          if (itemsProcessed == agents.list.length) {
            tableConfig.searching = false;
            loaded = true;
          }
        });
        Recount();
      });
      Vue.watchEffect(function () {
        if (!agents.list.length || !formFeatures.chats) return;
        agents.list.forEach(function (agent) {
          if (agent.isLoggedIn == true) {
            let index = IDs.findIndex(x => x == agent.id);
            if (index == -1) {
              IDs.push(agent.id);
            }
          } else {
            let index = IDs.findIndex(x => x == agent.id);
            if (index != -1) {
              IDs.splice(index, 1);
            }
          }
        });
        Recount();
      });
      

      function Recount() {
        if (!formFeatures.chats) return;
        agents.list.forEach(function (agent) {
          agent.countAssigned = 0;
          agent.countResolved = 0;
          agent.countAssignedString = agent.countAssigned.toString();
          agent.countResolvedString = agent.countResolved.toString();
        });
        formFeatures.chats.forEach(function (chat) {
          if (chat.assignedToID) {
            var agent = agents.get(chat.assignedToID);
            if (agent) {
              agent.countAssigned += 1;
              agent.countAssignedString = agent.countAssigned.toString();
            }
          }
          else if (chat.resolvedByID) {
            var agent = agents.get(chat.resolvedByID);
            if (agent) {
              agent.countResolved += 1;
              agent.countResolvedString = agent.countResolved.toString();
            }
          }
        });
      }

      function loadData() {
        var search = { timeStartUtc: moment().utc().startOf('day').subtract(0, 'days').format(), groups: [{ fields: [{ fieldID: 'type', searchString: 'Chat' }] }] };
        iservice.API('webapp-api-mq-managechats?api=agentsummary', null, { fields: search }).then(function (result) {
          //add new row to the list.
          formFeatures.chats = result.chats;
          Recount();
        }).catch(function (errors) {
          formFeatures.errors = errors;
        }).finally(function () {
        });
      }
      loadData();
      return { formFeatures, agents, tableConfig, ListCollapsed, IDs };
    }
  }

  iservice.components.messageQueueSearch = {
    name: 'vueapp-messagequeue-search',
    template: '#vueapp-messagequeue-search',
    route() { return { path: iservice.formPath + '/search-interactions', priority: 300, meta: { title: "iService | Search Interactions" } } },
    setup() {

      var router = VueRouter.useRouter();

      var formFeatures = Vue.reactive({ limitDateBefore: '', limitDateAfter: '', limitResultRoot: false, totalResults: 0, topicRef: [], agentRef: [], agents: [] })

      formFeatures.HaveRight = iservice.HaveRight('Tab.Top.MessageQueue');

      var fieldGroups = Vue.reactive({});
      fieldGroups.searchGroups = [[{
          where: 'anywhere',
          field: 'subject',
          text: ''
        }]];
      var maxFields = 10;
      var currentQuery = Vue.reactive({})
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function() {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }
      var interactions = iservice.InteractionData();
      var intProperties = iservice.IntPropertyData();
      var topics = iservice.TopicData();
      var agents = iservice.AgentData();
      if (!topics.loaded) topics.Load();
      Vue.watchEffect(function () {
        if (!agents.loaded) {
          agents.Load(); return;
        }
        agents.list.forEach(function (agent) {
          var name = agent.name || agent.email || ('Agent ' + agent.id);
          var option = {
            id: agent.id,
            name: name,
            displayname: agent.displayname,
            email: agent.email,
            search: agent.name + '\n' + agent.email,
            isaway: !agent.isLoggedIn && !agent.doEmpty
          };
          formFeatures.agents.push(option);
        })
      })
      var tableColumns = Vue.computed(() => {
        var columns = [
          Vue.reactive({
            name: 'expand', hidable: false,
            filterText: function () { },
            componentHead: 'iservice-table-head-empty', componentCell: 'iservice-table-cell-expand', shown: true, tdClass: 'expand-td' }),
          Vue.reactive({
            name: 'Subject', label: 'Subject',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'subject'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true }),
          Vue.reactive({
            name: 'Topic', label: 'Topic',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'topicName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true }),
          Vue.reactive({
            name: 'Status', label: 'Status',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'status'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'interaction-status-display', shown: false, queryString: 'cv1', var: 'status' }),
          Vue.reactive({
            name: 'Interaction Type', label: 'Interaction Type',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'type'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'interaction-type-display', shown: false, queryString: 'cv2', var: 'type' }),
          Vue.reactive({
            name: 'Contact Name', label: 'Contact Name',
            segment: false,
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'customerName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-interaction-customername-linked', shown: true}),
          Vue.reactive({
            name: 'Contact Email', label: 'Contact Email',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'fromAddress'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-interaction-from-linked', shown: true }),
          Vue.reactive({
            name: 'Date', label: 'Date', sortIncreasing: false,
            filterText: iservice.tables.DateFilterTextFunc(interactions, 'date'), sortText: iservice.tables.DateSortTextFunc(interactions, 'date'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true }),
          Vue.reactive({
            name: 'Assigned To', label: 'Assigned To',
            filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'agentName'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true }) ];

        if (!intProperties.loaded) {
          intProperties.Load();
          return columns;
        }
        var inc = 3
        intProperties.list.forEach(prop => {
          var column = Vue.reactive({
            name: 'int-property-' + prop.id,
            label: prop.name,
            filterText: iservice.tables.IntPropValueFilterTextFunc(prop.id),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-inbox-property',
            shown: false,
            propertyID: prop.id,
            segmentName: prop.segmentName,
            queryString: 'cv' + inc,
            var: 'property' + prop.id
          });
          columns.push(column);
          inc++;
        });
        return columns;
      });

      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        componentExpanded: 'interaction-view-loader',
        filterLabel: 'Filter Results',
        emptyMessage: 'No Interactions',
        pageSize: 5,
        expandable: true,
        searching: false,
        initialSort: 'Date',
        exportName: "Search Interactions",
        columns: tableColumns.value
      });
      Vue.watchEffect(() => {
        tableConfig.columns = tableColumns.value;
      });
      var IDs = Vue.reactive([]);
      var searchLists = Vue.reactive({ currentSearch: {} });

      searchLists.showWhereList = function (field)
      {
        return !searchLists.showSearchTopic(field) &&
          !searchLists.showSearchNone(field) &&
          !searchLists.showSearchStatus(field) &&
          !searchLists.showSearchType(field) &&
          !searchLists.showSearchAgent(field);
      }
      searchLists.showSearchTopic = function (field)
      {
        return field == 'topic';
      }

      searchLists.showSearchNone = function (field)
      {
        return field == '->threads';
      }

      searchLists.showSearchStatus = function(field)
      {
        return field == 'status';
      }

      searchLists.showSearchType = function(field)
      {
        return field == 'type';
      }

      searchLists.showSearchAgent = function (field)
      {
        return field == 'resolve' || field == 'assign';
      }
      searchLists.splitGroup = function (group, field) {
        fieldGroups.groups = [];
        for (var i = 0, len = fieldGroups.searchGroups.length; i < len; i++) {
          var cur = fieldGroups.searchGroups[i];

          if (cur !== group)
            fieldGroups.groups.push(cur);
          else {
            var partial = Vue.reactive([]);

            fieldGroups.groups.push(partial);

            for (var f = 0; f < group.length; f++) {
              partial.push(group[f]);

              if (group[f] === field) {
                partial = [];
                fieldGroups.groups.push(partial);
              }
            }
          }
        }
        fieldGroups.searchGroups = fieldGroups.groups;
      }

      searchLists.canAdd = function ()
      {
        var count = 0;

        for (var i = 0, len = fieldGroups.searchGroups.length; i < len; i++)
          count += fieldGroups.searchGroups[i].length;
        return count < maxFields;
      }

      searchLists.canRemove = function ()
      {
        return fieldGroups.searchGroups.length === 1 && fieldGroups.searchGroups[0].length === 1 ? false : true;
      }
      searchLists.addGroup = function()
      {
        if (!searchLists.canAdd())
          return;

        fieldGroups.searchGroups.push([{ where: 'anywhere', field: 'all', text: '' }]);
      }
      searchLists.removeField = function(field)
      {
        fieldGroups.groups = [];
       
        for (var g = 0; g < fieldGroups.searchGroups.length; g++) {
          var group = null;

          for (var f = 0, len = fieldGroups.searchGroups[g].length; f < len; f++) {
            var cur = fieldGroups.searchGroups[g][f];

            if (cur !== field) {
              if (!group)
                group = [];

              group.push(cur);
            }
          }

          if (group)
            fieldGroups.groups.push(group);
        }
        fieldGroups.searchGroups = fieldGroups.groups;
      }
      searchLists.mergeGroup = function (group)
      {
        fieldGroups.groups = [];

        for (var i = 0, len = fieldGroups.searchGroups.length; i < len; i++) {
          var cur = fieldGroups.searchGroups[i];

          fieldGroups.groups.push(cur);

          if (cur === group) {
            var next = fieldGroups.searchGroups[i + 1];
            for (var f = 0; f < next.length; f++)
              cur.push(next[f]);

            i++;
          }
        }
        fieldGroups.searchGroups = fieldGroups.groups;
      }


      function setTopicRef(el) {
        if (el) {
          formFeatures.topicRef.push(el)
        }
      }
      function setAgentRef(el) {
        if (el) {
          formFeatures.agentRef.push(el)
        }
      }
      Vue.onBeforeUpdate(function () {
        formFeatures.topicRef = [];
        formFeatures.agentRef = [];
      });
      var loaded = false;
      Vue.onUpdated(function () {
        Vue.watchEffect(function () {
          if (loaded) return;
          if (!topics.list.length) return
          if (!agents.list.length) return
          currentQueryParams = router.currentRoute.value.query;
          for (var i = 1; i <= maxFields; i++) {
            var t = currentQueryParams['t' + i]
            var topic = topics.get(t);
            var agent = agents.get(t);

            if (topic) {
              if (formFeatures.topicRef[i - 1]) {
                formFeatures.topicRef[i - 1].select(topic);
              }
            }
            if (agent) {
              if (formFeatures.agentRef[i - 1]) {
                formFeatures.agentRef[i - 1].select(agent);
              }
            }
          }
          loaded = true;
        })
      });
      
      var segments = iservice.SegmentData();
      if (!segments.loaded) {
        segments.Load();
      }
      Vue.watchEffect(function () {
        if (!topics.list.length) return;
        formFeatures.topics = topics.list;
      })
      var featureTopicMenu = Vue.reactive({ options: [], topicID: '' });
      
      Vue.watchEffect(function () {
        featureTopicMenu.options.length = 0;
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            group.options.push({ name: topic.name, id: topic.id, level: Math.min(2, topic.depth) });
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.id, r.id); });
          if (group.options.length < 1) return;
          featureTopicMenu.options.push(group);
        });
      })
      formFeatures.showMoreToggle = function () {
        formFeatures.showmore = !formFeatures.showmore;
      }

      var queryParams = Vue.reactive({});
      formFeatures.Search = function (e) {
        queryParams = Vue.reactive({});
        var fieldNum = 1, lastOp;
        if (formFeatures.limitDateAfter) 
          queryParams.after = moment.utc(formFeatures.limitDateAfter).format();

        if (formFeatures.limitDateBefore)
          queryParams.before = moment.utc(formFeatures.limitDateBefore).format();
     
        if (formFeatures.limitResultRoot)
          queryParams.limit = true;
        
        for (var i = 0, groups = fieldGroups.searchGroups.length; i < groups; i++) {
          var group = fieldGroups.searchGroups[i];

          for (var j = 0, fields = group.length; j < fields; j++) {
            var field = group[j];

            if (lastOp)
              queryParams['op' + (fieldNum - 1)] = lastOp;

            queryParams['w' + fieldNum] = field.where;
            queryParams['f' + fieldNum] = field.field;
            queryParams['t' + fieldNum] = field.text;
            lastOp = 'or';

            fieldNum++;
          }

          lastOp = 'and';
        }
        router.push({ query: queryParams });
        const isEqual = (...objects) => objects.every(obj => JSON.stringify(obj) === JSON.stringify(objects[0]));
        if (isEqual(searchLists.currentSearch, currentQuery)) {
          doSearch(); //search callback
        } else {
          //ChangeLocation(this.route.current.params);
          doSearch();
        }
        

      }
      initialize();
      formFeatures.loading = true;
      function doSearch() {

        formFeatures.param = {
          groups: []
        };

        if (formFeatures.limitDateAfter) {
          formFeatures.param.timeStartUtc = moment.utc(formFeatures.limitDateAfter).format();
        }

        if (formFeatures.limitDateBefore) {
          formFeatures.param.timeEndUtc = moment.utc(formFeatures.limitDateBefore).format();
        }
        collectParameters(formFeatures.param)
        formFeatures.param.groups.push({ fields: [{ where: 'entire', fieldID: 'not-findanswers', searchString: '' }] });

        if (formFeatures.limitResultRoot) {
          formFeatures.param.groups.push({ fields: [{ where: 'entire', fieldID: '->threads', searchString: '' }] });
        }
        tableConfig.searching = true;
        iservice.API('webapp-api-mq-search?api=search', null, { fields: formFeatures.param, pageNum: 0, perPage: 2500, sort: null }).then(data => {
          IDs.length = 0;
          formFeatures.totalResults = data.interactions.length;
          data.interactions.forEach(item => {
            var id = item.id;
            interactions.add(item);
            IDs.push(id);
          });
        }).finally(() => {
          tableConfig.searching = false;
          formFeatures.loading = false;
        });
      }
      function Clear() {
        formFeatures.totalResults = null;
      }

      interactions.WhenDeleted(id => {
        var index = IDs.indexOf(id);
        if (index == -1) return;
        IDs.splice(index, 1);
      });

      function collectParameters(params) {
        for (var g = 0, numGroup = fieldGroups.searchGroups.length; g < numGroup; g++) {
          var group = Vue.reactive({
            fields: []
          });

          params.groups.push(group);

          for (var f = 0, numField = fieldGroups.searchGroups[g].length; f < numField; f++) {
            var field = fieldGroups.searchGroups[g][f];

            group.fields.push({
              where: field.where,
              fieldID: field.field,
              searchString: field.text
            });
          }
        }
      }
      function initialize()
      {
       
        currentQuery = router.currentRoute.value.query;

        //enable disable column visibility on basis of query strings
        for (var i = 0; i < tableConfig.columns.length; i++) {
          if (typeof tableConfig.columns[i].queryString != 'undefined') {
            if (currentQuery.hasOwnProperty(tableConfig.columns[i].queryString)) {
              tableConfig.columns[i].shown = true;
            }
          }
        }
        if (currentQuery.f1 !== undefined) {
          fieldGroups.searchGroups = Vue.reactive([[]]);

          const dateNow = new Date();

          // Before and after dates are used in MQ search only
          if (currentQuery.before) {
            searchLists.currentSearch.before = currentQuery.before;
            formFeatures.limitDateBefore = moment(currentQuery.before).format('YYYY-MM-DD hh:mm A'); 
          }

          if (currentQuery.after) {
            searchLists.currentSearch.after = currentQuery.after;
            formFeatures.limitDateAfter = moment(currentQuery.after).format('YYYY-MM-DD hh:mm A'); 
          }
          if (currentQuery.limit) {
            searchLists.currentSearch.limit = currentQuery.limit;
            formFeatures.limitResultRoot = true;
          }

          var group = fieldGroups.searchGroups[0];

          for (var i = 1; i <= maxFields; i++) {
            var w = currentQuery['w' + i],
              f = currentQuery['f' + i],
              t = currentQuery['t' + i],
              op = currentQuery['op' + i],
              field = {
                where: w !== undefined ? w : 'anywhere',
                field: f !== undefined ? f : 'subject',
                text: t !== undefined ? t : ''
              };
            field.where = field.where.startsWith('ent') ? 'entire'
              : field.where.startsWith('s') ? 'start'
                : field.where.startsWith('end') ? 'end' : 'anywhere';

            group.push(field);

            searchLists.currentSearch['w' + i] = w;
            searchLists.currentSearch['f' + i] = f;
            searchLists.currentSearch['t' + i] = t;          

            if (op === undefined || i === maxFields)
              break;

            searchLists.currentSearch['op' + i] = op;

            if (op.startsWith('a')) {
              group = [];

              fieldGroups.searchGroups.push(group);
            }
          }

          doSearch();
        }
      }
      formFeatures.Reset = function () {
        router.replace({ query: {} });
        setTimeout(() => router.go(), 200);
      }
      
      return { fieldGroups, maxFields, searchLists, ListCollapsed, formFeatures, IDs, tableConfig, Clear, featureTopicMenu, setTopicRef, setAgentRef, agents};
    }
  }
  iservice.components.kbarticleViewLoader = {
    name: 'kb-article-view-loader',
    template: '#kb-article-view-loader',
    props: ['itemid'],
    setup(props) {
      var articles = iservice.ArticleData();

      var viewComponent = Vue.ref('');
      var loading = Vue.ref(true);
      iservice.API('webapp-api-findanswer?api=details', null, { articleID: props.itemid }).then(data => {
        articles.add(data);
        viewComponent.value = 'kb-article-view';
      }).catch(errors => {

      }).finally(() => {
        loading.value = false;
      });

      return { loading, viewComponent };
    }
  };
  iservice.components.kbArticleView = {
    name: 'kb-article-view',
    template: '#kb-article-view',
    props: ['itemid', 'type'],
    setup(props) {
      var articles = iservice.ArticleData();
      var topics = iservice.TopicData();
      var contacts = iservice.ContactsData();
      var ViewOperation = Vue.reactive({ component: '', type: 'interaction' });
      ViewOperation.Click = function (component) {
        ViewOperation.component = component;
      }

      var Columns = Vue.computed(() => {
        var columns = [
          Vue.reactive({
            name: 'ID', label: 'ID',
            type: 'number',
            filterText: iservice.tables.SimpleFilterTextFunc(contacts, 'id'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: false
          }),
          Vue.reactive({
            name: 'Name', label: 'Name',
            filterText: iservice.tables.SimpleFilterTextFunc(contacts, 'name'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-contact-name-linked', shown: true
          }),
          Vue.reactive({
            name: 'Email', label: 'Email',
            filterText: iservice.tables.SimpleFilterTextFunc(contacts, 'email'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-contact-email-linked', shown: true
          }),
          Vue.reactive({
            name: 'Company', label: 'Company',
            filterText: iservice.tables.SimpleFilterTextFunc(contacts, 'company'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Phone', label: 'Phone',
            filterText: iservice.tables.SimpleFilterTextFunc(contacts, 'phone'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          })];
        return columns;
      });
      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        filterLabel: 'Filter Subscribers',
        emptyMessage: 'No matching results',
        pageSize: 5,
        expandable: true,
        searching: false,
        initialSort: 'Name',
        columns: Columns.value,
        exportName: "Article Subscribers List"
      });
      var article = Vue.computed(function () {
        return articles.get(props.itemid);
      });
      var articleData = Vue.reactive({});

      articleData.topicID = article.value.topicID;
      articleData.questionHtml = article.value.questionHtml;
      articleData.answerHtml = article.value.answerHtml;

      var segmentID = Vue.computed(function () {
        return topics.get(article.value.topicID).segmentID;
      })
      var featureAction = Vue.reactive({});

      var displayQuestionHtml = Vue.ref('');
      var displayAnswerHtml = Vue.ref('');
      var isInline = Vue.reactive({});
      displayQuestionHtml.value = iservice.FindInlineAttachments(article.value.questionHtml, article.value.attachments, isInline);
      displayAnswerHtml.value = iservice.FindInlineAttachments(article.value.answerHtml, article.value.attachments, isInline);
      var realAttachments = Vue.ref(iservice.DownloadableFiles(article.value.attachments, isInline));
      var IDs = Vue.reactive([]);
      var featureDetails = Vue.reactive({ tab: 'details' });
      featureDetails.showMoreToggle = function () {
        featureDetails.showmore = !featureDetails.showmore;
      }
      featureDetails.TabClass = function (tab) { return { 'active': tab == featureDetails.tab }; }
      featureDetails.TabClick = function (tab) {
        tableConfig.searching = true;
        featureDetails.searchingData = true;
        featureDetails.tab = tab;
        if (tab == 'subscribers') {
          iservice.API('webapp-api-custinfo?api=search', null, {
            fields: {
              groups: [{
                fields: [{
                  where: 'entire',
                  fieldID: 'subscribed',
                  searchString: 'article-' + props.itemid
                }]
              }]
            }
          }).then(function (result) {
            featureDetails.currentSubscribers = result.contacts;
            featureDetails.numContacts = result.resultCount;
            result.contacts.forEach(function (contact) {
              contacts.add(contact);
              IDs.push(contact.id);
            });
            tableConfig.searching = false;
            featureDetails.searchingData = false;
          }).catch(function (errors) {
            tableConfig.searching = false;
          }).finally(function () {
            tableConfig.searching = false;
          });
        }
      }
      featureDetails.sendNotification = function () {
        iservice.API('webapp-api-findanswer?api=sendnotification', null, { articleID: props.itemid }).then(function (result) {
          iservice.SuccessMessage("Notifications sent to " + result.count + " subscribers.");
        }).catch(function (errors) {
          featureAction.errors = errors;
        }).finally(function () {

        });
      }

      var featureProps = Vue.reactive({});
      Vue.watchEffect(() => {
        if (!article.value) return;
        featureProps.label = 'Article Properties'
      });
      iservice.EditableProperties(featureProps, article.value.propertiesEdit);

      Vue.watchEffect(() => { featureDetails.haveProperties = featureProps.props.length; });
      Vue.watchEffect(() => { featureDetails.propertyMissingClass = { 'text-red': featureProps.missingRequired }; });
      Vue.watchEffect(() => { featureDetails.hasValues = featureProps.hasValues; });
      Vue.provide('featureDetails', featureDetails)
      var editor = Vue.ref(null);
      var anchorStocks = Vue.ref(null);
      var featureStockResponses = Vue.reactive({ filter: '', segmentID: segmentID.value, hidevariable: true  });
      featureStockResponses.Open = function () {
        featureStockResponses.Close = iservice.components.popupHolder.Open(anchorStocks, 'stock-responses-menu', false, featureStockResponses);
      }
      featureStockResponses.PasteVar = function (text) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteText(text);
      }
      featureStockResponses.PasteStock = function (html) {
        if (!editor || !editor.value) return;
        if (featureStockResponses.Close) featureStockResponses.Close();
        editor.value.PasteHtml(html);
      }
      var editor2 = Vue.ref(null);
      var anchorStocksAnswer = Vue.ref(null);
      var featureStockResponsesAnswer = Vue.reactive({ filter: '', segmentID: segmentID.value, hidevariable: true  });
      featureStockResponsesAnswer.Open = function () {
        featureStockResponsesAnswer.Close = iservice.components.popupHolder.Open(anchorStocksAnswer, 'stock-responses-menu', false, featureStockResponsesAnswer);
      }
      featureStockResponsesAnswer.PasteVar = function (text) {
        if (!editor2 || !editor2.value) return;
        if (featureStockResponsesAnswer.Close) featureStockResponsesAnswer.Close();
        editor.value.PasteText(text);
      }
      featureStockResponsesAnswer.PasteStock = function (html) {
        if (!editor2 || !editor2.value) return;
        if (featureStockResponsesAnswer.Close) featureStockResponsesAnswer.Close();
        editor2.value.PasteHtml(html);
      }
      Vue.provide('featureProps', featureProps);
      Vue.provide('articleData', articleData);
      function Clear() {
        featureDetails.numContacts = null;
      }
      featureDetails.UpdateArticle = function () {
        var details = {};
        featureDetails.loading = true;
        var attachments = [];
        if (article.value.attachments) {
          article.value.attachments.forEach(function (attachment) {
            attachments.push(attachment.attachmentID);
          });
        }
        details.attachments = attachments;
        details.questionHtml = articleData.questionHtml;
        details.answerHtml = articleData.answerHtml;
        details.propertiesEdit = featureProps.out;
        details.properties = featureProps.out;
        details.subject = article.value.subject;
        details.topicID = articleData.topicID;
        details.isPublic = article.value.public == 'public' ? true : false;
        iservice.API('webapp-api-findanswer?api=modify', null, { articleID: article.value.id, details: details }, null).then((response) => {
          iservice.SuccessMessage("Article updated successfully.");
        }).catch(function (errors) {
          featureDetails.loading = false;
          featureAction.errors = errors;
        }).finally(function () {
          featureDetails.loading = false;
        });
      }

      var featureQuestionTabs = Vue.reactive({ tab: 'body' });
      featureQuestionTabs.TabClass = function (tab) { return { 'active': tab == featureQuestionTabs.tab }; }
      featureQuestionTabs.TabClick = function (tab) {
        featureQuestionTabs.tab = tab;
      }
      var featureAnswerTabs = Vue.reactive({ tab: 'body' });
      featureAnswerTabs.TabClass = function (tab) { return { 'active': tab == featureAnswerTabs.tab }; }
      featureAnswerTabs.TabClick = function (tab) {
        featureAnswerTabs.tab = tab;
      }
      featureAction.HaveRight = function (action) {
        if (action == 'modify') return iservice.HaveRight('Tab.FindAnswers.Modify');
        else if (action == 'subscribe-notification') return iservice.HaveRight('Tab.FindAnswers.Subscribe.Notify');
        else return false;
      }
      
      return { article, articleData, realAttachments, displayQuestionHtml, displayAnswerHtml, featureDetails, featureProps, featureAction, ViewOperation, featureStockResponses, editor, anchorStocks, editor2, featureStockResponsesAnswer, anchorStocksAnswer, tableConfig, IDs, Clear, featureQuestionTabs, featureAnswerTabs};
    }
  }
  iservice.components.articleDetailsPanel = {
    name: 'article-details-panel',
    template: '#article-details-panel',
    props: ['itemid'],
    setup(props) {
      var articles = iservice.ArticleData();
      var segments = iservice.SegmentData();
      var featureProps = Vue.inject('featureProps');
      var articleData = Vue.inject('articleData');
      var featureDetails = Vue.inject('featureDetails');
      var topics = iservice.TopicData();
      if (!topics.loaded) topics.Load();
      if (!segments.loaded) segments.Load();
      var featureAction = Vue.reactive({});
      var featureTopicMenu = Vue.reactive({ options: [] });
      Vue.watchEffect(() => {
        featureTopicMenu.options.length = 0;
        featureTopicMenu.loading = true;
        if (!segments.loaded || !topics.loaded) {
          segments.Load(); topics.Load();
          return;
        }
        featureTopicMenu.loading = false;
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            group.options.push({ name: topic.name, id: topic.id, level: Math.min(2, topic.depth) });
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.value, r.value); });
          if (group.options.length < 1) return;
          featureTopicMenu.options.push(group);
        });
      });
      var article = Vue.computed(function () {
        return articles.get(props.itemid);
      });

      featureAction.topicSelected = function () {

        if (!articleData.topicID) return;
        var topic = topics.get(articleData.topicID);
        iservice.EditableProperties(featureProps, topic.faProperties);
        Vue.watchEffect(() => {
          featureDetails.haveProperties = featureProps.props.length;
        });
        Vue.watchEffect(() => { featureDetails.propertyMissingClass = { 'text-red': featureProps.missingRequired }; });
        Vue.watchEffect(() => { featureDetails.hasValues = featureProps.hasValues });
      }
      var attachmentURL = Vue.computed(() => {
        return function (intId, attId) {
          var roothPath = window.location.origin;
          return roothPath + '/File.aspx?interactionID=' + intId + '&fileID=' + attId;
        }
      });
      var isInline = Vue.reactive({});
      iservice.FindInlineAttachments(article.value.questionHtml, article.value.attachments, isInline);
      iservice.FindInlineAttachments(article.value.answerHtml, article.value.attachments, isInline);
      var realAttachments = Vue.ref(iservice.DownloadableFiles(article.value.attachments, isInline));
      var haveEmailDownload = iservice.HaveEmailDownload(article.value.attachments);
      var featureUploads = iservice.FeatureUploads();
      featureAction.UpdateArticle = function () {
        var details = {};
        var attachments = [];
        if (article.value.attachments) {
          article.value.attachments.forEach(function (attachment) {
            attachments.push(attachment.attachmentID);
          });
        }
        details.attachments = attachments;
        details.questionHtml = articleData.questionHtml;
        details.answerHtml = articleData.answerHtml;
        details.propertiesEdit = featureProps.out;
        details.properties = featureProps.out;
        details.subject = article.value.subject;
        details.topicID = articleData.topicID;
        details.isPublic = article.value.public == 'public' ? true : false;
        iservice.API('webapp-api-findanswer?api=modify', null, { articleID: article.value.id, details: details }, featureUploads.files).then((response) => {
          iservice.SuccessMessage("Article updated successfully.");
        }).catch(function (errors) {
          featureAction.errors = errors;
        });
      }
      var featureAttachmentDelete = Vue.reactive({ index: '', article: article.value, featureProps: featureProps, uploads: featureUploads.files, realAttachments: realAttachments });
      var anchorAttachment = Vue.ref(null);
      featureAttachmentDelete.Open = function (fileIndex) {
        featureAttachmentDelete.index = fileIndex;
        featureAttachmentDelete.Close = iservice.components.popupHolder.Open(anchorAttachment, 'article-attachment-delete-menu', true, featureAttachmentDelete);
      }
      var dateFormat = function (date) {
        return moment(date).format('MM/DD/YYYY hh:mm A');
      }
      var HaveRight = Vue.computed(function () {
        return iservice.HaveRight('Tab.FindAnswers.Modify');
      });
      return { article, realAttachments, haveEmailDownload, featureTopicMenu, featureAction, featureUploads, articleData, anchorAttachment, featureAttachmentDelete, dateFormat, HaveRight, attachmentURL };
    }
  };
  iservice.components.articleAttachmentDeleteMenu = {
    name: 'article-attachment-delete-menu',
    template: '#article-attachment-delete-menu',
    props: ['params'],
    setup(props) {
      var article = props.params.article;
      var realAttachments = props.params.realAttachments;
      var index = props.params.index;
      function DeleteFile() {
        var details = {};
        realAttachments.splice(index, 1);

        details.questionHtml = article.questionHtml;
        details.answerHtml = article.answerHtml;
        var attachments = [];
        if (realAttachments) {
          realAttachments.forEach(function (attachment) {
            attachments.push(attachment.attachmentID);
          })
        }
        details.attachments = attachments;
        details.propertiesEdit = props.params.featureProps.out;
        details.properties = props.params.featureProps.out;;
        details.subject = article.subject;
        details.topicID = article.topicID;
        details.isPublic = article.public == 'public' ? true : false;

        iservice.API('webapp-api-findanswer?api=modify', null, { articleID: article.id, details: details }, props.params.uploads).then((response) => {

        }).catch(function (errors) {

        }).finally(function () {
        });
      }
      return { DeleteFile };
    }
  };
  
})(iservice);