﻿var quoteconfig = window.quoteconfig || {};
(function (quoteconfig) {
  function InstallPlugin(plugins) {
    var requestTopicIDs = {};
    var quotepending = Vue.reactive({
      priority: 20,
      func: function (interaction) {
        if (interaction.status != 'Unassigned' && interaction.status != 'Queued') return false;
        //tags are in array type in manage open quote
        if (Array.isArray(interaction.tags))
          return interaction.tags[0] == 'Pending Quote';
        else // tags are in object type in myque pages
          return interaction.tags['Pending Quote'];
      },
      detailsComponent: 'quote-pending-view',
      answerComponent: 'quote-pending-answer',
      inboxComponent: 'messageQueueOpenQuotes',
      noMassUpdate: false
    });
    var quoterequest = {
      config: iservice.config.quoteConfig,
      priority: 200,
      func: function (data) {
        if (data.status != 'Unassigned' && data.status != 'Queued') return false;
        return requestTopicIDs[data.topicID] == true;
      },
      detailsComponent: 'quote-pending-view',
      answerComponent: 'quote-request-answer',
      inboxComponent: 'messageQueueMyInbox'
    };
    plugins.list.push(quotepending);
    plugins.list.push(quoterequest);
    plugins['quotepending'] = quotepending;
    iservice.config.quoteConfig.topics.forEach(topic => {
      requestTopicIDs[topic.requestTopicID] = true;
    });
  }

  quoteconfig.installed = !!iservice.config.quoteConfig;
  if (quoteconfig.installed) {
    var plugins = iservice.Plugins();
    InstallPlugin(plugins);
  }

})(quoteconfig);

var iservice = window.iservice || {};
(function (iservice) {
  iservice.components = iservice.components || {};

  function FindConfig(topicID, config) {
    return iservice.FindArray(config, function (topic) {
      if (topic.requestTopicID == topicID) return topic;
      if (topic.pendingTopicID == topicID) return topic;
    });
  }
  function GetPropOrder(propID, topicID, config) {
    var config = config;
    var order;
    var propertyOrder = config.propertiesOrder;
    if (propertyOrder) {
      for (var i = 0; i < propertyOrder.length; i++) {
        if (propertyOrder[i].id == propID) {
          order = propertyOrder[i].order;
          break;
        }
      }
    }
    return order;
  }
  
  function arraymove(arr, fromIndex, toIndex) {
    arr.splice(toIndex, 0, arr.splice(fromIndex, 1)[0])
  }

  //Quote Board Admin component
  iservice.components.adminQuoteBoard = {
    name: 'quote-board-admin',
    template: '#quote-board-admin',
    route() {
      return {
        path: iservice.formPath + '/admin/quote-admin/:id?', priority: 300,
        myformsUrl: iservice.formPath + '/admin/quote-admin', myformsLinkLabel: 'Quote Admin', myformsLinkClass: 'fa icon-c icon-qoute-admin',
        canAccess() {
          return iservice.HaveRight('Tab.Admin.Forms');
        },
        meta: { title: "iService | Quote Board Admin" }
      }
    },
    props: ['load', 'itemid'],
    setup(props) {
      var router = VueRouter.useRouter();
      var stockresponses = iservice.StockResponsesData();
      var topics = iservice.TopicData();
      var segments = iservice.SegmentData();
      var quotes = iservice.QuotesData();
      
     // var sortable = Sortable.create(el);
      var tableColumns = Vue.computed(() => {
        var columns = [
          Vue.reactive({
            name: 'Segment', label: 'Segment',
            filterText: iservice.tables.SimpleFilterTextFunc(quotes, 'segment'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Quote Topic', label: 'Quote Topic',
            filterText: iservice.tables.SimpleFilterTextFunc(quotes, 'quotetopic'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Follow Up Topic', label: 'Follow Up Topic',
            filterText: iservice.tables.SimpleFilterTextFunc(quotes, 'followuptopic'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          }),
          Vue.reactive({
            name: 'Stock Responses', label: 'Stock Responses',
            filterText: iservice.tables.SimpleFilterTextFunc(quotes, 'stockresponses'),
            componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
          })];

        return columns;
      });
      var tableConfig = Vue.reactive({
        componentControls: 'iservice-table-controls-standard',
        componentBody: 'iservice-table-body',
        componentStatus: 'iservice-table-status',
        componentRow: 'iservice-table-row-expandable',
        filterLabel: 'Filter Quote Configurations',
        emptyMessage: 'There are no quote configurations.',
        pageSize: 5,
        expandable: true,
        searching: false,
        initialSort: 'Segment',
        columns: tableColumns.value,
        exportName: 'Quote Admin'
      });
      var IDs = Vue.reactive([]);

      tableConfig.searching = true;

      var formFeatures = Vue.reactive({ errors: [], success: '', ChangeState: Function, ShowNew: false, ShowDelete: false, ShowDetails: false, ShowDeleteButton: false, Create: Function, Delete: Function, CancelDelete: Function, Save: Function, shownToggleRunning: false, createnew: false, loading: false, stocksRef: [], deleteStart: false });

      var quoteConfig = Vue.reactive({ config: {} });
      quoteConfig.config.topics = [];
      var featureConfig = Vue.reactive({ loading: true});
      iservice.API('webapp-api-admin?api=configuration', null, { application: 'agentapp', name: 'quoteConfig' }).then(data => {
        if (data.items.length > 0) {
          quoteConfig.config = data.items[0].data;
        }
        tableConfig.searching = false;
      }).catch(errors => {
        featureConfig.errors = errors;
      }).finally(() => {
        featureConfig.loading = false;
        //tableConfig.searching = false;
      });

      formFeatures.details = {
        requestTopicID: '',
        stockResponseIDs: [{}],
        pendingTopicID: ''
      }

      formFeatures.propLists = [];
      Vue.onMounted(function () {
        var el = document.getElementById('quotelist');
        var el2 = document.getElementById('quotelists');
        var sortable = new Sortable(el, {
          animation: 150,
          chosenClass: "chosen",
          onUpdate: function (evt) {
            arraymove(formFeatures.propLists, evt.oldIndex, evt.newIndex);
          }
        })
        var sortable2 = new Sortable(el2, {
          animation: 150,
          chosenClass: "chosen",
          onUpdate: function (evt) {
            arraymove(formFeatures.propLists, evt.oldIndex, evt.newIndex);
          }
        })
      })
      var selectedID = Vue.ref(null);
      var featureTopicMenu = Vue.reactive({ options: [], topicID: '' });
      Vue.watchEffect(function () {
        if (!stockresponses.loaded || !topics.loaded || !segments.loaded) {
          stockresponses.Load(); topics.Load(); segments.Load();
          return;
        }
        featureTopicMenu.options.length = 0;
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            var stocks = stockresponses.inSegment(topic.segmentID);
            if (!selectedID.value) {
              var isDisabled = false;
              var hasStocks = true;
              if (ConfiguredTopic(topic.id)) isDisabled = true;
              if (!stocks.length) hasStocks = false;
              group.options.push({ name: topic.name, id: topic.id, level: Math.min(2, topic.depth), disabled: isDisabled, hasStocks: hasStocks });
            } else {
              group.options.push({ name: topic.name, id: topic.id, level: Math.min(2, topic.depth)});
            }
            
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.id, r.id); });
          if (group.options.length < 1) return;
          featureTopicMenu.options.push(group);
        });
      })

      var topicGroups = Vue.computed(() => {
        var groups = [];
        if (!segments.loaded || !topics.loaded) {
          segments.Load(); topics.Load();
        }
        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topic => {
            group.options.push({ name: topic.name, id: topic.id, level: Math.min(2, topic.depth) });
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.value, r.value); });
          if (group.options.length < 1) return;
          groups.push(group);
        });
        return groups;
      });

      function ConfiguredTopic(topicID) {
        var error = false;
        if (!quoteConfig.config.topics) return false;
        for (var i = 0; i < quoteConfig.config.topics.length; i++) {
          if (quoteConfig.config.topics[i].requestTopicID == topicID) {
            error = true;
            break;
          }
        }
        return error;
      }
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }

      function Select(id) {
        selectedID.value = id;
        formFeatures.ShowNew = false;
        router.push({ params: { id } }); //update route paramater on selecting table row 
      }
      var selectedIDs = Vue.computed(() => {
        if (selectedID.value) return [selectedID.value];
        return [];
      });

      formFeatures.ChangeState = function (state = 'new', details = '') {
        formFeatures.ShowNew = state == 'new' ? true : false;
        formFeatures.ShowDelete = state == 'delete' ? true : false;

        formFeatures.ShowDetails = state == 'details' ? true : false;
        if (state == 'details') {
          if (details) {
            var id = details.id;
            selectedID.value = id;
            router.push({ params: { id } });
          }
        }
        if (state == 'new') {
          formFeatures.propLists = [];
          formFeatures.details = {
            requestTopicID: '',
            stockResponseIDs: [{}],
            pendingTopicID: ''
          }
        }
        if (state == 'cancel') {
          formFeatures.ShowDeleteButton = false;
        }
      }
      // formFeatures.haveRight = iservice.HaveRight('Tab.Admin.Mailings');


      function getLists(configuredData) {
        var lists = [];
        configuredData.forEach(function (item) {
          var stocks = [];
          var toQuotetopic = topics.get(item.requestTopicID);
          var openQuoteTopic = topics.get(item.pendingTopicID);
          var stockids = item.stockResponseIDs;
          stockids.forEach(function (stock) {
            var stockData = stockresponses.get(stock);
            if (!stockData) return;
            stocks.push(stockData.name);
          })
          lists.push({ id: item.id, 'segment': toQuotetopic.segmentName, 'quotetopic': toQuotetopic.name, 'followuptopic': openQuoteTopic.name, 'stockresponses': stocks.join(', ') });
        });
        return lists;
      }
      var loaded = false;
      var itemsProcessed = 0;
      Vue.watchEffect(function () {
        if (formFeatures.createnew) return;
        if (!topics.list.length) return;
        if (loaded) return;
        var quoteList = quoteConfig.config.topics ? getLists(quoteConfig.config.topics): []
        quoteList.forEach(function (quote) {
          var id = quote.id;
          quotes.add(quote);
          IDs.push(id);
          itemsProcessed++;
          if (itemsProcessed === quoteList.length) {
            tableConfig.searching = false;
            loaded = true;
          }
        });
       // tableConfig.searching = false;
      });

      Vue.watchEffect(function () {
        if (!formFeatures.ShowNew) return;
        var id = '';
        selectedID.value = '';
        router.push({ params: { id } });
      })

      function MakeTopic(id, requestTopicID, pendingTopicID, stockResponseIDs) {
        return {
          id: id,
          requestTopicID: requestTopicID,
          pendingTopicID: pendingTopicID,
          stockResponseIDs: iservice.TransformArray(stockResponseIDs, function (id) {
            return { id: id };
          })
        };
      }
      formFeatures.SelectRequestTopic = function (topicID) {
        if (!topicID) return;
        formFeatures.loaded = true;
        formFeatures.details = {
          id: formFeatures.details.id,
          requestTopicID: formFeatures.details.requestTopicID,
          pendingTopicID: formFeatures.details.pendingTopicID,
          stockResponseIDs: formFeatures.details.stockResponseIDs
        }
        formFeatures.responses = [];
        formFeatures.followUptopics = [];
        formFeatures.propLists = [];
        var topic = topics.get(topicID);
        formFeatures.selectedTopicName = topic.name;
        if (topic.caseProperties.length) {
          topic.caseProperties[0].properties.forEach(function (item) {
            var valueType = 'Text';
            if (item.isNumber) { valueType = 'Number' };
            if (item.isInteger) { valueType = 'Integer' };
            if (item.isDate) { valueType = 'Date' };
            if (item.isDatetime) { valueType = 'DateTime' };
            formFeatures.propLists.push({ id: item.propertyID, name: item.name, propertyType: "Case", valueType: valueType });
          });
        }

        segments.list.forEach(segment => {
          var group = Vue.reactive({ label: segment.name, options: [] });
          topics.inSegment(segment.id).forEach(topicItem => {
            if (topicItem.segmentID == topic.segmentID) {
              group.options.push({ name: topicItem.name, id: topicItem.id, level: Math.min(2, topicItem.depth) });
            }
          });
          group.options.sort((l, r) => { return topics.TreeSortCompare(l.id, r.id); });
          if (group.options.length < 1) return;
          formFeatures.followUptopics.push(group);
        });
        stockresponses.list.forEach(function (stock) {
          if (stock.segmentID == topic.segmentID) {
            formFeatures.responses.push(stock);
          }
        });
      }
      //check route params change effect and load details
      Vue.watchEffect(() => {
        if (formFeatures.ShowNew) return;
        if (!topics.list.length) return;
        if (router.currentRoute.value.params.id) {
          formFeatures.ShowDeleteButton = true;
          quoteConfig.config.topics.forEach(function (quoteItem) {
            if (router.currentRoute.value.params.id == quoteItem.id) {
              formFeatures.details = MakeTopic(quoteItem.id, quoteItem.requestTopicID, quoteItem.pendingTopicID, quoteItem.stockResponseIDs);
              formFeatures.propLists = quoteItem.propertiesOrder || [];
              formFeatures.responses = [];
              formFeatures.followUptopics = [];
              var topic = topics.get(quoteItem.requestTopicID);
              formFeatures.selectedTopicName = topic.name;

              segments.list.forEach(segment => {
                var group = Vue.reactive({ label: segment.name, options: [] });
                topics.inSegment(segment.id).forEach(topicItem => {
                  if (topicItem.segmentID == topic.segmentID) {
                    group.options.push({ name: topicItem.name, id: topicItem.id, level: Math.min(2, topicItem.depth) });
                  }
                });
                group.options.sort((l, r) => { return topics.TreeSortCompare(l.id, r.id); });
                if (group.options.length < 1) return;
                formFeatures.followUptopics.push(group);
              });
              stockresponses.list.forEach(function (stock) {
                if (stock.segmentID == topic.segmentID) {
                  formFeatures.responses.push(stock);
                }
              });
              selectedID.value = router.currentRoute.value.params.id;
              formFeatures.ChangeState('details');
            }
          });
        }
      });
      formFeatures.error = {};
      formFeatures.SaveClick = function (action) {
        formFeatures.errors = [];
        if (!formFeatures.details.requestTopicID) {
          formFeatures.errors.push({ msg: 'Please select To Quote topic.', details: { description: '' } });
          formFeatures.error.requestTopicID = true;
        }
        /*formFeatures.details.stockResponseIDs.forEach(function (item) {
          if (Object.keys(item).length === 0 ) {
            formFeatures.errors.push({ msg: 'Please select Stock Response.', details: { description: '' } });
            formFeatures.error.stock = true;
          }
        })*/
        
        if (!formFeatures.details.pendingTopicID) {
          formFeatures.errors.push({ msg: 'Please select Open Quote topic.', details: { description: '' } });
          formFeatures.error.pendingTopicID = true;
        }
        if (formFeatures.errors.length) return;
         
        if (formFeatures.loading) return;
        formFeatures.createnew = true;
        formFeatures.loading = true;
        var json = CreateConfig();
        if (json) {
          iservice.API('webapp-api-admin?api=configurationsave', null, { application: 'agentapp', items: [{ name: 'quoteConfig', websiteID: '', data: json }] }).then(data => {
            var quoteItemsData = getLists(quoteConfig.config.topics);
            if (action == 'new' || action == 'update') {

              formFeatures.deleteStart = false;
              var detail = formFeatures.details;
              quoteItemsData.forEach(function (row) {
                if (row.id == formFeatures.details.id) {
                  detail = row;
                }
              });

              formFeatures.ChangeState('details', detail);
              if (action == 'new') {
                quotes.add(detail);
                IDs.push(detail.id);
                iservice.SuccessMessage('Configuration added successfully.');
              }
              if (action == 'update') {
                iservice.SuccessMessage('Configuration saved.');
              }
            } else {
              formFeatures.deleteStart = true;
              formFeatures.ChangeState('cancel');
              formFeatures.ShowDeleteButton = false;
              const index = IDs.indexOf(selectedID.value);
              if (index > -1) {
                IDs.splice(index, 1);
              }
              selectedID.value = '';
              var id = '';
              router.push({ params: { id } })
              formFeatures.success = 'Configuration deleted successfully.';
            }
            formFeatures.success = 'Configuration saved.';

          }).catch(errors => {
            formFeatures.errors = errors;
          }).finally(() => {
            formFeatures.loading = false;
          });
          
        } else {
          formFeatures.loading = false;
        }
      }

      function CreateConfig() {
        var quoteData;
        var errors = false;

        formFeatures.propLists.forEach(function (item, key) {
          item.order = key + 1;
        });
        //add new
        if (formFeatures.ShowNew) {

          quoteData = {
            id: iservice.guidGenerator(),
            requestTopicID: formFeatures.details.requestTopicID,
            pendingTopicID: formFeatures.details.pendingTopicID,
            stockResponseIDs: iservice.TransformArray(formFeatures.details.stockResponseIDs, function (stock) {
              return stock.id;
            }),
            propertiesOrder: formFeatures.propLists
          };
          if (formFeatures.details) {
            formFeatures.details.id = quoteData.id;
          }


          if (!TopicExists(quoteConfig.config.topics, formFeatures.details)) { //add if topic not exists
            quoteConfig.config.topics.push(quoteData);
          } else {
            errors = true;
          }
        } else {
          //delete existed
          if (formFeatures.ShowDelete) {
            for (var i = 0; i < quoteConfig.config.topics.length; i++) {
              if (quoteConfig.config.topics[i].id == formFeatures.details.id) {
                quoteConfig.config.topics.splice(i, 1);
                break;
              }
            }
          } else { //update existed
            quoteData = {
              id: formFeatures.details.id,
              requestTopicID: formFeatures.details.requestTopicID,
              pendingTopicID: formFeatures.details.pendingTopicID,
              stockResponseIDs: iservice.TransformArray(formFeatures.details.stockResponseIDs, function (stock) {
                return stock.id;
              }),
              propertiesOrder: formFeatures.propLists
            };
            quoteConfig.config.topics.forEach(function (item) {
              if (item.id == formFeatures.details.id) {
                item.requestTopicID = quoteData.requestTopicID;
                item.pendingTopicID = quoteData.pendingTopicID;
                item.stockResponseIDs = quoteData.stockResponseIDs;
                item.propertiesOrder = quoteData.propertiesOrder;
              }
            });
          }
        }
        if (errors == true) {
          formFeatures.errors = ['"To Quote" topic already exists. Please choose another.'];
          return false;
        }
        if (errors == false) {
          var configJson = JSON.stringify(quoteConfig.config);
          return configJson;
        }
      }
      formFeatures.AddStock = function () {
        formFeatures.details.stockResponseIDs.push([{}]);
      }
      formFeatures.RemoveStock = function (index) {
        formFeatures.details.stockResponseIDs.splice(index, 1);
        if (!formFeatures.details.stockResponseIDs.length) {
          formFeatures.details.stockResponseIDs.push([{}]);
        }
      }
      function TopicExists(data, compareTo) {
        var error = false;
        for (var i = 0; i < data.length; i++) {
          if (data[i].requestTopicID == compareTo.requestTopicID) {
            error = true;
            break;
          }
        }
        return error;
      }
      return { formFeatures, ListCollapsed, tableConfig, IDs, Select, selectedIDs, selectedID, featureTopicMenu, featureConfig, topicGroups };
    }
  };
  iservice.components.messageQueueManageOpenQuotes = {
    name: 'vueapp-messagequeue-manage-open-quotes',
    template: '#vueapp-messagequeue-manage-open-quotes',
    route() { return { path: iservice.formPath + '/manage-open-quotes', priority: 200, meta: { title: "iService | Manage Open Quotes" } } },
    setup() {
      var myQueue = iservice.MyQueueData();
      var interactions = iservice.InteractionData();
      var intProperties = iservice.IntPropertyData();
      var topics = iservice.TopicData();
      var quotesData = Vue.reactive({ topics: [], loaded: false, props: [] });
      var tableConfig = Vue.reactive({ config: [] });
      var quoteColumns = Vue.reactive([]);
      var messages = iservice.Messages();
      var plugins = iservice.Plugins();

      topics.Load();
      intProperties.Load();
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }

      var sections = Vue.computed(() => {
        var list = [];
        if (iservice.config.quoteConfig) iservice.config.quoteConfig.topics.forEach(item => {
          var topic = topics.get(item.pendingTopicID);
          if (!topic) return;
          var section = Vue.reactive({ id: item.pendingTopicID });
          list.push(section);
        });
        return list;
      });

      var search = { groups: [{ fields: [{ fieldID: 'status', searchString: 'Unassigned', where: 'anywhere' }, { fieldID: 'status', searchString: 'Queued', where: 'anywhere' }] }] };

      var topicInteractions = Vue.computed(() => {
        var map = {};
        sections.value.forEach(section => {
          if (!map[section.id]) {
            map[section.id] = Vue.reactive([]);
          }
        });

        var interactionsData = [];
        tableConfig.loading = true;
        iservice.API('webapp-api-mq-managemessage?api=topics', null, { fields: search, pageNum: 0, perPage: 2500 }).then((data) => {
          iservice.ForArray(data.interactions, (message) => {
            if (!plugins.quotepending.func(message)) return;
            interactions.add(message);
            interactionsData.push(message);
          });
          interactionsData.forEach(interaction => {
            var int = interactions.get(interaction.id);
            if (!int) return;
            var list = map[int.topicID];
            if (!list) return;
            list.push(int.id);
          });
        }).catch(function () {

        }).finally(function () {
          tableConfig.loading = false;
        })
        return map;
      });
      if (iservice.config.quoteConfig) Vue.watchEffect(function () {
        if (quotesData.topics.length) return;
        if (!intProperties.list.length) return;
        iservice.config.quoteConfig.topics.forEach((group) => {
          var topic = topics.get(group.pendingTopicID);
          if (!topic) return;
          var topic = { id: group.pendingTopicID, requestedID: group.requestTopicID, parentID: '', prefix: '', name: topic.name };
          quotesData.topics.push(topic);
        });
        quotesData.topics.forEach((topic) => {
          var topicID = topic.id;
          quoteColumns[topicID] = [
            Vue.reactive({
              name: 'expand', hidable: false,
              filterText: function () { },
              componentHead: 'iservice-table-head-empty', componentCell: 'iservice-table-cell-expand', shown: true, tdClass: 'expand-td'
            }),
            Vue.reactive({
              name: 'Contact Name', label: 'Contact Name',
              filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'customerName'),
              componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
            }),
            Vue.reactive({
              name: 'Contact Email', label: 'Contact Email',
              filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'fromAddress'),
              componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: false
            }),
            Vue.reactive({
              name: 'When Quoted', label: 'When Quoted', sortIncreasing: false,
              filterText: iservice.tables.DateFromNowFilterTextFunc(interactions, 'date'), sortText: iservice.tables.DateSortTextFunc(interactions, 'date'),
              componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-date-from-now', shown: true
            }),
            Vue.reactive({
              name: 'Assigned To', label: 'Assigned To', sortIncreasing: false,
              filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'agentName'),
              componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
            })];
          tableConfig.config[topicID] = {
            componentControls: 'iservice-table-controls-standard',
            componentBody: 'iservice-table-body',
            componentStatus: 'iservice-table-status',
            componentRow: 'iservice-table-row-expandable',
            componentExpanded: 'interaction-view-loader',
            filterLabel: 'Filter Quotes',
            emptyMessage: 'No Quotes',
            pageSize: localStorage.getItem('manageQuotePageSize' + topicID) ? parseInt(localStorage.getItem('manageQuotePageSize' + topicID)) : 5,
            expandable: true,
            searching: false,
            initialSort: 'When Quoted',
            localstorage: true,
            tableName: 'manageQuotes',
            topicID: topicID,
            columns: quoteColumns[topicID],
            exportName: 'Manage Open Quotes'
          }

          var props = [];
          var propOrder = {};
          var topicDef = topics.get(topicID);
          if (topicDef) topicDef.caseProps.forEach(prop => {
            var config = FindConfig(topicID, iservice.config.quoteConfig.topics);
            var property = intProperties.get(prop.propertyID);
            if (property) {
              props.push(property);
              propOrder[property.id] = GetPropOrder(property.id, topicID, config);
            }
          });
          props.sort((l, r) => {
            var lo = propOrder[l.id];
            var ro = propOrder[r.id];
            if (lo > ro) return 1;
            if (lo < ro) return -1;
            return 0;
          });
          props.forEach(function (prop) {
            var column = Vue.reactive({
              name: 'property-' + prop.id,
              label: prop.name,
              filterText: iservice.tables.IntPropValueFilterTextFunc(prop.id),
              componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-inbox-property',
              shown: true,
              propertyID: prop.id
            });
            quoteColumns[topicID].push(column);
          });
        });
      });
      messages.Listen('messageQueueOpenQuotes', id => {
        //find and expand details for the given ID
      });
      
      return { myQueue, tableConfig, quotesData, topicInteractions, ListCollapsed };
    }
  };
  iservice.components.quoteView = {
    name: 'quote-pending-view',
    template: '#quote-pending-view',
    props: ['itemid', 'tableIDs'],
    setup(props) {
      var loggedIn = iservice.LoggedInData();
      var interactions = iservice.InteractionData();
      var tableIDs = Vue.computed(function () {
        return props.tableIDs;
      });
      var ViewOperation = Vue.reactive({ component: '', type: 'interaction' });
      ViewOperation.Click = function (component) {
        ViewOperation.component = component;
      }

      var actionComponent = Vue.ref('');
      function StopAction() {
        actionComponent.value = '';
        ViewOperation.component = '';
      }

      var interaction = Vue.computed(function () {
        return interactions.get(props.itemid);
      });

      var featureAction = Vue.reactive({});

      var displayHtml = Vue.ref('');
      var isInline = Vue.reactive({});
      displayHtml.value = iservice.FindInlineAttachments(interaction.value.bodyHtml, interaction.value.attachments, isInline);
      var realAttachments = Vue.ref(iservice.DownloadableFiles(interaction.value.attachments, isInline));

      var featureDetails = Vue.reactive({ tab: 'details' });
      Vue.watchEffect(() => { featureDetails.haveNote = interaction.value && interaction.value.note && interaction.value.note != ''; })
      featureDetails.TabClass = function (tab) { return { 'active': tab == featureDetails.tab }; }
      featureDetails.TabClick = function (tab) { featureDetails.tab = tab; }

      var actionMenuRef = Vue.ref(null);
      var featureActionMenu = Vue.reactive({ showPlain: false, showCase: false, showStatusAudit: false });
      Vue.watchEffect(() => { featureActionMenu.interaction = interaction.value; });
      Vue.watchEffect(() => { featureActionMenu.showDelete = iservice.HaveRight('Tab.MessageQueue.Delete'); });
      featureActionMenu.Click = function () {
        iservice.components.popupHolder.Open(actionMenuRef, 'interaction-view-action-menu', false, featureActionMenu);
      }
      featureActionMenu.Delete = function (e) {
        if (featureActionMenu.deleting) return;
        featureActionMenu.errors = featureActionMenu.success = null;
        featureActionMenu.deleting = true;
        iservice.API('webapp-api-history?api=delete', e, { interactionID: props.itemid }).then(data => {
          interactions.deleted(props.itemid);
          iservice.SuccessMessage('Interaction deleted successfully.');
        }).catch(errors => {
          featureActionMenu.errors = errors;
        }).finally(() => {
          featureActionMenu.deleting = true;
        });
      }

      var featureProps = Vue.reactive({});
      Vue.watchEffect(() => {
        if (!interaction.value) return;
        featureProps.label = iservice.types.IsQuestion(interaction.value.type) ? 'Question Properties' : (iservice.types.IsAnswer(interaction.value.type) ? 'Answer Properties' : 'Interaction Properties');
      });
      iservice.EditableProperties(featureProps, interaction.value.intProps);

      var featureCaseProps = Vue.reactive({ label: 'Case Properties'});
      iservice.EditableProperties(featureCaseProps, interaction.value.caseProps);

      var featurePropertySet = Vue.reactive({ working: false });
      Vue.watchEffect(() => { featurePropertySet.shown = iservice.HaveRight('Tab.CustomerInfo.ModifyHistory') && (featureProps.props.length || featureCaseProps.props.length); });
      featurePropertySet.Click = function (e) {
        if (featurePropertySet.working) return;
        featurePropertySet.errors = featurePropertySet.success = null;
        featurePropertySet.working = true;
        iservice.API('webapp-api-history?api=propertyset', e, { interactionID: props.itemid, values: featureProps.out, caseValues: featureCaseProps.out }).then(data => {
          interactions.add(data);
          featureProps.Reload();
          featureCaseProps.Reload();
          iservice.SuccessMessage('Saved successfully');
        }).catch(errors => {
          featurePropertySet.errors = errors;
        }).finally(() => {
          featurePropertySet.working = false;
        });
      }

      Vue.watchEffect(() => { featureDetails.haveProperties = featureCaseProps.props.length || featureProps.props.length; });
      Vue.watchEffect(() => { featureDetails.propertyMissingClass = { 'text-red': featureCaseProps.missingRequired || featureProps.missingRequired }; });
      Vue.watchEffect(() => { featureDetails.hasValues = featureCaseProps.hasValues || featureProps.hasValues; });

      var featureAnswer = Vue.reactive({});
      var canAnswer = Vue.computed(() => {
        if (!interaction.value) return false;
        if (interaction.value.type == 'Unsent Response' || interaction.value.type == 'Chat') return false;
        if (interaction.value.assignedToID != loggedIn.contactID) return false;
        if (interaction.value.status == 'Unassigned' || interaction.value.status == 'Queued' || interaction.value.status == 'Pending' || interaction.value.status == 'SentWaitingReply') return true;
        return false;
      });
      Vue.watchEffect(() => {
        featureAnswer.shown = canAnswer.value;
      });
      featureAnswer.Click = function (e) {
        actionComponent.value = 'interaction-answer-loader';
      }

      var featureGetThis = Vue.reactive({});
      var canGetThis = Vue.computed(() => {
        if (!interaction.value) return false;
        if (interaction.value.type == 'Unsent Response' || interaction.value.type == 'Chat') return false;
        if (!iservice.HaveRight('Tab.MessageQueue.Supervise')) return false;
        if (interaction.value.assignedToID == loggedIn.contactID) return false;
        if (interaction.value.status == 'Unassigned' || interaction.value.status == 'Queued' || interaction.value.status == 'SentWaitingReply') return true;
        if (interaction.value.status == 'Pending' && iservice.HaveRight('Tab.MessageQueue.Approve')) return true;
        return false;
      });
      Vue.watchEffect(() => {
        featureGetThis.shown = canGetThis.value;
      });
      featureGetThis.Click = function (e) {
        if (featureGetThis.working) return;
        featureGetThis.working = true;
        featureAction.errors = featureAction.success = null;
        iservice.API('webapp-api-mq-myqueue?api=assign', e, { interactionID: interaction.value.id, interactionIDs: [interaction.value.id], answer: { isMassUpdate: true, reassignAgentID: loggedIn.contactID } }).then(data => {
          interactions.add(data.question.details);
          interactions.add(data.response.details);
          interactions.add(data.thread.details);
          actionComponent.value = 'interaction-answer-loader';
        }).catch(errors => {
          featureAction.errors = errors;
        }).finally(() => {
          featureGetThis.working = false;
        });
      }

      return { actionComponent, interaction, StopAction, realAttachments, displayHtml, featureDetails, featureProps, featureCaseProps, featureActionMenu, actionMenuRef, featurePropertySet, featureGetThis, featureAnswer, featureAction, ViewOperation, tableIDs };
    }
  }
  iservice.components.quoteMyOpenQuotesDetails = {
    name: 'quote-my-open-quotes-details',
    template: '#quote-my-open-quotes-details',
    props: ['itemid', 'load', 'cancelable', 'data', 'tableIDs'],
    emits: ['cancel'],
    setup(props, context) {
      var featureTabs = Vue.reactive({ tab: 'details' });
      featureTabs.TabClass = function (tab) { return { 'active': tab == featureTabs.tab }; }
      featureTabs.TabClick = function (tab) { featureTabs.tab = tab; }
      var data = props.data;

      var componentAnswer = Vue.ref(null);
      var ViewOperation = Vue.reactive({ component: '', type: 'interaction' });
      ViewOperation.Click = function (component) {
        ViewOperation.component = component;
      }
      function StopAction() {
        ViewOperation.component = '';
      }
      var contactDetails = Vue.reactive({});
      var interactions = iservice.InteractionData();
      var interaction = Vue.computed(function () {
        return interactions.get(props.itemid)
      })
      contactDetails.contactId = interaction.value.customerID;
      var plugins = iservice.Plugins();
      var apiData = Vue.ref(null);
      function Cancel() {
        context.emit('cancel');
      }
      var router = VueRouter.useRouter();

      var featureLoad = Vue.reactive({ loaded: false, working: false });
      Vue.watchEffect(() => {
        var wantLoad = props.load === undefined || props.load;
        if (!wantLoad || featureLoad.loaded || featureLoad.working) return;
        featureLoad.working = true;
      });
      Vue.watchEffect(() => {
        if (featureLoad.loaded || !featureLoad.working) return;
        featureLoad.errors = null;
        iservice.API('webapp-api-mq-myqueue?api=details2', null, { interactionID: props.itemid }).then(function (data) {
          interactions.add(data.details);
          interactions.add(data.thread);
          data.thread.children.forEach(child => {
            interactions.add(child);
          });
          apiData.value = data;
          var plugin = plugins.FindPlugin(props.itemid);
          componentAnswer.value = plugin.answerComponent;
        }).catch(errors => {
          featureLoad.errors = errors;
          Cancel();
        }).finally(() => {
          featureLoad.working = false;
          featureLoad.loaded = true;
        });
      });

      return { featureTabs, data, ViewOperation, StopAction, componentAnswer, apiData, featureLoad, Cancel, interaction, contactDetails }
    }
  }
  iservice.components.quotePendingAnswer = {
    name: 'quote-pending-answer',
    template: '#quote-pending-answer',
    props: ['itemid', 'load', 'cancelable', 'data', 'tableIDs'],
    emits: ['cancel'],
    setup(props, context) {
      var interactions = iservice.InteractionData();
      var interaction = interactions.get(props.itemid);
      var Actions = Vue.reactive({ component: '', type: 'interaction' });
      var ResolveOperation = Vue.reactive({ component: '', type: 'interaction' });
      var tableIDs = Vue.computed(function () {
        return props.tableIDs;
      });
      if (props.actions) {
        Actions = props.actions;
      } else {
        Actions.Click = function (component) {
          Actions.component = component;
        }

        Actions.StopAction = function() {
          Actions.component = '';
        }
      }
      ResolveOperation.Click = function (component) {
        ResolveOperation.component = component
      }
      ResolveOperation.StopAction = function () {
        ResolveOperation.component = '';
      }

      function Cancel() {
        context.emit('cancel');
      }

      var questionDetails = Vue.ref(props.data.details);

      var answer = Vue.reactive({
        subject: questionDetails.value.responseSubject,
        mailboxID: questionDetails.value.defaultMailboxID,
        destEmail: questionDetails.value.fromAddress,
        destCC: '',
        destBCC: '',
        responseNote: questionDetails.value.note,
        bodyHtml: '',
        supressSubjectRef: false
      });

      var featureCaseProps = Vue.reactive({});
      iservice.EditableProperties(featureCaseProps, interaction.caseProps, out => { answer.threadProperties = out; });
      var featureQuestionProps = Vue.reactive({});
      iservice.EditableProperties(featureQuestionProps, interaction.intProps, out => { answer.messageProperties = out; });
      var featureAnswerProps = Vue.reactive({ responseProps: [] });
      iservice.CopyProps(featureAnswerProps.responseProps, questionDetails.value.responseProperties);
      iservice.EditableProperties(featureAnswerProps, featureAnswerProps.responseProps, out => { answer.answerProperties = out; });
      featureCaseProps.props.forEach((property) => {
        var config = FindConfig(questionDetails.value.topicID, iservice.config.quoteConfig.topics);
        property.def.order = GetPropOrder(property.def.propertyID, questionDetails.value.topicID, config);
      });
      featureCaseProps.props.sort((a, b) => parseInt(a.def.order) - parseInt(b.def.order));

      var questionIsInline = Vue.reactive({});
      iservice.FindInlineAttachments(interaction.bodyHtml, interaction.attachments, questionIsInline);
      var questionAttachments = Vue.ref(iservice.DownloadableFiles(interaction.attachments, questionIsInline));
      var haveEmailDownload = iservice.HaveEmailDownload(interaction.attachments);

      var bodyHtmlEdit = Vue.ref('');
      var answerIsInline = Vue.reactive({});
      bodyHtmlEdit.value = iservice.FindInlineAttachments(questionDetails.value.responseHtml, interaction.attachments, answerIsInline);
      bodyHtmlEdit.value = iservice.FindInlineAttachments(bodyHtmlEdit.value, questionDetails.value.responseAttachments, answerIsInline);
      var answerAttachments = Vue.ref(iservice.DownloadableFiles(questionDetails.value.responseAttachments, answerIsInline));
      Vue.watchEffect(() => {
        var keep = [];
        questionDetails.value.responseAttachments.forEach(attach => {
          if (answerIsInline[attach.attachmentID] && bodyHtmlEdit.value.indexOf(iservice.FileReference(attach)) != -1) keep.push(attach.attachmentID);
        });
        answerAttachments.value.forEach(attach => {
          keep.push(attach.attachmentID);
        });
        answer.attachments = keep;
      });
      Vue.watchEffect(() => {
        answer.bodyHtml = iservice.RestoreInlineAttachments(bodyHtmlEdit.value, interaction.attachments);
        answer.bodyHtml = iservice.RestoreInlineAttachments(answer.bodyHtml, questionDetails.value.responseAttachments);
      });
      var featureUploads = iservice.FeatureUploads();

      var inputBCC = Vue.ref(null);
      var inputCC = Vue.ref(null);
      var featureCC = Vue.reactive({ showBcc: false, anchor: inputCC });
      featureCC.ShowBCC = function () {
        featureCC.showBcc = true;
        setTimeout(() => { inputBCC.value.focus(); });
      }
      featureCC.ApplyReCC = function () {
        answer.destCC = questionDetails.value.reCC;
      }
      featureCC.Focus = function () {
        
      }
      featureCC.Change = function () {
        if (featureCC.Close) return;
        //featureCC.Close = iservice.components.popupHolder.Open(inputCC, 'address-autocomplete', true, featureCC);
      }
      featureCC.Blur = function () {
        //if (featureCC.Close) featureCC.Close();
      }
      featureCC.Complete = function (to) {
        answer.destCC = to;
      }

      var anchorForward = Vue.ref(null);

      var featureAction = Vue.reactive({});
      featureAction.SubmitAnswer = function (url, e, successMessage, doReset, removeOnSuccess = false) {
        if (featureAction.working) return;
        featureAction.working = true;
        featureAction.errors = featureAction.success = null;
        iservice.API(url, e, { interactionID: interaction.id, interactionIDs: [interaction.id], answer }, featureUploads.files).then(data => {
          interactions.add(data.thread.row);
          interactions.add(data.thread.details);
          interactions.add(data.question.row);
          interactions.add(data.question.details);
          interactions.add(data.response.row);
          interactions.add(data.response.details);
          if (data.details) {
            props.data.details = data.details;
            questionDetails.value = props.data.details;
          }
          iservice.SuccessMessage(successMessage);
          if (removeOnSuccess) {
            iservice.data.mq.numCanGet = data.numCanGet;
            iservice.data.mq.numCanApprove = data.numCanApprove;
          }
        }).catch(errors => {
          featureAction.errors = errors;
        }).finally(() => {
          featureAction.working = false;
        });
      }

      var featureForwardMenu = Vue.reactive({ interaction });
      Vue.watchEffect(() => { answer.reassignAgentID = featureForwardMenu.reassignAgentID; });
      Vue.watchEffect(() => { answer.notificationID = featureForwardMenu.notificationID; });
      Vue.watchEffect(() => { answer.forwardExternalAddress = featureForwardMenu.forwardExternalAddress; });
      Vue.watchEffect(() => { answer.forwardMailboxID = featureForwardMenu.forwardMailboxID; });
      featureForwardMenu.Open = function () {
        featureForwardMenu.Close = iservice.components.popupHolder.Open(anchorForward, 'quote-answer-menu-forward', true, featureForwardMenu);
      }
      featureForwardMenu.ReassignAgent = function (e) {
        if (featureForwardMenu.Close) featureForwardMenu.Close();
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=assign', e, 'Assigned.', false, true);
      }
      featureForwardMenu.UnassignQuestion = function (e) {
        if (featureForwardMenu.Close) featureForwardMenu.Close();
        featureAction.SubmitAnswer('webapp-api-mq-myqueue?api=unqueue', e, 'Unassigned.', false, true);
      }

      var featureTabs = Vue.reactive({ tab: 'details', haveProps: false, haveNote: false });
      Vue.watchEffect(() => {
        featureTabs.haveProps = featureCaseProps.props.length > 0 || featureQuestionProps.props.length > 0 || featureAnswerProps.props.length > 0;
        featureTabs.havePropValues = featureCaseProps.hasValues || featureQuestionProps.hasValues || featureAnswerProps.hasValues
        featureTabs.havePropRequired = featureCaseProps.missingRequired || featureQuestionProps.missingRequired || featureAnswerProps.missingRequired;
        featureTabs.propertyIconClass = { 'text-red': featureTabs.havePropRequired };
        featureTabs.haveNote = interaction && interaction.note && interaction.note != '';
      });
      featureTabs.TabClass = function (tab) { return { 'active': tab == featureTabs.tab, 'text-red': tab == 'property' && featureTabs.havePropRequired }; }
      featureTabs.PaneClass = function (tab) { return { 'show active': tab == featureTabs.tab }; }
      featureTabs.TabClick = function (tab) {
        featureTabs.tab = tab;
      }

      return { answer, Cancel, questionDetails, bodyHtmlEdit, featureAction, featureCaseProps, featureQuestionProps, featureAnswerProps, featureForwardMenu, featureTabs, questionAttachments, haveEmailDownload, anchorForward, interaction, Actions, ResolveOperation, tableIDs };
    }
  };
  iservice.components.quoteAnswerMenuForward = {
    name: 'quote-answer-menu-forward',
    template: '#quote-answer-menu-forward',
    props: ['params'],
    setup(props) {
      var loggedIn = iservice.LoggedInData();
      var agents = iservice.AgentData();
      var feature = Vue.computed(() => { return props.params; });


      var reassignStage = Vue.ref('closed');
      var reassignOptions = Vue.ref([]);
      Vue.watchEffect(() => {
        reassignOptions.value.length = 0;
        reassignOptions.loading = true;
        if (!agents.loaded) {
          agents.Load(); return;
        }
        reassignOptions.loading = false;
        var agentIDs = iservice.HaveRight('Tab.MessageQueue.Supervise') ? agents.forSegment(feature.value.interaction.segmentID) : agents.forInteraction(feature.value.interaction.id);
        agentIDs.forEach(agentID => {
          if (agentID == loggedIn.contactID) return;
          var agent = agents.get(agentID);
          var name = agent.name || agent.email || ('Agent ' + agent.id);
          var option = {
            id: agentID,
            name: name,
            email: agent.email,
            search: agent.name + '\n' + agent.email,
            disabled: !agent.isLoggedIn && agent.doEmpty,
            isaway: !agent.isLoggedIn && !agent.doEmpty
          };
          reassignOptions.value.push(option);
        });
        reassignOptions.value.sort((l, r) => {
          return l.name.localeCompare(r.name);
        });
        if (reassignStage.value == 'open' && reassignOptions.value.length < 1) {
          reassignStage.value = 'none';
        }
      });
      function ReassignStart() {
        if (reassignStage.value == 'open') {
          reassignStage.value = 'closed';
        }
        else if (reassignStage.value == 'closed') {
          reassignStage.value = 'open';
        }
      }

      return { feature, ReassignStart, reassignStage, reassignOptions };
    }
  };
  iservice.components.messageQueueOpenQuotes = {
    name: 'vueapp-messagequeue-myopenquotes',
    template: '#vueapp-messagequeue-myopenquotes',
    route() { return { path: iservice.formPath + '/my-open-quotes', priority: 200, meta: { title: "iService | My Open Quotes" } } },
    setup() {
      var myQueue = iservice.MyQueueData();
      var interactions = iservice.InteractionData();
      var intProperties = iservice.IntPropertyData();
      var topics = iservice.TopicData();
      var quotesData = Vue.reactive({ topics: [], loaded: false, props: [] });
      var plugins = iservice.Plugins();
      var tableConfig = Vue.reactive({ config: [] });
      var quoteColumns = Vue.reactive([]);
      var messages = iservice.Messages();

      topics.Load();
      intProperties.Load();
      var ListCollapsed = Vue.reactive({ isListCollapsed: true })
      ListCollapsed.Toggle = function () {
        return ListCollapsed.isListCollapsed = !ListCollapsed.isListCollapsed;
      }

      var sections = Vue.computed(() => {
        var list = [];
        if (iservice.config.quoteConfig) iservice.config.quoteConfig.topics.forEach(item => {
          var topic = topics.get(item.pendingTopicID);
          if (!topic) return;
          var section = Vue.reactive({ id: item.pendingTopicID });
          list.push(section);
        });
        return list;
      });

      var topicInteractions = Vue.computed(() => {
        var map = {};
        sections.value.forEach(section => {
          if (!map[section.id]) {
            map[section.id] = Vue.reactive([]);
          }
        });
        myQueue.subbox.messageQueueOpenQuotes.forEach(id => {
          var int = interactions.get(id);
          if (!int) return;
          var list = map[int.topicID];
          if (!list) return;
          list.push(id);
        });
        return map;
      });

      if (iservice.config.quoteConfig) Vue.watchEffect(function () {

        if (quotesData.topics.length) return;
        if (!intProperties.list.length) return;
        iservice.config.quoteConfig.topics.forEach((group) => {
          var topic = topics.get(group.pendingTopicID);
          if (!topic) return;
          var topic = { id: group.pendingTopicID, requestedID: group.requestTopicID, parentID: '', prefix: '', name: topic.name };
          quotesData.topics.push(topic);
        });
        quotesData.topics.forEach((topic) => {
          var topicID = topic.id;
          quoteColumns[topicID] = [
            Vue.reactive({
              name: 'expand', hidable: false,
              filterText: function () { },
              componentHead: 'iservice-table-head-empty', componentCell: 'iservice-table-cell-expand', shown: true, tdClass: 'expand-td'
            }),
            Vue.reactive({
              name: 'Contact Name', label: 'Contact Name',
              filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'customerName'),
              componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
            }),
            Vue.reactive({
              name: 'Contact Email', label: 'Contact Email',
              filterText: iservice.tables.SimpleFilterTextFunc(interactions, 'fromAddress'),
              componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-simple', shown: true
            }),
            Vue.reactive({
              name: 'When Quoted', label: 'When Quoted', sortIncreasing: false,
              filterText: iservice.tables.DateFromNowFilterTextFunc(interactions, 'date'), sortText: iservice.tables.DateSortTextFunc(interactions, 'date'),
              componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-date-from-now', shown: true
            })];
          tableConfig.config[topicID] = {
            componentControls: 'iservice-table-controls-standard',
            componentBody: 'iservice-table-body',
            componentStatus: 'iservice-table-status',
            componentRow: 'iservice-table-row-expandable',
            componentExpanded: 'quote-my-open-quotes-details',
            filterLabel: 'Filter Quotes',
            emptyMessage: 'No Quotes',
            pageSize: localStorage.getItem('myQuotePageSize' + topicID) ? parseInt(localStorage.getItem('myQuotePageSize' + topicID)) : 5,
            expandable: true,
            searching: false,
            localstorage: true,
            tableName: 'myQuotes',
            topicID: topicID,
            initialSort: 'When Quoted',
            columns: quoteColumns[topicID],
            exportName: 'My Open Quotes'
          }

          var props = [];
          var propOrder = {};
          var topicDef = topics.get(topicID);
          if (topicDef) topicDef.caseProps.forEach(prop => {
            var config = FindConfig(topicID, iservice.config.quoteConfig.topics);
            var property = intProperties.get(prop.propertyID);
            if (property) {
              props.push(property);
              propOrder[property.id] = GetPropOrder(property.id, topicID, config);
            }
          });
          props.sort((l, r) => {
            var lo = propOrder[l.id];
            var ro = propOrder[r.id];
            if (lo > ro) return 1;
            if (lo < ro) return -1;
            return 0;
          });
          props.forEach(function (prop) {
            var column = Vue.reactive({
              name: 'property-' + prop.id,
              label: prop.name,
              filterText: iservice.tables.IntPropValueFilterTextFunc(prop.id),
              componentHead: 'iservice-table-head-sortable-label', componentCell: 'iservice-table-cell-inbox-property',
              shown: true,
              propertyID: prop.id
            });
            quoteColumns[topicID].push(column);
          });
        });
      });

      messages.Listen('messageQueueOpenQuotes', id => {
        //find and expand details for the given ID
      });

      return { myQueue, tableConfig, quotesData, topicInteractions, ListCollapsed };
    }
  };
  iservice.components.quoteRequestAnswer = {
    name: 'quote-request-answer',
    template: '#quote-request-answer',
    props: ['itemid', 'load', 'cancelable', 'data'],
    emits: ['cancel'],
    setup(props, context) {
      var interactions = iservice.InteractionData();
      var loggedIn = iservice.LoggedInData();
      var mailboxes = iservice.MailboxData();
      var interaction = interactions.get(props.itemid);
      var plugins = iservice.Plugins();
      var quoteConfig = iservice.config.quoteConfig ? iservice.config.quoteConfig : { topics: []};
      function Cancel() {
        context.emit('cancel');
      }
      var apiData = props.data;
      var isQuote = true;
      var questionDetails = Vue.ref(props.data.details);
      ToggleComponent = Vue.reactive({ showQuote: true });
      ToggleComponent.Click = function () {
        ToggleComponent.showQuote = !ToggleComponent.showQuote;
      }
      var answer = Vue.reactive({
        subject: questionDetails.value.responseSubject,
        mailboxID: questionDetails.value.defaultMailboxID,
        destEmail: questionDetails.value.fromAddress,
        destCC: '',
        destBCC: '',
        responseNote: questionDetails.value.note,
        bodyHtml: '',
        supressSubjectRef: false
      });

      var featureCaseProps = Vue.reactive({});
      iservice.EditableProperties(featureCaseProps, interaction.caseProps, out => { answer.threadProperties = out; });
      var featureQuestionProps = Vue.reactive({});
      iservice.EditableProperties(featureQuestionProps, interaction.intProps, out => { answer.messageProperties = out; });
      var featureAnswerProps = Vue.reactive({ responseProps: [] });
      iservice.CopyProps(featureAnswerProps.responseProps, questionDetails.value.responseProperties);
      iservice.EditableProperties(featureAnswerProps, featureAnswerProps.responseProps, out => { answer.answerProperties = out; });
      featureCaseProps.props.forEach((property) => {
        var config = FindConfig(questionDetails.value.topicID, iservice.config.quoteConfig.topics);
        property.def.order = GetPropOrder(property.def.propertyID, questionDetails.value.topicID, config);
      });
      featureCaseProps.props.sort((a, b) => parseInt(a.def.order) - parseInt(b.def.order));

      var questionIsInline = Vue.reactive({});
      iservice.FindInlineAttachments(interaction.bodyHtml, interaction.attachments, questionIsInline);
      var questionAttachments = Vue.ref(iservice.DownloadableFiles(interaction.attachments, questionIsInline));
      var haveEmailDownload = iservice.HaveEmailDownload(interaction.attachments);

      var bodyHtmlEdit = Vue.ref('');
      var answerIsInline = Vue.reactive({});
      bodyHtmlEdit.value = iservice.FindInlineAttachments(questionDetails.value.responseHtml, interaction.attachments, answerIsInline);
      bodyHtmlEdit.value = iservice.FindInlineAttachments(bodyHtmlEdit.value, questionDetails.value.responseAttachments, answerIsInline);
      var answerAttachments = Vue.ref(iservice.DownloadableFiles(questionDetails.value.responseAttachments, answerIsInline));
      Vue.watchEffect(() => {
        var keep = [];
        questionDetails.value.responseAttachments.forEach(attach => {
          if (answerIsInline[attach.attachmentID] && bodyHtmlEdit.value.indexOf(iservice.FileReference(attach)) != -1) keep.push(attach.attachmentID);
        });
        answerAttachments.value.forEach(attach => {
          keep.push(attach.attachmentID);
        });
        answer.attachments = keep;
      });
      function AttachmentRemove(attach) {
        var pos = answerAttachments.value.indexOf(attach);
        if (pos == -1) return;
        answerAttachments.value.splice(pos, 1);
      }
      Vue.watchEffect(() => {
        answer.bodyHtml = iservice.RestoreInlineAttachments(bodyHtmlEdit.value, interaction.attachments);
        answer.bodyHtml = iservice.RestoreInlineAttachments(answer.bodyHtml, questionDetails.value.responseAttachments);
      });
      var featureUploads = iservice.FeatureUploads();


      var featureAction = Vue.reactive({ errors: [], success: ''});

      var featureTabs = Vue.reactive({ tab: 'details', haveProps: false });
      var featureTabsOriginal = Vue.reactive({ tab: 'quote_form'});
      Vue.watchEffect(() => {
        featureTabs.haveProps = featureCaseProps.props.length > 0 || featureQuestionProps.props.length > 0 || featureAnswerProps.props.length > 0;
        featureTabs.havePropValues = featureCaseProps.hasValues || featureQuestionProps.hasValues || featureAnswerProps.hasValues
        featureTabs.havePropRequired = featureCaseProps.missingRequired || featureQuestionProps.missingRequired || featureAnswerProps.missingRequired;
        featureTabs.propertyIconClass = { 'text-red': featureTabs.havePropRequired };
      });
      featureTabs.TabClass = function (tab) { return { 'active': tab == featureTabs.tab, 'text-red': tab == 'property' && featureTabs.havePropRequired }; }
      featureTabs.PaneClass = function (tab) { return { 'show active': tab == featureTabs.tab }; }
      featureTabs.TabClick = function (tab) {
        featureTabs.tab = tab;
      }
      featureTabsOriginal.TabClass = function (tab) { return { 'active': tab == featureTabsOriginal.tab }; }
      featureTabsOriginal.PaneClass = function (tab) { return { 'show active': tab == featureTabsOriginal.tab }; }
      featureTabsOriginal.TabClick = function (tab) {
        featureTabsOriginal.tab = tab;
        if (tab == 'preview') {
          if (!answer.stockID) {
            featureAction.errors = ['Please select a template for your quote.'];
            return;
          } 
          featureTabsOriginal.loadingPreview = true;
          var submitData = {}
          submitData = $.extend(submitData, answer);

          submitData.threadProperties = featureCaseProps.out;
          submitData.messageProperties = featureQuestionProps.out;
          submitData.answerProperties = featureAnswerProps.out;
          submitData.bodyHtml = answer.stockID ? "$" + "stock -id'" + answer.stockID + "'$" : answer.bodyHtml;

          iservice.API('webapp-api-mq-myqueue?api=savedraft', null, { interactionIDs: [props.itemid], answer: submitData, interactionID: props.itemid }, featureUploads.files).then(function (data) {
            featureTabsOriginal.finalHtml = data.details.responseFinalHtml
            featureTabsOriginal.loadingPreview = false;
          }).catch(function (errors) {
            featureAction.errors = errors;
            featureTabsOriginal.loadingPreview = false;
          });
        }
      }
      featureAction.SaveDraft = function () {

        featureAction.loading = true;
        if (!answer.stockID) {
          featureAction.errors = ['Please select a template for your quote.'];
          featureAction.loading = false;
          return;
        } 
        var submitData = {}
        submitData = $.extend(submitData, answer);

        submitData.threadProperties = featureCaseProps.out;
        submitData.messageProperties = featureQuestionProps.out;
        submitData.answerProperties = featureAnswerProps.out;

        iservice.API('webapp-api-mq-myqueue?api=savedraft', null, { interactionIDs: [props.itemid], answer: submitData, interactionID: props.itemid }, featureUploads.files).then(function (data) {
          featureAction.loading = false;
        }).catch(function (errors) {
          featureAction.errors = errors;
          featureAction.loading = false;
        });
      }
      var PrepareTicket = () => {
        var config = FindConfig(questionDetails.value.topicID, quoteConfig.topics);
        var ticket = {
          contactID: questionDetails.value.customerID,
          inResponseToID: questionDetails.value.id,
          topicID: config.pendingTopicID,
          tags: ['Pending Quote'],
          destEmail: answer.destEmail,
          subject: answer.subject,
          addSubjectRefNum: false,
          bodyHtml: "$" + "value -interaction(bodyHtml) -Pid'responseID'$",
          note: answer.note,
          action: 5,
          assignToAgentID: loggedIn.contactID
        }

        return ticket;
      }
      featureAction.SendQuote = function () {

        featureAction.loading = true;
        if (!answer.stockID) {
          featureAction.errors = ['Please select a template for your quote.'];
          featureAction.loading = false;
          return;
        } 
        var submitData = {}
        submitData = $.extend(submitData, answer);

        submitData.threadProperties = featureCaseProps.out;
        submitData.messageProperties = featureQuestionProps.out;
        submitData.answerProperties = featureAnswerProps.out;
        submitData.bodyHtml = answer.stockID ? "$" + "stock -id'" + answer.stockID + "'$" : answer.bodyHtml;

        iservice.API('webapp-api-quoteboard?api=send', null, { interactionIDs: [props.itemid], answer: submitData, interactionID: props.itemid, ticket: PrepareTicket() }, featureUploads.files).then(function (data) {
          featureAction.loading = false;
          iservice.SuccessMessage('Quote sent.');
        }).catch(function (errors) {
          featureAction.errors = errors;
          featureAction.loading = false;
        });
      }

      var addressFromSelect = Vue.ref(null);
      var featureAddresses = Vue.reactive({ mailboxes: [], mailbox: null });
      Vue.watchEffect(() => {
        featureAddresses.subjectRef = questionDetails.value.subjectRef;
      });
      Vue.watchEffect(() => {
        featureAddresses.mailboxes.length = 0;
        featureAddresses.loading = true;
        if (!mailboxes.loaded) {
          mailboxes.Load();
          return;
        }
        featureAddresses.loading = false;
        if (!addressFromSelect.value) return;
        mailboxes.inSegment(interaction.segmentID).forEach(mailbox => {
          var option = Vue.reactive({ id: mailbox.id, name: mailbox.name, replyTo: mailbox.replyTo, search: mailbox.name + '\n' + mailbox.replyTo });
          if (option.id == questionDetails.value.defaultMailboxID) {
            setTimeout(() => { addressFromSelect.value.select(option); });
          }
          featureAddresses.mailboxes.push(option);
        });
        featureAddresses.mailboxes.sort((l, r) => { return mailboxes.SortCompare(l.id, r.id); });
      });
      
      function findStockResponses() {
        var config = FindConfig(questionDetails.value.topicID, quoteConfig.topics);
        featureAction.stockResponses = [];
        var responseIDs = {};
        iservice.ForArray(config.stockResponseIDs, function (stockID) {
          responseIDs[stockID] = true;
        });
        iservice.ForArray(props.data.responsesSegment, (stock) => {
          if (responseIDs[stock.id]) {
            featureAction.stockResponses.push(stock);
          }
        });
      }
      findStockResponses();

      return { answer, Cancel, questionDetails, bodyHtmlEdit, featureAction, featureCaseProps, featureQuestionProps, featureAnswerProps, featureTabs, featureAddresses, featureUploads, questionAttachments, haveEmailDownload, answerAttachments, AttachmentRemove, featureTabsOriginal, addressFromSelect, apiData, isQuote, ToggleComponent };
    }
  };
  iservice.components.iserviceQuoteResolve = {
    name: 'iservice-quote-resolve',
    template: '#iservice-quote-resolve',
    props: ['itemid', 'load', 'cancelable', 'data', 'type', 'tableIDs'],
    emits: ['cancel'],
    setup(props, context) {
      var interactions = iservice.InteractionData();
      var interaction = interactions.get(props.itemid);
      function Cancel() {
        context.emit('cancel');
      }
      var formFeatures = Vue.reactive({ errors: [], success: '' });
      var questionDetails = Vue.ref(props.data);

      var answer = Vue.reactive({
        subject: questionDetails.value.responseSubject,
        mailboxID: questionDetails.value.defaultMailboxID,
        destEmail: questionDetails.value.fromAddress,
        destCC: '',
        destBCC: '',
        responseNote: questionDetails.value.note,
        bodyHtml: '',
        subjectRef: questionDetails.value.subjectRef,
        supressSubjectRef: false
      });

      var featureCaseProps = Vue.reactive({});
      iservice.EditableProperties(featureCaseProps, interaction.caseProps, out => { answer.threadProperties = out; });
      var featureQuestionProps = Vue.reactive({});
      iservice.EditableProperties(featureQuestionProps, interaction.intProps, out => { answer.messageProperties = out; });
      var featureAnswerProps = Vue.reactive({ responseProps: [] });
      iservice.CopyProps(featureAnswerProps.responseProps, questionDetails.value.responseProperties);
      iservice.EditableProperties(featureAnswerProps, featureAnswerProps.responseProps, out => { answer.answerProperties = out; });

      var questionIsInline = Vue.reactive({});
      iservice.FindInlineAttachments(interaction.bodyHtml, interaction.attachments, questionIsInline);
      var questionAttachments = Vue.ref(iservice.DownloadableFiles(interaction.attachments, questionIsInline));
      var haveEmailDownload = iservice.HaveEmailDownload(interaction.attachments);

      var bodyHtmlEdit = Vue.ref('');
      var answerIsInline = Vue.reactive({});
      bodyHtmlEdit.value = iservice.FindInlineAttachments(questionDetails.value.responseHtml, interaction.attachments, answerIsInline);
      bodyHtmlEdit.value = iservice.FindInlineAttachments(bodyHtmlEdit.value, questionDetails.value.responseAttachments, answerIsInline);
      var answerAttachments = Vue.ref(iservice.DownloadableFiles(questionDetails.value.responseAttachments, answerIsInline));
      Vue.watchEffect(() => {
        var keep = [];
        questionDetails.value.responseAttachments.forEach(attach => {
          if (answerIsInline[attach.attachmentID] && bodyHtmlEdit.value.indexOf(iservice.FileReference(attach)) != -1) keep.push(attach.attachmentID);
        });
        answerAttachments.value.forEach(attach => {
          keep.push(attach.attachmentID);
        });
        answer.attachments = keep;
      });
      Vue.watchEffect(() => {
        answer.bodyHtml = iservice.RestoreInlineAttachments(bodyHtmlEdit.value, interaction.attachments);
        answer.bodyHtml = iservice.RestoreInlineAttachments(answer.bodyHtml, questionDetails.value.responseAttachments);
      });
      var featureUploads = iservice.FeatureUploads();

      var featureTabs = Vue.reactive({ tab: 'address', haveProps: false });
      Vue.watchEffect(() => {
        featureTabs.haveProps = featureCaseProps.props.length > 0 || featureQuestionProps.props.length > 0 || featureAnswerProps.props.length > 0;
        featureTabs.havePropValues = featureCaseProps.hasValues || featureQuestionProps.hasValues || featureAnswerProps.hasValues
        featureTabs.havePropRequired = featureCaseProps.missingRequired || featureQuestionProps.missingRequired || featureAnswerProps.missingRequired;
        featureTabs.propertyIconClass = { 'text-red': featureTabs.havePropRequired };
      });
      featureTabs.TabClass = function (tab) { return { 'active': tab == featureTabs.tab, 'text-red': tab == 'property' && featureTabs.havePropRequired }; }
      featureTabs.PaneClass = function (tab) { return { 'show active': tab == featureTabs.tab }; }
      featureTabs.TabClick = function (tab) {
        featureTabs.tab = tab;
      }
      var BodyTabs = Vue.reactive({ tab: 'body' });

      BodyTabs.TabClass = function (tab) { return { 'active': tab == BodyTabs.tab } }
      BodyTabs.PaneClass = function (tab) { return { 'show active': tab == BodyTabs.tab }; }
      BodyTabs.TabClick = function (tab) {
        BodyTabs.tab = tab;
        if (tab == 'preview') {
          BodyTabs.loadingPreview = true;
          var submitData = {}
          submitData = $.extend(submitData, answer);

          submitData.threadProperties = featureCaseProps.out;
          submitData.messageProperties = featureQuestionProps.out;
          submitData.answerProperties = featureAnswerProps.out;

          iservice.API('webapp-api-mq-myqueue?api=savedraft', null, { interactionIDs: [props.itemid], answer: submitData, interactionID: props.itemid }, featureUploads.files).then(function (data) {
            BodyTabs.finalHtml = data.details.responseFinalHtml
            BodyTabs.loadingPreview = false;
          }).catch(function (errors) {
            formFeatures.errors = errors;
            BodyTabs.loadingPreview = false;
          });
        }
      }
      formFeatures.Resolve = function () {
        formFeatures.loading = true;
        var submitData = {}
        submitData = $.extend(submitData, answer);

        submitData.threadProperties = featureCaseProps.out;
        submitData.messageProperties = featureQuestionProps.out;
        submitData.answerProperties = featureAnswerProps.out;
        iservice.API('webapp-api-mq-myqueue?api=saveresolve', null, { interactionIDs: [props.itemid], answer: submitData, interactionID: props.itemid }, featureUploads.files).then(function (data) {
          iservice.SuccessMessage('Quote Resolved');
          const index = props.tableIDs.indexOf(props.itemid);
          if (index > -1) {
            props.tableIDs.splice(index, 1); 
          }
        }).catch(function (errors) {
          formFeatures.errors = errors;
          formFeatures.loading = false;
        }).finally(function () {
          formFeatures.loading = false;
        });
      }

      return { formFeatures, answer, Cancel, questionDetails, featureCaseProps, featureQuestionProps, featureAnswerProps, featureTabs, questionAttachments, haveEmailDownload, interaction, featureUploads, answerAttachments, BodyTabs };
    }
  };

})(iservice);