﻿var iservice = window.iservice || {};
(function (iservice) {
  iservice.components = iservice.components || {};
  iservice.tables = iservice.tables || {};

  iservice.tables.SimpleFilterTextFunc = function (module, memberName) {
    return function (id) {
      var obj = module.get(id);
      if (!obj) return '';
      var val = obj[memberName];
      if (!val) return '';
      return val.toString() || '';
    }
  }
  iservice.tables.SimpleFilterUserTypeFunc = function (module, memberName) {
    return function (id) {
      var obj = module.get(id);
      if (!obj) return '';
      var val = obj[memberName];
      var usertypes = iservice.UserTypesData();
      if (!val) return '';
      var usertype = usertypes.get(val);
      return usertype.name.toString() || '';
    }
  }
  iservice.tables.SimpleFilterMatchInTextFunc = function (module) {
    return function (id) {
      var obj = module.get(id);
      if (!obj) return '';
      var matchInString = [];
      if (obj.matchSubject) matchInString.push('Subject');
      if (obj.matchBody) matchInString.push('Body');
      if (obj.matchTo) matchInString.push('Mailbox Name');
      if (obj.matchFrom) matchInString.push('From');
      if (obj.matchContactProperty) matchInString.push('Contact Properties');
      if (obj.matchAttachments) matchInString.push('Text Attachments');
      return matchInString.join(', ');
    }
  }
  iservice.tables.SimpleFilterActionStepsTextFunc = function (module) {
    return function (id) {
      var obj = module.get(id);
      if (!obj) return '';
      var actionStepsString = [];
      if (obj.action) {
        obj.action.steps.forEach(function (step, key) {
          for (const item in step) {
            if (step[item]) actionStepsString.push(item);
          }
        });
        return actionStepsString.join(', ')
      } else {
        return '';
      }
    }
  }
  iservice.tables.SimpleFilterTextNotificationTypeFunc = function (module) {
    return function (id) {
      var obj = module.get(id);
      if (!obj) return '';
      var notificationtypes = iservice.notificationtypes;
      var notificationNames = [];
      var notificationsTypes = notificationtypes.filter(function (item) {
        return obj.listNames.includes(item.value);
      });
      notificationsTypes.forEach(function (type) {
        notificationNames.push(type.name);
      });
      return notificationNames.join(', ');
    }
  }
  iservice.tables.SimpleTopicParentFilterTextFunc = function (module, memberName) {
    return function (id) {
      var obj = module.get(id);
      if (!obj) return '';
      var val = obj[memberName];
      if (!val) return '';
      var parentTopic = module.get(val);
      return parentTopic.name;
    }
  }
  iservice.tables.SimpleQuestionFilterTextFunc = function (module, valueModule, memberName) {
    return function (id) {
      var obj = module.get(id);
      if (!obj) return '';
      var val = obj[memberName];
      if (!val) return '';
      var filter = valueModule.get(val);
      return filter.name;
    }
  }
  iservice.tables.SimpleAutoResponseFilterTextFunc = function (module, valueModule, memberName) {
    return function (id) {
      var obj = module.get(id);
      if (!obj) return '';
      var val = obj[memberName];
      if (!val) return '';
      var autoresponse = valueModule.get(val);
      return autoresponse.name;
    }
  }
  iservice.tables.SimpleServiceLevelFilterTextFunc = function (module, valueModule, memberName) {
    return function (id) {
      var obj = module.get(id);
      if (!obj) return '';
      var val = obj[memberName];
      if (!val) return '';
      var servicelevel = valueModule.get(val);
      return servicelevel.name;
    }
  }
  iservice.tables.DateFilterTextFunc = function (module, memberName) {
    return function (id) {
      var obj = module.get(id);
      if (!obj) return '';
      var date = obj[memberName];
      if (!date || date == '') return '';
      date = new Date(date);
      date = date.toISOString();
      return moment(date).format('YYYY-MM-DD hh:mm:ss A');
    }
  }
  iservice.tables.DateFromNowFilterTextFunc = function (module, memberName) {
    return function (id) {
      var obj = module.get(id);
      if (!obj) return '';
      var date = obj[memberName];
      if (!date || date == '') return '';
      return moment(date).fromNow();
    }
  }
  iservice.tables.DateSortTextFunc = function (module, memberName) {
    return function (id) {
      var obj = module.get(id);
      if (!obj) return '';
      var date = obj[memberName];
      if (!date || date == '') return '';
      return date;
    }
  }
  iservice.tables.IntPropValueFilterTextFunc = function (propertyID) {
    return function (id) {
      var int = iservice.data.interactions.get(id);
      var values = iservice.components.iserviceTableCellInboxProperty.ValueItems(int, propertyID);
      var all = '';
      for (var value of values) {
        if (value.description) all += value.description + '\n';
        if (value.value) all += value.value + '\n';
      }
      return all;
    }
  }
  iservice.tables.ContactPropValueFilterTextFunc = function (propertyID) {
    return function (id) {
      var contact = iservice.data.contacts.get(id);
      var values = iservice.components.iserviceTableCellContactProperty.ValueItems(contact, propertyID);
      var all = '';
      for (var value of values) {
        if (value.description) all += value.description + '\n';
        if (value.value) all += value.value + '\n';
      }
      return all;
    }
  } 

  iservice.components.iserviceTable = {
    name: 'iservice-table',
    template: '#iservice-table',
    props: ['config', 'IDs', 'selectedid', 'checkedids', 'updateFunction', 'deleteFunction'],
    emits: ['select', 'update:checkedids'],
    setup(props, context) {
      Vue.provide('config', props.config);
      var selectedID = Vue.computed(() => {
        return props.selectedid;
      });
      var tableIDs = Vue.computed(function(){
        return props.IDs;
      })
      var updateRow = Vue.computed(() => {
        return props.updateFunction;
      });
      var deleteFunction = Vue.computed(() => {
        return props.deleteFunction;
      });
      var updateMessages = Vue.reactive({ errors: {}, success: '' });
      function Messages(success, errors) {
        updateMessages.errors = errors;
        updateMessages.success = success;
      }
      Vue.provide('tableIDs', tableIDs);
      Vue.provide('selectedID', selectedID);
      Vue.provide('updateRow', updateRow);
      Vue.provide('deleteFunction', deleteFunction);
      Vue.provide('updateMessages', updateMessages);
      Vue.provide('Messages', Messages);
      Vue.provide('emits', {
        Select: function (id) {
          context.emit('select', id);
        }
      });
      var isVisible = Vue.reactive({});
      var columnByName = Vue.computed(function () {
        var map = {};
        props.config.columns.forEach(col => {
          if (isVisible[col.name] === undefined) isVisible[col.name] = col.shown;
          map[col.name] = col;
        });
        return map;
      });
      Vue.provide('isVisible', isVisible);
      var visibleColumns = Vue.computed(function () {
        function ColumnID(col, i) {
          var name = '';
          if (col.name) for (var c of col.name) {
            if (c >= 'a' && c <= 'z') name += c;
            else if (c >= 'A' && c <= 'Z') name += c;
            else if (c >= '0' && c <= '9') name += c;
          }
          else {
            name = 'Col' + i;
          }
          return name;
        }
        var cols = [];
        props.config.columns.forEach((col, i) => {
          if (!isVisible[col.name]) return;
          cols.push(col);
          col.tdID = ColumnID(col, i);
        })
        return cols;
      });
      Vue.provide('visibleColumns', visibleColumns);

      var currentSortColumn = Vue.ref(props.config.initialSort);
      var currentSortDir = Vue.ref(columnByName.value[currentSortColumn.value].sortIncreasing);
      if (currentSortDir.value === undefined) currentSortDir.value = true;
      var sortedIDs = Vue.computed(function () {
        var ids = props.IDs.slice(0);
        var column = columnByName.value[currentSortColumn.value];

        ids.sort(function (lid, rid) {
          if (column.type && column.type == 'number') {
            var l = (column.sortText ? column.sortText(lid) : (column.filterText ? column.filterText(lid) : '')) || '';
            var r = (column.sortText ? column.sortText(rid) : (column.filterText ? column.filterText(rid) : '')) || '';
            return l - r;
          } else {
            var l = (column.sortText ? column.sortText(lid) : (column.filterText ? column.filterText(lid) : '')) || '';
            var r = (column.sortText ? column.sortText(rid) : (column.filterText ? column.filterText(rid) : '')) || '';
            l = l.toLocaleLowerCase();
            r = r.toLocaleLowerCase();
            return l.localeCompare(r);
          }
        });
        if (!currentSortDir.value) ids.reverse();
        return ids;
      });
      var sortingColumn = Vue.reactive({ column: '' });
      function sortIconClass(column) {
        if (currentSortColumn.value != column) return 'sorting'; else 'sorting sorting_asc';
        
        if (currentSortDir.value) {
          return 'sorting sorting_asc';
        } else {
          return 'sorting sorting_desc';
        }
      }
      function Sort(colname) {
        sortingColumn.column = columnByName.value[colname];
        if (!sortingColumn.column) return;
        if (currentSortColumn.value == colname) {
          currentSortDir.value = !currentSortDir.value;
        }
        else {
          currentSortColumn.value = colname;
          currentSortDir.value = (sortingColumn.column.sortIncreasing === undefined) ? true : sortingColumn.column.sortIncreasing;
        }
      }
      Vue.provide('Sort', Sort);
      Vue.provide('sortIconClass', sortIconClass);

      var filterString = Vue.ref('');
      Vue.provide('filterString', filterString);
      var filteredIDs = Vue.computed(function () {
        var filter = filterString.value.toLowerCase();
        if (filter == '') return sortedIDs.value;
        var ids = [];
        sortedIDs.value.forEach(id => {
          for (var val of visibleColumns.value) {
            val = val.filterText(id);
            if (!val || val == '') continue;
            val = val.toLowerCase();
            if (val.indexOf(filter) != -1) {
              ids.push(id);
              break;
            }
          }
        });
        return ids;
      });
      Vue.provide('filteredIDs', filteredIDs);

      var pageSize = Vue.ref(props.config.pageSize || 5);
      Vue.provide('pageSize', pageSize);
      var numPages = Vue.computed(function () {
        if (filteredIDs.value.length < pageSize.value) return 1;
        return Math.ceil(filteredIDs.value.length / pageSize.value);
      });
      Vue.provide('numPages', numPages);
      var curPage = Vue.ref(0);
      Vue.watchEffect(() => {
        if (curPage.value >= numPages.value) curPage.value = numPages.value - 1;
      });
      Vue.provide('curPage', curPage);
      var pageLabels = Vue.computed(function () {
        var labels = [];
        for (var i = 0; i < numPages.value;) {
          if (i == 0) { labels.push(i); i++; continue; }
          if (i < curPage.value && i < curPage.value - 2) { labels.push(-1); i = curPage.value - 2; continue; }
          if (i <= curPage.value + 2) { labels.push(i); i++; continue; }
          if (i > curPage.value + 2 && i < numPages.value - 1) { labels.push(-1); i = numPages.value - 1; continue; }
          if (i == numPages.value - 1) { labels.push(numPages.value - 1); }
          break;
        }
        return labels;
      });
      Vue.provide('pageLabels', pageLabels);

      var firstRow = Vue.computed(function () {
        var first = curPage.value * pageSize.value;
        return first;
      });
      Vue.provide('firstRow', firstRow);
      
      var lastRow = Vue.computed(function () {
        var last = firstRow.value + pageSize.value - 1;
        if (last > filteredIDs.value.length - 1) last = filteredIDs.value.length - 1;
        return last;
      });
      Vue.provide('lastRow', lastRow);
      var visibleIDs = Vue.computed(function () {
        if (firstRow.value === undefined || lastRow === undefined) return [];
        var IDs = [];
        for (var i = firstRow.value; i <= lastRow.value; i++) {
          IDs.push(filteredIDs.value[i]);
        }
        return IDs;
      });
      Vue.provide('visibleIDs', visibleIDs);

      var featureMultiselect = Vue.reactive({ userChecked: {}, selectAll: false });
      featureMultiselect.CheckIDAllowed = function (id) {
        if (!props.config.checkIDAllowed) return true;
        return props.config.checkIDAllowed(id);
      }
      function UncheckAll() {
        featureMultiselect.selectAll = false;
        featureMultiselect.CheckAll();
      }
      featureMultiselect.CheckAll = function () {
        if (!featureMultiselect.selectAll) {
          for (var id in featureMultiselect.userChecked) {
            featureMultiselect.userChecked[id] = false;
          }
          return;
        }
        var IDs = [];
        if (props.config.checkAllMode == 'visible') IDs = visibleIDs.value;
        if (props.config.checkAllMode == 'filtered') IDs = filteredIDs.value;
        for (var id of IDs) {
          if (!featureMultiselect.CheckIDAllowed(id)) continue;
          featureMultiselect.userChecked[id] = true;
        }
      }
      var selectableMap = Vue.reactive({});
      var selectableIDs = Vue.computed(() => {
        var IDs = [];
        for (var id in selectableMap) selectableMap[id] = false;
        var potentials = [];
        if (props.config.checkAllMode == 'visible') potentials = visibleIDs.value;
        if (props.config.checkAllMode == 'filtered') potentials = filteredIDs.value;
        for (var id of potentials) {
          if (!featureMultiselect.CheckIDAllowed(id)) continue;
          IDs.push(id);
          selectableMap[id] = true;
        }
        return IDs;
      });
      Vue.watchEffect(() => {
        featureMultiselect.selectAll = false;
        if (selectableIDs.value.length < 1) return;
        for (var id of selectableIDs.value) {
          if (!featureMultiselect.userChecked[id]) return;
        }
        featureMultiselect.selectAll = true;
      });
      Vue.provide('featureMultiselect', featureMultiselect);
      Vue.provide('selectableIDs', selectableIDs);

      var selectedIDs = Vue.computed(() => {
        var IDs = [];
        for (var id in featureMultiselect.userChecked) {
          if (featureMultiselect.userChecked[id] && selectableMap[id]) IDs.push(id);
        }
        return IDs;
      });
      Vue.watchEffect(() => {
        context.emit('update:checkedids', selectedIDs.value);
      });

      var expanded = Vue.reactive({});
      var editRow = Vue.reactive({});
      var deleteRow = Vue.reactive({});
      Vue.provide('expanded', expanded);
      Vue.provide('editRow', editRow);
      Vue.provide('deleteRow', deleteRow);

      var EditDetails = function (id) {
        editRow[id] = !editRow[id];
      }
      var ShowDetails = function (id) {
        expanded[id] = !expanded[id];
        deleteRow[id] = false;
        if (expanded[id]) editRow[id] = false;
      }
      var ShowDelete = function (id) {
        expanded[id] = false;
        deleteRow[id] = !deleteRow[id];
      }
      Vue.provide('ShowDetails', ShowDetails);
      Vue.provide('EditDetails', EditDetails);
      Vue.provide('ShowDelete', ShowDelete);

      return { UncheckAll };
    }
  };

  iservice.components.iserviceTableControlsStandard = {
    name: 'iservice-table-controls-standard',
    template: '#iservice-table-controls-standard',
    setup() {

      return {}
    }
  };

  iservice.components.iserviceTableActions = {
    name: 'iservice-table-actions',
    template: '#iservice-table-actions',
    setup() {
      var config = Vue.inject('config');
      var isVisible = Vue.inject('isVisible');
      var filteredIDs = Vue.inject('filteredIDs');
      function Open() {
        iservice.components.popupHolder.Open(menuAnchorEl, 'iservice-table-actions-colvis', true, { config, isVisible }, { 'overflow-y': 'auto', 'min-width': '124px', 'max-height': '300px' });
      }
      var ExportExcel = function () {
        if (!config.exportName) return;
        var exp = {
          filename: config.exportName,
          sheet: { data: [] }
        };
        var headerRow = [];
        config.columns.forEach(function (column) {
          if (!isVisible[column.name]) return;
          if (column.name == 'expand') return;
          headerRow.push({ value: column.label, type: 'string' });
        });
        exp.sheet.data.push(headerRow);
        filteredIDs.value.forEach(function (id) {
          var rowCells = [];
          config.columns.forEach(function (column) {
            if (!isVisible[column.name]) return;
            if (column.name == 'expand') return;
            var value = column.filterText(id);
            rowCells.push({ value: value, type: (column.exportType ? column.exportType : 'string') });
          });
          exp.sheet.data.push(rowCells);
        });
        zipcelx(exp);
      }

      var menuAnchorEl = Vue.ref(null);

      return { Open, menuAnchorEl, ExportExcel };
    }
  }
  iservice.components.iserviceTableActionsColvis = {
    name: 'iservice-table-actions-colvis',
    template: '#iservice-table-actions-colvis',
    props: ['params'],
    setup(props) {
      var isVisible = Vue.computed(() => {
        if (!props.params) return {};
        return props.params.isVisible;
      });
      var columns = Vue.computed(() => {
        if (!props.params) return [];
        return props.params.config.columns;
      });
      var router = VueRouter.useRouter();
      
      function Click(item) {
        var name = item.column.name;
        isVisible.value[name] = !isVisible.value[name];
        const query = Object.assign({}, router.currentRoute.value.query);
        if (isVisible.value[name]) {
          if (typeof item.column.queryString != 'undefined') {
            if (!query.hasOwnProperty(item.column.queryString)) {
              query[item.column.queryString] = item.column.var;
            }
          }
        } else {
          if (query.hasOwnProperty(item.column.queryString)) {
            delete query[item.column.queryString]
          }
        }
        router.push({ query: query });
      }
      function groupBy(objectArray, property) {
        return objectArray.reduce((acc, obj) => {
          const key = obj.column[property] ? obj.column[property]: 'Default';
          if (!acc[key]) {
            acc[key] = [];
          }
          // Add object to list for given key's value
          acc[key].push(obj);
          return acc;
        }, {});
      }
      var picker = Vue.computed(() => {
        var cols = [];
        columns.value.forEach(src => {
          if (src.hidable === false) return;
          var col = {
            column: src,
            'class': { 'active': isVisible.value[src.name] }
          };
          cols.push(Vue.reactive(col));
        })
        return cols;
      });
      var groupedData = Vue.reactive({ groups : [] });
      Vue.watchEffect(function () {
        groupedData.groups = groupBy(picker.value, 'segmentName');
      })
      return { Click, picker, groupedData };
    }
  };

  iservice.components.iserviceTableHead = {
    name: 'iservice-table-head',
    template: '#iservice-table-head',
    setup() {
      var columns = Vue.inject('visibleColumns');
      return { columns };
    }
  }

  iservice.components.iserviceTableHeadSortableLabel = {
    name: 'iservice-table-head-sortable-label',
    template: '#iservice-table-head-sortable-label',
    props: ['column'],
    setup () {
      var Sort = Vue.inject('Sort');
      var sortIconClass = Vue.inject('sortIconClass');
      var config = Vue.inject('config');
      return { Sort, sortIconClass, config };
    }
  };

  iservice.components.iserviceTableHeadEmpty = {
    name: 'iservice-table-head-empty',
    template: '<th class="no-sort" style="min-width: 72px;"></th>',
    setup () {
      return {};
    }
  };

  iservice.components.iserviceTableBody = {
    name: 'iservice-table-body',
    template: '#iservice-table-body',
    setup() {

      return {};
    }
  }

  iservice.components.iserviceTableRows = {
    name: 'iservice-table-rows',
    template: '#iservice-table-rows',
    setup() {
      var config = Vue.inject('config');
      var IDs = Vue.inject('visibleIDs');
      return { config, IDs };
    }
  }

  iservice.components.iserviceTableRowExpandable = {
    name: 'iservice-table-row-expandable',
    template: '#iservice-table-row-expandable',
    props: ['itemid'],
    setup(props) {
      var interactions = iservice.InteractionData();
      var interaction = Vue.computed(function () {
        return interactions.get(props.itemid);
      });
      var columnStyle = Vue.computed(function () {
        if (!interaction.value.columnColor) return;
        return { backgroundColor: interaction.value.columnColor, '--bs-table-accent-bg': interaction.value.columnColor }
      });
      var config = Vue.inject('config');
      var columns = Vue.inject('visibleColumns');
      var expanded = Vue.inject('expanded');
      var deleteRow = Vue.inject('deleteRow');
      var ShowDelete = Vue.inject('ShowDelete');
      var deleteFunction = Vue.inject('deleteFunction');
      var editRow = Vue.inject('editRow');
      var selectedID = Vue.inject('selectedID');
      var messages = Vue.inject('updateMessages');
      var emits = Vue.inject('emits');
      var filteredIDs = Vue.inject('tableIDs');

      var showDetails = Vue.computed(() => {
        return expanded[props.itemid];
      });
      var showdelete = Vue.computed(() => {
        return deleteRow[props.itemid];
      });
      var editDetails = Vue.computed(() => {
        return editRow[props.itemid];
      });
      var rowClass = Vue.computed(() => {
        return props.itemid == selectedID.value ? 'row-active' : '';
      });
      var defaultClass = Vue.computed(() => {
        if (!config.hasOwnProperty('notclickable')) {
          return 'row-default';
        } else {
          return '';
        }
      });
      Vue.provide('showDetails', showDetails);
      Vue.provide('editDetails', editDetails);

      function Select(itemid) {
        if (config.hasOwnProperty("componentExpanded")) {
          expanded[itemid] = !expanded[itemid];
        }
        emits.Select(itemid);
      }

      return { config, columns, showDetails, rowClass, Select, editDetails, messages, showdelete, ShowDelete, deleteFunction, columnStyle, filteredIDs, defaultClass };
    }
  }

  iservice.components.iserviceTableCellExpand = {
    name: 'iservice-table-cell-expand',
    template: '#iservice-table-cell-expand',
    props: ['itemid', 'column', 'rowIDs'],
    setup() {
      var showDetails = Vue.inject('showDetails');
      var ShowDetails = Vue.inject('ShowDetails');

      return { showDetails, ShowDetails };
    }
  }
  iservice.components.iserviceTableCellGear = {
    name: 'iservice-table-cell-gear',
    template: '#iservice-table-cell-gear',
    props: ['itemid', 'column', 'rowIDs'],
    setup() {
      var showDetails = Vue.inject('showDetails');
      var ShowDetails = Vue.inject('ShowDetails');

      return { showDetails, ShowDetails };
    }
  }
  
  iservice.components.iserviceTableHeadSelectAll = {
    name: 'iservice-table-head-select-all',
    template: '#iservice-table-head-select-all',
    props: ['column'],
    setup() {
      var featureMultiselect = Vue.inject('featureMultiselect');
      var selectableIDs = Vue.inject('selectableIDs');

      return { featureMultiselect, selectableIDs };
    }
  }
  iservice.components.iserviceTableCellSelect = {
    name: 'iservice-table-cell-select',
    template: '#iservice-table-cell-select',
    props: ['itemid', 'column'],
    setup(props) {
      var featureMultiselect = Vue.inject('featureMultiselect');
      var selectableIDs = Vue.inject('selectableIDs');

      var canSelect = Vue.computed(() => {
        return selectableIDs.value.indexOf(props.itemid) != -1;
      });

      return { featureMultiselect, canSelect };
    }
  }
  iservice.components.iserviceTableCellExpandMultiselect = {
    name: 'iservice-table-cell-expand-multiselect',
    template: '#iservice-table-cell-expand-multiselect',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var showDetails = Vue.inject('showDetails');
      var ShowDetails = Vue.inject('ShowDetails');
      var featureMultiselect = Vue.inject('featureMultiselect');
      var selectableIDs = Vue.inject('selectableIDs');

      var canSelect = Vue.computed(() => {
        return selectableIDs.value.indexOf(props.itemid) != -1;
      });

      return { showDetails, ShowDetails, featureMultiselect, canSelect };
    }
  }

  iservice.components.iserviceTablePagesize = {
    name: 'iservice-table-pagesize',
    template: '#iservice-table-pagesize',
    setup() {
      var pageSize = Vue.inject('pageSize');
      var config = Vue.inject('config');
      function Change() {
        if (config.localstorage && config.tableName == 'manageQuotes' && config.topicID) {
          localStorage.setItem('manageQuotePageSize' + config.topicID, pageSize.value);
        }
        if (config.localstorage && config.tableName == 'myQuotes' && config.topicID) {
          localStorage.setItem('myQuotePageSize' + config.topicID, pageSize.value);
        }
      }
      return { pageSize, Change };
    }
  }

  iservice.components.iserviceTableDays = {
    name: 'iservice-table-days',
    template: '#iservice-table-days',
    setup() {
      var config = Vue.inject('config');

      return { config };
    }
  }

  iservice.components.iserviceTableFilter = {
    name: 'iservice-table-filter',
    template: '#iservice-table-filter',
    setup() {
      var config = Vue.inject('config');
      var filterString = Vue.inject('filterString');

      return { config, filterString };
    }
  }

  iservice.components.iserviceTableStatus = {
    name: 'iservice-table-status',
    template: '#iservice-table-status',
    setup() {

      return { };
    }
  }

  iservice.components.iserviceTableInfo = {
    name: 'iservice-table-info',
    template: '#iservice-table-info',
    setup() {
      var firstRow = Vue.inject('firstRow');
      var lastRow = Vue.inject('lastRow');
      var IDs = Vue.inject('filteredIDs');
      var config = Vue.inject('config');

      return { config, firstRow, lastRow, IDs };
    }
  }

  iservice.components.iserviceTablePaging = {
    name: 'iservice-table-paging',
    template: '#iservice-table-paging',
    setup() {
      var pageLabels = Vue.inject('pageLabels');
      var curPage = Vue.inject('curPage');
      var numPages = Vue.inject('numPages');
      function SetPage(num) {
        if (num < 0) num = 0;
        if (num >= numPages.value) num = numPages.value - 1;
        if (num != curPage.value) curPage.value = num;
      }

      return { pageLabels, curPage, SetPage };
    }
  }

  iservice.components.iserviceTableCellInboxProperty = {
    name: 'iservice-table-cell-inbox-property',
    template: '#iservice-table-cell-inbox-property',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var interactions = iservice.InteractionData();

      var int = interactions.get(props.itemid);
      var values = Vue.computed(() => {
        return iservice.components.iserviceTableCellInboxProperty.ValueItems(int, props.column.propertyID);
      });

      return { values };
    },
    ValueItems: function (int, propertyID) {
      var src = int && int.valuesMap[propertyID] ? int.valuesMap[propertyID] : [];
      var values = [];
      for (var val of src) {
        var item = {};
        if (val.description) item.description = val.description;
        if (val.valueDateUtc) item.value = moment(val.valueDateUtc).format('YYYY-MM-DD');
        else if (val.valueDatetimeUtc) item.value = moment(val.valueDatetimeUtc).format('YYYY-MM-DD hh:mm:ss A');
        else if (val.valueInt) item.value = ''+val.valueInt;
        else if (val.valueNumber) item.value = ''+ val.valueNumber;
        else if (val.value) item.value = val.value;
        if (item.description) item.description = iservice.BreakWord(item.description);
        if (item.value) item.value = iservice.BreakWord(item.value);
        values.push(item);
      }
      return values;
    }
  }
  iservice.components.iserviceTableCellContactProperty = {
    name: 'iservice-table-cell-contact-property',
    template: '#iservice-table-cell-contact-property',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var contacts = iservice.ContactsData();

      var contact = Vue.computed(function () {
        return contacts.get(props.itemid);
      });
      var values = Vue.computed(() => {
        return iservice.components.iserviceTableCellContactProperty.ValueItems(contact.value, props.column.propertyID);
      });

      return { values };
    },
    ValueItems: function (contact, propertyID) {
      var src = contact && contact.valuesMap[propertyID] ? contact.valuesMap[propertyID] : [];
      var values = [];
      for (var val of src) {
        var item = {};
        if (val.description) item.description = val.description;
        if (val.valueDateUtc) item.value = moment(val.valueDateUtc).format('YYYY-MM-DD');
        else if (val.valueDatetimeUtc) item.value = moment(val.valueDatetimeUtc).format('YYYY-MM-DD hh:mm:ss A');
        else if (val.valueInt) item.value = val.valueInt;
        else if (val.valueNumber) item.value = val.valueNumber;
        else if (val.value) item.value = val.value;
        if (item.description) item.description = iservice.BreakWord(item.description);
        if (item.value) item.value = iservice.BreakWord(item.value);
        values.push(item);
      }
      return values;
    }
  }
  iservice.components.iserviceTableCellSimple = {
    name: 'iservice-table-cell-simple',
    template: '#iservice-table-cell-simple',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var text = Vue.computed(() => {
        return iservice.BreakWord(props.column.filterText(props.itemid));
      });

      return { text };
    }
  }
  iservice.components.iserviceTableCellDateFromNow = {
    name: 'iservice-table-cell-date-from-now',
    template: '#iservice-table-cell-date-from-now',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var interactions = iservice.InteractionData();
      var text = Vue.computed(function () {
        return iservice.BreakWord(props.column.filterText(props.itemid));
      });
      var fullDate = Vue.computed(function () {
        if (!props.column.sortText) return '';
        var dateUtc = props.column.sortText(props.itemid);
        return moment(dateUtc).format('YYYY-MM-DD hh:mm:ss A');
      })
      return { text, fullDate };
    }
  }
  iservice.components.iserviceTableCellDateFromNowManage = {
    name: 'iservice-table-cell-date-from-now-manage',
    template: '#iservice-table-cell-date-from-now-manage',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var config = Vue.inject('config');
      var interactions = iservice.InteractionData();
      var feature = Vue.reactive({ text: "" })
      setInterval(function () {
        feature.text = iservice.BreakWord(props.column.filterText(props.itemid));
        var int = interactions.get(props.itemid);
        var now = moment();
        var then = moment(int.date);
        var diff = now.diff(then, 'minutes');
        var best;
        config.colorCodes.forEach(function (color) {
          if (best && best.minutes > color.minutes) return;
          if (color.minutes < diff) {
            best = color;
          }
        });
        if (best) {
          int.columnColor = best.color;
        }
        else {
          int.columnColor = '';
        }
      });
      var fullDate = Vue.computed(function () {
        var interaction = interactions.get(props.itemid);

        return moment(interaction.date).format('YYYY-MM-DD hh:mm:ss A');
      })
      return { feature, fullDate };
    }
  }
  iservice.components.iserviceTableCellDateFromNowArticles = {
    name: 'iservice-table-cell-date-from-now-articles',
    template: '#iservice-table-cell-date-from-now-articles',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var articles = iservice.ArticleData();

      var text = Vue.computed(() => {
        return iservice.BreakWord(props.column.filterText(props.itemid));
      });
      var fullDate = Vue.computed(function () {
        var article = articles.get(props.itemid);
        return moment(article.date).format('YYYY-MM-DD hh:mm:ss A'); 
      })
      return { text, fullDate };
    }
  }
  iservice.components.iserviceTableCellArticleSimple = {
    name: 'iservice-table-cell-article-simple',
    template: '#iservice-table-cell-article-simple',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var articles = iservice.ArticleData();
      var EditDetails = Vue.inject('EditDetails');
      var editDetails = Vue.inject('editDetails');
      var messages = Vue.inject('updateMessages');
      var Messages = Vue.inject('Messages');
      var ShowDelete = Vue.inject('ShowDelete');
      var text = Vue.computed(() => {
        return iservice.BreakWord(props.column.filterText(props.itemid));
      });
      var article = Vue.computed(() => {
        return articles.get(props.itemid);
      });
      var columnName = Vue.computed(function () {
        return props.column.name;
      })
      var updateRow = Vue.inject('updateRow');
      article.value.editSubject = article.value.subject;
      article.value.editpublic = article.value.public;
      var HaveRight = Vue.computed(function () {
        return iservice.HaveRight('Tab.FindAnswers.Modify');
      });
      return { text, EditDetails, editDetails, columnName, article, updateRow, messages, Messages, ShowDelete, HaveRight };
    }
  }
  iservice.components.iserviceTableCellPlaceholderSimple = {
    name: 'iservice-table-cell-placeholder-simple',
    template: '#iservice-table-cell-placeholder-simple',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var placeholders = iservice.PlaceholdersData();
      var EditDetails = Vue.inject('EditDetails');
      var editDetails = Vue.inject('editDetails');
      var text = Vue.computed(() => {
        return iservice.BreakWord(props.column.filterText(props.itemid));
      });
      var placeholder = Vue.computed(() => {
        return placeholders.get(props.itemid);
      });


      formFeature = Vue.reactive({})
      formFeature.wizards = {
        'common-javascript': { instruction: 'Pick version of javascript to use:', list: [{ name: 'Debug', value: 'ngappbuiltin-common-javascript-dev' }, { name: 'Release (minimized)', value: 'ngappbuiltin-common-javascript' }] },
        'myqueue-answer-actions': { instruction: 'Pick version of myqueue actions to use:', list: [{ name: 'Full (with Send)', value: 'ngappbuiltin-myqueue-answer-actions' }, { name: 'Portal (No Email)', value: 'ngappbuiltin-myqueue-answer-actions-portal' }, { name: 'Secure Messaging Portal', value: 'ngappbuiltin-myqueue-answer-actions-secure-message-portal' }, { name: 'Secure Notification Portal', value: 'ngappbuiltin-myqueue-answer-actions-secure-message-portal-notification' }] },

        'stockresponse-list': { instruction: 'Pick version of stockresponse list to use:', list: [{ name: 'Load Full Body', value: 'stockresponse-list-full' }, { name: 'Load Name Only', value: 'stockresponse-list-names' }] },

        'dashboard-body1': {
          instruction: 'Pick a widget', list: [
            { name: 'My Interaction Form', value: 'my-interaction-form' },
            { name: 'My Answers Form', value: 'my-answers-form' },
            { name: 'Support Form', value: 'support-form' }]
        },

        'dashboard-body2': {
          instruction: 'Pick a widget', list: [
            { name: 'My Interaction Form', value: 'my-interaction-form' },
            { name: 'My Answers Form', value: 'my-answers-form' },
            { name: 'Support Form', value: 'support-form' }]
        },
        'dashboard-body3': {
          instruction: 'Pick a widget', list: [
            { name: 'My Interaction Form', value: 'my-interaction-form' },
            { name: 'My Answers Form', value: 'my-answers-form' },
            { name: 'Support Form', value: 'support-form' }]
        },

        'dashboard-body4': {
          instruction: 'Pick a widget', list: [
            { name: 'My Interaction Form', value: 'my-interaction-form' },
            { name: 'My Answers Form', value: 'my-answers-form' },
            { name: 'Support Form', value: 'support-form' }]
        }
      };

      formFeature.PlaceholderChoices = function (placeholder) {
        var wiz = formFeature.wizards[placeholder.name];
        if (!wiz) return null;
        return wiz.list;
      }
      formFeature.getFormList = function (value, placeholder) {
        if (value !== 'empty') {
          if (placeholder.name != 'stockresponse-list' && placeholder.name != 'myqueue-answer-actions' && placeholder.name != 'common-javascript') {
            placeholder.value = value;
          } else {
            if (value)
              placeholder.value = value;
          }
        } else {
          placeholder.value = "";
        }
      }
      return { text, EditDetails, editDetails, placeholder, formFeature};
    }
  }
  iservice.components.iserviceTableCellWebsiteBuiltInEdit = {
    name: 'iservice-table-cell-website-builtin-edit',
    template: '#iservice-table-cell-website-builtin-edit',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var placeholders = iservice.PlaceholdersData();
      var showDetails = Vue.inject('showDetails');
      var ShowDetails = Vue.inject('ShowDetails');
      var EditDetails = Vue.inject('EditDetails');
      var editDetails = Vue.inject('editDetails');
      var placeholder = Vue.computed(() => {
        return placeholders.get(props.itemid);
      });
      var updateRow = Vue.inject('updateRow');
      return { showDetails, ShowDetails, EditDetails, editDetails, updateRow, placeholder };
    }
  }
  iservice.components.iserviceTableCellExpandRowEditDelete = {
    name: 'iservice-table-cell-expand-row-edit-delete',
    template: '#iservice-table-cell-expand-row-edit-delete',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var showDetails = Vue.inject('showDetails');
      var ShowDetails = Vue.inject('ShowDetails');
      var EditDetails = Vue.inject('EditDetails');
      var editDetails = Vue.inject('editDetails');
      var HaveRight = false;
      
      var HaveRight = Vue.computed(function () {
        return iservice.HaveRight('Tab.FindAnswers.Modify');
      });
      return { showDetails, ShowDetails, EditDetails, editDetails, HaveRight };
    }
  }
  iservice.components.iserviceTableCellInteractionFromLinked = {
    name: 'iservice-table-cell-interaction-from-linked',
    template: '#iservice-table-cell-interaction-from-linked',
    props: ['itemid'],
    setup(props) {
      var interactions = iservice.InteractionData();

      var fromAddress = Vue.computed(() => {
        var int = interactions.get(props.itemid);
        if (!int) return '';
        return iservice.BreakWord(int.fromAddress);
      });
      var customerID = Vue.computed(() => {
        var int = interactions.get(props.itemid);
        if (!int) return '';
        return int.customerID;
      });
      return { fromAddress, customerID };
    }
  }
  iservice.components.iserviceTableCellInteractionDate = {
    name: 'iservice-table-cell-interaction-date',
    template: '#iservice-table-cell-interaction-date',
    props: ['itemid'],
    setup(props) {
      var interactions = iservice.InteractionData();

      var date = Vue.computed(() => {
        var int = interactions.get(props.itemid);
        if (!int) return {};
        var date = int.date;
        if (!date || date == '') return '';
        return moment(date).format('YYYY-MM-DD hh:mm:ss A');
      });
      return { date };
    }
  }

  iservice.components.iserviceTableCellInteractionContactName = {
    name: 'iservice-table-cell-interaction-customername-linked',
    template: '#iservice-table-cell-interaction-customername-linked',
    props: ['itemid'],
    setup(props) {
      var interactions = iservice.InteractionData();

      var customerName = Vue.computed(() => {
        var int = interactions.get(props.itemid);
        if (!int) return '';
        return iservice.BreakWord(int.customerName);
      });
      var customerID = Vue.computed(() => {
        var int = interactions.get(props.itemid);
        if (!int) return '';
        return int.customerID;
      });
      return { customerName, customerID };
    }
  }
  iservice.components.iserviceTableCellContactName = {
    name: 'iservice-table-cell-contact-name-linked',
    template: '#iservice-table-cell-contact-name-linked',
    props: ['itemid'],
    setup(props) {
      var contacts = iservice.ContactsData();

      var contactName = Vue.computed(() => {
        var contact = contacts.get(props.itemid);
        if (!contact) return '';
        return iservice.BreakWord(contact.name);
      });
      var customerID = Vue.computed(() => {
        var int = contacts.get(props.itemid);
        if (!int) return '';
        return int.id;
      });
      return { contactName, customerID };
    }
  }
  iservice.components.iserviceTableCellContactEmail = {
    name: 'iservice-table-cell-contact-email-linked',
    template: '#iservice-table-cell-contact-email-linked',
    props: ['itemid'],
    setup(props) {
      var contacts = iservice.ContactsData();

      var contactEmail = Vue.computed(() => {
        var contact = contacts.get(props.itemid);
        if (!contact) return '';
        return iservice.BreakWord(contact.email);
      });
      var customerID = Vue.computed(() => {
        var int = contacts.get(props.itemid);
        if (!int) return '';
        return int.id;
      });
      return { contactEmail, customerID };
    }
  }
  iservice.components.iserviceTableCellContactSMTPEmail = {
    name: 'iservice-table-cell-smtpout-email-linked',
    template: '#iservice-table-cell-smtpout-email-linked',
    props: ['itemid'],
    setup(props) {
      var smtpout = iservice.SmtpOutData();

      var contactEmail = Vue.computed(() => {
        var message = smtpout.get(props.itemid);
        if (!message) return '';
        return iservice.BreakWord(message.to);
      });
      var customerID = Vue.computed(() => {
        var message = smtpout.get(props.itemid);
        if (!message) return '';
        return message.contactID;
      });
      return { contactEmail, customerID };
    }
  }
  iservice.components.iserviceTableCellTodayActivity = {
    name: 'iservice-table-cell-today-activity',
    template: '#iservice-table-cell-today-activity',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var agents = iservice.AgentData();
      var countAssigned = Vue.computed(() => {
        var agent = agents.get(props.itemid);
        if (!agent) return 0;
        return agent.countAssigned ? agent.countAssigned : 0;
      });
      var countResolved = Vue.computed(() => {
        var agent = agents.get(props.itemid);
        if (!agent) return 0;
        return agent.countResolved ? agent.countResolved : 0;
      });

      return { countAssigned, countResolved };
    }
  }
  iservice.components.iserviceTableCellAvailable = {
    name: 'iservice-table-cell-available',
    template: '#iservice-table-cell-available',
    props: ['itemid', 'column', 'rowIDs'],
    setup(props) {
      var isAvailable = Vue.computed(() => {
        return props.column.filterText(props.itemid) ? true : false;
      });
      return { isAvailable };
    }
  }
})(iservice);
