﻿var iservice = window.iservice || {};
(function (iservice) {

  iservice.formPath = ((formPath) => {
    var toRoot = window.rootPath;
    if (typeof toRoot === 'undefined' || !formPath.startsWith('/')) {
      return;
    }
    var rootFile = formPath;
    while (toRoot.endsWith('../')) {
      var lastSlash = rootFile.lastIndexOf('/');
      rootFile = rootFile.substring(0, lastSlash);
      toRoot = toRoot.substring(0, toRoot.length - 3);
    }
    var lastSlash = rootFile.lastIndexOf('/');
    window.rootPath = rootFile.substring(0, lastSlash + 1);

    var pathEnd = formPath.startsWith('/f/') ? formPath.indexOf('/', 3) : formPath.indexOf('/', 1);
    if (pathEnd == -1) return formPath;
    return formPath.substring(0, pathEnd);
  })(window.location.pathname);

  window.emptyStr = function(str) {
    return !str || str == '';
  }

  function GetElementID(e) {
    if (e && e.currentTarget && e.currentTarget.id) return e.currentTarget.id;
    if (e && e.submitter && e.submitter.id) return e.submitter.id;
    if (e && e.id) return e.id;
  }
  function GetPage() {
    if (iservice.formPath.startsWith('/')) return iservice.formPath.substr(1);
    return iservice.formPath;
  }

  iservice.BreakWord = function (str) {
    if (!str) return '';
    var out = '';
    for (var j = 0, len = 0; j < str.length; j++, len++) {
      var c = str[j];
      out += c;
      if (c == ' ') {
        len = 0;
      }
      else if (len == 10 || c == '.' || c == '@' || c == '-' || c == '\\' || c == '/' || c == '\n') {
        out += '\u200B';
        len = 0;
      }
    }
    return out;
  }


  iservice.API = function (url, e, params, uploads) {
    params = params || {};
    params._pageAction = GetElementID(e);
    params._pageName = GetPage();

    var payload = new FormData();
    Object.getOwnPropertyNames(params).forEach(name => {
      var value = params[name];
      if (typeof value != 'string') {
        if (value instanceof String) {
          value = value.toString();
        }
        else {
          value = JSON.stringify(value);
        }
      }
      if (value === undefined) return;
      payload.append(name, value);
    });
    if (uploads) uploads.forEach((upload, i) => {
      var name = '__upload' + i;
      payload.append(name, upload.blob);
    });

    if (!url.startsWith('http://') || !url.startsWith('https://')) {
      url = rootPath + 'f/' + url;
    }
    return axios.post(url, payload, { headers: { 'Content-Type': 'multipart/form-data', 'X-Content-Type-Options': 'nosniff' } }).then(function (response) {
      if (response.status != 200) {
        return Promise.reject(Vue.reactive(['Network error.']));
      }
      var contentType = response.headers['content-type'];
      if (!contentType || !contentType.startsWith('application/json')) {
        return Promise.reject(Vue.reactive(['Something went wrong. Your last action might not have worked. You can refresh your browser (F5) and try again.']));
      }
      var data = response.data;
      if (data.errors && data.errors.length) {
        var details = data.errorDetails;
        if (!details || !details.length) return Promise.reject(Vue.reactive(data.errors));
        var errors = [];
        for (var i = 0; i < data.errors.length; i++) {
          if (i < details.length) errors.push({ msg: data.errors[i], details: data.errorDetails[i] });
          else errors.push(data.errors[i]);
        }
        return Promise.reject(Vue.reactive(errors));
      }
      return response.data;
    });
  }

  iservice.HaveRight = function (right) {
    return false;
  }
  iservice.HaveSegment = function (segmentID) {
    return false;
  }
  iservice.SuccessMessage = function (message) {
    return;
  }

  iservice.WatchLoginChange = function (callback) {
    var contactID = Vue.ref(null);
    Vue.watchEffect(() => {
      if (contactID.value != iservice.data.loggedIn.contactID) {
        contactID.value = iservice.data.loggedIn.contactID;
        callback();
      }
    });
  }

  iservice.GetLocalStoragePositiveInt = function (name) {
    var val = localStorage.getItem(name);
    if (!val) return 0;
    var num = parseInt(val);
    if (isNaN(num)) return 0;
    return Math.floor(Math.trunc(num));
  }
  iservice.GetLocalStorageJson = function (name) {
    var val = localStorage.getItem(name);
    if (!val) return {};
    return JSON.parse(val);
  }
  //localStorage.setItem('name', 'val');

  iservice.EditableProperties = function (feature, properties, outFunc) {
    feature.Reload = function (keepValues) {
      var oldValues = {};
      if (keepValues) {
        feature.props.forEach(prop => {
          oldValues[prop.def.propertyID] = prop.values;
        });
      }
      feature.props = Vue.reactive([]);
      feature.propMap = Vue.reactive({});
      function Property(src) {
        var prop = Vue.reactive({});
        prop.def = src;
        prop.values = Vue.reactive([]);
        feature.props.push(prop);
        feature.propMap[src.propertyID] = prop;
        if (oldValues[src.propertyID]) {
          prop.values = oldValues[src.propertyID];
        }
        else if (src.values && src.values.length) src.values.forEach(val => {
          var value = Vue.reactive({});
          if (src.isDate) { value.valueDate = moment(val.valueDateUtc).toDate(); }
          else if (src.isDatetime) { value.valueDatetime = moment(val.valueDatetimeUtc).toDate(); }
          else if (src.isInteger) { value.valueInt = val.valueInt; }
          else if (src.isNumber) { value.valueNumber = val.valueNumber; }
          else { value.value = val.value; }
          if (src.hasDescriptions) { value.description = val.description; }
          prop.values.push(value);
        });
        if (prop.values.length < 1 && !src.isReadOnly) {
          prop.values.push(Vue.reactive({ value: '', valueNumber: null, valueInt: null, valueDate: null, valueDatetime: null }));
        }
        prop.AddValue = function () {
          prop.values.push(Vue.reactive({ value: '', valueNumber: null, valueInt: null, valueDate: null, valueDatetime: null }));
        }
        prop.RemoveValue = function (i) {
          prop.values.splice(i, 1);
          if (prop.values.length < 1) {
            prop.values.push(Vue.reactive({ value: '', valueNumber: null, valueInt: null, valueDate: null, valueDatetime: null }));
          }
        }
        prop.OutValues = function () {
          var out = { propertyID: prop.def.propertyID, values: [] };
          prop.values.forEach(val => {
            val.hasValue = false;
            var value;
            if (prop.def.isDate) {
              if (!val.valueDate) return;
              var off = moment(val.valueDate).utcOffset();
              value = { valueDateUtc: moment(val.valueDate).add(off, 'm').toISOString() };
            }
            else if (prop.def.isDatetime) {
              if (!val.valueDatetime) return;
              value = { valueDatetimeUtc: val.valueDatetime.toISOString() };
            }
            else if (prop.def.isInteger) {
              if (!val.hasOwnProperty('valueInt')) return;
              if (val.valueInt === undefined || val.valueInt === null || val.valueInt == '') return;
              value = { valueInt: val.valueInt };
            }
            else if (prop.def.isNumber) {
              if (!val.hasOwnProperty('valueNumber')) return;
              if (val.valueNumber === undefined || val.valueNumber === null || val.valueNumber == '') return;
              value = { valueNumber: val.valueNumber };
            }
            else {
              if (!val.value || val.value.length < 1) return;
              value = { value: val.value };
            }
            if (prop.def.hasDescriptions) {
              value.description = val.description;
            }
            val.hasValue = true;
            out.values.push(value);
          });
          return out;
        }
      }
      properties.forEach(src => {
        if (src.properties && src.properties.length) {
          src.properties.forEach(prop => {
            Property(prop);
          });
        }
        else Property(src);
      });
    }
    feature.Reload();
    Vue.watchEffect(() => {
      feature.out = [];
      feature.props.forEach(prop => {
        var out = prop.OutValues();
        feature.out.push(out);
        prop.hasValues = out.values.length > 0;
        prop.missingRequired = out.values.length < 1 && prop.def.isRequired;
        var val = out.values.length ? out.values[0] : null;
        var src = prop.def;
        if (!val) {
          prop.firstValue = null;
        }
        else {
          if (src.isDate) { prop.firstValue = moment(val.valueDateUtc).format('YYYY-MM-DD'); }
          else if (src.isDatetime) { prop.firstValue = moment(val.valueDatetimeUtc).format('YYYY-MM-DD hh:mm:ss A'); }
          else if (src.isInteger) { prop.firstValue = '' + val.valueInt; }
          else if (src.isNumber) { prop.firstValue = '' + val.valueNumber; }
          else { prop.firstValue = val.value; }
        }
      });
      feature.hasValues = false;
      feature.missingRequired = false;
      feature.props.forEach(prop => {
        if (prop.hasValues) feature.hasValues = true;
        if (prop.missingRequired) feature.missingRequired = true;
      });
      if (outFunc) outFunc(feature.out);
    });
  }
  iservice.CopyProps = function (destArray, srcArray, valuesMap) {
    if (srcArray && srcArray.length !== undefined) {
      destArray.length = 0;
      srcArray.forEach(group => {
        group.properties.forEach(prop => {
          destArray.push(prop);
          if (valuesMap) valuesMap[prop.propertyID] = prop.values;
        });
      });
    }
  }
  iservice.InteractionHasPropValue = function (int, propID, value) {
    return [int.intProps, int.caseProps].find(propList => {
      return propList.find(prop => {
        if (prop.id != propID) return false;
        return prop.values.find(val => {
          if (prop.isInteger) return ('' + val.valueInt).localeCompare(value) == 0;
          if (prop.isNumber) return ('' + val.valueNumber).localeCompare(value) == 0;
          if (prop.isDate) return false;
          if (prop.isDatetime) return false;
          return val.value && val.value.localeCompare(value) == 0;
        }) != null;
      }) != null;
    }) != null;
  }

  iservice.CidReference = function (attach) {
    if (!attach.contentID) return null;
    function quoteattr(s) {
      return ('' + s) /* Forces the conversion to string. */
          .replace(/&/g, '&amp;') /* This MUST be the 1st replacement. */
          .replace(/'/g, '&apos;') /* The 4 other predefined entities, required. */
          .replace(/"/g, '&quot;')
          .replace(/</g, '&lt;')
          .replace(/>/g, '&gt;');
    }
    return 'src="cid:' + quoteattr(attach.contentID) + '"';
  }
  iservice.FileReference = function (attach) {
    return 'src="' + rootPath + 'File.aspx?interactionID=' + attach.interactionID + '&amp;fileID=' + attach.attachmentID + '"';
  }
  iservice.FileSearch = function (attach) {
    return new RegExp('src="[^"]*File\\.aspx\\?interactionID=' + attach.interactionID + '&amp;fileID=' + attach.attachmentID + '"');
  }
  iservice.FindInlineAttachments = function (bodyHtml, attachments, isInlineMap) {
    if (!bodyHtml) return '';
    attachments.forEach(attach => {
      if (attach.attachmentID == 'email') return;
      var cidStr = iservice.CidReference(attach);
      if (!cidStr) return;
      var fileStr = iservice.FileReference(attach);
      if (bodyHtml.lastIndexOf(cidStr) != -1) {
        bodyHtml = bodyHtml.replace(cidStr, fileStr);
      }
      var fileRegex = iservice.FileSearch(attach);
      if (bodyHtml.match(fileRegex)) {
        isInlineMap[attach.attachmentID] = true;
      }
    });
    return bodyHtml;
  }
  iservice.RestoreInlineAttachments = function (bodyHtml, attachments) {
    if (!bodyHtml) return '';
    attachments.forEach(attach => {
      if (attach.attachmentID == 'email') return;
      var fileRegex = iservice.FileSearch(attach);
      var cidStr = iservice.CidReference(attach);
      if (!cidStr) return;
      bodyHtml = bodyHtml.replace(fileRegex, cidStr);
    });
    return bodyHtml;
  }
  iservice.DownloadableFiles = function (attachments, isInlineMap) {
    var found = [];
    if (attachments) attachments.forEach(attach => {
      if (attach.attachmentID == 'email' || isInlineMap[attach.attachmentID]) return;
      var name = attach.name;
      var parts = iservice.NameParts(name);
      if (parts.mime) {
        name = name.replace(' (' + parts.mime + ')', '');
      }
      var item = Vue.reactive({
        attachmentID: attach.attachmentID,
        name: name,
        iconClass: iservice.FileIcon(parts.ext)
      });
      found.push(item);
    });
    return Vue.reactive(found);
  }
  iservice.HaveEmailDownload = function (attachments) {
    return Vue.ref(function () {
      for (var i = 0; i < attachments.length; i++) {
        if (attachments[i].attachmentID == 'email') return true;
      }
      return false;
    }());
  }

  iservice.FeatureUploads = function () {
    var feature = Vue.reactive({ files: [] });
    feature.Reset = function () {
      feature.files.length = 0;
    }
    feature.Drop = function (e) {
      if (!e || !e.dataTransfer || !e.dataTransfer.items) return;
      for (var i = 0; i < e.dataTransfer.items.length; i++) {
        var file = e.dataTransfer.items[i].getAsFile();
        iservice.UploadFile(file, feature.files);
      }
    }
    feature.Choose = function (e) {
      if (!e || !e.target || !e.target.files) return;
      for (var i = 0; i < e.target.files.length; i++) {
        var file = e.target.files[i];
        iservice.UploadFile(file, feature.files);
      }
    }
    feature.Remove = function (file) {
      var pos = feature.files.indexOf(file);
      if (pos == -1) return;
      feature.files.splice(pos, 1);
    }
    return feature;
  }
  iservice.UploadFile = function (file, files) {
    var name = file.name;
    var parts = iservice.NameParts(name);
    if (parts.mime) {
      name = name.replace(' (' + parts.mime + ')', '');
    }
    var item = Vue.reactive({
      blob: file,
      name: name,
      iconClass: iservice.FileIcon(parts.ext)
    });
    files.push(item);
  }

  iservice.NameParts = function (filename) {
    var match = /\.(\w+)( +\((.*)\))?$/g.exec(filename);
    if (!match) return { ext: '', mime: '' };
    var ext = match[1];
    var mime = match[3];
    return { ext, mime };    
  }
  iservice.FileIcon = function (ext) {
    switch (ext) {
      case 'xls':
      case 'xlsx':
        return 'fa-file-excel-o';
      case 'pdf':
        return 'fa-file-pdf-o';
      case 'wav':
      case 'mpeg':
      case 'mp3':
      case 'ogg':
        return 'fa-file-audio-o';
      case 'doc':
      case 'docx':
        return 'fa-file-word-o';
      case 'zip':
      case '7z':
      case 'rar':
        return 'fa-file-archive-o';
      case 'gif':
      case 'png':
      case 'jpeg':
      case 'jpg':
      case 'bmp':
      case 'svg':
      case 'webp':
      case 'ico':
        return 'fa-file-image-o';
      case 'txt':
      case 'html':
      case 'css':
      case 'js':
      case 'md':
        return 'fa-file-text-o';
      case 'webm':
      case 'mp4':
      case 'avi':
        return 'fa-file-video-o';
      case 'ppt':
      case 'pptx':
        return 'fa-file-powerpoint-o';
    }
    return 'fa-file-o';
  }

  iservice.AttachmentIcons = function (attachments) {  //delete me
    function getFileExtensionIcon(fileName, type) {
      if (!type) {
        fileName = fileName.split('(');
        fileName = fileName[0].trim();
      }
      var fileNameRegex = /.+\.(\w+)$/g;
      var genericMimeType = 'fa-file-o';
      var match = fileNameRegex.exec(fileName);
      if (!match)
        return genericMimeType;

      var extension = match[1];

      switch (extension) {
        case 'xls':
        case 'xlsx':
          return 'fa-file-excel-o';

        case 'pdf':
          return 'fa-file-pdf-o';

        case 'wav':
        case 'mpeg':
        case 'mp3':
        case 'ogg':
          return 'fa-file-audio-o';

        case 'doc':
        case 'docx':
          return 'fa-file-word-o';

        case 'zip':
        case '7z':
        case 'rar':
          return 'fa-file-archive-o';

        case 'gif':
        case 'png':
        case 'jpeg':
        case 'jpg':
        case 'bmp':
        case 'svg':
        case 'webp':
        case 'ico':
          return 'fa-file-image-o';

        case 'txt':
        case 'html':
        case 'css':
        case 'js':
        case 'md':
          return 'fa-file-text-o';

        case 'webm':
        case 'mp4':
        case 'avi':
          return 'fa-file-video-o';

        case 'ppt':
        case 'pptx':
          return 'fa-file-powerpoint-o';
      }

      return genericMimeType;
    }
    if (!attachments || !attachments.length) return;
    attachments.forEach(function (attachment) {
      attachment.iconClass = attachment.type ? getFileExtensionIcon(attachment.name, attachment.type) : getFileExtensionIcon(attachment.name, null);
    });
  }
  function ClassName(str) {
    str = str.toLowerCase();
    var name = '';
    for (var i = 0; i < str.length; i++) {
      var c = str.charAt(i);
      if (c == ' ') name += '-';
      else if (c >= 'a' && c <= 'z') name += c;
      else if (c >= '0' && c <= '9') name += c;
    }
    return name;
  }
  iservice.SortID = function (lidstr, ridstr) {
    var lid = parseInt(lidstr);
    var rid = parseInt(ridstr);
    if (isNaN(lid)) return 1;
    if (isNaN(rid)) return -1;
    if (lid < rid) return -1;
    if (lid > rid) return 1;
    return 0;
  }
  function hasValue(obj, key, value) {
    return obj.hasOwnProperty(key) && obj[key] === value;
  }
  iservice.SanitizeAnswerData = function (details, answer, answerData, guid) { //delete me
    if (details.date) details.dateObj = new Date(details.date);
    answerData.propertySections = [];
    answer.caseProperties = null;
    answer.interactionProperties = null;
    answer.responseProperties = null;
    answer.leaveOpen = null;
    answer.expectReply = null;
    answer.supressSubjectRef = null;
    answer.notificationID = null;
    answer.bodyHtml = details.responseHtml;
    iservice.AttachmentIcons(details.attachments);
    iservice.AttachmentIcons(details.responseAttachments);
    iservice.SanitizeInlineImageAttachments(answer, 'bodyHtml', details.attachments, guid);
    iservice.SanitizeInlineImageAttachments(answer, 'bodyHtml', details.responseAttachments, guid);
    answer.bodyHtml = answer.bodyHtml.replace(/\r\n/g, '\n');
    answer.subject = details.responseSubject;
    answer.mailboxID = details.defaultMailboxID;
    answer.note = details.note;
    answerData.responseAttachments = details.responseAttachments;
    answer.attachments = [];
    if (details.responseAttachments) details.responseAttachments.forEach(function (attach) {
      answer.attachments.push(attach.attachmentID);
    });
    details.countNonInlineAttachments = 0;
    if (details.attachments) details.attachments.forEach(function (attach) {
      if (!attach.isInline) details.countNonInlineAttachments++;
    });
  }
  iservice.SanitizeInlineImageAttachments = function (hasBodyHtml, bodyVar, attachments, guid) { //delete me
    if (!attachments) return;
    if (!hasBodyHtml.removedInlineAttachments) hasBodyHtml.removedInlineAttachments = [];
    for (var i = 0; i < attachments.length; i++) {
      var attach = attachments[i];
      var idParam = guid ? 'guid=' + guid : 'interactionID=' + attach.interactionID;
      var oldSrc = 'src="cid:' + attach.contentID + '"';
      var newSrc = 'src="' + rootPath + 'File.aspx?' + idParam + '&amp;fileID=' + attach.attachmentID + '"';
      if (hasBodyHtml[bodyVar].lastIndexOf(oldSrc) !== -1) {
        hasBodyHtml[bodyVar] = hasBodyHtml[bodyVar].replace(oldSrc, newSrc);
      }
      if (hasBodyHtml[bodyVar].lastIndexOf(newSrc) !== -1) {
        attach.isInline = true;
        hasBodyHtml.removedInlineAttachments.push(attach);
      }
    }
  }
  iservice.UpdateTable = function (list, updatedItem, idFieldName) {
    if (updatedItem.isDeleted === undefined) updatedItem.isDeleted = false;
    for (var i = 0, len = list.length; i < len; i++) {
      if (list[i][idFieldName] == updatedItem[idFieldName]) {
        if (updatedItem.isDeleted == true) {
          list.splice(i, 1);
          return false;
        }
        else {
          list[i] = updatedItem;
          return true;
        }
      }
    }
    if (updatedItem.isDeleted == false) list.push(updatedItem);
    return false;
  }
  iservice.CollectProperties = function (groups) { //delete me
    var properties = [];
    for (var g = 0; g < groups.length; g++) for (var p = 0; p < groups[g].properties.length; p++) {
      var source = groups[g].properties[p];
      if (source.isReadOnly) continue;
      var prop = { propertyID: source.propertyID, propertyName: source.name, values: [] };
      properties.push(prop);

      for (var v = 0; v < source.values.length; v++) {
        var val = source.values[v];
        var value;
        if (source.isDate) {
          if (!val.valueDate) continue;
          var off = moment(val.valueDate).utcOffset();
          value = { valueDateUtc: moment(val.valueDate).add(off, 'm').toISOString() };
        }
        else if (source.isDatetime) {
          if (!val.valueDatetime) continue;
          value = { valueDatetimeUtc: val.valueDatetime.toISOString() };
        }
        else if (source.isInteger) {
          if (!val.hasOwnProperty('valueInt')) continue;
          if (val.valueInt === undefined || val.valueInt === null) continue;
          value = { value: '', valueInt: val.valueInt };
        }
        else if (source.isNumber) {
          if (!val.hasOwnProperty('valueNumber')) continue;
          if (val.valueNumber === undefined || val.valueNumber === null) continue;
          value = { value: '', valueNumber: val.valueNumber };
        }
        else {
          if (!val.value || val.value.length < 1) continue;
          value = { value: val.value };
        }
        if (source.hasDescriptions) {
          value.description = val.description;
        }
        prop.values.push(value);
      }
    }
    return properties;
  }
  iservice.ReplaceInlineImageRefrences = function (hasBodyHtml, bodyVar, guid) { //delete me
    if (!hasBodyHtml.removedInlineAttachments) return;
    for (var i = 0; i < hasBodyHtml.removedInlineAttachments.length; i++) {
      var attach = hasBodyHtml.removedInlineAttachments[i];
      var oldSrc = 'src="cid:' + attach.contentID + '"';
      var idParam = guid ? 'guid=' + guid : 'interactionID=' + attach.interactionID;
      var newSrc = 'src="' + rootPath + 'File.aspx?' + idParam + '&amp;fileID=' + attach.attachmentID + '"';
      if (hasBodyHtml[bodyVar].lastIndexOf(newSrc) !== -1) {
        hasBodyHtml[bodyVar] = hasBodyHtml[bodyVar].replace(newSrc, oldSrc);
      }
    }
  }
  iservice.PropertyValueExists = function (property) { //delete me
    if (!property) return false;
    if (property.values.length < 1) return false;
    var value = property.values[0];
    if (property.isDate) {
      var date = value.valueDate;
      return date ? true : false;
    }
    if (property.isDatetime) {
      var datetime = value.valueDatetime;
      return datetime ? true : false;
    }
    if (property.isInteger) return value.valueInt == null || value.valueInt == undefined ? false : true;
    if (property.isNumber) return value.valueNumber == null || value.valueNumber == undefined ? false : true;
    if (property.valueChoices.length) return value.value.length > 0;
    return value.value.trim().length > 0;
  }

  iservice.interactionTypeClasses = {
    'Agent Notification': {
      className: 'bg-gray-disabled',
      icon: 'icon-c icon-agent-notification-2',
      secondaryIcon: null
    },
    'Message Acknowledgment': {
      className: 'bg-gray-disabled ',
      icon: 'fa-reply',
      secondaryIcon: null
    },
    'Contact Creation': {
      className: 'bg-gray-disabled label-with-icon',
      icon: 'fa-address-book-o',
      secondaryIcon: null
    },
    'External Agent Notification': {
      className: 'bg-gray-disabled label-with-icon',
      icon: 'icon-c icon-external-notification-1',
      secondaryIcon: null
    },
    'Mass Mailing': {
      className: 'bg-gray-disabled label-with-icon',
      icon: 'fa-envelope',
      secondaryIcon: null
    },
    'Mass Mailing Article Update': {
      className: 'bg-gray-disabled label-with-icon',
      icon: 'icon-c icon-find-answer-update',
      secondaryIcon: null
    },
    'Password Reset': {
      className: 'bg-gray-disabled label-with-icon',
      icon: 'fa-key',
      secondaryIcon: null
    },
    'Alert': {
      className: 'bg-gray-disabled label-with-icon',
      icon: 'fa-exclamation-triangle',
      secondaryIcon: null
    },
    'Secure Response': {
      className: 'bg-green label-with-icon',
      icon: 'icon-c icon-secure-response',
      secondaryIcon: null
    },
    'Private Email': {
      className: 'bg-gray-disabled label-with-icon',
      icon: 'icon-c icon-secure-email',
      secondaryIcon: null
    },
    'Customer Email': {
      className: 'bg-blue label-with-icon',
      icon: 'fa-envelope',
      secondaryIcon: null
    },
    'Agent Ticket': {
      className: 'bg-blue label-with-icon',
      icon: 'fa-ticket',
      secondaryIcon: null
    },
    'Customer Ticket': {
      className: 'bg-blue label-with-icon',
      icon: 'fa icon-c icon-customer-ticket',
      secondaryIcon: null
    },
    'Chat': {
      className: 'bg-blue disabled label-with-icon',
      icon: 'fa-comments',
      secondaryIcon: null
    },
    'Agent Email': {
      className: 'bg-green label-with-icon',
      icon: 'icon-c icon-agent-email-3',
      secondaryIcon: null
    },
    'Agent Response': {
      className: 'bg-green label-with-icon',
      icon: 'icon-c icon-agent-response',
      secondaryIcon: null
    },
    'Secure Email': {
      className: 'bg-green-active label-with-icon',
      icon: 'icon-c icon-secure-email',
      secondaryIcon: null
    },
    'Public Note': {
      className: 'bg-teal-active label-with-icon',
      icon: 'fa fa-file-text-o',
      secondaryIcon: null
    },
    'Private Note': {
      className: 'bg-teal-active label-with-icon',
      icon: 'icon-c icon-public-note',
      secondaryIcon: null
    },
    'Resolve Note': {
      className: 'bg-teal-active label-with-icon',
      icon: 'icon-c icon-resolve-note',
      secondaryIcon: null
    },
    'Note': {
      className: 'bg-teal-active label-with-icon',
      icon: 'fa-file-text',
      secondaryIcon: null
    },
    'Unsent Response': {
      className: 'bg-teal-active label-with-icon',
      icon: 'icon-c icon-unsent-response',
      secondaryIcon: null
    },
    'Find Answer Article': {
      className: 'bg-teal-active label-with-icon',
      icon: 'fa-search',
      secondaryIcon: null
    },
    'Notification': {
      className: 'label bg-gray-disabled label-with-icon',
      icon: 'fa-bell',
      secondaryIcon: null
    },
    'Secure Notification': {
      className: 'label bg-gray-disabled label-with-icon',
      icon: 'fa-bell',
      secondaryIcon: null
    },
    'Ticket': {
      className: 'bg-blue label-with-icon',
      icon: 'fa-ticket',
      secondaryIcon: null
    },
    'Ask A Question': {
      className: 'bg-blue label-with-icon',
      icon: 'fa icon-c icon-customer-ticket',
      secondaryIcon: null
    }
  }
  iservice.interactionRowColorClasses = {
    'Agent Email': {
      className: 'bg-answers-sent'
    },
    'Agent Response': {
      className: 'bg-answers-sent'
    },
    'Message Acknowledgment': {
      className: 'bg-answers-sent'
    },
    'Secure Email': {
      className: 'bg-answers-sent'
    },
    'Public Note': {
      className: 'bg-answers'
    },
    'Private Note': {
      className: 'bg-answers'
    },
    'Resolve Note': {
      className: 'bg-answers'
    },
    'Unsent Response': {
      className: 'bg-answers'
    },
    'Find Answer Article': {
      className: 'bg-answers'
    },
    'Ticket': {
      className: 'bg-questions'
    },
    'Ask A Question': {
      className: 'bg-questions'
    },
    'Customer Email': {
      className: 'bg-questions'
    },
    'Agent Ticket': {
      className: 'bg-questions'
    },
    'Customer Ticket': {
      className: 'bg-questions'
    },
    'Chat': {
      className: 'bg-questions'
    },
    'Agent Notification': {
      className: 'bg-notification'
    },
    'Message Acknowledgment': {
      className: 'bg-notification'
    },
    'External Agent Notification': {
      className: 'bg-notification'
    },
    'Secure Notification': {
      className: 'bg-notification'
    }
  }
  iservice.interactionTypeLabels = {
    'Agent Notification': {
      label: 'Agent Notification'
    },
    'Message Acknowledgment': {
      label: 'Message Acknowledgment'
    },
    'Contact Creation': {
      label: 'Contact Creation'
    },
    'External Agent Notification': {
      label: 'External Agent Notification'
    },
    'Mass Mailing': {
      label: 'Mass Mailing'
    },
    'Mass Mailing Article Update': {
      label: 'Mass Mailing Article Update'
    },
    'Password Reset': {
      label: 'Password Reset'
    },
    Alert: {
      label: 'Alert'
    },
    'Secure Response': {
      label: 'Secure Response'
    },
    'Private Email': {
      label: 'Private Email'
    },
    'Customer Email': {
      label: 'Customer Email'
    },
    'Chat': {
      label: 'Chat'
    },
    'Agent Email': {
      label: 'Agent Email'
    },
    'Agent Response': {
      label: 'Agent Response'
    },
    'Secure Email': {
      label: 'Secure Email'
    },
    'Public Note': {
      label: 'Public Note'
    },
    'Private Note': {
      label: 'Private Note'
    },
    'Resolve Note': {
      label: 'Resolve Note'
    },
    'Note': {
      label: 'Note'
    },
    'Unsent Response': {
      label: 'Unsent Response'
    },
    'Find Answer Article': {
      label: 'Knowledge Base Article'
    },
    'Notification': {
      label: 'Secure Notification'
    },
    'Ticket': {
      label: 'Agent Ticket'
    },
    'Ask A Question': {
      label: 'Customer Ticket'
    }
  }
  iservice.notificationtypes = [
    { name: "Contact Creation", value: "accountcreation" },
    { name: "Message Acknowledgment", value: "topicautoresponse" },
    { name: "Agent Notification", value: "agentnotification" },
    { name: "Password Reset", value: "passwordresetnotification" },
    { name: "Secure Notification", value: "securemessagenotification" },
    { name: "Alert", value: "alert" },
    { name: "Knowledge Base Article Notify", value: "articlenotification" }
  ];
  iservice.NotificationTemplates = [
    {
      "agentnotification": [
        {
          name: "Reply at Start",
          bodyHtml: '$value -notifyresponse(bodyatstart)$<br><br>$value -notifyresponse(id)$ <br><br>Place your instruction for the receiving agent here.'
        },
        {
          name: "Reply in Middle",
          bodyHtml: 'Place your instruction for the receiving agent here.<br><br>$value -notifyresponse(id)$<br><br>$value -notifyresponse(body)$'
        },
        {
          name: "External Answer Form",
          bodyHtml: 'The variable below will create a link the agent can click to provide an answer using the External Answer form. Learn move about the external answer form at https://www.iservice.info/guides/users/external-answer-page.html<br><br>$value -forwardexternal(url)$'
        }
      ]
    },
    {
      "securenotification": [
        {
          name: "Secure Notifictaion",
          bodyHtml: 'Dear $value -customer(name)$,<br /><br /> An answer has been provided to a question you recently submitted to { insert your company name }.To protect your privacy, you will need to log into our secure message center to view the answer.Please log into your account at { insert the URL for your customer portal } to view the answer that has been provided.<br /><br />Your email address is used as your login.If you have forgotten your password or did not receive one when your account was created, you can request a new one by visiting the link below.<br /><br />{insert your tenant url here}/f/forgot-password<br /><br />This is an automatically generated message from One - to - One Service.If you have any questions, please visit the One-to-One Service website and select the Contact Us link.Do not respond to this email because it was generated from an account that is not monitored.'
        }
      ]
    },
    {
      "kbarticlenotification": [
        {
          name: "KB Article Notifictaion",
          bodyHtml: 'Dear $value -customer(name)$,<br /><br />An article that you have subscribed to within our knowledge base has been updated.We\'re sending you this notification because you indicated that you\'d like to know when the information is updated.The article was updated $value - today$.<br /><br />Some details about this article are shown below.<br /><br />Article Title: $value - question(subject)$ <br /><br />To view the new article, you can visit the link below.<br />$value - article(url)$ <br /><br />If you\'d like to update your subscriptions or discontinue receiving notifications, you can log into your support portal at {insert link to your support portal}.'
        }
      ]
    },
    {
      "passwordresetnotification": [
        {
          name: "Password Reset Notifictaion",
          bodyHtml: '<p>Dear $value -customer(name)$,<br /><br />We received a request to reset your password for {insert your tenant url here}. This is the customer interaction system we use to improve the way we communicate with you. Please click on the URL below and enter a new password.<br />$value -passwordreset(url)$<br /><br />This request was from a computer at the following IP address.<br />$value -passwordreset(ipaddress)$'
        }
      ]
    },
    {
      "accountcreation": [
        {
          name: "Basic Contact Creation",
          bodyHtml: 'Hello $value -customer(name)$,<br><br>We’ve created an account for you in our support center for your email address $value -question(fromemail)$.We use the support center to enhance the way we communicate with you.The account was created because we received an email from this address or you provided the e - mail address to one of our service agents.<br><br>Your login details are below:<br>Email: $value -question(fromemail)$<br>Password: $value -customer(password)$<br>           <br>Go to {enter the URL for your iService support center} to log into our support center. You can change your password and update your contact information as desired.'
        }
      ]
    },
    {
      "topicautoresponse": [
        {
          name: "Simple Acknowledgement",
          bodyHtml: '<table border="0" width="100%" style="height:50px;"><tbody>       <tr><td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td></tr></tbody>/table><table border="0"><tbody>        <tr><td><br><br>Dear $value -customer(name)$,<br><br>We received your question and are sending you this automated message to let you know we are working on an answer. If you would like to search our knowledge base for answers to common questions, please click on the link below.<br><br>{insert link to your support center}<br><br>We attempt to review and answer all inquiries within 4 hours. If you have an urgent question, please call us at the number shown below. When speaking with a representative, please refer to the email address from which you sent this inquiry ($value -question(fromemail)$).<br><br>Thanks for your patience as we review your question. We appreciate your business very much.<br><br>Regards,<br><br><strong></strong>The eShop Customer Support Team<br>Tel: 800-555-1212<br><br><br></td></tr><tr bgcolor="#d4d4d4"><td><b>Your Original Question is as Follows:</b><br><hr width="100%" size="2" align="center"></td></tr><tr bgcolor="#ededed"><td>        <br>$value -question(body)$<br><br></td></tr></tbody></table><br><br> <br><br><font size="1">Powered by <a href="http://www.1to1service.com">iService CRM</a></font><br>'
        }
      ]
    },
    {
      "alert": [
        {
          name: "Simple Table",
          bodyHtml: '<table border="0" cellpadding="0" cellspacing="0" style="background:#f1f1f1; border:0px solid #cccccc; color:#333; width:100%"><tbody><tr><td>&nbsp;</td><td width="880" style="width:880px"><center><table border="0" cellpadding="0" cellspacing="0" style="background:#ffffff; border:0px solid #cccccc; color:#333; width:880px"><tbody><!--header jhony seeds--><tr><td style="border-color:#ffffff;border-bottom:20px solid #577632"></td></tr><tr><td><br>This example generates a list of interactions that match the criteria for an alert. Use this template with the Send Notifcation action in the Alerts configuration.</td></tr><tr><td>&nbsp;</td></tr><tr><td style="text-align:center;padding-left:30px; padding-right:30px"><h1 style="font-size:26px; font-weight:500; text-align:center">Unanswered Question Alert</h1></td></tr><tr><td>&nbsp;</td></tr><tr><td></td></tr><tr><td style="border-color:#ffffff; vertical-align:top"><table border="0" cellpadding="0" cellspacing="0"><tbody><tr><td style="border-color:#ffffff; width:30px"></td><td><span style="font-family:Verdana; font-size:14px; line-height: 23px"><br><br><span style="font-weight: bold">Name of Alert:</span>&nbsp;$value -alert(name)$<br><span style="font-weight: 600">Number of Question:</span>&nbsp;$value -alert(count)$<br><b><br></b><br><span style="color: rgb(0, 0, 0); font-family: Arial, Verdana">The details for messages matching the criteria of this alert are shown below.</span><br><br></span></td><td style="width:30px">&nbsp;</td></tr><tr><td style="border-color:#ffffff; width:30px">&nbsp;</td><td><div style="font-family:Verdana; font-size:14px; line-height:24px"><div style="font-weight: bold; text-align:center">Unanswered Question List</div><div class="">&nbsp;</div><div style="width:820px"><table width="100%" cellspacing="3" cellpadding="3" border="0" align="left"><thead style="background:#dfdddd;color:#333; font-size:14px; font-weight:normal"><tr><th style="font-weight:normal">Action</th><th style="font-weight:normal">Date of Original Msg</th><th style="font-weight:normal">Contact Name</th><th style="font-weight:normal">Contact Email</th><th style="font-weight:normal">Subject</th><th style="font-weight:normal">Assigned Agent</th></tr></thead><tbody><!-- $repeat -alertitems$ --><tr style="font-size:14px"><td style="font-weight:normal;padding:8px"><a href="https://eshopadmin.iservicecrm.com/f/forward-external?id=$value -forwardexternal(guid)$" style="text-decoration:none; cursor:pointer"><button style="width:90px;font-size:12px; text-align:center;display:block;height:22px; background:#577632; color:#fff; text-decoration:none;border:none">Reply Now</button></a></td><td style="font-weight:normal;padding:8px">$value -alertitem(date)$ $value -alertitem(time)$</td><!--Edit the line below for your iService URL--><td style="font-weight:normal;padding:8px">$value -alertitem(name)$<br></td><td style="font-weight:normal;padding:8px">$value -alertitem(email)$</td><td style="font-weight:normal;padding:8px">$value -alertitem(subject)$</td><td style="font-weight:normal;padding:8px">$value -alertitem(agent)$</td></tr><!-- $endrepeat$  --></tbody></table></div></div></td><td style="width:30px">&nbsp;</td></tr></tbody></table></td></tr><tr><td style="border-color:#ffffff">&nbsp;</td></tr><tr><td style="border-color:#ffffff">&nbsp;</td></tr><tr><td style="border-color:#ffffff">&nbsp;</td></tr><!--footer--><tr><td style="border-color:#ffffff;border-bottom:2px solid #577632">&nbsp;</td></tr><tr><td style="border-color:#ffffff;background:#f1f1f1">&nbsp;</td></tr><tr><td style="text-align:center;background:#f1f1f1">&nbsp;</td></tr><tr><td style="text-align:center;background:#f1f1f1">&nbsp;</td></tr><tr><td style="text-align:center; background:#f1f1f1"><span style="font-size:13px">Powered by -<a href="https://iservice.us"> <span style="color:#3c8dbc">iService</span></a></span></td></tr><tr><td style="background:#f1f1f1">&nbsp;</td></tr><tr><td style="background:#f1f1f1">&nbsp;</td></tr><!--footer--></tbody></table></center></td><td>&nbsp;</td></tr></tbody></table>'
        }
      ]
    }
  ]
  iservice.PrinterInboxProperty = function (intProperties, propID) {
    return function (inboxproperties) {
      if (!inboxproperties.forID) {
        inboxproperties.forID = {};
        inboxproperties.forEach(function (prop) {
          inboxproperties.forID[prop.propertyID] = prop;
        });
      }
      var prop = inboxproperties.forID[propID];
      if (!prop) return '';
      var def = intProperties.get(propID);
      return PrintValues(prop.values, DefinitionType(def));
    }
  }
  iservice.RemoveFromArrayInPlace = function (arr, func) {
    //func = takes (item, index), returns true if item should be removed
    var start = arr.length;
    var num = 0;
    for (var i = arr.length - 1; i >= 0; i--) {
      if (func(arr[i], i)) {
        start = i;
        num++;
      }
      else if (num > 0) {
        arr.splice(start, num);
        num = 0;
      }
    }
    if (num > 0) arr.splice(start, num);
  }
  iservice.FindArray = function(arr, func) {
    if (!arr) return null;
    for (var i = 0; i < arr.length; i++) {
      var item = arr[i];
      var ret = func(item, i);
      if (ret) {
        return ret;
      }
    }
    return null;
  }
  iservice.ForArray = function (arr, func) {
    if (!arr) return;
    for (var i = 0; i < arr.length; i++) {
      var item = arr[i];
      var ret = func(item, i);
      if (ret === false) return;
    }
  }

  iservice.TransformArray = function (arr, func) {
    if (!arr) return [];
    var arrnew = [];
    iservice.ForArray(arr, function (el) {
      arrnew.push(func(el));
    });
    return arrnew;
  }
  iservice.guidGenerator = function () {
    var S4 = function () {
      return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
    };
    return (S4() + S4());
  }
  iservice.nameInitials = function (name) {
    function getParts(name) {
      if (name.indexOf(' ') >= 0)
        return name.split(' ');
      if (name.indexOf('@') >= 0)
        return getParts(name.split('@')[0]);
      if (name.indexOf('.') >= 0)
        return name.split('.');
      if (name.indexOf('_') >= 0)
        return name.split('_');
      return [name, ''];
    }
    var parts = getParts(name);
    return parts[0][0] + (parts[1][0] || '');
  };
  iservice.nameStyle = function (name) {
    function getHashCode(str) {
      // from https://stackoverflow.com/a/7616484/95970
      var hash = 0, i, chr;
      if (!str.length)
        return hash;
      for (i = 0; i < str.length; i++) {
        chr = str.charCodeAt(i);
        hash = (hash << 5) - hash + chr;
        hash |= 0; // Convert to 32bit integer
      }
      return hash;
    }
    function getDarkColor(seed) {
      const hexDigits = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'];
      // from https://stackoverflow.com/a/20114561/95970
      var color = '#';
      seed = Math.abs(seed);
      color += hexDigits[seed % 5];
      for (var i = 1; i <= 5; i++)
        color += hexDigits[seed * i % 15];
      return color;
    }
    return { 'background-color': getDarkColor(getHashCode(name)) };
  };
  iservice.SetPropertyValue = function (groups, propertyName, value) {
    if (!groups) return;
    groups.forEach(function (group) {
      group.properties.forEach(function (property) {
        if (property.name == propertyName) property.values = [{ value: value }];
      });
    });
  }
  iservice.HourMinSec = function(totalSeconds) {
    function twoDigits(val) {
      return (val < 10) ? '0' + val : val;
    }
    if (totalSeconds < 0) totalSeconds = -totalSeconds;
    if (totalSeconds > 35999999) totalSeconds = 35999999;
    var hours = Math.floor(totalSeconds / 3600);
    var minutes = Math.floor((totalSeconds - hours * 3600) / 60);
    var seconds = Math.floor(totalSeconds - hours * 3600 - minutes * 60);
    return (hours > 0) ? (hours + ':' + twoDigits(minutes) + ':' + twoDigits(seconds)) :
      (minutes + ':' + twoDigits(seconds));
  }
  iservice.HaveAccess = function (agent, topic) {
    if (!agent.haveSegment[topic.segmentID]) return false;
    if (!topic.skillIDs || !topic.skillIDs.length) return true;
    return !topic.skillIDs.find(function (skillID) {
      return !agent.haveSkill[skillID];
    });
  }
  iservice.FormsstepsOptions = [
    {
      label: "Forms Actions",
      options: [
        { name: "Admin Forms", value: "adminForms" },
        { name: "Admin Stock Responses", value: "adminStockResponses" },
        { name: "Admin User Types", value: "adminUsertypes" },
        { name: "Admin Departments", value: "adminDepartments" },
        { name: "Admin Segments", value: "adminSegments" },
        { name: "Admin Interaction Properties", value: "adminInteractionProperties" },
        { name: "Admin Mailboxes", value: "adminMailboxes" },
        { name: "Admin Topics", value: "adminTopics" },
        { name: "Admin Auto Responses", value: "adminAutoresponses" },
        { name: "Admin Agents", value: "adminAgents" },
        { name: "Admin Massmail Lists", value: "adminMassLists" },
        { name: "Admin Massmail Campaigns", value: "adminMassCampaigns" },
        { name: "Admin Massmail Criterias", value: "adminMassCriterias" },
        { name: "Admin Massmail Mailings", value: "adminMassMailings" },
        { name: "Admin Massmail Messages", value: "adminMassMessages" },
        { name: "Admin Massmail Subscriptions", value: "adminSubscriptions" },
        { name: "Massmail Send", value: "mailingsend" },
        { name: "Admin Lists (aka Additional Details)", value: "adminLists" },
        { name: "Admin OpenID", value: "adminOpenID" },
        { name: "Admin Websites", value: "adminWebsites" },
        { name: "Admin Skills", value: "adminSkills" },
        { name: "Captcha Verify", value: "captchaVerify" },
        { name: "Chat Create", value: "chatCreate" },
        { name: "Chat End", value: "chatEnd" },
        { name: "Chat Forward", value: "chatForward" },
        { name: "Chat Get Next", value: "chatGetNext" }, 
        { name: "Chat Keep Alive", value: "chatKeepAlive" },
        { name: "Chat Send", value: "chatSend" },
        { name: "Chat Toggle Availability", value: "chatToggleAvailability" },
        { name: "Create Agent Email", value: "agentEmail" },
        { name: "Create Note", value: "note" },
        { name: "Create Ticket/Ask A Question", value: "ticket" },
        { name: "Find/Create Contact", value: "contactFindCreate" },
        { name: "Customer Info Set Details", value: "customerInfoDetails" },
        { name: "Interaction Set Properties", value: "interactionHistoryProperties" },
        { name: "Logout", value: "logout" },
        { name: "Message Queue Get Approve", value: "messageQueueGetApprove" },
        { name: "Message Queue Get Next", value: "messageQueueGetNext" },
        { name: "Message Queue Save Resolve", value: "messageQueueSaveResolve" },
        { name: "Message Queue Remove From My Queue", value: "messageQueueUnqueue" },
        { name: "Message Queue QA Reject", value: "messageQueueQAReject" },
        { name: "Message Queue Change Topic", value: "messageQueueChangeTopic" },
        { name: "Message Queue Mark As Spam", value: "messageQueueMarkSpam" },
        { name: "Message Queue Save Draft", value: "messageQueueSaveDraft" },
        { name: "Message Queue Assign To Agent", value: "messageQueueAssign" },
        { name: "Message Queue Forward To Segment", value: "messageQueueChangeSegment" },
        { name: "Message Queue Forward To External Agent", value: "messageQueueForwardExternal" },
        { name: "Message Queue Save Note", value: "messageQueueSaveNote" },
        { name: "Message Queue Send", value: "messageQueueSend" },
        { name: "My Account Subscriptions", value: "myAccountSubscriptions" },
        { name: "My Account Details", value: "myAccountDetails" },
        { name: "Password Reset Send", value: "passwordResetSend" },
        { name: "Password Reset Confirm", value: "passwordResetConfirm" },
        { name: "Find Answer Create", value: "findAnswerCreate" },
        { name: "Find Answer Modify", value: "findAnswerModify" },
        { name: "Find Answer Subscribe", value: "findAnswerSubscribe" },
        { name: "Find Answer Feedback", value: "findAnswerFeedback" },
        { name: "Find Answer Send Notification", value: "findAnswerSendNotification" },
        { name: "Find Answer Delete", value: "findAnswerDelete" },
      ]
    }
  ];
  iservice.FiltersStepsOptions = [
    {
      label: "Filter Actions",
      options: [
        { name: "Change Topic", value: "changeTopic" },
        { name: "Change Interaction Parent", value: "interactionSetParent" },
        { name: "Filter", value: "filter" },
        { name: "Forward External", value: "forwardExternal" },
        { name: "Resolve Interaction", value: "interactionResolve" },
        { name: "Round Robin Assignment", value: "roundRobin" },
        { name: "Select Agents", value: "selectAgents" },
        { name: "Send Agent Notification", value: "sendAutoResponse" },
        { name: "Set Interaction Property", value: "interactionPropertySet" },
      ]
    }
  ];
  iservice.AlertsStepsOptions = [
    {
      label: "Alerts Actions",
      options: [
        { name: "Change Topic", value: "changeTopic" },
        { name: "Round Robin Assignment", value: "roundRobin" },
        { name: "Resolve Interaction", value: "interactionResolve" },
        { name: "Select Agents", value: "selectAgents" },
        { name: "Send Agent Notification", value: "sendAutoResponse" },
        { name: "Set Interaction Property", value: "interactionPropertySet" }
      ]
    }
  ];

  iservice.FindProp = function (segments, name) {
    for (var i = 0; i < segments.length; i++) {
      for (var j = 0; j < segments[i].properties.length; j++) {
        var prop = segments[i].properties[j];
        if (prop.name == name) return prop;
      }
    }
  }
  iservice.insertAtCaret = function (areaId, text) {
    var txtarea = document.getElementById(areaId);
    if (!txtarea) {
      return;
    }

    var scrollPos = txtarea.scrollTop;
    var strPos = 0;
    var br = ((txtarea.selectionStart || txtarea.selectionStart == '0') ?
      "ff" : (document.selection ? "ie" : false));
    if (br == "ie") {
      txtarea.focus();
      var range = document.selection.createRange();
      range.moveStart('character', -txtarea.value.length);
      strPos = range.text.length;
    } else if (br == "ff") {
      strPos = txtarea.selectionStart;
    }

    var front = (txtarea.value).substring(0, strPos);
    var back = (txtarea.value).substring(strPos, txtarea.value.length);
    txtarea.value = front + text + back;
    strPos = strPos + text.length;
    if (br == "ie") {
      txtarea.focus();
      var ieRange = document.selection.createRange();
      ieRange.moveStart('character', -txtarea.value.length);
      ieRange.moveStart('character', strPos);
      ieRange.moveEnd('character', 0);
      ieRange.select();
    } else if (br == "ff") {
      txtarea.selectionStart = strPos;
      txtarea.selectionEnd = strPos;
      txtarea.focus();
    }

    txtarea.scrollTop = scrollPos;
  }
  var names = {};
  var MakeName = function (name) {
    var base = name.replace(/\W/g, '');
    var end = 0;
    name = base;
    while (names[name]) {
      end++;
      name = base + end;
    }
    names[name] = true;
    return name;
  }
  function htmlEscape(str) {
    return String(str).replace(/&/g, '&amp;').replace(/"/g, '&quot;').replace(/'/g, '&#39;').replace(/</g, '&lt;').replace(/>/g, '&gt;').replace(/\//g, '&#x2F;');
  }

  iservice.MakePropertyRow = function (prop, FlagProperty) {
    var id = MakeName(prop.name),
      idConfirm = MakeName(id + 'confirm');
    var flagTextarea = (prop.isMultiLine) ? ' -textarea' : '';
    function FlagID(id) {
      return " -id'" + id + "'";
    };
    function FlagConfirm(id) {
      return " -confirm'" + id + "'";
    };
    var row = {
      name: htmlEscape(prop.name),
      optional: true,
      multiline: prop.isMultiLine,
      purpose: prop.purpose,
      input: '$input{req}' + FlagProperty(prop, id) + FlagID(id) + flagTextarea + '$',
      inputConfirm: '$input' + FlagProperty(prop, id) + FlagID(idConfirm) + FlagConfirm(id) + flagTextarea + '$'
    }; if (prop.hasDescriptions) {
      var idDescribe = MakeName(id + 'describe'), idDescribeConfirm = MakeName(id + 'describeconfirm'); function FlagDescribe(id) { return " -description'" + id + "'"; }; row.describe = true; row.inputDescribe = '$input' + FlagProperty(prop, id) + FlagID(idDescribe) + FlagDescribe(id) + '$'; row.inputDescribeConfirm = '$input' + FlagProperty(prop, id) + FlagID(idDescribeConfirm) + FlagConfirm(idDescribe) + FlagDescribe(idConfirm) + '$';
    } return row;
  }

  iservice.tenant = {};
  iservice.accessRights = {};
  iservice.loggedIn = {};

  function GetSegment(id) {
    return AddToContainer(iservice.tenant, { id: id }, 'segments', 'id');
  }
  iservice.tenant.AddSegment = function (seg) {
    AddObject(seg, 'segments', 'id');
  }
  iservice.tenant.AddMailbox = function (mailbox) {
    AddObjectSegment(mailbox, 'mailboxes', 'id', 'segmentID', 'segmentName');
    mailbox.nameWithSeg = '(' + mailbox.segmentName + ') ' + mailbox.name;
  }
  iservice.tenant.AddTopic = function (topic) {
    AddObjectSegment(topic, 'topics', 'id', 'segmentID', 'segmentName');
  }
  iservice.tenant.AddAutoResponse = function (autoResponse) {
    AddObjectSegment(autoResponse, 'autoResponses', 'id', 'segmentID', 'segmentName');
    autoResponse.nameWithSeg = '(' + autoResponse.segmentName + ') ' + autoResponse.name;
  }
  iservice.tenant.AddExternalNotify = function (externalNotify) {
    AddObjectSegment(externalNotify, 'externalNotify', 'id', 'segmentID', 'segmentName');
    externalNotify.nameWithSeg = '(' + externalNotify.segmentName + ') ' + externalNotify.name;
  }
  iservice.tenant.AddContactProperty = function (property) {
    AddObjectSegment(property, 'contactProperties', 'id', 'segmentID', 'segmentName');
    property.nameWithSeg = '(' + property.segmentName + ') ' + property.name;
  }
  iservice.tenant.AddInteractionProperty = function (property) {
    AddObjectSegment(property, 'intProperties', 'id', 'segmentID', 'segmentName');
    property.nameWithSeg = '(' + property.segmentName + ') ' + property.name;
  }
  iservice.tenant.AddFilter = function (filter) {
    AddObjectSegment(filter, 'filters', 'id', 'segmentID', 'segmentName');
    filter.nameWithSeg = '(' + filter.segmentName + ') ' + filter.name;
  }
  iservice.tenant.AddMailbox = function (mailbox) {
    AddObjectSegment(mailbox, 'mailboxes', 'id', 'segmentID', 'segmentName');
    mailbox.nameWithSeg = '(' + mailbox.segmentName + ') ' + mailbox.name;
  }
  iservice.tenant.AddForm = function (form) {
    if (form) form.idNum = +form.id;
    AddObject(form, 'forms', 'id');
  }
  iservice.tenant.AddFormBuiltin = function (form) {
    AddObject(form, 'formsbuiltin', 'id');
  }
  iservice.tenant.ResetForms = function () {
    iservice.tenant.formsLoaded = false;
    iservice.tenant.forms = null;
  }
  iservice.tenant.AddDepartment = function (department) {
    AddObject(department, 'departments', 'id');
  }
  iservice.tenant.AddWebsite = function (website) {
    if (website) website.idNum = +website.id;
    AddObject(website, 'websites', 'id');
  }
  iservice.tenant.ResetWebsites = function () {
    iservice.tenant.websitesLoaded = false;
    iservice.tenant.websites = null;
  }
  iservice.tenant.AddAgent = function (agent) {
    AddObject(agent, 'agents', 'id');
  }
  function ForKeys(obj, func) {
    var names = Object.keys(obj);
    for (var i = 0; i < names.length; i++) {
      var item = obj[names[i]];
      var name = names[i];
      func(item, name);
    }
  }
  function AddToContainer(container, obj, listName, idName) {
    if (!container[listName]) container[listName] = { list: [], map: {} };
    var id = obj[idName];
    if (!id || !id.length) return;
    var existing = container[listName].map[id];
    if (!existing) {
      container[listName].list.push(obj);
      container[listName].map[id] = obj;
      return obj;
    }
    else {
      ForKeys(obj, function (item, name) {
        existing[name] = item;
      });
      return existing;
    }
  }
  function AddObject(obj, listName, idName) {
    AddToContainer(iservice.tenant, obj, listName, idName);
  }
  function AddObjectSegment(obj, listName, idName, segmentIDName) {
    AddObject(obj, listName, idName);
    var segment = GetSegment(obj[segmentIDName]);
    if (segment) {
      AddToContainer(segment, obj, listName, idName);
    }
  }
  iservice.AdminList = function (types, callback) {
    var willReturn = {};
    var typesList = '';
    for (var i = 0; i < types.length; i++) {
      var type = types[i];
      if (type == 'segments' && iservice.tenant.segmentsLoaded) { willReturn.segments = iservice.tenant.segments.list; continue; }
      if (type == 'forms' && iservice.tenant.formsLoaded) { willReturn.forms = iservice.tenant.forms.list; continue; }
      if (type == 'formsbuiltin' && iservice.tenant.formsbuiltinLoaded) { willReturn.formsbuiltin = iservice.tenant.formsbuiltin.list; continue; }
      if (type == 'websites' && iservice.tenant.websitesLoaded) { willReturn.websites = iservice.tenant.websites.list; continue; }
      if (type == 'contactProperties' && iservice.tenant.contactPropertiesLoaded) { willReturn.contactProperties = iservice.tenant.contactProperties.list; continue; }
      if (type == 'intProperties' && iservice.tenant.intPropertiesLoaded) { willReturn.intProperties = iservice.tenant.intProperties.list; continue; }
      if (type == 'topics' && iservice.tenant.topicsLoaded) { willReturn.topics = iservice.tenant.topics.list; continue; }
      if (type == 'filters' && iservice.tenant.filtersLoaded) { willReturn.filters = iservice.tenant.filters.list; continue; }
      if (type == 'departments' && iservice.tenant.departmentsLoaded) { willReturn.departments = iservice.tenant.departments.list; continue; }
      if (type == 'autoResponses' && iservice.tenant.autoResponsesLoaded) { willReturn.autoResponses = iservice.tenant.autoResponses.list; continue; }
      if (type == 'externalNotify' && iservice.tenant.externalNotifyLoaded) { willReturn.externalNotify = iservice.tenant.externalNotify.list; continue; }
      if (type == 'mailboxes' && iservice.tenant.mailboxesLoaded) { willReturn.mailboxes = iservice.tenant.mailboxes.list; continue; }
      if (type == 'agents' && iservice.tenant.agentsLoaded) { willReturn.agents = iservice.tenant.agents.list; continue; }
      typesList += ',' + type;
    }
    if (!typesList.length) { callback(willReturn); return function () { return false; } }
    typesList += ',';
    return iservice.API('webapp-api-admin?api=lists', null, { types: typesList }, null).then(function (data) {
      if (data.segments) { for (var i = 0; i < data.segments.length; i++) { iservice.tenant.segmentsLoaded = true; iservice.tenant.AddSegment(data.segments[i]); } willReturn.segments = iservice.tenant.segments ? iservice.tenant.segments.list : []; }
      if (data.forms) { for (var i = 0; i < data.forms.length; i++) { iservice.tenant.formsLoaded = true; iservice.tenant.AddForm(data.forms[i]); } willReturn.forms = iservice.tenant.forms ? iservice.tenant.forms.list : []; }
      if (data.formsbuiltin) { for (var i = 0; i < data.formsbuiltin.length; i++) { iservice.tenant.formsbuiltinLoaded = true; iservice.tenant.AddFormBuiltin(data.formsbuiltin[i]); } willReturn.formsbuiltin = iservice.tenant.formsbuiltin ? iservice.tenant.formsbuiltin.list : []; }
      if (data.websites) { for (var i = 0; i < data.websites.length; i++) { iservice.tenant.websitesLoaded = true; iservice.tenant.AddWebsite(data.websites[i]); } willReturn.websites = iservice.tenant.websites ? iservice.tenant.websites.list : []; }
      if (data.contactProperties) { for (var i = 0; i < data.contactProperties.length; i++) { iservice.tenant.contactPropertiesLoaded = true; iservice.tenant.AddContactProperty(data.contactProperties[i]); } willReturn.contactProperties = iservice.tenant.contactProperties ? iservice.tenant.contactProperties.list : []; }
      if (data.intProperties) { for (var i = 0; i < data.intProperties.length; i++) { iservice.tenant.intPropertiesLoaded = true; iservice.tenant.AddInteractionProperty(data.intProperties[i]); } willReturn.intProperties = iservice.tenant.intProperties ? iservice.tenant.intProperties.list : []; }
      if (data.topics) { for (var i = 0; i < data.topics.length; i++) { iservice.tenant.topicsLoaded = true; iservice.tenant.AddTopic(data.topics[i]); } willReturn.topics = iservice.tenant.topics ? iservice.tenant.topics.list : []; }
      if (data.filters) { for (var i = 0; i < data.filters.length; i++) { iservice.tenant.filtersLoaded = true; iservice.tenant.AddFilter(data.filters[i]); } willReturn.filters = iservice.tenant.filters ? iservice.tenant.filters.list : []; }
      if (data.departments) { for (var i = 0; i < data.departments.length; i++) { iservice.tenant.departmentsLoaded = true; iservice.tenant.AddDepartment(data.departments[i]); } willReturn.departments = iservice.tenant.departments ? iservice.tenant.departments.list : []; }
      if (data.autoResponses) { for (var i = 0; i < data.autoResponses.length; i++) { iservice.tenant.autoResponsesLoaded = true; iservice.tenant.AddAutoResponse(data.autoResponses[i]); } willReturn.autoResponses = iservice.tenant.autoResponses ? iservice.tenant.autoResponses.list : []; }
      if (data.externalNotify) { for (var i = 0; i < data.externalNotify.length; i++) { iservice.tenant.externalNotifyLoaded = true; iservice.tenant.AddExternalNotify(data.externalNotify[i]); } willReturn.externalNotify = iservice.tenant.externalNotify ? iservice.tenant.externalNotify.list : []; }
      if (data.mailboxes) { for (var i = 0; i < data.mailboxes.length; i++) { iservice.tenant.mailboxesLoaded = true; iservice.tenant.AddMailbox(data.mailboxes[i]); } willReturn.mailboxes = iservice.tenant.mailboxes ? iservice.tenant.mailboxes.list : []; }
      if (data.agents) { for (var i = 0; i < data.agents.length; i++) { iservice.tenant.agentsLoaded = true; iservice.tenant.AddAgent(data.agents[i]); } willReturn.agents = iservice.tenant.agents ? iservice.tenant.agents.list : []; }
      callback(willReturn);
    });
  };

  iservice.SortFunction = function (fields) {
    function SortCompare(left, right) {
      for (var i = 0, len = fields.length; i < len; i++) {
        var column = fields[i].column == 'status' && left.currentStatus && right.currentStatus ? 'currentStatus' : fields[i].column,
          l = left[column],
          r = right[column];
        var lStr = (typeof l == 'string' || l instanceof String),
          rStr = (typeof r == 'string' || r instanceof String);
        var lEmpty = (l === undefined || l === null || (lStr && l.length < 1));
        var rEmpty = (r === undefined || r === null || (rStr && r.length < 1));
        if (lEmpty && rEmpty) continue;
        if (lEmpty) return (fields[i].ascend) ? -1 : 1;
        if (rEmpty) return (fields[i].ascend) ? 1 : -1;
        if (lStr) { l = l.toLowerCase(); }
        if (rStr) { r = r.toLowerCase(); }
        if (lStr && rStr) {
          if (fields[i].ascend) return l.localeCompare(r, undefined, { numeric: true });
          return r.localeCompare(l, undefined, { numeric: true });
        }
        if (l < r) return (fields[i].ascend) ? -1 : 1;
        if (r < l) return (fields[i].ascend) ? 1 : -1;
      }
      return 0;
    }
    return SortCompare;
  }
  iservice.SortSimple = function (sortField, sortOrder) {
    return iservice.SortFunction([{ column: sortField, ascend: sortOrder == -1 }]);
  }

})(iservice);

var websync = {};
(function (websync) {
  websync.log = [];
  var client = null;
  var channels = {};
  function WebsyncInit() {
    if (client != null) return;
    client = new fm.websync.client(rootPath + 'websync.ashx');
    //client.setDisableWebSockets(true);
    client.connect({
      onSuccess: function (e) {
        websync.log.push('Client connect success.');
      },
      onFailure: function (e) {
        websync.log.push('Client connect failure.');
      },
      onStreamFailure: function (e) { }
    });
  };
  function UnsubImpl(channelName, channel, token) {
    return function () {
      var receive = channel.receives[token];
      if (receive) {
        delete channel.receives[token];
      }
      if (Object.keys(channel.receives).length < 1) {
        delete channels[channelName];
        WebsyncInit();
        client.unsubscribe({ channel: channelName });
      }
    }
  }
  function Receive(channel, e) {
    Object.getOwnPropertyNames(channel.receives).forEach(name => {
      var receive = channel.receives[name];
      receive.onReceive(e);
    });
  }
  websync.subscribe = function (channelName, onReceive) {
    var channel = channels[channelName];
    var firstTime = !channel;
    if (firstTime) {
      channel = { index: 1, receives: {} };
      channels[channelName] = channel;
    }
    var token = channel.index + '';
    channel.receives[token] = {
      onReceive: onReceive
    };
    channel.index++;
    if (firstTime) {
      WebsyncInit();
      client.subscribe({
        channel: channelName,
        meta: { sid: iservice.data.loggedIn.sid },
        onSuccess: function (e) {
          websync.log.push('Subscribe success: ' + channelName);
        },
        onFailure: function (e) {
          websync.log.push('Subscribe failure: ' + channelName + ': ' + e.getException().message);
        },
        onReceive: function (e) {
          Receive(channel, e);
        }
      });
    }
    return UnsubImpl(channelName, channel, token);
  };

})(websync);
