import
{
  Service,
  IPageScope
} from 'iService.common';

import { AgentOperationController } from './AgentOperation';

declare const iservice: any;

export class DirectiveIserviceAgentOperationCreateTicketController extends AgentOperationController
{
  static $inject = ['$attrs', '$scope', 'iService.api.serial', 'iService.topics', 'iService.agentStatus', 'iService.stockresponses', '$q' ];

  agentList: { id: string, email: string }[];
  us: any;
  createTicket: any;

  constructor(attrs: angular.IAttributes, scope, API, topics, agents, stockresponses, $q) {
    super(attrs);
    var us = this;

    const baseOnInit = us.$onInit;
    us.$onInit = () => {
      baseOnInit && baseOnInit.call(us);
      us.loading = true;
      $q.all([topics.promise(), agents.promise(), stockresponses.promise()]).then( () => {
        API('webapp-api-history?api=startticket', null, 'StartTicket', {
          interactionID: us.operationDetails.inResponseToID,
          topicID: us.operationDetails.topicID,
          segmentID: us.operationDetails.segmentID,
          customerID: us.operationDetails.contactID
        }).init( () => {
          us.loading = true;
        }).then( (result) => {
          us.addresses = result.addresses;
          us.operationDetails.subject = '';
          us.operationDetails.destEmail = result.addresses.length ? result.addresses[0] : '';
          us.operationDetails.bodyHtml = '';
          
          us.topicSelectedOperationHandler();
          iservice.SanitizePropertyGroups(result.interactionProperties);
          this.interactionProperties = result.interactionProperties;

          iservice.SanitizePropertyGroups(result.caseProperties);
          this.caseProperties = result.caseProperties;
        }).catch( (errors) => {
          this.localErrors = errors;
        }).finally(function () {
          us.loading = false;
        });
      });
    };
    /*watch properties to show red dot, check after each 2 second when all properties are changed*/
    scope.$watch(() => { return [this.interactionProperties, this.caseProperties] }, (newValue, oldValue) => {
      scope.proparr = [];
       scope.proparr2 = [];
      
      if (typeof this.interactionProperties != 'undefined') {
        for (let section of this.interactionProperties) {
          for (let property of section.properties) {
            if (property.isRequired && property.values[0].value == '') {
              scope.proparr.push(null);
            }
            if (property.values[0].value != '') {
              scope.propertyRequiredError = false;
              scope.proparr.push(property.values[0].value);
              scope.proparr2.push(property.values[0].value);
            }
            if (scope.proparr.includes(null)) {
              scope.propertyRequiredError = true;
            }
          }
        }
      }
      if (typeof this.caseProperties != 'undefined') {
        for (let section of this.caseProperties) {
          for (let property of section.properties) {
            if (property.isRequired && property.values[0].value == '') {
              scope.proparr.push(null);
            }
            if (property.values[0].value != '') {
              scope.propertyRequiredError = false;
              scope.proparr.push(property.values[0].value);
              scope.proparr2.push(property.values[0].value);
            }
            if (scope.proparr.includes(null)) {
              scope.propertyRequiredError = true;
            }
          }
        }
      }
    }, true);

    us.topicSelectedOperationHandler = function ()
    {
      us.interactionProperties = topics.QuestionProperties(us.operationDetails.topicID);
      us.caseProperties = topics.CaseProperties(us.operationDetails.topicID);
      us.agentList = [];
      agents.list().forEach(function (agent) {
        var haveTopic = agent.haveTopic[us.operationDetails.topicID];
        var haveSegment = agent.haveSegment[us.operationDetails.segmentID];
        var isSupervisor = iservice.HaveRight('Tab.MessageQueue.Supervise');
        if (haveTopic || (haveSegment && isSupervisor)) {
          us.agentList.push(agent);
        }
      });
      us.operationDetails.assignToAgentID = '';
      us.operationDetails.mailboxID = '';
      us.stockSections = [{ name: 'Segment Responses', responses: stockresponses.segment(us.operationDetails.segmentID) }, { name: 'Agent Responses', responses: stockresponses.agent() }];
    };

    us.createTicket = () =>
    {
      this.localErrors = null;
      this.localSuccess = null;
      if (!us.operationDetails.topicID) {
        this.localErrors = ['Please select a topic.'];
        return;
      }

      us.operationDetails.properties = iservice.CollectProperties(us.interactionProperties);
      us.operationDetails.caseProperties = iservice.CollectProperties(us.caseProperties);
      us.operationDetails.action = 5;

      var displayedBodyHtml = us.operationDetails.bodyHtml;

      API('webapp-api-custinfo?api=createticket', null, 'CreateTicket', { details: us.operationDetails }, us.uploads).init(function () {
        us.loading = true;
      }).then(( results ) => {
        
        iservice.SanitizeHistoryRow(results.row);
        if (us.caseRecord) {
          us.caseRecord.children.push(results.row);
        }
        us.onComplete();
        this.localSuccess = ['Ticket successfully created.'];
      }).catch( (errors) => {
        this.localErrors = errors;
        us.operationDetails.bodyHtml = displayedBodyHtml;
      }).finally(function () {
        us.loading = false;
      });
    };
  }
}

DirectiveIserviceAgentOperationCreateTicketController.prototype.operationTitle = 'Create Ticket';