(function ()
{
  'use strict';

  var app = angular.module('iService', ['ngRoute', 'ngAnimate', 'ngSanitize', 'ui.select', 'ui.bootstrap', 'angularMoment', 'iService.base', 'iService.ng', 'iService.directive', 'iService.common', 'dndLists']);

  function Toggler() {
    var show = true;
    return {
      Collapsed: function ()
      {
        return !show;
      },
      Toggle: function () {
        show = !show;
      },
      Class: function () {
        return show ? 'glyphicon-chevron-right__expanded' : '';
      }
    };
  }
  function IServiceEvent() {
    var ids = 0;
    var subscriptions = {};
   
    function publish() {
      var args = arguments; 
      Object.keys(subscriptions).forEach(function (i) {
        if (!subscriptions[i].apply(null, args))
         return;
      });
    }
    function subscribe(callback) {
      var subscriptionId = ids++;
      subscriptions[String(subscriptionId)] = callback;
      return function () {
        delete subscriptions[subscriptionId];
      };
    }
    return { publish: publish, subscribe: subscribe };
  }
  function LimitCharacters(value) {
    var characters = value.length;
  }
  function RouteID(idParam) {
    var id = +idParam;
    if (isNaN(id) || !Number.isInteger(id) || id < 1) {
      id = undefined;
    }
    return id;
  }
  function MessagesContainer($scope) {
    return iservice.MessagesContainer($scope);
  }
  function NewDeleteDetailer($scope, $location, rootUrl, name, listUrl, newUrl, deleteUrl, detailsUrl, itemListFunc, itemLoadFunc) {
    MessagesContainer($scope);
    var nameNew = 'new' + name;
    var nameSelected = 'selected' + name;
    $scope[nameSelected] = null;
    $scope.ItemListRowClass = function (item) {
      if (item == $scope[nameSelected]) return 'row-active';
      return '';
    }
    $scope.ItemListSelect = function (item) {
      if ($scope[nameSelected] === item) return;
      $scope.ChangeState('show', item);
    }
    $scope.ItemListUrl = function () {
      if (!itemListFunc()) return '';
      return listUrl;
    }
    $scope.ItemNewUrl = function () {
      if (!$scope[nameNew]) return '';
      return newUrl;
    }
    $scope.ItemDetailsUrl = function (item) {
      var selectedItem = $scope[nameSelected];
      if (!selectedItem) return '';
      if (item !== selectedItem) return '';
      if ($scope[nameNew]) return '';
      if (selectedItem.deleting == true) return '';
      return detailsUrl;
    }
    $scope.ItemDeleteUrl = function () {
      var selectedItem = $scope[nameSelected];
      if (!selectedItem) return '';
      if (!selectedItem.deleting) return '';
      return deleteUrl;
    }
    $scope.ShowDeleteButton = function () {
      return $scope[nameSelected] && !$scope[nameSelected].deleting;
    }
    $scope.CancelDelete = function () {
      $scope[nameSelected].deleting = false;
    }
    $scope.ChangeState = function (state, showItem) {
      $scope.messages.Clear();
      if (state == 'show' && !showItem) state = 'none';
      if (state == 'new' || state == 'newadminform' ) {
        $scope[nameNew] = ($scope.NewItem) ? $scope.NewItem() : {};
        $scope[nameSelected] = null;
        if (state != 'newadminform')
        $location.path(rootUrl, false);
      }
      else if (state == 'show') {
        $scope[nameNew] = null;
        $scope[nameSelected] = showItem;
        $scope[nameSelected].deleting = false;
        $location.path(rootUrl + '/' + showItem.id, false);
        itemLoadFunc(showItem);
      }
      else if (state == 'none') {
        $scope[nameNew] = null;
        $scope[nameSelected] = null;
        $location.path(rootUrl, false);
      }
    }
  }
  function IFrameContainer($interval, id) {
    var promise = null;
    var height = 200;
    var iframe = {};
    function getDocHeight(doc) {
      doc = doc || document;
      // stackoverflow.com/questions/1145850/
      var body = doc.body, html = doc.documentElement;
      if (!body || !html) return 10;
      var height = Math.max(body.scrollHeight, body.offsetHeight,
          html.clientHeight, html.scrollHeight, html.offsetHeight);
      return height;
    }
    iframe.height = function () {
      if (promise) return height + 'px';
      promise = $interval(function () {
        var ifrm = document.getElementById(id);
        if (!ifrm) {
          $interval.cancel(promise);
          return;
        }
        var doc = ifrm.contentDocument ? ifrm.contentDocument :
            ifrm.contentWindow.document;
        height = getDocHeight(doc);
        return height + 'px';
      }, 100);
    }
    return iframe;
  }
  function CustomPluginHelper($scope, API, websites) {
    var helper = {
      configFormName: '',
      formSaveAPI: 'forms',
      websiteSaveAPI: 'websites'
    };
    var configID, bodyID, headID;

    helper.Load = function (doneFunc) {
      websites.then(function () {
        var list = websites.list();
        var count = list.length;
        LoadWebsite(list[0], 0);
        function LoadWebsite(website, i) {
          API('webapp-api-admin?api=websitedetails', null, 'WebsiteDetails', { websiteID: website.id }).then(function (result) {
            website.details = result.details;
            i++;
            if (i < count) {
              LoadWebsite(list[i], i);
            }
            else {
              LoadWebsiteDone();
            }
          }).catch(function (errors) {
            if (doneFunc) doneFunc(errors);
          });
        }
        function LoadWebsiteDone() {
          list.forEach(function (website) {
            if (!configID) {
              helper.configID = configID = iservice.WebsiteGetPlaceholder(website.details, helper.configFormName);
              helper.websiteName = website.name;
            }
            if (!bodyID) bodyID = iservice.WebsiteGetPlaceholder(website.details, 'agentapp-plugins-body');
            if (!headID) headID = iservice.WebsiteGetPlaceholder(website.details, 'agentapp-plugins-head');
          });
          if (doneFunc) doneFunc();
        }
      });
    }
    helper.Save = function (configText, configDescription, bodyText, headText, doneFunc) {
      websites.then(function () {
        var list = websites.list();
        var count = list.length;
        SaveConfig();
        function SaveConfig() {
          var details = {
            formID: configID,
            name: helper.configFormName,
            body: configText,
            isFullPage: true,
            successRedirectUrl: '',
            description: configDescription
          };
          var verb = configID ? 'modify' : 'new';
          API('webapp-api-admin?api=' + helper.formSaveAPI + '&mode=' + verb, null, 'FormCreate', { formID: configID, details: details }).then(function (result) {
            configID = result.row.id;
            SaveBody();
          }).catch(function (errors) {
            if (doneFunc) doneFunc(errors);
          });
        }
        function SaveBody() {
          var details = {
            formID: bodyID,
            name: 'agentapp-plugins-body',
            body: bodyText,
            isFullPage: true,
            successRedirectUrl: '',
            description: 'Body html for plugins in all agent pages.'
          };
          if (!bodyText || !bodyText.length) {
            SaveHead();
          }
          else if (bodyID) {
            LoadExisting();
          }
          else {
            SaveForm();
          }
          function LoadExisting() {
            API('webapp-api-admin?api=formdetails', null, 'FormDetails', { formID: bodyID }).then(function (result) {
              details.body = result.details.body;
              if (!details.body.includes(bodyText)) {
                details.body = bodyText + '\r\n' + details.body;
              }
              SaveForm();
            }).catch(function (errors) {
              if (doneFunc) doneFunc(errors);
            });
          }
          function SaveForm() {
            var verb = bodyID ? 'modify' : 'new';
            API('webapp-api-admin?api=' + helper.formSaveAPI + '&mode=' + verb, null, 'FormCreate', { formID: bodyID, details: details }).then(function (result) {
              bodyID = result.row.id;
              SaveHead();
            }).catch(function (errors) {
              if (doneFunc) doneFunc(errors);
            });
          }
        }
        function SaveHead() {
          var details = {
            formID: headID,
            name: 'agentapp-plugins-head',
            body: headText,
            isFullPage: true,
            successRedirectUrl: '',
            description: 'Head html for plugins in all agent pages.'
          };
          if (!headText || !headText.length) {
            SaveWebsite(list[0], 0);
          }
          else if (headID) {
            LoadExisting();
          }
          else {
            SaveForm();
          }
          function LoadExisting() {
            API('webapp-api-admin?api=formdetails', null, 'FormDetails', { formID: headID }).then(function (result) {
              details.body = result.details.body;
              if (!details.body.includes(headText)) {
                details.body = headText + '\r\n' + details.body;
              }
              SaveForm();
            }).catch(function (errors) {
              if (doneFunc) doneFunc(errors);
            });
          }
          function SaveForm() {
            var verb = headID ? 'modify' : 'new';
            API('webapp-api-admin?api=' + helper.formSaveAPI + '&mode=' + verb, null, 'FormCreate', { formID: headID, details: details }).then(function (result) {
              headID = result.row.id;
              SaveWebsite(list[0], 0);
            }).catch(function (errors) {
              if (doneFunc) doneFunc(errors);
            });
          }
        }
        function SetPlaceholder(details, name, formID) {
          var placeholder = FindArray(details.placeholders, function (placeholder) {
            if (placeholder.name == name) return placeholder;
          });
          if (placeholder) {
            placeholder.value = formID;
          }
          else {
            details.placeholders.push({ name: name, value: formID });
          }
        }
        function SaveWebsite(website, i) {
          SetPlaceholder(website.details, helper.configFormName, configID);
          if (bodyID) SetPlaceholder(website.details, 'agentapp-plugins-body', bodyID);
          if (headID) SetPlaceholder(website.details, 'agentapp-plugins-head', headID);
          API('webapp-api-admin?api=' + helper.websiteSaveAPI + '&mode=modify', null, 'WebsiteModify', { websiteID: website.id, details: website.details }).then(function (result) {
            i++;
            if (i < count) {
              SaveWebsite(list[i], i);
            }
            else {
              if (doneFunc) doneFunc();
            }
          }).catch(function (errors) {
            if (doneFunc) doneFunc(errors);
          });
        }
      });
    }
    return helper;
  }

  app.controller('ControllerMain', ['$scope', ControllerMain]);
  function ControllerMain($scope) {
    $scope.activePage = 'admin';
    $scope.title = 'Admin Tools';
    iservice.pageName = 'admin';
    $scope.SetTitle = function (value) {
      $scope.title = value;
    }
  }

  app.controller('ControllerCreditMemoAdminMain', ['$scope', ControllerCreditMemoAdminMain]);
  function ControllerCreditMemoAdminMain($scope) {
    $scope.SetTitle('Credit Memo Admin');
  }
  app.controller('ControllerCreditMemoAdminForm', ['$scope', '$rootScope', 'iService.api.serial', '$http', 'iService.topics', 'iService.intProperties', 'iService.agents', 'iService.stockresponses', 'iService.segments', 'iService.mailboxes', 'plugins.creditmemo', 'iService.plugins', 'iService.websites', ControllerCreditMemoAdminForm]);
  function ControllerCreditMemoAdminForm($scope, $rootScope, API, http, topics, intProperties, agents, stockresponses, segments, mailboxes, creditmemo, plugins, websites) {
    $scope.config = creditmemo;
    $scope.admin = {
      loading: true
    };

    topics.then(function () {
      intProperties.then(function () {
        agents.then(function () {
          stockresponses.then(function () {
            segments.then(function () {
              mailboxes.then(function () {
                websites.then(function () {
                  $scope.admin.loading = false;
                });
              });
            });
          });
        });
      });
    });

    var pluginHelper = CustomPluginHelper($scope, API, websites);
    pluginHelper.configFormName = 'credit-memo-config';
    pluginHelper.Load(function (errors) {
      $scope.configID = pluginHelper.configID;
      $scope.websiteName = pluginHelper.websiteName;
    });

    $scope.SaveClick = function () {
      if ($scope.admin.loading) return;
      $scope.admin.loading = true;
      topics.then(function () {
        var error = creditmemo.CheckConfig();
        if (error !== false) {
          $scope.saveErrors = [error];
        }
        var configText = CreateConfig();
        var bodyText = "$include -placeholder'credit-memo-body'$";
        var headText = "$include -placeholder'credit-memo-head'$";
        pluginHelper.Save(configText, 'Configuration for credit memo workflow.', bodyText, headText, function (errors) {
          if (errors) {
            $scope.saveErrors = errors;
          }
          else {
            $scope.saveSuccess = 'Configuration saved.';
          }
          $scope.admin.loading = false;
        });
      });
    }
    var agentMap = {};
    function CreateConfig() {
      [$scope.config.agentListOpsUnder5k, $scope.config.agentListOpsUnder25k, $scope.config.agentListOpsOver25k, $scope.config.agentListFinance, $scope.config.agentListFinanceManager].forEach(function (agentIDs) {
        RemoveFromArrayInPlace(agentIDs, function (id) {
          return !agentMap[id];
        });
      });
      var topic = topics.get($scope.config.topicIDPreApproval);
      if (topic) {
        var segmentID = topic.segmentID;
          
        $scope.config.segmentID = segmentID;
      }
      if ($scope.config.agentMap) delete $scope.config.agentMap;
      var configJson = angular.toJson($scope.config);
      return 'var userCreditMemoConfig = ' + configJson + ';';
    }
    intProperties.then(function (data) {
      $scope.properties = [];
      for (var i = 0; i < intProperties.picker().length; i++) {
        if (intProperties.picker()[i].id !== '') {
          $scope.properties.push(intProperties.picker()[i]);
        }
      }
    });
    stockresponses.then(function (data) {
      $scope.stockresponses = [];
      for (var i = 0; i < stockresponses.picker().length; i++) {
        if (stockresponses.picker()[i].id !== '') {
          $scope.stockresponses.push(stockresponses.picker()[i]);
        }
      }
    });
    agents.then(function (data) {
      $scope.agents = [];
      for (var i = 0; i < agents.picker().length; i++) {
        if (agents.picker()[i].id !== '') {
          $scope.agents.push(agents.picker()[i]);
        }
      }
      agentMap = {};
      $scope.agents.forEach(function (agent) {
        agentMap[agent.id] = agent;
      });
    });
    topics.then(function (data) {
      $scope.topicsList = [];
      for (var i = 0; i < topics.picker().length; i++) {
        if (topics.picker()[i].id !== '') {
          $scope.topicsList.push(topics.picker()[i]);
        }
      }
    });
    mailboxes.then(function (data) {
      $scope.mailboxesList = [];
      for (var i = 0; i < mailboxes.picker().length; i++) {
        if (mailboxes.picker()[i].id !== '') {
          $scope.mailboxesList.push(mailboxes.picker()[i]);
        }
      }

    });
    segments.then(function (data) {
      $scope.segmentsList = [];
      for (var i = 0; i < segments.picker().length; i++) {
        if (segments.picker()[i].id !== '') {
          $scope.segmentsList.push(segments.picker()[i]);
        }
      }

    });
  }

  function guidGenerator() {
    var S4 = function () {
      return (((1 + Math.random()) * 0x10000) | 0).toString(16).substring(1);
    };
    return (S4() + S4());
  }
  app.controller('ControllerQuoteAdminMain', ['$scope', ControllerQuoteAdminMain]);
  function ControllerQuoteAdminMain($scope) {
    //$scope.SetTitle('Quote Admin');
  }
  app.controller('ControllerQuoteAdminMainForm', ['$scope', '$route', 'iService.api.serial', 'iService.topics', 'iService.stockresponses', 'iService.segments', 'plugins.quoteconfig', 'iService.websites', '$location', '$timeout', 'iService.plugins', ControllerQuoteAdminMainForm]);
  function ControllerQuoteAdminMainForm($scope, $route, API, topics, stockresponses, segments, quoteconfig, websites, $location, $timeout, plugins) {
    $scope.config = quoteconfig;
    $scope.admin = {
      loading: true
    };
    $scope.models = {
      //selected: null,
      propLists: {
        "lists": []
      }
    }
    
    $scope.NewItem = function () {
      $scope.models = {
        propLists: {
          "lists": []
        }
      }
      return {
        stockResponseIDs: [{}],
        quoteDetails: null
      }
    }
    
    
    function getLists(configuredData) {
      var lists = [];
      configuredData.forEach(function (item) {
        var stocks = [];
        var toQuotetopic = topics.get(item.requestTopicID);
        var openQuoteTopic = topics.get(item.pendingTopicID);
        var stockids = item.stockResponseIDs;
        stockids.forEach(function (stock) {
          var stockData = stockresponses.get(stock);
          if (!stockData) return;
          stocks.push(stockData.name);
        });
        lists.push({ id: item.id, 'segment': toQuotetopic.segmentName, 'quotetopic': toQuotetopic.name, 'followuptopic': openQuoteTopic.name, 'stockresponses': stocks.join(', ') });
      });
      return lists;
    }
    topics.then(function () {
      stockresponses.then(function () {
        segments.then(function () {
          websites.then(function () {
            $scope.admin.loading = false;
            $scope.topicList = topics.list();
            $scope.Items = getLists($scope.config.topics);

            NewDeleteDetailer($scope, $location, '/quote-admin', 'QuoteAdmin', 'quote-board-admin.html', 'admin-quote-new.html', 'admin-quote-delete.html', 'admin-quote-details.html', function () { return $scope.Items; }, LoadDetails);

            var columns = [
              { name: 'Segment', var: 'segment', split: true },
              { name: 'Quote Topic', var: 'quotetopic', split: true },
              { name: 'Follow Up Topic', var: 'followuptopic', split: true },
              { name: 'Stock Responses', var: 'stockresponses', split: true }];

            var QuoteAdminView = iservice.TableViewer($scope, $timeout,
              { itemsFunc: function () { return $scope.Items; }, watchFuncs: [function () { return $scope.Items; }], columns: columns, startSort: 'segment', exportName: 'Quote Admin', pageSize: 5 });
            $scope.QuoteAdminView = QuoteAdminView;

            var initialID = $route.current.params.quoteID;
            if (initialID) $scope.Items.some(function (quote) {
              if (quote.id != initialID) return false;
              $scope.ChangeState('show', quote);
              return true;
            });
          });
        });
      });
    });

    function MakeTopic(id, requestTopicID, pendingTopicID, stockResponseIDs) {
      return {
        id: id,
        requestTopicID: requestTopicID,
        pendingTopicID: pendingTopicID,
        stockResponseIDs: TransformArray(stockResponseIDs, function (id) {
          return { id: id };
        })
      };
    }
    function LoadDetails(quote) {
      $scope.config.topics.forEach(function (quoteItem) {
        if (quote.id == quoteItem.id) {
          $scope.quoteDetails = MakeTopic(quoteItem.id, quoteItem.requestTopicID, quoteItem.pendingTopicID, quoteItem.stockResponseIDs);
          $scope.models.propLists.lists = quoteItem.propertiesOrder || [];
          $scope.responses = [];
          $scope.followUptopics = [];
          topics.then(function () {

            var topic = topics.get(quoteItem.requestTopicID);
            $scope.selectedTopicName = topic.name;

            topics.list().forEach(function (topicItem) {
              if (topicItem.segmentID == topic.segmentID) {
                $scope.followUptopics.push(topicItem);
              }
            });
            stockresponses.then(function () {
              stockresponses.list().forEach(function (stock) {
                if (stock.segmentID == topic.segmentID) {
                  $scope.responses.push(stock);
                }
              });
            });
          });
          
        }
      });
    }

    $scope.websites = [];
    $scope.forms = [];
    var pluginHelper = CustomPluginHelper($scope, API, websites);
    pluginHelper.configFormName = 'quote-board-config';
    pluginHelper.Load(function (errors) {
      $scope.configID = pluginHelper.configID;
      $scope.websiteName = pluginHelper.websiteName;
    });
    $scope.SaveClick = function (action, details) {
      if ($scope.admin.loading) return;
      $scope.admin.loading = true;
      $scope.messages.Clear();
      topics.then(function () {
        var configText = CreateConfig(details);
        var bodyText = "$" + "include -placeholder'quote-board-body'$";
        var headText = "$" + "include -placeholder'quote-board-head'$";
        if (configText) {
          pluginHelper.Save(configText, 'Configuration for quote process workflow.', bodyText, headText, function (errors) {
            if (errors) {
              $scope.messages.saveErrors = errors;
            }
            else {
              $scope.Items = getLists(quoteconfig.topics);
              if (action == 'new' || action == 'update') {
                var detail = details;
                $scope.Items.forEach(function (row) {
                  if (row.id == details.id) {
                    detail = row;
                  }
                });

                $scope.ChangeState('show', detail);
                if (action == 'new') {
                  $scope.messages.toggleSuccess = 'Configuration added successfully.';
                }
                if (action == 'update') {
                  $scope.messages.toggleSuccess = 'Configuration saved.';
                }
              } else {
                $scope.ChangeState('none');
                $scope.messages.deleteSuccess = 'Configuration deleted successfully.';
              }
              
            }
            $scope.admin.loading = false;
          });
        } else { $scope.admin.loading = false; }
        
      });
    }
    $scope.DeleteStart = function () {
      $scope.messages.Clear();
      $scope.selectedQuoteAdmin.deleting = true;
    }
    function TopicExists(data, compareTo) {
      var error = false;
      for (var i = 0; i < data.length; i++) {
        if (data[i].requestTopicID == compareTo.requestTopicID) {
          error = true;
          break;
        }
      }
      return error;
    }
    $scope.ConfiguredTopic = function (topicID) {
      var error = false;
      for (var i = 0; i < quoteconfig.topics.length; i++) {
        if (quoteconfig.topics[i].requestTopicID == topicID) {
          error = true;
          break;
        }
      }
      return error;
    }
    function CreateConfig(details) {
      var quoteData;
      var errors = false;

      $scope.models.propLists.lists.forEach(function (item, key) {
        item.order = key + 1;
      });

      //add new
      if ($scope.newQuoteAdmin) {
        
        quoteData = {
          id: guidGenerator(),
          requestTopicID: $scope.newQuoteAdmin.requestTopicID,
          pendingTopicID: $scope.newQuoteAdmin.pendingTopicID,
          stockResponseIDs: TransformArray($scope.newQuoteAdmin.stockResponseIDs, function (stock) {
            return stock.id;
          }),
          propertiesOrder: $scope.models.propLists.lists
        };
        if (details) {
          details.id = quoteData.id;
        }
        
        
        if (!TopicExists(quoteconfig.topics, $scope.newQuoteAdmin)) { //add if topic not exists
          quoteconfig.topics.push(quoteData);
        } else {
          errors = true;
        }
      } else {
        //delete existed
        if ($scope.selectedQuoteAdmin.deleting) {
          for (var i = 0; i < quoteconfig.topics.length; i++) {
            if (quoteconfig.topics[i].id == $scope.quoteDetails.id) {
              quoteconfig.topics.splice(i, 1);
              break;
            }
          }
        } else { //update existed
          quoteData = {
            id: $scope.quoteDetails.id,
            requestTopicID: $scope.quoteDetails.requestTopicID,
            pendingTopicID: $scope.quoteDetails.pendingTopicID,
            stockResponseIDs: TransformArray($scope.quoteDetails.stockResponseIDs, function (stock) {
              return stock.id;
            }),
            propertiesOrder: $scope.models.propLists.lists
          };
          quoteconfig.topics.forEach(function (item) {
            if (item.id == $scope.quoteDetails.id) {
              item.requestTopicID = quoteData.requestTopicID;
              item.pendingTopicID = quoteData.pendingTopicID;
              item.stockResponseIDs = quoteData.stockResponseIDs;
              item.propertiesOrder = quoteData.propertiesOrder;
            }
          });
        }
      }
      if (errors == true) {
        $scope.messages.saveErrors = ['"To Quote" topic already exists. Please choose another.'];
        return false;
      }
      if (errors == false) {
        //$scope.Items = getLists(quoteconfig.topics);
        var configJson = angular.toJson(quoteconfig);
        return 'var userConfig = ' + configJson + ';';
      }
    }

    $scope.modelAsJson = '';
    // Model to JSON for demo purpose
    $scope.$watch('models', function (newValue, oldValue) {
      if (newValue != oldValue) {
        $scope.modelAsJson = angular.toJson(newValue, true);
      }
    }, true);
    
    
    $scope.SelectRequestTopic = function (selectedTopic, details) {
      details = MakeTopic(details.id, '', '', []);
      $scope.responses = [];
      $scope.followUptopics = [];
      $scope.models.propLists.lists = [];
      topics.then(function () {
        var topic = topics.get(selectedTopic);
        $scope.selectedTopicName = topic.name;
        if (topic.caseProperties.length) {
          topic.caseProperties[0].properties.forEach(function (item) {
            var valueType = 'Text';
            if (item.isNumber) { valueType = 'Number' };
            if (item.isInteger) { valueType = 'Integer' };
            if (item.isDate) { valueType = 'Date' };
            if (item.isDatetime) { valueType = 'DateTime' };
            $scope.models.propLists.lists.push({ id: item.propertyID, name: item.name, propertyType: "Case", valueType: valueType });
          });
        }
        topics.list().forEach(function (topicItem) {
          if (topicItem.segmentID == topic.segmentID) {
            $scope.followUptopics.push(topicItem);
          }
        });
        stockresponses.then(function () {
          stockresponses.list().forEach(function (stock) {
            if (stock.segmentID == topic.segmentID) {
              $scope.responses.push(stock);
            }
          });
        });
      });
    }
    
    $scope.AddStock = function (details) {
      details.stockResponseIDs.push([{}]);
    }
    $scope.RemoveStock = function (details, index) {
      details.stockResponseIDs.splice(index, 1);
      if (!details.stockResponseIDs.length) {
        details.stockResponseIDs.push([{}]);
      }
    }
  }

  app.controller('ControllerManageAgentEmailFormMain', ['$scope', ControllerManageAgentEmailFormMain]);
  function ControllerManageAgentEmailFormMain($scope) {
    $scope.SetTitle('Manage Agent Email Form');
  }
  app.controller('ControllerManageAgentEmailForm', ['$scope', '$rootScope', 'iService.api.serial', '$http', 'iService.topics', 'iService.segments', 'iService.mailboxes', 'iService.websites', ControllerManageAgentEmailForm]);
  function ControllerManageAgentEmailForm($scope, $rootScope, API, http, topics, segments, mailboxes, websites) {
    $scope.segments = segments;
    $scope.topics = topics;
    $scope.mailboxes = mailboxes;
    $scope.admin = {
      segmentID: '',
      loading: true
    };

    $scope.agentEmailConfig = { };
    var emptySegment = {
      mailingLists: [],
      mailboxMap: {},
      expectingResponse: false
    };
    $scope.mailboxMaplength = function (obj) {
      var keys = Object.keys(obj);
      return keys.length;
    }

    if (window.agentEmailConfig) {
      $scope.configuredSegments = window.agentEmailConfig;
      segments.then(function () {
        segments.list().forEach(function (segment) {
          if (window.agentEmailConfig[segment.id]) {
            var segment = window.agentEmailConfig[segment.id];
            $scope.agentEmailConfig[segment.id] = segment;
            if (!$scope.admin.segmentID) {
              $scope.currentSegment = segment;
              $scope.admin.segmentID = segment.id;
            }
          }
        });
      });
    }
    topics.then(angular.noop);
    mailboxes.then(angular.noop);

    topics.then(function () {
      segments.then(function () {
        mailboxes.then(function () {
          websites.then(function () {
            $scope.admin.loading = false;
          });
        });
      });
    });

    var pluginHelper = CustomPluginHelper($scope, API, websites);
    pluginHelper.configFormName = 'agent-email-config';
    pluginHelper.formSaveAPI = 'agentemailform';
    pluginHelper.websiteSaveAPI = 'agentemailwebsite';
    pluginHelper.Load(function (errors) {

    });

    $scope.SelectSegment = function () {
      var segmentID = $scope.admin.segmentID;
      var segment = $scope.agentEmailConfig[segmentID];
      if (!segment) {
        segment = angular.copy(emptySegment);
        $scope.agentEmailConfig[segmentID] = segment;
      }
      segment.id = segmentID;
      $scope.currentSegment = segment;
    }

    $scope.Save = function (isValid) {
      if ($scope.admin.loading || !isValid) return;
      $scope.admin.loading = true;
      topics.then(function () {
        var configText = CreateConfig();
        var bodyText = '<script type="text/javascript" src="$value -rootpath$f/agent-email-config"></script>';
        pluginHelper.Save(configText, 'Configuration for quick agent email.', bodyText, null, function (errors) {
          if (errors) {
            $scope.saveErrors = errors;
          }
          else {
            $scope.saveSuccess = 'Configuration saved.';
          }
          $scope.admin.loading = false;
        });
      });
    }
    function CreateConfig() {
      var configJson = angular.toJson($scope.agentEmailConfig);
      return "var agentEmailConfig = " + configJson + ';';
    }
    $scope.ClearTopic = function (segment, topicID) {
      delete segment.mailboxMap[topicID];
    }
    $scope.AddList = function (segment) {
      segment.mailingLists.push({});
    }
    $scope.DeleteList = function (segment, $index) {
      segment.mailingLists.splice($index, 1);
    }
  }

 

  app.controller('ControllerAdmin', ['$scope', ControllerAdmin]);
  function ControllerAdmin($scope) {

  }

  app.controller('ControllerAdminWebsitesMain', ['$scope', ControllerAdminWebsitesMain]);
  function ControllerAdminWebsitesMain($scope) {
    $scope.SetTitle('Websites');
  }
  app.controller('ControllerAdminWebsites', ['$scope', '$route', '$location', '$timeout', 'iService.api.serial', 'iService.websites', 'iService.segments', 'iService.mailboxes', 'iService.autoResponses','iService.intProperties', '$q', ControllerAdminWebsites]);
  function ControllerAdminWebsites($scope, $route, $location, $timeout, API, websites, segments, mailboxes, autoResponses, intProperties, $q) {

    $q.all([segments.promise(), mailboxes.promise(), autoResponses.promise(), intProperties.promise()]).then(function () {
      $scope.websites = websites;
      $scope.segments = segments;
      $scope.mailboxes = mailboxes.list();
      $scope.itemsToggler = Toggler();
      NewDeleteDetailer($scope, $location, '/website', 'Website', 'admin-website-list.html', 'admin-website-new.html', 'admin-website-delete.html', 'admin-website-details.html', websites.list, LoadDetails);

      $scope.Items = function () { return websites.list(); }
      var columns = [
        { name: 'ID', var: 'id', visible: false },
        { name: 'Name', var: 'name', split: true },
        { name: 'Segments', var: 'segmentnames', split: true },
        { name: 'Domains', var: 'domainsnames', split: true }];
      var websitesView = iservice.TableViewer($scope, $timeout,
        { itemsFunc: $scope.Items, watchFuncs: [websites.Watch], columns: columns, startSort: 'name', exportName: 'Websites', pageSize: 5 });
      $scope.websitesView = websitesView;

      var initialID = +$route.current.params.websiteID;
      if (isNaN(initialID) || !Number.isInteger(initialID) || initialID < 1) {
        initialID = undefined;
      }

      websites.then(function (data) {
        if (initialID) websites.list().some(function (website) {
          if (website.id != initialID) return false;
          $scope.ChangeState('show', website);
          return true;
        });
      }).catch(angular.noop);

      $scope.Save = function (idPrefix, website) {
        if (validateProtocol(website.details.rootUrl)) {
          API('webapp-api-admin?api=websites&mode=modify', idPrefix, 'WebsiteModify', { websiteID: website.id, details: website.details }).init(function () {
            $scope.messages.Clear();
          }).then(function (result) {
            websitesView.BreakWords(result.row);
            //modify the row in the list.
            angular.extend(website, result.row);
            //save the details
            angular.extend(website.details, result.details);
            $scope.messages.saveSuccess = 'Website updated successfully.';
          }).catch(function (errors) {
            $scope.messages.saveErrors = errors;
          });
        } else {
          $scope.messages.saveErrors = ['Website Root URL is not a valid URL.'];
        }
      }
      $scope.updatePlaceholder = function (idPrefix, website, placeholder) {
        var hasBeenReadExist, donotreplyExist, deletedExist, archivedExist;
        var error = false;
        $scope.requiredPropertySetup = false;
        if ((placeholder.name == 'myqueue-answer-actions') && (placeholder.value == 'ngappbuiltin-myqueue-answer-actions-secure-message-portal' || placeholder.value == 'ngappbuiltin-myqueue-answer-actions-secure-message-portal-notification')) {
            hasBeenReadExist = intProperties.getByName('HasBeenRead') ? true : false;
            donotreplyExist = intProperties.getByName('DoNotReply') ? true : false;
            deletedExist = intProperties.getByName('Deleted') ? true : false;
            archivedExist = intProperties.getByName('Archived') ? true : false;
          if (!hasBeenReadExist || !donotreplyExist || !deletedExist || !archivedExist) {
            $scope.requiredPropertySetup = true;
            error = true;
            return false;
          }
        }

        if (!error) {
          API('webapp-api-admin?api=websites&mode=modify', idPrefix, 'WebsiteModify', { websiteID: website.id, details: website.details }).init(function () {
            $scope.messages.Clear();
          }).then(function (result) {
            websitesView.BreakWords(result.row);
            //modify the row in the list.
            angular.extend(website, result.row);
            //save the details
            angular.extend(website.details, result.details);
            placeholder.savedSuccess = true;
            placeholder.edit = false;
            $timeout(function () {
              placeholder.savedSuccess = false;
            }, 1500);
          }).catch(function (errors) {
            $scope.messages.saveErrors = errors;
          });
        }
      }
      $scope.computePasswordNotificationTemplateList = function (details) {
        $timeout(function () {
          $scope.autoResponses = [];
          var allowedSegments = {};
          var mailbox = mailboxes.get(details.passwordResetMailboxID);
          if (mailbox) {
            allowedSegments[mailbox.segmentID] = true;
          }
          else {
            details.segmentIDs.forEach(function (id) {
              allowedSegments[id] = true;
            });
          }
          autoResponses.list().forEach(function (response) {
            if (response.lists.indexOf('passwordresetnotification') == -1) return;
            if (allowedSegments[response.segmentID] != true) return;
            $scope.autoResponses.push(response);
          });
        });
      }
      $scope.Create = function (idPrefix) {
        if (validateProtocol($scope.newWebsite.rootUrl)) {
          API('webapp-api-admin?api=websites&mode=new', idPrefix, 'WebsiteCreate', { details: $scope.newWebsite }).init(function () {
            $scope.messages.Clear();
          }).then(function (result) {
            //add new row to the list.
            websites.add(result.row);
            $scope.ChangeState('show', result.row);
            $scope.messages.createSuccess = 'Website added successfully.';
          }).catch(function (errors) {
            $scope.messages.createErrors = errors;
          });
        } else {
          $scope.messages.createErrors = ['Website Root URL is not a valid URL.'];
        }
      }
      $scope.DeleteStart = function () {
        $scope.messages.Clear();
        $scope.selectedWebsite.deleting = true;
      }
      $scope.DeleteConfirm = function (idPrefix) {
        API('webapp-api-admin?api=websites&mode=delete', idPrefix, 'WebsiteDelete', { websiteID: $scope.selectedWebsite.id }).init(function () {
          $scope.messages.Clear();
        }).then(function (result) {
          //remove row from list
          websites.remove(result.row);
          $scope.ChangeState('none');
          $scope.messages.deleteSuccess = 'Website deleted successfully.';
        }).catch(function (errors) {
          $scope.messages.deleteErrors = errors;
        });
      }
      $scope.placeholderEdit = function (placeholder) {
        if (placeholder.edit) {
          placeholder.edit = false;
          if (!placeholder.value) {
            placeholder.value = placeholder.defaultValue;
          }
        } else {
          placeholder.edit = true;
        }
      }
      $scope.selectedForm = null;
      $scope.getFormList = function (value, placeholder) {
        if (value !== 'empty') {
          if (placeholder.name != 'stockresponse-list' && placeholder.name != 'myqueue-answer-actions' && placeholder.name != 'common-javascript') {
            placeholder.value = value;
          } else {
            if (value)
              placeholder.value = value;
          }
        } else {
          placeholder.value = "";
        }
      }
      function filterbuilin(placeholder) {
        return placeholder.defaultValue === null ? false : true;
      }
      function LoadDetails(website) {
        website.details = { id: website.id }
        API('webapp-api-admin?api=websitedetails', null, 'WebsiteDetails', { websiteID: website.id }).init(function () {
          $scope.loading = true;
        }).then(function (result) {
          $scope.computePasswordNotificationTemplateList(result.details);
          //save the details
          angular.extend(website.details, result.details);
          var columns = [
            { name: 'Placeholder Name', var: 'name', split: true },
            { name: 'Form ID/Builtin Name', var: 'value', split: true },
          ];
          var filteredPlaceholders = [];
          result.details.placeholders.forEach(function (placeholder) {
            if (filterbuilin(placeholder)) {
              filteredPlaceholders.push(placeholder);
            }
          });

          var websitesBuiltinView = iservice.TableViewer($scope, $timeout,
            { itemsFunc: function () { return filteredPlaceholders; }, watchFuncs: [result.details.Watch], columns: columns, startSort: 'name', exportName: 'Builtin Placeholders', pageSize: 5 });
          $scope.websitesBuiltinView = websitesBuiltinView;

        }).catch(function (errors) {
          //couldn't load
          $scope.ChangeState('none');
        }).finally(function () {
          $scope.loading = false;
        });
      }
    })
    
  }
  app.controller('ControllerAdminWebsitesDetails', ['$scope', '$timeout', ControllerAdminWebsitesDetails]);
  function ControllerAdminWebsitesDetails($scope, $timeout) {
    var details = $scope.details;
    if (!details.domains) details.domains = [];
    if (!details.segmentIDs) details.segmentIDs = [];
    details.domainToAdd = '';

    var wizards = {
      'common-javascript': { instruction: 'Pick version of javascript to use:', list: [{ name: 'Debug', value: 'ngappbuiltin-common-javascript-dev' }, { name: 'Release (minimized)', value: 'ngappbuiltin-common-javascript' }] },
      'myqueue-answer-actions': { instruction: 'Pick version of myqueue actions to use:', list: [{ name: 'Full (with Send)', value: 'ngappbuiltin-myqueue-answer-actions' }, { name: 'Portal (No Email)', value: 'ngappbuiltin-myqueue-answer-actions-portal' }, { name: 'Secure Messaging Portal', value: 'ngappbuiltin-myqueue-answer-actions-secure-message-portal' }, { name: 'Secure Notification Portal', value: 'ngappbuiltin-myqueue-answer-actions-secure-message-portal-notification' }] },

      'stockresponse-list': { instruction: 'Pick version of stockresponse list to use:', list: [{ name: 'Load Full Body', value: 'stockresponse-list-full' }, { name: 'Load Name Only', value: 'stockresponse-list-names' }] },

      'dashboard-body1': {
        instruction: 'Pick a widget', list: [
          { name: 'My Interaction Form', value: 'my-interaction-form' },
          { name: 'My Answers Form', value: 'my-answers-form' },
          { name: 'Support Form', value: 'support-form' }]
      },

      'dashboard-body2': {
        instruction: 'Pick a widget', list: [
          { name: 'My Interaction Form', value: 'my-interaction-form' },
          { name: 'My Answers Form', value: 'my-answers-form' },
          { name: 'Support Form', value: 'support-form' }]
      },
      'dashboard-body3': {
        instruction: 'Pick a widget', list: [
          { name: 'My Interaction Form', value: 'my-interaction-form' },
          { name: 'My Answers Form', value: 'my-answers-form' },
          { name: 'Support Form', value: 'support-form' }]
      },

      'dashboard-body4': {
        instruction: 'Pick a widget', list: [
        { name: 'My Interaction Form', value: 'my-interaction-form' },
        { name: 'My Answers Form', value: 'my-answers-form' },
        { name: 'Support Form', value: 'support-form' }]
      }
    };
    
    $scope.DomainAdd = function (domainToAdd) {
      details.domains.push(domainToAdd);
      details.domainToAdd = '';
    }
    $scope.DomainRemove = function (domain) {
      details.domains.remove(domain);
    }
    $scope.AddPlaceholder = function () {
      $scope.selectedWebsite.details.placeholders.unshift({ name: "", value: "", defaultValue: null });
    }
    $scope.DeletePlateholderConfirm = function (placeholder) {
      $scope.selectedWebsite.details.placeholders.remove(placeholder);
    }
    $scope.TogglePlaceholderWizard = function (placeholder) {
      placeholder.visible = !placeholder.visible;
    }
    $scope.PlaceholderWizard = function (placeholder) {
      var wiz = wizards[placeholder.name];
      if (!wiz) return null;
      if (wiz.list) return 'admin-website-details-placeholder-builtin-choices.html';
      return null;
    }
    $scope.PlaceholderChoices = function (placeholder) {
      var wiz = wizards[placeholder.name];
      if (!wiz) return null;
      return wiz.list;
    }
    $scope.FilterCustom = function (placeholder) {
      return placeholder.defaultValue === null ? true : false;
    }
    $scope.FilterBuiltin = function (placeholder) {
      return placeholder.defaultValue === null ? false : true;
    }
    $scope.placeholderGroup = 'custom';
    $scope.PlaceholderShow = function (group) {
      $scope.placeholderGroup = group;
    }
    $scope.PlaceholderCustomClass = function () {
      return $scope.placeholderGroup == 'custom' ? 'active btn-primary' : 'btn-default';
    }
    $scope.PlaceholderBuiltinClass = function () {
      return $scope.placeholderGroup == 'builtin' ? 'active btn-primary' : 'btn-default';
    }
    $scope.PlaceholderGroup = function () {
      return $scope.placeholderGroup == 'custom' ? 'admin-website-details-placeholder-custom.html' : 'admin-website-details-placeholder-builtin.html';
    }
    $scope.ShowPlaceholderWizard = function (placeholder) {
      return placeholder.visible;
    }
    $scope.PlacedholderChoiceClick = function (placeholder, choice) {
      placeholder.value = choice.value;
    }
  }
  app.controller('ControllerAdminWebsitesPlaceholder', ['$scope', ControllerAdminWebsitesPlaceholder]);
  function ControllerAdminWebsitesPlaceholder($scope) {
    $scope.isDeleting = false;
    $scope.DeletePlateholderStart = function () {
      $scope.isDeleting = true;
    }
    $scope.DeletePlateholderCancel = function () {
      $scope.isDeleting = false;
    }
  }

  app.controller('ControllerAdminSkillsMain', ['$scope', ControllerAdminSkillsMain]);
  function ControllerAdminSkillsMain($scope) {
    $scope.SetTitle('Skills');
  }
  app.controller('ControllerAdminSkills', ['$scope', '$route', '$location', '$timeout', 'iService.api.serial', 'iService.intProperties', 'iService.skills', 'iService.segments', '$q', ControllerAdminSkills]);
  function ControllerAdminSkills($scope, $route, $location, $timeout, API, intProperties, skills, segments, $q) {
    $scope.intProperties = intProperties;
    $scope.skills = skills;
    $scope.itemsToggler = Toggler();
    NewDeleteDetailer($scope, $location, '/skill', 'Skill', 'admin-skill-list.html', 'admin-skill-new.html', 'admin-skill-delete.html', 'admin-skill-details.html', skills.list, LoadDetails);

    $scope.Items = function () { return skills.list(); }
    var columns = [
      { name: 'ID', var: 'id', visible: false },
      { name: 'Skill Name', var: 'name', split: true },
      { name: 'Property Name', var: 'propertyID', split: true, printFunc: PrinterNameWithSegment(intProperties, segments) },
      { name: 'Property Value', var: 'propertyValue', split: true }];
    var skillsView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [skills.Watch, intProperties.Watch, segments.Watch], columns: columns, startSort: 'name', exportName: 'Skills', pageSize: 5 });
    $scope.skillsView = skillsView;

    var initialID = +$route.current.params.skillID;
    if (isNaN(initialID) || !Number.isInteger(initialID) || initialID < 1) {
      initialID = undefined;
    }
   
    var propertyTypes = {};
    $scope.propertyTypes = propertyTypes;
    skills.then(function () {
      if (initialID) skills.list().some(function (skill) {
        if (skill.id != initialID) return false;
        $scope.ChangeState('show', skill);
        return true;
      });
      intProperties.then(function () {
        intProperties.list().forEach(function (property) {
          propertyTypes[property.id] = (property.isNumber ? 'number' : (property.isDate ? 'date' : (property.isInteger ? 'integer' : 'text')));
        });
      });
    }).catch(angular.noop);
    function LoadValue(details) {
      intProperties.then(function () {
        if (propertyTypes[details.propertyID] == 'date') {
          var date = moment(details.propertyValue);
          if (date.isValid()) {
            details.propertyValueDate = new Date(date);
          }
        }
      });
    }
    function SaveValue(details) {
      if (propertyTypes[details.propertyID] == 'date') {
        details.propertyValue = moment(details.propertyValueDate).format('YYYY-MM-DD');
      }
    }

    $scope.Save = function (idPrefix, skill) {
      SaveValue(skill.details);
      API('webapp-api-admin?api=skills&mode=modify', idPrefix, 'SkillModify', { skillID: skill.id, details: skill.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        skillsView.BreakWords(result.row);
        //modify the row in the list.
        angular.extend(skill, result.row);
        //save the details
        angular.extend(skill.details, result.details);
        $scope.messages.saveSuccess = 'Skill changes saved.'
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      });
    }
    $scope.Create = function (idPrefix, details) {
      SaveValue(details);
      API('webapp-api-admin?api=skills&mode=new', idPrefix, 'SkillCreate', { details: details }).init(function () {
        $scope.messages.Clear();       
      }).then(function (result) {
        //add new row to the list.
        skills.add(result.row);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'Skill created.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
    $scope.DeleteStart = function () {
      $scope.selectedSkill.deleting = true;
      $scope.messages.Clear();
      API('webapp-api-admin?api=skilleffects', null, 'SkillEffects', { skillID: $scope.selectedSkill.id }).then(function (result) {
        $scope.selectedSkill.effects = result.effects;
      }).finally(angular.noop);
    }
    $scope.DeleteConfirm = function (idPrefix) {
      API('webapp-api-admin?api=skills&mode=delete', idPrefix, 'SkillDelete', { skillID: $scope.selectedSkill.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //remove row from list
        skills.remove(result.row);
        $scope.ChangeState('none');
        $scope.messages.deleteSuccess = 'Skill deleted.';
      }).catch(function (errors) {
        $scope.deleteErrors = errors;
      });
    }
    function LoadDetails(skill) {
      skill.details = { id: skill.id }
      API('webapp-api-admin?api=skilldetails', null, 'SkillDetails', { skillID: skill.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        //save the details
        angular.extend(skill.details, result.details);
        LoadValue(skill.details);
        skill.effects = result.details.effects;
      }).catch(function (errors) {
        //couldn't load
        $scope.loading = false;
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }

    $scope.RemoveFromDepartment = function (idPrefix, skill, department) {
      if (department.removeRunning) return;
      API('webapp-api-admin?api=departmentdetails', idPrefix, 'DepartmentDetails', { departmentID: department.departmentID }).init(function () {
        department.removeRunning = true;
        $scope.messages.Clear();
      }).then(function (result) {
        RemoveFromArrayInPlace(result.details.skillIDs, function (id) {
          return id == skill.id;
        });
        return API('webapp-api-admin?api=departments&mode=modify', idPrefix, 'DepartmentModify', { departmentID: department.departmentID, details: result.details }).catch(function (errors) {
          if (errors && errors.length) errors[0].msg = 'Error modifying department. ' + errors[0].msg;
          return $q.reject(errors);
        });
      }).then(function (result) {
        $scope.messages.saveSuccess = 'Department updated.';
        RemoveFromArrayInPlace(skill.effects.departments, function (item) {
          return item === department;
        });
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      }).finally(function () {
        department.removeRunning = false;
      });
    }
    $scope.RemoveFromTopic = function (idPrefix, skill, topic) {
      if (topic.removeRunning) return;
      API('webapp-api-admin?api=topicdetails', idPrefix, 'TopicDetails', { topicID: topic.topicID }).init(function () {
        topic.removeRunning = true;
        $scope.messages.Clear();
      }).then(function (result) {
        RemoveFromArrayInPlace(result.details.skills, function (id) {
          return id == skill.id;
        });
        return API('webapp-api-admin?api=topics&mode=modify', idPrefix, 'TopicModify', { topicID: topic.topicID, details: result.details }).catch(function (errors) {
          if (errors && errors.length) errors[0].msg = 'Error modifying topic. ' + errors[0].msg;
          return $q.reject(errors);
        });
      }).then(function (result) {
        $scope.messages.saveSuccess = 'Topic updated.';
        RemoveFromArrayInPlace(skill.effects.topics, function (item) {
          return item === topic;
        });
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      }).finally(function () {
        topic.removeRunning = false;
      });
    }
    $scope.RemoveFromAgent = function (idPrefix, skill, agent) {
      if (agent.removeRunning) return;
      API('webapp-api-admin?api=agentdetails', idPrefix, 'AgentDetails', { agentID: agent.contactID }).init(function () {
        agent.removeRunning = true;
        $scope.messages.Clear();
      }).then(function (result) {
        RemoveFromArrayInPlace(result.details.skillIDs, function (id) {
          return id == skill.id;
        });
        return API('webapp-api-admin?api=agentmodify', idPrefix, 'AgentModify', { agentID: agent.contactID, details: result.details, settings: [], custinfo: { properties: [] }, membership: result.custinfo.membership }).catch(function (errors) {
          if (errors && errors.length) errors[0].msg = 'Error modifying agent. ' + errors[0].msg;
          return $q.reject(errors);
        });
      }).then(function (result) {
        $scope.messages.saveSuccess = 'Agent updated.';
        RemoveFromArrayInPlace(skill.effects.agents, function (item) {
          return item === agent;
        });
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      }).finally(function () {
        agent.removeRunning = false;
      });
    }
  }
  app.filter("groupBy", ["$parse", "$filter", function ($parse, $filter) {
    return function (array, groupByField) {
      var result = [];
      var prev_item = null;
      var groupKey = false;
      var filteredData = $filter('orderBy')(array, groupByField);
      if (filteredData) {
        for (var i = 0; i < filteredData.length; i++) {
          groupKey = false;
          if (prev_item !== null) {
            if (prev_item[groupByField] !== filteredData[i][groupByField]) {
              groupKey = true;
            }
          } else {
            groupKey = true;
          }
          if (groupKey) {
            filteredData[i]['group_by_key'] = true;
          } else {
            filteredData[i]['group_by_key'] = false;
          }
          result.push(filteredData[i]);
          prev_item = filteredData[i];
        }
      }
      
      return result;
    }
  }])

  app.controller('ControllerAdminAgentsMain', ['$scope', ControllerAdminAgentsMain]);
  function ControllerAdminAgentsMain($scope) {
    $scope.SetTitle('Agents');
  }
  app.controller('ControllerAdminAgents', ['$scope', '$route', '$location', '$timeout', '$q', 'iService.api.serial', 'iService.agents', 'iService.usertypes', 'iService.segments', 'iService.departments', 'iService.skills', ControllerAdminAgents]);
  function ControllerAdminAgents($scope, $route, $location, $timeout, $q, API, agents, usertypes, segments, departments, skills) {
    $scope.agents = agents;
    $scope.usertypes = usertypes;
    $scope.segments = segments;
    $scope.departments = departments;
    $scope.skills = skills;
    $scope.itemsToggler = Toggler();
    $scope.licenseToggle = Toggler();
    $scope.stockToggle = Toggler();
    NewDeleteDetailer($scope, $location, '/agent', 'Agent', 'admin-agent-list.html', 'admin-agent-new.html', 'admin-agent-delete.html', 'admin-agent-details.html', agents.list, LoadDetails);
    $scope.loading = true;
    agents.then(function () {
      $scope.loading = false;
    });
    $scope.Items = function () { return agents.list(); }
    var columns = [
      { name: 'ID', var: 'id', visible: false },
      { name: 'Email', var: 'email', split: true },
      { name: 'Full Name', var: 'name', split: true },
      { name: 'Windows Login Name', var: 'windowsLoginName', visible: false },
      { name: 'User Type', var: 'typeID', printFunc: PrintUserType },
      { name: 'Segment Access', var: 'segments', split: true },
      { name: 'Skills', var: 'skills', split: true },
      { name: 'Department Membership', var: 'departments', split: true },
      { name: 'Receive Notifications', var: 'doReceive', visible: false },
      { name: 'Logged In', var: 'isLoggedIn' },
      { name: 'Chat Available', var: 'isAvailable' },
      { name: 'Last Activity', var: 'activity' },
      { name: 'Empty Inbox', var: 'doEmpty', visible: false },
      { name: 'Inactivity Timeout', var: 'timeout', visible: false }];
    var agentsUpdate = 0;
    var agentsView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [function () { return agentsUpdate; }], columns: columns, startSort: 'email', exportName: 'Agents', pageSize: 5 });
    $scope.agentsView = agentsView;

    function PrintUserType(typeID, agent) {
      if (typeID) {
        usertypes.then(function () {
          agent.typeIDP = usertypes.get(agent.typeID).name;
          return agent.typeIDP;
        });
      }
    }
    var agentID = +$route.current.params.agentID;
    if (isNaN(agentID) || !Number.isInteger(agentID) || agentID < 1) {
      agentID = undefined;
    }
   
    var stockID = +$route.current.params.stockID;
    if (isNaN(stockID) || !Number.isInteger(stockID) || stockID < 1) {
      stockID = undefined;
    }
    var tab = '';
    var startTab = $route.current.params.stockID ? 'stock' : $route.current.params.tab;
    if (startTab != 'settings' && startTab != 'contact' && startTab != 'stock') startTab = '';

    function ShowCount(agentsMax, agentsNum, agentsMaxChat, agentsChat) {
      $scope.agentsMax = agentsMax;
      $scope.agentsNum = agentsNum;
      if ($scope.agentsMax == 0) {
        $scope.agentsPercent = null;
      }
      else {
        $scope.agentsPercent = Math.round(100 * $scope.agentsNum / $scope.agentsMax) + '%';
      }
      $scope.agentsMaxChat = agentsMaxChat;
      $scope.agentsChat = agentsChat;
      if ($scope.agentsMaxChat == 0) {
        $scope.agentsChatPercent = null;
      }
      else {
        $scope.agentsChatPercent = Math.round(100 * $scope.agentsChat / $scope.agentsMaxChat) + '%';
      }
    }
    agents.then(function () {
      if (agentID) agents.list().some(function (agent) {
        if (agent.id != agentID) return false;
        $scope.ChangeState('show', agent);
        return true;
      });
      agentsUpdate++;
      WebsyncInit();
      ShowCount(agents.data().agentsMax, agents.data().agentsNum, agents.data().agentsMaxChat, agents.data().agentsChat)
    }).catch(angular.noop);
    $scope.AgentDetailsTabUrl = function (agent) {
      if (agent !== $scope.selectedAgent) return '';
      if (tab == 'settings') return 'admin-agent-details-settings.html';
      if (tab == 'contact') return 'admin-agent-details-contact.html';
      if (tab == 'stock') return 'admin-agent-details-stock.html';
      return 'admin-agent-details-info.html';
    }
    $scope.TabClass = function (name) {
      return (name == tab) ? 'active' : '';
    }
    $scope.ChangeTab = function (newTab) {
      if (newTab == tab) return;
      tab = newTab;
      if (tab == '') {
        $location.path('/agent/' + $scope.selectedAgent.id, false);
      }
      else {
        $location.path('/agent/' + $scope.selectedAgent.id + '/' + tab, false);
      }
      if (tab == 'stock') {
        $scope.selectedAgent.ChangeState('none');
      }
    }

    $scope.RecieveSave = function (idPrefix, agent) {
      API('webapp-api-admin?api=agentdetails', idPrefix, 'AgentDetails', { agentID: agent.id }).init(function () {
        agent.recieveRunning = true;
        $scope.messages.Clear();
      }).then(function (result) {
        iservice.SanitizePropertyGroups(result.settings);
        iservice.SanitizePropertyGroups(result.custinfo.properties);
        iservice.SetPropertyValue(result.settings, 'Receive Notifications', (agent.doReceive) ? 'True' : 'False');
        var settings = iservice.CollectProperties(result.settings);
        var custinfo = {
          properties: []
        };
        return API('webapp-api-admin?api=agentmodify', idPrefix, 'AgentModify', { agentID: agent.id, details: result.details, settings: settings, custinfo: custinfo, membership: result.custinfo.membership }).catch(function (errors) {
          if (errors && errors.length) errors[0].msg = 'Error modifying agent. ' + errors[0].msg;
          return $q.reject(errors);
        });
      }).then(function (result) {
        $scope.messages.toggleSuccess = 'Agent updated.'
        if (agent.settings) {
          iservice.SetPropertyValue(agent.settings, 'Receive Notifications', (agent.doReceive) ? 'True' : 'False');
        }
      }).catch(function (errors) {
        $scope.messages.toggleErrors = errors;
      }).finally(function () {
        agent.recieveRunning = false;
      });
    }
    $scope.EmptySave = function (idPrefix, agent) {
      API('webapp-api-admin?api=agentdetails', idPrefix, 'AgentDetails', { agentID: agent.id }).init(function () {
        agent.emptyRunning = true;
        $scope.messages.Clear();
      }).then(function (result) {
        iservice.SanitizePropertyGroups(result.settings);
        iservice.SanitizePropertyGroups(result.custinfo.properties);
        iservice.SetPropertyValue(result.settings, 'Empty Inbox When Logged Out', (agent.doEmpty) ? 'True' : 'False');
        var settings = iservice.CollectProperties(result.settings);
        var custinfo = {
          properties: []
        };
        return API('webapp-api-admin?api=agentmodify', idPrefix, 'AgentModify', { agentID: agent.id, details: result.details, settings: settings, custinfo: custinfo, membership: result.custinfo.membership }).catch(function (errors) {
          if (errors && errors.length) errors[0].msg = 'Error modifying agent. ' + errors[0].msg;
          return $q.reject(errors);
        });
      }).then(function (result) {
        $scope.messages.toggleSuccess = 'Agent updated.'
        if (agent.settings) {
          iservice.SetPropertyValue(agent.settings, 'Empty Inbox When Logged Out', (agent.doEmpty) ? 'True' : 'False');
        }
      }).catch(function (errors) {
        $scope.messages.toggleErrors = errors;
      }).finally(function () {
        agent.emptyRunning = false;
      });
    }
    $scope.Save = function (idPrefix, agent) {
      iservice.SetPropertyValue(agent.settings, 'Receive Notifications', (agent.doReceive) ? 'True' : 'False');
      iservice.SetPropertyValue(agent.settings, 'Empty Inbox When Logged Out', (agent.doEmpty) ? 'True' : 'False');
      iservice.SetPropertyValue(agent.settings, 'Inactivity Timeout Minutes', agent.timeout);
      var settings = iservice.CollectProperties(agent.settings);
      var custinfo = {
        properties: iservice.CollectProperties(agent.custinfo.properties),
        setLogins: iservice.CanModifyLogins(agent.custinfo.properties) == true,
        logins: iservice.CollectLogins(agent.custinfo.properties)
      };
      
      var custProps = [];
      //remove agents settings properties from custinfo properties
      custinfo.properties.forEach(function (item) {       
        if (item.propertyName !== 'Receive Notifications' && item.propertyName !== 'Empty Inbox When Logged Out' && item.propertyName !== 'Inactivity Timeout Minutes' && item.propertyName !== 'Notification Address') {
          custProps.push(item);
        }
      });
      custinfo.properties = custProps;
      API('webapp-api-admin?api=agentmodify&mode=modify', idPrefix, 'AgentModify', { agentID: agent.id, details: agent.details, settings: settings, custinfo: custinfo, membership: agent.custinfo.membership }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //sanitize the data.
        iservice.SanitizePropertyGroups(result.settings);
        iservice.SanitizePropertyGroups(result.custinfo.properties);
        agentsView.BreakWords(result.row);
        //modify the row in the list.
        if (result.row.id != '') {
          angular.extend(agent, result.row);
          //save the details
          angular.extend(agent.details, result.details);
          agent.settings = result.settings;
          agent.custinfo = result.custinfo;
        }
        else {
          agent.isDeleted = true;
          agents.remove(agent);
        }
        $scope.messages.agentSuccess = 'Agent updated successfully.';
        ShowCount($scope.agentsMax, result.agentsNum, $scope.agentsMaxChat, result.agentsChat);
        agentsUpdate++;
      }).catch(function (errors) {
        $scope.messages.agentErrors = errors;
      });
    }
    $scope.Create = function (idPrefix, details) {

      if (typeof details != 'undefined' && typeof details.userTypeID != 'undefined' && typeof details.emailAddress != 'undefined') {
        API('webapp-api-admin?api=agents&mode=new', idPrefix, 'AgentCreate', { details: details }).init(function () {
          $scope.messages.Clear();
        }).then(function (result) {
          //add new row to the list.
          agents.add(result.row);
          $scope.ChangeState('show', result.row);
          $scope.messages.createSuccess = 'Agent added successfully.';
          ShowCount($scope.agentsMax, result.agentsNum, $scope.agentsMaxChat, result.agentsChat);
          agentsUpdate++;
        }).catch(function (errors) {
          $scope.messages.createErrors = errors;
        });
      } else {
        $scope.messages.createErrors = ['There are required fields in this form that must have a value set before it can be submitted. Those fields are highlighted in red.'];
      }      
    }

    $scope.alreadyExists = false;
    $scope.showUsertype = false;
    $scope.showPassword = false;
    $scope.step1 = true;
    $scope.checkUserExists = function (idPrefix, details) {
      $scope.step1 = true;
      $scope.step2 = false;
      $scope.step3 = false;
      $scope.step4 = false;
      if (typeof details != 'undefined' && typeof details.emailAddress != 'undefined') {
        API('webapp-api-custinfo?api=search', idPrefix, null, {
          fields: {
            groups: [
              {
                fields: [{
                  where: 'anywhere',
                  fieldID: 'login',
                  searchString: details.emailAddress
                }]
              }
            ]
          }
        }).init(function () {

          $scope.messages.Clear();
        }).then(function (result) {
          if (result.resultCount > 0) {
            $scope.showUsertype = false;
            $scope.alreadyExists = true;
            $scope.step1 = false;
            $scope.step2 = true;
          } else {
            $scope.alreadyExists = false;
            $scope.showPassword = true;
            $scope.showUsertype = false;
            $scope.step1 = false;
            $scope.step2 = false;
            $scope.step4 = true;
          }
        }).catch(function (errors) {
          //$scope.messages.createErrors = errors;
        });
      }
      
    }
    
    $scope.Continue = function (idPrefix, details) {
      $scope.step1 = false;
      $scope.step2 = false;
      $scope.step3 = true;
      $scope.step4 = false;
      $scope.showUsertype = true;
    }
    $scope.ContinueNext = function (idPrefix, details) {
      $scope.step1 = false;
      $scope.step2 = false;
      $scope.step3 = true;
      $scope.step4 = false;
      $scope.showUsertype = true;
    }
    $scope.CancelNewUser = function () {
      $scope.ChangeState('none');
      $scope.alreadyExists = false;
      $scope.showUsertype = false;
      $scope.showPassword = false;
      $scope.step1 = true;
      $scope.step2 = false;
      $scope.step3 = false;
      $scope.step4 = false;
    }
    $scope.DeleteStart = function () {
      $scope.messages.Clear();
      $scope.selectedAgent.deleting = true;
    }
    
    $scope.RemoveAgent = function (idPrefix, agent) {
      if (agent.removeRunning) return;
      API('webapp-api-admin?api=agentdetails', idPrefix, 'AgentDetails', { agentID: agent.id }).init(function () {
        agent.removeRunning = true;
        $scope.messages.Clear();
      }).then(function (result) {
        result.details.userTypeID = 2;
        API('webapp-api-admin?api=agentmodify&mode=modify', idPrefix, 'AgentModify', { agentID: agent.id, details: result.details, settings: [], custinfo: { properties: [] }, membership: result.custinfo.membership }).then(function (response) {
          if (response.row.id == '') {
            agent.isDeleted = true;
            agent.removeRunning = false;
            agents.remove(agent);
            $scope.messages.removeSuccess = 'Agent removed successfully.';
            ShowCount($scope.agentsMax, response.agentsNum, $scope.agentsMaxChat, response.agentsChat);
            $scope.ChangeState('none');
            agentsUpdate++;
          }
        }).catch(function (errors) {
          $scope.messages.removeErrors = errors;
          agent.removeRunning = false;
        });
      });
    }
    $scope.Logout = function (idPrefix, agent) {
      API('webapp-api-admin?api=agents&mode=logout', idPrefix, 'AgentLogout', { agentID: agent.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        $scope.messages.agentSuccess = 'Agent logged out successfully.';
        agentsUpdate++;
      }).catch(function (errors) {
        $scope.messages.agentErrors = errors;
      });
    }
    function LoadDetails(agent) {
      agent.details = { id: agent.id, loadUpdate: 0 }
      API('webapp-api-admin?api=agentdetails', null, 'AgentDetails', { agentID: agent.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        //sanitize the data.
        iservice.SanitizePropertyGroups(result.settings);
        iservice.SanitizePropertyGroups(result.custinfo.properties);

        //save the details
        angular.extend(agent.details, result.details);
        agent.settings = result.settings;
        agent.custinfo = result.custinfo;
        const globalSegment = agent.custinfo.properties[0];
        globalSegment.name = 'Contact Information';
        agent.details.loadUpdate++;
        NewDeleteDetailer(agent, $location, '/agent/' + agent.id + '/stock', 'Stock', 'admin-agent-details-stock-list.html', 'admin-agent-details-stock-new.html', 'admin-agent-details-stock-delete.html', 'admin-agent-details-stock-details.html', function () { return agent.details.stockResponses; }, MakeLoadStock(agent));
        if (startTab) {
          $scope.ChangeTab(startTab);
        }
        if (stockID && agent.details && agent.details.stockResponses) agent.details.stockResponses.some(function (stock) {
          if (stock.id != stockID) return false;
          agent.ChangeState('show', stock);
          return true;
        });
      }).catch(function (errors) {
        //couldn't load
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }
    function MakeLoadStock(agent) {
      return function (stock) {
        stock.details = { id: stock.id }
        API('webapp-api-admin?api=stockresponsedetails', null, 'StockDetails', { stockID: stock.id }).init(function () {
          $scope.loading = true;
        }).then(function (result) {
          //save the details
          angular.extend(stock.details, result.details);
          stock.effects = result.details.effects;
        }).catch(function (errors) {
          //couldn't load
          agent.ChangeState('none');
        }).finally(function () {
          $scope.loading = false;
        });
      }
    }
    function WebsyncInit() {
      var agentChannel = {};
      var feedback = websync.FeedbackObject($scope);
      agentChannel.feedback = feedback;
      function MessageReceive(msg) {
        var update = msg.getData();
        var row = {
          id: update.contactID,
          email: update.emailAddress,
          name: update.firstName,
          typeID: update.typeID,
          segments: update.segentsNames,
          skills: update.skillsNames,
          departments: update.departmentNames,
          doReceive: update.doReceive,
          isLoggedIn: update.isLoggedIn,
          isAvailable: update.isAvailable,
          activity: update.activity,
          doEmpty: update.doEmpty,
          timeout: update.timeout,
          isDeleted: update.isDeleted
        };
        agentsView.BreakWords(row);
        var agent = agents.get(row.id);
        if (!agent) return;
        if (!row.isDeleted) {
          angular.extend(agent, row);
          agentsView.BreakWords(agent);
        }
        else {
          agents.remove(agent);
        }
      }
      feedback.Unsubscribe = websync.subscribe('/adminagents/' + iservice.loggedIn.tenantID, MessageReceive, feedback);
    }
  }
  app.controller('ControllerAdminAgentsStockDetails', ['$scope', '$timeout', 'iService.api.serial', ControllerAdminAgentsStockDetails]);
  function ControllerAdminAgentsStockDetails($scope, $timeout, API) {
    var agent = $scope.agent;
    $scope.Items = function () { return agent.details.stockResponses; }
    var columns = [
      { name: 'ID', var: 'id', visible: false },
      { name: 'Name', var: 'name', split: true },
      { name: 'Description', var: 'description', split: true }];
    function WatchLoad() { return agent.details.loadUpdate; }
    var stocksView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [WatchLoad], columns: columns, startSort: 'name', exportName: 'Stock Responses', pageSize: 5 });
    $scope.stocksView = stocksView;

    $scope.SaveStock = function (idPrefix, agent, stock) {
      API('webapp-api-admin?api=stockresponses&mode=modify', idPrefix, 'StockModify', { stockID: stock.id, details: stock.details }).init(function () {
        agent.messages.Clear();
      }).then(function (result) {
        //modify the row in the list.
        angular.extend(stock, result.row);
        //save the details
        angular.extend(stock.details, result.details);
        agent.messages.saveSuccess = 'Stock Responses updated successfully.'
      }).catch(function (errors) {
        agent.messages.saveErrors = errors;
      });
    }

    $scope.CreateStock = function (idPrefix, agent, details) {
      details.contactID = agent.id;
      API('webapp-api-admin?api=stockresponses&mode=new', idPrefix, 'StockCreate', { details: details }).init(function () {
        agent.messages.Clear();
      }).then(function (result) {
        //add new row to the list.
        agent.details.stockResponses.push(result.row);
        agent.details.loadUpdate++;
        agent.ChangeState('show', result.row);
        agent.messages.createSuccess = 'Stock Responses added successfully.';
      }).catch(function (errors) {
        agent.messages.createErrors = errors;
      });
    }
    $scope.DeleteStart = function (agent) {
      agent.selectedStock.deleting = true;
    }
    $scope.DeleteStock = function (idPrefix, agent, stock) {
      API('webapp-api-admin?api=stockresponses&mode=delete', idPrefix, 'StockResponseDelete', { stockID: stock.id }).init(function () {
        agent.messages.Clear();
      }).then(function (result) {
        //remove the response from it's owner's list.
        $scope.responses.remove($scope.response);
        agent.messages.deleteSuccess = 'Stock Response deleted successfully.';
      }).catch(function (errors) {
        agent.messages.deleteErrors = errors;
      });
    }
  }
  app.controller('ControllerAdminAgentsDetails', ['$scope', 'iService.usertypes', 'iService.segments', 'iService.departments', 'iService.skills', ControllerAdminAgentsDetails]);
  function ControllerAdminAgentsDetails($scope, usertypes, segments, departments, skills) {
    $scope.usertypes = usertypes;
    $scope.segments = segments;
    $scope.departments = departments;
    $scope.skills = skills;
  }
  app.controller('ControllerSegmentProperties', ['$scope', ControllerSegmentProperties]);
  function ControllerSegmentProperties($scope) {
    $scope.segmentToggle = Toggler();
    $scope.propertiesLeft = [];
    $scope.propertiesRight = [];
    var segment = $scope.segment;
    var properties = segment.properties.filter(function (prop) {
      if (prop.name == 'Receive Notifications') { $scope.settingReceiveNotifications = prop; return false; }
      if (prop.name == 'Notification Address') { $scope.settingNotficationAddress = prop; return false; }
      if (prop.name == 'Empty Inbox When Logged Out') { $scope.settingEmptyInboxWhenLoggedOut = prop; return false; }
      if (prop.name == 'Inactivity Timeout Minutes') { $scope.settingInactivityTimeoutMinutes = prop; return false; }
      return true;
    });
    var totalHeight = properties.reduce(function (prev, property) {
      return (property.values.length * (property.isMultiLine ? 2 : 1)) + prev;
    }, 0);
    var leftHeight = 0;
    properties.forEach(function (property) {
      if (leftHeight < totalHeight / 2) {
        $scope.propertiesLeft.push(property);
      }
      else {
        $scope.propertiesRight.push(property);
      }
      leftHeight += property.values.length * (property.isMultiLine ? 2 : 1);
    });
  }

  app.controller('ControllerAdminUserTypesMain', ['$scope', ControllerAdminUserTypesMain]);
  function ControllerAdminUserTypesMain($scope) {
    $scope.SetTitle('User Types');
  }
  app.controller('ControllerAdminUserTypes', ['$scope', '$route', '$location', '$timeout', 'iService.api.serial', 'iService.accessRights', 'iService.usertypes', ControllerAdminUserTypes]);
  function ControllerAdminUserTypes($scope, $route, $location, $timeout, API, accessRights, usertypes) {
    $scope.itemsToggler = Toggler();
    $scope.accessRights = accessRights;
    $scope.usertypes = usertypes;
 
    NewDeleteDetailer($scope, $location, '/usertype', 'UserType', 'admin-usertype-list.html', 'admin-usertype-new.html', 'admin-usertype-delete.html', 'admin-usertype-details.html', usertypes.list, LoadDetails);
    
    $scope.Items = function () { return usertypes.list(); }
    var columns = [
      { name: 'ID', var: 'id', visible: false },
      { name: 'User Type Name', var: 'name', split: true },
      { name: 'User Count', var: 'count' }];
    var usertypesView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [usertypes.Watch], columns: columns, startSort: 'name', exportName: 'User Types', pageSize: 5 });
    $scope.usertypesView = usertypesView;

    var initialID = +$route.current.params.usertypeID;
    if (isNaN(initialID) || !Number.isInteger(initialID) || initialID < 1) {
      initialID = undefined;
    }

    usertypes.then(function () {
      if (initialID) usertypes.list().some(function (usertype) {
        if (usertype.id != initialID) return false;
        $scope.ChangeState('show', usertype);
        return true;
      });
    }).catch(angular.noop);
    $scope.filterSelectedType = function (item) {
      if (item.id != $scope.selectedUserType.id) {
        return true;
      }
      else {
        return false;
      }
    }
    $scope.Save = function (idPrefix, usertype) {
      API('webapp-api-admin?api=usertypes&mode=modify', idPrefix, 'UserTypeModify', { usertypeID: usertype.id, details: usertype.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //modify the row in the list.
        angular.extend(usertype, result.row);
        //save the details
        angular.extend(usertype.details, result.details);
        $scope.messages.saveSuccess = 'User Type updated successfully.'
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      });
    }
    $scope.Create = function (idPrefix) {
      API('webapp-api-admin?api=usertypes&mode=new', idPrefix, 'UserTypeCreate', { details: $scope.newUserType }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //add new row to the list.
        usertypes.add(result.row);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'User type added successfully.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
    $scope.DeleteStart = function () {
      $scope.selectedUserType.deleting = true;
      $scope.messages.Clear();
      API('webapp-api-admin?api=usertypeeffects', null, 'UserTypeEffects', { usertypeID: $scope.selectedUserType.id }).then(function (result) {
        $scope.selectedUserType.effects = result.effects;
      }).finally(angular.noop);
    }
    $scope.DeleteConfirm = function (idPrefix) {
      API('webapp-api-admin?api=usertypes&mode=delete', idPrefix, 'UserTypeDelete', { usertypeID: $scope.selectedUserType.id, replacetypeID: $scope.selectedUserType.replaceTypeID }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //remove row from list
        usertypes.remove(result.row);
        $scope.ChangeState('none');
        $scope.messages.deleteSuccess = 'User type deleted successfully.';
      }).catch(function (errors) {
        $scope.messages.deleteErrors = errors;
      });
    }
    function LoadDetails(usertype) {
      usertype.details = { id: usertype.id }
      API('webapp-api-admin?api=usertypedetails', null, 'UserTypeDetails', { usertypeID: usertype.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        //save the details
        angular.extend(usertype.details, result.details);
        usertype.effects = result.details.effects;
      }).catch(function (errors) {
        //couldn't load
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }
  }

  app.controller('ControllerAdminAlertsMain', ['$scope', '$interval', 'iService.rootPath', ControllerAdminAlertsMain]);
  function ControllerAdminAlertsMain($scope, $interval, rootPath) {
    $scope.SetTitle('Alerts');
    $scope.iframe = IFrameContainer($interval, $scope.idPrefix + '_IFrame');
    $scope.iframe.src = rootPath + 'Admin.aspx?admin=alert';
  }

  app.controller('ControllerAdminDepartmentsMain', ['$scope', ControllerAdminDepartmentsMain]);
  function ControllerAdminDepartmentsMain($scope) {
    $scope.SetTitle('Departments');
  }
  app.controller('ControllerAdminDepartments', ['$scope', '$route', '$timeout', '$location', 'iService.api.serial',  'iService.departments', 'iService.segments', 'iService.skills', ControllerAdminDepartments]);
  function ControllerAdminDepartments($scope, $route, $timeout, $location, API, departments, segments, skills) {
    $scope.itemsToggler = Toggler();
    $scope.departments = departments;
    $scope.segments = segments;
    $scope.skills = skills;
    $scope.items = [];
    NewDeleteDetailer($scope, $location, '/department', 'Department', 'admin-department-list.html', 'admin-department-new.html', 'admin-department-delete.html', 'admin-department-details.html', departments.list, LoadDetails);

    $scope.Items = function () { return departments.list(); }
    var columns = [
      { name: 'Id', var: 'id', visible: true },
      { name: 'Department Name', var: 'name', visible: true, split: true },
      { name: 'Segments', var: 'segmentnames', visible: true, split: true },
      { name: 'Skills', var: 'skillnames', visible: true, split: true }
    ];
    var departmentsView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [departments.Watch, segments.Watch, skills.Watch], columns: columns, startSort: 'id', exportName: 'Departments List', pageSize: 5 });
    $scope.departmentsView = departmentsView;


    var initialID = +$route.current.params.departmentID;
    if (isNaN(initialID) || !Number.isInteger(initialID) || initialID < 1) {
      initialID = undefined;
    }

    departments.then(function () {
      //$scope.items = departments.list();
      if (initialID) departments.list().some(function (department) {
        if (department.id != initialID) return false;
        $scope.ChangeState('show', department);
        return true;
      });
    }).catch(angular.noop);

    $scope.Save = function (idPrefix, department) {
      API('webapp-api-admin?api=departments&mode=modify', idPrefix, 'DepartmentModify', { departmentID: department.id, details: department.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //modify the row in the list.
        angular.extend(department, result.row);
        //save the details
        angular.extend(department.details, result.details);
        $scope.messages.saveSuccess = 'Department updated successfully.'
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      });
    }
    $scope.Create = function (idPrefix) {
      API('webapp-api-admin?api=departments&mode=new', idPrefix, 'DepartmentCreate', { details: $scope.newDepartment }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //add new row to the list.
        departments.add(result.row);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'Department added successfully.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
    $scope.DeleteStart = function () {
      $scope.selectedDepartment.deleting = true;
      $scope.messages.Clear();
      API('webapp-api-admin?api=departmenteffects', null, 'DepartmentEffects', { departmentID: $scope.selectedDepartment.id }).then(function (result) {
        $scope.selectedDepartment.effects = result.effects;
      }).finally(angular.noop);
    }
    $scope.DeleteConfirm = function (idPrefix) {
      API('webapp-api-admin?api=departments&mode=delete', idPrefix, 'DepartmentDelete', { departmentID: $scope.selectedDepartment.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //remove row from list
        departments.remove(result.row);
        $scope.ChangeState('none');
        $scope.messages.deleteSuccess = 'Department deleted successfully.';
      }).catch(function (errors) {
        $scope.messages.deleteErrors = errors;
      });
    }
    function LoadDetails(department) {
      department.details = { id: department.id }
      API('webapp-api-admin?api=departmentdetails', null, 'DepartmentDetails', { departmentID: department.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        //save the details
        angular.extend(department.details, result.details);
        department.effects = result.details.effects;
      }).catch(function (errors) {
        //couldn't load
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }
  }

  app.controller('ControllerAdminListsMain', ['$scope', ControllerAdminListsMain]);
  function ControllerAdminListsMain($scope) {
    $scope.SetTitle('List Settings');
  }
  app.controller('ControllerAdminLists', ['$scope', '$route', '$timeout', '$location', 'iService.api.serial', 'iService.lists', ControllerAdminLists]);
  function ControllerAdminLists($scope, $route, $timeout, $location, API, lists) {
    $scope.itemsToggler = Toggler();
    $scope.lists = lists;
    $scope.items = [];
    NewDeleteDetailer($scope, $location, '/list', 'List', 'admin-list-list.html', 'admin-list-new.html', 'admin-list-delete.html', 'admin-list-details.html', lists.list, LoadDetails);

    $scope.Items = function () { return lists.list(); }
    var columns = [
      { name: 'Id', var: 'id', visible: true },
      { name: 'List Name', var: 'name', visible: true, split: true },
      { name: 'Description', var: 'description', visible: true, split: true }
    ];
    var listsView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [lists.Watch], columns: columns, startSort: 'id', exportName: 'Lists', pageSize: 5 });
    $scope.listsView = listsView;

    var initialID = +$route.current.params.listID;
    if (isNaN(initialID) || !Number.isInteger(initialID) || initialID < 1) {
      initialID = undefined;
    }

    lists.then(function () {
      //$scope.items = lists.list();
      if (initialID) lists.list().some(function (list) {
        if (list.id != initialID) return false;
        $scope.ChangeState('show', list);
        return true;
      });
    }).catch(angular.noop);

    $scope.Save = function (idPrefix, list) {
      API('webapp-api-admin?api=listsadmin&mode=modify', idPrefix, 'ListModify', { listID: list.id, details: list.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //modify the row in the list.
        angular.extend(list, result.row);
        //save the details
        angular.extend(list.details, result.details);
        $scope.messages.saveSuccess = 'List updated successfully.'
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      });
    }
    $scope.NewItem = function () {
      return { name: '', description: '', values: [] };
    }
    $scope.Create = function (idPrefix) {
      API('webapp-api-admin?api=listsadmin&mode=new', idPrefix, 'ListCreate', { details: $scope.newList }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //add new row to the list.
        lists.add(result.row);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'List added successfully.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
    $scope.DeleteStart = function () {
      $scope.messages.Clear();
      $scope.selectedList.deleting = true;
    }
    $scope.DeleteConfirm = function (idPrefix) {
      API('webapp-api-admin?api=listsadmin&mode=delete', idPrefix, 'ListDelete', { listID: $scope.selectedList.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //remove row from list
        lists.remove(result.row);
        $scope.ChangeState('none');
        $scope.messages.deleteSuccess = 'List deleted successfully.';
      }).catch(function (errors) {
        $scope.messages.deleteErrors = errors;
      });
    }
    function LoadDetails(list) {
      list.details = { id: list.id }
      API('webapp-api-admin?api=listdetails', null, 'ListDetails', { listID: list.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        //save the details
        angular.extend(list.details, result.details);
        list.effects = result.details.effects;
        var segmentByID = {};
        var segments = [];
        [list.effects.contactValues, list.effects.contactDescriptions, list.effects.interactionValues, list.effects.interactionDescriptions].forEach(function (proplist) {
          proplist.sort(function (l, r) {
            return l.name.localeCompare(r.name);
          });
          proplist.forEach(function (prop) {
            var segment = segmentByID[prop.segmentID];
            if (!segment) {
              segment = { id: prop.segmentID, name: prop.segmentName, properties: [], propertyByID: {} };
              segmentByID[prop.segmentID] = segment;
              segments.push(segment);
            }
            if (!segment.propertyByID[prop.propertyID]) {
              segment.propertyByID[prop.propertyID] = prop;
              segment.properties.push(prop);
            }
          });
        });
        segments.sort(function (l, r) {
          return l.name.localeCompare(r.name);
        });
        segments.forEach(function (segment) {
          segment.properties.sort(function (l, r) {
            return l.name.localeCompare(r.name);
          });
        });
        list.effects.segments = segments;
      }).catch(function (errors) {
        //couldn't load
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }
  }
  app.controller('ControllerAdminListDetails', ['$scope', ControllerAdminListDetails]);
  function ControllerAdminListDetails($scope) {
    var ctrl = this;
    ctrl.valueToAdd = '';
    $scope.ValueAdd = function (details) {
      var val = ('' + ctrl.valueToAdd).trim();
      if (val.length < 1) return;
      details.values.push(ctrl.valueToAdd);
      ctrl.valueToAdd = '';
    }
    $scope.ValueRemove = function (details, value) {
      details.values.remove(value);
    }
  }

  function formatDate(date) {
    var d = new Date(date),
      dformat = [d.getMonth() + 1,
      d.getDate(),
      d.getFullYear()].join('/') + ' ' +
        [d.getHours(),
        d.getMinutes(),
        d.getSeconds()].join(':');
    return d;
  }
  app.controller('ControllerAdminMassmailingMain', ['$scope', ControllerAdminMassmailingMain]);
  function ControllerAdminMassmailingMain($scope) {
    $scope.SetTitle('Mailings');
  }
  app.controller('ControllerAdminMassmailing', ['$scope', '$route', '$timeout', '$location', 'iService.api.serial','iService.massmailings', 'iService.masslists', 'iService.massmessages', 'iService.topics', 'iService.mailboxes', 'iService.masscriterias', ControllerAdminMassmailing]);
  function ControllerAdminMassmailing($scope, $route, $timeout, $location, API, massmailings, masslists, massmessages, topics, mailboxes, masscriterias) {
    $scope.itemsToggler = Toggler();
    $scope.massmailings = massmailings;
    $scope.masslists = masslists;
    $scope.massmessages = massmessages;
    $scope.topics = topics;
    $scope.mailboxes = mailboxes;
    $scope.masscriterias = masscriterias;
    $scope.items = [];
    NewDeleteDetailer($scope, $location, '/massmailing', 'Massmailing', 'admin-massmailing-list.html', 'admin-massmailing-new.html', 'admin-massmailing-delete.html', 'admin-massmailing-details.html', massmailings.list, LoadDetails);
    $scope.messages.names.push('sendSuccess');
    $scope.messages.names.push('sendErrors');

    $scope.Items = function () { return massmailings.list(); }

    var columns = [
      { name: 'Id', var: 'id', visible: true },
      { name: 'Mailing Name', var: 'name', visible: true, split: true },
      { name: 'Description', var: 'description', visible: true, split: true },
      { name: 'Topic', var: 'topicID', split: true, printFunc: PrinterName(topics) },
      { name: 'List', var: 'listID', split: true },
      { name: 'Message', var: 'messageID', split: true },
      { name: 'Mailbox', var: 'mailboxID', split: true },
      { name: 'Last Mailing', var: 'lastMail', printFunc: PrintDate, sortRaw:true, exportType: 'timestamp', exportFunc: (date) => { return date; }  }
    ];
    function PrintDate(date, row) {
      if (date) {
        row.lastMail = formatDate(row.lastMail);
        row.lastMailT = moment(row.lastMail).format('YYYY-MM-DD hh:mm:ss A');
        return moment(row.lastMail).fromNow();
      }
      return '';
    }
    var massmailingsView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [massmailings.Watch, topics.Watch], columns: columns, startSort: 'lastMail', exportName: 'Mass Mailing Lists', pageSize: 5 });
    $scope.massmailingsView = massmailingsView;

    var initialID = +$route.current.params.massmailingID;
    if (isNaN(initialID) || !Number.isInteger(initialID) || initialID < 1) {
      initialID = undefined;
    }

    massmailings.then(function () {
      $scope.items = massmailings.list();
      if (initialID) massmailings.list().some(function (item) {
        if (item.id != initialID) return false;
        $scope.ChangeState('show', item);
        return true;
      });
    }).catch(angular.noop);

    $scope.Save = function (idPrefix, massmailing) {
      API('webapp-api-admin?api=massmailings&mode=modify', idPrefix, 'MassmailingModify', { massmailingID: massmailing.id, details: massmailing.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //modify the row in the list.
        angular.extend(massmailing, result.row);
        //save the details
        angular.extend(massmailing.details, result.details);
        $scope.messages.saveSuccess = 'Mailing updated successfully.'
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      });
    }
    $scope.Create = function (idPrefix) {
      API('webapp-api-admin?api=massmailings&mode=new', idPrefix, 'MassmailingCreate', { details: $scope.newMassmailing }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //add new row to the list.
        massmailings.add(result.row);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'Mailing added successfully.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
    $scope.DeleteStart = function () {
      $scope.selectedMassmailing.deleting = true;
      $scope.messages.Clear();
      API('webapp-api-admin?api=massmailingeffects', null, 'MassmailingEffects', { massmailingID: $scope.selectedMassmailing.id }).then(function (result) {
        $scope.selectedMassmailing.effects = result.effects;
      }).finally(angular.noop);
    }
    $scope.DeleteConfirm = function (idPrefix) {
      API('webapp-api-admin?api=massmailings&mode=delete', idPrefix, 'MassmailingDelete', { massmailingID: $scope.selectedMassmailing.id, replacetypeID: $scope.selectedMassmailing.replaceTypeID }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //remove row from list
        massmailings.remove(result.row);
        $scope.ChangeState('none');
        $scope.messages.deleteSuccess = 'Mailing deleted successfully.';
      }).catch(function (errors) {
        $scope.messages.deleteErrors = errors;
      });
    }
    function LoadDetails(massmailing) {
      massmailing.details = { id: massmailing.id }
      API('webapp-api-admin?api=massmailingdetails', null, 'MassmailingDetails', { massmailingID: massmailing.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        //save the details
        angular.extend(massmailing.details, result.details);
        massmailing.effects = result.details.effects;
      }).catch(function (errors) {
        //couldn't load
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }
  }
  app.controller('ControllerAdminMassmailingDetails', ['$scope', 'iService.api.serial', ControllerAdminMassmailingDetails]);
  function ControllerAdminMassmailingDetails($scope, API) {
    var send;
    function Reset() {
      send = { mailingID: $scope.massmailing.id, criteriaID: '', checkCount: '' };
      $scope.send = send;
    }
    Reset();

    $scope.SendCheck = function (idPrefix, massmailing, criteriaID) {
      API('webapp-api-mailing?api=send&mode=check', idPrefix, 'SendCheck', { details: send }).init(function () {
        $scope.messages.Clear();
        send.checkCount = ''; 
        $scope.running = true;
      }).then(function (result) {
        send.checkCount = result.count;
      }).catch(function (errors) {
        $scope.messages.sendErrors = errors;
      }).finally(function () {
        $scope.running = false;
      });
    }

    $scope.SendConfirm = function (idPrefix, massmailing, criteriaID) {
      API('webapp-api-mailing?api=send&mode=send', idPrefix, 'SendCheck', { details: send }).init(function () {
        $scope.messages.Clear();
        send.checkCount = '';
        $scope.running = true;
      }).then(function (result) {
        $scope.messages.sendSuccess = 'Mailing sent to ' + result.count + ' contact' + (result.count == '1' ? '' : 's') + '.';
        Reset();
      }).catch(function (errors) {
        $scope.messages.sendErrors = errors;
      }).finally(function () {
        $scope.running = false;
      });
    }
  }

  app.controller('ControllerAdminMasslistMain', ['$scope', ControllerAdminMasslistMain]);
  function ControllerAdminMasslistMain($scope) {
    $scope.SetTitle('Mailing Lists');
  }
  app.controller('ControllerAdminMasslist', ['$scope', '$route', '$location', 'iService.api.serial', '$timeout', '$q', 'iService.masslists', 'iService.segments', ControllerAdminMasslist]);
  function ControllerAdminMasslist($scope, $route, $location, API, $timeout, $q, masslists, segments) {
    $scope.itemsToggler = Toggler();
    $scope.masslists = masslists;
    $scope.segments = segments;
    NewDeleteDetailer($scope, $location, '/masslist', 'Masslist', 'admin-masslist-list.html', 'admin-masslist-new.html', 'admin-masslist-delete.html', 'admin-masslist-details.html', masslists.list, LoadDetails);

    $scope.Items = function () { return masslists.list(); }
    var columns = [
      { name: 'ID', var: 'id', visible: false },
      { name: 'List Name', var: 'name', split: true },
      { name: 'Description', var: 'description', split: true },
      { name: 'Segments', var: 'segments', split: true },
      { name: 'Shown In My Account', var: 'shown' },
      { name: 'Subscribers', var: 'subCount' },
      { name: 'Unsubscribers', var: 'unsubCount' },
      { name: 'Last Mailing', var: 'lastMail', printFunc: PrintDate, sortRaw: true, exportType: 'timestamp', exportFunc: (date) => { return date; } }];

    function PrintDate(date, row) {
      if (date) {
        row.lastMailT = moment(row.lastMail).format('YYYY-MM-DD hh:mm:ss A');
        return moment(row.lastMail).fromNow();
      }
      return '';
    }
    var masslistsView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [masslists.Watch], columns: columns, startSort: 'name', exportName: 'Mass Mailing Lists', pageSize: 5 });
    $scope.masslistsView = masslistsView;

    var initialID = +$route.current.params.masslistID;
    if (isNaN(initialID) || !Number.isInteger(initialID) || initialID < 1) {
      initialID = undefined;
    }

    masslists.then(function () {
      $scope.items = masslists.list();
      if (initialID) masslists.list().some(function (item) {
        if (item.id != initialID) return false;
        $scope.ChangeState('show', item);
        return true;
      });
    }).catch(angular.noop);

    $scope.ShownSave = function (idPrefix, masslist) {
      API('webapp-api-admin?api=masslistdetails', idPrefix, 'MasslistDetails', { masslistID: masslist.id }).init(function () {
        masslist.shownRunning = true;
        $scope.messages.Clear();
      }).then(function (result) {
        result.details.shown = masslist.shown;
        return API('webapp-api-admin?api=masslists&mode=modify', idPrefix, 'MasslistModify', { masslistID: masslist.id, details: result.details }).catch(function (errors) {
          if (errors && errors.length) errors[0].msg = 'Error modifying mailing list. ' + errors[0].msg;
          return $q.reject(errors);
        });
      }).then(function (result) {
        $scope.messages.toggleSuccess = 'Mailing List updated.'
        if (masslist.details) masslist.details.shown = masslist.shown;
      }).catch(function (errors) {
        $scope.messages.toggleErrors = errors;
      }).finally(function () {
        masslist.shownRunning = false;
      });
    }

    $scope.Save = function (idPrefix, masslist) {
      API('webapp-api-admin?api=masslists&mode=modify', idPrefix, 'MasslistModify', { masslistID: masslist.id, details: masslist.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //modify the row in the list.
        angular.extend(masslist, result.row);
        //save the details
        angular.extend(masslist.details, result.details);
        $scope.messages.saveSuccess = 'Mailing List updated successfully.'
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      });
    }
    $scope.Create = function (idPrefix) {
      API('webapp-api-admin?api=masslists&mode=new', idPrefix, 'MasslistCreate', { details: $scope.newMasslist }).init(function () {
        $scope.createErrors = null;
      }).then(function (result) {
        //add new row to the list.
        masslists.add(result.row);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'Mailing List added successfully.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
    $scope.DeleteStart = function () {
      $scope.selectedMasslist.deleting = true;
      $scope.messages.Clear();
      API('webapp-api-admin?api=masslisteffects', null, 'MasslistEffects', { masslistID: $scope.selectedMasslist.id }).then(function (result) {
        $scope.selectedMasslist.effects = result.effects;
      }).finally(angular.noop);
    }
    $scope.DeleteConfirm = function (idPrefix) {
      API('webapp-api-admin?api=masslists&mode=delete', idPrefix, 'MasslistDelete', { masslistID: $scope.selectedMasslist.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //remove row from list
        masslists.remove(result.row);
        $scope.ChangeState('none');
        $scope.messages.deleteSuccess = 'Mailing List deleted successfully.';
      }).catch(function (errors) {
        $scope.messages.deleteErrors = errors;
      });
    }
    function LoadDetails(masslist) {
      masslist.details = { id: masslist.id }
      API('webapp-api-admin?api=masslistdetails', null, 'MasslistDetails', { masslistID: masslist.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        //save the details
        angular.extend(masslist.details, result.details);
        masslist.effects = result.details.effects;
      }).catch(function (errors) {
        //couldn't load
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }

    $scope.detailTab = 'details';
    $scope.ContentTabClass = function (tab) {
      if (tab == $scope.detailTab) return 'active';
      return '';
    }
    $scope.ContentTabChange = function (tab) {
      $scope.detailTab = tab;
    }
    $scope.ContentPaneShowing = function (tab) {
      return $scope.detailTab == tab;
    }
  }
  app.controller('ControllerAdminMasslistMembers', ['$scope', '$timeout', 'iService.api.serial', ControllerAdminMasslistMembers]);
  function ControllerAdminMasslistMembers($scope, $timeout, API) {
    var masslist = $scope.masslist;
    var tab = $scope.detailTab;
    MessagesContainer($scope);

    var contactsUpdate = 0;
    $scope.Items = function () {
      if ($scope.mode == 'add' || $scope.mode == 'remove') return $scope.searchResults;
      return $scope.currentSubscribers;
    }
    var columns = [
      { name: 'ID', var: 'id', visible: false },
      { name: 'Name', var: 'name', split: true },
      { name: 'Email', var: 'email', split: true },
      { name: 'Company', var: 'company', split: true },
      { name: 'Phone', var: 'phone', split: true }];
    var contactsView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [function () { return contactsUpdate; }], columns: columns, startSort: 'name', exportName: 'Contacts', pageSize: 5 });
    $scope.contactsView = contactsView;

    $scope.memberSearch = (tab == 'subs') ? iservice.searchfields.NewSearch('entire', 'subscribed', masslist.id) : iservice.searchfields.NewSearch('entire', 'opted-out', masslist.id);
    function DoSearch() {
      API('webapp-api-custinfo?api=search', null, 'ListMembers', { fields: { groups: $scope.memberSearch } }).init(function () {
        $scope.searching = true;
        $scope.currentSubscribers = [];
      }).then(function (result) {
        $scope.currentSubscribers = result.contacts;
        $scope.numContacts = result.resultCount;
        if ((tab == 'subs')) {
          masslist.subCount = result.resultCount;
        }
        if ((tab == 'unsubs')) {
          masslist.unsubCount = result.resultCount;
        }
        
        contactsView.filterString = '';
        contactsUpdate++;
      }).catch(function (errors) {

      }).finally(function () {
        $scope.searching = false;
      });
    }
    DoSearch();

    $scope.mode = 'show';
    $scope.StartAdd = function () {
      $scope.mode = 'add';
      $scope.searchGroups = iservice.searchfields.NewSearch();
      $scope.searchResults = null;
      contactsUpdate++;
    }
    $scope.StartRemove = function () {
      $scope.mode = 'remove';
      $scope.searchGroups = iservice.searchfields.NewSearch();
      $scope.searchResults = null;
      contactsUpdate++;
    }
    $scope.StopModify = function () {
      $scope.mode = 'show';
      contactsUpdate++;
    }

    $scope.ContactSearchClick = function () {
      var search = { groups: [] };
      if ($scope.mode == 'add') {
        $scope.searchGroups.forEach(function (group) { search.groups.push(group); });
        iservice.searchfields.NewFieldGroup(search.groups, 'entire', 'subscribed-not', masslist.id);
      }
      else if ($scope.mode == 'remove') {
        iservice.searchfields.NewFieldGroup(search.groups, 'entire', 'subscribed', masslist.id);
        $scope.searchGroups.forEach(function (group) { search.groups.push(group); });
      }
      API('webapp-api-custinfo?api=search', null, 'NewMembers', { fields: search }).init(function () {
        $scope.searching = true;
        $scope.searchResults = [];
        contactsUpdate++;
      }).then(function (result) {
        $scope.searchResults = result.contacts;
        contactsUpdate++;
      }).catch(function () {

      }).finally(function () {
        $scope.searching = false;
      });
    }
    function DoAdd(idPrefix, IDs) {
      API('webapp-api-admin?api=masssubscriptions&mode=add', idPrefix, 'MassSubscribe', { contactIDs: IDs, masslistID: masslist.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        $scope.messages.saveSuccess = 'Added contacts to the mailing list.';
        $scope.StopModify();
        DoSearch();
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      }).finally(function () {

      });
    }
    $scope.SubscribeSelectedClick = function (idPrefix) {
      DoAdd(idPrefix, contactsView.selectedIDs);
    }
    $scope.SubscribeAllClick = function (idPrefix) {
      DoAdd(idPrefix, contactsView.filteredItems.map(function (item) { return item.id; }));
    }
    function DoRemove(idPrefix, IDs) {
      API('webapp-api-admin?api=masssubscriptions&mode=remove', idPrefix, 'MassUnsubscribe', { contactIDs: IDs, masslistID: masslist.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        $scope.messages.saveSuccess = 'Removed contacts from the mailing list.';
        $scope.StopModify();
        DoSearch();
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      }).finally(function () {

      });
    }
    $scope.RemoveSelectedClick = function (idPrefix) {
      DoRemove(idPrefix, contactsView.selectedIDs);
    }
    $scope.RemoveAllClick = function (idPrefix) {
      DoRemove(idPrefix, contactsView.filteredItems.map(function (item) { return item.id; }));
    }
  }
  
  app.controller('ControllerAdminMassmessageMain', ['$scope', ControllerAdminMassmessageMain]);
  function ControllerAdminMassmessageMain($scope) {
    $scope.SetTitle('Mailing Messages');
  }
  app.controller('ControllerAdminMassmessage', ['$scope', '$route', '$location', '$timeout', '$q', 'iService.api.serial', 'iService.massmessages', 'iService.topics', 'iService.stockresponses', 'iService.segments', ControllerAdminMassmessage]);
  function ControllerAdminMassmessage($scope, $route, $location, $timeout, $q, API, massmessages, topics, stockresponses, segments) {
    $scope.itemsToggler = Toggler();
    $scope.massmessages = massmessages;
    $scope.topics = topics;
    $scope.stockresponses = stockresponses;
    NewDeleteDetailer($scope, $location, '/massmessage', 'Massmessage', 'admin-massmessage-list.html', 'admin-massmessage-new.html', 'admin-massmessage-delete.html', 'admin-massmessage-details.html', massmessages.list, LoadDetails);

    $scope.Items = function () { return massmessages.list(); }
    var columns = [
      { name: 'ID', var: 'id', visible: false },
      { name: 'Name', var: 'name', split: true },
      { name: 'Description', var: 'description', split: true },
      { name: 'Topic', var: 'topicID', split: true, printFunc: PrinterNameWithSegment(topics, segments) },
      { name: 'Subject Line', var: 'subject', split: true },
      { name: 'Include Ref', var: 'addRefNum', split: true },
      { name: 'Last Mailing', var: 'lastMail', printFunc: PrintDate, sortRaw: true, exportType: 'timestamp', exportFunc: (date) => { return date; } }
    ];
    function PrintDate(date, row) {
      if (date) {
        row.lastMailT = moment(row.lastMail).format('YYYY-MM-DD hh:mm:ss A');
        return moment(row.lastMail).fromNow();
      }
      return '';
    }
    var massmessagesView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [massmessages.Watch, topics.Watch, segments.Watch], columns: columns, startSort: 'name', exportName: 'Mass Mailing Lists', pageSize: 5 });
    $scope.massmessagesView = massmessagesView;

    var initialID = +$route.current.params.massmessageID;
    if (isNaN(initialID) || !Number.isInteger(initialID) || initialID < 1) {
      initialID = undefined;
    }

    massmessages.then(function () {
      if (initialID) massmessages.list().some(function (item) {
        if (item.id != initialID) return false;
        $scope.ChangeState('show', item);
        return true;
      });
    }).catch(angular.noop);

    $scope.IncludeSave = function (idPrefix, massmessage) {
      API('webapp-api-admin?api=massmessagedetails', idPrefix, 'MassmessageDetails', { massmessageID: massmessage.id }).init(function () {
        massmessage.includeRunning = true;
        $scope.messages.Clear();
      }).then(function (result) {
        result.details.addRefNum = massmessage.addRefNum;
        return API('webapp-api-admin?api=massmessages&mode=modify', idPrefix, 'MassmessageModify', { massmessageID: massmessage.id, details: result.details }).catch(function (errors) {
          if (errors && errors.length) errors[0].msg = 'Error modifying message. ' + errors[0].msg;
          return $q.reject(errors);
        });
      }).then(function (result) {
        $scope.messages.toggleSuccess = 'Message updated.'
        if (massmessage.details) massmessage.details.addRefNum = massmessage.addRefNum;
      }).catch(function (errors) {
        $scope.messages.toggleErrors = errors;
      }).finally(function () {
        massmessage.includeRunning = false;
      });
    }
    $scope.Save = function (idPrefix, massmessage) {
      API('webapp-api-admin?api=massmessages&mode=modify', idPrefix, 'MassmessageModify', { massmessageID: massmessage.id, details: massmessage.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //modify the row in the list.
        angular.extend(massmessage, result.row);
        //save the details
        angular.extend(massmessage.details, result.details);
        $scope.messages.saveSuccess = 'Message updated successfully.'
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      });
    }
    $scope.Create = function (idPrefix) {
      API('webapp-api-admin?api=massmessages&mode=new', idPrefix, 'MassmessageCreate', { details: $scope.newMassmessage }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //add new row to the list.
        massmessages.add(result.row);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'Message created successfully.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
    $scope.DeleteStart = function () {
      $scope.selectedMassmessage.deleting = true;
      $scope.messages.Clear();
      API('webapp-api-admin?api=massmessageeffects', null, 'MassmessageEffects', { massmessageID: $scope.selectedMassmessage.id }).then(function (result) {
        $scope.selectedMassmessage.effects = result.effects;
      }).finally(angular.noop);
    }
    $scope.DeleteConfirm = function (idPrefix) {
      API('webapp-api-admin?api=massmessages&mode=delete', idPrefix, 'MassmessageDelete', { massmessageID: $scope.selectedMassmessage.id, replacetypeID: $scope.selectedMassmessage.replaceTypeID }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //remove row from list
        massmessages.remove(result.row);
        $scope.ChangeState('none');
        $scope.messages.deleteSuccess = 'Message deleted successfully.';
      }).catch(function (errors) {
        $scope.messages.deleteErrors = errors;
      });
    }
    function LoadDetails(massmessage) {
      massmessage.details = { id: massmessage.id }
      API('webapp-api-admin?api=massmessagedetails', null, 'MassmessageDetails', { massmessageID: massmessage.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        //save the details
        angular.extend(massmessage.details, result.details);
        massmessage.effects = result.details.effects;
      }).catch(function (errors) {
        //couldn't load
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }

    var plainTextID = null;
    $scope.PlainID = function (id) {
      plainTextID = id;
    }
    $scope.PastePlain = function (container) {
      var plainTextArea = $('#' + plainTextID);
      if (!plainTextArea || !container.varString) return;
      plainTextArea.focus();
      plainTextArea.replaceSelectedText(container.varString, "collapseToEnd");
    }
    var htmlEditorID = null;
    $scope.HtmlID = function (id) {
      htmlEditorID = id;
    }
    $scope.PasteHtml = function (container) {
      if (!htmlEditorID || !container.varString) return;
      CKEDITOR.instances[htmlEditorID].insertHtml(container.varString, 'text');
    }
  }

  app.controller('ControllerAdminMasscriteriaMain', ['$scope', ControllerAdminMasscriteriaMain]);
  function ControllerAdminMasscriteriaMain($scope) {
    $scope.SetTitle('Mailing Criteria');
  }
  app.controller('ControllerAdminMasscriteria', ['$scope', '$timeout', '$route', '$location', 'iService.api.serial', 'iService.masscriterias', 'iService.contactsearchfields', 'iService.segments', ControllerAdminMasscriteria]);
  function ControllerAdminMasscriteria($scope, $timeout, $route, $location, API, masscriterias, contactsearchfields, segments) {
    $scope.itemsToggler = Toggler();
    $scope.masscriterias = masscriterias;
    $scope.contactsearchfields = contactsearchfields;
    $scope.segments = segments;
    NewDeleteDetailer($scope, $location, '/masscriteria', 'Masscriteria', 'admin-masscriteria-list.html', 'admin-masscriteria-new.html', 'admin-masscriteria-delete.html', 'admin-masscriteria-details.html', masscriterias.list, LoadDetails);

    $scope.Items = function () { return masscriterias.list(); }
    var columns = [
      { name: 'ID', var: 'id', visible: false },
      { name: 'Name', var: 'name', split: true },
      { name: 'Description', var: 'description', split: true },
      { name: 'Segments', var: 'segments', split: true }];
    var masscriteriasView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [masscriterias.Watch], columns: columns, startSort: 'name', exportName: 'Mass Mailing Criterias', pageSize: 5 });
    $scope.masscriteriasView = masscriteriasView;

    var initialID = +$route.current.params.masscriteriaID;
    if (isNaN(initialID) || !Number.isInteger(initialID) || initialID < 1) {
      initialID = undefined;
    }

    masscriterias.then(function () {
      if (initialID) masscriterias.list().some(function (item) {
        if (item.id != initialID) return false;
        $scope.ChangeState('show', item);
        return true;
      });
    }).catch(angular.noop);

    $scope.Save = function (idPrefix, masscriteria) {
      API('webapp-api-admin?api=masscriterias&mode=modify', idPrefix, 'MasscriteriaModify', { masscriteriaID: masscriteria.id, details: masscriteria.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //modify the row in the list.
        angular.extend(masscriteria, result.row);
        //save the details
        angular.extend(masscriteria.details, result.details);
        $scope.messages.saveSuccess = 'Criteria updated successfully.'
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      });
    }
    $scope.NewItem = function () {
      return { fields: [] };
    }
    $scope.Create = function (idPrefix) {
      API('webapp-api-admin?api=masscriterias&mode=new', idPrefix, 'MasscriteriaCreate', { details: $scope.newMasscriteria }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //add new row to the list.
        masscriterias.add(result.row);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'Criteria added successfully.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
    $scope.DeleteStart = function () {
      $scope.selectedMasscriteria.deleting = true;
      $scope.messages.Clear();
      API('webapp-api-admin?api=masscriteriaeffects', null, 'MasscriteriaEffects', { masscriteriaID: $scope.selectedMasscriteria.id }).then(function (result) {
        $scope.selectedMasscriteria.effects = result.effects;
      }).finally(angular.noop);
    }
    $scope.DeleteConfirm = function (idPrefix) {
      API('webapp-api-admin?api=masscriterias&mode=delete', idPrefix, 'MasscriteriaDelete', { masscriteriaID: $scope.selectedMasscriteria.id, replacetypeID: $scope.selectedMasscriteria.replaceTypeID }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //remove row from list
        masscriterias.remove(result.row);
        $scope.ChangeState('none');
        $scope.messages.deleteSuccess = 'Criteria deleted successfully.';
      }).catch(function (errors) {
        $scope.messages.deleteErrors = errors;
      });
    }
    function LoadDetails(masscriteria) {
      masscriteria.details = { id: masscriteria.id }
      API('webapp-api-admin?api=masscriteriadetails', null, 'MasscriteriaDetails', { masscriteriaID: masscriteria.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        //save the details
        angular.extend(masscriteria.details, result.details);
        masscriteria.effects = result.details.effects;
      }).catch(function (errors) {
        //couldn't load
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }
  }

  app.controller('ControllerAdminMasscampaignMain', ['$scope', ControllerAdminMasscampaignMain]);
  function ControllerAdminMasscampaignMain($scope) {
    $scope.SetTitle('Mailing Campaigns');
  }
  app.controller('ControllerAdminMasscampaign', ['$scope', '$route', '$location', '$timeout', '$q', 'iService.api.serial', 'iService.masscampaigns', 'iService.segments', 'iService.massmessages', 'iService.mailboxes', ControllerAdminMasscampaign]);
  function ControllerAdminMasscampaign($scope, $route, $location, $timeout, $q, API, masscampaigns, segments, massmessages, mailboxes) {
    $scope.itemsToggler = Toggler();
    $scope.masscampaigns = masscampaigns;
    $scope.massmessages = massmessages;
    $scope.mailboxes = mailboxes;
    $scope.segments = segments;
    $scope.items = [];
    NewDeleteDetailer($scope, $location, '/masscampaign', 'Masscampaign', 'admin-masscampaign-list.html', 'admin-masscampaign-new.html', 'admin-masscampaign-delete.html', 'admin-masscampaign-details.html', masscampaigns.list, LoadDetails);

    $scope.Items = function () { return masscampaigns.list(); }
    var columns = [
      { name: 'ID', var: 'id', visible: false },
      { name: 'Name', var: 'name', split: true },
      { name: 'Description', var: 'description', split: true },
      { name: 'Segments', var: 'segments', split: true },
      { name: 'Shown In My Account', var: 'shown' },
      { name: 'Subscribers', var: 'subCount' },
      { name: 'Unsubscribers', var: 'unsubCount' }];
    var masscampaignView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [masscampaigns.Watch], columns: columns, startSort: 'name', exportName: 'Mass Mailing Campaigns', pageSize: 5 });
    $scope.masscampaignView = masscampaignView;

    var initialID = +$route.current.params.masscampaignID;
    if (isNaN(initialID) || !Number.isInteger(initialID) || initialID < 1) {
      initialID = undefined;
    }

    masscampaigns.then(function () {
      $scope.items = masscampaigns.list();
      if (initialID) masscampaigns.list().some(function (item) {
        if (item.id != initialID) return false;
        $scope.ChangeState('show', item);
        return true;
      });
    }).catch(angular.noop);

    $scope.ShownSave = function (idPrefix, masscampaign) {
      API('webapp-api-admin?api=masscampaigndetails', idPrefix, 'MasscampaignDetails', { masscampaignID: masscampaign.id }).init(function () {
        masscampaign.shownRunning = true;
        $scope.messages.Clear();
      }).then(function (result) {
        result.details.shown = masscampaign.shown;
        return API('webapp-api-admin?api=masscampaigns&mode=modify', idPrefix, 'MasscampaignModify', { masscampaignID: masscampaign.id, details: result.details }).catch(function (errors) {
          if (errors && errors.length) errors[0].msg = 'Error modifying campaign. ' + errors[0].msg;
          return $q.reject(errors);
        });
      }).then(function (result) {
        $scope.messages.toggleSuccess = 'Mailing Campaign updated.'
        if (masscampaign.details) masscampaign.details.shown = masscampaign.shown;
      }).catch(function (errors) {
        $scope.messages.toggleErrors = errors;
      }).finally(function () {
        masscampaign.shownRunning = false;
      });
    }
    $scope.Save = function (idPrefix, masscampaign) {
      API('webapp-api-admin?api=masscampaigns&mode=modify', idPrefix, 'MasscampaignModify', { masscampaignID: masscampaign.id, details: masscampaign.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //modify the row in the list.
        angular.extend(masscampaign, result.row);
        //save the details
        angular.extend(masscampaign.details, result.details);
        $scope.messages.saveSuccess = 'Campaign updated successfully.'
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      });
    }
    $scope.Create = function (idPrefix) {
      API('webapp-api-admin?api=masscampaigns&mode=new', idPrefix, 'MasscampaignCreate', { details: $scope.newMasscampaign }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //add new row to the list.
        masscampaigns.add(result.row);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'Campaign added successfully.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
    $scope.DeleteStart = function () {
      $scope.selecteMasscampaign.deleting = true;
      $scope.messages.Clear();
      API('webapp-api-admin?api=masscampaigneffects', null, 'MasscampaignEffects', { masscampaignID: $scope.selecteMasscampaign.id }).then(function (result) {
        $scope.selecteMasscampaign.effects = result.effects;
      }).finally(angular.noop);
    }
    $scope.DeleteConfirm = function (idPrefix) {
      API('webapp-api-admin?api=masscampaigns&mode=delete', idPrefix, 'MasscampaignDelete', { masscampaignID: $scope.selecteMasscampaign.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //remove row from list
        masscampaigns.remove(result.row);
        $scope.ChangeState('none');
        $scope.messages.deleteSuccess = 'Campaign deleted successfully.';
      }).catch(function (errors) {
        $scope.messages.deleteErrors = errors;
      });
    }
    function LoadDetails(masscampaign) {
      masscampaign.details = { id: masscampaign.id }
      API('webapp-api-admin?api=masscampaigndetails', null, 'MasscampaignDetails', { masscampaignID: masscampaign.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        //save the details
        angular.extend(masscampaign.details, result.details);
        masscampaign.effects = result.details.effects;
      }).catch(function (errors) {
        //couldn't load
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }
    $scope.NewItem = function () {
      return { events: [] }
    }
    $scope.detailTab = 'details';
    $scope.ContentTabClass = function (tab) {
      if (tab == $scope.detailTab) return 'active';
      return '';
    }
    $scope.ContentTabChange = function (tab) {
      $scope.detailTab = tab;
    }
    $scope.ContentPaneShowing = function (tab) {
      return $scope.detailTab == tab;
    }

    $scope.EventViewUrl = function (event) {
      if (event.editing) return 'admin-masscampaign-info-events-edit.html';
      return 'admin-masscampaign-info-events-row.html';
    }
    $scope.NewEvent = function (details) {
      details.addEvent = true;
      details.newEvent = { dayOffset: 0 };
    }
    $scope.EditEvent = function (event) {
      event.editing = true;
      event.original = { dayOffset: event.dayOffset, messageID: event.messageID, mailboxID: event.mailboxID };
    }
    $scope.DeleteEvent = function (details, event) {
      details.events = details.events.filter(function (item) {
        return item !== event;
      });
    }
    $scope.SaveEvent = function (event) {
      event.editing = false;
    }
    $scope.CancelSaveEvent = function (event) {
      event.editing = false;
      angular.extend(event, event.original);
    }
    $scope.CreateEvent = function (details) {
      details.events.push(details.newEvent);
      details.addEvent = false;
    }
    $scope.CancelCreateEvent = function (details) {
      details.addEvent = false;
    }
  }
  app.controller('ControllerAdminMasscampaignMembers', ['$scope', '$timeout', 'iService.api.serial', ControllerAdminMasscampaignMembers]);
  function ControllerAdminMasscampaignMembers($scope, $timeout, API) {
    var masscampaign = $scope.masscampaign;
    var tab = $scope.detailTab;
    MessagesContainer($scope);

    var contactsUpdate = 0;
    $scope.Items = function () {
      if ($scope.mode == 'add' || $scope.mode == 'remove') return $scope.searchResults;
      return $scope.currentSubscribers;
    }
    var columns = [
      { name: 'ID', var: 'id', visible: false },
      { name: 'Name', var: 'name', split: true },
      { name: 'First Name', var: 'firstname', split: true, printFunc: PrinterPropertyValues() },
      { name: 'Last Name', var: 'lastname', split: true, printFunc: PrinterPropertyValues() },
      { name: 'Email', var: 'email', split: true },
      { name: 'Company', var: 'company', split: true },
      { name: 'Phone', var: 'phone', split: true }];
    var contactsView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [function () { return contactsUpdate; }], columns: columns, startSort: 'name', exportName: 'Contacts', pageSize: 5 });
    $scope.contactsView = contactsView;

    var listID = 'campaign-' + masscampaign.id;
    $scope.memberSearch = (tab == 'subs') ? iservice.searchfields.NewSearch('entire', 'subscribed', listID) : iservice.searchfields.NewSearch('entire', 'opted-out', listID);
    function DoSearch() {
      API('webapp-api-custinfo?api=search', null, 'ListMembers', { fields: { groups: $scope.memberSearch } }).init(function () {
        $scope.searching = true;
        $scope.currentSubscribers = [];
      }).then(function (result) {
        $scope.currentSubscribers = result.contacts;
        $scope.numContacts = result.resultCount;
        masscampaign.subCount = result.resultCount;
        contactsView.filterString = '';
        contactsUpdate++;
      }).catch(function (errors) {

      }).finally(function () {
        $scope.searching = false;
      });
    }
    DoSearch();

    $scope.mode = 'show';
    $scope.StartAdd = function () {
      $scope.mode = 'add';
      $scope.searchGroups = iservice.searchfields.NewSearch();
      $scope.searchResults = null;
      contactsUpdate++;
    }
    $scope.StartRemove = function () {
      $scope.mode = 'remove';
      $scope.searchGroups = iservice.searchfields.NewSearch();
      $scope.searchResults = null;
      contactsUpdate++;
    }
    $scope.StopModify = function () {
      $scope.mode = 'show';
      contactsUpdate++;
    }

    $scope.ContactSearchClick = function () {
      var search = { groups: [] };
      if ($scope.mode == 'add') {
        $scope.searchGroups.forEach(function (group) { search.groups.push(group); });
        iservice.searchfields.NewFieldGroup(search.groups, 'entire', 'subscribed-not', listID);
      }
      else if ($scope.mode == 'remove') {
        iservice.searchfields.NewFieldGroup(search.groups, 'entire', 'subscribed', listID);
        $scope.searchGroups.forEach(function (group) { search.groups.push(group); });
      }
      API('webapp-api-custinfo?api=search', null, 'NewMembers', { fields: search }).init(function () {
        $scope.searching = true;
        $scope.searchResults = [];
        contactsUpdate++;
      }).then(function (result) {
        $scope.searchResults = result.contacts;
        contactsUpdate++;
      }).catch(function () {

      }).finally(function () {
        $scope.searching = false;
      });
    }
    function DoAdd(idPrefix, IDs) {
      API('webapp-api-admin?api=masssubscriptions&mode=add', idPrefix, 'MassSubscribe', { contactIDs: IDs, masscampaignID: masscampaign.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        $scope.messages.saveSuccess = 'Added contacts to the campaign.';
        $scope.StopModify();
        DoSearch();
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      }).finally(function () {

      });
    }
    $scope.SubscribeSelectedClick = function (idPrefix) {
      DoAdd(idPrefix, contactsView.selectedIDs);
    }
    $scope.SubscribeAllClick = function (idPrefix) {
      DoAdd(idPrefix, contactsView.filteredItems.map(function (item) { return item.id; }));
    }
    function DoRemove(idPrefix, IDs) {
      API('webapp-api-admin?api=masssubscriptions&mode=remove', idPrefix, 'MassUnsubscribe', { contactIDs: IDs, masscampaignID: masscampaign.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        $scope.messages.saveSuccess = 'Removed contacts from the campaign.';
        $scope.StopModify();
        DoSearch();
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      }).finally(function () {

      });
    }
    $scope.RemoveSelectedClick = function (idPrefix) {
      DoRemove(idPrefix, contactsView.selectedIDs);
    }
    $scope.RemoveAllClick = function (idPrefix) {
      DoRemove(idPrefix, contactsView.filteredItems.map(function (item) { return item.id; }));
    }
  }

  app.controller('ControllerAdminServicelevelsMain', ['$scope', '$interval', 'iService.rootPath', 'iService.servicelevels', '$location', '$timeout', '$route', 'iService.intProperties', 'iService.contactProperties', 'iService.api.serial', ControllerAdminServicelevelsMain]);
  function ControllerAdminServicelevelsMain($scope, $interval, rootPath, servicelevels, $location, $timeout, $route, intProperties, contactProperties, API) {
    $scope.SetTitle('Service Levels');

    $scope.itemsToggler = Toggler();
    $scope.servicelevels = servicelevels;
    $scope.items = [];
    NewDeleteDetailer($scope, $location, '/servicelevel', 'ServiceLevels', 'admin-servicelevel.html', 'admin-servicelevel-new.html', 'admin-servicelevel-delete.html', 'admin-servicelevel-details.html', servicelevels.list, LoadDetails);
    servicelevels.then(function () {

      servicelevels.list().forEach(function (item) {
        item.propType = printPropertyType(item);
        item.propValue = printPropertyValue(item);
        item.propName = '';
        if (item.contactPropertyID) {
          contactProperties.then(function () {          
            item.propName = contactProperties.get(item.contactPropertyID).option;
          });
          
        }
        if (item.intPropertyID) {
          intProperties.then(function () {           
            item.propName = intProperties.get(item.intPropertyID).option;
          });

        }
      });
      $scope.Items = function () { return servicelevels.list(); }
           
      var columns = [
        { name: 'Id', var: 'id', visible: true },
        { name: 'Name', var: 'name', visible: true },
        { name: 'Minutes', var: 'minutes', visible: true },
        { name: 'Property Type', var: 'propType', visible: true },
        { name: 'Property Name', var: 'propName', visible: true },
        { name: 'Property Value', var: 'propValue', visible: true }
      ];
      var serviceLevelView = iservice.TableViewer($scope, $timeout,
        { itemsFunc: $scope.Items, watchFuncs: [servicelevels.Watch], columns: columns, startSort: 'id', exportName: 'Service Levels', pageSize: 5 });

      $scope.serviceLevelView = serviceLevelView;

    });
    $scope.intProps = '';
    $scope.contactProps = '';
    intProperties.then(function () {
      $scope.intProps = intProperties.list();
    });
    contactProperties.then(function () {
      $scope.contactProps = contactProperties.list();
    });
    function printPropertyType(row) {
      if (row) {
        if (row.contactPropertyID) return "Contact";
        if (row.intPropertyID) return "Interaction";
      }
      return "";
    }
    
    function printPropertyValue(row) {
      if (row) {
        if (row.contactPropertyID) return row.contactPropertyValue;
        if (row.intPropertyID) return row.intPropertyValue;      
      }
      return "";
    }

    var initialID = +$route.current.params.servicelevelID;
    if (isNaN(initialID) || !Number.isInteger(initialID) || initialID < 1) {
      initialID = undefined;
    }

    servicelevels.then(function () {
      if (initialID) servicelevels.list().some(function (servicelevel) {
        if (servicelevel.id != initialID) return false;
        $scope.ChangeState('show', servicelevel);
        return true;
      });
    }).catch(angular.noop);

    $scope.Save = function (idPrefix, servicelevel) {
      var error = false;
      var field = [];
      if (!servicelevel.name) {
        field.push("name");
        error = true;
      }
      if (!servicelevel.minutes) {
        field.push("minutes");
        error = true;
      }
      if (!error) {
        var details = {
          contactPropertyID: servicelevel.propType ? servicelevel.contactPropertyID : "",
          contactPropertyValue: servicelevel.propType ? servicelevel.contactPropertyValue: "",
          id: servicelevel.id,
          intPropertyID: servicelevel.propType ? servicelevel.intPropertyID: "",
          intPropertyValue: servicelevel.propType ? servicelevel.intPropertyValue: "",
          minutes: servicelevel.minutes,
          name: servicelevel.name
        }
        API('webapp-api-admin?api=servicelevels&mode=modify', idPrefix, 'ServiceLevelModify', { servicelevelID: servicelevel.id, details: details }).init(function () {
          $scope.messages.Clear();
        }).then(function (result) {
          $scope.messages.saveSuccess = 'Service Level updated successfully.'
        }).catch(function (errors) {
          $scope.messages.saveErrors = errors;
        });
      } else {
        $scope.messages.saveErrors = ['Service level ' + field.join(', ')+' is required.'];
      }
      
    }
    $scope.NewItem = function () {
      return { name: '', minutes: '', propType: '', propName: '', propValue: '' };
    }
    $scope.newServiceLevel = { name: '', minutes: '', propType: '', propName: '', propValue: '' };
    $scope.Create = function (idPrefix) {
      var error = false;
      var field = [];
      if (!$scope.newServiceLevel.name) {
        field.push("name");
        error = true;
      }
      if (!$scope.newServiceLevel.minutes) {
         field.push("minutes");
        error = true;
      }
      if (!error) {
        var details = {
          contactPropertyID: $scope.newServiceLevel.contactPropertyID && $scope.newServiceLevel.propType ? $scope.newServiceLevel.contactPropertyID : "",
          contactPropertyValue: $scope.newServiceLevel.contactPropertyValue && $scope.newServiceLevel.propType ? $scope.newServiceLevel.contactPropertyValue : "",
          id: $scope.newServiceLevel.id,
          intPropertyID: $scope.newServiceLevel.intPropertyID && $scope.newServiceLevel.propType ? $scope.newServiceLevel.intPropertyID : "",
          intPropertyValue: $scope.newServiceLevel.intPropertyValue && $scope.newServiceLevel.propType ? $scope.newServiceLevel.intPropertyValue : "",
          minutes: parseInt($scope.newServiceLevel.minutes),
          name: $scope.newServiceLevel.name
        }
        API('webapp-api-admin?api=servicelevels&mode=new', idPrefix, 'ServiceLevelCreate', { details: details }).init(function () {
          $scope.messages.Clear();
        }).then(function (result) {
          //add new row to the list.
          result.row.propType = printPropertyType(result.row);
          result.row.propValue = printPropertyValue(result.row);
          result.row.propName = '';
          if (result.row.contactPropertyID) {
            contactProperties.then(function () {
              result.row.propName = contactProperties.get(result.row.contactPropertyID).option;
            });
          }
          if (result.row.intPropertyID) {
            intProperties.then(function () {
              result.row.propName = intProperties.get(result.row.intPropertyID).option;
            });
          }
          servicelevels.add(result.row);
          $scope.ChangeState('show', result.row);
          $scope.messages.createSuccess = 'Service Level added successfully.';
        }).catch(function (errors) {
          $scope.messages.createErrors = errors;
        });
      } else {
        $scope.messages.createErrors = ['Service level ' + field.join(', ') + ' is required.'];
      }
    }
    $scope.DeleteStart = function () {
      $scope.messages.Clear();
      $scope.selectedServiceLevels.deleting = true;
    }
    $scope.DeleteConfirm = function (idPrefix) {
      API('webapp-api-admin?api=servicelevels&mode=delete', idPrefix, 'ServiceLevelDelete', { servicelevelID: $scope.selectedServiceLevels.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //remove row from list
        servicelevels.remove(result.row);
        $scope.ChangeState('none');
        $scope.messages.deleteSuccess = 'Service Level deleted successfully.';
      }).catch(function (errors) {
        $scope.messages.deleteErrors = errors;
      });
    }
    function LoadDetails(servicelevel) {
      servicelevel.details = { id: servicelevel.id }
      API('webapp-api-admin?api=serviceleveldetails', null, 'ServiceLevelDetails', { servicelevelID: servicelevel.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {       
        //save the details
          result.details.propType = printPropertyType(result.details);
          result.details.propValue = printPropertyValue(result.details);
          result.details.propName = '';
          if (result.details.contactPropertyID) {
              contactProperties.then(function () {
                  result.details.propName = contactProperties.get(result.details.contactPropertyID).option;
              });

          }
          if (result.details.intPropertyID) {
              intProperties.then(function () {
                  result.details.propName = intProperties.get(result.details.intPropertyID).option;
              });
          }
        angular.extend(servicelevel.details, result.details);
        
      }).catch(function (errors) {
        //couldn't load
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }

  }

  app.controller('ControllerAdminFormsMain', ['$scope', ControllerAdminFormsMain]);
  function ControllerAdminFormsMain($scope) {
    $scope.SetTitle('Forms');
    $scope.formsGroup = 'form';
  }
  app.controller('ControllerAdminFormsMainBuiltin', ['$scope', ControllerAdminFormsMainBuiltin]);
  function ControllerAdminFormsMainBuiltin($scope) {
    $scope.SetTitle('Built-in Forms');
    $scope.formsGroup = 'formbuiltin';
  }
  app.controller('ControllerAdminForms', ['$scope', '$route', '$location', 'iService.api.serial', 'iService.forms', 'iService.formsbuiltin', '$q', '$timeout', ControllerAdminForms]);
  function ControllerAdminForms($scope, $route, $location, API, forms, formsbuiltin, $q, $timeout) {
    var userMode = $scope.formsGroup == 'form';
    var currentForms = userMode ? forms : formsbuiltin;
    $scope.currentForms = currentForms;
    $scope.itemsToggler = Toggler();
    NewDeleteDetailer($scope, $location, '/' + $scope.formsGroup, 'Form', 'admin-' + $scope.formsGroup + '-list.html', 'admin-form-new.html', 'admin-form-delete.html', 'admin-form-details.html', currentForms.list, LoadDetails);

    $scope.Items = function () { return currentForms.list(); }
    var columns = [];
    if (userMode) columns.push({ name: 'ID', var: 'id', visible: true });
    columns.push({ name: 'Name', var: 'name', split: true });
    columns.push({ name: 'Description', var: 'description', split: true });
    if (userMode) columns.push({ name: 'Modifier', var: 'modifierName', split: true });
    if (userMode) columns.push({ name: 'Modify Time', var: 'modifyTime', split: true, printFunc: PrintUtcDate, exportType: 'timestamp', exportFunc: function (date) { return date; } });
    var formsView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [currentForms.Watch], columns: columns, startSort: 'name', exportName: userMode ? 'Forms' : 'Built In Forms', pageSize: 5 });
    $scope.formsView = formsView;

    var initialID = $route.current.params.formID;
    if (userMode) {
      initialID = +initialID;
      if (isNaN(initialID) || !Number.isInteger(initialID) || initialID < 1) {
        initialID = undefined;
      }
    }

    currentForms.then(function () {
      if (initialID) currentForms.list().some(function (form) {
        if (form.id != initialID) return false;
        $scope.ChangeState('show', form);
        return true;
      });
    });

    $scope.ChangeGroup = function (group) {
      if (group == $scope.formsGroup) return;
      $location.path('/' + group);
    }
    $scope.GroupButtonClass = function (group) {
      return $scope.formsGroup == group ? 'active btn-primary' : 'btn-default';
    }
    $scope.Save = function (idPrefix, form) {
      API('webapp-api-admin?api=forms&mode=modify', idPrefix, 'FormModify', { formID: form.id, details: form.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //modify the row in the list.
        angular.extend(form, result.row);
        //save the details
        angular.extend(form.details, result.details);
        $scope.messages.saveSuccess = 'Form updated successfully.'
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      });
    }
    $scope.Preview = function (formid) {

      window.open($location.protocol() + "://" + $location.host() + ":" + $location.port() + '/f/' + formid);
    };
    $scope.FormActionsUrl = function () {
      if (userMode) return 'admin-form-buttons.html';
      return '';
    }
    $scope.NewItem = function () {
      return { actions: [] };
    }
    $scope.Create = function (idPrefix) {
      API('webapp-api-admin?api=forms&mode=new', idPrefix, 'FormCreate', { details: $scope.newForm }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //add new row to the list.
        forms.add(result.row);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'Form added successfully.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
    $scope.DeleteStart = function () {
      $scope.selectedForm.deleting = true;
      $scope.messages.Clear();
    }
    $scope.DeleteConfirm = function (idPrefix) {
      var form = $scope.selectedForm;
      API('webapp-api-admin?api=forms&mode=delete', idPrefix, 'FormDelete', { formID: form.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //remove row from list
        forms.remove(result.row);
        $scope.ChangeState('none');
        $scope.messages.deleteSuccess = 'Form deleted successfully.';
      }).catch(function (errors) {
        $scope.messages.deleteErrors = errors;
      });
    }
    $scope.FormWizardUrl = function () {
      if (!$scope.wizard) return '';
      return 'admin-form-wizard.html';
    }
    function LoadDetails(form) {
      form.details = { id: form.id }
      API('webapp-api-admin?api=formdetails', null, 'FormDetails', { formID: form.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        //save the details
        angular.extend(form.details, result.details);
      }).catch(function (errors) {
        //couldn't load
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }
    var DefChangeState = $scope.ChangeState;
    $scope.ChangeState = function (state, showForm) {
      if (state == 'wizard') {
        $scope.wizard = {};
        $scope.newForm = null;
        $scope.selectedForm = null;
        //$location.path('/' + $scope.formsGroup, false);
      }
      else {
        $scope.wizard = null;
        DefChangeState(state, showForm);
      }
    }
  }
  app.controller('ControllerAdminFormsDetails', ['$scope', 'iService.contactProperties', 'iService.intProperties', ControllerAdminFormsDetails]);
  function ControllerAdminFormsDetails($scope, contactProperties, intProperties) {
    $scope.variableFilter = '';
    $scope.variableToggle = Toggler();
    $scope.ActionDeleteStart = function (action) {
      action.deleting = true;
    }
    $scope.ActionDeleteConfirm = function (actions, action) {
      actions.remove(action);
    }
    $scope.ActionDeleteCancel = function (action) {
      action.deleting = false;
    }
    $scope.AddAction = function (actions) {
      actions.push({
        checkInputID: null,
        checkValue: null,
        submitAction: {
          steps: []
        }
      });
    }

    var plainTextArea = null;
    $scope.PasteTarget = function (id) {
      plainTextArea = $('#' + id);
    }
    $scope.PasteVariable = function (text) {
      if (!plainTextArea) return;
      plainTextArea.focus();
      plainTextArea.replaceSelectedText(text, "collapseToEnd");
    }

    var groupContact = {
      name: 'Contact',
      items: [
        { text: 'Email Address', v: "$input -email -id'enteruniquenamehere'$",
          subs: [
            { text: 'Confirm', v: "$input -email -id'enteruniquenamehere' -confirm'uniquenameofwhatthisconfirms'$" },
            { text: 'Self List', v: "$input -email(self) -id'enteruniquenamehere'$"}]
        },
        { text: 'Password', v: "$input -password -id'enteruniquenamehere'$",
          subs: [
            { text: 'Confirm', v: "$input -password -id'enteruniquenamehere' -confirm'uniquenameofwhatthisconfirms'$"}]
        }]
    };
    var groupInt = {
      name: 'Interaction',
      items: []
    };
    $scope.variableGroups = [
      { name: 'Basic',
        items: [
          { text: 'Any Field', v: "$input -id'enteruniquenamehere'$" },
          { text: 'Any Text Area', v: "$input -textarea -id'enteruniquenamehere'$" },
          { text: 'Form Login', v: "$loginname -id'enterniquenamehere'$" },
          { text: 'Form Password', v: "$loginpassword -id'enterniquenamehere'$" },
          { text: 'Error Message', v: "$errormessage$" },
          { text: 'Form ID', v: "$value -formID$" }
        ]
      },
      groupContact,
      groupInt,
      { name: 'Conditionals',
        items: [
          { text: 'Logged In', v: "$if -loggedin$\r\n$endif$\r\n",
            subs: [
              { text: 'With Else', v: "$if -loggedin$\r\n$else$\r\n$endif$\r\n"}]
          },
          { text: 'Post', v: "$if -ispostback$\r\n$endif$\r\n",
            subs: [
              { text: 'With Else', v: "$if -ispostback$\r\n$else$\r\n$endif$\r\n"}]
          },
          { text: 'Submit Success', v: "$if -submitsuccess$\r\n$endif$\r\n",
            subs: [
              { text: 'With Else', v: "$if -submitsuccess$\r\n$else$\r\n$endif$\r\n"}]
          }]
      },
      { name: 'Logged In',
        items: [
          { text: 'User Name', v: "$value -loggedin(name)$" },
          { text: 'User Contact ID', v: "$value -loggedin(contactid)$" },
          { text: 'Session ID', v: "$value -loggedin(sessionID)$"}]
      },
      { name: 'My Account',
        items: [
          { text: 'Mailing Lists', v: "$if -myaccountlists$\r\n$repeat -myaccountlists$\r\n  $value -myaccountlist(id)$\r\n  $value -myaccountlist(name)$\r\n  $value -myaccountlist(description)$\r\n  $input -id'enteruniquenamehere' -myaccount(list)$\r\n$endrepeat$\r\n$else$\r\nNo Mailing Lists\r\n$endif$\r\n" },
          { text: 'Campaigns', v: "$if -myaccountcampaigns$\r\n$repeat -myaccountcampaigns$\r\n  $value -myaccountcampaign(id)$\r\n  $value -myaccountcampaign(name)$\r\n  $value -myaccountcampaign(description)$\r\n  $input -id'enteruniquenamehere' -myaccount(campaign)$\r\n$endrepeat$\r\n$else$\r\nNo Campaigns\r\n$endif$\r\n" },
          { text: 'Articles', v: "$if -myaccountarticles$\r\n$repeat -myaccountarticles$\r\n  $value -myaccountarticle(id)$\r\n  $value -myaccountarticle(subject)$\r\n  $value -myaccountarticle(topic)$\r\n  $input -id'enteruniquenamehere' -myaccount(article)$\r\n$endrepeat$\r\n$else$\r\nNo Article Subscriptions\r\n$endif$\r\n"}]
      },
      { name: 'Ask A Question',
        items: [
          { text: 'Public Topics', v: "$input -id'enteruniquenamehere' -topics$"}]
      },
      { name: 'Find Answers',
        items: [
          { text: 'Topics - Flat', v: "$repeat -topics(findanswer)$\r\n  $value -topic(id)$\r\n  $value -topic(parentid)$\r\n  $value -topic(name)$\r\n  $value -topic(messagecount)$\r\n  $value -topic(segmentid)$\r\n  $value -topic(segmentname)$\r\n  $value -topic(visibility)$\r\n$endrepeat$\r\n" },
          { text: 'Topics - Tree', v: "$repeat -topictree(findanswer)$\r\n  $value -topic(id)$\r\n  $value -topic(parentid)$\r\n  $value -topic(name)$\r\n  $value -topic(messagecount)$\r\n  $value -topic(segmentid)$\r\n  $value -topic(segmentname)$\r\n  $value -topic(visibility)$\r\n  $if -topictree(findanswer)$\r\n    $repeat -topictree(findanswer)$\r\n      $value -topic(id)$\r\n      $value -topic(parentid)$\r\n      $value -topic(name)$\r\n      $value -topic(messagecount)$\r\n      $value -topic(segmentid)$\r\n      $value -topic(segmentname)$\r\n      $value -topic(visibility)$\r\n    $endrepeat$\r\n  $endif$\r\n$endrepeat$\r\n" },
          { text: 'Article List For Topic', v: "$if -articlelist(topic) -Ptopic'nameofinputwithtopicID' -Ppagenum'1' -Pperpage'100' -Psort'DATE_REVERSE'$\r\n$repeat -articlelist(topic) -Ptopic'nameofinputwithtopicID' -Ppagenum'1' -Pperpage'100' -Psort'DATE_REVERSE'$\r\n  $value -articlelist(id)$\r\n  $value -articlelist(subject)$\r\n  $value -articlelist(date)$\r\n  $value -articlelist(topicid)$\r\n  $value -articlelist(topicname)$\r\n  $value -articlelist(rating)$\r\n  $value -articlelist(viewcount)$\r\n  $value -articlelist(public)$\r\n$endrepeat$\r\n$else$\r\nNo Articles\r\n$endif$\r\n" },
          { text: 'Article Search Results In Topic', v: "$if -articlelist(search) -Ptopic'nameofinputwithtopicID' -Psearchid'nameofinputwithsearchterms' -Ppagenum'1' -Pperpage'100' -Psort'DATE_REVERSE'$\r\n$repeat -articlelist(search) -Ptopic'nameofinputwithtopicID' -Psearchid'nameofinputwithsearchterms' -Ppagenum'1' -Pperpage'100' -Psort'DATE_REVERSE'$\r\n  $value -articlelist(id)$\r\n  $value -articlelist(subject)$\r\n  $value -articlelist(date)$\r\n  $value -articlelist(topicid)$\r\n  $value -articlelist(topicname)$\r\n  $value -articlelist(rating)$\r\n  $value -articlelist(viewcount)$\r\n  $value -articlelist(public)$\r\n$endrepeat$\r\n$else$\r\nNo Articles\r\n$endif$\r\n" },
          { text: 'Article Data', v: "$if -article -Pid'nameofinputwitharticleID'$\r\n  $value -article(id) -Pid'nameofinputwitharticleID'$\r\n  $value -article(subject)$\r\n  $value -article(date)$\r\n  $value -article(question)$\r\n  $value -article(answer)$\r\n  $value -article(public)$\r\n  $value -article(segmentid)$\r\n  $value -article(segmentname)$\r\n  $value -article(topicid)$\r\n  $value -article(topicname)$\r\n  $value -article(viewcount)$\r\n  $value -article(rating)$\r\n  $value -article(myrating)$\r\n  $value -article(creatorname)$\r\n  $if -attachments -Pid'nameofinputwitharticleID'$\r\n    $repeat -attachments -Pid'nameofinputwitharticleID'$\r\n      $value -attachment(id)$\r\n      $value -attachment(ownerid)$\r\n      $value -attachment(name)$\r\n    $endrepeat$\r\n  $else$\r\n    No attachments.\r\n  $endif$\r\n$else$\r\nNo Article Found\r\n$endif$\r\n"}]
      }];
    var segmentID = '0';
    contactProperties.finally(function () {
      contactProperties.list().forEach(function (prop) {
        var item = { text: prop.name, v: "$input -id'enteruniquenamehere' -contactProperty" + prop.id + "'" + prop.name.replace(/\W/g, '') + "'$" };
        if (segmentID != prop.segmentID) {
          item.segmentName = prop.segmentName;
          segmentID = prop.segmentID;
        }
        if (prop.hasDescriptions) item.subs = [
          { text: 'Desc', v: "$input -contactProperty" + prop.id + "'" + prop.name.replace(/\W/g, '') + "' -id'enteruniquenamehere' -description'uniquenameofwhatthisdescribes'$" }];
        groupContact.items.push(item);
      });
    });
    segmentID = '0';
    intProperties.finally(function () {
      intProperties.list().forEach(function (prop) {
        var item = {
          text: prop.name, v: "$input -id'enteruniquenamehere' -interactionProperty" + prop.id + "'" + prop.name.replace(/\W/g, '') + "'$", purpose: prop.purpose
        };
        if (segmentID != prop.segmentID) {
          item.segmentName = prop.segmentName;
          segmentID = prop.segmentID;
        }
        if (prop.hasDescriptions) item.subs = [
          { text: 'Desc', v: "$input -interactionProperty" + prop.id + "'" + prop.name.replace(/\W/g, '') + "' -id'enteruniquenamehere' -description'uniquenameofwhatthisdescribes'$" }];
        groupInt.items.push(item);
      });
    });
  }
  app.controller('ControllerAdminFormsWizard', ['$scope', 'iService.api.serial', 'iService.topics', 'iService.contactProperties', 'iService.intProperties', 'iService.forms', ControllerAdminFormsWizard]);
  function ControllerAdminFormsWizard($scope, API, topics, contactProperties, intProperties, forms) {
    $scope.topics = topics;
    var wizard = $scope.wizard;
    wizard.details = {};
    wizard.step = 1;
    var names = {};
    var MakeName = function (name) {
      var base = name.replace(/\W/g, '');
      var end = 0;
      name = base;
      while (names[name]) {
        end++;
        name = base + end;
      }
      names[name] = true;
      return name;
    }
    function MakePropertyRow(prop, FlagProperty) {
      var id = MakeName(prop.name),
          idConfirm = MakeName(id + 'confirm');
      var flagTextarea = (prop.isMultiLine) ? ' -textarea' : '';
      function FlagID(id) {
        return " -id'" + id + "'";
      };
      function FlagConfirm(id) {
        return " -confirm'" + id + "'";
      };
      var row = {
        name: htmlEscape(prop.name),
        optional: true,
        multiline: prop.isMultiLine,
        purpose: prop.purpose,
        input: '$input{req}' + FlagProperty(prop, id) + FlagID(id) + flagTextarea + '$',
        inputConfirm: '$input' + FlagProperty(prop, id) + FlagID(idConfirm) + FlagConfirm(id) + flagTextarea + '$'
      }; if (prop.hasDescriptions) {
        var idDescribe = MakeName(id + 'describe'), idDescribeConfirm = MakeName(id + 'describeconfirm'); function FlagDescribe(id) { return " -description'" + id + "'"; }; row.describe = true; row.inputDescribe = '$input' + FlagProperty(prop, id) + FlagID(idDescribe) + FlagDescribe(id) + '$'; row.inputDescribeConfirm = '$input' + FlagProperty(prop, id) + FlagID(idDescribeConfirm) + FlagConfirm(idDescribe) + FlagDescribe(idConfirm) + '$';
      } return row;
    }
    contactProperties.finally(function () {
      wizard.contactProperties = [];
      wizard.contactProperties.push({ name: 'Email Address', selected: true, required: true, optional: false, describe: false, input: "$input -email -id'email'{req}$", inputConfirm: "$input -email -id'emailconfirm' -confirm'email'$" });
      wizard.contactProperties.push({ name: 'Password', optional: true, describe: false, input: "$input -password -id'password'{req}$", inputConfirm: "$input -password -id'passwordconfirm' -confirm'password'$" });
      contactProperties.list().forEach(function (prop) {
        var row = MakePropertyRow(prop, function (prop, id) { return ' -contactProperty' + prop.id + "'" + id + "'"; });
        wizard.contactProperties.push(row);
      });
    });
    intProperties.finally(function () {
      wizard.intProperties = [];
      wizard.intProperties.push({ name: 'Subject', selected: true, required: true, optional: false, describe: false, unconfirm: true, input: "$input -id'subject'{req}$" });
      wizard.intProperties.push({ name: 'Body', selected: true, required: true, optional: false, describe: false, unconfirm: true, input: "$input -id'body'{req} -textarea$" });

      intProperties.list().forEach(function (prop) {
        if (prop.isEditable) {
          var row = MakePropertyRow(prop, function (prop, id) { return ' -interactionProperty' + prop.id + "'" + id + "'"; });
          wizard.intProperties.push(row);
        }
      });
    });
    $scope.WizStepClass = function (num) {
      if (num == wizard.step) return 'btn-primary';
      return 'btn-default';
    }
    $scope.WizardBodyUrl = function () {
      if (wizard.step == 1) return 'admin-form-wizard-body-type.html';
      if (wizard.step == 2 && wizard.type == 'contact') return 'admin-form-wizard-body-contact.html';
      if (wizard.step == 2 && wizard.type == 'ticket') return 'admin-form-wizard-body-ticket-topic.html';
      if (wizard.step == 3) return 'admin-form-wizard-body-ticket-contactproperties.html';
      if (wizard.step == 4) return 'admin-form-wizard-body-ticket-intproperties.html';
      return '';
    }
    $scope.WizBodyPrefix = function () {
      if (wizard.step == 1) return 'Step1';
      if (wizard.step == 2 && wizard.type == 'contact') return 'Step2Contact';
      if (wizard.step == 2 && wizard.type == 'ticket') return 'Step2Ticket';
      if (wizard.step == 3) return 'Step3Ticket';
      if (wizard.step == 4) return 'Step4Ticket';
      return 'Step';
    }
    $scope.TypeNext = function () {
      if (!wizard.type) return;
      if (!wizard.details.name) return;
      wizard.step = 2;
    }
    $scope.Back = function () {
      wizard.step = wizard.step - 1;
    }
    $scope.TicketTopicNext = function () {
      if (!wizard.topicID) return;
      wizard.step = 3;
    }
    $scope.TicketContactNext = function () {
      wizard.step = 4;
    }
    function WritePropertyHtml(prop) {
      var body = '      ' + prop.name + ': ';
      if (prop.describe) body += prop.inputDescribe;
      if (prop.multiline) body += '<br>\r\n      ';
      body += prop.input.replace('{req}', (prop.required ? ' -required' : '')) + '<br>\r\n';
      if (!prop.unconfirm && prop.confirm) {
        body += '      Confirm ' + prop.name + ': ';
        if (prop.describe) body += prop.inputDescribeConfirm;
        if (prop.multiline) body += '<br>\r\n      ';
        body += prop.inputConfirm + '<br>\r\n';
      } return body;
    }
    $scope.ContactFinish = function (idPrefix) {
      var body = '<html>\r\n  <body>\r\n    <form method="POST">\r\n';
      wizard.contactProperties.forEach(function (prop) {
        if (!prop.selected) return;
        body += WritePropertyHtml(prop);
      });
      body += '      <input type="submit" id="ok" name="ok" value="Create/Update Contact" /><br>\r\n';
      body += '      <div style="background-color:yellow;">$errormessage$</div>\r\n';
      body += '    </form>\r\n  </body>\r\n</html>';
      wizard.details.body = body;
      wizard.details.actions = [{ submitAction: { steps: [{ contactFindCreate: { emailSearch: "$form -id'email'$", ignoreBlankFields: true, resultName: 'contactID' } }] } }];
      API('webapp-api-admin?api=forms&mode=new', idPrefix, 'FormContactWizard', { details: wizard.details }).init(function () {
        $scope.errors = [];
      }).then(function (result) {
        //add new row to the list.
        forms.add(result.row);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'Form added successfully.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
    $scope.TicketFinish = function (idPrefix) {
      var body = '<html>\r\n  <body>\r\n    <form method="POST">\r\n';
      wizard.contactProperties.forEach(function (prop) {
        if (!prop.selected) return;
        body += WritePropertyHtml(prop);
      });
      wizard.intProperties.forEach(function (prop) {
        if (!prop.selected) return;
        body += WritePropertyHtml(prop);
      });
      body += '      <input type="submit" id="ok" name="ok" value="Submit Question" /><br>\r\n';
      body += '      <div style="background-color:yellow;">$errormessage$</div>\r\n';
      body += '    </form>\r\n  </body>\r\n</html>';
      wizard.details.body = body;
      wizard.details.actions = [{ submitAction: { steps: 
        [{ contactFindCreate: { emailSearch: "$form -id'email'$", ignoreBlankFields: true, resultName: 'contactID' } }, 
          { ticket: { isTicketNotAskAQuestion: false, contactID: "$result -id'contactID'$", emailAddress: "$form -id'email'$", topicID: '' + wizard.topicID, subject: "$form -id'subject'$", bodyHtml: "$form -id'body'$", resultName: 'interactionID' } }] } }];
      API('webapp-api-admin?api=forms&mode=new', idPrefix, 'FormTicketWizard', { details: wizard.details }).init(function () {
        $scope.errors = [];
      }).then(function (result) {
        //add new row to the list.
        forms.add(result.row);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'Form added successfully.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
  }

  app.controller('ControllerAdminSMTPOut', ['$scope', '$interval', 'iService.rootPath', '$timeout', ControllerAdminSMTPOut]);
  function ControllerAdminSMTPOut($scope, $interval, rootPath, $timeout) {
    $scope.SetTitle('SMTP Out Monitor');
    $scope.iframe = IFrameContainer($interval, $scope.idPrefix + '_IFrame');
    $scope.iframe.src = rootPath + 'Admin.aspx?admin=smtpout';
  }

  app.controller('ControllerAdminOpenID', ['$scope', 'iService.api.serial', ControllerAdminOpenID]);
  function ControllerAdminOpenID($scope, API) {
    $scope.SetTitle('Open ID Settings');
    MessagesContainer($scope);

    API('webapp-api-admin?api=openiddetails', $scope.idPrefix, 'OpenIDDetails', { }).init(function () {
      $scope.messages.Clear();
      $scope.details = {};
    }).then(function (result) {
      $scope.details = result;
    }).finally(function () {
    });

    $scope.SaveClick = function (idPrefix) {
      API('webapp-api-admin?api=openidset', idPrefix, 'OpenIDSave', { details: $scope.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        $scope.details = result;
        $scope.messages.saveSuccess = 'Saved successfully.';
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      }).finally(function () {
      });
    };

  }

  app.controller('ControllerAdminSegmentIntproperty', ['$scope', '$location', '$timeout', 'iService.intProperties', '$q', 'iService.api.serial', 'iService.propertyTypes', ControllerAdminSegmentIntproperty]);
  function ControllerAdminSegmentIntproperty($scope, $location, $timeout, intProperties, $q, API, propertyTypes) {
    $scope.loadingProperties = true;
   
    $q.all([intProperties.promise(), propertyTypes.promise()]).then(function () {

      $scope.Items = function () { return (intProperties).segment($scope.routeSegmentID) }

      NewDeleteDetailer($scope, $location, '/segment/' + $scope.routeSegmentID + '/intproperty', 'intProperty', 'admin-segment-intproperty-list.html', 'admin-segment-intproperty-new.html', 'admin-segment-intproperty-delete.html', 'admin-segment-intproperty-details.html', intProperties.list, LoadDetails);
     
      var columns = [
        { name: 'ID', var: 'id', visible: true },
        { name: 'Name', var: 'name', split: true },
        { name: 'Type', var: 'purpose', split: true },
        { name: 'Value Type', var: 'type', split: true, printFunc: printListName },
        { name: 'Required', var: 'isRequired', split: true },
        { name: 'Shared', var: 'isShared', split: true },
        { name: 'Multi Line', var: 'isMultiLine', split: true, visible: false },
        { name: 'Allow Multiple', var: 'allowMultiple', split: true, visible: false },
        { name: 'Has Description', var: 'hasDescriptions', split: true, visible: false },
        { name: 'Customer Viewable', var: 'isViewable', split: true, visible: false },
        { name: 'Customer Editable', var: 'isEditable', split: true, visible: false }
      ];

      $scope.propertytypes = [
        { name: 'Answer', value: 'Answer' },
        { name: 'Case', value: 'Case' },
        { name: 'Question', value: 'Question' },
        { name: 'Knowledge Base', value: 'Knowledge Base' }
      ];
      function printListName(type, row) {
        var listName = '';
        if (type) {
          for (var i = 0; i < propertyTypes.list().length; i++) {
            if (propertyTypes.list()[i].typeID == type) {
              listName = propertyTypes.list()[i].name;
              break;
            }
          }
        }
        return listName;
      }
      
      $scope.types = propertyTypes.list();
      
      var intPropertyView = iservice.TableViewer($scope, $timeout,
        { itemsFunc: $scope.Items, watchFuncs: [intProperties.Watch], columns: columns, startSort: 'name', exportName: 'IntProperties', pageSize: 5 });
      
      $scope.intPropertyView = intPropertyView;
      $scope.showMultiLine = false
      $scope.toggleMultiLine = function (details) {
        if (details.type != 'text') {
          details.isMultiLine = false;
          $scope.showMultiLine = false
        } else {
          $scope.showMultiLine = true
          console.log($scope.showMultiLine)
        }
      }
      $scope.loadingProperties = false;
      $scope.Save = function (idPrefix, intproperty) {
        if (!intproperty) return;
        intproperty.isOwnerViewable = intproperty.isViewable;
        intproperty.isOwnerEditable = intproperty.isEditable;
        API('webapp-api-admin?api=intproperties&mode=modify', idPrefix, 'intPropertyModify', { propertyID: intproperty.id, details: intproperty }).init(function () {
          $scope.saveRunning = true;
          $scope.messages.Clear();
        }).then(function (result) {
          //modify the row in the list.
          angular.extend(intproperty, result.row);
          //save the details
          //  angular.extend(intproperty, result.details);
          $scope.messages.saveSuccess = 'Interaction Property updated successfully.'
        }).catch(function (errors) {
          $scope.messages.saveErrors = errors;
        }).finally(function () {
          $scope.saveRunning = false;
        });
      }
      $scope.Create = function (idPrefix, details) {
        var error = false;
        var emptyFields = [];
        if (!details.type) {
          error = true;
          emptyFields.push("Value Type");
        }
        if (!details.name) {
          error = true;
          emptyFields.push("Property Name");
        }
        if (!details.purpose) {
          error = true;
          emptyFields.push("Property Type");
        }
        if (error == false) {
          details.segmentID = $scope.routeSegmentID;
          details.isOwnerViewable = details.isViewable;
          details.isOwnerEditable = details.isEditable;
          API('webapp-api-admin?api=intproperties&mode=new', idPrefix, 'intPropertyCreate', { details: details }).init(function () {
            $scope.messages.Clear();
          }).then(function (result) {
            //add new row to the list.
            intProperties.add(result.row, $scope.routeSegmentID);
            $scope.messages.createSuccess = 'Interaction Property added successfully.';
            $scope.ChangeState('show', result.row);
          }).catch(function (errors) {
            $scope.messages.createErrors = errors;
          }).finally(function () {
            $scope.messages.createSuccess = 'Interaction Property added successfully.';
          });
        } else {
          $scope.messages.createErrors = ["You are missing required fields (" + emptyFields.join(', ')+")"];
        }
        
      }
      $scope.EnableRequired = function (idPrefix, intproperty) {
        API('webapp-api-admin?api=intpropertiesdetails', null, 'IntPropertyDetails', { propertyID: intproperty.id }).init(function () {
          intproperty.enableRunning = true;
          $scope.messages.Clear();
        }).then(function (result) {
          result.details.isRequired = intproperty.isRequired;
          return API('webapp-api-admin?api=intproperties&mode=modify', idPrefix, 'intPropertyModify', { propertyID: intproperty.id, details: result.details }).then(function (response) {

          });
        }).then(function (result) {
          $scope.messages.toggleSuccess = 'Interaction Property updated.'
          if (intproperty.details) intproperty.details.isRequired = intproperty.isRequired;
        }).catch(function (errors) {
          $scope.messages.toggleErrors = errors;
        }).finally(function () {
          intproperty.enableRunning = false;
        });
      }
      $scope.EnableToggle = function (idPrefix, intproperty, action) {
        API('webapp-api-admin?api=intpropertiesdetails', null, 'IntPropertyDetails', { propertyID: intproperty.id }).init(function () {
          intproperty.enableRunning = true;
          $scope.messages.Clear();
        }).then(function (result) {
          
          if (action == 'shared')
            result.details.isShared = intproperty.isShared;
          if (action == 'multiline')
            result.details.isMultiLine = intproperty.isMultiLine;
          if (action == 'allowmultiple')
            result.details.allowMultiple = intproperty.allowMultiple;
          if (action == 'description')
            result.details.hasDescriptions = intproperty.hasDescriptions;
          if (action == 'viewable')
            result.details.isViewable = intproperty.isViewable;
          if (action == 'editable')
            result.details.isEditable = intproperty.isEditable;
          return API('webapp-api-admin?api=intproperties&mode=modify', idPrefix, 'intPropertyModify', { propertyID: intproperty.id, details: result.details }).then(function (response) {
            
          });
        }).then(function (result) {
          $scope.messages.toggleSuccess = 'Interaction Property updated.'
          if (intproperty.details) intproperty.details.isShared = intproperty.isShared;
          if (intproperty.details && action == 'shared') intproperty.details.isShared = intproperty.isShared;
          if (intproperty.details && action == 'multiline') intproperty.details.isMultiLine = intproperty.isMultiLine;
          if (intproperty.details && action == 'allowmultiple') intproperty.details.allowMultiple = intproperty.allowMultiple;
          if (intproperty.details && action == 'description') intproperty.details.hasDescriptions = intproperty.hasDescriptions;
          if (intproperty.details && action == 'viewable') intproperty.details.isViewable = intproperty.isViewable;
          if (intproperty.details && action == 'editable') intproperty.details.isEditable = intproperty.isEditable;
        }).catch(function (errors) {
          $scope.messages.toggleErrors = errors;
        }).finally(function () {
          intproperty.enableRunning = false;
        });
      }
      $scope.DeleteStart = function () {
        $scope.selectedintProperty.deleting = true;
        $scope.messages.Clear();
      }
      $scope.DeleteConfirm = function (idPrefix) {
        API('webapp-api-admin?api=intproperties&mode=delete', idPrefix, 'intPropertyDelete', { propertyID: $scope.selectedintProperty.id }).init(function () {
          $scope.messages.Clear();
        }).then(function (result) {
          //remove row from list
          intProperties.remove(result.details);
          $scope.ChangeState('none');
          $scope.messages.deleteSuccess = 'Interaction Property deleted successfully.';
        }).catch(function (errors) {
          $scope.messages.deleteErrors = errors;
        });
      }
      function LoadDetails(intProperty) {
        intProperty.details = { id: intProperty.id }
        API('webapp-api-admin?api=intpropertiesdetails', null, 'IntPropertyDetails', { propertyID: intProperty.id }).init(function () {
          $scope.loading = true;
        }).then(function (result) {
         // LoadResponses(result.details);
          result.details.isViewable = result.details.isOwnerViewable;
          result.details.isEditable = result.details.isOwnerEditable;
          angular.extend(intProperty.details, result.details);
          API('webapp-api-admin?api=intpropertyeffects', null, 'IntPropertyEffects', { propertyID: intProperty.id }).init(function () {
            $scope.loading = true;
          }).then(function (result) {
            intProperty.details.effects = result.effects;
          }).catch(function (errors) {

          }).finally(function () {
        
          });
      
        }).catch(function (errors) {
          //couldn't load
          $scope.ChangeState('none');
        }).finally(function () {
          $scope.loading = false;
        });
      }

      if ($scope.routeIntpropID) {
        var property = intProperties.get($scope.routeIntpropID);
        if (property) {
          $scope.ChangeState('show', property);
          return;
        }
        $scope.ChangeState('none');
      }
    });
  }

  app.controller('ControllerAdminSegmentContactproperty', ['$scope', '$location', '$interval', '$timeout', 'iService.contactProperties', 'iService.rootPath', '$q', 'iService.api.serial', 'iService.propertyTypes', ControllerAdminSegmentContactproperty]);
  function ControllerAdminSegmentContactproperty($scope, $location, $interval, $timeout, contactProperties, rootPath, $q, API, propertyTypes) {
    $scope.loadingProperties = true;
    $q.all([contactProperties.promise(), propertyTypes.promise()]).then(function () {
      $scope.Items = function () { return (contactProperties).segment($scope.routeSegmentID) }

      NewDeleteDetailer($scope, $location, '/segment/' + $scope.routeSegmentID + '/contactproperty', 'contactProperty', 'admin-segment-contactproperty-list.html', 'admin-segment-contactproperty-new.html', 'admin-segment-contactproperty-delete.html', 'admin-segment-contactproperty-details.html', contactProperties.list, LoadDetails);

      var columns = [
        { name: 'ID', var: 'id', visible: true },
        { name: 'Name', var: 'name', split: true },
        { name: 'Type', var: 'purpose', split: true },
        { name: 'Value Type', var: 'type', split: true, printFunc: printListName },
        { name: 'Required', var: 'isRequired', split: true },
        { name: 'Shared', var: 'isShared', split: true },
        { name: 'Multi Line', var: 'isMultiLine', split: true, visible: false },
        { name: 'Allow Multiple', var: 'allowMultiple', split: true, visible: false },
        { name: 'Has Description', var: 'hasDescriptions', split: true, visible: false },
        { name: 'Customer Viewable', var: 'isViewable', split: true, visible: false },
        { name: 'Customer Editable', var: 'isEditable', split: true, visible: false }
      ];

      $scope.propertytypes = [
        { name: 'Agent', value: 'Agent' },
        { name: 'Contact', value: 'Contact' }
      ];
      function printListName(type, row) {
        var listName = '';
        if (type) {
          for (var i = 0; i < propertyTypes.list().length; i++) {
            if (propertyTypes.list()[i].typeID == type) {
              listName = propertyTypes.list()[i].name;
              break;
            }
          }
        }
        return listName;
      }

      $scope.types = propertyTypes.list();

      var contactPropertyView = iservice.TableViewer($scope, $timeout,
        { itemsFunc: $scope.Items, watchFuncs: [contactProperties.Watch], columns: columns, startSort: 'name', exportName: 'ContactProperties', pageSize: 5 });

      $scope.contactPropertyView = contactPropertyView;
      $scope.showMultiLine = false
      $scope.toggleMultiLine = function (details) {
        if (details.type != 'text') {
          details.isMultiLine = false;
          $scope.showMultiLine = false
        } else {
          $scope.showMultiLine = true
        }
      }
      $scope.loadingProperties = false;
      $scope.Save = function (idPrefix, contactproperty) {
        if (!contactproperty) return;
        contactproperty.isOwnerViewable = contactproperty.isViewable;
        contactproperty.isOwnerEditable = contactproperty.isEditable;
        API('webapp-api-admin?api=contactproperties&mode=modify', idPrefix, 'contactPropertyModify', { propertyID: contactproperty.id, details: contactproperty }).init(function () {
          $scope.saveRunning = true;
          $scope.messages.Clear();
        }).then(function (result) {
          //modify the row in the list.
          angular.extend(contactproperty, result.row);
          $scope.messages.saveSuccess = 'Contact Property updated successfully.'
        }).catch(function (errors) {
          $scope.messages.saveErrors = errors;
        }).finally(function () {
          $scope.saveRunning = false;
        });
      }
      $scope.Create = function (idPrefix, details) {
        var error = false;
        var emptyFields = [];
        if (!details.type) {
          error = true;
          emptyFields.push("Value Type");
        }
        if (!details.name) {
          error = true;
          emptyFields.push("Property Name");
        }
        if (!details.purpose) {
          error = true;
          emptyFields.push("Property Type");
        }
        if (error == false) {
          details.segmentID = $scope.routeSegmentID;
          details.isOwnerViewable = details.isViewable;
          details.isOwnerEditable = details.isEditable;
          API('webapp-api-admin?api=contactproperties&mode=new', idPrefix, 'contactPropertyCreate', { details: details }).init(function () {
            $scope.messages.Clear();
          }).then(function (result) {
            contactProperties.add(result.row, $scope.routeSegmentID);
            $scope.messages.createSuccess = 'Contact Property added successfully.';
            $scope.ChangeState('show', result.row);
          }).catch(function (errors) {
            $scope.messages.createErrors = errors;
          }).finally(function () {
            $scope.messages.createSuccess = 'Contact Property added successfully.';
          });
        } else {
          $scope.messages.createErrors = ["You are missing required fields (" + emptyFields.join(', ') + ")"];
        }

      }

      $scope.EnableToggle = function (idPrefix, contactproperty, action) {
        API('webapp-api-admin?api=contactpropertiesdetails', null, 'ContactPropertyDetails', { propertyID: contactproperty.id }).init(function () {
          contactproperty.enableRunning = true;
          $scope.messages.Clear();
        }).then(function (result) {
          if(action == 'shared')
            result.details.isShared = contactproperty.isShared;
          if (action == 'multiline')
            result.details.isMultiLine = contactproperty.isMultiLine;
          if (action == 'allowmultiple')
            result.details.allowMultiple = contactproperty.allowMultiple;
          if (action == 'description')
            result.details.hasDescriptions = contactproperty.hasDescriptions;
          if (action == 'viewable')
            result.details.isViewable = contactproperty.isViewable;
          if (action == 'editable')
            result.details.isEditable = contactproperty.isEditable;
          return API('webapp-api-admin?api=contactproperties&mode=modify', idPrefix, 'contactPropertyModify', { propertyID: contactproperty.id, details: result.details }).then(function (response) {

          });
        }).then(function (result) {
          $scope.messages.toggleSuccess = 'Contact Property updated.'
          if (contactproperty.details && action == 'shared') contactproperty.details.isShared = contactproperty.isShared;
          if (contactproperty.details && action == 'multiline') contactproperty.details.isMultiLine = contactproperty.isMultiLine;
          if (contactproperty.details && action == 'allowmultiple') contactproperty.details.allowMultiple = contactproperty.allowMultiple;
          if (contactproperty.details && action == 'description') contactproperty.details.hasDescriptions = contactproperty.hasDescriptions;
          if (contactproperty.details && action == 'viewable') contactproperty.details.isViewable = contactproperty.isViewable;
          if (contactproperty.details && action == 'editable') contactproperty.details.isEditable = contactproperty.isEditable;
        }).catch(function (errors) {
          $scope.messages.toggleErrors = errors;
        }).finally(function () {
          contactproperty.enableRunning = false;
        });
      }
      $scope.DeleteStart = function () {
        $scope.selectedcontactProperty.deleting = true;
        $scope.messages.Clear();
      }
      $scope.DeleteConfirm = function (idPrefix) {
        API('webapp-api-admin?api=contactproperties&mode=delete', idPrefix, 'contactPropertyDelete', { propertyID: $scope.selectedcontactProperty.id }).init(function () {
          $scope.messages.Clear();
        }).then(function (result) {
          //remove row from list
          contactProperties.remove(result.details);
          $scope.ChangeState('none');
          $scope.messages.deleteSuccess = 'Contact Property deleted successfully.';
        }).catch(function (errors) {
          $scope.messages.deleteErrors = errors;
        });
      }
      function LoadDetails(contactProperty) {
        contactProperty.details = { id: contactProperty.id }
        API('webapp-api-admin?api=contactpropertiesdetails', null, 'ContactPropertyDetails', { propertyID: contactProperty.id }).init(function () {
          $scope.loading = true;
        }).then(function (result) {
          result.details.isViewable = result.details.isOwnerViewable;
          result.details.isEditable = result.details.isOwnerEditable;
          angular.extend(contactProperty.details, result.details);
          API('webapp-api-admin?api=contactpropertyeffects', null, 'ContactPropertyEffects', { propertyID: contactProperty.id }).init(function () {
            $scope.loading = true;
          }).then(function (result) {
            contactProperty.details.effects = result.effects;
          }).catch(function (errors) {

          }).finally(function () {

          });
            
        }).catch(function (errors) {
          //couldn't load
          //$scope.ChangeState('none');
        }).finally(function () {
          $scope.loading = false;
        });
      }

      if ($scope.routeContactpropID) {
        var property = contactProperties.get($scope.routeContactpropID);
        if (property) {
          $scope.ChangeState('show', property);
          return;
        }
        $scope.ChangeState('none');
      }
    });
  }

  app.controller('ControllerAdminSegmentNotification', ['$scope', '$interval', 'iService.rootPath', '$location', '$timeout', ControllerAdminSegmentNotification]);
  function ControllerAdminSegmentNotification($scope, $interval, rootPath, $location, $timeout) {
    $scope.iframe = IFrameContainer($interval, $scope.idPrefix + '_IFrame');
    $scope.iframe.src = rootPath + 'Admin.aspx?admin=segment&segmentID=' + $scope.routeSegmentID + '&segtab=auto';
  }

  app.controller('ControllerAdminSegmentTopic', ['$scope', '$rootScope', '$timeout', '$location', '$q', 'iService.api.serial', 'iService.rootPath', 'iService.topics', 'iService.filters', 'iService.autoResponses', 'iService.servicelevels', 'iService.skills', 'iService.intProperties', 'iService.segments', 'iService.topicsDeleted', '$uibModal', ControllerAdminSegmentTopic]);
  function ControllerAdminSegmentTopic($scope, $rootScope, $timeout, $location, $q, API, rootPath, topics, filters, autoResponses, servicelevels, skills, intProperties, segments, topicsDeleted, $uibModal) {
    $scope.loadingTopics = true;
    var showDeleted = false;
    function IsShowDeleted() {
      return showDeleted;
    }
    $scope.IsShowDeleted = IsShowDeleted;
    Object.defineProperty($scope, 'showDeleted', {
      set: function (show) { showDeleted = show; },
      get: function () { return showDeleted; }
    });
    $q.all([topics.promise(), topicsDeleted.promise(), filters.promise(), autoResponses.promise(), servicelevels.promise(), intProperties.promise()]).then(function () {
      $scope.Items = function () { return (showDeleted ? topicsDeleted : topics).segment($scope.routeSegmentID); }
      var columns = [
        { name: 'ID', var: 'id', visible: false },
        { name: 'Topic Name', var: 'name', split: true, printFunc: PrintTopicName },
        { name: 'Parent Name', var: 'parentID', split: true, printFunc: PrintTopicParentName(topics) },
        { name: 'Visibility', var: 'visibility', split: true },
        { name: 'Skills', var: 'skillNames', split: true },
        { name: 'Properties', var: 'propertyNames', split: true },
        { name: 'Customer Ticket Filter', var: 'askAQuestionFilterID', split: true, printFunc: PrintAAQName, visible: false },
        { name: 'Message Acknowledgment Notification', var: 'autoResponseID', split: true, printFunc: PrintAutoResponse, visible: false },
        { name: 'Agent Notification', var: 'agentNotifyID', split: true, printFunc: PrintAgentNotification, visible: false },
        { name: 'Service Level', var: 'serviceLevelID', split: true, printFunc: PrintServiceLevel, visible: false }
      ];

      NewDeleteDetailer($scope, $location, '/segment/' + $scope.routeSegmentID + '/topic', 'Topic', 'admin-segment-topic-list.html', 'admin-segment-topic-new.html', 'admin-segment-topic-delete.html', 'admin-segment-topic-details.html', topics.list, LoadDetails);
      var listUpdate = 0;
      var topicsView = iservice.TableViewer($scope, $timeout,
        { itemsFunc: $scope.Items, watchFuncs: [topics.Watch, topicsDeleted.Watch, function () { return listUpdate; }], columns: columns, startSort: 'name', exportName: 'Topics', pageSize: 5 });
      $scope.topicsView = topicsView;

      $scope.loadingTopics = false;
     
      function PrintTopicName(name, topic) {
        if (name) {
          //remove segment name
          name = topic.name.split('--');
          if (name.length > 1) {
            return name[1];
          } else {
            return name[0];
          }
        }
        return '';
      }
      function PrintTopicParentName(objService) {
        return function (objID) {
          var obj = objService.get(objID);
          if (!obj) return '';
          var obj = obj.name.split('--');
          if (obj.length > 1) {
            return obj[1];
          } else {
            return obj[0];
          }
        }
      }
      function PrintAAQName(id, topic) {
          var name = filters.get(id);
          if (name) {
            return name.name;
          }
      }
      function PrintAutoResponse(id, topic) {
        var name = autoResponses.get(id);
        if (name) {
          return name.name;
        }
      }
      function PrintAgentNotification(id, topic) {
        var name = autoResponses.get(id);
        if (name) {
          return name.name;
        }
      }
      function PrintServiceLevel(id, topic) {
        var name = servicelevels.get(id);
        if (name) {
          return name.name;
        }
        return "";
      }
      
      $scope.topics = topics;
      $scope.filters = filters;
      $scope.autoResponses = autoResponses;
      $scope.servicelevels = servicelevels;
      $scope.skills = skills;
      $scope.intProperties = intProperties;
      $scope.filterSkills = function (item) {
        return item.propertyID == "" && item.propertyValue == "";
      };

      $scope.NewItem = function () {
        return {
          visibility: 'private',
          autoResponseID: '',
          neverAutoResponse: true,
          agentNotifyID: '',
          neverAgentNotify: true,
          askAQuestionFilterID: '',
          serviceLevelID: '',
          skills: [],
          properties: []
        }
      }
      var topicsData = (showDeleted ? topicsDeleted : topics).segment($scope.routeSegmentID);
     
      $scope.topicList = [];
      $scope.rootTopics = [];
      var isRoot = {};
      var topicByID = {};
      var curChild = null;
      ForArray(topicsData, function (topic) {
        topicByID[topic.id] = topic;
        var parent = topicByID[topic.parentID];
        if (!parent) {
          isRoot[topic.id] = true;
          topic.name = topic.option;
          $scope.rootTopics.push(topic);
          topic.children = [];
        }
        else if (isRoot[parent.id]) {
          parent.children.push(topic);
          topic.children = [];
          curChild = topic;
        }
        else {
          curChild.children.push(topic);
        }
      });

      $scope.isPrivate = function (topicID) {
        if (topicID) {
          var topic = topics.get(topicID);
          if (topic && topic.visibility == 'private') {
            return true;
          }
        }
        return false;
      }
      $scope.openTreePreviewModal = function () {
        $rootScope.modalInstance = $uibModal.open({
          animation: true,
          ariaLabelledBy: 'modal-title',
          ariaDescribedBy: 'modal-body',
          backdrop: true,
          templateUrl: 'treeview.html',
          controller: function ($scope, $uibModalInstance) {
            $scope.topicView = function (topic) {
              $scope.ItemListSelect(topic);
              $uibModalInstance.dismiss();
            }
          },
          scope: $scope
        });
        $rootScope.modalInstance.result.then(function () {

        });

      }
      $scope.openPreviewModal = function () {
        $uibModal.open({
          animation: true,
          ariaLabelledBy: 'modal-title',
          ariaDescribedBy: 'modal-body2',
          backdrop: true,
          templateUrl: 'preview.html',
          controller: function ($scope) {

          },
          scope: $scope
        });
      }
      $scope.levels = [{ id: '', name: '-- Segment Default --', option: '-- Segment Default --' }].concat(servicelevels.list());
      $scope.otherSegments = [];
      $scope.responsesAck = [{ id: '', name: '-- None --' }, { id: 'seg', name: '-- Segment Default --' }];
      $scope.responsesAgent = [{ id: '', name: '-- None --' }, { id: 'seg', name: '-- Segment Default --' }];
      segments.list().forEach(function (segment) {
        if (segment.id == $scope.routeSegmentID) return;
        segment.sharedProperties = [];
        intProperties.segment(segment.id).forEach(function (property) {
          if (property.isShared) segment.sharedProperties.push(property);
        });
        if (segment.sharedProperties.length < 1) return;
        $scope.otherSegments.push(segment);
      });

      autoResponses.segment($scope.routeSegmentID).some(function (response) {
        response.lists.some(function (list) {
          list = list.toLowerCase();
          if (list == 'topicautoresponse') $scope.responsesAck.push(response);
          if (list == 'agentnotification') $scope.responsesAgent.push(response);
        });
      });

      $scope.ToggleDeleted = function () {
        listUpdate++;
      }
      function LoadResponses(details) {
        if (details.autoResponseID == '' && details.neverAutoResponse == false) details.autoResponseID = 'seg';
        if (details.agentNotifyID == '' && details.neverAgentNotify == false) details.agentNotifyID = 'seg';
      }
      function StoreResponses(details) {
        details.neverAutoResponse = details.autoResponseID == '';
        if (details.autoResponseID == 'seg') details.autoResponseID = '';
        details.neverAgentNotify = details.agentNotifyID == '';
        if (details.agentNotifyID == 'seg') details.agentNotifyID = '';
      }
      $scope.Save = function (idPrefix, topic) {
        if (!topic) return;
        StoreResponses(topic.details);
        API('webapp-api-admin?api=topics&mode=modify', idPrefix, 'TopicModify', { topicID: topic.id, details: topic.details }).init(function () {
          $scope.saveRunning = true;
          $scope.messages.Clear();
        }).then(function (result) {
          angular.extend(topic, result.row);
          angular.extend(topic.details, result.details);
          $scope.messages.saveSuccess = 'Topic updated successfully.'
          if (topic.visibility == 'deleted') {
            if (topics.get(topic.id)) {
              topics.remove({ id: topic.id, isDeleted: true });
              topicsDeleted.add(topic);
            }
            showDeleted = true;
          }
          else {
            if (topicsDeleted.get(topic.id)) {
              topicsDeleted.remove({ id: topic.id, isDeleted: true });
              topics.add(topic);
            }
            showDeleted = false;
          }
          listUpdate++;
        }).catch(function (errors) {
          $scope.messages.saveErrors = errors;
        }).finally(function () {
          LoadResponses(topic.details);
          $scope.saveRunning = false;
        });
      }
      if ($scope.routeTopicID) {
        var topic = topics.get($scope.routeTopicID);
        if (topic) {
          $scope.ChangeState('show', topic);
          showDeleted = false;
          return;
        }
        topic = topicsDeleted.get($scope.routeTopicID);
        if (topic) {
          $scope.ChangeState('show', topic);
          showDeleted = true;
          return;
        }
        $scope.ChangeState('none');
      }

      $scope.Create = function (idPrefix, details) {
        details.segmentID = $scope.routeSegmentID;
        StoreResponses(details);
        API('webapp-api-admin?api=topics&mode=new', idPrefix, 'TopicCreate', { details: details }).init(function () {
          $scope.messages.Clear();
        }).then(function (result) {
          //add new row to the list.
          topics.add(result.row, $scope.routeSegmentID);
          $scope.ChangeState('show', result.row);
          $scope.messages.createSuccess = 'Topic added successfully.';
        }).catch(function (errors) {
          $scope.messages.createErrors = errors;
        });
      }
      function LoadDetails(topic) {
        topic.details = { id: topic.id }
        API('webapp-api-admin?api=topicdetails', null, 'TopicDetails', { topicID: topic.id }).init(function () {
          $scope.loading = true;
        }).then(function (result) {
          LoadResponses(result.details);
          angular.extend(topic.details, result.details);
          topic.effects = result.details.effects;
        }).catch(function (errors) {
          //couldn't load
          $scope.ChangeState('none');
        }).finally(function () {
          $scope.loading = false;
        });
      }
      
    });
  }

  app.controller('ControllerAdminSegmentFilter', ['$scope', '$interval', 'iService.rootPath', ControllerAdminSegmentFilter]);
  function ControllerAdminSegmentFilter($scope, $interval, rootPath) {
    $scope.iframe = IFrameContainer($interval, $scope.idPrefix + '_IFrame');
    $scope.iframe.src = rootPath + 'Admin.aspx?admin=segment&segmentID=' + $scope.routeSegmentID + '&segtab=filter';
  }

  app.controller('ControllerAdminSegmentsMain', ['$scope','$rootScope', '$route', '$location', 'iService.rootPath', 'iService.segments', ControllerAdminSegmentsMain]);
  function ControllerAdminSegmentsMain($scope, $rootScope, $route, $location, rootPath, segments) {
    $scope.segments = segments;
    $scope.routeSegmentID = RouteID($route.current.params.segmentID);
    $scope.routeIntpropID = RouteID($route.current.params.intpropertyID);
    $scope.routeContactpropID = RouteID($route.current.params.contactpropertyID);
    $scope.routeMailboxID = RouteID($route.current.params.mailboxID);
    $scope.routeResponseID = RouteID($route.current.params.notificationID);
    $scope.routeStockID = RouteID($route.current.params.stockID);
    $scope.routeTopicID = RouteID($route.current.params.topicID);
    $scope.routeFilterID = RouteID($route.current.params.filterID);
    $rootScope.defaultMailboxMissing = null;
    $rootScope.defaultNotificationMissing = null;
    if ($scope.routeSegmentID) segments.then(function (data) {
      $scope.routeSegment = segments.get($scope.routeSegmentID);
    });
    $scope.Heading = function () {
      if ($scope.routeSegment) return $scope.routeSegment.name + ' Settings';
      return 'Segment Settings';
    }

    $scope.routeTab = $route.current.params.tab;
    if ($scope.routeIntpropID) $scope.routeTab = 'intproperty';
    if ($scope.routeContactpropID) $scope.routeTab = 'contactproperty';
    if ($scope.routeMailboxID) {
      $scope.routeTab = 'mailbox';
      $scope.SetTitle('Mailbox');
    }
    if ($scope.routeResponseID) $scope.routeTab = 'notification';
    if ($scope.routeStockID) {
      $scope.routeTab = 'stock';
      $scope.SetTitle('Stock Response');
    }
    if ($scope.routeTopicID) $scope.routeTab = 'topic';
    if ($scope.routeFilterID) $scope.routeTab = 'filter';
    if ($scope.routeTab != 'intproperty' && $scope.routeTab != 'contactproperty' && $scope.routeTab != 'mailbox' && $scope.routeTab != 'notification' && $scope.routeTab != 'stock' && $scope.routeTab != 'topic' && $scope.routeTab != 'filter') {
      $scope.routeTab = 'details';
      $scope.SetTitle('Segment');
    }
    $scope.TabClass = function (tab) {
      if (tab == $scope.routeTab) return 'active';
      return '';
    }
    $scope.TabClick = function (tab) {
      if (tab != 'details' && tab != 'stock') return;
      $scope.routeTab = tab;
      var tabPath = $scope.routeTab == 'details' ? '' : ('/' + $scope.routeTab);
      $location.path('/segment/' + $scope.routeSegmentID + tabPath, false);
    }
    $scope.ItemDetailsUrl = function (segment) {
      if (segment != $scope.routeSegment) return '';
      if ($scope.routeTab == 'details') return 'admin-segment-details.html';
      if ($scope.routeTab == 'stock') return rootPath + 'f/admin-segment-stock';
      if ($scope.routeTab == 'mailbox') return rootPath + 'f/admin-segment-mailbox';
      if ($scope.routeTab == 'intproperty') return rootPath + 'f/admin-segment-intproperty';
      if ($scope.routeTab == 'contactproperty') return rootPath + 'f/admin-segment-contactproperty';
      if ($scope.routeTab == 'notification') return rootPath + 'f/admin-segment-notification';
      if ($scope.routeTab == 'topic') return rootPath + 'f/admin-segment-topic';
      if ($scope.routeTab == 'filter') return rootPath + 'f/admin-segment-filter';
    }
  }
  app.controller('ControllerAdminSegments', ['$scope','$rootScope', '$location', 'iService.api.serial', 'iService.segments', 'iService.mailboxes', 'iService.servicelevels', 'iService.autoResponses', ControllerAdminSegments]);
  function ControllerAdminSegments($scope, $rootScope, $location, API, segments, mailboxes, servicelevels, autoresponses) {
    $scope.segments = segments;
    $scope.itemsToggler = Toggler();
    $scope.items = [];
    $scope.mailboxes = mailboxes;
    $scope.servicelevels = servicelevels;
    $scope.responsesAck = [{ id: '', name: '-- None --'}];
    $scope.responsesCreate = [{ id: '', name: '-- None --' }];
    $scope.responsesAgent = [{ id: '', name: '-- None --' }];
    $scope.responsesArticle = [{ id: '', name: '-- None --' }];
    $scope.parentSegments = [{ id: '1', name: '-- None --'}];
    MessagesContainer($scope);

    function GetValidParents(segment) {
      var all = segments.list();
      $scope.parentSegments = [{ id: '1', name: '-- None --' }];
      var parentForID = {};
      segments.list().forEach(function (segment) {
        parentForID[segment.id] = segment.parentID;
      });
      var invalidParents = {};
      invalidParents[segment.id] = true;
      segments.list().forEach(function (segment) {
        var id = segment.id;
        while (id) {
          if (invalidParents[id]) break;
          id = parentForID[id];
        }
        if (id) invalidParents[segment.id] = true;
      });
      segments.list().forEach(function (segment) {
        if (segment.id == '1' || invalidParents[segment.id]) return;
        $scope.parentSegments.push(segment);
      });
    }
    segments.then(function (data) {
      LoadDetails($scope.routeSegment);
      GetValidParents($scope.routeSegment);
    }).catch(angular.noop);

    autoresponses.then(function (data) {
      autoresponses.segment($scope.routeSegment.id).some(function (response) {
        response.lists.some(function (list) {
          list = list.toLowerCase();
          if (list == 'accountcreation') $scope.responsesCreate.push(response);
          if (list == 'topicautoresponse') $scope.responsesAck.push(response);
          if (list == 'agentnotification') $scope.responsesAgent.push(response);
          if (list == 'articlenotification') $scope.responsesArticle.push(response);
        });
      });
    }).catch(angular.noop);

    $scope.Save = function (idPrefix, segment) {
      API('webapp-api-admin?api=segments&mode=modify', idPrefix, 'SegmentModify', { segmentID: segment.id, details: segment.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //modify the row in the list.
        angular.extend(segment, result.row);
        //save the details
        angular.extend(segment.details, result.details);
        GetValidParents(segment);
        segments.Reparent();
        $scope.messages.saveSuccess = 'Segment updated successfully.';
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      });
    }

    function LoadDetails(segment) {
      segment.details = { id: segment.id }
      API('webapp-api-admin?api=segmentdetails', null, 'SegmentDetails', { segmentID: segment.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        if (!result.details.mailboxID) {
          $rootScope.defaultMailboxMissing = "Please setup your default mailbox.";
        }
        if (!result.details.responseIDAck && !result.details.responseIDArticle && !result.details.responseIDCreate && !result.details.responseIDNotify) {
          $rootScope.defaultNotificationMissing = 'Please setup your default notification.';
        }
        //save the details
        angular.extend(segment.details, result.details);
      }).catch(function (errors) {
        //couldn't load
        //$scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }
  }
  app.controller('ControllerAdminSegmentStocks', ['$scope', '$location', '$timeout', 'iService.api.serial', 'iService.stockresponses', 'iService.intProperties', ControllerAdminSegmentStocks]);
  function ControllerAdminSegmentStocks($scope, $location, $timeout, API, stockresponses, intProperties) {
    $scope.items = [];
    $scope.stockData = [{
      name: 'Segment Responses',
      responses: ''
    }, {
        name: 'Agent Responses',
        responses: []
    }];
    $scope.pasteEvent = IServiceEvent();
    $scope.stockresponses = stockresponses;
    NewDeleteDetailer($scope, $location, '/segment/' + $scope.routeSegmentID + '/stock', 'Stock', 'admin-segment-stock-list.html', 'admin-segment-stock-new.html', 'admin-segment-stock-delete.html', 'admin-segment-stock-details.html', stockresponses.list, LoadDetails);

    $scope.Items = function () { return stockresponses.segment($scope.routeSegmentID); }
    var columns = [
      { name: 'Id', var: 'id', visible: true },
      { name: 'Name', var: 'name', visible: true, split: true },
      { name: 'Description', var: 'description', visible: true, split: true },
    ];
    var stockresponsesView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [stockresponses.Watch], columns: columns, startSort: 'name', exportName: 'Stock Responses', pageSize: 5 });
    $scope.stockresponsesView = stockresponsesView;

    stockresponses.then(function () {
      $scope.items = stockresponses.segment($scope.routeSegmentID);
      $scope.stockData[0].responses = stockresponses.segment($scope.routeSegmentID);
      if ($scope.routeStockID) {
        $scope.ChangeState('show', stockresponses.get($scope.routeStockID));
        if (!$scope.selectedStock) {
          $scope.ChangeState('none');
        }
      }
    });


    $scope.Save = function (idPrefix, stock) {
      API('webapp-api-admin?api=stockresponses&mode=modify', idPrefix, 'StockModify', { stockID: stock.id, details: stock.details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //modify the row in the list.
        angular.extend(stock, result.row);
        //save the details
        angular.extend(stock.details, result.details);
        $scope.messages.saveSuccess = 'Stock Resopnses updated successfully.'
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      });
    }
    $scope.Create = function (idPrefix, details) {
      details.segmentID = $scope.routeSegmentID;
      API('webapp-api-admin?api=stockresponses&mode=new', idPrefix, 'StockCreate', { details: details }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //add new row to the list.
        stockresponses.addToSegment(result.row, $scope.routeSegmentID);
        $scope.items = stockresponses.segment($scope.routeSegmentID);
        $scope.ChangeState('show', result.row);
        $scope.messages.createSuccess = 'Stock Responses added successfully.';
      }).catch(function (errors) {
        $scope.messages.createErrors = errors;
      });
    }
    $scope.DeleteStart = function () {
      $scope.selectedStock.deleting = true;
      $scope.messages.Clear();
    }
    $scope.DeleteConfirm = function (idPrefix) {
      API('webapp-api-admin?api=stockresponses&mode=delete', idPrefix, 'StockDelete', { stockID: $scope.selectedStock.id }).init(function () {
        $scope.messages.Clear();
      }).then(function (result) {
        //remove row from list
        stockresponses.remove(result.row);
        $scope.items = stockresponses.segment($scope.routeSegmentID);
        $scope.ChangeState('none');
        $scope.messages.deleteSuccess = 'Stock Response deleted successfully.'
      }).catch(function (errors) {
        $scope.messages.deleteErrors = errors;
      });
    }
    function LoadDetails(stock) {
      stock.details = { id: stock.id }
      API('webapp-api-admin?api=stockresponsedetails', null, 'StockDetails', { stockID: stock.id }).init(function () {
        $scope.loading = true;
      }).then(function (result) {
        //save the details
        angular.extend(stock.details, result.details);
        stock.effects = result.details.effects;
      }).catch(function (errors) {
        //couldn't load
        $scope.ChangeState('none');
      }).finally(function () {
        $scope.loading = false;
      });
    }
  }
  app.controller('ControllerAdminSegmentMailbox', ['$scope', '$location', '$timeout', '$q', 'iService.api.serial', 'iService.mailboxesadmin', 'iService.topics', 'iService.filters', ControllerAdminSegmentMailbox]);
  function ControllerAdminSegmentMailbox($scope, $location, $timeout, $q, API, mailboxes, topics, filters) {
    $scope.Items = function () { return mailboxes.segment($scope.routeSegmentID); }
    $scope.loadingMailboxes = true;
    $q.all([mailboxes.promise(), filters.promise(), topics.promise()]).then(function () {
      var columns = [
        { name: 'ID', var: 'id', visible: false },
        { name: 'Display Name', var: 'displayName', split: true },
        { name: 'Reply To Address', var: 'replyTo', split: true },
        { name: 'Default Topic', var: 'topicID', split: true, printFunc: PrinterName(topics) },
        { name: 'Enabled', var: 'enabled' },
        { name: 'Incoming Server Name', var: 'inServer', visible: false },
        { name: 'Incoming Protocol', var: 'inProtocol', visible: false },
        { name: 'Outgoing Protocol', var: 'outProtocol', visible: false },
        { name: 'Incoming Username', var: 'inUsername', visible: false },
        { name: 'Incoming Password', var: 'inPassword', visible: false },
        { name: 'Incoming Port Number', var: 'inPort', visible: false },
        { name: 'Filter', var: 'filterID', visible: false, printFunc: PrinterName(filters) },
        { name: 'Incoming Use Encryption', var: 'inEncrypt', visible: false },
        { name: 'Outgoing Use Encryption', var: 'outEncrypt', visible: false },
        { name: 'Outgoing Server Name', var: 'outServer', visible: false },
        { name: 'Outgoing Port Number', var: 'outPort', visible: false },
        { name: 'Outgoing Username', var: 'outUsername', visible: false },
        { name: 'Outgoing Password', var: 'outPassword', visible: false },
        { name: 'Errors', var: 'errorCount' }
      ];
      var mailboxesView = iservice.TableViewer($scope, $timeout,
        { itemsFunc: $scope.Items, watchFuncs: [mailboxes.Watch, topics.Watch], columns: columns, startSort: 'displayName', exportName: 'Mailboxes', pageSize: 5 });
      $scope.mailboxesView = mailboxesView;
      $scope.mailboxes = mailboxes;
      $scope.topics = topics;

      $scope.filters = filters;
      $scope.segmentFilters = [];
      $scope.segmentFilters.push({ id: '', option: '-- None --' });
      filters.segment($scope.routeSegmentID).forEach(function (filter) {
        $scope.segmentFilters.push(filter);
      });

      $scope.HasServer = function (protocol) {
        if (protocol == 'imap' || protocol == 'pop' || protocol == 'ews' || protocol == 'smtp' || protocol == 'smtpauth') return true;
        return false;
      }
      $scope.HasEncrypt = function (protocol) {
        if (protocol == 'imap' || protocol == 'pop' || protocol == 'smtp' || protocol == 'smtpauth') return true;
        return false;
      }
      $scope.HasPort = function (protocol) {
        if (protocol == 'imap' || protocol == 'pop' || protocol == 'smtpauth') return true;
        return false;
      }
      $scope.HasPassword = function (protocol) {
        if (protocol == 'imap' || protocol == 'pop' || protocol == 'smtpauth' || protocol == 'ews') return true;
        return false;
      }
      $scope.HasUsername = function (protocol) {
        if (protocol != 'smtp') return true;
        return false;
      }
      $scope.HasEwsOAuth = function (protocol) {
        if (protocol == 'ews-oauth') return true;
        return false;
      }
      function Empty(str) {
        return str == null || str.trim() == '';
      }
      function AnyEmpty() {
        for (var i = 0; i < arguments.length; i++) {
          if (Empty(arguments[i])) return true;
        }
        return false;
      }
      $scope.SaveDisabled = function (details) {
        if (details.inProtocol == 'pop' || details.inProtocol == 'imap') {
          if (AnyEmpty(details.inServer, details.inPort, details.inUsername, details.isPassword)) return true;
        }
        if (details.inProtocol == 'ews') {
          if (AnyEmpty(details.inServer, details.inUsername, details.isPassword)) return true;
        }
        if (details.inProtocol == 'ews-oauth') {
          if (AnyEmpty(details.ewsAppID, details.ewsSecret, details.ewsTenantID, details.inUsername)) return true;
        }
        if (details.outProtocol == 'smtp') {
          if (AnyEmpty(details.outServer, details.displayName, details.replyTo)) return true;
        }
        if (details.outProtocol == 'smtpauth' || details.outProtocol == 'ews') {
          if (AnyEmpty(details.outServer, details.outPort, details.outUsername, details.outPassword, details.displayName, details.replyTo)) return true;
        }
        if (details.outProtocol == 'ews-oauth') {
          if (AnyEmpty(details.ewsAppID, details.ewsSecret, details.ewsTenantID, details.outUsername, details.displayName, details.replyTo)) return true;
        }
      }

      $scope.NewItem = function () {
        return {
          inProtocol: 'pop',
          inEncrypt: true,
          inPort: 995,
          outPort: 465,
          outProtocol: 'smtp',
          outEncrypt: true
        }
      }
      NewDeleteDetailer($scope, $location, '/segment/' + $scope.routeSegmentID + '/mailbox', 'Mailbox', 'admin-segment-mailbox-list.html', 'admin-segment-mailbox-new.html', 'admin-segment-mailbox-delete.html', 'admin-segment-mailbox-details.html', mailboxes.list, LoadDetails);

      if ($scope.routeMailboxID) {
        $scope.ChangeState('show', mailboxes.get($scope.routeMailboxID));
        if (!$scope.selectedMailbox) {
          $scope.ChangeState('none');
        }
      }

      $scope.EnableSave = function (idPrefix, mailbox) {
        API('webapp-api-admin?api=mailboxdetails', idPrefix, 'MailboxDetails', { mailboxID: mailbox.id }).init(function () {
          mailbox.enableRunning = true;
          $scope.messages.Clear();
        }).then(function (result) {
          result.details.enabled = mailbox.enabled;
          return API('webapp-api-admin?api=mailboxes&mode=modify', idPrefix, 'MailboxModify', { mailboxID: mailbox.id, details: result.details }).catch(function (errors) {
            if (errors && errors.length) errors[0].msg = 'Error modifying mailbox. ' + errors[0].msg;
            return $q.reject(errors);
          });
        }).then(function (result) {
          $scope.messages.toggleSuccess = 'Mailbox updated.'
          if (mailbox.details) mailbox.details.enabled = mailbox.enabled;
        }).catch(function (errors) {
          $scope.messages.toggleErrors = errors;
        }).finally(function () {
          mailbox.enableRunning = false;
        });
      }

      $scope.Save = function (idPrefix, mailbox) {
        if (!mailbox) return;
        API('webapp-api-admin?api=mailboxes&mode=modify', idPrefix, 'MailboxModify', { mailboxID: mailbox.id, details: mailbox.details }).init(function () {
          $scope.saveRunning = true;
          $scope.messages.Clear();
        }).then(function (result) {
          //modify the row in the list.
          angular.extend(mailbox, result.row);
          mailbox.enabled = result.details.enabled;
          //save the details
          angular.extend(mailbox.details, result.details);
          $scope.messages.saveSuccess = 'Mailbox updated successfully.'
        }).catch(function (errors) {
          $scope.messages.saveErrors = errors;
        }).finally(function () {
          $scope.saveRunning = false;
        });
      }
      $scope.Create = function (idPrefix, details) {
        details.ownerSegmentID = $scope.routeSegmentID;
        API('webapp-api-admin?api=mailboxes&mode=new', idPrefix, 'MailboxCreate', { details: details }).init(function () {
          $scope.messages.Clear();
        }).then(function (result) {
          //add new row to the list.
          mailboxes.add(result.row, $scope.routeSegmentID);
          $scope.ChangeState('show', result.row);
          $scope.messages.createSuccess = 'Mailbox added successfully.';
        }).catch(function (errors) {
          $scope.messages.createErrors = errors;
        });
      }
      $scope.DeleteStart = function () {
        $scope.selectedMailbox.deleting = true;
        $scope.messages.Clear();
      }
      $scope.DeleteConfirm = function (idPrefix) {
        API('webapp-api-admin?api=mailboxes&mode=delete', idPrefix, 'MailboxDelete', { mailboxID: $scope.selectedMailbox.id }).init(function () {
          $scope.messages.Clear();
        }).then(function (result) {
          //remove row from list
          mailboxes.remove(result.row);
          $scope.ChangeState('none');
          $scope.messages.deleteSuccess = 'Mailbox deleted successfully.';
        }).catch(function (errors) {
          $scope.messages.deleteErrors = errors;
        });
      }
      function LoadDetails(mailbox) {
        mailbox.details = { id: mailbox.id, errorsUpdate: 0 }
        API('webapp-api-admin?api=mailboxdetails', null, 'MailboxDetails', { mailboxID: mailbox.id }).init(function () {
          $scope.loading = true;
        }).then(function (result) {
          //save the details
          angular.extend(mailbox.details, result.details);
          mailbox.details.errorsUpdate++;
          mailbox.effects = result.details.effects;
        }).catch(function (errors) {
          //couldn't load
          $scope.ChangeState('none');
        }).finally(function () {
          $scope.loading = false;
        });
      }
      $scope.loadingMailboxes = false;
    });
  }
  app.controller('ControllerAdminSegmentMailboxErrors', ['$scope', '$timeout', 'iService.api.serial', ControllerAdminSegmentMailboxErrors]);
  function ControllerAdminSegmentMailboxErrors($scope, $timeout, API) {
    var details = $scope.details;
    $scope.Items = function () { return details.userErrors; }
    var errorsColumns = [
      { name: 'Date', var: 'timeUTC', visible: true, ascend: false },
      { name: 'Error Message', var: 'message', split: true }];
    var errorsView = iservice.TableViewer($scope, $timeout,
      { itemsFunc: $scope.Items, watchFuncs: [function () { return details.errorsUpdate; }], columns: errorsColumns, startSort: 'timeUTC', exportName: 'Mailbox Errors', pageSize: 5 });
    $scope.errorsView = errorsView;

    $scope.ClearErrorsClick = function (idPrefix, mailbox) {
      API('webapp-api-admin?api=mailboxes&mode=errors', idPrefix, 'MailboxClearError', { mailboxID: mailbox.id }).init(function () {
        $scope.messages.Clear();
        $scope.clearErrorsRunning = true;
      }).then(function (result) {
        //modify the row in the list.
        mailbox.errorCount = 0;
        //modify details.
        mailbox.details.userErrors = [];
        mailbox.details.errorsUpdate++;
        $scope.messages.saveSuccess = 'Mailbox errors cleared.'
      }).catch(function (errors) {
        $scope.messages.saveErrors = errors;
      }).finally(function () {
        $scope.clearErrorsRunning = false;
      });
    }
  }

  app.config(['$routeProvider', 'iService.rootPath', function (route, rootPath) {
    route.when('/website', { templateUrl: rootPath + 'f/admin-website', controller: 'ControllerAdminWebsitesMain' });
    route.when('/website/:websiteID', { templateUrl: rootPath + 'f/admin-website', controller: 'ControllerAdminWebsitesMain' });
    route.when('/skill', { templateUrl: rootPath + 'f/admin-skill', controller: 'ControllerAdminSkillsMain' });
    route.when('/skill/:skillID', { templateUrl: rootPath + 'f/admin-skill', controller: 'ControllerAdminSkillsMain' });
    route.when('/agent', { templateUrl: rootPath + 'f/admin-agent', controller: 'ControllerAdminAgentsMain' });
    route.when('/agent/:agentID', { templateUrl: rootPath + 'f/admin-agent', controller: 'ControllerAdminAgentsMain' });
    route.when('/agent/:agentID/:tab', { templateUrl: rootPath + 'f/admin-agent', controller: 'ControllerAdminAgentsMain' });
    route.when('/agent/:agentID/stock/:stockID', { templateUrl: rootPath + 'f/admin-agent', controller: 'ControllerAdminAgentsMain' });
    route.when('/usertype', { templateUrl: rootPath + 'f/admin-usertype', controller: 'ControllerAdminUserTypesMain' });
    route.when('/usertype/:usertypeID', { templateUrl: rootPath + 'f/admin-usertype', controller: 'ControllerAdminUserTypesMain' });
    route.when('/list', { templateUrl: rootPath + 'f/admin-list', controller: 'ControllerAdminListsMain' });
    route.when('/list/:listID', { templateUrl: rootPath + 'f/admin-list', controller: 'ControllerAdminListsMain' });
    route.when('/alert', { templateUrl: rootPath + 'f/admin-alert', controller: 'ControllerAdminAlertsMain' });
    route.when('/alert/:alertID', { templateUrl: rootPath + 'f/admin-alert', controller: 'ControllerAdminAlertsMain' });
    route.when('/department', { templateUrl: rootPath + 'f/admin-department', controller: 'ControllerAdminDepartmentsMain' });
    route.when('/department/:departmentID', { templateUrl: rootPath + 'f/admin-department', controller: 'ControllerAdminDepartmentsMain' });
    route.when('/massmailing', { templateUrl: rootPath + 'f/admin-massmailing', controller: 'ControllerAdminMassmailingMain' });
    route.when('/massmailing/:massmailingID', { templateUrl: rootPath + 'f/admin-massmailing', controller: 'ControllerAdminMassmailingMain' });
    route.when('/masslist', { templateUrl: rootPath + 'f/admin-masslist', controller: 'ControllerAdminMasslistMain' });
    route.when('/masslist/:masslistID', { templateUrl: rootPath + 'f/admin-masslist', controller: 'ControllerAdminMasslistMain' });
    route.when('/masscriteria', { templateUrl: rootPath + 'f/admin-masscriteria', controller: 'ControllerAdminMasscriteriaMain' });
    route.when('/masscriteria/:masscriteriaID', { templateUrl: rootPath + 'f/admin-masscriteria', controller: 'ControllerAdminMasscriteriaMain' });
    route.when('/massmessage', { templateUrl: rootPath + 'f/admin-massmessage', controller: 'ControllerAdminMassmessageMain' });
    route.when('/massmessage/:massmessageID', { templateUrl: rootPath + 'f/admin-massmessage', controller: 'ControllerAdminMassmessageMain' });
    route.when('/masscampaign', { templateUrl: rootPath + 'f/admin-masscampaign', controller: 'ControllerAdminMasscampaignMain' });
    route.when('/masscampaign/:masscampaignID', { templateUrl: rootPath + 'f/admin-masscampaign', controller: 'ControllerAdminMasscampaignMain' });
    route.when('/manageagentemail', { templateUrl: rootPath + 'f/admin-manageagentemail', controller: 'ControllerManageAgentEmailFormMain' });
    route.when('/creditmemoadmin', { templateUrl: rootPath + 'f/admin-creditmemoadmin', controller: 'ControllerCreditMemoAdminMain' });
    route.when('/quote-admin', { templateUrl: rootPath + 'f/quote-admin', controller: 'ControllerQuoteAdminMain' });
    route.when('/quote-admin/:quoteID', { templateUrl: rootPath + 'f/quote-admin', controller: 'ControllerQuoteAdminMain' });
    route.when('/form', { templateUrl: rootPath + 'f/admin-form', controller: 'ControllerAdminFormsMain' });
    route.when('/form/:formID', { templateUrl: rootPath + 'f/admin-form', controller: 'ControllerAdminFormsMain' });
    route.when('/formbuiltin', { templateUrl: rootPath + 'f/admin-form', controller: 'ControllerAdminFormsMainBuiltin' });
    route.when('/formbuiltin/:formID', { templateUrl: rootPath + 'f/admin-form', controller: 'ControllerAdminFormsMainBuiltin' });
    route.when('/servicelevel', { templateUrl: rootPath + 'f/admin-servicelevel', controller: 'ControllerAdminServicelevelsMain' });
    route.when('/servicelevel/:servicelevelID', { templateUrl: rootPath + 'f/admin-servicelevel', controller: 'ControllerAdminServicelevelsMain' });
    route.when('/segment/:segmentID', { templateUrl: rootPath + 'f/admin-segment', controller: 'ControllerAdminSegmentsMain' });
    route.when('/segment/:segmentID/:tab', { templateUrl: rootPath + 'f/admin-segment', controller: 'ControllerAdminSegmentsMain' });
    route.when('/segment/:segmentID/intproperty/:intpropertyID', { templateUrl: rootPath + 'f/admin-segment', controller: 'ControllerAdminSegmentsMain' });
    route.when('/segment/:segmentID/contactproperty/:contactpropertyID', { templateUrl: rootPath + 'f/admin-segment', controller: 'ControllerAdminSegmentsMain' });
    route.when('/segment/:segmentID/mailbox/:mailboxID', { templateUrl: rootPath + 'f/admin-segment', controller: 'ControllerAdminSegmentsMain' });
    route.when('/segment/:segmentID/notification/:notificationID', { templateUrl: rootPath + 'f/admin-segment', controller: 'ControllerAdminSegmentsMain' });
    route.when('/segment/:segmentID/stock/:stockID', { templateUrl: rootPath + 'f/admin-segment', controller: 'ControllerAdminSegmentsMain' });
    route.when('/segment/:segmentID/topic/:topicID', { templateUrl: rootPath + 'f/admin-segment', controller: 'ControllerAdminSegmentsMain' });
    route.when('/segment/:segmentID/filter/:filterID', { templateUrl: rootPath + 'f/admin-segment', controller: 'ControllerAdminSegmentsMain' });
    route.when('/smtpout', { templateUrl: rootPath + 'f/admin-smtpout', controller: 'ControllerAdminSMTPOut' });
    route.when('/openid', { templateUrl: rootPath + 'f/admin-openid', controller: 'ControllerAdminOpenID' });
    route.otherwise('/');
  }]);
  app.run(['$route', '$rootScope', '$location', function ($route, $rootScope, $location) {
    var original = $location.path;
    $location.path = function (path, reload) {
      if (reload === false) {
        var lastRoute = $route.current;
        var un = $rootScope.$on('$locationChangeSuccess', function () {
          $route.current = lastRoute;
          un();
        });
      }
      return original.apply($location, [path]);
    };
  }]);
  app.directive('stepInput6', function () {
    return {
      restrict: 'E',
      transclude: true,
      scope: {
        label: '='
      },
      template: '<div class="col-md-6"><div class="form-group"><label class="col-sm-4 control-label text-left">{{ label }}</label><div class="col-sm-8"><ng-transclude></ng-transclude></div></div></div>'
    };
  });
  app.directive('basicInput', function () {
    return {
      restrict: 'E',
      transclude: true,
      scope: {
        label: '='
      },
      template: '<div class="form-group"><label class="col-sm-12 control-label text-left">{{ label }}</label><div class="col-sm-12"><ng-transclude></ng-transclude></div></div>'
    };
  });
  app.directive('stepInput12', function () {
    return {
      restrict: 'E',
      transclude: true,
      scope: {
        label: '='
      },
      template: '<div class="col-md-12"><div class="form-group"><label class="col-sm-12 control-label text-left">{{ label }}</label><div class="col-sm-12"><ng-transclude></ng-transclude></div></div></div>'
    };
  });
  app.directive('variablePicker', ['iService.stockresponses', 'iService.contactProperties', function (stockresponses, contactProperties) {
    return {
      restrict: 'E',
      transclude: true,
      scope: {
        pasteVar: '<'
      },
      templateUrl: 'admin-variable-picker.html',
      link: function (scope) {
        scope.PasteVar = function (item) {
          scope.pasteVar(item);
        }
        var pickerTab = 'var';
        scope.IsTab = function (tab, yesval, noval) {
          if (tab == pickerTab) return yesval;
          return noval;
        }
        scope.ChangeTab = function (tab) {
          if (tab == pickerTab) return;
          pickerTab = tab;
        }
        scope.contactVars = [
          { name: 'Name', varString: '$value -customer(name)$' }];
        scope.answerVars = [
          { name: 'Email', varString: '$value -question(fromemail)$' },
          { name: 'Reference Number', varString: '$value -question(ref)$' },
          { name: 'Original Date', varString: '$value -question(date)$' },
          { name: 'Original Time', varString: '$value -question(time)$' },
          { name: 'Subject', varString: '$value -question(subject)$' },
          { name: 'Original Body', varString: '$value -question(body)$' },
          { name: 'Quoted Question', varString: '$value -question(quotedbody)$' }];
        scope.linkVars = [
          { name: 'Link', varString: '$link changeme$' },
          { name: 'Link w/ Url', varString: "$link changeme -url 'changeme'$" },
          { name: 'Link My Account Subscriptions', varString: "$link myaccount -url 'Form.aspx?formID=setme'$" },
          { name: '<a></a> Link', varString: "$linka changeme -text 'changeme'$" },
          { name: '<a></a> Link w/Url', varString: "$linka changeme  -text 'changeme' -url 'changeme'$" },
          { name: '<a></a> Link My Account Subscriptions', varString: "$linka myaccount -text 'changeme' -url 'Form.aspx?formID=setme'$" },
          { name: '<img /> Viewed Tracking Image', varString: '$linkimg viewed$' }];
        scope.timeVars = [
          { name: 'Today', text: '$value -today$' }];
        scope.propertySegments = [];
        scope.stockresponses = stockresponses;
        contactProperties.then(function (data) {
          var segments = {};
          contactProperties.list().forEach(function (property) {
            var segment = segments[property.segmentID];
            if (!segment) {
              segment = { name: property.segmentName, properties: [] };
              segments[property.segmentID] = segment;
              scope.propertySegments.push(segment);
            }
            segment.properties.push(property);
            property.varString = '$value -customer(property' + (property.name.indexOf("'") >= 0 ? property.id : "'" + property.name + "'") + ')$'
          });
        });
        stockresponses.then(function (data) {
          stockresponses.list().forEach(function (stock) {
            var nameparam = stock.name.indexOf("'") >= 0 ? '' : " -name'" + stock.name + "'";
            var idparam = " -id'" + stock.id + "'";
            stock.varString = "$stock" + idparam +  nameparam + '$';
          });
        });
      }
    };
  }]);

  iservice.ProcessLogin(loggedIn);
})();